%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% Written (W) 2005-2009 Gunnar Raetsch, Gabriele Schweikert, Jonas Behr, Soeren Sonnenburg, Alexander Zien, Georg Zeller, Andre Noll, Cheng Soon Ong, Petra Philips
% Copyright (C) 2005-2009 Max Planck Society and Fraunhofer Institute FIRST 
%
function mgene_config
% setup_mgene
%
% sets a few global variables with system dependent paths
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% mgene paths
global MGENE_SRC_PATH MGENE_PATH MGENE_TMP_PATH MGENE_LD_LIBRARY_PATH MGENE_PATH_LIST

% interpreter paths
global MATLAB_BIN_PATH OCTAVE_BIN_PATH 

% shogun path
global SHOGUN_PATH 

% Sun Gridengine related paths
global RPROC_PATH USE_SGE SGE_BIN_PATH SGE_TMP_PATH RPROC_LD_LIBRARY_PATH RPROC_PATH_LIST

% optimizer related paths
global OPTIMIZER OPTIMIZER_PATH OPTIMIZER_TOOLBOX_PATH 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% configuration (adapt to you configuration)
MGENE_PATH = getenv('MGENE_PATH') ;
MGENE_SRC_PATH = getenv('MGENE_SRC_PATH') ;
MGENE_LD_LIBRARY_PATH = getenv('MGENE_LD_LIBRARY_PATH') ;
SHOGUN_PATH = getenv('SHOGUN_PATH') ;
MATLAB_BIN_PATH = getenv('MATLAB_BIN_PATH') ;
INTERPRETER = getenv('INTERPRETER') ;
OCTAVE_BIN_PATH = getenv('OCTAVE_BIN_PATH') ;
SGE_BIN_PATH = getenv('SGE_BIN_PATH') ;
SGE_TMP_PATH = getenv('SGE_TMP_PATH') ;
OPTIMIZER = getenv('OPTIMIZER') ;
OPTIMIZER_PATH = getenv('OPTIMIZER_PATH') ;
OPTIMIZER_TOOLBOX_PATH = getenv('OPTIMIZER_TOOLBOX_PATH') ;

if ~isempty(SGE_BIN_PATH),
  USE_SGE = 1 ;
  SGE_TMP_PATH = [MGENE_PATH '/tmp/sge'] ;
  RPROC_PATH = [MGENE_SRC_PATH '/rproc'] ;
else
  USE_SGE = 0 ;
  SGE_TMP_PATH = '' ;
  RPROC_PATH = '' ;
end ;

MGENE_TMP_PATH = [MGENE_PATH '/tmp'] ;

fprintf('mGene is configured with the following paths (can be changed via ./setup_mgene.sh):\n') ;
fprintf('\tMGENE_PATH\t\t= %s\n', MGENE_PATH) ;
fprintf('\tMGENE_SRC_PATH\t\t= %s\n', MGENE_SRC_PATH) ;
fprintf('\tMGENE_TMP_PATH\t\t= %s\n', MGENE_TMP_PATH) ;
if length(MGENE_LD_LIBRARY_PATH)>65,
  fprintf('\tMGENE_LD_LIBRARY_PATH\t= %s...\n', MGENE_LD_LIBRARY_PATH(1:65)) ;
else
  fprintf('\tMGENE_LD_LIBRARY_PATH\t= %s\n', MGENE_LD_LIBRARY_PATH) ;
end ;
fprintf('\tSHOGUN_PATH\t\t= %s\n', SHOGUN_PATH) ;
fprintf('\tINTERPRETER\t\t= %s\n', INTERPRETER) ;
fprintf('\tMATLAB_BIN_PATH\t\t= %s\n', MATLAB_BIN_PATH) ;
fprintf('\tOCTAVE_BIN_PATH\t\t= %s\n', OCTAVE_BIN_PATH) ;
fprintf('\tSGE_BIN_PATH\t\t= %s\n', SGE_BIN_PATH) ;
fprintf('\tSGE_TMP_PATH\t\t= %s\n', SGE_TMP_PATH) ;
fprintf('\tOPTIMIZER\t\t= %s\n', OPTIMIZER) ;
fprintf('\tOPTIMIZER_PATH\t\t= %s\n', OPTIMIZER_PATH) ;
fprintf('\tOPTIMIZER_TOOLBOX_PATH\t= %s\n', OPTIMIZER_TOOLBOX_PATH) ;
fprintf('\n\n') ;

% Normalize working directory
[ret,hostname]=unix('hostname') ;
assert(ret==0) ;
hostname(hostname==sprintf('\n')) = [] ;

if ~isempty(MGENE_TMP_PATH)
  unix(sprintf('mkdir -p %s', MGENE_TMP_PATH)) ;  % for temporary files
end ;

if USE_SGE && ~isempty(SGE_TMP_PATH)
  unix(sprintf('mkdir -p %s', SGE_TMP_PATH)) ;  % for temporary files
end ;

MGENE_PATH_LIST = {} ;

addpath(MGENE_SRC_PATH) ; 
MGENE_PATH_LIST{end+1} = MGENE_SRC_PATH ;

MGENE_MEX_PATH=[MGENE_PATH '/mex'] ;
addpath(MGENE_MEX_PATH) ; 
MGENE_PATH_LIST{end+1} = MGENE_MEX_PATH ;

MGENE_RPROC_PATH=[MGENE_SRC_PATH '/rproc'] ;
addpath(MGENE_RPROC_PATH) ;
MGENE_PATH_LIST{end+1} = MGENE_MEX_PATH ;

addpath(SHOGUN_PATH) ;
MGENE_PATH_LIST{end+1} = SHOGUN_PATH ;

if ~isempty(OPTIMIZER_TOOLBOX_PATH),
  addpath(OPTIMIZER_TOOLBOX_PATH) ;
  MGENE_PATH_LIST{end+1} = OPTIMIZER_TOOLBOX_PATH ;
end ;

if USE_SGE,
  RPROC_PATH_LIST = MGENE_PATH_LIST ;
  RPROC_LD_LIBRARY_PATH = MGENE_LD_LIBRARY_PATH ;
end ;

% switch off a few expected warnings
engine = determine_engine() ;
if isequal(engine, 'octave'),
  warning('off', 'mgene:no_model') ;
  warning('off', 'Octave:precedence-change') ;
  warning('off', 'Octave:function-name-clash') ;
  warning('off', '') ;
  warning('off', 'Octave:num-to-str') ;
  warning('off', 'Octave:function-name-clash') ;
  warning('off', 'Octave:divide-by-zero') ;
  warning('off', 'Octave:future-time-stamp');
  warning('off', 'load_genomic:contig_boundary') ;
  warning('off', 'merge_blocks:multiple_truths') ;
  warning('off', 'solve_qp:constraints') ;
  warning('off', 'correct_cleave_and_tss_pos:find_nearest_pos') ;
  warning('off', 'Octave:assign-as-truth-value') ;
else
  warning('off', 'MATLAB:typeaheadBufferOverflow');
  %dbstop error
end ;

% make sure no process stops with a debug prompt
global g_ignore_keyboard
g_ignore_keyboard=1 ;

