%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% Written (W) 2005-2009 Gunnar Raetsch, Gabriele Schweikert, Jonas Behr, Soeren Sonnenburg, Alexander Zien, Georg Zeller, Andre Noll, Cheng Soon Ong, Petra Philips
% Copyright (C) 2005-2009 Max Planck Society and Fraunhofer Institute FIRST 
%
function mgene_genometool(fasta_fname, base_dir) ;
% mgene_genometool(fasta_fname, genome_config_dir) ;

if nargin~=2,
  first_arg=[] ;
  if nargin>=1, first_arg=fasta_fname; end ;

  if show_usage('mgene_genometool', '<fasta-file>, <genome-dir>', ...
     nargin, first_arg),
     return
  end ;
end ;

genometool(fasta_fname, '', '', base_dir) ;


return

function genometool(fasta_fname, fasta_dir, information_file, base_dir) ;
% genometool(fasta_fname, fasta_dir, information_file, genome_config_dir) ;

global MGENE_SRC_PATH 

[ret, timedate] = unix('date') ;
assert(ret==0) ;
timedate(timedate==sprintf('\n')) = [] ;
fprintf('started genometool at %s\n', timedate) ;


mkdir(base_dir) ;

if exist('information_file') && ~isempty(information_file)
  fprintf('Results are written to: %s\n', information_file) ;

  fd = fopen(information_file, 'w+') ;
else
  fd=1 ;
end ;
fprintf(fd,'---------------------------------------------------- \n');
fprintf(fd,'GenomeTool started %s\n', timedate) ;
fprintf(fd,'---------------------------------------------------- \n\n');
if fd~=1,
  fclose(fd)   
end

genome_config = [base_dir '/genome.config'] ;

fprintf('Using fasta file from %s \n\n',fasta_fname) ;

[ret,home]=unix('echo $HOME');
assert(ret==0) ;
python_script=sprintf('%s/make_gio.py',MGENE_SRC_PATH);
if exist('information_file')  && ~isempty(information_file),
  python_call = sprintf('python %s %s %s %s', python_script, fasta_fname, base_dir, information_file);
else
  tmpname = tempname 
  python_call = sprintf('python %s %s %s %s && cat %s && rm %s', python_script, fasta_fname, base_dir, tmpname, tmpname, tmpname)
end ;
ret=system(python_call);
assert(ret==0) ;

if exist('information_file')  && ~isempty(information_file),
  fd = fopen(information_file, 'a+') ;
else
  fd = 1 ;
end ;

[ret, timedate] = unix('date') ;
assert(ret==0) ;
timedate(timedate==sprintf('\n')) = [] ;
fprintf(fd,'\n---------------------------------------------------- \n');
fprintf(fd,'finished GenomeTool at %s\n', timedate) ;
fprintf(fd,'---------------------------------------------------- \n');

if fd~=1,
  fclose(fd) ;
end ;

return

function [seqs,names]=read_fasta_file(fname, name_only)
% [seqs,names]=read_fasta_file(fname)
% 
% reads all entries in a fasta file
  
if nargin<2, name_only=0; end ; 

fd=fopen(fname,'r') ;
str=fread(fd) ;

% fix new lines
num13=sum(str==13) ;
num10=sum(str==10) ;
if num10==0 && num13>0,
  str(str==13)=10 ;
end ;
if num10>0 && num13>0,
  assert(num10==num13) ;
  str(str==13)=[] ;
end ;

newseq=find(str=='>') ;
newseq(end+1) = length(str)+1 ;

seqs={} ;
names={} ;

for i=1:length(newseq)-1,
  seq=char(str(newseq(i):newseq(i+1)-1)) ;
  idx=find(seq==10, 1, 'first') ;
  assert(length(idx)==1) ;
  name = seq(2:idx-1)' ;
  names{end+1}=name ;
  if ~name_only,
    seq  = seq(idx+1:end)' ;
    seq(seq==10) = [] ;
    seqs{end+1} = seq ;
  end ;
end ;

fclose(fd) ;

return

function genome_info=init_genome(config_fname)
% genome_info=init_genome(config_fname)  
%
% global g_genome_info ;

global g_genome_info genome_strings;

genome_strings = [];
g_est_fnames={} ;
g_prot_fnames={} ;
g_cdna_fnames={} ;
g_flcdna_fnames={} ;
g_contig_names={} ;
g_flat_fnames={} ;
g_fasta_fnames={} ;
g_annotation_fnames={} ;
g_alphabet='acgt' ;
g_basedir='' ;
g_max_intron_len=1000000 ;
g_min_intron_len=10 ;
g_merge_est_transcripts=1 ;

fd=fopen(config_fname, 'r') ;
while ~feof(fd),
  line = fgetl(fd) ;
  if ~ischar(line), break ; end ;
  if isempty(line), continue ; end ;
  if line(1)=='#', continue ; end ;
  if length(line)>8 && isequal(line(1:7),'BASEDIR')
    g_basedir=deblank(line(9:end)) ;
  elseif length(line)>21 && isequal(line(1:21), 'MERGE_EST_TRANSCRIPTS')
    g_merge_est_transcripts = str2num(line(22:end)) ; 
  elseif length(line)>15 && isequal(line(1:15),'MAX_DOUBLE_QGAP')
  elseif length(line)>15 && isequal(line(1:15),'MAX_DOUBLE_TGAP')
  elseif length(line)>17 && isequal(line(1:17),'MAX_CDNA_DELETION')
  elseif length(line)>18 && isequal(line(1:18),'MAX_CDNA_INSERTION')
  elseif length(line)>21 && isequal(line(1:21),'TERMINAL_EXON_END_TOL')
  elseif length(line)>14 && isequal(line(1:14),'MIN_INTRON_LEN')
    g_min_intron_len = str2num(line(15:end)) ; 
    assert(~isempty(g_min_intron_len) & g_min_intron_len<1000) ;
  elseif length(line)>14 && isequal(line(1:14),'MAX_INTRON_LEN')
    g_max_intron_len = str2num(line(15:end)) ; 
    assert(~isempty(g_max_intron_len) & g_max_intron_len>1000) ;
  elseif length(line)>18 && isequal(line(1:18),'MIN_EST_COVER_FRAC')

  elseif length(line)>19 && isequal(line(1:19),'MIN_PROT_COVER_FRAC')

  elseif length(line)>19 && isequal(line(1:19),'MIN_CDNA_COVER_FRAC')
  
  elseif length(line)>18 && isequal(line(1:18),'BLAT_BEST_HIT_ONLY')
  
  elseif length(line)>20 && isequal(line(1:20),'BLAT_BEST_HIT_MARGIN')

  elseif length(line)>8 && isequal(line(1:8),'ALPHABET')
    g_alphabet=deblank(line(10:end)) ;
  elseif length(line)>7 && isequal(line(1:7),'CONTIGS'),
    for i=1:str2num(line(8:end))
      l2=fgetl(fd) ; 
      idx=strfind(l2,'  ') ;
      while ~isempty(idx), 
        l2(idx)=[] ;
        idx=strfind(l2,'  ') ;
      end ;
      l2(l2==' ')=sprintf('\t') ;
      elems=separate(l2) ; 
      assert(length(elems)==3) ;
      g_contig_names{i}=elems{1} ;
      g_flat_fnames{i}=path_with_basedir(elems{2}, g_basedir) ;
      g_fasta_fnames{i}=path_with_basedir(elems{3}, g_basedir) ; 
    end ;
  elseif length(line)>8 && isequal(line(1:8),'ESTFILES'),
    for i=1:str2num(line(9:end))
      g_est_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>9 && isequal(line(1:9),'PROTFILES'),
    for i=1:str2num(line(10:end))
      g_prot_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>9 && isequal(line(1:9),'CDNAFILES'),
    for i=1:str2num(line(10:end))
      g_cdna_fnames{end+1}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>11 && isequal(line(1:11),'FLCDNAFILES'),
    for i=1:str2num(line(12:end))
      g_flcdna_fnames{end+1}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>15 && isequal(line(1:15),'ANNOTATIONFILES'),
    for i=1:str2num(line(16:end))
      g_annotation_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif ~isempty(line),
    error(sprintf('cannot understand "%s" in config file', line)) ;
  end ;
end ;
fclose(fd) ;

genome_info=[] ;
genome_info.est_fnames=g_est_fnames ;
genome_info.prot_fnames=g_prot_fnames ;
genome_info.cdna_fnames=g_cdna_fnames ;
genome_info.flcdna_fnames=g_flcdna_fnames ;
genome_info.contig_names=g_contig_names ;
genome_info.flat_fnames=g_flat_fnames ;
genome_info.fasta_fnames=g_fasta_fnames ;
genome_info.annotation_fnames=g_annotation_fnames ;
genome_info.alphabet=g_alphabet ;
genome_info.basedir=g_basedir ;
genome_info.max_intron_len = g_max_intron_len ;
genome_info.min_intron_len = g_min_intron_len ;
genome_info.merge_est_transcripts = g_merge_est_transcripts ;
g_genome_info=genome_info ;

return ;


function p=path_with_basedir(p1,basedir)
  
  if p1(1)=='/',
    p=p1; 
    return ;
  else
    if basedir(end)=='/',
      p=[basedir p1] ;
    else
      p=[basedir '/' p1] ;
    end ;
  end ;
  
  

return

function f=separate(str, delim, merge_multiple) ;
% f=separate(str, delimiter, merge_multiple_delimiters) ;
  
if nargin<2, delim=sprintf('\t') ; end ;
if nargin<3, merge_multiple=0 ; end ;
f={} ;
idx=[0 find(str==delim) length(str)+1] ;
if merge_multiple,
  idx2=find(idx(1:end-1)~=idx(2:end)-1) ;
  idx=[idx(idx2) idx(end)] ;
end 
for i=1:length(idx)-1
  f{i}=deblank(str(idx(i)+1:idx(i+1)-1)) ;
end ;

return

function x = fexist(filename,gz)
% FEXIST Check if file exists
%        FEXIST('filename') returns:
%        0 if filename does not exist
%        1 if filename exists in the given path

if nargin<2,
  gz=0;
end ;

fd = fopen(filename);
if fd ~= -1
  x = 1;
  fclose(fd);
elseif gz,
  fd = fopen([filename '.gz']);
  if fd~=-1,
    x=1 ;
    fclose(fd) ;
  else
    x=0 ;
  end ;
else
  x = 0;
end

return

function ret=keyboard_allowed()
% ret=keyboard_allowed()
%
% returns 1, if a keyboard command would be allowed

global g_ignore_keyboard
global THIS_IS_A_RPROC_PROCESS  

if isequal(g_ignore_keyboard, 1),
  ret=0 ;
  return ;
end ;

if isequal(THIS_IS_A_RPROC_PROCESS, 1),
  ret=0 ;
  return ;
end ;

ret=1 ;


return

