function mgene_train(fasta_fname, fasta_dir, gff_fname, version, seqid, source, level, fn_mGene_predictor, mGene_predictor_dir, run_locally, use_old_files, train_options, converters)
%  mgene_train(fasta_fname, fasta_dir, gff_fname, fn_mGene_predictor, mGene_predictor_dir, run_locally, version, seqid, source, level,  use_old_files, train_options, converters)

[ret, timedate] = unix('date') ;
timedate(timedate==sprintf('\n')) = [] ;
fprintf('started mgene_train at %s\n', timedate) ;

fprintf('Results are written to: %s\n', fn_mGene_predictor) ;

run_locally = -1 ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Data Processing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

output_dir = mGene_predictor_dir ;

config_dir = sprintf('%s/genome_dir',output_dir) ;
info_name = sprintf('%s/genome_info',output_dir) ;

annotation_fname = sprintf('%s/anno_info',output_dir);
annotation_dir = sprintf('%s/',output_dir);

ignore_missing_referenced_entries = 1 ;
coding_flag = 1 ;
nice_mkdir(config_dir);
fd=fopen(fn_mGene_predictor, 'w+') ; 
fprintf(fd,'------------- \n')
fprintf(fd,'mGeneTrain started %s\n', timedate) ;
fprintf(fd,'------------- \n\n')

fprintf(fd, 'Training Based on FASTA File : %s\n',fasta_fname) ; 
fprintf(fd, '                    GFF File : %s\n',gff_fname) ; 
fclose(fd) ;

galaxy_genometool(fasta_fname, fasta_dir, info_name, config_dir) ;
unix(sprintf('cat %s >> %s', info_name, fn_mGene_predictor)) ;
fd=fopen(fn_mGene_predictor, 'a+') ; fprintf(fd, '\n\n') ; fclose(fd) ;

genome_config = [config_dir '/genome.config'] ;
genome_info = init_genome(genome_config);

if run_locally==-1,
  run_locally_gff2anno = rproc_policy('gff2anno', genome_info)
else
  run_locally_gff2anno=run_locally ;
end ;
galaxy_gff2anno(config_dir, gff_fname, annotation_fname, annotation_dir, version, seqid, source, level, ignore_missing_referenced_entries, run_locally_gff2anno, converters) ;

unix(sprintf('cat %s >> %s',annotation_fname, fn_mGene_predictor)) ;
fd=fopen(fn_mGene_predictor, 'a+') ; fprintf(fd, '\n\n') ; fclose(fd) ;

[ret, timedate] = unix('date') ;
timedate(timedate==sprintf('\n')) = [] ;
fd=fopen(fn_mGene_predictor, 'a+') ; 

fprintf(fd, 'mGeneTrain (started %s)\n',  timedate)
fprintf(fd, '----------------------------\n\n') ;

genes = load_struct(sprintf('%s/genes.mat', annotation_dir), 'genes') ;
fprintf(fd, 'Read %i from GFF files\n', length(genes)) ;
clear genes ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% SIGNALS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
options = [] ;
if 1%~run_locally
   
  [engine,env]=determine_engine;
  if isequal(env, 'internal'),
    bdir = sprintf('/fml/ag-raetsch/home/%s/svn/projects/genefinding/',whoami);
    

    jobinfo = rproc_empty(0) ;
    options.start_dir = [bdir 'galaxy'];
    %options.addpaths = {bdir, [bdir 'data_processing_sensors'],[bdir 'sensors'],[bdir 'contents'],[bdir 'data_processing_lsl'],[bdir '/lsl_paper'],[bdir 'galaxy'],...
    %                    [bdir 'utils'],[bdir 'template_files'],[bdir 'template_files/Contents'],[bdir 'template_files/Organisms'],...
    %                    [bdir 'template_files/Methods'],[bdir 'template_files/Signals'],[bdir 'template_files/lsl/'], [bdir 'template_files/exp']...
    %                    [bdir '../splicing/splicegraphs/'], [bdir '../splicing/splicegraphs/detect_altsplice/'], [bdir '../../tools/utils'],[bdir '../../tools/genomes']...
    %                    [bdir '../../tools/prob_plif'],...
    %                    '/fml/ag-raetsch/home/raetsch/tmp/src'};
    %% '/fml/ag-raetsch/home/raetsch/svn/projects/genefinding/shogun'};
    %                    %'/fml/ag-raetsch/home/jonas/shogun/trunk/src/matlab/'};
    %options.addpaths = {'/fml/ag-raetsch/home/raetsch/tmp/src'};

    options.envstr = 'export LD_LIBRARY_PATH=~/mgene_galaxy/shogun.matlab_new/trunk/src/libshogun:~/mgene_galaxy/shogun.matlab_new/trunk/src/libshogunui';

    % options.identifier = sprintf('%spred%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;
    MEMREQ = 10000;
    time = 60*10;
  else
    
    options.envstr = 'export LD_LIBRARY_PATH=/home/galaxy/shogun.octave_new/trunk/src/libshogun/:/home/galaxy/shogun.octave_new/trunk/src/libshogunui';

    MEMREQ = 3500;
    time = 60*10;  
  end ;

end

signal_names = {'acc', 'don', 'tis', 'cdsStop', 'tss', 'cleave'} ;

P.covered_genic_positions_only = 1 ;
P.config_dir = config_dir ; 
P.annotation_fname = annotation_fname ;
P.annotation_dir = annotation_dir ;
P.train_on = 1 ;

% P.run_locally = run_locally;
% determine the number of contigs and 
% if there are less than ten then submit the subjobs
genome_config = [config_dir '/genome.config'] ;
genome_info = init_genome(genome_config);
num_contigs = length(genome_info.contig_names); 
%clear genome_info genome_config


%if num_contigs>10
%  fprintf('MORE than 10 contigs => subjobs will NOT be submitted\n');
%  P.run_locally = 1;
%else
%  fprintf('LESS than 10 contigs => subjobs will be submitted\n');
%  P.run_locally = 0;
%end

if exist('use_old_files')
  P.use_old_files = use_old_files;
else
  P.use_old_files = 0;
end
P.rproc_options = options;
P.run_locally = run_locally ;

num_jobs = 0;

fprintf(fd,'------------- \n')
fprintf(fd,'start signal and content training and prediction\n') ;
fprintf(fd,'--- this may take a while ---  \n') ;
fprintf(fd,'------------- \n\n')
fclose(fd) ;


for signal_idx = 1 :length(signal_names)
  num_jobs = num_jobs + 1;
  P.signal_name = signal_names{signal_idx} ;
  
  % label generation
  P.spf_label_fname = sprintf('%s/signal_%s_label.spf',output_dir, P.signal_name) ;
  P.spf_label_dir = sprintf('%s/signal_%s_label_files',output_dir, P.signal_name) ;

  % training
  P.tsp_train_info = sprintf('%s/signal_%s_classifier_info',output_dir, P.signal_name) ;
  P.tsp_train_dir = sprintf('%s/signal_%s_classifier_files',output_dir, P.signal_name) ;
  
  % prediction
  P.spf_pred_fname = sprintf('%s/signal_%s_predictions.spf',output_dir, P.signal_name) ;
  P.spf_pred_dir = sprintf('%s/signal_%s_predictions_files',output_dir, P.signal_name) ;
  
  % evaluation
  P.eval_fname = sprintf('%s/signal_%s_eval.txt',output_dir, P.signal_name) ;

  if run_locally==-1,
    run_locally_signal_wrapper = rproc_policy('signal_wrapper', genome_info) ;
  else
    run_locally_signal_wrapper = run_locally ;
  end ;

  if ~run_locally_signal_wrapper,
    [mem_req, time_req, opts] = rproc_memtime_policy(['signal_wrapper:' signal_names{signal_idx}], [], options) ;

    jobinfo(num_jobs) = rproc('signal_wrapper', P, mem_req, opts, time_req);
  else
    signal_wrapper(P);
  end
  train_dirs.(P.signal_name) =  P.tsp_train_dir;
  train_files.(P.signal_name) = P.tsp_train_info;
  pred_dirs.(P.signal_name) =  P.spf_pred_dir;
  pred_files.(P.signal_name) = P.spf_pred_fname;
end
clear P 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% TRAIN, PREDICT AND EVAL ALL CONTENTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% rmpath ~raetsch/svn/projects/genefinding/shogun
% addpath /fml/ag-raetsch/home/jonas/shogun/trunk/src/matlab

P.config_dir = config_dir; 
P.annotation_fname = annotation_fname;
P.annotation_dir = annotation_dir;
P.vs_content_names = [];
P.intergenic_dist_to_genes = [];
P.covered_genic_positions_only = 1 ;

if exist('use_old_files')
  P.use_old_files = use_old_files;
else
  P.use_old_files = 0;
end

P.run_locally = run_locally ;
% P.run_locally = run_locally;
%if num_contigs>10
%  fprintf('MORE than 10 contigs => subjobs will NOT be submitted\n');
%  P.run_locally = 1;
%else
%  fprintf('LESS than 10 contigs => subjobs will be submitted\n');
%  P.run_locally = 0;
%end
P.rproc_options = options;

content_names = {'frame0', 'intron', 'cds_exon', 'utr3exon', 'utr5exon', 'intergenic'} ;

P.train_on = 1;
P.use_old_files = 0 ;
for content_idx = 1:length(content_names) ;
  num_jobs = num_jobs + 1;
  P.content_name = content_names{content_idx} ;
  
  % label generation 
  P.cpf_label_fname = sprintf('%s/content_%s_label.cpf',output_dir, P.content_name) ;
  P.cpf_label_dir = sprintf('%s/content_%s_label_files',output_dir, P.content_name) ;
  
  % training
  P.tcp_train_info = sprintf('%s/content_%s_classifier_info',output_dir, P.content_name) ;
  P.tcp_train_dir = sprintf('%s/content_%s_classifier_files',output_dir, P.content_name) ;
  
  % prediction
  P.cpf_pred_fname = sprintf('%s/content_%s_predictions.spf',output_dir, P.content_name) ;
  P.cpf_pred_dir = sprintf('%s/content_%s_predictions_files',output_dir, P.content_name) ;
  
  % evaluation
  P.eval_fname = sprintf('%s/content_%s_eval.txt',output_dir, P.content_name) ;
  
  if run_locally==-1,
    run_locally_content_wrapper = rproc_policy('content_wrapper', genome_info) ;
  else
    run_locally_content_wrapper = run_locally ;
  end ;

  if ~run_locally_content_wrapper,
    [mem_req, time_req, opts] = rproc_memtime_policy(['content_wrapper:' content_names{content_idx}], [], options) ;

    jobinfo(num_jobs) = rproc('content_wrapper', P, mem_req, opts,time_req);
  else
    content_wrapper(P);
  end
  train_dirs.(P.content_name) =  P.tcp_train_dir;
  train_files.(P.content_name) = P.tcp_train_info;
  pred_dirs.(P.content_name) =  P.cpf_pred_dir;
  pred_files.(P.content_name) = P.cpf_pred_fname;
end

if ~isempty(jobinfo),
  [jobinfo,num_crashed] = rproc_wait(jobinfo, 60, 1, -1);
end

save([output_dir '/prediction_files.mat'],'pred_dirs', '-v7');

fd=fopen(fn_mGene_predictor, 'a+') ; fprintf(fd, '\nUnderlying signal predictors:\n-----------------------------\n\n') ; fclose(fd) ;
for signal_idx = 1:length(signal_names)
  unix(sprintf('cat %s >> %s', train_files.(signal_names{signal_idx}), fn_mGene_predictor)) ;
  fd=fopen(fn_mGene_predictor, 'a+') ; fprintf(fd, '\n\n') ; fclose(fd) ;
end ;

fd=fopen(fn_mGene_predictor, 'a+') ; fprintf(fd, '\n\nUnderlying content predictors:\n------------------------------\n\n') ; fclose(fd) ;
for content_idx = 1:length(content_names) ;
  unix(sprintf('cat %s >> %s', train_files.(content_names{content_idx}), fn_mGene_predictor)) ;
  fd=fopen(fn_mGene_predictor, 'a+') ; fprintf(fd, '\n\n') ; fclose(fd) ;
end ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% TRAIN, PREDICT AND EVAL Gene Structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


logfile = [output_dir '/log_train'];
pred_dirs.gene_predictor = sprintf('%s/gene_predictor/',output_dir);
fn_gene_predictor = sprintf('%s/gene_predictor.dat',output_dir);

save([output_dir '/trained_model.mat'],'train_dirs', '-v7');

signal_fns = ''; % spf-files if no matlab files 
content_fns = ''; % cpf-files if no matlab files 

save('workspace_mgenetrain', '-v7')
galaxy_gene_train(config_dir, annotation_dir, logfile, ...
           signal_fns, pred_dirs.tss, signal_fns, pred_dirs.tis,...  
	   signal_fns, pred_dirs.acc, signal_fns, pred_dirs.don,... 
           signal_fns, pred_dirs.cdsStop, signal_fns, pred_dirs.cleave,...
           content_fns, pred_dirs.intergenic, content_fns, pred_dirs.utr5exon, ...
           content_fns, pred_dirs.cds_exon, content_fns, pred_dirs.intron,...
           content_fns, pred_dirs.utr3exon, fn_gene_predictor, pred_dirs.gene_predictor, train_options);


galaxy_make_mgene_predictor(fn_gene_predictor, pred_dirs.gene_predictor, ...
			train_files.tss, train_dirs.tss, train_files.tis, train_dirs.tis,...
			train_files.acc, train_dirs.acc, train_files.don, train_dirs.don,...
			train_files.cdsStop, train_dirs.cdsStop, train_files.cleave, train_dirs.cleave, ... 
			train_files.intergenic, train_dirs.intergenic, train_files.utr5exon, train_dirs.utr5exon,...
			train_files.cds_exon, train_dirs.cds_exon, train_files.intron, train_dirs.intron,...
			train_files.utr3exon, train_dirs.utr3exon, fn_mGene_predictor, mGene_predictor_dir);


[ret, timedate] = unix('date') ;
timedate(timedate==sprintf('\n')) = [] ;
fprintf(fd,'finished mgene_train at %s\n', timedate) ;


fd=fopen(fn_mGene_predictor, 'a+') ; 
fprintf(fd, '\n mGene.web Train finished : %s\n', timedate) ;  
fclose(fd) ;

return


function [ ok ] = nice_mkdir( dir );
if ~isdir(dir)
  ok = ( 0 == unix( [ 'mkdir -p ', dir ] ) );
  ok = ok & ( 0 == unix( [ 'chmod g+t ', dir ] ) );
end


return

function [engine, environment, basedir, mccdir, license_type] = determine_engine() ;
% [engine, environment, basedir, mccdir, license_type] = determine_engine() ;
% 
% returns either 'matlab' or 'octave' in variable engine
% and 'internal' or 'galaxy' in variable environment
% basedir is the directory of the matlab or octave instance
% mccdir is the directory of the matlab compiler (does not yet exist for octave)

global g_license_type ;

if isempty(g_license_type),
  g_license_type = 'academic' ;
end ;
license_type = g_license_type ;

lic=license ;
if ~isequal(lic, 'GNU General Public License'),
  engine='matlab' ;
else
  engine='octave' ;
end ;

environment='internal' ;
if isequal(whoami, 'galaxy'),
  environment='galaxy' ;
end ;

if isequal(engine, 'matlab') && isequal(environment,  'internal'),
  basedir = '/fml/ag-raetsch/share/software/matlab/' ;
elseif isequal(engine, 'matlab') && isequal(environment,  'galaxy'),
  basedir = '/home/galaxy/matlab-7.6/' ;
elseif isequal(engine, 'octave') && isequal(environment,  'internal'),
  basedir = '/fml/ag-raetsch/share/software/octave-3.0.3/' ;
elseif isequal(engine, 'octave') && isequal(environment,  'galaxy'),
  basedir = '/home/galaxy/octave-3.0.4' ;
end ;

if isequal(environment,  'internal'),
  mccdir = '/fml/ag-raetsch/share/software/matlab/' ;
elseif isequal(environment,  'galaxy'),
  mccdir = '/home/galaxy/matlab-7.6/' ;
end ;

return

return

function x = fexist(filename,gz)
% FEXIST Check if file exists
%        FEXIST('filename') returns:
%        0 if filename does not exist
%        1 if filename exists in the given path

if nargin<2,
  gz=0;
end ;

fd = fopen(filename);
if fd ~= -1
  x = 1;
  fclose(fd);
elseif gz,
  fd = fopen([filename '.gz']);
  if fd~=-1,
    x=1 ;
    fclose(fd) ;
  else
    x=0 ;
  end ;
else
  x = 0;
end

return

function f=separate(str, delim, merge_multiple) ;
% f=separate(str, delimiter, merge_multiple_delimiters) ;
  
if nargin<2, delim=sprintf('\t') ; end ;
if nargin<3, merge_multiple=0 ; end ;
f={} ;
idx=[0 find(str==delim) length(str)+1] ;
if merge_multiple,
  idx2=find(idx(1:end-1)~=idx(2:end)-1) ;
  idx=[idx(idx2) idx(end)] ;
end 
for i=1:length(idx)-1
  f{i}=deblank(str(idx(i)+1:idx(i+1)-1)) ;
end ;

return

function ret=keyboard_allowed()
% ret=keyboard_allowed()
%
% returns 1, if a keyboard command would be allowed

global g_ignore_keyboard
global THIS_IS_A_RPROC_PROCESS  

if isequal(g_ignore_keyboard, 1),
  ret=0 ;
  return ;
end ;

if isequal(THIS_IS_A_RPROC_PROCESS, 1),
  ret=0 ;
  return ;
end ;

ret=1 ;


return

function genome_info=init_genome(config_fname)
% genome_info=init_genome(config_fname)  
%
% global g_genome_info ;

global g_genome_info genome_strings;

genome_strings = [];
g_est_fnames={} ;
g_prot_fnames={} ;
g_cdna_fnames={} ;
g_flcdna_fnames={} ;
g_contig_names={} ;
g_flat_fnames={} ;
g_fasta_fnames={} ;
g_annotation_fnames={} ;
g_alphabet='acgt' ;
g_basedir='' ;
g_max_intron_len=1000000 ;
g_min_intron_len=10 ;
g_merge_est_transcripts=1 ;

fd=fopen(config_fname, 'r') ;
while ~feof(fd),
  line = fgetl(fd) ;
  if ~ischar(line), break ; end ;
  if isempty(line), continue ; end ;
  if line(1)=='#', continue ; end ;
  if length(line)>8 && isequal(line(1:7),'BASEDIR')
    g_basedir=deblank(line(9:end)) ;
  elseif length(line)>21 && isequal(line(1:21), 'MERGE_EST_TRANSCRIPTS')
    g_merge_est_transcripts = str2num(line(22:end)) ; 
  elseif length(line)>15 && isequal(line(1:15),'MAX_DOUBLE_QGAP')
  elseif length(line)>15 && isequal(line(1:15),'MAX_DOUBLE_TGAP')
  elseif length(line)>17 && isequal(line(1:17),'MAX_CDNA_DELETION')
  elseif length(line)>18 && isequal(line(1:18),'MAX_CDNA_INSERTION')
  elseif length(line)>21 && isequal(line(1:21),'TERMINAL_EXON_END_TOL')
  elseif length(line)>14 && isequal(line(1:14),'MIN_INTRON_LEN')
    g_min_intron_len = str2num(line(15:end)) ; 
    assert(~isempty(g_min_intron_len) & g_min_intron_len<1000) ;
  elseif length(line)>14 && isequal(line(1:14),'MAX_INTRON_LEN')
    g_max_intron_len = str2num(line(15:end)) ; 
    assert(~isempty(g_max_intron_len) & g_max_intron_len>1000) ;
  elseif length(line)>18 && isequal(line(1:18),'MIN_EST_COVER_FRAC')

  elseif length(line)>19 && isequal(line(1:19),'MIN_PROT_COVER_FRAC')

  elseif length(line)>19 && isequal(line(1:19),'MIN_CDNA_COVER_FRAC')
  
  elseif length(line)>18 && isequal(line(1:18),'BLAT_BEST_HIT_ONLY')
  
  elseif length(line)>20 && isequal(line(1:20),'BLAT_BEST_HIT_MARGIN')

  elseif length(line)>8 && isequal(line(1:8),'ALPHABET')
    g_alphabet=deblank(line(10:end)) ;
  elseif length(line)>7 && isequal(line(1:7),'CONTIGS'),
    for i=1:str2num(line(8:end))
      l2=fgetl(fd) ; 
      idx=strfind(l2,'  ') ;
      while ~isempty(idx), 
        l2(idx)=[] ;
        idx=strfind(l2,'  ') ;
      end ;
      l2(l2==' ')=sprintf('\t') ;
      elems=separate(l2) ; 
      assert(length(elems)==3) ;
      g_contig_names{i}=elems{1} ;
      g_flat_fnames{i}=path_with_basedir(elems{2}, g_basedir) ;
      g_fasta_fnames{i}=path_with_basedir(elems{3}, g_basedir) ; 
    end ;
  elseif length(line)>8 && isequal(line(1:8),'ESTFILES'),
    for i=1:str2num(line(9:end))
      g_est_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>9 && isequal(line(1:9),'PROTFILES'),
    for i=1:str2num(line(10:end))
      g_prot_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>9 && isequal(line(1:9),'CDNAFILES'),
    for i=1:str2num(line(10:end))
      g_cdna_fnames{end+1}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>11 && isequal(line(1:11),'FLCDNAFILES'),
    for i=1:str2num(line(12:end))
      g_flcdna_fnames{end+1}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>15 && isequal(line(1:15),'ANNOTATIONFILES'),
    for i=1:str2num(line(16:end))
      g_annotation_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif ~isempty(line),
    error(sprintf('cannot understand "%s" in config file', line)) ;
  end ;
end ;
fclose(fd) ;

genome_info=[] ;
genome_info.est_fnames=g_est_fnames ;
genome_info.prot_fnames=g_prot_fnames ;
genome_info.cdna_fnames=g_cdna_fnames ;
genome_info.flcdna_fnames=g_flcdna_fnames ;
genome_info.contig_names=g_contig_names ;
genome_info.flat_fnames=g_flat_fnames ;
genome_info.fasta_fnames=g_fasta_fnames ;
genome_info.annotation_fnames=g_annotation_fnames ;
genome_info.alphabet=g_alphabet ;
genome_info.basedir=g_basedir ;
genome_info.max_intron_len = g_max_intron_len ;
genome_info.min_intron_len = g_min_intron_len ;
genome_info.merge_est_transcripts = g_merge_est_transcripts ;
g_genome_info=genome_info ;

return ;


function p=path_with_basedir(p1,basedir)
  
  if p1(1)=='/',
    p=p1; 
    return ;
  else
    if basedir(end)=='/',
      p=[basedir p1] ;
    else
      p=[basedir '/' p1] ;
    end ;
  end ;
  
  

return

function array = load_struct(infile_name, arrayname)
% array = load_struct(infile_name, arrayname)
  
if nargin<2,
  arrayname='array' ;
end ;

L_=load(infile_name) ;
if isfield(L_,'numvars')
  fprintf('arrayload2 loads %i chunks:', L_.numvars) ;
  ix=1;
  L=load(sprintf('%s_%i.mat', infile_name, ix), sprintf('%s%d', arrayname, ix)) ;
  eval(sprintf('array = L.%s1;', arrayname)) ;
  fprintf('.') ;
  for ix = 2:L_.numvars
    L=load(sprintf('%s_%i.mat', infile_name, ix), sprintf('%s%d', arrayname, ix)) ;
    if size(array,1) == 1
      array = [array, eval(sprintf('L.%s%d', arrayname, ix))];
    elseif size(array,2) == 1
      array = [array; eval(sprintf('L.%s%d', arrayname, ix))];
    end
    fprintf('.') ;
  end
  fprintf('\n') ;
  if isfield(L_, 'dimensions'),
    array = reshape(array, L_.dimensions) ;
  end ;
else
  eval(sprintf('array=L_.%s;', arrayname)) ;
end

return

