%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% Written (W) 2005-2009 Gunnar Raetsch, Gabriele Schweikert, Jonas Behr, Soeren Sonnenburg, Alexander Zien, Georg Zeller, Andre Noll, Cheng Soon Ong, Petra Philips
% Copyright (C) 2005-2009 Max Planck Society and Fraunhofer Institute FIRST 
%
function  [dummy1, dummy2]=load_and_predict(P, Test_Data)
% load_and_predict(P, Test_Data)

% used for both evaluation on the leftout set of the training region and
% prediction on test regions
% 
% * IF PAR.eval_on is set to 1 prediction is performed on the evaluation
%   set specified by PAR.SETs.which_val. The model saved in PAR.fn_SVMs is used for
%   prediction. Additionally RSV and RFV are determined.
%  
% * IF PAR.eval_on is set to 0  only prediction is performed on the test set
%   specified by PAR.SETs.which_pred.
% 
% 

  dummy1 = [] ;
  dummy2 = [] ;

fn_SVMs = P.fn_SVMs ;
SETs = P.SETs;
pred_from_seq = P.pred_from_seq;
Signal = P.Signal;
fn_examples = P.fn_examples;
fn_filter_settings = P.fn_filter_settings;

if isfield(P,'avg_all_svms') 
  avg_all_svms = P.avg_all_svms;
else
  avg_all_svms = 0;
end

if isfield(P,'check_overlap')
  check_overlap = P.check_overlap;
else
  check_overlap = 1;
end

if isfield(P,'eval_on') && P.eval_on
  which_set = SETs.which_val;
  if isfield(P,'fn_eval')
    fn_pred = P.fn_eval;
  else
    fn_pred = [];
  end
elseif isfield(P,'plif_on') && P.plif_on
  which_set = SETs.which_plif;
  if isfield(P,'fn_plifs')
    fn_pred = P.fn_plifs;
  else
    fn_pred = [];
  end
else
  which_set = SETs.which_pred;
  if isfield(P,'fn_pred')
    fn_pred = P.fn_pred;
  else
    fn_pred = [];
  end
end

if pred_from_seq
  fn_pos = P.fn_pos;
  if isequal(fn_pos(1),'~')
    fn_pos = sprintf('%s/%s', getenv('HOME'), fn_pos(2:end));
  end
  region = P.region;
  fn_pred = P.fn_pred;
  chunk_idx = 1;% P.chunk_idx;
  RPROC = struct;
end

load(fn_SVMs, 'Train') ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% load data 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isfield(SETs,'test_subsample_pos')
  subsample_pos = 1;
else
  subsample_pos = SETs.test_subsample_pos;
end
if ~isfield(SETs,'test_subsample_neg')
  subsample_neg = 1;
else
  subsample_neg= SETs.test_subsample_neg;
end


if ~pred_from_seq
  pred_from_seq=0;
  if ~exist('Test_Data')
    Test_Data = feval(Signal.load_fct,Signal,fn_filter_settings,fn_examples, which_set, 'test',subsample_neg, subsample_pos);
  end
  
  %  if isequal(PAR.method.name,'SVM')
    % [RFV RSV]=tssTest(Train, Test_Data, fn_SVMs, fn_pred);
    % varargout{1} = RFV;
    % varargout{2} = RSV;
 %  elseif isequal(PAR.method.name,'PWM')
    % hmm_test(Train, Test_Data, fn_SVMs,fn_pred);
 %  end
else %genomweite Vorhersage
  
  region = load_sequence(region);
  region = retrieve_signals(region,fn_pos,Signal.name,{'svm'});
  Test_Data.seq = region.seq;
  Test_Data.pos = region.Signals.(Signal.name).pos';
  Test_Data.svm = region.Signals.(Signal.name).svm';
  idx = find((Test_Data.pos<Signal.lwin_big+1)|(Test_Data.pos>length(Test_Data.seq)-Signal.rwin_big-1));
  if length(idx)==length(Test_Data.pos), 
    % error('no examples in valid region found'); 
  end
  Test_Data.pos(idx)=[];
  Test_Data.svm(idx)=[];
  % load(PAR.fn_pos,'pos','seq','region_id','svm_num') ;
  if ~all(ismember(unique(Test_Data.seq),['ACGT']))
    warning('sequence alphabet different from ACGT; setting all Ns to A')
    Test_Data.seq(~ismember(Test_Data.seq,['ACGT']))='A';
  end
  if ~avg_all_svms
    idx = find(Test_Data.svm==which_set);
  else
    idx = 1:length(Test_Data.svm);
  end
  if isequal(Signal.method.predict_fct,'svm_content_predict')
    idx = [idx length(Test_Data.svm)];
  end
  if length(idx)==0, warning('no examples for specified svm found'); end
  Test_Data.pos = Test_Data.pos(idx);
  Test_Data.svm = Test_Data.svm(idx);
  if isfield(RPROC, 'divide4pred') && RPROC.divide4pred
    junk_length = ceil(length(Test_Data.pos)/RPROC.numjunks) ;
    idx_sub = (chunk_idx-1)*junk_length+1:min(chunk_idx*junk_length,length(Test_Data.pos));
    Test_Data.pos = Test_Data.pos(idx_sub);
    Test_Data.svm = Test_Data.svm(idx_sub);
    fn_pred = sprintf('%s%i.mat',fn_pred, chunk_idx);
  end
  %   Test_Data.region_id = Test_Data.region_id(idx);
end

Test_Data.PAR = P;

feval(Signal.method.predict_fct, Train, Test_Data,fn_SVMs,fn_pred,pred_from_seq, check_overlap);

return

function PAR=exp_galaxy_A_thaliana(PAR)

if nargin<1 
  PAR=init_PAR;
end

PAR.date=date;
PAR.exp_name='';
PAR.dir_name ='';

PAR.organism.name = 'A_thaliana';  

PAR.SETs.num_partitions=5;
PAR.SETs.num_splits=5;
%PAR.SETs.which_pred_svm = 1;

PAR.tasks.modelsel_on = 1;
PAR.tasks.learn_plifs_on =1;
PAR.tasks.pred_from_seq=1;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;
PAR.tasks.prepare_labels=0;
PAR.tasks.pred_on_genome = 0 ;
PAR.tasks.train_on = 1;
PAR.tasks.run_locally=1;

PAR.FN.input_dir='' ;
PAR.FN.output_dir='' ;

PAR.RPROC.collect_jobs = 1;
PAR.RPROC.options.ncpus = 1;
PAR.RPROC.exm_per_batch = 1;
PAR.RPROC.time_req = 3600;
PAR.RPROC.MEMREQ = 3000;

return

function PAR=exp_galaxy_C_elegans(PAR)

if nargin<1 
  PAR=init_PAR;
end

PAR.date=date;
PAR.exp_name='';
PAR.dir_name ='';

PAR.organism.name = 'C_elegans'; 

PAR.SETs.num_partitions=1;
PAR.SETs.num_splits=1;
PAR.SETs.which_pred_svm = 1;

PAR.tasks.modelsel_on = 0;
PAR.tasks.learn_plifs_on =0;
PAR.tasks.pred_from_seq=1;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;
PAR.tasks.prepare_labels=0;
PAR.tasks.pred_on_genome = 1;
PAR.tasks.train_on = 0;
PAR.tasks.run_locally=1;

PAR.FN.input_dir='/tmp';
PAR.FN.output_dir='/tmp';

PAR.RPROC.collect_jobs = 1;
PAR.RPROC.options.ncpus = 1;
PAR.RPROC.exm_per_batch = 1;
PAR.RPROC.time_req = 3600;
PAR.RPROC.MEMREQ = 3000;


return

function PAR=exp_galaxy_D_melanogaster(PAR)

if nargin<1 
  PAR=init_PAR;
end

PAR.date=date;
PAR.exp_name='';
PAR.dir_name ='';

PAR.organism.name = 'D_melanogaster';  

PAR.SETs.num_partitions=1;
PAR.SETs.num_splits=1;
PAR.SETs.which_pred_svm = 1;

PAR.tasks.modelsel_on = 0;
PAR.tasks.learn_plifs_on =0;
PAR.tasks.pred_from_seq=1;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;
PAR.tasks.prepare_labels=0;
PAR.tasks.pred_on_genome = 1;
PAR.tasks.train_on = 0;
PAR.tasks.run_locally=1;

PAR.FN.input_dir='/tmp';
PAR.FN.output_dir='/tmp';

PAR.RPROC.collect_jobs = 1;
PAR.RPROC.options.ncpus = 1;
PAR.RPROC.exm_per_batch = 1;
PAR.RPROC.time_req = 3600;
PAR.RPROC.MEMREQ = 3000;


return

function PAR=exp_galaxy_D_rerio(PAR)

if nargin<1 
  PAR=init_PAR;
end

PAR.date=date;
PAR.exp_name='';
PAR.dir_name ='';

PAR.organism.name = 'D_rerio';  

PAR.SETs.num_partitions=1;
PAR.SETs.num_splits=1;
PAR.SETs.which_pred_svm = 1;

PAR.tasks.modelsel_on = 0;
PAR.tasks.learn_plifs_on =0;
PAR.tasks.pred_from_seq=1;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;
PAR.tasks.prepare_labels=0;
PAR.tasks.pred_on_genome = 1;
PAR.tasks.train_on = 0;
PAR.tasks.run_locally=1;

PAR.FN.input_dir='/tmp';
PAR.FN.output_dir='/tmp';

PAR.RPROC.collect_jobs = 1;
PAR.RPROC.options.ncpus = 1;
PAR.RPROC.exm_per_batch = 1;
PAR.RPROC.time_req = 3600;
PAR.RPROC.MEMREQ = 3000;


return

function regions=init_regions(fn_genome_config, organism)
% regions=init_regions(fn_genome_config,organism)

  
regions(1).chr = [];
regions(1).chr_num = [];
regions(1).strand = [];
regions(1).start = [];
regions(1).stop = [];
regions(1).offset =[];
regions(1).id = [];
regions(1).organism = [];
regions(1).config = [];

if ~exist('fn_genome_config')
  return ;
else
  regions = regions([]) ;
  genome_config = init_genome(fn_genome_config);
  strands = '+-';
  for c=1:length(genome_config.contig_names)
    %+strand
    d = dir(genome_config.flat_fnames{c}) ;
    for s=1:length(strands)
      region = [];
      region.chr=genome_config.contig_names{c};
      region.chr_num = c;
      region.strand=strands(s);
      region.start = 1;
      region.stop = d.bytes ;
      switch strands(s)
       case '+',
        region.offset      = region.start - 1;
       case '-',
        region.offset      = region.stop + 1;
      end
      region.id = (c-1)*2+s;
      region.organism = [];
      if exist('organism')
        region.organism = organism;
      end
      region.config = fn_genome_config;
      regions(region.id) = region;
    end
  end
end


return

function PAR=create_PAR(Exp, PAR_config, verb)
%  PAR=create_PAR(Exp,PAR_config,verb)
%
% creates PAR with all parameters for one genefinding
% experiment. For a new Organism X a new template file must be added to /template_files/Organisms/X.m. 
% (e.g. C_elegans.m) 
% For each signal a standard method is applied (usually SVM with one or
% more spectrum kernels)
%  
% INPUT  - Exp: filename of current experiment  (e.g. exp_C_elegans_nGASP)
%        - verb: verbose
%
% OUTPUT - PAR. struct with all information needed for genefinding experiment  
  
Signals = {'acceptor','donor','tis','cdsStop','tss','cleave','polya','transacceptor'};
sig_names = {'acc','don','tis','cdsStop','tss','cleave','polya','transacc'};
Methods = {'SVM_WD','SVM_WD','SVM_COMBINED','SVM_COMBINED','SVM_COMBINED','SVM_COMBINED','SVM_COMBINED','SVM_WD'};

content_names = {'intergenic','utr5exon','cds_exon','utr3exon','polya_tail','intron','frame0'};
Methodc = 'SVM_Cont';
changeable_fields = {'organism','label_source','Signals','Contents','LSL','RPROC','SETs','regions','tasks','Source_organisms','FN'} ;

if nargin<3,
  verb=0;
end


PAR = init_PAR;

if nargin>0
  fprintf('parameters from exp file: %s \n',Exp)
  if ~exist('PAR_config') || isempty(PAR_config),
    PAR_exp = feval(Exp) ;
  else
    try,
      PAR_exp = feval(Exp, PAR_config) ;
    catch
      PAR_exp = feval(Exp) ;
    end 
    clear PAR_config ;
  end ;
  assert(all(ismember(fieldnames(PAR_exp),changeable_fields)))
  if ~isfield(PAR_exp,'Contents')
    PAR_exp.Contents = struct;
  end
  if ~isfield(PAR_exp,'Signals')
    PAR_exp.Signals = struct;
  end
  PAR.organism = set_default(PAR_exp,'organism',PAR.organism)  ;  
  PAR.label_source = set_default(PAR_exp,'label_source',PAR.label_source) ;
  PAR.RPROC = set_default(PAR_exp,'RPROC',PAR.RPROC)  ; 
  PAR.SETs = set_default(PAR_exp,'SETs',PAR.SETs)  ; 
  PAR.SETs = make_partitions(PAR.SETs);
  PAR.regions = set_default(PAR_exp,'regions',PAR.regions)  ; 
  PAR.tasks = set_default(PAR_exp,'tasks',PAR.tasks)  ; 
  PAR.Source_organisms = set_default(PAR_exp,'Source_organisms',PAR.Source_organisms)  ; 
end

if isempty(PAR.organism.name),
  [organism, label_source] = dummy_organism(PAR.label_source, sig_names);
else
  [organism, label_source] = feval(PAR.organism.name,PAR.label_source,sig_names);
end
PAR.organism = set_default(PAR,'organism',organism)  ;
PAR.label_source = set_default(PAR,'label_source',label_source)  ;

%-----------------------------
%%% prepare all signals
%-----------------------------

for i=1:length(Signals)
  Signal = feval(Signals{i},PAR.label_source);
  if exist('PAR_exp')
    if isfield(PAR_exp.Signals, sig_names{i}) && isfield(PAR_exp.Signals.(sig_names{i}),'method')
      me = PAR_exp.Signals.(sig_names{i}).method;
      PAR_exp.Signals.(sig_names{i}) = rmfield(PAR_exp.Signals.(sig_names{i}),'method');
    end
    Signal = set_default(PAR_exp.Signals,sig_names{i},Signal)  ;
  end
  Signal.method = feval(Methods{i},Signal);
  if exist('me')
    PAR_exp.Signals.(sig_names{i}).method = me;
    Signal = set_default(PAR_exp.Signals,sig_names{i},Signal)  ;
    clear me
  end
  % Signal = Signal.domain_adapt.transfer_fct(Signal);
  PAR.Signals.(sig_names{i}) = Signal;
end

%-----------------------------
%%% prepare all contents
%-----------------------------
for i=1:length(content_names)
  Content = feval(content_names{i});
  if exist('PAR_exp')
    if isfield(PAR_exp.Contents, content_names{i}) && isfield(PAR_exp.Contents.(content_names{i}),'method')
      me = PAR_exp.Contents.(content_names{i}).method;
      PAR_exp.Contents.(content_names{i}) = rmfield(PAR_exp.Contents.(content_names{i}),'method');
    end
    Content = set_default(PAR_exp.Contents,content_names{i},Content)  ;
  end
  Content.method = feval(Methodc,Content);
  if exist('me')
    PAR_exp.Contents.(content_names{i}).method = me;
    Content = set_default(PAR_exp.Contents,content_names{i},Content)  ;
    clear me
  end
  % Content = Content.domain_adapt.transfer_fct(Content);
  PAR.Contents.(content_names{i}) = Content;
end

%-----------------------------
%%% prepare LSL
%-----------------------------

LSL = lsl_template; 
% LSL = LSL.domain_adapt.transfer_fct(LSL);
if exist('PAR_exp')
  LSL = set_default(PAR_exp,'LSL',LSL)  ; 
end
PAR.LSL=LSL;

%-----------------------------
%%% prepare filenames
%-----------------------------
FN = init_FN;
if  exist('PAR_exp')
  FN = set_default(PAR_exp,'FN',FN)  ;
end

FN = create_filenames(FN,PAR.organism,PAR.Signals,PAR.Contents,verb);
if  exist('PAR_exp')
  FN = set_default(PAR_exp,'FN',FN)  ;
end
PAR.FN=FN;

%-----------------------------
%%% Add polya consensus if file available
%-----------------------------

if fexist(PAR.FN.input_sig.polya.fn_polya)
  load(PAR.FN.input_sig.polya.fn_polya,'consensus')
  PAR.Signals.polya.consensus = consensus;
end

%PAR.FN.source_directory = '~/svn/projects/genefinding/';
%[tmp,PAR.FN.svn_info]=unix('svn info');

PAR.FN.svn_info = '' ;

fprintf('loading model from %s\n', PAR.LSL.method.model_fct)
PAR.model = feval(PAR.LSL.method.model_fct,PAR.organism,PAR.LSL, ...
                  PAR.Signals);


return

function PAR=A_thaliana_acc_SVM_WD(PAR,create_dir)
  
if nargin<1
  PAR=init_PAR;
elseif nargin<2
  create_dir=0;
end
PAR = A_thaliana(PAR);
PAR = acceptor(PAR);
PAR = SVM_WD(PAR);

PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = make_partitions(PAR);
PAR = create_filenames(PAR);
if create_dir
  create_directories(PAR);
end

return

function PAR=A_thaliana_don_SVM_WD(PAR,create_dir)

if nargin<1
  PAR = init_PAR;
end
if nargin<2
  create_dir=0;
end
  
PAR = A_thaliana(PAR);
PAR = donor(PAR);
PAR = SVM_WD(PAR);

PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = make_partitions(PAR);
PAR = create_filenames(PAR);
if create_dir
  create_directories(PAR);
end


return

function PAR=C_elegans_acc_SVM_WD(PAR,create_dir)
  
if nargin<1
  PAR=[];
end
if nargin<2
  create_dir=0;
end
  
PAR = C_elegans(PAR);
PAR = acceptor(PAR);
PAR = SVM_WD(PAR);

PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = make_partitions(PAR);
PAR = create_filenames(PAR);

if create_dir
  create_directories(PAR);
end

return

function PAR=C_elegans_don_SVM_WD(PAR,create_dir)
  
if nargin<1
  PAR=[];
end
if nargin<2
  create_dir=0;
end
  
PAR = C_elegans(PAR);
PAR = donor(PAR);
PAR = SVM_WD(PAR);

PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = make_partitions(PAR);
PAR = create_filenames(PAR);

if create_dir
  create_directories(PAR);
end

return

function PAR=D_melanogaster_acc_SVM_WD(PAR)
  
if nargin<1
  PAR=[];
end
  
PAR = D_melanogaster(PAR);
PAR = acceptor(PAR);
PAR = SVM_WD(PAR);

PAR = make_partitions(PAR);

PAR.date=date;
PAR.user='Jonas';
PAR.exp_name='acceptor';

%PAR.method.kernels{1}.lwin=PAR.Signals.(PAR.Signal_name).lwin;
%PAR.method.kernels{1}.rwin=PAR.Signals.(PAR.Signal_name).rwin;
PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = create_filenames(PAR);

PAR.tasks.modelsel_on = 1;
PAR.tasks.learn_plifs_on = 1;
PAR.tasks.pred_from_seq = 0;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;

% this is the setting for the splice paper
PAR.regions.cluster_paralogs = 0;

PAR.RPROC.MEMREQ = 8000 ;

PAR.method.kernels{1}.order=22 ;
PAR.method.par_ms.C = 3 ;

return

function PAR=D_melanogaster_don_SVM_WD(PAR)
  
if nargin<1
  PAR=[];
end
  
PAR = D_melanogaster(PAR);
PAR = donor(PAR);
PAR = SVM_WD(PAR);

PAR = make_partitions(PAR);

PAR.date=date;
PAR.user='Jonas';
PAR.exp_name='donor';

%PAR.method.kernels{1}.lwin=PAR.Signals.(PAR.Signal_name).lwin;
%PAR.method.kernels{1}.rwin=PAR.Signals.(PAR.Signal_name).rwin;
PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = create_filenames(PAR);

PAR.tasks.modelsel_on = 1;
PAR.tasks.learn_plifs_on =1;
PAR.tasks.pred_from_seq=0;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;

% this is the setting for the splice paper
PAR.regions.cluster_paralogs = 0;

PAR.RPROC.MEMREQ = 8000 ;

PAR.method.kernels{1}.order=22 ;
PAR.method.par_ms.C = 3 ;

return

function PAR=D_rerio_acc_SVM_WD(PAR)
  
if nargin<1
  PAR=[];
end
  
PAR = D_rerio(PAR);
PAR = acceptor(PAR);

% test only one model ... 
PAR.Signals.(PAR.Signal_name).order = 22;
PAR.Signals.(PAR.Signal_name).shift = 0;
PAR.Signals.(PAR.Signal_name).C = 3; 

PAR = SVM_WD(PAR);

PAR = make_partitions(PAR);

PAR.date=date;
PAR.user='Jonas';
PAR.exp_name='acceptor';

PAR.method.kernels{1}.lwin=PAR.Signals.(PAR.Signal_name).lwin;
PAR.method.kernels{1}.rwin=PAR.Signals.(PAR.Signal_name).rwin;
PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = create_filenames(PAR);

PAR.tasks.modelsel_on = 1;
PAR.tasks.learn_plifs_on =1;
PAR.tasks.pred_from_seq=0;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;

PAR.SETs.train_subsample_pos = 1 ;
PAR.SETs.train_subsample_neg = 0.33 ;

PAR.RPROC.options.ncpus = 12 ; 

return

function PAR=D_rerio_don_SVM_WD(PAR)
  
if nargin<1
  PAR=[];
end
  
PAR = D_rerio(PAR);
PAR = donor(PAR);

% test only one model ... 
PAR.Signals.(PAR.Signal_name).order = 22;
PAR.Signals.(PAR.Signal_name).shift = 0;
PAR.Signals.(PAR.Signal_name).C = 3; 

PAR = SVM_WD(PAR);

PAR = make_partitions(PAR);

PAR.date=date;
PAR.user='Jonas';
PAR.exp_name='donor';

PAR.method.kernels{1}.lwin=PAR.Signals.(PAR.Signal_name).lwin;
PAR.method.kernels{1}.rwin=PAR.Signals.(PAR.Signal_name).rwin;
PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = create_filenames(PAR);

PAR.tasks.modelsel_on = 1;
PAR.tasks.learn_plifs_on =1;
PAR.tasks.pred_from_seq=0;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;

PAR.SETs.train_subsample_pos = 1 ;
PAR.SETs.train_subsample_neg = 0.33 ;

PAR.RPROC.options.ncpus = 12 ; 

return

function PAR=init_PAR(signal_names,content_names);
 
  
% -------------------------------------------------------------------------------------
% Experiments
% -------------------------------------------------------------------------------------
if nargin<1
  signal_names = {'acc','don','tis','cdsStop','tss','cleave','polya','transacc'};
end
if nargin<2
  content_names = {'intergenic','utr5exon','cds_exon','utr3exon','polya_tail','intron','frame0'};
end




PAR.date = date;
PAR.user = whoami;
  
PAR.organism.clade = [];
PAR.organism.name = '';
PAR.organism.full_name = [];
PAR.organism.release = [];
PAR.organism.version = [];
PAR.organism.genebuilt = [];

% -------------------------------------------------------------------------------------
% info regarding original gene structure
% -------------------------------------------------------------------------------------


info_genes.transcript.anno = -1;
info_genes.transcript.est = 0;
info_genes.transcript.cDNA = 1;
info_genes.transcript.fulllength = 2;

info_genes.transcript_status.confirmed = 1;
info_genes.transcript_status.partially_confirmed = 0;
info_genes.transcript_status.predicted = -1;


info_genes.pseudo.dummy = [];

% -------------------------------------------------------------------------------------
% info regarding label source
% -------------------------------------------------------------------------------------

label_source.use_anno = [];
label_source.use_confirmed = [];
for j=1:length(signal_names)
  signal_name = signal_names{j};
  label_source.(signal_name).anno = [];
  label_source.(signal_name).est = [];
  label_source.(signal_name).cDNA = [];
  label_source.(signal_name).fulllength = [];
  label_source.(signal_name).db = [];
  label_source.(signal_name).from_fn_candsites = [];
  
  info_genes.(signal_name).anno = -1;
  info_genes.(signal_name).est = 0;
  info_genes.(signal_name).cDNA = 1;
  info_genes.(signal_name).fulllength = 2;
  info_genes.(signal_name).db = 3;
  info_genes.(signal_name).from_fn_candsites = 4;
end
for j=1:length(content_names)
  content_name = content_names{j};
  label_source.(content_name).from_fn_candsites = [];
end

label_source.tis.maxorf = [];
info_genes.tis.maxorf = 5;
label_source.cdsStop.maxorf = [];
info_genes.cdsStop.maxorf = 5;

label_source.transacc.anno_SL1 = [];
label_source.transacc.anno_SL2 = [];
label_source.transacc.pred = [];
info_genes.transacc.anno_SL1 = 5;
info_genes.transacc.anno_SL2 = 6;
info_genes.transacc.pred = 7;

label_source.polya.from_cleave = [];
info_genes.polya.from_cleave = 5;


PAR.label_source = label_source;
PAR.info_genes = info_genes;


PAR.Signal_name = [];
for j=1:length(signal_names)
  signal_name = signal_names{j};
  PAR.Signals.(signal_name) = struct;
end

cnt=0;
PAR.Content_name = [];
for j=1:length(content_names)
  PAR.Contents.(content_names{j}) = struct;
  PAR.Contents.(content_names{j}).name = content_names{j};
  PAR.Contents.(content_names{j}).type = cnt;
  cnt=cnt+1;
end

PAR.LSL.method = struct;

% -------------------------------------------------------------------------------------
% memory requirements
% -------------------------------------------------------------------------------------

PAR.RPROC.collect_jobs = 0 ; 
PAR.RPROC.options.ncpus = 1 ; 
PAR.RPROC.options.waitonfull = [];
PAR.RPROC.options.maxjobs = [] ;
PAR.RPROC.options.immediately = 0;
PAR.RPROC.options.immediately_bg = 0;
PAR.RPROC.options.addpaths = [];
PAR.RPROC.options.rmpaths = [];

PAR.RPROC.MEMREQ = [];
PAR.RPROC.time_req = [];
PAR.RPROC.exm_per_batch = 1 ;


% -------------------------------------------------------------------------------------
% SPLITS AND PARTITIONS
% -------------------------------------------------------------------------------------

PAR.SETs.num_splits = 5;
PAR.SETs.num_partitions = 5;
PAR.SETs.train_subsample_neg = 1;
PAR.SETs.test_subsample_neg = 1;
PAR.SETs.train_subsample_pos = 1;
PAR.SETs.test_subsample_pos = 1;
%for lsl
PAR.SETs.num_train = [] ; 
PAR.SETs.ignore_incomplete = 1 ; 


% -------------------------------------------------------------------------------------
% original data processing stuff
% -------------------------------------------------------------------------------------

PAR.regions.max_length = 1000000;
PAR.regions.min_genes_per_region = 1;
PAR.regions.offset_test = 20000;
PAR.regions.offset_train = 1000;
PAR.regions.minCutDist = 250;
PAR.regions.cluster_paralogs = 0;
PAR.regions.merge_confirmed = 0;

% -------------------------------------------------------------------------------------
% TASKS
% -------------------------------------------------------------------------------------

PAR.tasks.signals.prepare_labels = 0;
PAR.tasks.signals.train = 0;
PAR.tasks.signals.modelsel = 0;
PAR.tasks.signals.learn_plifs = 0;
PAR.tasks.signals.get_train_error = 0;

PAR.tasks.signals.pred_on_genome = 0; 
PAR.tasks.signals.verbose = 0; 

PAR.tasks.signals.run_locally = 0;

% -------------------------------------------------------------------------------------
% transfer from other organisms
% -------------------------------------------------------------------------------------

PAR.Source_organisms.names = {};
PAR.Source_organisms.dir_names = {};
PAR.Source_organisms.exp_names = {};


% -------------------------------------------------------------------------------------
% FN
% -------------------------------------------------------------------------------------
PAR.FN = struct;


return

function Signal = acceptor(label_source,signal_name)

% Signal = acceptor(label_source,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. In this function only
%          the field acc (transcc) is modified.  
%        - label_source is a struct that should contain a field
%          (signal_name). this is used in the called function splicesite_template.
%        - name can either be 'acc' or 'transacc'  
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: acceptor/donor -> splicesite_template -> signal_template
% (function acceptor is also called from transacc)
  
  
if nargin<3 
  signal_name = 'acc';
end

Signal = splicesites_template(label_source,signal_name);

%%%% SET ACCEPTOR STUFF

Signal.consensus = {'AG'};
Signal.offset = 2;

Signal.lwin = {[-60]} ;
Signal.rwin = {[80]} ;

Signal.order = [ 22];
Signal.shift = 0.5;
Signal.C = [3]; 

Signal.Conf_names = {Signal.Conf_names{:},'alt_3prime'};

Signals.(signal_name) = Signal;



return

function Signal = donor(label_source)

% Signals = donor(Signals,signal_name,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field don is modified.
%        - label_source is a struct that should contain a field don. 
%          this is used in the called function signal_template.
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: acceptor/donor -> splicesite_template -> signal_template

 
signal_name = 'don';

Signal = splicesites_template(label_source,signal_name);

%%%% SET DONOR STUFF

Signal.consensus = {'GC' 'GT'};
Signal.offset = 0;

Signal.lwin = {[-80]};
Signal.rwin = {[60]};

Signal.order = [22];
Signal.shift = 0.5;
Signal.C = [3]; 

Signal.Conf_names = {Signal.Conf_names{:},'alt_5prime'};




return

function Signal = tss(label_source,has_transacc)
  
% Signals = tss(Signals,label_source,has_transacc)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field tss is modified.
%        - label_source is a struct that should contain a field tss. 
%          this is used in the called function signal_template.
%        - has_transacc: is per default set to zero, only 1 for nematodes. 
%       
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tss/cleave -> transcriptends_template -> signal_template

  
if nargin<3
  has_transacc=0; 
end
   
signal_name = 'tss';

Signal = transcriptends_template(label_source,signal_name);

%%%% SET TSS STUFF


Signal.lwin_big = 601;
Signal.rwin_big = 901;

Signal.kernel_name= {'SP','WDS','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};

Signal.lwin = { [-600] [-70] [   0] };
Signal.rwin = { [+100] [+70] [+900] };
Signal.wordlen = { [4] [] [4] };

Signal.order ={[] [16] []};
Signal.shift ={[] [0] []};
Signal.shift_const ={[] [28] []};
Signal.C = 2; % log2(C)=1.0

if has_transacc
  Signal.Conf_names = {Signal.Conf_names{:},'has_trans'};
  Signal.filter_label.train.rm_trans = 1;
  Signal.filter_label.eval.rm_trans = 1;
  Signal.filter_label.test.rm_trans = 1;
else
  Signal.filter_label.train.rm_trans = 0;
  Signal.filter_label.eval.rm_trans = 0;
  Signal.filter_label.test.rm_trans = 0;
end

%%%%%% for human
%Signal.sampling.downsample_neg = 1/160; % <=1 (for no downsampling: 1)



return

function Signal = cleave(label_source)

% Signals = cleave(Signals,signal_name,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field cleave is modified.
%        - label_source is a struct that should contain a field cleave. 
%          this is used in the called function signal_template.
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tss/cleave -> transcriptends_template -> signal_template

  
signal_name = 'cleave';

Signal = transcriptends_template(label_source,signal_name);

%%%% SET CLEAVE STUFF

Signal.lwin_big = 410;
Signal.rwin_big = 810;

Signal.kernel_name= {'SP','WDS','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};

Signal.lwin = {[-400], [-60], [0]};
Signal.rwin = {[100], [60], [800]};
Signal.wordlen = {[5], [], [3]}; 

Signal.order = {[],[18],[]};
Signal.shift = {[],[0.1],[]};
Signal.shift_const = {[],[0],[]};
Signal.C = 1; % log2(C)=0



return

function Signal = polya(label_source,fn_polya)
  
% Signals = polya(Signals,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field polya is modified.
%        - label_source is a struct that should contain a field polya. 
%          this is used in the called function signal_template.
%       
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: polya -> signal_template

signal_name = 'polya' ; 
Signal = signal_template(label_source,signal_name);

%%%% SET POLYA STUFF

Signal.name = signal_name;
Signal.type = signal_name; 
Signal.label_fct = 'get_cand_others';
Signal.filter_fct = 'filter_cands';

Signal.offset = 0;

Signal.lwin_big = 310;
Signal.rwin_big = 310;

Signal.kernel_name= {'WDS','SP','SP','SP','SP','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};
Signal.subkernel_weights = [1 0.2 0.2 0.2 0.2 0.2];

Signal.lwin = {-300 -300 -149 1 151 -300} ;
Signal.rwin = {306 -150 0 150 306 306};
Signal.shift={[0 0.1 0.4],[],[],[],[],[]};
%Signal.shift={[0.1],[],[],[],[],[]};
Signal.shift_const={[0 10 20],[],[],[],[],[]};
%Signal.shift_const={[0],[],[],[],[],[]};
Signal.wordlen={[],[4],[4],[4],[4],[4]};
Signal.order={[18],[],[],[],[],[]};
Signal.C = [0.01 0.5 10];

Signal.consensus = {''};
if exist('fn_polya')
  Signal.consensus_fn = fn_polya;
  if fexist(Signal.consensus_fn)
    load(Signal.consensus_fn,'consensus')
    Signal.consensus = consensus;
  end
end

Signal.filter_label.train.use_intergenic=0;
Signal.filter_label.train.use_alt=0;
%Signal.filter_label.eval.use_intergenic=0;
%Signal.filter_label.eval.use_alt=0;
Signal.filter_label.test.use_intergenic=0;
Signal.filter_label.test.use_alt=0;

Signal.filter_label.train.conf_pos.anno = 0;
Signal.filter_label.train.conf_pos.from_cleave = [201 301 401];
Signal.filter_label.train.conf_pos.db = 0;
Signal.filter_label.train.conf_pos.fulllength = 0;
Signal.filter_label.train.onlywithpos = 0; 

%Signal.filter_label.eval.conf_pos.anno = 0;
%Signal.filter_label.eval.conf_pos.from_cleave = [201 301 401];
%Signal.filter_label.eval.conf_pos.db = 0;
%Signal.filter_label.eval.conf_pos.fulllength = 0;
%Signal.filter_label.eval.onlywithpos = 0; 

Signal.filter_label.test.conf_pos.anno = 0;
Signal.filter_label.test.conf_pos.from_cleave = [201 301 401];
Signal.filter_label.test.conf_pos.db = 0;
Signal.filter_label.test.conf_pos.fulllength = 0;
Signal.filter_label.test.onlywithpos = 0; 


Signal.Conf_names = {Signal.Conf_names{:},['has_' signal_name]};



Signal.label_source_par.polya.wl = 1;
Signal.label_source_par.polya.POSconstr =1;
Signal.label_source_par.polya.maxpcCov = 95;
Signal.label_source_par.polya.logdir = [];

return

function Signal = tis(label_source)
  
% Signals = tis(Signals,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field tis is modified.
%        - label_source is a struct that should contain a field tis. 
%          this is used in the called function signal_template.
%       
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tis/cdsStop -> translatends_template -> signal_template

    
signal_name = 'tis';

Signal = translatends_template(label_source,signal_name);

%%%% SET TIS STUFF

Signal.consensus = {'ATG'};


Signal.kernel_name= {'SP','WDS','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};
Signal.lwin = {[-200] [-30] [0]} ; 
Signal.rwin = {[0] [110] [180]}; 
Signal.shift={[],[0],[]};
Signal.shift_const={[],[0],[]};
Signal.order={[],[24],[]};  
Signal.wordlen = {[3] [] [5]}; 
Signal.C = 1; % log2(C)=1.0










return

function Signal = cdsStop(label_source)
  
% Signals = cdsStop(Signals,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field cdsStop is modified.
%        - label_source is a struct that should contain a field cdsStop. 
%          this is used in the called function signal_template.
%       
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tis/cdsStop -> translatends_template -> signal_template
   
signal_name = 'cdsStop';

Signal = translatends_template(label_source,signal_name);

%%%% SET cdsStop STUFF

Signal.consensus = {'TAA','TAG','TGA'};


Signal.kernel_name= {'SP','WDS','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};
Signal.lwin = {[-200] [-110] [0]} ; 
Signal.rwin = {[0] [10] [180]};
Signal.shift={[],[0],[]};
Signal.shift_const={[],[0],[]};
Signal.order={[],[20],[]};  
Signal.wordlen = {[2] [] [4]}; 
Signal.C = 2.8284; % logC=1.5



return

function Signal = transacceptor(label_source)

% Signals = transacceptor(Signals,label_source)
% INPUT  - Signals is a struct with a field for each Signal. In this function only
%          the field transcc is modified.  
%        - label_source is a struct that should contain a field
%          transacc. this is used in the called function splicesite_template.
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tansacceptor -> acceptor -> splicesite_template -> signal_template

INF = 1e6;  
signal_name = 'transacc';

Signal = acceptor(label_source,signal_name);

%%%% SET transacc STUFF

Signal.lwin = {[-190]} ;
Signal.rwin = {[80]};


Signal.filter_label.train.conf.anno_SL1 = 0;
Signal.filter_label.train.conf.anno_SL2 = [0 INF];
Signal.filter_label.train.conf.pred = [0 INF];

Signal.filter_label.eval.conf.anno_SL1 = 0;
Signal.filter_label.eval.conf.anno_SL2 = [0 INF];
Signal.filter_label.eval.conf.pred = [0 INF];


Signal.filter_label.test.conf.anno_SL1 = 0;
Signal.filter_label.test.conf.anno_SL2 = [0 INF];
Signal.filter_label.test.conf.pred = [0 INF];


Signal.filter_label.train.use_intergenic = 1;
Signal.filter_label.train.use_alt = 0;
Signal.filter_label.eval.use_intergenic = 1;
Signal.filter_label.eval.use_alt = 0;
Signal.filter_label.test.use_intergenic = 1;
Signal.filter_label.test.use_alt = 0;
 
Signal.filter_label.train.use_pred = 1;
Signal.filter_label.eval.use_pred = 1;
Signal.filter_label.test.use_pred = 1;

Signal.filter_label.train.only_acc = 1;
Signal.filter_label.eval.only_acc = 1;
Signal.filter_label.test.only_acc = 1;

Signal.filter_label.train.pos_dist = 20;
Signal.filter_label.eval.pos_dist = 20;
Signal.filter_label.test.pos_dist = 20;

Signal.export_settings.conf_cum_thresh = 0.05 ;
Signal.export_settings.resolution = 1 ;

Signal.Conf_names = {Signal.Conf_names{:},'isacc','has_tss','has_trans','dist'};


Signal.label_source_par.transacc.win_anno_SL1 = 50;
Signal.label_source_par.transacc.win_anno_SL2 = 50;
Signal.label_source_par.transacc.win_pred = 5;
Signal.label_source_par.transacc.threshold_pred = 0.5;


return

function Signal = signal_template(label_source,signal_name)

% Signals = signal_template(Signal,label_source,signal_name)
% Signals is a struct with a field for each Signal. In this function only
% the field signal_name is added. Default values are set.


if nargin<2
  signal_name = 'dummy';
end

%%%%%

Signal.type = []; 
Signal.name = signal_name;
Signal.consensus = {[]};
Signal.offset = [];

Signal.lwin_big = [];
Signal.rwin_big = [];

Signal.label_fct = [];
Signal.filter_fct = 'filter_cands';
Signal.load_fct = 'load_data';
Signal.score_fct = [];

Signal.Conf_names = { 'altgenic','gene_id' };

Signal.sampling.downsample_neg = 1; % <=1 (for no downsampling: 1)  
Signal.sampling.mindist = 1; % >=1 (for all: 1)  
Signal.sampling.pos_neg_ratio = 1/10;

Signal.resolution   = 1; % for lsl

conf.USE_ALL = 0;
conf.conf_pos = struct;
conf.conf_pos_skip = struct;
conf.any_conf_pos = [];
conf.any_conf_pos_skip = [];
conf.conf_neg = struct;
conf.conf_neg_skip = struct;
conf.any_conf_neg = [];
conf.any_conf_neg_skip = [];
conf.use_intergenic=[];
conf.use_alt = [];
conf.use_label = 'label';

Signal.filter_label.train = conf ; 
Signal.filter_label.test = conf ;
Signal.filter_label.eval = conf ;

Signal.export_settings.conf_cum_thresh = 0 ;
Signal.export_settings.resolution = 1 ;

Signal.domain_adapt.name = 'target_only';
Signal.domain_adapt.Source_organisms = [];

Signal.method = struct;

label_source = label_source.(signal_name);
sources = fieldnames(label_source);
for i=1:length(sources)
  if label_source.(sources{i})
    Signal.Conf_names = {Signal.Conf_names{:},[sources{i} '_conf']};
    if isequal(signal_name,'acc')|isequal(signal_name,'don')
      Signal.Conf_names = {Signal.Conf_names{:},[sources{i} '_conf_skip']};
    end
  end
end




return

function  [Conf, Conf_cum] = Out2Conf(Out,prob, prob_cum,limits);

% [Conf, Conf_cum] = Out2Conf(Output ,prob, prob_cum,limits);
%  
% function transforms outputs of an SVM to probability and cumulative
% probability values by means of piecewise linear functions (PLIF). 
% Use learn_plifs to determine the function (it will return supporting
% points 'limits' and corresponding y-values, 'prob' and 'prob_cum')
%
% INPUT   Out       vector of SVM Output to be transformed
%         prob
%         prob_cum
%         limits    
%
% OUTPUT  Conf      vector (same length as Out) with estimated
%                   probability for true positive
%         Conf_cum  vector (same length as Out) with estimated 
%                   cumulative probability true positive
% 
% Functions called: -> pwl_add
%
% see also: learn_plifs

bins = length(prob); 
Conf = zeros(1, length(Out)) ;
Conf_cum = zeros(1, length(Out)) ;
for i=1:length(Out)
  x(:,1) = pwl_add(zeros(bins, 1), Out(i), limits) ;
  Conf_cum(i) = x'*prob_cum;
  Conf(i) = x'*prob;
end ;

return

function vec = pwl_add(vec, value, limits)
% vec = pwl_add(vec, value, limits)
  
idx = sum(limits<=value) ;

if idx==0,
  vec(1)=vec(1)+1 ;
elseif idx==length(limits)-1,
  vec(end)=vec(end)+1 ;
else
  vec(idx+1) = vec(idx+1) + (value-limits(idx))/(limits(idx+1)-limits(idx)) ;  
  vec(idx)   = vec(idx)   + (limits(idx+1)-value)/(limits(idx+1)-limits(idx)) ;
end ;
return

function Signal = splicesites_template(label_source,signal_name)

% Signals = splicesites_template(Signals,label_source,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field signal_name is added or
%          modified. 
%          Default values are set.
%        - label_source is a struct that should contain a field (signal_name). 
%          this is used in the called function signal_template.
%        - signal_name: 'don', 'acc','transacc'
%  
% OUTPUT Signals with modified field signal_name; 
%  
% HIERACHY: acceptor/donor -> splicesite_template -> signal_template
%
% see also transcriptends_template/translat_template
  
  
INF=1e6;

Signal = signal_template(label_source,signal_name);

%%%% SET SPLICE SITE STUFF

Signal.type = 'splicesites'; 
Signal.label_fct = 'get_cand_splicesites';

Signal.lwin_big = 199;
Signal.rwin_big = 199;

Signal.filter_label.train.use_intergenic=0;
Signal.filter_label.train.use_alt=0;
Signal.filter_label.train.use_alt_neg=0;
Signal.filter_label.test.use_intergenic=0;
Signal.filter_label.test.use_alt=0;
Signal.filter_label.test.use_alt_neg=0;

Signal.filter_label.train.use_label = 'label';
Signal.filter_label.test.use_label = 'label';


Signal.filter_label.train.use_decoys = 1;
Signal.filter_label.test.use_decoys = 1;


Signal.filter_label.train.conf_pos.est_clusters = [0 INF];
Signal.filter_label.train.conf_pos.cDNA = [0 INF];
Signal.filter_label.train.conf_pos.fulllength = [0 INF];
Signal.filter_label.train.conf_pos.anno = [0:1] ;

Signal.filter_label.train.conf_pos_skip.est_clusters = [0 INF];
Signal.filter_label.train.conf_pos_skip.cDNA = [0 INF];
Signal.filter_label.train.conf_pos_skip.fulllength = [0 INF];
Signal.filter_label.train.conf_pos_skip.anno = [-inf] ;

Signal.filter_label.train.conf_neg.est_clusters = [0 INF];
Signal.filter_label.train.conf_neg.cDNA = [0 INF];
Signal.filter_label.train.conf_neg.fulllength = [0 INF];
Signal.filter_label.train.conf_neg.anno = [-1:1] ;

Signal.filter_label.train.conf_neg_skip.est_clusters = [0 INF];
Signal.filter_label.train.conf_neg_skip.cDNA = [0 INF];
Signal.filter_label.train.conf_neg_skip.fulllength = [0 INF];
Signal.filter_label.train.conf_neg_skip.anno = [-1:1] ;

Signal.filter_label.test = Signal.filter_label.train;
Signal.filter_label.eval = Signal.filter_label.train;

% Signal.save_confs = 1;
% Signal.save_alts = 1;


Signal.export_settings.conf_cum_thresh = 0.5 ;
Signal.export_settings.resolution = 1 ;


Signal.Conf_names = {Signal.Conf_names{:},'alt_exon','alt_intron'};





return

function Signal = transcriptends_template(label_source,signal_name)

% Signals = transcriptends_template(Signals,label_source,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field signal_name is added or
%          modified. 
%          Default values are set.
%        - label_source is a struct that should contain a field (signal_name). 
%          this is used in the called function signal_template.
%        - signal_name:'tss', 'cleave'
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tss/cleave -> transcriptends_template -> signal_template
%
% see also splicesite_template/translat_template
  
  
  
Signal = signal_template(label_source,signal_name);

%%%% SET TRANSCRIPT END STUFF

Signal.type = 'transcriptends'; 
Signal.label_fct = 'get_cand_others';


Signal.consensus = [];
Signal.offset = 0;

Signal.sampling.downsample_neg = 1/10; %% take randomly every 10th porsitions 
Signal.sampling.mindist = 10 ;


Signal.resolution   = 20; % for lsl

Signal.filter_label.train.conf_pos.anno = [0 1];
Signal.filter_label.train.conf_pos.est_clusters = [2:3];
Signal.filter_label.train.conf_pos.db = 0;
Signal.filter_label.train.conf_pos.fulllength = 0;
Signal.filter_label.train.onlywithpos=0; 

Signal.filter_label.train.use_intergenic=1;
Signal.filter_label.train.use_alt=0;

Signal.filter_label.test = Signal.filter_label.train;
Signal.filter_label.eval = Signal.filter_label.train;

Signal.export_settings.conf_cum_thresh = 0.01 ;
Signal.export_settings.resolution = 10 ;

Signal.Conf_names = {Signal.Conf_names{:},['has_' signal_name]};



return

function Signal = translatends_template(label_source,signal_name)

% Signals = translatends_template(Signals,label_source,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field signal_name is added or
%          modified. 
%          Default values are set.
%        - label_source is a struct that should contain a field (signal_name). 
%          this is used in the called function signal_template.
%        - signal_name:'tis', 'cdsStop'
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tis/cdsStop -> translatends_template -> signal_template
%
% see also splicesite_template/transcriptends_template
  
INF=1e6;
  
Signal = signal_template(label_source,signal_name);

%%%% SET TRANSLAT END STUFF

Signal.type = 'translatends' ;
Signal.label_fct = 'get_cand_others';

Signal.offset = 0;
Signal.lwin_big = 210;
Signal.rwin_big = 200;



Signal.filter_label.train.conf_pos.anno = [0 1] ;
Signal.filter_label.train.conf_pos.maxorf = INF;
Signal.filter_label.test.conf_pos.anno = [0 1] ;
Signal.filter_label.test.conf_pos.maxorf =INF;

Signal.filter_label.train.use_intergenic=0;
Signal.filter_label.train.use_alt=0;
Signal.filter_label.test.use_intergenic=0;
Signal.filter_label.test.use_alt=0;

Signal.filter_label.eval = Signal.filter_label.test;

% PAR.label_source.cds.par.max_nof_paths = 100;
% PAR.label_source.cds.par.nof_best_kept = 2;


return

function jobinfo =  concat_genomewide_predictons(fn_genome_config,fn_pos,fn_pred,num_splits,avg_all_svms,subtract_diff,run_locally, RPROC)

% concat_genomewide_predictons(fn_genome_config,fn_pos,fn_pred,num_splits,run_locally, RPROC);

jobinfo = rproc_empty(0) ;
q=0;
% options.ncpus = 1;
% options.identifier = sprintf('%sconcat%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;


genome_info = init_genome(fn_genome_config);
P.avg_all_svms = avg_all_svms;
P.subtract_diff = subtract_diff;
P.num_splits = num_splits;
for c = 1:length(genome_info.contig_names)
  for s = '+-'
    P.fn_pred_all = sprintf('%scontig_%i%s_all.mat',fn_pred,c,s);
    P.fn_pred_split = sprintf('%scontig_%i%s_split=',fn_pred,c,s);
    if fexist(P.fn_pred_all)
      continue
    else
      q=q+1;
      if ~run_locally
        [mem_req, time_req, opts] = rproc_memtime_policy('concat_genomewide_predictons_helper',  0, RPROC.options) ;

        if isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs 
          jobinfo(q) = rproc_create('concat_genomewide_predictons_helper', P, mem_req, opts, time_req) ;
        else
          jobinfo(q) = rproc('concat_genomewide_predictons_helper', P, mem_req, opts, time_req);
        end
      else
        concat_genomewide_predictons_helper(P)
      end
    end
  end
end

if ~run_locally
   rproc_wait(jobinfo, 20, 1, -1);
end

return

function [jobinfo] = save_predictions(fn_genome_config, fn_pred, ...
                                      Signal_name, resolution, conf_cum_thresh, strands, run_locally, RPROC, do_gzip, save_as, inventory)
%  [jobinfo] =
%  save_predictions(fn_genome_config,fn_pred,strands,run_locally,RPROC,do_gzip,save_as,inventory)
%
%  Loads predictions corresponding to each contig for a signal
%  defined in PAR.Signal_name.
%  Saves them to binary files which can be read by 
%  interval_query.
%

%
%  see retrieve_signals, interval_query 
if nargin<2
  strands='+-';
end

if ~exist('do_gzip', 'var'),
  do_gzip = 1 ;
end ;
if ~exist('save_as', 'var'),
  save_as.spf_ascii = 1 ;
  save_as.spf_binary = 1 ;
  save_as.wiggle = 0 ;
end ;

jobinfo = rproc_empty(0) ;
% options = RPROC.options;
% options.ncpus = 1;
% options.identifier = sprintf('%scheck%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;
% MEMREQ = RPROC.MEMREQ;

fid=1;q=0;

genome_info = init_genome(fn_genome_config);

P.fn_genome_config = fn_genome_config;
P.Signal_name = Signal_name;
P.fn_pred = fn_pred;
P.conf_cum_thresh = conf_cum_thresh;
P.resolution = resolution ;
P.save_as = save_as ;


% options.identifier = sprintf('%ssave%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;
for c = 1:length(genome_info.contig_names)
  for s = strands
    P.chrom = c;
    P.strand = s;
    P.do_gzip = do_gzip ;
    filename = sprintf('%scontig_%i%s', fn_pred, c, s);

    one_missing=0 ;
    for k=1:length(inventory)
      if ~fexist([filename '.' inventory{k}]) || ~fexist([filename '_' inventory{k} '.spf']) || ~fexist([filename '_' inventory{k} '_spf.mat'])
        one_missing = 1 ;
      end ;
    end ;
    if one_missing,
      if ~run_locally
        q=q+1;

        [mem_req, time_req, opts] = rproc_memtime_policy('save_predictions_helper',  0, RPROC.options) ;%RPROC.MEMREQ ;

        if isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs 
          jobinfo(q) = rproc_create('save_predictions_helper', P, mem_req, opts, time_req) ;
        else
          jobinfo(q) = rproc('save_predictions_helper', P, mem_req, opts, time_req);
        end
      else
        save_predictions_helper(P)
      end
    end
  end %%% loop over strand
end %%% loop over contigs 


rproc_wait(jobinfo, 20, 1, -1);



return

function fn_out = save_predictions_as_wiggle(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)
% fn_out = save_predictions_as_wiggle(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)

%assert(issorted(pos))
assert(isequal(pos, sort(pos)))
assert(length(pos)==length(score))

if ~exist('do_gzip', 'var'),
  do_gzip = 1 ;
end ;

fn_out=[out_fn_prefix '_' score_name '.wiggle'];

if fexist(fn_out)|fexist([fn_out '.gz'])
  fprintf(1, 'file exists, it will NOT be overwritten\n')
%  keyboard
  return
end

fprintf(1,'save as wiggle: %s\n', fn_out);
[fd msg] = fopen(fn_out, 'w+');
disp(msg)

fprintf(fd,'browser position chr%s:1-50,000\n', contig_name);
fprintf(fd,'browser hide all\n');
fprintf(fd,'browser pack Genes\n');
fprintf(fd,'browser full altGraph\n');
fprintf(fd,'browser viewLimits=%1.2f:%1.2f\n', min(score), max(score));


fprintf(fd,['track type=wiggle_0 name="%s_%s%s" description="%s chr%s %s" ' ...
            '\n'],signal_name, score_name, strand, score_name, ...
        contig_name, strand);
fprintf(fd,'variableStep chrom=chr%s\n',contig_name);

for j=1:length(pos)
  fprintf(fd,'%i\t%.3f\n', pos(j), score(j));
end

fclose(fd);

if do_gzip,
  ret = unix(sprintf('gzip %s',fn_out));
  if ret~=0
    warning('gzip failed')
  end
end ;

return

function save_predictions_as_sigpred(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)
% save_predictions_as_wiggle(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)

%assert(issorted(pos))
assert(isequal(pos, sort(pos)))
assert(length(pos)==length(score))

if ~exist('do_gzip', 'var'),
  do_gzip = 1 ;
end ;

fn_out=[out_fn_prefix '_' score_name '.spf'];

if fexist(fn_out) | fexist([fn_out '.gz'])
  fprintf(1, 'file exists, it will NOT be overwritten\n')
  return
end
fprintf(1,'save in signal prediction format (spf): %s\n', fn_out);
[fd msg] = fopen(fn_out, 'w+');

fprintf(fd, '##Seqid\tType\tSource\tStart\tStrand\tScore\n') ;
for j=1:length(pos)
  fprintf(fd, '%s\t%s\t%s\t%i\t%c\t%.3f\n', contig_name, signal_name, score_name, pos(j), strand, score(j)) ;
end

fclose(fd);

if do_gzip,
  ret = unix(sprintf('gzip %s',fn_out)) ;
  if ret~=0
    warning('gzip failed') ;
  end
end 


return

function save_predictions_as_sigpred_bin(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand)
% save_predictions_as_sigpred_bin(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand)

%assert(issorted(pos))
assert(isequal(pos, sort(pos)))
assert(length(pos)==length(score))

fn_out=[out_fn_prefix '_' score_name '_spf.mat'];

if fexist(fn_out) 
  fprintf(1, 'file exists, it will NOT be overwritten\n')
  return
end
fprintf(1,'save in binary signal prediction format (spf): %s\n', fn_out);
save(fn_out, '-V7', 'contig_name', 'signal_name', 'score_name', 'pos', 'strand', 'score') ;

return

function PAR=A_thaliana(PAR)

PAR = organism_template(PAR);

PAR.organism.name = 'A_thaliana';

PAR.label_source.use_anno = 0;
PAR.label_source.use_confirmed = 1;

PAR.label_source.tss.est_clusters = 1;
PAR.label_source.tss.fulllength = 0;
PAR.label_source.tss.db = 0;

PAR.label_source.cleave.est_clusters = 1;
PAR.label_source.cleave.fulllength = 0;
PAR.label_source.cleave.db = 0;

PAR.label_source.cds.anno = 0;
PAR.label_source.cds.maxorf = 0;


PAR.regions.cluster_paralogs = 1;

return

function [organism label_source] = C_elegans(label_source_orig,signal_names)

[organism label_source] = Nematodes(label_source_orig,signal_names);

organism.name = 'C_elegans';
organism.full_name = 'Caenorhabditis_elegans';
organism.release = '';
organism.version = '';
organism.genebuilt = '';
return

function PAR = D_melanogaster(PAR)


PAR = organism_template(PAR);

PAR.organism.clade = 'insect';
PAR.organism.name = 'D_melanogaster';

PAR.label_source.use_anno = 0;
PAR.label_source.use_confirmed = 1;

PAR.label_source.cleave.est_clusters = 1;
PAR.label_source.cleave.fulllength = 1;
PAR.label_source.cleave.db = 0;

PAR.label_source.tss.est_clusters  = 1;
PAR.label_source.tss.fulllength   = 0;
PAR.label_source.tss.db = 0;

return

function PAR=D_rerio(PAR)
  
PAR = organism_template(PAR);

PAR.organism.name = 'D_rerio';

PAR.label_source.use_anno = 0;
PAR.label_source.use_confirmed = 1;

PAR.label_source.tss.est_clusters  = 1;
PAR.label_source.tss.fulllength   = 1;
PAR.label_source.tss.db = 0;

PAR.label_source.cleave.est_clusters = 1;
PAR.label_source.cleave.fulllength = 1;
PAR.label_source.cleave.db = 0;

return

function [label_source] = organism_template(label_source_orig,signal_names)


% PAR = set_label_source(PAR)
% 
% sets defaults for all label_sources (don't use anno, use EST confirmed only.)
% values that are already assigned will NOT be overriden.
  
  
%% default values for general use of annotation and EST confrimed sequences
label_source = label_source_orig;

if isempty(label_source.use_anno)
  label_source.use_anno = 0;
end
if isempty(label_source.use_confirmed)
  label_source.use_confirmed = 1;
end


%% default values for individual signals


for j=1:length(signal_names)
  signal_name = signal_names{j};
  if label_source.use_anno
    label_source.(signal_name).anno = 1;
  else
    label_source.(signal_name).anno = 0;
  end
  if label_source.use_confirmed & ~isequal(signal_name,'transacc')
    label_source.(signal_name).est_clusters = 1;
    label_source.(signal_name).cDNA = 1;
    label_source.(signal_name).fulllength = 1;
  elseif ~label_source.use_confirmed & ~isequal(signal_name,'transacc')
    label_source.(signal_name).est_clusters = 0;
    label_source.(signal_name).cDNA = 0;
    label_source.(signal_name).fulllength = 0;
  end
  label_source.(signal_name).db = 0;
  label_source.(signal_name).from_fn_candsites = 1 ;
end

label_source.tis.maxorf = 0;
label_source.cdsStop.maxorf = 0;

label_source.transacc.anno_SL1 = 0;
label_source.transacc.anno_SL2 = 0;
label_source.transacc.pred = 1;
label_source.transacc.db = 0;



return

function method = SVM_WD(Signal)

method = SVM(Signal);
method.feature_type = 'char';

kernel.name = 'WD';
kernel.order = Signal.order;
kernel.shift = 0 ;
kernel.lwin = Signal.lwin{1};
kernel.rwin = Signal.rwin{1};

method.kernels{1} = kernel; 


return

function method = SVM(Signal)
  
method = method_template;

% set parameters, model selection 

method.name = 'SVM';
method.train_fct = 'svm_train';
method.predict_fct = 'svm_predict';
method.kernels = {[]};
method.poim_maxOrder = 8;
method.svm_name = 'LIGHT';
method.qpsize = 50;
method.threads = 16;
method.center = Signal.lwin_big;
method.par_ms.C = Signal.C;

%PAR.method.svm_name = 'GPBTSVM';PAR.method.qpsize=500;

% use multi kernel learning
method.use_mkl = 0;





return

function SETs = make_partitions(SETs)
 

if SETs.num_partitions==1
  return
end
  
SETs.partitions = eye(SETs.num_partitions,SETs.num_partitions)+1;  
for i = 1:2:SETs.num_partitions-1
  SETs.partitions(i:i+1,i:i+1) = SETs.partitions(i:i+1,i:i+1)+1;
end
if mod(SETs.num_partitions,2)
  SETs.partitions(end,end-1:end) =  SETs.partitions(end,end-1:end)+1;
end
if SETs.num_splits~=SETs.num_partitions
  assert(mod(SETs.num_splits,SETs.num_partitions)==0)
  SETs.partitions = repmat(SETs.partitions,1,SETs.num_splits/SETs.num_partitions);
end

if ~isfield(SETs,'partition_train_on') 
  [tmp,idx] = unique(SETs.partitions==1,'rows');
  SETs.partition_train_on = zeros(1,SETs.num_partitions);
  SETs.partition_train_on(idx) = 1;
end



SETs.info.train = 1;
SETs.info.eval = 2;
SETs.info.plif = 2;
SETs.info.pred = 3;

for i=1:SETs.num_partitions
  assert(isequal(unique(SETs.partitions(i,:)),unique([SETs.info.train ...
                      SETs.info.eval SETs.info.plif SETs.info.pred ])))
end

assert(all(sum(SETs.partitions==SETs.info.pred)==1))
assert(length(unique(sum(SETs.partitions'==SETs.info.train)))==1)
assert(length(unique(sum(SETs.partitions'==SETs.info.eval)))==1)
assert(length(unique(sum(SETs.partitions'==SETs.info.plif)))==1)
assert(length(unique(sum(SETs.partitions'==SETs.info.pred)))==1)




return

function x = equal(M1, M2)
% EQUAL(M1, M2) tests M1 and M2 for equality
% Function to test whether two matrices are identical
%
% returns 1 if true and 0 if false

x=isequal(M1, M2) 

return

function method = method_template

% Signals = method_template(Signal,signal_name)
% Signals is a struct with a field for each Signal. In this function only
% the field Signal.(signal_name).method is added. Default values are set.

method.name = [];
method.train_fct = [];
method.predict_fct = [];

% PAR.method.svm_name = 'GPBTSVM';PAR.method.qpsize=500;
% use conservation information
% method.use_cons = 0;
% load training examples only once and calculate all models in one job
% method.submit_batch=0;

method.plif.bins = 50;
method.plif.min_incr = 1e-6;

method.use_cons = 0;
method.feature_type = [];
%assert(max(abs([PAR.Signals.(PAR.Signal_name).lwin{:}]))<PAR.Signals.(PAR.Signal_name).lwin_big)

%assert(max(abs([PAR.Signals.(PAR.Signal_name).lwin{:}]))<PAR.Signals.(PAR.Signal_name).rwin_big)




return


function [ ok ] = nice_mkdir( dir );
if ~isdir(dir)
  ok = ( 0 == unix( [ 'mkdir -p ', dir ] ) );
  ok = ok & ( 0 == unix( [ 'chmod g+t ', dir ] ) );
end


return

function create_directories(FN,organism,verb,signal)

% create_directories(PAR)
%   
% function creates all necessary directories for a gene finding
% experiment as defined by PAR.
  
  
if nargin<3
  verb=0;
end

if  nargin>3
  signalnames{1} = signal;
else
  signalnames = fieldnames(FN.input_sig);
end


%%%%%%%%%%%%%%%
%%% GENERAL INPUT FILES 
%%%%%%%%%%%%%%%

exp_base_in = sprintf('%s/%s/%s',FN.input_dir,organism.name,FN.dir_name);

input_dir = sprintf('%s/orig_data/',exp_base_in);
if verb
  fprintf(1, 'dir for original data: \n%s\n\n', input_dir)
end
nice_mkdir(input_dir);


%%%%%%%%%%%%%%%%%%%%%%%%%%
%% OUTPUT DIRs 
%%%%%%%%%%%%%%%%%%%%%%%%%

exp_base_out = sprintf('%s/%s/%s',FN.output_dir,organism.name,FN.dir_name);
output_dir = sprintf('%s/genome_data/',exp_base_out);
if verb
  fprintf(1, 'dir for genes, split and regions: \n%s\n\n', output_dir)
end
nice_mkdir(output_dir);

nice_mkdir(FN.exp_dir);


%%%%%%%%%%%%%%%
%%% SIGNAL SPECIFIC Dirs
%%%%%%%%%%%%%%%


for s=1:length(signalnames)
  if ~isfield( FN.input_sig, signalnames{s})
    continue
  end
  signals = FN.input_sig.(signalnames{s});
  signal_fns = fieldnames(signals);
  for i=1:length(signal_fns)
    name = signals.(signal_fns{i}) ;
    iii = max(strfind(name,'/'));
    if ~isempty(iii)
      nice_mkdir(name(1:iii));
    end 
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%
%% SIGNAL SPECIFIC OUTPUT DIRs 
%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin<4
  signalnames = fieldnames(FN.output_sig);
end
  
for s=1:length(signalnames)
  if ~isfield( FN.output_sig, signalnames{s})
    continue
  end
  signals = FN.output_sig.(signalnames{s});
  signal_fns = fieldnames(signals);
  for i=1:length(signal_fns)
    name = signals.(signal_fns{i}) ;
    if isempty(strmatch(FN.output_dir,name)) &verb
      fprintf(['incomplete signal output file name, signal output directories not generated; '...
               'run config for %s\n'],signalnames{s})
      break
    end
    iii = max(strfind(name,'/'));
    if ~isempty(iii)
      nice_mkdir(name(1:iii));
    end 
  end
end

%%%%%%%%%%%%%%%
%%% CONTENT SPECIFIC Dirs
%%%%%%%%%%%%%%%

if isfield(FN, 'input_cont')
  if nargin<4
    signalnames = fieldnames(FN.input_cont);
  end
  for s=1:length(signalnames)
    if ~isfield( FN.input_cont, signalnames{s})
      continue
    end
    signals = FN.input_cont.(signalnames{s});
    signal_fns = fieldnames(signals);
    for i=1:length(signal_fns)
      name = signals.(signal_fns{i}) ;
      iii = max(strfind(name,'/'));
      if ~isempty(iii)
        nice_mkdir(name(1:iii));
      end 
    end
  end
end
%%%%%%%%%%%%%%%%%%%%%%%%%
%% CONTENT SPECIFIC OUTPUT DIRs 
%%%%%%%%%%%%%%%%%%%%%%%%%

if isfield(FN, 'output_cont')
  if nargin<4
    signalnames = fieldnames(FN.output_cont);
  end
  for s=1:length(signalnames)
    if ~isfield( FN.output_cont, signalnames{s})
      continue
    end
    signals = FN.output_cont.(signalnames{s});
    signal_fns = fieldnames(signals);
    for i=1:length(signal_fns)
      name = signals.(signal_fns{i}) ;
      if isempty(strmatch(FN.output_dir,name)) &verb
        fprintf(['incomplete signal output file name, signal output directories not generated; '...
                 'run create_filenames first for\n'])
        break
      end
      iii = max(strfind(name,'/'));
      if ~isempty(iii)
        nice_mkdir(name(1:iii));
      end 
    end
  end
end
%%%%%%%%%%%%%%%%%%%%%%%%%
%% LSL SPECIFIC
%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin<4
  
  if isfield(FN, 'output_lsl')
    fns = fieldnames(FN.output_lsl);
    for i=1:length(fns)
      name = FN.output_lsl.(fns{i}) ;
      if isempty(strmatch(FN.output_dir,name)) &verb
        fprintf('incomplete signal output file name, signal output directories not generated;')
        break
      end
      iii = max(strfind(name,'/'));
      if ~isempty(iii)
        nice_mkdir(name(1:iii));
      end 
    end
  end
  if isfield(FN, 'input_lsl')
    fns = fieldnames(FN.input_lsl);
    for i=1:length(fns)
      if ~iscell(FN.input_lsl.(fns{i}))
        name = FN.input_lsl.(fns{i}) ;
        if isempty(strmatch(FN.input_dir,name)) &verb
          fprintf('incomplete signal input file name, signal input directories not generated;')
          break
        end
        iii = max(strfind(name,'/'));
        if ~isempty(iii)
          nice_mkdir(name(1:iii));
        end 
      else
        for j=1:length(FN.input_lsl.(fns{i}))
          name = FN.input_lsl.(fns{i}){j} ;
          if isempty(strmatch(FN.input_dir,name)) &verb
            fprintf('incomplete signal input file name, signal input directories not generated;')
            break
          end
          iii = max(strfind(name,'/'));
          if ~isempty(iii)
            nice_mkdir(name(1:iii));
          end 
        end
      end
    end
  end
  
  fn = sprintf('%s/%s/current_predictions', FN.input_dir, organism.name);
  nice_mkdir(fn);

end
return

function FN = create_filenames(FN,organism,Signals,Contents,verb)

%
% FN requires fields :
% 	- input_dirC_
% 	- output_dir
%	- exp_name
%	- FN.fn_genes_train
%	- date
%	- user
%	- organism
%

if nargin<2
  verb=0;
end

if ~isfield(FN,'dir_name_ext')
  dir_name_ext = '';
else
  dir_name_ext = [ '_', FN.dir_name_ext ];
end;

if isempty(FN.dir_name)
  FN.dir_name ='';
end
if isempty(FN.exp_name)
  FN.exp_name = '';
end
if isempty(FN.lsl_exp_name)
  FN.lsl_exp_name = '';
end

assert(isfield(FN,'exp_name'))
assert(isfield(FN,'dir_name'))

%%%%%%%%%%%%%%%
%%% GENERAL INPUT GENOME FILES 
%%%%%%%%%%%%%%%
exp_base_in = sprintf('%s/%s/%s/%s',FN.genome_dir,organism.name,organism.release,organism.genebuilt);

input_fns = fieldnames(FN.genome);
for i=1:length(input_fns)
  old_name = getfield(FN.genome,input_fns{i});
  if iscell(old_name)
    new_name ={};
    for j=1:length(old_name)
      if isempty(strmatch(FN.genome_dir,old_name{j}))
        new_name{j}  = sprintf('%s/%s',exp_base_in,old_name{j} );
      else
        new_name{j}  = old_name{j} ;
      end
    end
  elseif length(strfind(old_name,'/'))==1
    exp_base_in_ = sprintf('%s/%s/%s',FN.genome_dir,organism.name,organism.release);
    new_name  = sprintf('%s/%s',exp_base_in_,old_name );
  elseif isempty(strmatch(FN.genome_dir,old_name)) 
    new_name  = sprintf('%s/%s',exp_base_in,old_name );
  else
    new_name  = old_name ;
  end
  FN.genome = setfield(FN.genome,input_fns{i},new_name);
end  
if verb
  fprintf(1, 'genebuild from dir: \n%s\n\n', exp_base_in)
end

%%%%%%%%%%%%%%%
%%% GENERAL INPUT FILES 
%%%%%%%%%%%%%%%

exp_base_in = sprintf('%s/%s/%s',FN.input_dir,organism.name,FN.dir_name);

input_fns = fieldnames(FN.input);
for i=1:length(input_fns)
  old_name = getfield(FN.input,input_fns{i});
  if iscell(old_name)
    new_name ={};
    for j=1:length(old_name)
      if isempty(strmatch(FN.input_dir,old_name{j}))
        new_name{j}  = sprintf('%s/orig_data/%s',exp_base_in,old_name{j} );
      else
        new_name{j}  = old_name{j} ;
      end
    end
  else
    if isempty(strmatch(FN.input_dir,old_name)) 
      new_name  = sprintf('%s/orig_data/%s',exp_base_in,old_name );
    else
      new_name  = old_name ;
    end
  end
  FN.input = setfield(FN.input,input_fns{i},new_name);
end  
input_dir=sprintf('%s/orig_data/',exp_base_in);
if verb
  fprintf(1, 'dir for original data: \n%s\n\n', input_dir)
end


%%%%%%%%%%%%%%%%%%%%%%%%%%
%% OUTPUT DIRs AND FNs  
%%%%%%%%%%%%%%%%%%%%%%%%%
exp_base_out = sprintf('%s/%s/%s',FN.output_dir,organism.name,FN.dir_name);

output_fns = fieldnames(FN.output);
for i=1:length(output_fns)
  old_name = getfield(FN.output,output_fns{i});
  if isempty(old_name)
    old_name = '';
  end 
  if iscell(old_name)
    new_name ={};
    for j=1:length(old_name)
      if isempty(strmatch(FN.output_dir,old_name{j})) 
        new_name{j}  = sprintf('%s/genome_data/%s',exp_base_out,old_name{j} );
      else
        new_name{j}  = old_name{j} ;
      end
    end
  else
    if  isempty(strmatch(FN.output_dir,old_name)) 
      new_name  = sprintf('%s/genome_data/%s',exp_base_out,old_name );
    else
      new_name  = old_name ;
    end
  end
  FN.output = setfield(FN.output,output_fns{i},new_name);
end  

output_dir=sprintf('%s/genome_data/',exp_base_out);
if verb
  fprintf(1, 'dir for genes, split and regions: \n%s\n\n', output_dir)
end
if ~isfield( FN,'exp_dir')||isempty(FN.exp_dir)|| isempty(strmatch(FN.input_dir,FN.exp_dir)) 
  FN.exp_dir = sprintf('%s/%s',exp_base_out,FN.exp_name);
end

%%%%%%%%%%%%%%%
%%% SIGNAL SPECIFIC INPUT FILES 
%%%%%%%%%%%%%%%

signalnames = fieldnames(FN.input_sig);
for s=1:length(signalnames)
  signals = getfield(FN.input_sig,signalnames{s});
  signal_fns = fieldnames(signals);
  for i=1:length(signal_fns)
    old_name = getfield(signals,signal_fns{i}) ;
    if isempty(old_name)
       old_name = '';
    end 
    if isempty(strmatch(FN.input_dir,old_name)) 
      new_name  = sprintf('%s/%s/%s/%s',exp_base_in,FN.exp_name,dir_name_ext,old_name );
      % new_name  = sprintf('%s/%s/sensors/%s%s/%s',exp_base_in,FN.exp_name,signalnames{s},dir_name_ext,old_name );
    else
      new_name  = old_name ; 
    end
    signals = setfield(signals,signal_fns{i},new_name);
  end 
  FN.input_sig = setfield(FN.input_sig,signalnames{s},signals);
  
  % sig_input = sprintf('%s/%s/sensors/%s%s',exp_base_in,FN.exp_name,signalnames{s},dir_name_ext,old_name);
  if verb
    fprintf(1, 'dir for label_files and examples: \n%s\n\n', new_name)
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%
%% SIGNAL SPECIFIC OUTPUT DIRs AND FNs  
%%%%%%%%%%%%%%%%%%%%%%%%%



%output_fns = fieldnames(FN.output);
%for i=1:length(output_fns)
%  old_name = getfield(FN.output,output_fns{i});
%  new_name  = sprintf('%s/%s_%s/%s',FN.output_dir,FN.exp_name,...
%                      organism.name,old_name );
%  FN.output = setfield(FN.output,output_fns{i},new_name);
%end  

signalnames = fieldnames(FN.output_sig);
for s=1:length(signalnames)
  signals = FN.output_sig.(signalnames{s});
  signal_fns = fieldnames(signals);
  method = Signals.(signalnames{s}).method;
  for i=1:length(signal_fns)
    old_name = getfield(signals,signal_fns{i}) ;
    dir_name = sprintf('%s/%s/%s',exp_base_out,FN.exp_name,dir_name_ext);
    % dir_name = sprintf('%s/%s/sensors/%s%s',exp_base_out,FN.exp_name,signalnames{s},dir_name_ext);
    if isfield(FN, 'custom_output')
      dir_name = sprintf('%s/%s',dir_name,FN.custom_output);
    else
      dir_name = sprintf('%s/output_%s',dir_name,method.name);
      if isequal(method.name, 'SVM')
        if length(method.kernels)==1&&~isempty(method.kernels{1})
          if isfield(method.kernels{1},'name')
            kernel_names = method.kernels{1}.name; 
          else
            kernel_names = method.kernels{1}.kernel_name;
          end
        else
          kernel_names = 'COMBINED';
        end
        dir_name = sprintf('%s%s',dir_name,kernel_names);
      end
    end
    if isempty(strmatch(FN.output_dir,old_name)) 
      new_name  = sprintf('%s/%s',dir_name, old_name );
    else
      new_name = old_name;
    end
    signals = setfield(signals,signal_fns{i},new_name);
  end
  FN.output_sig = setfield( FN.output_sig, signalnames{s}, signals );
  if verb
    fprintf(1, 'dir for training and test data: \n%s\n\n', dir_name) 
  end
end


%%%%%%%%%%%%%%%
%%% CONTENT SPECIFIC INPUT FILES 
%%%%%%%%%%%%%%%

if isfield(FN, 'input_cont')
  cont_names = fieldnames(FN.input_cont);
  for s=1:length(cont_names)
    content = getfield(FN.input_cont,cont_names{s});
    cont_fns = fieldnames(content);
    for i=1:length(cont_fns)
      old_name = getfield(content,cont_fns{i}) ;
      if isempty(strmatch(FN.input_dir,old_name)) 
        new_name  = sprintf('%s/%s/%s/%s',exp_base_in,FN.exp_name,dir_name_ext,old_name );
        % new_name  = sprintf('%s/%s/contents/%s%s/%s',exp_base_in,FN.exp_name, cont_names{s},dir_name_ext,old_name );
      else
        new_name  = old_name ; 
      end
      content = setfield(content,cont_fns{i},new_name);
    end 
    FN.input_cont = setfield(FN.input_cont,cont_names{s},content);
  end

  %cont_input = sprintf('%s/%s/contents/%s',exp_base_in,FN.exp_name,signal_dir_name);
  if verb
    fprintf(1, 'dir for content specific input: \n%s\n\n', cont_input)
  end
end

%%%%%%%%%%%%%%%
%%% CONTENT SPECIFIC OUTPUT FILES 
%%%%%%%%%%%%%%%

if isfield(FN, 'output_cont')
  cont_names = fieldnames(FN.output_cont);
  for s=1:length(cont_names)
    content = getfield(FN.output_cont,cont_names{s});
    cont_fns = fieldnames(content);
    method = Contents.(cont_names{s}).method;
    for i=1:length(cont_fns)
      old_name = getfield(content,cont_fns{i}) ;
      dir_name = sprintf('%s/%s/%s',exp_base_out,FN.exp_name,dir_name_ext);
      if isfield(FN, 'custom_output')
        dir_name = sprintf('%s/%s',dir_name,FN.custom_output);
      else
        dir_name = sprintf('%s/output_%s',dir_name,method.name);
        if isequal(method.name, 'SVM')
          if length(method.kernels)==1&&~isempty(method.kernels{1})
            if isfield(method.kernels{1},'name')
              kernel_names = method.kernels{1}.name; 
            else
              kernel_names = method.kernels{1}.kernel_name;
            end
          else
            kernel_names = 'COMBINED';
          end
          dir_name = sprintf('%s%s',dir_name,kernel_names);
        end
      end
      if isempty(strmatch(FN.output_dir,old_name)) 
        new_name  = sprintf('%s/%s',dir_name, old_name );
      else
        new_name = old_name;
      end
      content = setfield(content,cont_fns{i},new_name);
    end 
    FN.output_cont = setfield(FN.output_cont, cont_names{s},content);
  end

  %cont_output = sprintf('%s/%s/contents/%s',exp_base_out,FN.exp_name,signal_dir_name);
  if verb
    fprintf(1, 'dir for content specific output: \n%s\n\n', cont_output)
  end
end

%%%%%%%%%%%%%%%
%%% LSL STUFF
%%%%%%%%%%%%%%%

if isfield(FN, 'input_lsl')
  fns = fieldnames(FN.input_lsl);
  FN.input_lsl.dir = sprintf('%s/%s/%s/%s/lsl/%s/', ...
			     exp_base_out,organism.name,FN.dir_name,FN.exp_name,FN.lsl_exp_name ); 
  for i=1:length(fns)
    if ~iscell(FN.input_lsl.(fns{i}))
      old_name = FN.input_lsl.(fns{i}) ;
      if isempty(strmatch(FN.input_lsl.dir,old_name))&~isequal(fns{i},'fn_old_training_blocks')...
            &~isequal(fns{i},'fn_old_test_blocks')&~isequal(fns{i},'fn_old_split')
        new_name  = sprintf('%s/%s',FN.input_lsl.dir,old_name);
      else
        new_name  = old_name ; 
      end
      FN.input_lsl.(fns{i}) = new_name;
    else
      for j=1:length(FN.input_lsl.(fns{i}))
        old_name = FN.input_lsl.(fns{i}){j} ;
        if isempty(strmatch(FN.input_lsl.dir,old_name))&~isequal(fns{i},'fn_old_training_blocks')...
              &~isequal(fns{i},'fn_old_test_blocks')&~isequal(fns{i},'fn_old_split')
          new_name  = sprintf('%s/%s',FN.input_lsl.dir,old_name);
        else
          new_name  = old_name ; 
        end
        FN.input_lsl.(fns{i}){j} = new_name;
      end    
    end
  end
end
%cont_output = sprintf('%s/%s/contents/%s',exp_base_out,FN.exp_name,signal_dir_name);
if verb
  fprintf(1, 'dir for lsl specific input: \n%s\n\n', cont_output)
end

if isfield(FN, 'output_lsl')
  fns = fieldnames(FN.output_lsl);
  FN.output_lsl.dir = sprintf('%s/%s/%s/%s/lsl/%s/', ...
			      exp_base_out, organism.name,FN.dir_name,FN.exp_name,FN.lsl_exp_name ); 
  for i=1:length(fns)
    old_name = FN.output_lsl.(fns{i}) ;
    if isempty(strmatch(FN.output_lsl.dir,old_name))
      new_name  = sprintf('%s/%s',FN.output_lsl.dir,old_name);
    else
      new_name  = old_name ; 
    end
    FN.output_lsl.(fns{i}) = new_name;
  end 
end
%cont_output = sprintf('%s/%s/contents/%s',exp_base_out,FN.exp_name,signal_dir_name);
if verb
  fprintf(1, 'dir for lsl specific output: \n%s\n\n', cont_output)
end


return

function PAR=masterscript_data(PAR,first)
  
%PAR=masterscript_data(PAR,first)

if nargin<2
  first =1 ;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% PREPARE COMPLETE GENE STRUCTURE 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if PAR.tasks.prepare_labels
  if first==1 
    fprintf('start preparing gene structure for organism: %s\n\n',PAR.organism.name)
    PAR.add_signals.tss = 1; 
    PAR.add_signals.cleave = 1 ; 
    PAR.add_signals.polya = 1;
    PAR.add_signals.tis = 1;
    PAR.add_signals.cdsStop = 1;
    PAR.add_signals.transacc = 0; 
  elseif first==2
    fprintf('adding transacc sites to gene structure for organism: %s\n\n',PAR.organism.name)
    PAR.add_signals.tss = 0; 
    PAR.add_signals.cleave = 0 ; 
    PAR.add_signals.polya = 0;
    PAR.add_signals.tis = 0;
    PAR.add_signals.cdsStop = 0;
    PAR.add_signals.transacc = 1; 
  end
  
  [genes_sensors,genes_merged] = prepare_complete_genes(PAR);
  if first==2
    PAR.add_signals.tss = 1; 
    PAR.add_signals.cleave = 1 ; 
    PAR.add_signals.polya = 1;
    PAR.add_signals.tis = 1;
    PAR.add_signals.cdsStop = 1;
    PAR.add_signals.transacc = 1; 
  end
  genes = genes_sensors;
  if fexist(PAR.FN.output.fn_genes_signals) || fexist([PAR.FN.output.fn_genes_signals '.mat']),
    warning(sprintf('do not save file as it already exists: %s', PAR.FN.output.fn_genes_signals)) ;
    L=load_struct(PAR.FN.output.fn_genes_signals, 'genes') ;
    assert(isequal(L, genes)) ;
    clear L
  else
    save_struct(PAR.FN.output.fn_genes_signals,genes,'genes')
    %save(PAR.FN.output.fn_genes_signals,'-append','PAR')
    save_append(PAR.FN.output.fn_genes_signals, 1, 'PAR', PAR)
  end ;
  genes = genes_merged;
  if fexist(PAR.FN.output.fn_genes_merged) || fexist([PAR.FN.output.fn_genes_merged '.mat']),
    warning(sprintf('do not save file as it already exists: %s', PAR.FN.output.fn_genes_signals)) ;
    L=load_struct(PAR.FN.output.fn_genes_merged, 'genes') ;
    assert(isequal(L, genes)) ;
    clear L
  else
    save_struct(PAR.FN.output.fn_genes_merged,genes,'genes')
    %save(PAR.FN.output.fn_genes_signals,'-append','PAR')
    save_append(PAR.FN.output.fn_genes_signals, 1, 'PAR', PAR)
  end 
  clear genes_merged, genes_sensors
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% PREPARE REGIONS AND BLOCKS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% keyboard
if first==1 &...
         (~fexist(PAR.FN.output.fn_trivial_regions) |...
          ~fexist(PAR.FN.output.fn_blocks)|...
          ~fexist(PAR.FN.output.fn_test_blocks))
  [trivial_regions,blocks,training,test] = prepare_regions(PAR);
  if ~fexist(PAR.FN.output.fn_trivial_regions)| ~fexist(PAR.FN.output.fn_blocks)  
    save(PAR.FN.output.fn_trivial_regions,'trivial_regions','PAR');
    save(PAR.FN.output.fn_blocks, 'blocks','PAR');
  end
  if 1 % ~isempty(training.blocks)&~fexist(PAR.FN.output.fn_training_blocks)
    %save(PAR.FN.output.fn_training_blocks,'-struct','training');
    save_append(PAR.FN.output.fn_training_blocks, 0, training);
    %save(PAR.FN.output.fn_training_blocks,'-append' ,'PAR');
    save_append(PAR.FN.output.fn_training_blocks, 1, 'PAR', PAR);
    PAR.regions.offset=PAR.regions.offset_train;
    save_splits2bed(training.blocks,training.split,PAR.FN.output.fn_training_blocks(1:end-4),PAR);
  end
  if 1 % ~isempty(test.blocks)&~fexist(PAR.FN.output.fn_test_blocks)
    %save(PAR.FN.output.fn_test_blocks,'-struct','test');
    save_append(PAR.FN.output.fn_test_blocks, 0, test);
    %save(PAR.FN.output.fn_test_blocks,'-append','PAR');
    save_append(PAR.FN.output.fn_test_blocks, 1, 'PAR', PAR);
    PAR.regions.offset=PAR.regions.offset_test;
    save_splits2bed(test.blocks,test.split,PAR.FN.output.fn_test_blocks(1:end-4),PAR);
  end
end

return

function [PAR, jobinfo ,all_done]=masterscript_sensors(PAR,check_files,keep_all_files,verb)
% [PAR, jobinfo ,all_done] = masterscript_sensors(PAR,check_files,verb)

%cd ~/svn/projects/genefinding/sensors
%paths

tasks = PAR.tasks.signals;  
%run_locally = tasks.run_locally;

FN = PAR.FN;  
fn_genome_config = PAR.FN.input.fn_genome_config;
 
if nargin<2 
  check_files =0;
end
if nargin<3 
  keep_all_files = 0;
end
if nargin<4 
  verb = 0;
end

fid =1;
fprintf(fid,'\n\n-----------\n');
fprintf(fid,'SENSOR :  %s\n',PAR.Signal_name);
fprintf(fid,'-----------\n');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% --- Train SENSORS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PAR.method.submit_batch=1;
all_done=0;
%train_ok = 0;

jobinfo=struct ;

if tasks.prepare_labels
  %fprintf('start training for sensor %s on organism %s with method %s: %s\n\n',sensor,organism,method)
  gen_training_examples4signal(PAR) ;
end
if tasks.modelsel | tasks.train
  jobinfo = ms_gfwd_starter(PAR);
  if ~isempty(jobinfo), return ; end ;
end
if tasks.get_train_error
  jobinfo = predict_testplif_geterror(PAR)
  if ~isempty(jobinfo), return ; end ;
end
if tasks.learn_plifs
  calc_plifs(PAR);
end

%if ~train_ok
%  fprintf('something wrong with training files, check first!!  \n')
%end


if ~keep_all_files
  unix(sprintf('rm -rf %s', FN.input_sig.(PAR.Signal_name).fn_candsites));
  unix(sprintf('rm -rf %s*', FN.input_sig.(PAR.Signal_name).fn_examples));
  unix(sprintf('rm -rf %s*', FN.input_sig.(PAR.Signal_name).fn_example_statistics(1:end-4)));  
  unix(sprintf('rm -rf %s', FN.input_sig.(PAR.Signal_name).fn_pos));
  unix(sprintf('rm -rf %s', FN.input_sig.(PAR.Signal_name).fn_filter_settings));

  unix(sprintf('rm -rf %s', FN.output_sig.(PAR.Signal_name).fn_pred));
  unix(sprintf('rm -rf %s/ms_set*', FN.output_sig.(PAR.Signal_name).fn_SVMs));
  unix(sprintf('rm -rf %s/partition=*', FN.output_sig.(PAR.Signal_name).fn_SVMs));  
end

if ~tasks.pred_on_genome
  all_done = 1 ;%train_ok ;
  return
end

return

function  jobinfo = gen_all_positions(fn_genome_config,fn_pos,Signal,fn_training_blocks,fn_test_blocks, num_splits,run_locally,RPROC) 
  
jobinfo = rproc_empty(0);
if ~run_locally,  
  jobinfo = rproc_empty(0) ;
  options = RPROC.options;
  % options.ncpus = 1;
  % options.identifier = sprintf('%spos%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;
  % options.identifier = sprintf('pos%s_%s',PAR.Signal_name,PAR.method.name) ;
end

strands = '+-';
genome_info = init_genome(fn_genome_config);

% keyboard

blocks = [];
if num_splits~=1 
  % load(FN.output.fn_regions, 'regions')
  % load(FN.output.fn_split,'split')
  if ~isempty(fn_training_blocks) && fexist(fn_training_blocks)

    % load training splits
    TR = [] ;
    TR.blocks = load_struct(fn_training_blocks, 'blocks');
    tmp = load(fn_training_blocks, 'split') ;
    TR.split = tmp.split ;
    if ~isempty(TR.split)& ~isempty(TR.blocks)
      has_split = isfield(TR.blocks,'split')&&~isempty(TR.blocks(1).split);
      splits = fieldnames(TR.split);
      for f=1:length(splits)
        regids = getfield(TR.split, splits{f});
        [temp i1 i2] = intersect(regids, [TR.blocks.id]);
        if ~has_split
          for ii2=i2, 
            [TR.blocks(ii2).split] = deal(f);
          end 
          %[TR.blocks(i2).split] = deal(f);
        else
          assert(all([TR.blocks(i2).split] == f))      
        end
      end
      clear temp i1 i2 splits regids 
      blocks = [blocks TR.blocks];
      clear TR
    end
  end
  if ~isempty(fn_test_blocks) && fexist(fn_test_blocks)
    TE=[] ;
    TE.blocks = load_struct(fn_test_blocks, 'blocks') ;
    tmp = load(fn_test_blocks, 'split');
    TE.split = tmp.split ;
    has_split = isfield(TE.blocks,'split');  
    splits = fields(TE.split);
    for f=1:length(splits)
      regids = getfield(TE.split, splits{f});
      [temp i1 i2] = intersect(regids, [TE.blocks.id]);
      if ~has_split
        for ii2=i2, 
          [TE.blocks(ii2).split] = deal(f);
        end 
        %[TE.blocks(i2).split] = deal(f);
      else
        assert(all([TE.blocks(i2).split] == f))      
      end
    end
    if isfield(TE.blocks,'right_dist')
      TE.blocks = rmfield(TE.blocks,'right_dist');
    end
    if isfield(TE.blocks,'right_dist')
      TE.blocks = rmfield(TE.blocks,'right_dist');
    end
    blocks = [blocks TE.blocks];
    clear TE;
  end
end

%[temp, num_files] = unix(sprintf('ls %scontig_*.pos | wc -l',fn_pos));
%d=dir(sprintf('%scontig_*.pos',fn_pos));
%num_files = length(d) ;
%[temp, num_files1] = unix(sprintf('ls %scontig_*.svm | wc -l',fn_pos));
%d=dir(sprintf('%scontig_*.svm',fn_pos));
%num_files1=length(d) ;

%----------
% generate positions
%-----------
trivial_regions = init_regions(fn_genome_config);

% comp_region  = ones(1,length(trivial_regions));
num_jobs=0;


for t=1:length(trivial_regions)
  if fexist(sprintf('%scontig_%i%s.pos',fn_pos,trivial_regions(t).chr_num,trivial_regions(t).strand))&...
        fexist(sprintf('%scontig_%i%s.svm',fn_pos,trivial_regions(t).chr_num,trivial_regions(t).strand))
    % comp_region(t)  = 0;
    continue
  end
  num_jobs = num_jobs+1;
  
  P.region = trivial_regions(t); 
  P.blocks = blocks;
  P.Signal = Signal;
  P.fn_pos = fn_pos;
  P.num_splits = num_splits;

  if ~run_locally
    [mem_req, time_req, opts] = rproc_memtime_policy('gen_all_positions_rproc',  0, options) ;
    jobinfo(num_jobs) = rproc_create('gen_all_positions_rproc', P, mem_req, opts, time_req) ;
  else
    gen_all_positions_rproc(P);
  end
end

return

return

function regions=load_sequence(regions);

% regions=load_sequence(regions);
  
genome_info = init_genome(regions(1).config);


for i=1:length(regions)
  if isfield(regions(i),'num_fragments')&&~isempty(regions(i).num_fragments)
    num_fragments = regions(i).num_fragments;
  else
    num_fragments = 1;
  end
  for p=1:num_fragments
    if mod(i,100)==0,
      fprintf(1,'region %i of %i\r',i,length(regions))
    end ;
    chr = regions(i).chr_num(p) ;
    str = regions(i).strand(p) ;
    start = regions(i).start(p) ;
    stop = regions(i).stop(p) ;
    seq = upper(load_genomic(genome_info.contig_names{chr},str,start,stop,genome_info));
    assert(ischar(seq)) ;
    idx = (~ismember(seq,'ACGT')) ;
    seq(idx)='N';
    if p==1
      regions(i).seq = seq;
    else
      regions(i).seq = [regions(i).seq ,seq];
    end
  end
end

return

function jobinfo = convert_predictions2confs(fn_genome_config,fn_pred,fn_svms,num_splits,run_locally, RPROC)
% jobinfo = convert_predictions2confs(fn_genome_config,fn_pred,fn_svms,num_splits,run_locally, RPROC)

jobinfo = rproc_empty(0) ;
% options.ncpus = 1;
% options.identifier = sprintf('%sconf%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;

fid=1;q=0;
for p = 1:num_splits
  fprintf(fid,'partition %i\n',p);
  
  fn_SVM = sprintf('%s%i.mat',fn_svms,p) ;
 
  %--------------------------------------------------------------------------------%
  % load parameters from SVM file
  %--------------------------------------------------------------------------------%
  
  genome_info = init_genome(fn_genome_config);
  clear  prob prob_cum limits Conf Conf_cum

  if ~(all(ismember({'prob', 'prob_cum','limits'}, who_file(fn_SVM))))
    warning('Variables prob, prob_cum and or limits not found in %s',fn_SVM)
    error('calculate PLIFS first: execute this function with PAR.tasks.pred_from_seq=0; to calculate PLIFS')
  else
    load(fn_SVM, 'Train');
    PAR_new = load(fn_SVM, 'prob', 'prob_cum', 'limits');
    if ~isfield(Train.PAR.SETs, 'which_pred')
      Train.PAR.SETs.which_pred = Train.PAR.SETs.which_pred_svm;
    end
  end
  assert(Train.PAR.SETs.which_pred == p)
  %------------------------------------------------------------------------------------%
  % perform transformations
  %------------------------------------------------------------------------------------%
  for c = 1:length(genome_info.contig_names)
    for s = '+-'
      PAR_new.fn_pred = sprintf('%scontig_%i%s_split=%i.mat',fn_pred,c,s,p);
      for j=1:20
        if ~fexist(PAR_new.fn_pred)
	  fprintf(fid,'\n waiting for file: %s\n',PAR_new.fn_pred);
          pause(30) 
	end
      end
      if ~fexist(PAR_new.fn_pred)
       fprintf(fid,'\n file not found: %s', PAR_new.fn_pred);
       fprintf(fid,'\n run genomewide_prediction first\n');
      %elseif all(ismember({'Conf', 'Conf_cum'}, who_file(PAR_new.fn_pred))),
      %  continue
      else
        fprintf(fid,'\n start transforming on contig %i%s\n',c,s);
        %%% Convert SVM Outputs on test set `
        q=q+1;
        if ~run_locally

          [mem_req, time_req, opts] = rproc_memtime_policy('Out2Confhelper',  0, RPROC.options) ;

          if isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs 
            jobinfo(q) = rproc_create('Out2Confhelper', PAR_new, mem_req, opts, time_req) ;
          else
            jobinfo(q) = rproc('Out2Confhelper', PAR_new, mem_req, opts, time_req);
          end
        else
          Out2Confhelper(PAR_new);
        end
      end
    end%loop over strands
  end%loop over contigs
end %% loop over partitions

rproc_wait(jobinfo, 20, 1, -1);

return

function genome_info=init_genome(config_fname)
% genome_info=init_genome(config_fname)  
%
% global g_genome_info ;

global g_genome_info genome_strings;

genome_strings = [];
g_est_fnames={} ;
g_prot_fnames={} ;
g_cdna_fnames={} ;
g_flcdna_fnames={} ;
g_contig_names={} ;
g_flat_fnames={} ;
g_fasta_fnames={} ;
g_annotation_fnames={} ;
g_alphabet='acgt' ;
g_basedir='' ;
g_max_intron_len=1000000 ;
g_min_intron_len=10 ;
g_merge_est_transcripts=1 ;

fd=fopen(config_fname, 'r') ;
while ~feof(fd),
  line = fgetl(fd) ;
  if ~ischar(line), break ; end ;
  if isempty(line), continue ; end ;
  if line(1)=='#', continue ; end ;
  if length(line)>8 && isequal(line(1:7),'BASEDIR')
    g_basedir=deblank(line(9:end)) ;
  elseif length(line)>21 && isequal(line(1:21), 'MERGE_EST_TRANSCRIPTS')
    g_merge_est_transcripts = str2num(line(22:end)) ; 
  elseif length(line)>15 && isequal(line(1:15),'MAX_DOUBLE_QGAP')
  elseif length(line)>15 && isequal(line(1:15),'MAX_DOUBLE_TGAP')
  elseif length(line)>17 && isequal(line(1:17),'MAX_CDNA_DELETION')
  elseif length(line)>18 && isequal(line(1:18),'MAX_CDNA_INSERTION')
  elseif length(line)>21 && isequal(line(1:21),'TERMINAL_EXON_END_TOL')
  elseif length(line)>14 && isequal(line(1:14),'MIN_INTRON_LEN')
    g_min_intron_len = str2num(line(15:end)) ; 
    assert(~isempty(g_min_intron_len) & g_min_intron_len<1000) ;
  elseif length(line)>14 && isequal(line(1:14),'MAX_INTRON_LEN')
    g_max_intron_len = str2num(line(15:end)) ; 
    assert(~isempty(g_max_intron_len) & g_max_intron_len>1000) ;
  elseif length(line)>18 && isequal(line(1:18),'MIN_EST_COVER_FRAC')

  elseif length(line)>19 && isequal(line(1:19),'MIN_PROT_COVER_FRAC')

  elseif length(line)>19 && isequal(line(1:19),'MIN_CDNA_COVER_FRAC')
  
  elseif length(line)>18 && isequal(line(1:18),'BLAT_BEST_HIT_ONLY')
  
  elseif length(line)>20 && isequal(line(1:20),'BLAT_BEST_HIT_MARGIN')

  elseif length(line)>8 && isequal(line(1:8),'ALPHABET')
    g_alphabet=deblank(line(10:end)) ;
  elseif length(line)>7 && isequal(line(1:7),'CONTIGS'),
    for i=1:str2num(line(8:end))
      l2=fgetl(fd) ; 
      idx=strfind(l2,'  ') ;
      while ~isempty(idx), 
        l2(idx)=[] ;
        idx=strfind(l2,'  ') ;
      end ;
      l2(l2==' ')=sprintf('\t') ;
      elems=separate(l2) ; 
      assert(length(elems)==3) ;
      g_contig_names{i}=elems{1} ;
      g_flat_fnames{i}=path_with_basedir(elems{2}, g_basedir) ;
      g_fasta_fnames{i}=path_with_basedir(elems{3}, g_basedir) ; 
    end ;
  elseif length(line)>8 && isequal(line(1:8),'ESTFILES'),
    for i=1:str2num(line(9:end))
      g_est_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>9 && isequal(line(1:9),'PROTFILES'),
    for i=1:str2num(line(10:end))
      g_prot_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>9 && isequal(line(1:9),'CDNAFILES'),
    for i=1:str2num(line(10:end))
      g_cdna_fnames{end+1}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>11 && isequal(line(1:11),'FLCDNAFILES'),
    for i=1:str2num(line(12:end))
      g_flcdna_fnames{end+1}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>15 && isequal(line(1:15),'ANNOTATIONFILES'),
    for i=1:str2num(line(16:end))
      g_annotation_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif ~isempty(line),
    error(sprintf('cannot understand "%s" in config file', line)) ;
  end ;
end ;
fclose(fd) ;

genome_info=[] ;
genome_info.est_fnames=g_est_fnames ;
genome_info.prot_fnames=g_prot_fnames ;
genome_info.cdna_fnames=g_cdna_fnames ;
genome_info.flcdna_fnames=g_flcdna_fnames ;
genome_info.contig_names=g_contig_names ;
genome_info.flat_fnames=g_flat_fnames ;
genome_info.fasta_fnames=g_fasta_fnames ;
genome_info.annotation_fnames=g_annotation_fnames ;
genome_info.alphabet=g_alphabet ;
genome_info.basedir=g_basedir ;
genome_info.max_intron_len = g_max_intron_len ;
genome_info.min_intron_len = g_min_intron_len ;
genome_info.merge_est_transcripts = g_merge_est_transcripts ;
g_genome_info=genome_info ;

return ;


function p=path_with_basedir(p1,basedir)
  
  if p1(1)=='/',
    p=p1; 
    return ;
  else
    if basedir(end)=='/',
      p=[basedir p1] ;
    else
      p=[basedir '/' p1] ;
    end ;
  end ;
  
  

return

function [str,valid_splice,ok] = load_genomic(contig,strand,start,stop, genome_info,check_splice)
% function [str,valid_splice,ok]=load_genomic(contig,strand,start,stop, genome_info,check_splice)

if nargin<6
  check_splice=0;
end ;
ok=1 ;

if ~check_splice 
  valid_splice=[];
  % fprintf(1,'no splice site consensus check!')
end ;


if length(start)>1 | length(stop)>1,
  if nargout<3
    assert(length(start)==length(stop)) ;
    assert(all([start(2:end)-stop(1:end-1)]>=0));
  else
    if ~(length(start)==length(stop)) || ~(all([start(2:end)-stop(1:end-1)]>=0)),
      ok=0 ;
      str='' ;
      valid_splice = [] ;
      return ;
    end ;
  end ;
  valid_splice = zeros(length(start),2); 
  if strand == '+', 
    idx = 1:length(start) ; 
  else 
    idx = length(start):-1:1 ; 
  end ;
  str = '' ;
  for i=idx  %1:length(start)
    seq = load_genomic(contig, strand, start(i)-5, stop(i)+5, genome_info) ;
    str = [str seq(6:end-5)] ;

    % check internal splicing sites
    if check_splice
      % if (i~=1 && strand == '+') || (i~=length(start) && strand == '-')
      if (strand == '+') || (i~=length(start) && strand == '-')
        valid_splice(i,1) = isequal(seq(4:5),'ag') ;
        %if ~isequal(seq(4:5),'ag'), fprintf('acc: %s %c\n', seq(3:6), strand); end ;
        % assert(isequal(seq(4:5),'ag'))
      end ;
      %if i~=idx(end)
      % if (i~=1 && strand == '-') || (i~=length(start) && strand == '+')
      if (strand == '-') || (i~=length(start) && strand == '+')        
        valid_splice(i,2) = ismember(seq(end-5:end-4), {'gt','gc'}) ;
        %if ~isequal(seq(end-4:end-3),'gt')||isequal(seq(end-4:end-3),'gc'), fprintf('don: %s %c\n', seq(end-5:end-2), strand); end ;
        % assert(isequal(seq(end-4:end-3),'gt')||isequal(seq(end-4:end-3),'gc'))
      end ; 
    end ;
  end
  return ;
else
  valid_splice=[0 0];
end ;


if ischar(contig),
  %contig_idx = find(ismember(upper(genome_info.contig_names),upper(contig))) ;
  contig_idx = strmatch(upper(contig), upper(genome_info.contig_names), 'exact') ;
  assert(all(size(contig_idx)==1) & ~isempty(contig_idx)) ;
else
  contig_idx = contig ;
end ;
  
fname=genome_info.flat_fnames{contig_idx} ;
if genome_info.alphabet(1)=='a',
  NN='n' ;
else
  NN='N' ;
end ;

d=dir(fname) ; left_n = 0 ; right_n=0 ;
if isinf(stop), stop=d.bytes; end ;

if start<1 | stop>d.bytes,
  warning('load_genomic:contig_boundary', 'boundary of contig reached (start: %i, stop: %i, d.bytes: %i), padding with "n"', start, stop,d.bytes) ;

  if start<1,
    left_n = -(start-1) ;
    start = 1 ;
  end ;
  if stop>d.bytes,
    right_n = stop-d.bytes ;
  end ;
  if start>d.bytes && stop>d.bytes,
    str = char(NN*ones(1,stop-start)) ;
    return ;
  end ;
end ;

fd=fopen(fname,'r') ;
fseek(fd, start-1, -1) ;
str=char(fread(fd,stop-start+1, 'char=>char'))' ;
fclose(fd) ;
if left_n>0,
  str=char([NN*ones(1,left_n) str]) ;
end ;
if right_n>0,
  str=char([str NN*ones(1,right_n)]) ;
end ;

assert(length(str)==stop-start+left_n+1)
%let = bigunique(str) ;
let = unique(str) ;
if genome_info.alphabet(1)=='a' && ~isempty(setdiff(let,'acgtn')),
  % warning(sprintf('invalid letter in genome! ("%s")', setdiff(let,'acgtn')));
  let = setdiff(let, 'acgtn') ;
  for c=let,
    str(str==c) = 'n';
  end ;
elseif genome_info.alphabet(1)=='A' && ~isempty(setdiff(let,'ACGTN')),
  warning(sprintf('invalid letter in genome! ("%s")', setdiff(let,'ACGTN')));
  let = setdiff(let, 'ACGTN') ;
  for c=let,
    str(str==c) = 'N';
  end ;
end

if strand=='-',
  str=reverse_complement(str) ;
end ;

return ;

return

function POS=  find_consensus(seq,Signal,strand)
%  POS=  find_consensus(seq,Signal,strand) 

consensus = Signal.consensus ; 
lwin_big = Signal.lwin_big;
rwin_big = Signal.rwin_big;
offset = Signal.offset;



POS = zeros(1,floor(length(seq)/10));
num = 0;
seq=upper(seq);
for m = 1:length(consensus)  
  cons = upper(consensus{m});
  pos_tmp = findstr(seq(lwin_big+1:end-rwin_big),cons)+lwin_big;
  POS(num+[1:length(pos_tmp)]) = pos_tmp;
  num = num + length(pos_tmp);
end
POS(num+1:end)=[];
clear num 
POS = POS + offset;
%if strand=='-' & strcmp(Signal.name,'polya')
%  POS = -POS + length(seq)+1-length(cons)+1;
%elseif strand=='-' 
if strand=='-'
  POS = -POS + length(seq)+1;
end
POS = unique(sort(POS));

return

function x=reverse_complement(y)

if length(y)>200*1024*1024
  x=uint8(y) ;
  for i=1:length(y),
    x(i)=y(end-i+1) ;
  end ;
  y=x ;
else
  y=y(end:-1:1) ;
  x=y ;
end ;

x(y=='a')='t' ;
x(y=='c')='g' ;
x(y=='g')='c' ;
x(y=='t')='a' ;
x(y=='n')='n' ;
x(y=='A')='T' ;
x(y=='C')='G' ;
x(y=='G')='C' ;
x(y=='T')='A' ;
x(y=='N')='N' ;

x=char(x) ;

return

function jobinfo = genomewide_predictions(fn_genome_config,fn_pos,fn_pred,fn_svms,num_splits,avg_all_svms,run_locally, RPROC, verb)
% jobinfo = genomewide_predictions(fn_pos,fn_pred,fn_genome_config,verb)


if nargin<9
  verb=0;
end
if nargin<7
  run_locally=1;
end
if nargin<6
  avg_all_svms = 0;
end

jobinfo = rproc_empty(0) ;
if ~run_locally
  jobinfo = rproc_empty(0) ;
  options = RPROC.options;
  options.ncpus = 1;
  % options.identifier = sprintf('%spred%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;
  MEMREQ = RPROC.MEMREQ;
end

    
fid=1;
q=0;

genome_info = init_genome(fn_genome_config);
num_pred = num_splits * length(genome_info.contig_names)*2;
fprintf(fid,'total number of jobs for prediction (num_svms*num_contigs*num_strands): %i\n',num_pred);

trivial_regions = init_regions(fn_genome_config);

for p = 1:num_splits
  fprintf(fid,'partition %i\n',p);
  
  fn_SVM = sprintf('%s%i.mat',fn_svms,p) ;
  %--------------------------------------------------------------------------------%
  % load parameters from SVM file
  %--------------------------------------------------------------------------------%
  
  load(fn_SVM,'Train');
  assert(Train.PAR.SETs.which_pred == p)
  
  PAR_new = Train.PAR;
  
  if isfield(PAR_new.Signal.method,'genomewide_predict_fct') && ~isempty(PAR_new.Signal.method.genomewide_predict_fct)
    PAR_new.Signal.method.predict_fct =  PAR_new.Signal.method.genomewide_predict_fct;
  end
  PAR_new.fn_SVMs = fn_SVM;
  PAR_new.pred_from_seq=1;
  PAR_new.avg_all_svms = avg_all_svms ; 
  if ~isfield(PAR_new.SETs,'which_pred')
    PAR_new.SETs.which_pred=PAR_new.SETs.which_pred_svm;
  end  
  assert(Train.PAR.SETs.which_pred == p)
  clear Train
  
  %------------------------------------------------------------------------------------%
  % perform genome-wide predictions
  %------------------------------------------------------------------------------------%
  for r_idx = 1:length(trivial_regions) 
    c = trivial_regions(r_idx).chr_num;
    s = trivial_regions(r_idx).strand;
    PAR_new.fn_pos = fn_pos;
    PAR_new.fn_pred = sprintf('%scontig_%i%s_split=%i.mat',fn_pred,c,s,p);
    if verb
      fprintf(fid,'\n start predicting on contig %i%s\n',c,s);
    end
    if ~fexist(PAR_new.fn_pred) 
      PAR_new.region = trivial_regions(r_idx);
      PAR_new.plif_on = 0;
      PAR_new.eval_on = 0;

      q=q+1; 
      if ~run_locally


        if ~isfield(RPROC,'divide4pred') || ~RPROC.divide4pred

          [mem_req, time_req, opts] = rproc_memtime_policy('load_and_predict',  0, options) ;

          if isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs 
            jobinfo(q) = rproc_create('load_and_predict', PAR_new, mem_req, opts, time_req) ;
          else
            jobinfo(q) = rproc('load_and_predict', PAR_new, mem_req, opts, time_req) ;
          end
        else
          q=q-1;
          
          output = [];
          pos = [];
          all_done=1;Temp_old =[];
          for chunk_idx=1:RPROC.numchunks
            PAR_new.chunk_idx = chunk_idx;PAR_new.RPROC.numchunks = RPROC.numchunks ;
            PAR_new.RPROC.divide4pred = RPROC.divide4pred;

            [mem_req, time_req, opts] = rproc_memtime_policy('load_and_predict',  0, options) ;

            if  ~fexist(sprintf('%s%i.mat',PAR_new.fn_pred, chunk_idx)) && isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs 
              q=q+1;
              jobinfo(q) = rproc_create('load_and_predict', PAR_new, mem_req, opts, time_req);
              all_done=0;
            elseif  ~fexist(sprintf('%s%i.mat',PAR_new.fn_pred, chunk_idx))
              q=q+1;
              jobinfo(q) = rproc('load_and_predict', PAR_new, mem_req, opts, time_req);
              all_done=0;
            else 
              Temp = load(sprintf('%s%i.mat',PAR_new.fn_pred, chunk_idx) )
              Temp.PAR = rmfield(Temp.PAR,'RPROC');
              Temp.PAR = rmfield(Temp.PAR,'chunk_idx');
              if ~isempty(Temp_old)
                assert(isequal(Temp_old.svmFileName,Temp.svmFileName));
                assert(isequal(Temp_old.PAR,Temp.PAR));
                assert(isequal(Temp_old.sg_ver,Temp.sg_ver));
              end
              Temp_old = Temp;
              output = [output Temp.output];
              pos = [pos Temp.pos];
            end
          end
          if all_done
            Temp = rmfield(Temp,'output');Temp = rmfield(Temp,'pos');
            %save(PAR_new.fn_pred,'struct','Temp');
            save_append(PAR_new.fn_pred, 0, Temp);
            %save(PAR_new.fn_pred,'-append','output','pos');
            inventory = {'output', 'pos'};
	    assert(isequal(size(pos),size(output)))
            save_append(PAR_new.fn_pred, 1, 'inventory', inventory,  'output', output, 'pos', pos);
          end
        end
      else
        load_and_predict(PAR_new)
      end
    else
      if verb
        fprintf(fid,'\n file already exists ');
      end
    end
  end%loop over contigs
end %% loop over partitions

if ~exist('jobinfo')| isempty(jobinfo),
  jobinfo = rproc_empty(0) ;
  return 
end

% [jobinfo, meta_jobinfo] = rproc_submit_batch(jobinfo, PAR.RPROC.exm_per_batch) ;
if ~(isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs),
%   [jobinfo,num_crashed] = rproc_wait(jobinfo,20,1,0);
end ;

[jobinfo,num_crashed] = rproc_wait(jobinfo, 30, 1, -1);


return

function x = fexist(filename,gz)
% FEXIST Check if file exists
%        FEXIST('filename') returns:
%        0 if filename does not exist
%        1 if filename exists in the given path

if nargin<2,
  gz=0;
end ;

fd = fopen(filename);
if fd ~= -1
  x = 1;
  fclose(fd);
elseif gz,
  fd = fopen([filename '.gz']);
  if fd~=-1,
    x=1 ;
    fclose(fd) ;
  else
    x=0 ;
  end ;
else
  x = 0;
end

return

function f=separate(str, delim, merge_multiple) ;
% f=separate(str, delimiter, merge_multiple_delimiters) ;
  
if nargin<2, delim=sprintf('\t') ; end ;
if nargin<3, merge_multiple=0 ; end ;
f={} ;
idx=[0 find(str==delim) length(str)+1] ;
if merge_multiple,
  idx2=find(idx(1:end-1)~=idx(2:end)-1) ;
  idx=[idx(idx2) idx(end)] ;
end 
for i=1:length(idx)-1
  f{i}=deblank(str(idx(i)+1:idx(i+1)-1)) ;
end ;

return

function [Data,par] = load_data(Signal,fn_filter_settings,fn_examples, which, filter_mode,subsample_neg,subsample_pos)
% [Data,par] = load_data(Signal,fn_filter_settings,fn_examples, which, filter_mode,subsample_neg,subsample_pos)
% [Data,par] = load_data(PAR, which, filter_mode,subsample_neg,subsample_pos)
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% load data of training set
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% method = PAR.method;
% Signal = PAR.Signals.(PAR.Signal_name);
%  FN = PAR.FN;
% input_sig = FN.input_sig.(Signal_name);
% fn_filter_settings = input_sig.fn_filter_settings;
% fn_examples = input_sig.fn_examples;

method = Signal.method;

XT = []; LT = [] ; POS.region_id = []; POS.pos=[]; POS_tmp = []; 
SC = []; use_sc = [];
if nargin<4
  subsample_neg=1;
end
if nargin<5
  subsample_pos=1;
end

use_cons = Signal.method.use_cons;
% if nargin<6
%   use_cons = 0;
% end
for w = which
  % create filename of input file
  filename = gen_example_filename(Signal,fn_filter_settings,fn_examples,filter_mode,w);
  
  fprintf('loading XT, LT and POS from file %s \n',filename)
  L = load(filename,'LT');
  L.XT = load_matrix(filename, 'XT');
  P = load(filename, 'POS');
  %if isfield(signal,'center')
  %  par.center = signal.center;
  %else
  %  par.center = size(L.XT,1)./2;
  %end
  
  have_pos = isfield(P, 'POS') ;
  have_lt = isfield(L, 'LT') ;
  if ~have_lt, L.LT=[] ; end ;
  have_sc = isfield(L, 'SC_') ;
  if ~have_sc, L.SC_=[] ; end ;  

  [L.XT, L.LT, P.POS, L.SC_] = subsample(L.XT, L.LT, P.POS, L.SC_, subsample_neg, subsample_pos) ;

  if ~have_lt, L=rmfield(L, 'LT') ; end ;
  if ~have_sc, L=rmfield(L, 'SC_') ; end ;
  
  if ~isfield(method,'feature_type') || isempty(method.feature_type) ||isequal(method.feature_type, 'char')
    L.XT_char = char(L.XT) ;
    for i=1:size(L.XT,1)
      L.XT_char(i,:) = char(double(L.XT(i,:))) ;
    end ;
    XT = [XT L.XT_char];
  elseif isequal(method.feature_type, 'uint8')
    L.XT_char = uint8(L.XT) ;
    for i=1:size(L.XT,1)
      L.XT_char(i,:) = uint8(double(L.XT(i,:))) ;
    end ;
    XT = [XT L.XT_char];
  elseif isequal(method.feature_type, 'double')
    XT = [XT L.XT];
  else
    error('unknown feature type; should be "char" or "double"')
  end
  if isfield(L,'LT')
    if size(L.LT,1)>size(L.LT,2)
      L.LT = L.LT';
    end
    LT = [LT L.LT] ;
  end
  
  if (isfield(L,'SC_')&& isfield(L.use_sc)) || use_cons
    fprintf('using consensus information\n');
    SC = [SC L.SC_] ;
    use_sc = [use_sc L.use_sc];
  end

  POS.region_id =[POS.region_id P.POS.region_id];
  POS.pos = [POS.pos P.POS.pos];
end ;
%%% just for compatibility with older versions 
  
% check whether everything is ok (matlab bug successfully avoided?)
for i=1:size(XT,1)
  if all(XT(i,:)==0)
    % i
    % keyboard
  end
  if  ~isfield(method,'feature_type') || isequal(method.feature_type, 'char')
    assert(all(XT(i,:)~=0)) ;
  end
end ;

clear L
if ~isempty(LT)
  fprintf('%i positive, %i negative examples (%.2f %% positives), sequences of length %i \n',sum(LT==1),sum(LT==-1),sum(LT==1)/length(LT)*100, size(XT,1));
end

if ~isempty(SC)
  XT(:,~use_sc) = [] ;
  POS(~use_sc) =[];
  LT(~use_sc) = [];
  SC(:,~use_sc) = [];
  if ~isempty(LT)
    fprintf('%i positive, %i negative examples (%.2f %% positives) \n',sum(LT==1),sum(LT==-1),sum(LT==1)/sum(LT==1|LT==-1)*100);
  end
end

Data.LT = LT;
% Data.XT = char(XT) ;
Data.XT = XT;
Data.Pos = POS;
clear XT LT POS
%this assert is very mem consuming
%assert(~any(XT(:)~='A' & XT(:)~='G' & XT(:)~='C' & XT(:)~='T')) ;
% XT = to_upper_case(char(XT)) ;
if ~isempty(SC)
  Data.SC = char(SC) ;
  assert(~any(SC(:)~='A' & SC(:)~='G' & SC(:)~='C' & SC(:)~='T')) ;
  
  % SC = to_upper_case(char(SC)) ;
end


assert(size(Data.XT,2)==length(Data.LT))
assert(length(Data.LT)==length(Data.Pos.pos))
assert(length(Data.LT)==length(Data.Pos.region_id))

return

function [num_data, num_pos, num_neg] = count_data(Signal, fn_filter_settings, fn_examples, which, filter_mode, subsample_neg, subsample_pos)
% [num_data, num_pos, num_neg] = count_data(Signal, fn_filter_settings, fn_examples, which, filter_mode,subsample_neg,subsample_pos)
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% count data of training set
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

method = Signal.method;

LT = [] ; POS.pos=[]; 
if nargin<4
  subsample_neg=1;
end
if nargin<5
  subsample_pos=1;
end

for w = which
  % create filename of input file
  filename = gen_example_filename(Signal,fn_filter_settings,fn_examples,filter_mode,w);
  
  fprintf('loading LT and POS from file %s \n',filename)
  L = load(filename, 'LT');
  P = load(filename, 'POS');
  
  have_pos = isfield(P, 'POS') ;
  have_lt = isfield(L, 'LT') ;
  if ~have_lt, L.LT=[] ; end ;

  [tmp, L.LT, P.POS] = subsample(L.LT, L.LT, P.POS, [], subsample_neg, subsample_pos) ;

  if ~have_lt, L=rmfield(L, 'LT') ; end ;
  
  if isfield(L,'LT')
    if size(L.LT,1)>size(L.LT,2)
      L.LT = L.LT';
    end
    LT = [LT L.LT] ;
  end
  
  POS.pos = [POS.pos P.POS.pos];
end ;
%%% just for compatibility with older versions 
  
clear L
if ~isempty(LT)
  fprintf('%i positive, %i negative examples (%.2f %% positives)\n',sum(LT==1),sum(LT==-1),sum(LT==1)/length(LT)*100);
  num_data = length(LT) ;
  num_pos = sum(LT==1) ;
  num_neg = sum(LT==-1) ;
else
  num_data = length(POS.pos) ;
  num_pos = nan ;
  num_neg = nan ;
end

return

function [Data,par] = load_content_data(Content,fn_filter_settings,fn_examples, which, filter_mode,subsample_neg,subsample_pos)
% [Data,par] = load_content_data(Content,fn_filter_settings,fn_examples, which, filter_mode,subsample_neg,subsample_pos)


method = Content.method;

XT = []; LT = [] ;  POS.region_id = []; POS.pos=[]; POS_tmp = []; 

if nargin<4
  subsample_neg=1;
end
if nargin<5
  subsample_pos=1;
end

use_cons = method.use_cons;
% if nargin<6
%   use_cons = 0;
% end
for w = which
  % create filename of input file
  filename = gen_example_filename(Content,fn_filter_settings,fn_examples,filter_mode,w);
  
  fprintf('loading XT, LT and POS from file %s \n',filename)
  L = load(filename,'LT','XT','POS');
  %  L.XT = load_matrix(filename, 'XT');
  % [L.XT, L.LT, P.POS, L.SC_] = subsample(L.XT, L.LT, P.POS, L.SC_, subsample_neg, subsample_pos) ;

  if isfield(L,'LT')
    if size(L.LT,1)>1,
      L.LT = L.LT';
    end
    LT = [LT L.LT] ;
  end
  POS.region_id =[POS.region_id L.POS.region_id];
  POS.pos = [POS.pos L.POS.pos];
  
  % generate feature vectors
  %----------------------------------------------------------------
  dim = 0;
  for o=1:length(method.kernels)
    dim = dim + 4^method.kernels{o}.wordlen;
  end
  xt = zeros(dim,length(L.XT));
  for j=1:length(L.XT) 
    dim = 0;
    for o=1:length(method.kernels)
      [mask hist] = compute_mers(L.XT{j}, method.kernels{o}.wordlen, method.kernels{o}.stepping, method.kernels{o}.offset);
      xt(dim+[1:4^method.kernels{o}.wordlen],j) = hist';
      dim = dim+ 4^method.kernels{o}.wordlen;
    end
  end
  clear L
  XT=[XT xt];
  
end ;

if ~isempty(LT)
  fprintf('%i positive, %i negative examples (%.2f %% positives), sequences of length %i \n',sum(LT==1),sum(LT==-1),sum(LT==1)/length(LT)*100, size(XT,1));
end


Data.LT = LT;
% Data.XT = char(XT) ;
Data.XT = XT;
Data.Pos = POS;
clear XT LT POS

assert(size(Data.XT,2)==length(Data.LT))
% assert(length(Data.LT)==length(Data.Pos.pos))
% assert(length(Data.LT)==length(Data.Pos.region_id))







return

function regions = retrieve_signals(regions,base_dir,signal_name,fields,which_pos,offset)

% regions = retrieve_signals(regions,base_dir,signal_name,fields,which_pos)
%
% loads all candidate positions and addional informations as specified in
% fields,(i.e.'label','anno_cover','est_cover','cdna_cover','Conf_cum' )
% from binary fiels and adds them to the field Signals.signal_name of the
% structure regions 
%
% see also save_label_files
if nargin<6
  offset = 0;
end
  
if nargin<5
  which_pos.local = 1;
  which_pos.contig = 1;
end

genome_info = init_genome(regions(1).config);

f_name = sprintf('%scontig_%i%s.pos', base_dir,regions(1).chr_num,regions(1).strand );
if ~fexist(f_name),
  fprintf('file %s does not exist\n', f_name);
end

if ~isempty(fields)
  f_idx = [];
  for f=1:length(fields),
    f_name = sprintf('%scontig_%i%s.%s', base_dir,regions(1).chr_num,regions(1).strand, fields{f});
    if fexist(f_name),
      f_idx = [f_idx f];
    else
      fprintf('file %s does not exist\n', f_name);
    end
  end
  if ~isempty(f_idx),
    fields = fields(f_idx);
  else
    error('no files available')
  end
end

if ~isfield(regions,'Signals')
  regions(1).Signals = [];
end


for r=1:length(regions)
  if isfield(regions(r),'num_fragments')&&~isempty(regions(r).num_fragments)
    num_fragments = regions(r).num_fragments;
  else
    num_fragments = 1;
  end
  prefix = 0;
  
  SIGNALS.pos = [];
  SIGNALS.contig_pos = [];
  for f=1:length(fields)
    SIGNALS.(fields{f})= [];
  end
  if ismember('pos2',fields)
    SIGNALS.contig_pos2 = [];
  end
  for p=1:num_fragments
    contig_length = dir(genome_info.flat_fnames{regions(r).chr_num(p)}) ;
    contig_length = contig_length.bytes;
    %if 1%r==5145
    %  fprintf(1, 'regions.stop: %i, regions.start: %ir: %i',regions(r).start,regions(r).stop, r)
    %end
    if regions(r).start(p)<1
      warning('regions.start<1; set to 1!')
      regions(r).start(p) = 1;
    end
    if regions(r).stop(p)>contig_length
      warning('regions.stop>contig length; set to contig_length!')
      regions(r).stop(p) = contig_length;
    end
    f_name = sprintf('%scontig_%i%s',base_dir, regions(r).chr_num(p), ...
                     regions(r).strand(p));
    if fexist([f_name '_sha1sum.mat'])
      [tmp,sha1sum] = unix(sprintf('sha1sum %s.pos',f_name));
      sha1sum = separate(sha1sum,' ');
      Old = load([f_name '_sha1sum'],'sha1sum');
      Old.sha1sum = separate(Old.sha1sum,' ');
      if ~isequal(Old.sha1sum{1},sha1sum{1})
        warning('sha1sum different')
      end
      %  else
      %    warning('no sha1sum check performed')
    end
    
    if isempty(fields)
      fields{1} ='pos';
    end
    if regions(r).start(p)+offset>regions(r).stop(p)-offset
      regions(r).Signals.(signal_name) = SIGNALS;
      continue
    end
    d=dir([f_name '.pos']) ;
    if d(1).bytes==0,
      contig_pos = [] ;
      score = zeros(0, length(fields)) ;
    else
      [contig_pos,score] = interval_query(f_name,fields,[regions(r).start(p)+offset;regions(r).stop(p)-offset]);
    end ;
    if ismember('pos2',fields)
      if ~isempty(contig_pos)
        idx_p = find(strcmp('pos2',fields));
        contig_pos2 = score(:,idx_p);
      else
        contig_pos2 =[];
      end
    end
    assert(isequal(contig_pos, sort(contig_pos))) ;
    %assert(issorted(contig_pos))
    if ~isempty(contig_pos)
      if ~(contig_pos(1)>=regions(r).start(p)&contig_pos(end)<=regions(r).stop(p))
        if keyboard_allowed()
          keyboard
        end;
      end
    end
    if which_pos.local
      if regions(r).strand(p)=='+'
        pos = contig_pos - regions(r).start(p) + 1+prefix;
        if exist('contig_pos2')
          pos2 = contig_pos2 - regions(r).start(p) + 1+prefix;
        end
        SIGNALS.pos = [SIGNALS.pos pos];
      else
        pos = regions(r).stop(p) - contig_pos + 1;
        [pos,idx] = sort(pos);
        score = score(idx,:);
        contig_pos = contig_pos(idx);
        if exist('contig_pos2')
          contig_pos2 = contig_pos2(idx);
          pos2 = regions(r).stop(p) - contig_pos2 + 1;
          SIGNALS.pos = [SIGNALS.pos pos2];
        else
          SIGNALS.pos = [SIGNALS.pos pos];
        end
      end
    end
    if which_pos.contig
      SIGNALS.contig_pos = [SIGNALS.contig_pos contig_pos];
      if exist('contig_pos2')
        SIGNALS.contig_pos2 = [SIGNALS.contig_pos2 contig_pos2];
        if regions(r).strand(p)=='+'
          SIGNALS.pos2 = [SIGNALS.pos2 pos2];
        else
          SIGNALS.pos2 = [SIGNALS.pos2 pos];
        end
      end
    end
    for f=1:length(fields)
      if strmatch(fields{f},'label') ix=f; end
      if ~isequal(fields{f},'pos')&~isequal(fields{f},'pos2')
        SIGNALS.(fields{f})= [SIGNALS.(fields{f}), score(:,f)];
      end
    end
    if exist('ix')
      u = unique(score(:,ix));
      if length(u)==1 assert(u==-1|u==1),
      elseif length(u)==2 assert(all(u==[-1;1])),
      elseif isempty(u),
      else error('unique(label) is not [-1;1]');
      end
    end
    regions(r).Signals.(signal_name) = SIGNALS;
    prefix = prefix +regions(r).stop(p)- regions(r).start(p);
  end %%loop over fragments
end %% loop over blocks

return

function [RFV, RSV, train_test_overlap] = svm_predict(Train, Data, svmFileName, outFileName, pred_from_seq, check_overlap);
% [RFV, RSV] = svm_predict(Train, Data, svmFileName,outFileName);
%
%------------------------------------------------------------------------------%
% deal with parameters 
%------------------------------------------------------------------------------%
% for usage with rproc

  if nargin==1
    Data  = Train.Data;
    svmFileName = Train.svmFileName;
    outFileName = Train.outFileName;
    Train = Train.Train;
  end
  
  fprintf('working for outputs to be written to %s\n', outFileName);
  
  method = Train.PAR.Signal.method ;
  
  if ~exist('check_overlap', 'var'),
    check_overlap=1 ;
  end ;
  
  if( ~isfield(method,'conservWeighting') )
    method.conservWeighting = 0;
  end;
  if( method.conservWeighting == 0 )
    W1 = [];
    meanCons = [];
  else
    W1 = res.W;
    assert( ~isempty(W1) );
    meanCons = mean( W1, 1 );
  end;
  
  %------------------------------------------------------------------------------%
  % init svm and set kernel parameters
  %------------------------------------------------------------------------------%
  if (pred_from_seq && ~isempty(Data.pos)) || (~pred_from_seq && ~isempty(Data.XT))
    init_svm_and_kernels(method,pred_from_seq);
    sg_ver = sg('get_version');
  else
    sg_ver = 'empty_prediction' ;
  end ;
  %------------------------------------------------------------------------------%
  % init training features with labels and examples that only contain those of 
  % support vectors
  %------------------------------------------------------------------------------%

  if (pred_from_seq && ~isempty(Data.pos)) || (~pred_from_seq && ~isempty(Data.XT))
    sg('clean_features', 'TRAIN');
    % D.pos0 = method.center;
    D.XT = Train.XT;
    D.LT = Train.LT;
    
    setFeatures(method, D,'TRAIN',0,0);
  end 
  
  clear D 
  
  %------------------------------------------------------------------------------%
  % init test features with examples if pred_from_seq==0 or with seq and positions
  % in the case of genome wide predictions
  %------------------------------------------------------------------------------%

  if (pred_from_seq && ~isempty(Data.pos)) || (~pred_from_seq && ~isempty(Data.XT))
    sg('clean_features', 'TEST' );
    if ~pred_from_seq
      D.XT = Data.XT;
      if isfield(method, 'feature_type') && isequal(method.feature_type,'double') && method.normalize_features
        D.XT(Train.idx_rm,:)  = [];
        D.XT = D.XT - repmat(Train.mean,size(D.XT,2),1)';
        D.XT = D.XT./repmat(Train.std,size(D.XT,2),1)';
      end
      % D.pos0 = method.center;
      D.pos = Data.Pos;
    else
      D.seq = Data.seq;
      assert(size(Data.pos,1)==1|size(Data.pos,2)==2|isempty(Data.pos))
      if size(Data.pos,1)~=1
        D.pos = Data.pos';
      else
        D.pos = Data.pos;
      end
    end
    
    if ~isempty(D.pos)
      setFeatures(method, D, 'TEST', pred_from_seq, 0);
      D_pos = D.pos ;
      clear D
      %------------------------------------------------------------------------------%
      % setup svm with trainingdata
      %------------------------------------------------------------------------------%
      %Train.XT should contain only support vector examples
      assert(size(Train.alphas, 1)==size(Train.XT,2))
      % alphas are in sparse format col_1=value; col_2=0-based index
      % change index to 0:size(alphas,2)-1 consistent with XTs  
      Train.alphas(:,2)= (0:size(Train.alphas, 1)-1)';
      sg('set_svm', Train.b, Train.alphas);
      
      %------------------------------------------------------------------------------%
      % classify
      %------------------------------------------------------------------------------%
      if( method.use_mkl )
        sg( 'set_subkernel_weights', betas );
      end;
      output = sg( 'svm_classify' );
      
      % work-around for from_position_list bug
      if length(D_pos) == 1 && isequal(class(D_pos), 'double') && length(output)==2,
        output=output(1) ;
      end ;
      clear D_pos
    else
      output = [];
    end
  else
    output = [] ;
  end 
  
  % check for overlaps between training and testing data
  
  if check_overlap && ~isequal(determine_engine, 'octave'),
    
    train_subset = randperm(size(Train.XT, 2)) ;
    train_subset = train_subset(1:min(length(train_subset), 100)) ;
    
    if ~pred_from_seq
      [tmp, idx, idx_] = intersect(Train.XT(:, train_subset)', Data.XT', 'rows') ;
    else
      P=[] ; 
      for i=1:length(train_subset), 
        P = [P strfind(Data.seq, Train.XT(:,train_subset(i))') + method.center] ; 
      end ;
      [tmp,idx,idx_]=intersect(P, Data.pos);
    end ;
    if length(idx)>0,
      warning('Overlap between Train and Test sets discovered (%i/%i sequences)\n', length(idx), length(train_subset)) ;
      if length(idx)/length(train_subset)>0.1,
        warning('Large overlap between Train and Test sets discovered (%i/%i sequences)\n', length(idx), length(train_subset)) ;
      end ;
    end ;
  end 
      
      
      
  %------------------------------------------------------------------------------%
  % save output
  %------------------------------------------------------------------------------%
  
  PAR = Data.PAR;
  PAR.Signal = Train.PAR.Signal;
  
  if isfield(Data, 'LT')
    RSV = calcrocscore(output, Data.LT); 
    if isnan(RSV), 
      RSV=0.5 ;
    end ;
    RFV= calcrfcscore(output, Data.LT);
    if isnan(RFV),
      RFV=0 ;
    end;
    LT = Data.LT;
    %   save( outFileName, 'output','LT','PAR', 'svmFileName','RSV','RFV' );
    POS = Data.Pos;
    fprintf( 'save output to %s\n',outFileName );
    inventory = {'output', 'POS'};
    %save( outFileName, '-V7', 'inventory', 'output','LT','POS', 'Train', 'svmFileName','RSV','RFV','sg_ver','PAR' );
    save( outFileName, '-V7', 'inventory', 'output','LT','POS', 'svmFileName','RSV','RFV','sg_ver','PAR' );
  else
    pos = Data.pos;
    fprintf( 'save output to %s\n',outFileName );
    %save( outFileName, '-V7', 'output', 'pos', 'Train', 'svmFileName','sg_ver','PAR' );
    if isfield(PAR.Signal, 'export_settings') && isfield(PAR.Signal.export_settings, 'resolution') && PAR.Signal.export_settings.resolution>1
      pos_short = zeros(1,floor(length(pos)/2));
      output_short = zeros(1,floor(length(pos)/2));
      % reduce number of predictions (take the better one out of two) 
      for j=2:2:length(pos)
        if output(j-1)>output(j)
          output_short(j/2) = output(j-1);
          pos_short(j/2) = pos(j-1);
        else
          output_short(j/2) = output(j);
          pos_short(j/2) = pos(j);
        end
      end
      output = output_short;
      pos = pos_short;
    end
    
    inventory = {'output', 'pos'};
    assert(isequal(size(pos),size(output)) || (isempty(pos) && isempty(output)));
    save( outFileName, '-V7', 'inventory', 'output', 'pos', 'svmFileName','sg_ver','PAR' );
  end
      
      
return

function init_svm_and_kernels(method,pred_from_seq)
% init_svm_and_kernels(method,pred_from_seq)
  
  
  conservWdsWeighting =0;
  %------------------------------------------------------------------------------%
  % init svm
  %------------------------------------------------------------------------------%
  
  % === cleaning
  sg( 'clean_features', 'TRAIN' );
  sg( 'clean_preproc' );
  sg( 'clean_kernel');
  %clear sg;
  
  %sg( 'loglevel', 'ALL');   % DEBUG
  if ~isfield(method, 'svm_name')
    warning( 'no svm specified, set to default: LIGHT')
    method.svm_name='LIGHT'
  end
  
  % === SVM
  sg( 'new_svm', method.svm_name);
  %sg( 'svm_epsilon', 1e-4 );
  %%%sg( 'svm_epsilon', PAR.svmEps);
  sg('svm_epsilon', 1e-4);
  %%%sg( 'send_command', sprintf( 'use_linadd %d', PAR.useLinadd ) );
  sg('use_linadd', true);
  %%%sg( 'svm_qpsize', PAR.qpSize );
  if ~isfield(method, 'qpsize')
    warning( 'qpsize not specified, set to default: 50')
    method.qpsize=50;
  end
  sg('svm_qpsize', method.qpsize) ; 
  if( conservWdsWeighting > 0 )
    sg('use_batch_computation', false);
  else
    sg('use_batch_computation', true);
  end
  
  % === parallel training?
  if isfield(method,'threads')
    sg('threads', method.threads);
  else
    fprintf('number of threads not specified, set to default: 16')
    sg('threads', 16);
  end;
  
  
  %------------------------------------------------------------------------------%
  % init kernels
  %------------------------------------------------------------------------------%
  
  nofKernels = length(method.kernels);
  if nofKernels>1 && ~isfield(method.kernels{2},'name') && ~isfield(method.kernels{2}, 'kernel_name')
    % for content sensors; several orders are trained simulatuously.
    % but this is done by concatination of the feature vector directly.
    nofKernels = 1;
  end
  
  assert( nofKernels >= 1 );
  if( nofKernels == 1 )
    addKernel = {'set_kernel'};
  else
    addKernel = {'add_kernel', 1};
    sg( 'set_kernel', 'COMBINED', 100);
  end;
  
  for k = 1:nofKernels
    kernel = method.kernels{k};
    % for compatibility with older versions
    if ~isfield(kernel,'kernel_name') && isfield(kernel, 'name')
      kernel.kernel_name = kernel.name;
    end
    fprintf(1, '%s: %s\n', sprintf('%s %1.1f', addKernel{:}), kernel.kernel_name)
    
    switch kernel.kernel_name
     case 'WD'
      if ~isfield(kernel, 'mismatch')
        kernel.mismatch=0;
      end
      assert(length(kernel.order)==1)
      assert(length(kernel.mismatch)==1)
      sg( addKernel{:}, 'WEIGHTEDDEGREE', 'CHAR', 10, kernel.order, kernel.mismatch, true, 1) ;
      
     case 'WDS'
      if ~isfield(kernel, 'mismatch')
        kernel.mismatch=0;
      end
      if ~isfield(kernel, 'shift')
        kernel.shift=0;
      end
      if ~isfield(kernel, 'shift_const')
        kernel.shift_const=0;
      end
      assert(length(kernel.order)==1)
      assert(length(kernel.mismatch)==1)
      len = kernel.rwin-kernel.lwin+1;
      shifts = ceil(kernel.shift * abs( (kernel.lwin:kernel.rwin)+1 ));
      shifts = shifts + kernel.shift_const;
      %shifts = sprintf( '%i ', shifts );
      sg( addKernel{:}, 'WEIGHTEDDEGREEPOS3', 'CHAR', 10, kernel.order, kernel.mismatch, len, 1, int32(shifts)) ;
      clear shifts len
      
     case 'SP'
      use_sign = false;
      % use_sign and prediction from position lists does not work
      %assert(~use_sign|~PAR.tasks.pred_from_seq)
      
      %addKernel_ = {'add_kernel', 0};
      
      sg( addKernel{:}, 'COMMSTRING', 'WORD', 10, use_sign);
      if pred_from_seq
        %sg('loglevel','ALL')
        sg( 'use_diagonal_speedup',true)
      end
     case 'RBF'
      if ~isfield(kernel, 'cache')
        kernel.cache = 40;
      end
      assert(length(kernel.sigma)==1)
      sg( addKernel{:}, 'GAUSSIAN', 'REAL', kernel.cache, kernel.sigma) ;
      
     case 'POLY'
      if ~isfield(kernel, 'cache')
        kernel.cache = 40;
      end
      if ~isfield(kernel, 'type')
        kernel.type = 'REAL';
      end 
      if ~isfield(kernel, 'norm')
        kernel.type = 'R';
      end
      assert(length(kernel.degree)==1)
      sg( addKernel{:}, 'POLY', Kernel.type, kernel.cache, kernel.degree, kernel.norm) ;
      
     case 'LINEAR'
      if ~isfield(kernel, 'cache')
        kernel.cache = 40;
      end
      if ~isfield(kernel, 'type')
        kernel.type = 'REAL';
      end
      sg( addKernel{:}, 'LINEAR', kernel.type, kernel.cache) ;
      
     otherwise
      fprintf(1, 'not yet implemented: kernel_name: %s\n', kernel.kernel_name)
    end;
    
  end;%loop over kernels
  
  if isfield(method, 'subkernel_weights') 
    assert(length(method.subkernel_weights)==nofKernels)
    sg('set_subkernel_weights', method.subkernel_weights);
  end
  
  % for compatibility with older versions
  if ~isfield(method.par_ms,'C') && isfield(method.kernels{1}, 'C')
    method.par_ms.C = method.kernels{1}.C;
  end
  sg('c', method.par_ms.C) ;
  
  disp('done with kernel setup');

return

function setFeatures(method, Data, svmMode, pred_from_seq, use_preproc);
% setFeatures(method,Data, svmMode,pred_from_seq,use_preproc);


% method = PAR.method;
% tasks = PAR.tasks;
% pred_from_seq = tasks.pred_from_seq;

  
%------------------------------------------------------------------------------%
% check
%------------------------------------------------------------------------------%

assert( strcmp(svmMode,'TRAIN') || strcmp(svmMode,'TEST') );

%assert( isscalar(Data.pos0) );
%assert( size(XT,2) > 0 );
%assert( ischar( XT ) );

if nargin<5
  use_preproc=1;
end
if ~isfield(method,'conservWdsWeighting')
  conservWdsWeighting=0;
else
  conservWdsWeighting= method.conservWdsWeighting;
end
%------------------------------------------------------------------------------%
% pass data
%------------------------------------------------------------------------------%

%sg('loglevel', 'ALL');
if pred_from_seq
  seq = Data.seq;
  idx = find(seq~='A'&seq~='C'&seq~='G'&seq~='T');
  if ~isempty(idx)
    warning('non AGCT-Symbols found; changed to As')
    seq(idx)='A';
  end
  pos = Data.pos;
else
  XT = Data.XT;
  %pos0=Data.pos0;
  if isfield(Data, 'LT')
    LT = Data.LT;
  end
  if isfield(Data, 'W')
    W = Data.W;
  end
end

%==============================================================================%
% set labels
%==============================================================================%
if strcmp(svmMode,'TRAIN')
  sg( 'set_labels', 'TRAIN', LT ); 
end
%==============================================================================%
% set features
%==============================================================================%



nofKernels = length(method.kernels);
if nofKernels>1 && ~isfield(method.kernels{2},'name') && ~isfield(method.kernels{2}, 'kernel_name')
  % for content sensors; several orders are trained simulatuously.
  % but this is done by concatination of the feature vector directly.
  nofKernels = 1;
end
assert( nofKernels >= 1 );
if( nofKernels == 1 )
  addFeats = 'set_features';
else
  addFeats = 'add_features';
end;


for k = 1:nofKernels
  kernel = method.kernels{k};
  % for compatibility with older versions
  if ~isfield(kernel, 'kernel_name') && isfield(kernel,'name')
    kernel.kernel_name = kernel.name;
  end
  if ~pred_from_seq   
    if ~isfield(method,'center') || isempty(method.center)
      select= [1:size(XT,1)];
    else
      if ~isfield(kernel,'lwin') 
        warning('no lwin specified for kernel')
        kernel.lwin = -method.center;
      end
      if ~isfield(kernel,'rwin') 
        warning('no rwin specified for kernel')
        kernel.rwin = -method.center+size(XT,1)-1;
      end
      select = (kernel.lwin:kernel.rwin)+method.center+1;
    end
    if ~isequal(select,[1:size(XT,1)])
      XT_temp = XT(select,:);
    else
      XT_temp = XT;
    end
    %keyboard;
  else
    %positiones should be a 0-based index
    pos_temp=pos+kernel.lwin-1;
    winsize_temp=kernel.rwin-kernel.lwin+1;
    assert(length(pos_temp)>0)
    assert(length(seq)>max(pos_temp)+winsize_temp)
    assert(winsize_temp>0)

    % work-around for from_position_list bug
    if length(pos_temp)==1,
      pos_temp = [pos_temp pos_temp] ;
    end ;
  end
  %------------------------------------------------------------------------------%
  % kernel: WD or WDS
  %------------------------------------------------------------------------------%

  if strcmp(kernel.kernel_name,'WD') || strcmp(kernel.kernel_name,'WDS')
    if ~pred_from_seq
      fprintf( '--> XT[%d:%d], num_examples=%i,  order=%d\n', min(select), max(select),size(XT,2), kernel.order );
      sg( addFeats, svmMode, XT_temp, 'DNA' );
    else
      sg( addFeats, svmMode,{seq}, 'DNA' );
      sg('from_position_list','TEST', winsize_temp, int32(pos_temp));
      % NEW
      %sg( addFeats, svmMode, {seq}, 'DNA', 'from_position_list', winsize_temp, int32(pos_temp), 1);
    end
  end;
  

  %------------------------------------------------------------------------------%
  % kernel: Spectrum
  %------------------------------------------------------------------------------%

  if strcmp(kernel.kernel_name,'SP' )
    if~pred_from_seq 
      fprintf( '--> XT[%d:%d], wordlen=%d\n', min(select), max(select), kernel.wordlen );
      sg( addFeats, svmMode, XT_temp, 'DNA');
    
      if( ~isfield(kernel,'gaps') )
        sg( 'convert', svmMode, 'STRING', 'CHAR', 'STRING', 'WORD', kernel.wordlen, kernel.wordlen-1 ) ;
      else
        fprintf('--> convert\n')
        sg( 'convert', svmMode, 'STRING', 'CHAR', 'STRING', 'WORD', kernel.wordlen, kernel.wordlen-1, kernel.gaps );
      end;
      
      if( strcmp(svmMode,'TRAIN') && use_preproc )
        fprintf('--> clean_preproc\n')
        sg( 'clean_preproc' );
        fprintf('--> add_preproc SORTWORDSTRING\n')
        sg( 'add_preproc', 'SORTWORDSTRING' );
        fprintf('--> attach_preproc %s\n', svmMode)
        sg( 'attach_preproc', svmMode );
      end;
    else
      % from_position_list doesnot work for training for spectrum kernel  
      assert(strcmp(svmMode,'TEST')) ;
      sg( addFeats, svmMode, {seq}, 'DNA' );
      % window is cut out from sequence if vonvertion is made after calling 'from_position_list
      % if convertion is done at this place the window size must be reduced by -ordnung-1;
      % (not yet tested) this woud save a lot of time and space. 
      % 
      if( ~isfield(kernel,'gaps') )
        sg( 'convert', svmMode, 'STRING', 'CHAR', 'STRING', 'WORD', kernel.wordlen, 0);
      else
        fprintf('--> convert\n')
        sg( 'convert', svmMode, 'STRING', 'CHAR', 'STRING', 'WORD', kernel.wordlen, 0, kernel.gaps );
      end;
      sg('from_position_list','TEST', winsize_temp- kernel.wordlen+1, int32(pos_temp+kernel.wordlen-1));
    end
  end;
 
  %------------------------------------------------------------------------------%
  % kernel:  RBF 
  %------------------------------------------------------------------------------%
 
  if strcmp(kernel.kernel_name,'RBF')
    fprintf( '--> XT[%d:%d], num_examples=%i,  sigma=%d\n', min(select), max(select),size(XT,2), kernel.sigma );
    sg( addFeats, svmMode, XT_temp);    
  end;
  
  %------------------------------------------------------------------------------%
  % kernel:  POLYNOMIAL
  %------------------------------------------------------------------------------%
 
  if strcmp(kernel.kernel_name,'POLY')
    fprintf( '--> XT[%d:%d], num_examples=%i,  degree=%d\n', min(select), max(select),size(XT,2), kernel.degree );
    sg( addFeats, svmMode, XT_temp);    
  end;
  
  %------------------------------------------------------------------------------%
  % kernel:  LINEAR
  %------------------------------------------------------------------------------%
  
  if strcmp(kernel.kernel_name,'LINEAR')
    fprintf( '--> XT[%d:%d], num_examples=%i,\n', min(select), max(select),size(XT,2) );
    sg( addFeats, svmMode, XT_temp);    
  end;
end%loop over kernels
%------------------------------------------------------------------------------%
% done
%------------------------------------------------------------------------------%

if( use_preproc & strcmp(svmMode,'TEST')  & ~pred_from_seq )
   fprintf('--> attach_preproc %s', svmMode)
  sg( 'attach_preproc', svmMode );
end;

clear XT;
sg( 'init_kernel', svmMode );

% --- set feature weights
if( conservWdsWeighting > 0 )
  W1 = W( select1, : );
  if( conservWdsWeightNorming )
    fprintf( '--> conserv(WDS): using normalized feature weights!\n' );
    meanW = mean( W1, 1 );
    W1 = W1 .* repmat( 1./meanW, size(W1,1), 1 );
  else
    fprintf( '--> conserv(WDS): using raw feature weights!\n' );
    W1 = 3/size(W1,1) * W1;
  end;
  sg( 'set_subkernel_weights_combined', W1, 1 );
  clear W1;
end;



return

function [trivial_regions,blocks,training,test] = prepare_regions(PAR);
%  [trivial_regions,training_split,training_blocks,test_split,test_blocks] = prepare_regions(PAR,genes);


% generate trivial regions
if fexist(PAR.FN.output.fn_trivial_regions)
  fprintf(1,'loading trivial regions from file %s\n',PAR.FN.output.fn_trivial_regions)
  load(PAR.FN.output.fn_trivial_regions,'trivial_regions');
  assert(isequal(trivial_regions(1).config,PAR.FN.input.fn_genome_config));
else
  fprintf(1,'initializing trivial regions\n')
  trivial_regions = init_regions(PAR);
  save(PAR.FN.output.fn_trivial_regions,'trivial_regions');
end

% generate training and test blocks and splits, 
% considering training and test region
if fexist(PAR.FN.output.fn_training_blocks)& fexist(PAR.FN.output.fn_test_blocks)
  Old = load(PAR.FN.output.fn_training_blocks, 'PAR');
  assert(isequal(Old.PAR.FN.output.fn_trivial_regions,PAR.FN.output.fn_trivial_regions));
  assert(isequal(Old.PAR.FN.input.fn_genome_config,PAR.FN.input.fn_genome_config));
  diff_fields = compare_PAR(PAR,Old.PAR) ;
  if ismember('regions',diff_fields)
    warning('regions exist, Parameters are different ')
    %fprintf('regions exist, Parameters are different ')
  else
    fprintf(1,'loading split and blocks from file %s \n',PAR.FN.output.fn_training_blocks)
    training = load(PAR.FN.output.fn_training_blocks)
    assert(length(fieldnames(training.split))==PAR.SETs.num_splits)
  end
  Old = load(PAR.FN.output.fn_test_blocks, 'PAR');
  assert(isequal(Old.PAR.FN.output.fn_trivial_regions,PAR.FN.output.fn_trivial_regions));
  assert(isequal(Old.PAR.FN.input.fn_genome_config,PAR.FN.input.fn_genome_config));
  diff_fields = compare_PAR(PAR,Old.PAR) ;
  if ismember('regions',diff_fields)
    warning('regions exist, Parameters are different ')
    %fprintf('regions exist, Parameters are different ')
  else
    fprintf(1,'loading split and blocks from file %s \n',PAR.FN.output.fn_test_blocks)
    test = load(PAR.FN.output.fn_test_blocks)
    assert(length(fieldnames(test.split))==PAR.SETs.num_splits)
  end
else
  if isfield(PAR.FN.output,'fn_genes_merged') && fexist(PAR.FN.output.fn_genes_merged)
    load(PAR.FN.output.fn_genes_merged,'genes');
  else
    genes=[];
  end
  [training, test] = assign_subsets(trivial_regions,PAR.FN.input.fn_trainings_regions,PAR.FN.input.fn_test_regions,genes,PAR.SETs,PAR.regions,PAR.tasks)
end

if 1
  if isfield(PAR.FN.output,'fn_genes_merged') && fexist(PAR.FN.output.fn_genes_merged)
    load(PAR.FN.output.fn_genes_merged,'genes');
  else
    genes=[];
  end
  load(PAR.FN.input.fn_trainings_regions,'training_regions');
  load(PAR.FN.input.fn_test_regions,'test_regions');
  load(PAR.FN.output.fn_trivial_regions,'trivial_regions');
  if  ~exist('training')
    training = load(PAR.FN.output.fn_training_blocks);
  end
  if  ~exist('test')
    test = load(PAR.FN.output.fn_test_blocks);
  end
  % load(PAR.FN.output.fn_blocks,'blocks');
  blocks =test.blocks;
  check_and_plot_splits(trivial_regions(1:min(10,length(trivial_regions))),training_regions,test_regions,genes,blocks,training, test)
end
  

return

function [dummy1, dummy2]=gen_all_positions_rproc(P)

% gen_all_positions_rproc(P)
% P has fields:  
%   - region;  
%   - blocks 
%   - Signal 
%   - fn_pos 
%   - num_splits 
%   
  

dummy1=[] ;
dummy2=[] ;
  
% use the rproc.addpath way to add paths in client processes!!
%
%paths  

region = P.region;  
blocks = P.blocks;
Signal = P.Signal;
fn_pos = P.fn_pos;
num_splits =  P.num_splits;


fprintf('start loading sequence for region %i%s\r',region.chr_num,region.strand)
tt = load_sequence(region);
if ~isempty(Signal.consensus)
  pos = find_consensus(tt.seq,Signal,region.strand);
elseif region.strand=='+'
  pos = Signal.lwin_big+1:length(tt.seq)-Signal.rwin_big; 
elseif region.strand=='-'
  pos = Signal.rwin_big+1:length(tt.seq)-Signal.lwin_big; 
end    

clear tt;

%-------------
% Determine SVM
%-------------
svm_num = zeros(size(pos));

% faster if there are many blocks and few splits
% needs more memory
if ~isempty(blocks),
  splits=[blocks.split] ; 
else
  splits=[] ;
end ;
assert(length(splits)==length(blocks)) ;
for split = unique(splits),
  fprintf('split %i\r',split)
  
  split_idx = find(split==splits) ;

  split_pos = {} ; p=length(split_idx); 
  for b_idx=split_idx,
    assert(~isfield(blocks, 'num_fragments') || blocks(b_idx).num_fragments<=1) ;
    if region.strand==blocks(b_idx).strand(1) && region.chr_num==blocks(b_idx).chr_num(1),
      if isfield(blocks(b_idx),'reg_no_overlap') && ~isempty(blocks(b_idx).reg_no_overlap)
        split_pos{p} = blocks(b_idx).start+[blocks(b_idx).reg_no_overlap(1):blocks(b_idx).reg_no_overlap(2)];
      else
        split_pos{p} = blocks(b_idx).start:blocks(b_idx).stop ;
      end
    end ;
    p=p-1 ;
  end ;
  split_pos = [split_pos{:}] ;
  [tmp,idx]=intersect(pos, split_pos); 
  assert(all(svm_num(idx)==0 | svm_num(idx)==split))
  svm_num(idx) = split ;
end ;

if 0, % slow version
  for b_idx = 1:length(blocks)
    fprintf('block %i\r',b_idx)
    block = blocks(b_idx);
    if ~isfield(block, 'num_fragments'),
      block.num_fragments = 1 ;
    end ;
    if ~isfield(block, 'reg_no_overlap'),
      block.reg_no_overlap = [block.start block.stop] ;
    end ;
    for f_idx = 1:block.num_fragments
      if f_idx>1
        error('block.reg_no_overlap not implemented for more than one fragment')
      end
      if region.strand==block.strand(f_idx) && region.chr_num==block.chr_num(f_idx);
        % idx =
        % find(pos>=block.start(f_idx)+PAR.regions.offset&pos<=block.stop(f_idx)-PAR.regions.offset);
        
        idx = find(pos>=block.reg_no_overlap(1) & pos<=block.reg_no_overlap(2));
        assert(all(svm_num(idx)==0 | svm_num(idx)==block.split))
        svm_num(idx) = block.split;
        % keyboard
      end
    end    
  end
end ;

filename = sprintf('%scontig_%i%s',fn_pos,region.chr_num,region.strand)
if isequal(filename(1),'~')
  filename = sprintf('%s/%s', getenv('HOME'), filename(2:end));
end

if ~isempty(find(svm_num==0))

  idx = find(svm_num==0);
  % nums = repmat([1:num_splits],1,ceil(length(idx)/num_splits));
  % nums = nums(randperm(length(nums)));
  % make blocks instead (faster computing for contents)

  num_blocks = round(max(length(idx)/10000, num_splits));
  nums = zeros(1, num_blocks*ceil(length(idx)/num_blocks)) ;
  p=1 ;
  for i=1:num_blocks
    nums( p : p+ceil(length(idx)/num_blocks)-1 ) = repmat( mod(num_blocks,num_splits) + 1, 1, ceil(length(idx)/num_blocks)) ;
    p = p + ceil( length(idx)/num_blocks ) ;
  end
  assert(p-1==length(nums));
  svm_num(idx) = nums(1:length(idx));

end

if ~isempty(pos)
  save_score_pos(pos,svm_num, filename ,{'svm'});
else
  unix(sprintf('touch %s.pos',filename));
  unix(sprintf('touch %s.svm',filename));
  warning('no position found')
end

return

function [dummy1, dummy2]=Out2Confhelper(PAR)
% Out2Confhelper(PAR)
try
  pause('on')
  warning('off', 'MATLAB:typeaheadBufferOverflow');  
catch
  % works only for matlab
end

dummy1=[] ;
dummy2=[] ;

try
  load(PAR.fn_pred,'output') ;
catch
  pause(60)
  load(PAR.fn_pred,'output') ;
end
disp('Convert SVM Output to Confidence on fn_pred')
[Conf, Conf_cum] = Out2Conf(output, PAR.prob, PAR.prob_cum, PAR.limits);

if ~(all(Conf>=0-1e-8&Conf<=1+1e-8))
  save('~/tmp/ws_Out2Confhelper_error');
  error('Conf out of range [0, 1]');
end
if ~(all(Conf_cum>=0-1e-8&Conf_cum<=1+1e-8))
  save('~/tmp/ws_Out2Confhelper_error');
  error('Conf_cum out of range [0, 1]');
end

fprintf('append Conf and Conf_cum to file %s ', PAR.fn_pred)
%save(PAR.fn_pred,'-append','Conf','Conf_cum');
inventory = {'output', 'pos', 'Conf', 'Conf_cum'};
save_append(PAR.fn_pred, 1, 'inventory', inventory,'Conf', Conf, 'Conf_cum', Conf_cum);

return

function [dummy1, dummy2]=concat_genomewide_predictons_helper(P)
% concat_genomewide_predictons_helper(P)

try
  pause('on')
  warning('off', 'MATLAB:typeaheadBufferOverflow');
catch
  % works only for matlab
end


dummy1=[] ;
dummy2=[] ;

fn_pred_all = P.fn_pred_all ;
fn_pred_split = P.fn_pred_split;
num_splits = P.num_splits;
avg_all_svms = P.avg_all_svms;
subtract_diff = P.subtract_diff;

% for H_sapiens_nohomologs:
% prefix = '/fml/ag-raetsch/share/projects/genefinding/H_sapiens_nohomologs/sensors/don/output/pred';
% fn_pred_all = sprintf('%scontig_%i%s_all.mat',prefix,c,s);

contig_pos = [];
contig_out = [];
contig_conf= [];
contig_cnfc= [];
contig_cnt=[] ;
contig_svm = [];
for p=1:num_splits
  fn_pred = sprintf('%s%i.mat', fn_pred_split, p);
  % for H_sapiens_nohomologs:
  % prefix = '/fml/ag-raetsch/share/projects/genefinding/H_sapiens_nohomologs/sensors/don/output/pred';
  % fn_pred = sprintf('%scontig_%i%s%s_%i.mat',prefix,c,s,PAR.Signal_name,p);

  for kk=1:10
    try
      clear pos output Conf Conf_cum
      load(fn_pred, 'inventory');
      %inventory
      for j=1:length(inventory)
        if kk>1,
          fprintf('trying to load variable "%s" from %s (trial %i)\n', inventory{j}, fn_pred, kk);
        end ;
        load(fn_pred,inventory{j})
        assert(exist(inventory{j},'var')==1);	 
      end
      break ;
    catch
      if kk==10      
        fprintf('giving up waiting for file %s\n', fn_pred)
      else
        fprintf('waiting for file: %s \n', fn_pred);    
        pause(30)
      end	
    end
  end

  if ~avg_all_svms
    if ~isempty(intersect(contig_pos, pos))
      warning('positions found in multiple splits');
      %% hack
      [tmp,idx1,idx2] = intersect(contig_pos, pos);
      pos(idx2)=[];
      output(idx2)=[];
      if exist('Conf') && exist('Conf_cum')
        Conf(idx2)=[];
        Conf_cum(idx2)=[];
      %else
      %  warning('no confs found')
      end
      %%hack
    end
    % assert(isempty(intersect(contig_pos, pos)));
    contig_pos = [contig_pos pos];
    contig_out = [contig_out output];
    contig_svm = [contig_svm repmat(p,size(pos))];
    if exist('Conf') && exist('Conf_cum')
      contig_conf = [contig_conf Conf];
      contig_cnfc = [contig_cnfc Conf_cum];
    end
    
  else
    [tmp,idx1,idx2] = intersect(contig_pos, pos);
    assert(length(idx1)==length(contig_pos))
    if isempty(contig_pos)
      contig_pos = pos;
      contig_out = output;
      if exist('Conf') && exist('Conf_cum')
        contig_conf = Conf(idx2);
        contig_cnfc = Conf_cum(idx2) ;
        contig_cnt = 1 ;
      else
        clear contig_conf contig_cnfc contig_cnt ;
      end
    else
      % otherwise the following will be wrong
      assert(isequal(sort(contig_pos), contig_pos)) ;
      assert(isequal(idx1, 1:length(idx1))) ;

      contig_out(idx1) = contig_out(idx1)+output(idx2);
      if exist('Conf') && exist('Conf_cum')
        contig_conf(idx1) = contig_conf(idx1) + Conf(idx2) ;
        contig_cnfc(idx1) = contig_cnfc(idx1) + Conf_cum(idx2) ;
        contig_cnt(idx1) = contig_cnt(idx1) + 1 ;
      else
        clear contig_conf contig_cnfc contig_cnt;
      end
    end
  end
  clear output pos Conf Conf_cum
end 
[pos sort_idx] = sort(contig_pos);
output = contig_out(sort_idx);
svm = contig_svm(sort_idx);

if avg_all_svms
  output = output./contig_cnt ;
end

% for contents: make sure there are smooth transitions at split boundaries
if subtract_diff
  ii = find(svm(1:end-1)~=svm(2:end));
  fprintf('number of blocks: %i\n',length(ii));
  for j=ii
    output(j+1:end) = output(j+1:end) - (output(j+1)-output(j));
  end
end

if exist('contig_conf') && exist('contig_cnfc') && isequal(size(contig_conf), size(contig_pos)) && isequal(size(contig_cnfc), size(contig_pos))
  if avg_all_svms
    contig_conf = contig_conf./(contig_cnt+1e-16) ;
    contig_cnfc = contig_cnfc./(contig_cnt+1e-16) ;
  end ;
  Conf = contig_conf(sort_idx);
  Conf_cum = contig_cnfc(sort_idx);
  
  assert(all(Conf>=0-1e-8&Conf<=1+1e-8));
  assert(all(Conf_cum>=0-1e-8&Conf_cum<=1+1e-8));
  fprintf('saving output, pos, Conf and Conf_cum\n')
  fprintf('save: %s\n',fn_pred_all)
  inventory = {'output', 'pos', 'Conf', 'Conf_cum'};
  save(fn_pred_all, '-V7','inventory', 'output', 'pos', 'Conf', 'Conf_cum')
else
  inventory = {'output', 'pos'};
  fprintf('saving output and pos without Conf and Conf_cum\n')
  fprintf('save: %s\n',fn_pred_all)
  save(fn_pred_all, '-V7','inventory', 'output', 'pos')
end

return

function [dummy1, dummy2]=save_predictions_helper(P)
% save_predictions_helper(P) 
try
  pause('on')
  warning('off', 'MATLAB:typeaheadBufferOverflow');  
catch
  % works only for matlab
end


dummy1=[] ;
dummy2=[] ;

fn_genome_config = P.fn_genome_config;
chrom = P.chrom;
strand = P.strand;
fn_pred = P.fn_pred;
Signal_name = P.Signal_name;
resolution = P.resolution;
conf_cum_thresh = P.conf_cum_thresh;
do_gzip = P.do_gzip ;
save_as = P.save_as ;

genome_info = init_genome(fn_genome_config); 
S = dir(genome_info.flat_fnames{chrom}) ;
contig_length = S.bytes;
CHR_NAME = genome_info.contig_names{chrom};
fprintf('contig: %i%s\n', chrom,strand);

%----------------------------
%% GET ALL PREDICTIONS from mat file
%----------------------------

filename = sprintf('%scontig_%i%s_all.mat', fn_pred, chrom, strand) ;
for kk=1:30 % it can take easily several minutes until the file is visible
   try 
     clear pos output Conf Conf_cum
     load(filename, 'inventory');
     for j=1:length(inventory)
        if kk>1,
           fprintf('trying to load variable "%s" from %s (trial %i)\n', inventory{j}, filename, kk);
        end ;
        load(filename,inventory{j})
        assert(exist(inventory{j}, 'var')==1);	 
     end
     break ;
   catch
     if kk==30      
       fprintf('giving up waiting for file %s\n', filename)
     else
       fprintf('waiting for file: %s \n', filename);    
       pause(30)
     end	
  end
end


% always save global coordinates on forward strand
if isequal(strand, '-'),
  pos = contig_length - pos + 1;
end
assert(all(pos>=1));

[pos sort_idx] = sort(pos);
output = output(sort_idx);
if exist('Conf') && exist('Conf_cum')
  Conf = Conf(sort_idx);
  Conf_cum = Conf_cum(sort_idx);
  
  %----------------------------
  %% downsample if demanded
  %----------------------------
  
  assert(all(Conf>=0-1e-8&Conf<=1+1e-8));
  Conf(Conf<0)=0 ;
  Conf(Conf>1)=1 ;
end

side = floor(resolution/2) ;
if (resolution~=1), 
  fprintf('Reducing resolution to one prediction per %i nucleotides\n', resolution) ;
  idx = 1:resolution:length(pos) ;
  cc = zeros(1,length(idx)) ;
  co = zeros(1,length(idx)) ;
  oo = zeros(1,length(idx)) ;
  pp = zeros(1,length(idx)) ;
  
  for j=1:length(idx),
    %if mod(j,1000)==0, fprintf(' %i  \r', j), end ;
    win = max(1,idx(j)-side):min(idx(j)+side,length(pos)) ;
    neighbour = find((pos(win)>=pos(idx(j))-side) & (pos(win)<pos(idx(j))+side)) ;
    %pos(win(neighbour))
    if exist('Conf') && exist('Conf_cum')
      [mv,mi] = max(Conf_cum(win(neighbour))) ;
      assert(Conf_cum(win(neighbour(mi)))==mv) ;
      cc(j) = Conf_cum(win(neighbour(mi))) ;
      co(j) = Conf(win(neighbour(mi))) ;
    else
      [mv,mi] = max(output(win(neighbour))) ;
    end
    pp(j) = pos(win(neighbour(mi))) ;
    oo(j) = output(win(neighbour(mi))) ;
  end;
  pos = pp ;
  Conf_cum = cc ;
  Conf = co ;
  output = oo ;
end ;

%----------------------------
%%check if files exist
%----------------------------
filename = sprintf('%scontig_%i%s', fn_pred, chrom, strand);
if fexist([filename '.pos']),
  fprintf('removing position file (%s). \n', filename);
  unix(sprintf('rm %s', [filename '.pos']));
end
if fexist([filename '.output']),
  fprintf('removing output file (%s)\n', filename);
  unix(sprintf('rm %s', [filename '.output']));
end
if fexist([filename '.Conf']),
  fprintf('removing Conf file (%s)\n', filename);
  unix(sprintf('rm %s', [filename '.Conf']));
end
if fexist([filename '.Conf_cum']),
  fprintf('removing Conf_cum file (%s)\n', filename);
  unix(sprintf('rm %s', [filename '.Conf_cum']));
end

%Conf=round(Conf*1000)/1000;
%output = round(output*1000)/1000;
%Conf_cum=round(Conf_cum*1000)/1000; 
if exist('Conf') && exist('Conf_cum') %&& ~all(Conf_cum==0)&&~all(Conf==0)
  save_score_pos(pos, [output; Conf; Conf_cum], filename, {'output', 'Conf', 'Conf_cum'});
else
  save_score_pos(pos, [output], filename, {'output'});
end

if isequal(Signal_name, 'transacc'),
  disp('Using acceptor predictions in combination with transacc predictions (for wiggle files only)') ;
  FN2.fn_pred = strrep(fn_pred,'transacc', 'acc') ;
  
  filename2 = sprintf('%scontig_%i%s_all.mat', FN2.fn_pred,chrom,strand);
  L=load(filename2, 'pos', 'Conf', 'Conf_cum', 'output');
  % always save global coordinates on forward strand
  if isequal(strand, '-'),
    L.pos = contig_length - L.pos + 1;
  end
  assert(all(pos>=1));
  if ~isequal(sort(L.pos),pos)
    warning('position lists not identical') ;
    setdiff(L.pos, pos)
    setdiff(pos, L.pos)
    
    [tmp,idx1,idx2]=intersect(pos,L.pos) ;
    
    L.Conf = L.Conf(idx2) ;
    L.Conf_cum = L.Conf_cum(idx2) ;
    L.output = L.output(idx2) ;
    L.pos = L.pos(idx2) ;
    
    Conf = Conf(idx1) ;
    Conf_cum = Conf_cum(idx1) ;
    output = output(idx1) ;
    pos = pos(idx1) ;
  else
    [L.pos sort_idx] = sort(L.pos);
    L.Conf = L.Conf(sort_idx);
    L.Conf_cum = L.Conf_cum(sort_idx);
    L.output = L.output(sort_idx);
  end ;
  
  Conf_cum = Conf_cum .* L.Conf_cum ;
  Conf = Conf .* L.Conf ;
  output = output + L.output ;
  clear L ;
end ;

%----------------------------
%% SAVE PREDICTIONS as SPF files
%---------------------------- 

% saving in signal prediction format (without filtering)
if save_as.spf_ascii,
  save_predictions_as_sigpred(filename, pos, output, 'output', Signal_name, CHR_NAME, strand, do_gzip);
end ;
if save_as.spf_binary,
  save_predictions_as_sigpred_bin(filename, pos, output, 'output', Signal_name, CHR_NAME, strand);
end ;

if exist('Conf') %&& ~all(Conf==0)
  if save_as.spf_ascii,
    save_predictions_as_sigpred(filename, pos, Conf, 'Conf', Signal_name, CHR_NAME, strand, do_gzip);
  end ;
  if save_as.spf_binary,
    save_predictions_as_sigpred_bin(filename, pos, Conf, 'Conf', Signal_name, CHR_NAME, strand);
  end ;
end ;

if exist('Conf_cum') %&&  ~all(Conf_cum==0)
  if save_as.spf_ascii,
    save_predictions_as_sigpred(filename, pos, Conf_cum, 'Conf_cum', Signal_name, CHR_NAME, strand, do_gzip);
  end ;
  if save_as.spf_binary,
    save_predictions_as_sigpred_bin(filename, pos, Conf_cum, 'Conf_cum', Signal_name, CHR_NAME, strand);
  end
end ;

%----------------------------
%% SAVE PREDICTIONS as wiggle files
%---------------------------- 

% rounding for wiggle format output
output = round(output*1000)/1000;

%----------------------------
% REMOVE PREDICTIOND BELOW THRESHOLD
%----------------------------


if exist('Conf') && exist('Conf_cum') && save_as.wiggle

  fprintf('Removing all predictions below %.2f before saving wiggle file',conf_cum_thresh) 
  idx = find(Conf > conf_cum_thresh);
  output   = output(idx);
  pos = pos(idx);
  Conf = round(Conf*1000)/1000;
  Conf_cum = round(Conf_cum*1000)/1000; 
  Conf     = Conf(idx); 
  Conf_cum = Conf_cum(idx);
  if save_as.wiggle
    save_predictions_as_wiggle(filename, pos, Conf, 'Conf', Signal_name, CHR_NAME, strand, do_gzip);
    save_predictions_as_wiggle(filename, pos, Conf_cum, 'Conf_cum', Signal_name, CHR_NAME, strand, do_gzip);
  end ;

end

if save_as.wiggle
  save_predictions_as_wiggle(filename, pos, output, 'output', Signal_name, CHR_NAME, strand, do_gzip);
end

return

function jobinfo = gen_training_examples4signal(PAR)
% gen_labels_and_train(PAR)

%paths
  

Signal_name = PAR.Signal_name;
Signal = PAR.Signals.(PAR.Signal_name);
num_splits = PAR.SETs.num_splits;
label_source  = PAR.label_source.(Signal_name);
info = PAR.info_genes.(PAR.Signal_name);


FN = PAR.FN;
fn_genome_config = FN.input.fn_genome_config ;
fn_training_blocks = FN.output.fn_training_blocks;
fn_genes_signals = FN.output.fn_genes_signals ;

input_sig = FN.input_sig.(Signal_name);
fn_candsites =  input_sig.fn_candsites;
fn_filter_settings = input_sig.fn_filter_settings;
fn_examples = input_sig.fn_examples;
fn_example_statistics = input_sig.fn_example_statistics;

run_locally = PAR.tasks.signals.run_locally.pos ;

if ~run_locally
  RPROC = PAR.RPROC;
end



jobinfo = [] ;
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% load  trivial_regions, blocks, splits
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

trivial_regions = init_regions(fn_genome_config);


if ~fexist(fn_training_blocks)
  error('training block file does not exist; create it first with prepare_regions')
else
  blocks = load_struct(fn_training_blocks, 'blocks') ;
  load(fn_training_blocks, 'split') ;
  assert(length(fieldnames(split))==num_splits);
end


generate_examples=1;
if fexist([fn_candsites 'PAR.mat'])
  fprintf(1,'files with candidate pos already exists\n')
  Old = load([fn_candsites 'PAR.mat'],'P');
  diff_fields = compare_PAR(Signal,Old.P.Signal) ;
  if ismember('Signal',Signal_name)
    warning('label file exists, Signal parameters are differnt!')  
    % fprintf('label file exists, Signal parameters are differnt!')  
  end
  %[temp, num_files] = unix(sprintf('ls %scontig_*.pos | wc -l',fn_candsites));
  d=dir(sprintf('%scontig_*.pos',fn_candsites));
  num_files = length(d) ;

  if (num_files==length(trivial_regions))
    generate_examples=0;
    fprintf(1,'all cand files found; no label generation\n')
  else
    fprintf(1,'%i/%i cand files missing; start label generation\n',length(trivial_regions)-num_files,length(trivial_regions))
  end
  clear Old
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% generating .pos .label files
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if generate_examples
  fprintf(1,'generating labels\n')
  jobinfo = rproc_empty(0) ;
  
  P.fn_genes_signals = fn_genes_signals ;
  P.fn_candsites = fn_candsites;
  P.label_source = label_source;
  P.Signal = Signal; 
  P.info = info;
  
  D.P = P ;
  for j=1:length(trivial_regions)
    D.t_region = trivial_regions(j);
    %keyboard
    filename = sprintf('%scontig_%i%s',fn_candsites,trivial_regions(j).chr_num,trivial_regions(j).strand);
    if fexist(filename)
      continue
    end
    if run_locally
      save_label_files_starter(D);
    else
      [mem_req, time_req, opts] = rproc_mem_policy('save_label_files_starter',  0, RPROC.options) ;%RPROC.MEMREQ ;

      jobinfo(j) = rproc('save_label_files_starter', D, mem_req, opts, time_req) ;
      % save_label_files_starter(D)
    end;
    % save_label_files(t_region,PAR,genes)
  end
  if ~isempty(jobinfo),
    [jobinfo_1, num_crashed]=rproc_wait(jobinfo, 10, 1, -1);
  else
    num_crashed = 0 ;
    jobinfo_1 = rproc_empty(0) ;
  end ;
  %[temp, num_files] = unix(sprintf('ls %scontig_*.pos | wc -l',fn_candsites));
  d=dir(sprintf('%scontig_*.pos',fn_candsites));
  num_files = length(d) ;
  assert(num_files==length(trivial_regions))
  if num_crashed>0, 
    num_crashed
    %return
  end 
  if ~isempty(jobinfo_1)
    rproc_cleanup(jobinfo_1) ;
  end ;
end
clear P

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% generating example files
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

modes = {'train','test'};
for mode=modes
  [filename, exists]=gen_example_filename(Signal,fn_filter_settings,fn_examples,mode{1},num_splits);
  if ~exists
   
    fprintf(1, 'SAVE EXAMPLES ....');
    
    P.Signal = Signal;
    P.num_splits = num_splits;
    P.fn_training_blocks = fn_training_blocks;
    P.fn_candsites = fn_candsites;
    P.fn_filter_settings = fn_filter_settings ;
    P.fn_examples = fn_examples;
    P.fn_example_statistics = fn_example_statistics;
    if 0% ~run_locally
      D.blocks = blocks;
      D.split = split;
      D.P = P;

      [mem_req, time_req, opts] = rproc_mem_policy('save_examples',  0, RPROC.options) ;%RPROC.MEMREQ ;

      jobinfo = rproc('save_examples', D, mem_req, opts, time_req);
      [jobinfo_1, num_crashed]=rproc_wait(jobinfo, 60, 1, -1);
    else
      save_examples(blocks,split,P) ; num_crashed = 0 ;
    end
    if num_crashed>0
      return
    end
  end
end


return

function new_field = set_default(old_struct,field,defaults,silent)  ;

% new_struct = set_default(old_struct,defaults,overwrite)  ;
% sets all values of all fields in old_struct to the default values specified in
% defaults IF field is empty in old_struct. If overwrite is set all entries
% will be overwritten.
% defaults must be a struct.
%
% e.g. 
% default.use_cons = 0;
% default.plif.bins = 50;
% method.use_cons = 0;
% method = set_default(method,default,1) 

if ~isfield(old_struct,field)
  new_field = defaults;
  return
end  

old_struct = old_struct.(field);

if nargin<4
  silent = 1 ;
end

names = fieldnames(defaults);
for j=1:length(names)
  if ~isfield(old_struct,names{j}) || isempty(old_struct.(names{j}))||...
        (iscell(old_struct.(names{j}))&&length(old_struct.(names{j}))==1&&isempty(old_struct.(names{j}){1}))
    if ~isfield(old_struct,names{j}) && ~silent
      warning('new field added')
      names{j}
    end
    
    old_struct.(names{j}) = defaults.(names{j});
  elseif isstruct(defaults.(names{j}))
    old_struct.(names{j}) = set_default(old_struct,names{j},defaults.(names{j}));
    %names2 =  fieldnames(defaults.(names{j}));
    %for k= 1:length(names2)
    %  if ~isfield(old_struct.(names{j}),names2{k})||isempty(old_struct.(names{j}).(names2{k}))||overwrite
    %    old_struct.(names{j}).(names2{k}) = defaults.(names{j}).(names2{k});
    %  end
    %end
  end
end
% if length(fieldnames(PAR))>length(names)
%   keyboard
% end
% PAR = orderfields(PAR,defaults);

new_field = orderfields(old_struct,defaults);

return

function method = SVM_WDS(Signal)

% Signals = SVM_WDS(Signals,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. In this function only
%          the field (signal_name) is modified.
%          Particuarly a field method is added.
%        - signal_name can be something in {'acc','don','tss','cleave','polya','tis','cdsStop'};
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: SVM_WDS -> SVM -> method_template

  
method = SVM(Signal) ;
method.feature_type = 'char';


kernel.name = 'WDS';
kernel.order = Signal.order;
kernel.shift = Signal.shift;
kernel.lwin =  Signal.lwin{1};
kernel.rwin = Signal.rwin{1};

method.kernels{1} = kernel; 

return

function method = SVM_COMBINED(Signal)

% Signals = SVM_COMBINED(Signals,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. In this function only
%          the field (signal_name) is modified.
%          Particuarly a field method is added.
%        - signal_name can be something in {'acc','don','tss','cleave','polya','tis','cdsStop'};
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: SVM_COMBINMED -> SVM -> method_template


method = SVM(Signal) ;

nofKernels = length(Signal.kernel_name);

assert(length(Signal.lwin)==nofKernels)
assert(length(Signal.rwin)==nofKernels)


for k=1:nofKernels
  for n = Signal.kernel_par_names 
    method.kernels{k}.(n{1}) = Signal.(n{1}){k};
  end
end
if isfield(Signal,'subkernel_weights')
  method.subkernel_weights = Signal.subkernel_weights;
end


return

function Content = intergenic

name = 'intergenic';

Content = content_template(name);

Content.wordlen = {[3],[4],[5]};
Content.stepping = 1; 
Content.offset = 0;
return

function Content=content_template(name)

Content.name = name;
Content.consensus = [];
Content.lwin_big = 0;
Content.rwin_big = 0;
%Content.order = [3 4 5];
Content.label_fct = 'get_cand_contents';
Content.load_fct = 'load_content_data';

Content.filter_label.train.USE_ALL = 1;
Content.filter_label.test.USE_ALL = 1;
Content.filter_label.eval.USE_ALL = 1;
Content.Conf_names = {} ;
Content.C = [1]; 

Content.wordlen = { };
Content.stepping = 1;
Content.offset = 0;

Content.export_settings.conf_cum_thresh = 0.5 ;
Content.export_settings.resolution = 1 ;


%PAR.Contents.type_descrs = { 'intergenic', 'utr_exon', 'cds_exon','intron', 'intercistronic' };
%PAR.Contents.type_abbrs = { 'ige' 'utr' 'cex' 'ino' 'ics' };
%PAR.Contents.num_types = length(PAR.Contents.type_descrs); 



return

function Content=utr5exon

name = 'utr5exon';

Content = content_template(name);

Content.wordlen = {[3],[4],[5],[6]};
Content.stepping = 1; 
Content.offset = 0;
return

function Content = cds_exon

name = 'cds_exon';

Content = content_template(name);

Content.wordlen = {[3],[4],[5]};
Content.stepping = 1; 
Content.offset = 0;
return

function Content = frame0(info)

name = 'frame0';

Content = content_template(name);
Content.wordlen = {[3],[4],[5],[6]};
Content.stepping = 3;

Content.offset = mod(3-mod(Content.wordlen{1},3),3);
 

return

function Content = utr3exon

name = 'utr3exon';

Content = content_template(name);

Content.wordlen = {[3],[4],[5]};
Content.stepping = 1; 
Content.offset = 0;
return

function Content=intron

name = 'intron';

Content = content_template(name);

Content.wordlen = {[3],[4],[5]};
Content.stepping = 1; 
Content.offset = 0;
return

function LSL = lsl_template
  

method.name = 'lsl' ;
method.model_fct = 'make_model';
method.train_fct = 'lsl_train';
method.predict_fct = 'lsl_predict';

method.dt = datestr(now,'dd-mm-yyyy');
method.user = 'GS';
method.plif_bins   = 20 ;   % number of plif bins

%%%

%%Signals
use.signals.trans = 0 ;
use.signals.transacc = 0 ;
use.signals.tss = 1;
use.signals.polya = 1;
use.signals.cleave =  1;

%%States
use.states.SL1 = 0 ;
use.states.SL2 = 0;

%%transitions
use.transitions.allow_missing_tss = 0;
use.transitions.one_gene = 0 ; % force exactly one gene
use.transitions.use_multi = 1 ; % allow zero or more than one gene 

%%loss
use.loss.orf_only = 0;
use.model.one_gene = 0;
use.model.allow_missing_tss = 0;
use.model.use_multi = 1;

%%Segments & lengths
use.segments.transexon = 0 ;
use.segments.intercistronic = 0 ;
use.segments.intergenictrans = 0;
use.segments.polya_tail = 1;

%%Content 

use.contents.frame = 1;
use.contents.pre_comp = 1;

%%additional features
use.addfeatures.conservation = 0 ;
use.addfeatures.ests = 0 ;
use.addfeatures.proteins = 0 ;
use.addfeatures.tiling = 0 ;
use.addfeatures.rna_seq = 0 ;

% keyboard
method.use = use;



%----- REGULARIZATION
%C = 1 ;
C = 1e-1;
%%% linear regularization
method.par_ms.C_regul.lengths = C ;        % linear smoothness for len_hist
method.par_ms.C_regul.signals = C ;        % for sig_hist
method.par_ms.C_regul.contents = C ;       % for content_hist
method.par_ms.C_regul.transitions = C ;    % for

method.par_ms.C_regul.tiling = C ;  
method.par_ms.C_regul.rna_seq = C ;  


%%% quadratic regularization
C_sq = 1e-1;

method.par_ms.C_regul.transitions_sq = C_sq;
method.par_ms.C_regul.plif_ys_sq = C_sq*0.1 ;  % small_plif_ys_penalty
method.par_ms.C_regul.smoothness_sq = C_sq ;


%----------- for predictions only
method.par_ms.cleave_offset = 6;
%-----------

method.solver = 'dual';
method.ignore_init_file = 1;

method.opt_step = 1000 ;
method.max_neg = 2 ;
method.margin = 1 ;

method.INF = 1e20;
method.max_num_iterations = 100;

method.exm_per_solve = 400 ;
% method.exm_per_batch = 5 ;

%method.cleave_offset = 6 ;
% -------
% -------
LSL.method = method;


return

function [model] = make_model(organism,LSL,Signals,old)
% [model] = make_model(bins)

if nargin<4
  old=0;
end
  
model.bins = LSL.method.plif_bins ;
% model.use_gc =  LSL.method.use_gc;
model.use = LSL.method.use;
polya_sixmers = Signals.polya.consensus;

model = define_segments(model);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%define features

model = define_contents(model);
model = define_lengths(model,organism);
model = define_signals(model,polya_sixmers);
model = define_addContentFeatures(model);

model.cnt_plifs = model.cnt_contents+model.cnt_lengths+model.cnt_signals+model.cnt_tiling;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% remember the ordering of these fields
model.segments_fields = fieldnames(model.segments) ;
model.contents_fields = fieldnames(model.contents) ;
model.lengths_fields = fieldnames(model.lengths) ;
model.signals_fields = fieldnames(model.signals) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%define states

model = define_states(model);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define mappings between length, contents and segments

model = define_mappings(model);
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

model = gen_transition_matrix(model) ;
% transition_pointers used to be called penalties
model = define_transition_pointers(model);
if old
  model.penalties = model.transition_pointers;
end
model.A(~isinf(model.A))=0 ;



model.cnt_parameters = model.cnt_transitions + model.cnt_plifs*model.bins ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% define loss 
model =  define_loss(model);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% mappings of PLIFS to parameters

if ~old
  map_plif2param = init_weights(model);
  map_plif2param = rmfield(map_plif2param,'transitions');
  weights_name = fieldnames(map_plif2param);
  start=0;
  for w=1:length(weights_name)
    len=length(getfield(map_plif2param, weights_name{w}));
    model.plif2param{w} = start+(1:len);
    start = start + len;
  end
  model.transitions2param = start+1:model.cnt_parameters ;
  
  assert(model.cnt_parameters-model.cnt_transitions==start)
end

return

function model = define_segments(model)
% declare segment types

  cnt=0;  
model.segments.intergenic = cnt ;cnt=cnt+1;
model.segments.utr5exon   = cnt ;cnt=cnt+1;
if model.use.segments.transexon 
  model.segments.transexon  = cnt ;cnt=cnt+1;
end
model.segments.cds_exon   = cnt ;cnt=cnt+1;
model.segments.utr3exon   = cnt ;cnt=cnt+1;
if model.use.segments.polya_tail
  model.segments.polya_tail = cnt ;cnt=cnt+1;
end
if model.use.segments.intercistronic
  model.segments.intercistronic  = cnt ;cnt=cnt+1;
  model.segments.intergenictrans  = cnt ;cnt=cnt+1;
end
model.segments.intron     = cnt ;cnt=cnt+1;

model.cnt_segments = length(fieldnames(model.segments));


return

function model = define_contents(model)
  
% declare content types

fid = 1;  
model.mod_words = [];
% the first contents consider all k-mers. the last three
% frame-content sensors only consider codons, with an offset from
% the end of the segment

content_start =  1 ;
cnt=0;

model.contents.intergenic = content_start+cnt ;cnt=cnt+1;
model.mod_words = [model.mod_words; 1 0] ;
model.contents.utr_exon   = content_start+cnt ;cnt=cnt+1;
model.mod_words = [model.mod_words; 1 0] ;
model.contents.cds_exon   = content_start+cnt ;cnt=cnt+1;
model.mod_words = [model.mod_words; 1 0] ;
model.contents.intron     = content_start+cnt ;cnt=cnt+1;
model.mod_words = [model.mod_words; 1 0] ;
if model.use.segments.intercistronic
  model.contents.intercistronic = content_start+cnt;cnt=cnt+1;
  model.mod_words = [model.mod_words; 1 0] ;
end
if model.use.contents.frame
  model.contents.cds_frame0   = content_start+cnt ;cnt=cnt+1;
  model.mod_words = [model.mod_words; 3 0] ;
  model.contents.cds_frame1   = content_start+cnt ;cnt=cnt+1;
  model.mod_words = [model.mod_words; 3 1] ;
  model.contents.cds_frame2   = content_start+cnt ;cnt=cnt+1;
  model.mod_words = [model.mod_words; 3 2] ;
end
model.cnt_contents = length(fieldnames(model.contents));

%hack
if model.cnt_contents<8
  fprintf(fid,'introducing dummy contents\n')
  for i= 1:8-model.cnt_contents
    if i==1
      model.contents.('dummy')= content_start+cnt;cnt=cnt+1;
    else
      model.contents.(['dummy' num2str(i)])= content_start+cnt;cnt=cnt+1;
    end
  end
  model.cnt_contents = length(fieldnames(model.contents));
  model.mod_words = [model.mod_words; 1 0] ;
end

% declare content range
model.contents_range = struct;
content_names = fieldnames(model.contents) ;
for s = 1:length(content_names)
  model.contents_range = setfield(model.contents_range, content_names{s},[-10,10]);
end


return

function model = define_lengths(model,organism)
  
fid = 1; 
% declare length types
length_start = model.cnt_contents+1 ;
cnt = 0;

model.lengths.intergenic = length_start+cnt; cnt = cnt+1;
model.lengths.utr5exon = length_start+cnt ;cnt = cnt+1;
if model.use.segments.transexon 
  model.lengths.transexon = length_start+cnt ;cnt = cnt+1;
end
model.lengths.single_cds_exon = length_start+cnt ;cnt = cnt+1;
model.lengths.first_cds_exon = length_start+cnt ;cnt = cnt+1;
model.lengths.middle_cds_exon = length_start+cnt ;cnt = cnt+1;
model.lengths.last_cds_exon = length_start+cnt ;cnt = cnt+1;
model.lengths.utr3exon = length_start+cnt;cnt = cnt+1;
if model.use.segments.polya_tail 
  model.lengths.polya_tail = length_start+cnt;cnt = cnt+1;
end
model.lengths.intron = length_start+cnt ;cnt = cnt+1;
if model.use.segments.intercistronic
  model.lengths.intercistronic = length_start+cnt ;cnt = cnt+1;
  model.lengths.intergenictrans = length_start+cnt ;cnt = cnt+1;
end
model.lengths.intergenic_long = length_start+cnt ;cnt = cnt+1;

model.cnt_lengths = length(fieldnames(model.lengths));


% declare length range
s1=which([organism.name '_lengths']);
s2=which([organism.clade '_lengths']);
if ~isempty(s1)
  fprintf(fid,'taking lengths range from file: %s_lengths.m \n',organism.name)
  model.lengths_range = feval([organism.name '_lengths']);
elseif ~isempty(s2)
  fprintf(fid,'taking lengths range from file: %s_lengths.m \n',organism.clade)
  model.lengths_range = feval([organism.clade '_lengths']);
else
  fprintf(fid,'no lengths range specified, using default settings\n')
  model.lengths_range = feval('default_lengths');
end

return

function lengths_range = default_lengths

lengths_range.intergenic = [0, 21000] ;
%lengths_range.intergenictrans = [0, 21000] ;
lengths_range.utr5exon   = [0, 8000] ;
% lengths_range.transexon  = [1, 1000] ;
lengths_range.single_cds_exon = [100, 8000] ; %%%100, from don => acc+stop
lengths_range.first_cds_exon  = [3, 8000] ;
lengths_range.middle_cds_exon   = [3, 8000] ;
lengths_range.last_cds_exon   = [1, 8000] ;
lengths_range.utr3exon    = [1, 8000] ;
lengths_range.polya_tail  = [0, 250] ;
% lengths_range.intercistronic = [1, 5000] ;
lengths_range.intergenic_long = [10500-50, 10500+50] ; % due to irregular spacing 
lengths_range.intron      = [30, 21000] ;%%%30, from don => acc+stop

return

function model = define_signals(model,polya_sixmers)
  
% declare signal types
cnt=1;
signal_start = model.cnt_contents + model.cnt_lengths ;

if model.use.signals.tss
  model.signals.tss = signal_start+cnt;
  model.consensus.tss = {} ;
  model.consensus_pos.tss = [];
  cnt=cnt+1;
end

model.signals.tis = signal_start+cnt;
model.consensus.tis = {'atg'} ;
model.consensus_pos.tis = [0];
cnt=cnt+1;

model.signals.don = signal_start+cnt;
model.consensus.don = {'gt','gc'} ;
model.consensus_pos.don = [0,0];
cnt=cnt+1;

model.signals.acc = signal_start+cnt;
model.consensus.acc = {'ag'} ;
model.consensus_pos.acc = [2];
cnt=cnt+1;

model.signals.cdsStop = signal_start+cnt ;
model.consensus.cdsStop = {'taa','tga','tag'} ;
model.consensus_pos.cdsStop = [0,0,0];
cnt=cnt+1;

if model.use.signals.polya
  model.signals.polya = signal_start+cnt ;
  model.consensus.polya = polya_sixmers;
  model.consensus_pos.polya = zeros(1,length(polya_sixmers)) ;
  cnt=cnt+1;
end
if model.use.signals.cleave
  model.signals.cleave = signal_start+cnt ;
  model.consensus.cleave = {} ;
  model.consensus_pos.cleave = [];
  cnt=cnt+1;
end

if model.use.signals.trans
  model.signals.trans = signal_start+cnt;
  model.consensus.trans = {'ag'} ;
  model.consensus_pos.trans = [2];
  cnt=cnt+1;
end
if model.use.signals.transacc
  model.signals.transacc = signal_start+cnt;
  model.consensus.transacc = {'ag'} ;
  model.consensus_pos.transacc = [2];
  cnt=cnt+1;
end

model.cnt_signals = length(fieldnames(model.signals));


% declare signal ranges for PLiFs

if model.use.signals.tss
  model.signals_range.tss = [0,1] ;
end
model.signals_range.tis = [0,1] ;
model.signals_range.don = [0,1] ;
model.signals_range.acc = [0,1] ;
model.signals_range.cdsStop = [0,1] ;
if model.use.signals.polya
  model.signals_range.polya = [0,1] ;
end
if model.use.signals.cleave
  model.signals_range.cleave = [0,1] ;
end
if model.use.signals.trans
  model.signals_range.trans = [0,1] ;
end
if model.use.signals.transacc
  model.signals_range.transacc = [0,1] ;
end



return

function model = define_addContentFeatures(model)
  
% plifs for tiling array data
if model.use.addfeatures.tiling
  tiling_start =  model.cnt_contents + model.cnt_lengths + model.cnt_signals;
  content_names = fieldnames(model.contents);
  model.tiling_plif_ids = [];
  for s = 1:length(content_names)
    model.tiling.(content_names{s}) = tiling_start+s ;
    model.tiling_plif_ids = [model.tiling_plif_ids tiling_start+s];
  end
  model.cnt_tiling = length(fieldnames(model.tiling));
  % declare range
  model.tiling_range = struct;
  tiling_names = fieldnames(model.tiling) ;
  for s = 1:length(tiling_names)
    model.tiling_range = setfield(model.tiling_range, tiling_names{s},[6,11]);% ca. 2%-98% Percentile
  end
else
  model.cnt_tiling = 0;
end

return

function model = define_states(model)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% define states
model.states(1) = struct ;
cnt=0 ;

% sequence start 
cnt=cnt+1 ;
model.states(cnt).consensus={} ;
model.states(cnt).consensus_pos=[] ;
model.states(cnt).non_consensus={} ;
model.states(cnt).non_consensus_pos=[] ;
model.states(cnt).name='seq_start' ;
model.states(cnt).signal=[] ;
model.state_ids.seq_start=cnt ;

signal_names = fieldnames(model.signals) ;
for i=1:model.cnt_signals
  if strcmp(signal_names{i}, 'transacc'), continue; end ;
  if ~strcmp(signal_names{i},'acc') && ~strcmp(signal_names{i},'don') ...
        && ~strcmp(signal_names{i}, 'tis') && ~strcmp(signal_names{i}, 'cdsStop')...
        && ~strcmp(signal_names{i}, 'trans') && ~strcmp(signal_names{i}, 'cleave') 
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
    model.state_ids=setfield(model.state_ids, signal_names{i},cnt) ;
  elseif strcmp(signal_names{i},'cleave') 
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
    model.state_ids=setfield(model.state_ids, signal_names{i},[cnt]) ;
    if model.use.segments.intercistronic
      cnt=cnt+1 ;
      model.states(cnt).name = signal_names{i};
      model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
      model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
      model.states(cnt).non_consensus = {} ;
      model.states(cnt).non_consensus_pos = [] ;
      model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
      model.state_ids=setfield(model.state_ids, signal_names{i},[cnt-1 cnt]) ;
    end
  elseif strcmp(signal_names{i},'tis') | strcmp(signal_names{i},'cdsStop')
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.state_ids=setfield(model.state_ids, signal_names{i},[cnt-1 cnt]) ;
    if strcmp(signal_names{i}, 'tis')
    model.states(cnt).name = 'tis_acc' ;
      model.states(cnt).consensus = {'agatg'} ;
      model.states(cnt).consensus_pos = 2 ;
      model.states(cnt).signal = [model.signals.acc model.signals.tis] ;%%%% !!! 2 Zahlen
    else
      model.states(cnt).name = 'cdsStop_acc';
      model.states(cnt).consensus = {'agtaa', 'agtag', 'agtga'} ;
      model.states(cnt).consensus_pos = [2 2 2] ;
      model.states(cnt).signal = [model.signals.acc model.signals.cdsStop] ;%%%% !!! 2 Zahlen
    end
  elseif strcmp(signal_names{i},'trans') & model.use.states.SL1 & model.use.states.SL2
    cnt=cnt+1 ;
    model.states(cnt).name = 'SL1';
    model.states(cnt).consensus = getfield(model.consensus,'trans');
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,'trans');
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc]  ;
    cnt=cnt+1 ;
    model.states(cnt).name = 'SL1';
    model.states(cnt).consensus = {'agatg'} ; 
    model.states(cnt).consensus_pos = 2;
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc model.signals.tis]  ;
    model.state_ids = setfield(model.state_ids, 'SL1', [cnt-1 cnt]) ;
    cnt=cnt+1 ;
    model.states(cnt).name = 'SL2';
    model.states(cnt).consensus = getfield(model.consensus,'trans');
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,'trans');
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc]  ;
    cnt=cnt+1 ;
    model.states(cnt).name = 'SL2';
    model.states(cnt).consensus = {'agatg'} ; 
    model.states(cnt).consensus_pos = 2;
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc model.signals.tis]  ;%%%% !!! 3 Zahlen
    model.state_ids = setfield(model.state_ids, 'SL2', [cnt-1 cnt]) ;
  elseif strcmp(signal_names{i},'trans') & ~model.use.states.SL1 & ~model.use.states.SL2
    cnt=cnt+1 ;
    model.states(cnt).name = 'trans';
    model.states(cnt).consensus = getfield(model.consensus,'trans');
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,'trans');
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc]  ;
    cnt=cnt+1 ;
    model.states(cnt).name = 'trans';
    model.states(cnt).consensus = {'agatg'} ; 
    model.states(cnt).consensus_pos = 2;
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc model.signals.tis]  ;
    model.state_ids = setfield(model.state_ids, 'SL1', [cnt-1 cnt]) ;
  elseif strcmp(signal_names{i},'don')
    cnt_don = cnt ;
    cnt=cnt+8 ;
    for id = cnt_don+1:cnt_don+8 ;
      model.states(id).consensus = getfield(model.consensus,signal_names{i});
      model.states(id).consensus_pos = getfield(model.consensus_pos,signal_names{i});
      model.states(id).non_consensus = {} ;
      model.states(id).non_consensus_pos = [] ;
      model.states(id).signal=model.signals.don ;
    end ;
    model.states(cnt_don+1).name='UTR5_don' ;
    model.states(cnt_don+2).name='CDS_don_0' ;
    model.states(cnt_don+3).name='CDS_don_1a' ;
    model.states(cnt_don+3).non_consensus = {'t'} ;
    model.states(cnt_don+3).non_consensus_pos= [1] ;
    model.states(cnt_don+4).name='CDS_don_1b' ;
    model.states(cnt_don+4).consensus = {'tgt','tgc'} ;
    model.states(cnt_don+4).consensus_pos= [1,1] ;
    model.states(cnt_don+5).name='CDS_don_2a' ;
    model.states(cnt_don+5).non_consensus = {'ta', 'tg'} ;
    model.states(cnt_don+5).non_consensus_pos= [2,2] ;
    model.states(cnt_don+6).name='CDS_don_2b' ;
    model.states(cnt_don+6).consensus = {'tagt', 'tagc'} ;
    model.states(cnt_don+6).consensus_pos= [2,2] ;
    model.states(cnt_don+7).name='CDS_don_2c' ;
    model.states(cnt_don+7).consensus = {'tggt', 'tggc'} ;
    model.states(cnt_don+7).consensus_pos= [2,2] ;
    model.states(cnt_don+8).name='UTR3_don' ;
    
    model.state_ids.don=cnt_don+1:cnt_don+8 ;
  elseif strcmp(signal_names{i},'acc')
    cnt_acc = cnt ;
    cnt=cnt+8 ;
    for id = cnt_acc+1:cnt_acc+8 ;
      model.states(id).consensus = {'ag'} ;
      model.states(id).consensus_pos= [2] ;
      model.states(id).non_consensus={} ;
      model.states(id).non_consensus_pos=[] ;
      model.states(id).signal=model.signals.acc ;
    end ;
    model.states(cnt_acc+1).name='UTR5_acc' ;
    model.states(cnt_acc+2).name='CDS_acc_0' ;
    model.states(cnt_acc+3).name='CDS_acc_1a' ;
    model.states(cnt_acc+4).name='CDS_acc_1b' ;
    model.states(cnt_acc+4).non_consensus = {'aa','ag','ga'} ;
    model.states(cnt_acc+4).non_consensus_pos= [0,0,0] ;
    model.states(cnt_acc+5).name='CDS_acc_2a' ;
    model.states(cnt_acc+6).name='CDS_acc_2b' ;
    model.states(cnt_acc+6).consensus = {'agt', 'agc'} ;
    model.states(cnt_acc+6).consensus_pos= [2, 2] ;
    model.states(cnt_acc+7).name='CDS_acc_2c' ;
    model.states(cnt_acc+7).non_consensus = {'a'} ;
    model.states(cnt_acc+7).non_consensus_pos= [0] ;
    model.states(cnt_acc+8).name='UTR3_acc' ;
 
    model.state_ids.acc=cnt_acc+1:cnt_acc+8 ;

    if isfield(model.use, 'intron_long_state')&& model.use.intron_long_state
      cnt_intron_long = cnt;
      cnt = cnt+6;
      model.states(cnt_intron_long+1).consensus={} ;
      model.states(cnt_intron_long+1).consensus_pos=[] ;
      model.states(cnt_intron_long+1).non_consensus={} ;
      model.states(cnt_intron_long+1).non_consensus_pos=[] ;
      model.states(cnt_intron_long+1).name='intron_long_0' ;
      model.states(cnt_intron_long+1).signal=[] ;

      model.states(cnt_intron_long+2).consensus={} ;
      model.states(cnt_intron_long+2).consensus_pos=[] ;
      model.states(cnt_intron_long+2).non_consensus={} ;
      model.states(cnt_intron_long+2).non_consensus_pos=[] ;
      model.states(cnt_intron_long+2).name='intron_long_1a' ;
      model.states(cnt_intron_long+2).signal=[] ;
    
      model.states(cnt_intron_long+3).consensus={} ;
      model.states(cnt_intron_long+3).consensus_pos=[] ;
      model.states(cnt_intron_long+3).non_consensus={} ;
      model.states(cnt_intron_long+3).non_consensus_pos=[] ;
      model.states(cnt_intron_long+3).name='intron_long_1b' ;
      model.states(cnt_intron_long+3).signal=[] ;

      model.states(cnt_intron_long+4).consensus={} ;
      model.states(cnt_intron_long+4).consensus_pos=[] ;
      model.states(cnt_intron_long+4).non_consensus={} ;
      model.states(cnt_intron_long+4).non_consensus_pos=[] ;
      model.states(cnt_intron_long+4).name='intron_long_2a' ;
      model.states(cnt_intron_long+4).signal=[] ;

      model.states(cnt_intron_long+5).consensus={} ;
      model.states(cnt_intron_long+5).consensus_pos=[] ;
      model.states(cnt_intron_long+5).non_consensus={} ;
      model.states(cnt_intron_long+5).non_consensus_pos=[] ;
      model.states(cnt_intron_long+5).name='intron_long_2b' ;
      model.states(cnt_intron_long+5).signal=[] ;

      model.states(cnt_intron_long+6).consensus={} ;
      model.states(cnt_intron_long+6).consensus_pos=[] ;
      model.states(cnt_intron_long+6).non_consensus={} ;
      model.states(cnt_intron_long+6).non_consensus_pos=[] ;
      model.states(cnt_intron_long+6).name='intron_long_1a' ;
      model.states(cnt_intron_long+6).signal=[] ;

      model.state_ids.intergenic_long= cnt_intron_long+1:cnt_intron_long+6;
    end
%    cnt
  end
end

% intergenic_long
cnt=cnt+1 ;
model.states(cnt).consensus={} ;
model.states(cnt).consensus_pos=[] ;
model.states(cnt).non_consensus={} ;
model.states(cnt).non_consensus_pos=[] ;
model.states(cnt).name='intergenic_long' ;
model.states(cnt).signal=[] ;
model.state_ids.intergenic_long=cnt ;

% sequence end
cnt=cnt+1 ;
model.states(cnt).consensus={} ;
model.states(cnt).consensus_pos=[] ;
model.states(cnt).non_consensus={} ;
model.states(cnt).non_consensus_pos=[] ;
model.states(cnt).name='seq_end' ;
model.states(cnt).signal=[] ;
model.state_ids.seq_end = cnt ;

model.cnt_states = cnt ; 

return

function model = define_mappings(model)
  

%%mappings between length and contents
  
model.plif_links = [] ;
model.plif_links(end+1,:) = [model.lengths.intergenic      model.contents.intergenic] ;
model.plif_links(end+1,:) = [model.lengths.first_cds_exon  model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.last_cds_exon   model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.middle_cds_exon model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.single_cds_exon model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.intron          model.contents.intron] ;
if model.use.segments.transexon
  model.plif_links(end+1,:) = [model.lengths.transexon       model.contents.utr_exon] ;
end
model.plif_links(end+1,:) = [model.lengths.utr5exon        model.contents.utr_exon] ;
model.plif_links(end+1,:) = [model.lengths.utr3exon        model.contents.utr_exon] ;
if model.use.segments.polya_tail
  model.plif_links(end+1,:) = [model.lengths.polya_tail      model.contents.utr_exon] ;
end
if model.use.segments.intercistronic
  model.plif_links(end+1,:) = [model.lengths.intercistronic  model.contents.intercistronic] ;
  model.plif_links(end+1,:) = [model.lengths.intergenictrans  model.contents.intergenic] ;
end
model.plif_links(end+1,:) = [model.lengths.intergenic_long model.contents.intergenic] ;


if isfield(model.state_ids, 'intron_long') 
  model.plif_links(end+1,:) = [model.lengths.intron_long model.contents.intron] ;  
end

%%mappings between length and additional contents (tiling)

if model.use.addfeatures.tiling
  model.tiling_links = [] ;   
  model.tiling_links(end+1,:) = [model.lengths.intergenic      model.tiling.intergenic] ;
  model.tiling_links(end+1,:) = [model.lengths.first_cds_exon  model.tiling.cds_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.last_cds_exon   model.tiling.cds_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.middle_cds_exon model.tiling.cds_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.single_cds_exon model.tiling.cds_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.intron          model.tiling.intron] ;
  if model.use.segments.transexon
    model.tiling_links(end+1,:) = [model.lengths.transexon       model.tiling.transexon] ;
  end
  model.tiling_links(end+1,:) = [model.lengths.utr5exon        model.tiling.utr_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.utr3exon        model.tiling.utr_exon] ;
  if model.use.segments.polya_tail
    model.tiling_links(end+1,:) = [model.lengths.polya_tail      model.tiling.utr_exon] ;
  end
  model.tiling_links(end+1,:) = [model.lengths.intergenic_long model.tiling.intergenic] ;
  if model.use.segments.intercistronic
    model.tiling_links(end+1,:) = [model.lengths.intercistronic  model.tiling.intercistronic] ;
  end
end

%%mappings between length and segments

model.seg_links = [] ;
model.seg_links(end+1,:) = [model.lengths.intergenic      model.segments.intergenic] ;
model.seg_links(end+1,:) = [model.lengths.first_cds_exon  model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.last_cds_exon   model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.middle_cds_exon model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.single_cds_exon model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.intron          model.segments.intron] ;
if model.use.segments.transexon
  model.seg_links(end+1,:) = [model.lengths.transexon       model.segments.transexon] ;
end
model.seg_links(end+1,:) = [model.lengths.utr5exon        model.segments.utr5exon] ;
model.seg_links(end+1,:) = [model.lengths.utr3exon        model.segments.utr3exon] ;
if model.use.segments.polya_tail
  model.seg_links(end+1,:) = [model.lengths.polya_tail      model.segments.polya_tail] ;
end
if model.use.segments.intercistronic
  model.seg_links(end+1,:) = [model.lengths.intergenictrans  model.segments.intergenictrans] ;
  model.seg_links(end+1,:) = [model.lengths.intercistronic  model.segments.intercistronic] ;
end
model.seg_links(end+1,:) = [model.lengths.intergenic_long model.segments.intergenic] ;
%

return

function model = gen_transition_matrix(model,PAR)

use=model.use;  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define transitions

A = -inf(model.cnt_states, model.cnt_states);
p = -inf(model.cnt_states,1);
q = -inf(model.cnt_states,1) ;
p(model.state_ids.seq_start) = 0 ;
q(model.state_ids.seq_end) = 0 ;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% start constructing transition matrix A

orf_from = -1*ones(model.cnt_states,1) ;
orf_to = -1*ones(model.cnt_states,1) ;

orf_from(model.state_ids.acc(2:7))= [ 0 1 1 2 2 2] ;
orf_from(model.state_ids.tis)     = 0 ; % AGAGT case
if model.use.states.SL1 & model.use.states.SL2
  orf_from(model.state_ids.SL1(2)) = 0 ; % AGAGT case
  orf_from(model.state_ids.SL2(2)) = 0 ; % AGAGT case
elseif model.use.signals.trans
  orf_from(model.state_ids.trans(2)) = 0 ; % AGAGT case
end
orf_to(model.state_ids.don(2:7))  = [0 1 1 2 2 2] ;
orf_to(model.state_ids.cdsStop(1))   = 0 ;
orf_info=[orf_from, orf_to] ;

%%%%%%%%%%%
%% no gene / intergenic stuff
%%%%%%%%%%%
if ~use.model.one_gene,
  A(model.state_ids.seq_start,  model.state_ids.seq_end) = model.lengths.intergenic ;
end
A(model.state_ids.intergenic_long, model.state_ids.intergenic_long) = 0; % no penalty et al
A(model.state_ids.seq_start, model.state_ids.intergenic_long) = model.lengths.intergenic_long ;
A(model.state_ids.intergenic_long, model.state_ids.seq_end) = model.lengths.intergenic_long ;

%%%%%%%%%%%
% 5' UTR 
%%%%%%%%%%%
gene_start_states_in = [];
gene_start_states_out = [];
mRNA_start_states_out = [model.state_ids.tis];

if use.signals.tss,
  gene_start_states_in = [gene_start_states_in model.state_ids.tss] ;
  gene_start_states_out = [gene_start_states_out model.state_ids.tss] ;
  if use.model.allow_missing_tss,
    assert(false) ;
    gene_start_states_in = [gene_start_states_in model.state_ids.tis] ;
  end ;
else
  assert(false)
  gene_start_states_in = [gene_start_states_in model.state_ids.tis] ;
end ;

if use.signals.trans,
  gene_start_states_in = [gene_start_states_in model.state_ids.SL1] ;
  gene_start_states_out = [gene_start_states_out model.state_ids.SL1(1)] ;
  gene_start_states_out = [gene_start_states_out model.state_ids.SL2(1)] ;
  mRNA_start_states_out = [mRNA_start_states_out model.state_ids.SL1(2)] ;%'AGAGT cases'
  mRNA_start_states_out = [mRNA_start_states_out model.state_ids.SL2(2)] ;%'AGAGT cases'
  if use.signals.tss, 
    A(model.state_ids.tss, model.state_ids.SL1)    = model.lengths.transexon ;  
  end ;
  if use.segments.intercistronic & use.signals.cleave ,
    A(model.state_ids.cleave(1), model.state_ids.SL1) = model.lengths.intergenictrans ;
    A(model.state_ids.cleave(2), model.state_ids.SL1) = model.lengths.intercistronic ;
    A(model.state_ids.cleave(2), model.state_ids.SL2) = model.lengths.intercistronic ;
  end ;
end ;

% gene start
A(model.state_ids.seq_start,gene_start_states_in) = model.lengths.intergenic ;
if use.model.use_multi
  A(model.state_ids.intergenic_long, gene_start_states_in) = model.lengths.intergenic_long ; 
end
    
A(gene_start_states_out,model.state_ids.tis(1)) = model.lengths.utr5exon ;
A(gene_start_states_out,model.state_ids.don(1)) = model.lengths.utr5exon ;
% spliced 5'utr
A(model.state_ids.acc(1), model.state_ids.don(1)) = model.lengths.utr5exon ;
A(model.state_ids.don(1), model.state_ids.acc(1)) = model.lengths.intron ;
% last 5'utr exon
A(model.state_ids.acc(1),model.state_ids.tis(1)) = model.lengths.utr5exon ;
A(model.state_ids.don(1), model.state_ids.tis(2)) = model.lengths.intron ; 


%%%%%%%%%%%
% CDS STUFF
%%%%%%%%%%%
A(mRNA_start_states_out, model.state_ids.cdsStop(1)) = model.lengths.single_cds_exon ; % trans(2) ok 
A(mRNA_start_states_out, model.state_ids.don(2:7)) = model.lengths.first_cds_exon ; % trans(2) ok
for i=2:7,
  A(model.state_ids.don(i), model.state_ids.acc(i)) = model.lengths.intron ;
  if isfield(model.state_ids, 'intron_long')
    A(model.state_ids.don(i), model.state_ids.intron_long(i-1)) = model.lengths.intron;
    A(model.state_ids.intron_long(i-1), model.state_ids.acc(i)) = model.lengths.intron_long;
    A(model.state_ids.intron_long(i-1),model.state_ids.intron_long(i-1)) = model.lengths.intron_long;
  end
end ;
A(model.state_ids.acc(2:7), model.state_ids.don(2:7)) = model.lengths.middle_cds_exon ;
A(model.state_ids.acc(2:7), model.state_ids.cdsStop(1)) = model.lengths.last_cds_exon ;
% agstop 
A(model.state_ids.don(2), model.state_ids.cdsStop(2)) = model.lengths.intron ;



%%%%%%%%%%%
% 3' UTR
%%%%%%%%%%%
A(model.state_ids.cdsStop, model.state_ids.don(8)) = model.lengths.utr3exon ;
A(model.state_ids.don(8), model.state_ids.acc(8)) = model.lengths.intron ;
A(model.state_ids.acc(8), model.state_ids.don(8)) = model.lengths.utr3exon ;

if use.signals.polya,
  % polyA has to be in last 3'UTR exon
  A(model.state_ids.cdsStop, model.state_ids.polya) = model.lengths.utr3exon ;
  A(model.state_ids.acc(8), model.state_ids.polya)=model.lengths.utr3exon ;
end ;

gene_stop_states = [];    
if use.signals.cleave,
  gene_stop_states = [gene_stop_states model.state_ids.cleave(1)];
  A(model.state_ids.cdsStop, model.state_ids.cleave) = model.lengths.utr3exon ;
  A(model.state_ids.acc(8), model.state_ids.cleave) = model.lengths.utr3exon ;
  if use.signals.polya,
    A(model.state_ids.polya, model.state_ids.cleave) = model.lengths.polya_tail ;
  end
else
  gene_stop_states = [gene_stop_states model.state_ids.cdsStop];
  if use.signals.polya,
    gene_stop_states = [gene_stop_states model.state_ids.polya];
  end
end ;
A(gene_stop_states, model.state_ids.seq_end)    = model.lengths.intergenic ; 
A(gene_stop_states, model.state_ids.intergenic_long) = model.lengths.intergenic_long ; 
if use.model.use_multi,
  A(gene_stop_states, gene_start_states_in) = model.lengths.intergenic ;    
end ;

if use.segments.intercistronic 
  for k=model.state_ids.SL1,
    idx = find(A(:,k)==model.lengths.intergenic) ;
    A(idx,k) = model.lengths.intergenictrans ;   
  end ;
end
%%%%%%%%%%%
% end transition definition
%%%%%%%%%%%


%%% technical stuff for intergenic_long

transitions = [];
for i=1:length(gene_start_states_in)
  transitions = [transitions; model.state_ids.seq_start gene_start_states_in(i)];
  for j=1:length(gene_stop_states)
    transitions = [transitions; gene_stop_states(j) gene_start_states_in(i)];
    transitions = [transitions; gene_stop_states(j) model.state_ids.seq_end] ;
  end
end

model.intergenic_long.transitions = unique(transitions,'rows');
model.intergenic_long.max_short_length = model.lengths_range.intron(2) ;
model.intergenic_long.step = model.lengths_range.intron(2); % use the intron length as maximal step size
model.intergenic_long.long_segment_parts = model.lengths_range.intron(2)/2-25 ;
%model.intergenic_long.step = 30 ; % makes about 30 (step) * 40 (grid)=1200nt per step

%%% 

q(gene_stop_states) = 0 ;

model.gene_start_states_in = gene_start_states_in;
model.gene_start_states_out = gene_start_states_out;
model.mRNA_start_states_out = mRNA_start_states_out;
model.gene_stop_states = gene_stop_states;

model.A = A;
model.p = p;
model.q = q;
model.orf_info = orf_info;
model.active_transitions = find(~isinf(A)) ;
model.cnt_transitions = length(model.active_transitions) ;



return

function model = define_transition_pointers(model)
  
A = model.A;
mRNA_start_states_out=model.mRNA_start_states_out;
%%% transition_pointers contain for each allowed transition pointers to lengths plifs
transition_pointers = zeros(size(A)) ;
transition_pointers(~isinf(A))=A(~isinf(A)) ;
transition_pointers = transition_pointers' ;

%%% add pointers to content PLIFs
transition_pointers(:,:,2) = zeros(size(transition_pointers,1), size(transition_pointers,2), 1) ;
for i=1:size(model.plif_links,1)
  [idx1,idx2] = find(transition_pointers(:,:,1)==model.plif_links(i,1));
  for j=1:length(idx1)
    transition_pointers(idx1(j),idx2(j),2) = model.plif_links(i,2);
  end ;
end


%%% add pointers to tiling PLIFs
if model.use.addfeatures.tiling
  dim3 = size(transition_pointers,3);
  transition_pointers(:,:,dim3+1) = zeros(size(transition_pointers,1), size(transition_pointers,2), 1) ;
  for i=1:size(model.tiling_links,1)
    [idx1,idx2] = find(transition_pointers(:,:,1)==model.tiling_links(i,1));
    for j=1:length(idx1)
      transition_pointers(idx1(j),idx2(j),3) = model.tiling_links(i,2);
    end ;
  end
end

%%% add pointers to frame PLIFs
tmp = zeros(size(transition_pointers(:,:,1))) ;
tmp(mRNA_start_states_out, model.state_ids.cdsStop(1)) = model.contents.cds_frame0 ;% single cds exon  

if ~isfield(model.use.contents, 'pre_comp') || ~model.use.contents.pre_comp,
  tmp(mRNA_start_states_out, model.state_ids.don(2)) = model.contents.cds_frame0 ;% first cds exon 
  tmp(mRNA_start_states_out, model.state_ids.don(3:4)) = model.contents.cds_frame1 ;% first cds exon  
  tmp(mRNA_start_states_out, model.state_ids.don(5:7)) = model.contents.cds_frame2 ;% first cds exon 
  
  tmp(model.state_ids.acc(2:7), model.state_ids.don(2)) = model.contents.cds_frame0 ;% middle cds exon 
  tmp(model.state_ids.acc(2:7), model.state_ids.don(3:4)) = model.contents.cds_frame1 ;% middle cds exon 
  tmp(model.state_ids.acc(2:7), model.state_ids.don(5:7)) = model.contents.cds_frame2 ;% middle cds exon 
  tmp(model.state_ids.acc(2:7), model.state_ids.cdsStop(1)) = model.contents.cds_frame0 ;% last cds exon 
else
  tmp(mRNA_start_states_out, model.state_ids.don(2:7)) = model.contents.cds_frame0 ;% first cds exon 
  tmp(model.state_ids.acc(2),model.state_ids.don(2:7)) = model.contents.cds_frame0 ;% middle cds exon 
  tmp(model.state_ids.acc(3:4),model.state_ids.don(2:7)) = model.contents.cds_frame1 ;% middle cds exon 
  tmp(model.state_ids.acc(5:7),model.state_ids.don(2:7)) = model.contents.cds_frame2 ;% middle cds exon 
  tmp(model.state_ids.acc(2), model.state_ids.cdsStop(1)) = model.contents.cds_frame0 ;% last cds exon 
  tmp(model.state_ids.acc(3:4), model.state_ids.cdsStop(1)) = model.contents.cds_frame1 ;% last cds exon 
  tmp(model.state_ids.acc(5:7), model.state_ids.cdsStop(1)) = model.contents.cds_frame2 ;% last cds exon 
end ;


dim3 = size(transition_pointers,3);
transition_pointers(:,:,dim3+1) = tmp' ; % don't touch this---tmp has to be transposed!

a_trans = zeros(4,sum(~isinf(A(:)))) ;
% [from,to,val,segment]
k = 0 ;
for i=1:size(A,1)
  idx = find(~isinf(A(i,:))) ;
  val = A(i,idx) ;
  pen = transition_pointers(idx,i,1) ;
  a_trans(1,k+1:k+length(idx))=i-1 ;
  a_trans(2,k+1:k+length(idx))=idx-1 ;  
  a_trans(3,k+1:k+length(idx))=val ;    
  for q=1:length(pen) 
    idx_=find(model.seg_links(:,1)==pen(q)) ;
    if isempty(idx_), 
      assert(pen(q)==0) ; 
    else
      assert(length(idx_)==1) ;
      pen(q) = model.seg_links(idx_,2) ;
    end ;
  end ;
  a_trans(4,k+1:k+length(idx))= pen;
  k=k+length(idx) ;
end ;
%a_trans = a_trans' ;
%[tmp,idx] = sort(a_trans(:,2)) ;
%a_trans = a_trans(idx,:) ;

model.transition_pointers = transition_pointers ;
model.a_trans = a_trans;

return

function model = define_loss(model)
  

% create loss matrices

equiv = [];
  
different = [model.segments.intergenic, model.segments.cds_exon, 3 ;
             model.segments.intergenic, model.segments.intron, 3;
             model.segments.intron, model.segments.cds_exon, 2];

%--------------------------------
if ~model.use.loss.orf_only
  different = [different;
               model.segments.intergenic, model.segments.utr5exon, 2;
               model.segments.intergenic, model.segments.utr3exon, 2];
  if model.use.segments.intercistronic
    different = [different;
                 model.segments.intercistronic, model.segments.utr5exon, 2;
                 model.segments.intercistronic, model.segments.utr3exon, 2;
                 model.segments.intergenictrans, model.segments.utr5exon, 2;
                 model.segments.intergenictrans, model.segments.utr3exon, 2];
    
  end
else
  equiv = [equiv;
           model.segments.intergenic model.segments.utr3exon ;
           model.segments.intergenic model.segments.utr5exon] ;
  if model.use.segments.intercistronic
    equiv = [equiv;
             model.segments.intercistronic model.segments.utr3exon ; 
             model.segments.intercistronic model.segments.utr5exon ;
             model.segments.intergenictrans model.segments.utr3exon ;
             model.segments.intergenictrans model.segments.utr5exon] ;
  end
end
%--------------------------------
if model.use.segments.polya_tail
  equiv = [equiv;
           model.segments.utr3exon, model.segments.polya_tail];
  different = [different;
               model.segments.intergenic, model.segments.polya_tail, 1];    
end
 %-------------------------------- 
if model.use.segments.intercistronic
  
  different = [different;
               model.segments.intercistronic, model.segments.cds_exon, 3 ;
               model.segments.intercistronic, model.segments.intron, 3;
               model.segments.intercistronic, model.segments.transexon, 2;
               model.segments.intergenic, model.segments.intercistronic, 0.1;
               model.segments.intergenictrans, model.segments.intercistronic, 0.1;
               model.segments.intergenictrans, model.segments.cds_exon, 3 ;
               model.segments.intergenictrans, model.segments.intron, 3;
               model.segments.intergenictrans, model.segments.transexon, 2;
               model.segments.intergenic, model.segments.intergenictrans, 0.1];

  if model.use.segments.polya_tail
    different = [different;
                 model.segments.intercistronic, model.segments.polya_tail,1;
                 model.segments.intergenictrans, model.segments.polya_tail, 1];
  end
end
%--------------------------------
if model.use.segments.transexon
  equiv = [equiv;
           model.segments.utr5exon, model.segments.transexon] ;     
  different = [different;
               model.segments.intergenic, model.segments.transexon, 1] ;
  if isfield(model.segments, 'intergenictrans'),
    different = [different;
                 model.segments.intergenictrans, model.segments.transexon, 1];    
  end ;
end
%--------------------------------

model.loss.segments = ones(model.cnt_segments) - eye(model.cnt_segments);
for i=1:size(different,1)
  model.loss.segments(different(i,1)+1, different(i,2)+1) = different(i,3) ;
  % model.loss.segments(different(i,1)+1, different(i,2)+1) = different(i,3)+1 ;warning('make loss matrix asymetric to enhance sensitivity')
  model.loss.segments(different(i,2)+1, different(i,1)+1) = different(i,3) ;
end ;

for i=1:size(equiv,1)
  model.loss.segments(equiv(i,1)+1, equiv(i,2)+1) = 0 ;
  model.loss.segments(equiv(i,2)+1, equiv(i,1)+1) = 0 ;
end ;


% tripple loss for coding
%model.loss.segments(model.segments.cds_exon+1, :) = 3*model.loss.segments(model.segments.cds_exon+1, :) ;
%model.loss.segments(:, model.segments.cds_exon+1) = 3*model.loss.segments(:, model.segments.cds_exon+1) ;

% first part is for segments, the second for nucleotides
model.loss.segments = [model.loss.segments 0.0*model.loss.segments] ;

if model.use.signals.tss
  model.loss.zero_range.tss = 10 ;
end
model.loss.zero_range.tis = 0;
model.loss.zero_range.don = 0;
model.loss.zero_range.acc = 0;
if model.use.signals.trans
  model.loss.zero_range.trans = 0;
end
model.loss.zero_range.cdsStop = 0;
if model.use.signals.polya
  model.loss.zero_range.polya = 10 ;
end
if model.use.signals.cleave
  model.loss.zero_range.cleave = 10 ;
end

model.loss.confirmed = 1 ;
model.loss.confirmed_tol = 1000 ; % region around confirmed gene
model.loss.unconfirmed = 1 ;

% model.loss.mustbe_corr.intergenic = 0 ;
% model.loss.mustbe_corr.utr5exon   = 0;
% model.loss.mustbe_corr.transexon  = 0 ;
% model.loss.mustbe_corr.cds_exon   = 1 ;
% model.loss.mustbe_corr.utr3exon   = 0 ;
% model.loss.mustbe_corr.polya_tail = 0 ;
% model.loss.mustbe_corr.intercistronic  = 0 ;
% model.loss.mustbe_corr.intron     = 1 ;
return ;

return

function weights=init_weights(model) ;
% weights=init_weights(model) ;

weights = struct;

%%% content detectors
content_names = fieldnames(model.contents) ;
for s = 1:model.cnt_contents,
  weights = setfield(weights, ['contents_' content_names{s}], zeros(1,model.bins));
end

%%% length detectors
length_names = fieldnames(model.lengths) ;
for s = 1:model.cnt_lengths,
  weights = setfield(weights, ['lengths_' length_names{s}], zeros(1,model.bins));
end

%%% signal detectors
signal_names = fieldnames(model.signals) ;
for s = 1:model.cnt_signals,
  weights = setfield(weights, ['signals_' signal_names{s}], zeros(1,model.bins));
end

%%% tiling detectors
if isfield(model, 'tiling')
  tiling_names = fieldnames(model.tiling) ;
  for s = 1:model.cnt_tiling,
    weights = setfield(weights, ['tiling_' tiling_names{s}], zeros(1,model.bins));
  end
end
%%% rna_seq detectors
if isfield(model, 'rna_seq')
  rna_seq_names = fieldnames(model.rna_seq) ;
  for s = 1:model.cnt_rna_seq,
    weights = setfield(weights, ['rna_seq_' rna_seq_names{s}], zeros(1,model.bins));
  end
end
%%% rna_seq_intron_track detectors
if isfield(model, 'rna_seq_intron_track')
  rna_seq_intron_track_names = fieldnames(model.rna_seq_intron_track) ;
  for s = 1:model.cnt_rna_seq_intron_track,
    weights = setfield(weights, ['rna_seq_intron_track_' rna_seq_intron_track_names{s}], zeros(1,model.bins));
  end
end

%%% transitions
weights.transitions = zeros(1,model.cnt_transitions) ;

return

function diff_fields = compare_PAR(PAR_1,PAR_2,silent)

if nargin<3
  silent=0;
end

diff_fields ={};
if isequal(PAR_1,PAR_2)
  fprintf(1,'\t parameters identical\n')
else
  fields_1 = fieldnames(PAR_1);
  fields_2 = fieldnames(PAR_2);
  common_fields = intersect(fields_1,fields_2);
  for i=1:length(common_fields)
    par_1 = getfield(PAR_1,common_fields{i});
    par_2 = getfield(PAR_2,common_fields{i});
    if ~isequal(par_1,par_2)
      diff_fields{end+1} = common_fields{i};
      if ~silent
        fprintf(1,'\t field "%s" not identical\n',common_fields{i})
        fprintf(1,'\t %s, 1:\n',common_fields{i});disp(par_1)
        fprintf(1,'\t %s, 2:\n',common_fields{i});disp(par_2)
      end
    end
  end
  fields_x1= setdiff(fields_1,fields_2);
  if ~isempty(fields_x1)
    if ~silent
      fprintf(1,'\t additional fields in PAR_1: \n')
    end
    diff_fields = {diff_fields{:},fields_x1{:}};
  end
  fields_x2 = setdiff(fields_2,fields_1);
  if ~isempty(fields_x2)
    if ~silent
      fprintf(1,'\t additional fields in PAR_2: \n')
    end
    diff_fields = {diff_fields{:},fields_x2{:}};
  end
end
   
return

function [fn, exists]=gen_example_filename(Signal,fn_filter_settings,fn_examples,which,split)

% [fn, exists]=gen_example_filename(Signal,fn_filter_settings,fn_examples,which,split)
% [fn, exists]=gen_example_filename(PAR,which,split)
% which is 'train','eval','test'

% sig = PAR.Signal_name;
% Signal = PAR.Signals.(sig); 
% fn_filter_settings =  PAR.FN.input_sig.(sig).input_sig;
% fn_examples = PAR.FN.input_sig.(sig).fn_examples;

whichs = fieldnames(Signal.filter_label);


if isnumeric(which)
  assert(which<=length(whichs))
  which = whichs{which};
else
  assert(ismember(which,whichs))
end
clear whichs

current_settings = Signal.filter_label.(which);

if fexist(fn_filter_settings)
  load(fn_filter_settings, 'filter_settings')
  idx = find([filter_settings.split]==split);
else
  filter_settings = [];
  idx = [];
end
idx_setting = [];is_equal=0;
for i=idx
  temp = rmfield(filter_settings(i),'split');
  temp = rmfield(temp,'filename');
  if isequal(temp, current_settings)
    idx_setting = [idx_setting i];
    is_equal=1;
  else
    assert(length(fieldnames(temp))==length(fieldnames(current_settings))&&...
           all(strcmp(sort(fieldnames(temp)),sort(fieldnames(current_settings)))))
  end
end
assert(length(idx_setting)<=1);
if isempty(idx_setting)
  temp = struct;
end
assert(length(setdiff(fieldnames(temp),fieldnames(current_settings)))==0)

if ~isempty(idx_setting)
  fn = filter_settings(idx_setting).filename;
  if fexist(fn)
    exists = 1;
  else
    exists = 0;
  end
else 
  settings_names = fieldnames(current_settings);
  s = length(filter_settings)+1;
  for j = 1:length(settings_names)
    filter_settings(s).(settings_names{j}) = current_settings.(settings_names{j});
  end
  filter_settings(s).split = split;
  filter_settings(s).filename = sprintf('%s_settings=%i_split=%i.mat',fn_examples, s, split);
  fn = filter_settings(s).filename;
  if fexist(fn)
    error('I do not know where that file comes from!!!')
  else
    exists = 0;
  end 
  save(fn_filter_settings, 'filter_settings')
end



       
%fn = sprintf('%s_%s_pc=%i_pa=%i_nc=%i_na=%i_split=%i.mat',...
%	input_sig.fn_examples,PAR.Signal_name,pos_conf,...
%	pos_anno,neg_conf,neg_anno, s);

return

function save_examples(blocks,split,P)
%save_examples(blocks,split,P)
%
% INPUT 	regions with field Signals with field `Signal.name` with fields:
%			-pos
%			-anno_cover
%			-est_cover
%			-cdna_cover
%	PAR with fields
%			-organism
%			-Signal.filter_label
%			-Signal.name
%			-Signal.consensus
%			-FN.input_sig.fn_examples
%			-FN.input.fn_split
%			-FN.input.fn_regions


%only for RPROC
if nargin<3
  %addpath ..
  % use the rproc.addpath way to add paths in client processes!!
  %paths
  P = blocks.P;
  split = blocks.split;
  blocks = blocks.blocks;
end

Signal = P.Signal ; 
num_splits = P.num_splits;

fn_training_blocks = P.fn_training_blocks;
fn_candsites = P.fn_candsites;
fn_filter_settings = P.fn_filter_settings;
fn_examples = P.fn_examples;
fn_example_statistics = P.fn_example_statistics;

Signal_name = Signal.name;
lwin = Signal.lwin_big;
rwin = Signal.rwin_big;

fields = fieldnames(Signal.filter_label);
fprintf( 1, 'save_examples... (%d fields, %d splits)\n', length(fields), num_splits );
for ff = 1:length(fields)
   for ss=1:num_splits
    %---------------------------------------------
    %CREATE OUTPUT FILENAME
    %----------------------------------------------  
    filename = gen_example_filename(Signal,fn_filter_settings,fn_examples,ff,ss);
    if ~fexist(filename)
      num_examples = 0;
      POS.fn_training_blocks = fn_training_blocks;	 
      split_reg_ids = getfield(split,sprintf('split_%i',ss));
      blocks_idx = find(ismember([blocks.id],split_reg_ids));
      split_regs = blocks(blocks_idx);
      clear blocks_idx split_reg_ids
      
      %if ~isfield(split_regs,'seq') 
      %  fprintf(1,'loading sequence...\n')
      %  split_regs = load_sequence(split_regs);
      %  fprintf(1,'\n')
      %end
      
      %---------------------------------------------
      % retreive label information
      %---------------------------------------------
      
      % if ~isfield(split_regs, 'Signals')
      %   temp_region = retrieve_signals(split_regs(1),fn_candsites,Signal_name,{'label',Sig.Conf_names{:}});
      % end
      
      
      %---------------------------------------------
      %----heuristic--->estimate size of chararray; 
      % -some positions are filtered out by the filerfunction: Signal.filter_fct
      % -if the size is to small then this function is very time-consuming
      % -otherwise it is very space-consuming
      %---------------------------------------------
      %total_num_pos = retrieve_num_pos(split_regs, fn_candsites) ;
      %num_pos = ceil(total_num_pos) ;
     
      %--------------------------------------------------
      % determine the maximal number of training examples
      % for the given sensor
      %--------------------------------------------------
      t = cputime;
      num_pos_ex = 0;
      num_neg_ex = 0;
      for r=1:length(split_regs)
        if ~isfield(split_regs, 'Signals')
          temp_region = retrieve_signals(split_regs(r),fn_candsites,Signal_name,{'label'});
	  num_pos_ex = num_pos_ex+sum(temp_region.Signals.(Signal_name).label == 1);
	  num_neg_ex = num_neg_ex+sum(temp_region.Signals.(Signal_name).label == -1);
        end
      end
      fprintf(1,'it took %fs to determine the true number of positive and negative examples\n', cputime-t);
      vec=subsample_policy([num_pos_ex num_neg_ex], Signal_name);
      max_num_pos = vec(1);
      max_num_neg = vec(2);
      reduction_factor_pos = max_num_pos/num_pos_ex;
      reduction_factor_neg = max_num_neg/num_neg_ex;
      num_pos = max_num_pos+max_num_neg;

      XT=zeros([lwin+rwin,num_pos],'int8');
      LT=zeros(1,num_pos);
      POS.pos=zeros(1,num_pos);
      POS.region_id=zeros(1,num_pos);

      if isfield(Signal.filter_label.(fields{ff}), 'USE_ALL')&&...
            Signal.filter_label.(fields{ff}).USE_ALL
        Signal.Conf_names = {} ;
      end ;
      
      t = cputime;
      fprintf(1,'starting with split %i\n',ss)
      for r=1:length(split_regs)
        if mod(r,100)==0,
          fprintf('region %i/%i\r', r, length(split_regs)) ;
        end ;
        
        if ~isfield(split_regs, 'Signals')
          temp_region = retrieve_signals(split_regs(r),fn_candsites,Signal_name,{'label',Signal.Conf_names{:}});
        end
        
        sig_field = getfield(temp_region.Signals,Signal_name);
        if isfield(Signal.filter_label.(fields{ff}),'use_label')
          sig_field.label = sig_field.(Signal.filter_label.(fields{ff}).use_label);
        end
        
        if ~isfield(split_regs,'seq') 		
          if isfield(Signal.filter_label.(fields{ff}), 'USE_ALL')&&...
                Signal.filter_label.(fields{ff}).USE_ALL
            if temp_region.strand == '+',
              loffset=0 ; % remember the offset for later
              tmp = max(1,temp_region.start-lwin) ;
              loffset = temp_region.start - tmp ;
              temp_region.start = tmp ;
              temp_region.stop = temp_region.stop + rwin ;
            else
              tmp = temp_region.start-rwin ;
              temp_region.start = tmp ;
              temp_region.stop = temp_region.stop + lwin ;
              loffset = lwin ;
            end ;
          end ;
          temp_region = load_sequence(temp_region);
        end
        seq = temp_region.seq;
        
        %---------------------
        %FILTER EXAMPLES
        %---------------------
        if ~isfield(Signal.filter_label.(fields{ff}), 'USE_ALL')||...
              ~Signal.filter_label.(fields{ff}).USE_ALL
          if ~isfield(Signal.filter_label.(fields{ff}),'conf_pos') ...
                && isfield(Signal.filter_label.(fields{ff}),'conf')
            error('fix this: filtering will not work')
          end
          idx = feval(Signal.filter_fct, temp_region, Signal, fields{ff},1);
        else
          sig_field.pos = sig_field.pos + loffset ;
          idx = find(sig_field.pos>lwin&sig_field.pos<length(seq)-rwin+1);
	  pos_idx = find(sig_field.label==1);
          if reduction_factor_pos<1
	    keep = randperm(length(pos_idx));
	    pos_idx = pos_idx(keep(1:floor(length(pos_idx)*reduction_factor_pos)));
	  end
	  neg_idx = find(sig_field.label==-1);
	  if reduction_factor_neg<1
	    keep = randperm(length(neg_idx));
	    neg_idx = neg_idx(keep(1:floor(length(neg_idx)*reduction_factor_neg)));
	  end
	  if reduction_factor_pos<1||reduction_factor_neg<1
            if ~isempty(pos_idx) && ~isempty(neg_idx),
              un=[pos_idx', neg_idx'] ;
            elseif isempty(pos_idx),
              un = neg_idx' ;
            else
              un = pos_idx' ;
            end ;
	    idx = intersect(idx, un);
	  end
        end
        if isempty(idx); continue; end
        
        %--------------------
        %CREATE XTs
        %--------------------
        XT_temp = zeros(lwin+rwin,length(idx),'int8');
        rid_temp= zeros(1,length(idx))+temp_region.id;
        for row=1:lwin+rwin
          XT_temp(row,:)= seq(sig_field.pos(idx)-lwin+row-1);
        end%loop over rows
        LT_temp= sig_field.label(idx)';
        pos_temp = sig_field.pos(idx)';
        assert(size(LT,2)==num_examples||LT(num_examples+1)==0)
        assert(num_examples==0||LT(num_examples)~=0)
        
        %[size(XT,2) num_examples+length(idx) ceil(size(XT,2)*r/length(split_regs))]
        if size(XT,2)<num_examples+length(idx), 
          warning('extending XT') ;
          POS.pos(ceil(size(XT,2)*1.2)) = 0 ;
          POS.region_id(ceil(size(XT,2)*1.2)) = 0 ;
          LT(ceil(size(XT,2)*1.2)) = 0 ;
          XT(1,ceil(size(XT,2)*1.2)) = 0 ;
        end ;
        
        XT(:, num_examples+1:num_examples+length(idx))= XT_temp;	
        LT(num_examples+1:num_examples+length(idx))= LT_temp;
        POS.pos(num_examples+1:num_examples+length(idx))= pos_temp;
        POS.region_id(num_examples+1:num_examples+length(idx))=rid_temp;
        num_examples = num_examples+length(idx);
      end%loop over blocks
      fprintf(1, 'time required for one split: %.2f\n',cputime-t)
      fprintf(1,'save file %s\n',filename)
      %=== remove zeros
      assert(all(LT(num_examples+1:end)==0))
      LT = LT(1:num_examples);
      XT = XT(:,1:num_examples);
      POS.pos = POS.pos(1:num_examples);
      POS.region_id = POS.region_id(1:num_examples);
      
      %----------------------------------------------- 
      % remove examples that appear more than once
      %----------------------------------------------- 
      if 0
        [tmp1 idx tmp2] = unique(XT','rows');
        idx = idx(randperm(length(idx)));
        XT = XT(:,idx);
        LT = LT(idx);
        POS.pos = POS.pos(idx);
        POS.region_id = POS.region_id(idx);
      end
      %-----------------------------------------------
      % exclude examples containing non ACGT-Symbols 
      %-----------------------------------------------
      %xtpos=XT~='A'&XT~='C'&XT~='G'&XT~='T';
      %save space
      xtpos=XT~='A';
      for s='CGT'
        xtpos=xtpos&XT~=s;
      end
      cols = find(sum(xtpos));
      clear xtpos
      if length(cols)>0
        warning(sprintf('%i examples containing non ACGT symbols excluded',length(cols)))
        XT(:,cols)=[];
        LT(cols)=[];
        POS.pos(cols)=[];
        POS.region_id(cols)=[];
      end
      %----------------------------------------------- 
      % save matrix in pieces into one file
      %----------------------------------------------- 
      assert(sum(LT==1)>0);
      fprintf( 'trying to save %s\n', filename );
      save_matrix(filename, XT, 'XT');
      %save(filename,'-append','LT','POS','PAR');
      save_append(filename, 1,'LT', LT, 'POS', POS, 'P', P);
      % save_append(filename, 1,'LT', LT, 'POS', POS, 'PAR', PAR);
    end%if ~fexist(filename)
   end%loop over splits
   
   stat = save_example_statistics(Signal, fn_filter_settings, ...
                                  fn_examples, fn_example_statistics, ...
                                  num_splits);
end%loop over filter_label fields


return

function [total_num_pos, num_pos] = retrieve_num_pos(regions,base_dir)
% num_pos = retrieve_num_pos(regions,base_dir)
%

num_pos = zeros(1,length(regions)) ;
for r=1:length(regions)
  f_name = sprintf('%scontig_%i%s',base_dir,regions(r).chr_num,regions(r).strand);
  d=dir([f_name '.pos']) ;
  if d(1).bytes==0,
    warning('empty pos file') ;
    num_pos(r) = 0 ;
  else
    num_pos(r) = get_num_pos(f_name, [regions(r).start; regions(r).stop]) ;
  end ;
end
total_num_pos = sum(num_pos) ;

return

function stat=save_example_statistics(Signal,fn_filter_settings,fn_examples,fn_example_statistics,num_splits)
% stat = save_example_statistics(Signal,fn_filter_settings,fn_examples,fn_example_statistics,num_splits)


pos = 0;
neg = 0;
for j=1:num_splits
  f = gen_example_filename(Signal,fn_filter_settings,fn_examples,1,j) ;
  l=load(f, 'LT');
  pos = pos + sum(l.LT==1);
  neg = neg + sum(l.LT==-1);
  stat.(sprintf('split_%i_num_pos',j))=sum(l.LT==1);
  stat.(sprintf('split_%i_num_neg',j))=sum(l.LT==-1);
  stat.(sprintf('split_%i_frac_pos',j))=sum(l.LT==1)/sum(l.LT==-1)*100;
end

stat.all_num_pos = pos;
stat.all_num_neg = neg;
stat.all_frac_pos=pos/neg*100;

assert(pos>0);
assert(neg>0);


fn = fn_example_statistics(1:end-4);
num = 1;
while fexist(sprintf('%s_%i.mat',fn,num))
  num=num+1;
end

save(sprintf('%s_%i.mat',fn,num), 'stat');

return

function save_append(fname__, append_flag, varargin)
% save_append(fname__, append_flag, varargin)
%
% octave compatible version of save -append

try,
  if append_flag
    L__=load(fname__) ;
  else
    L__=struct ;
  end ;
catch 
  L__=struct ;
end 

if length(varargin)>1,
  
  % assume varargin to have the following structure: varname1, var1, ...
  
  assert(mod(length(varargin),2)==0) ;
  for i=1:2:length(varargin),
    L__.(varargin{i}) = varargin{i+1} ;
  end ;
  
  f=fieldnames(L__) ;
  for i=1:length(f),
    eval(sprintf('%s=L__.%s;', f{i}, f{i})) ;
  end ;
  
  eval(sprintf('save -V7 %s %s', fname__, sprintf('%s ', f{:}))) ;
else
  
  % assume the second argument is a struct
  assert(length(varargin)==1) ;
  
  f = fieldnames(varargin{1}) ;
  for i=1:2:length(f),
    L__.(f{i}) = varargin{1}.(f{i}) ;
  end ;
  
  f=fieldnames(L__) ;
  for i=1:length(f),
    eval(sprintf('%s=L__.%s;', f{i}, f{i})) ;
  end ;
  
  eval(sprintf('save -V7 %s %s', fname__, sprintf('%s ', f{:}))) ;
  
end ;




return

function jobinfo = ms_gfwd_starter(PAR)

% ms_gfwd_starter(PAR)
%
% used for modelselection as well as simple training and evaluation 
% INPUT : struct PAR as generated by create_PAR
%         fields used: 
%         - Signal_name: string specifying the signal that is actually computed
%           (e.g. 'acc','don') 
% 
%         - Signals.(Signal_name) : struct containing all necessary information to
%           compute the signal,
% 
%         - SETs : struct with information about split and partitions, 
%   
%         - FN : struct containing all used filenames including: 
%           FN.input_sig.(Signal_name).fn_filter_settings; 
%           FN.input_sig.(Signal_name).fn_examples; 
%           FN.output_sig.(Signal_name).fn_models;
%           FN.output_sig.(Signal_name).fn_SVMs;
%           FN.output_sig.(Signal_name).fn_motifs;
%   
%         - tasks.signals struct specifying what will be done
%           fields include run_locally (default=1), run_all,
%  
%         - RPROC (only necessary when tasks.signals.run_locally=0)
  
  
fid = 1;%fopen(PAR_1.FN.fn_log ,'a+');
FN = PAR.FN;
if ~isempty(PAR.Signal_name)
  Signal_name = PAR.Signal_name;
  Signal = PAR.Signals.(Signal_name);
  input_fn = FN.input_sig.(Signal_name);
  output_fn = FN.output_sig.(Signal_name);
  tasks = PAR.tasks.signals;
else
  Signal_name = PAR.Content_name;
  Signal = PAR.Contents.(Signal_name);
  input_fn = FN.input_cont.(Signal_name);
  output_fn = FN.output_cont.(Signal_name);
  tasks = PAR.tasks.contents;
end
fn_filter_settings = input_fn.fn_filter_settings;
fn_examples = input_fn.fn_examples;


method = Signal.method;
SETs = PAR.SETs;

if ~isfield(tasks,'run_locally')
  tasks.run_locally = 1;
end
if ~isfield(tasks,'run_all')
  tasks.run_all = 0;
end 

if ~tasks.run_locally.train
  RPROC = PAR.RPROC;
  organism = PAR.organism.name;
  RPROC.options.identifier = sprintf('%s%s',Signal_name,upper(organism)) ;
  RPROC.options.ncpus = 1;
  %if isfield(RPROC.options, 'addpaths') && iscell(RPROC.options.addpaths)
  %  RPROC.options.addpaths = {RPROC.options.addpaths{:}, fileparts(which(sg))};
  %else
  %  RPROC.options.addpaths = {fileparts(which(sg))};
  %end
  %path
end  

fprintf(fid,'\nstarting ms_gfwd_starter \n');
fprintf(fid,'date: %s\n',date);
fprintf(fid,'\nSITE: %s\n',Signal_name);


%------------------------------------------------------------------------%
% write numeric kernelpars into field for model selection
%------------------------------------------------------------------------%

if isfield(method, 'kernels') && ~isempty(method.kernels{1})
  method.par_ms = collect_ms_parameters(method.kernels, method.par_ms);
end

%------------------------------------------------------------------------%
% initiate parameters
%------------------------------------------------------------------------%
Model_param = getfield(method,'par_ms');
Model_param = orderfields(Model_param) ;

[MODELS,param_names] = generate_all_models(orderfields(method.par_ms));
if ~tasks.modelsel
  fprintf('no model selection. taking first model;\n')
  MODELS = MODELS(:,1);
end

fprintf(fid,'number of models to be tested %s: %i\n',Signal_name,size(MODELS,2));
num_models_tested = size(MODELS,2);
RSV_eval = zeros(SETs.num_partitions,size(MODELS,2));
RFV_eval = zeros(SETs.num_partitions,size(MODELS,2));

%------------------------------------------------------------------------%
% check for saved model files and save models if non are found
%------------------------------------------------------------------------%
if ~fexist(output_fn.fn_models) && ~fexist([output_fn.fn_models '.mat']) 
  save(output_fn.fn_models,'MODELS','Model_param')
elseif (fexist(output_fn.fn_models) || fexist([output_fn.fn_models '.mat']) ) 
  LL = load(output_fn.fn_models);
  assert(isequal(LL.Model_param,Model_param))
  if ~isequal(LL.MODELS,MODELS)
    MODELS=append_new_models(MODELS, LL.MODELS);
    old=LL;
    save(output_fn.fn_models,'MODELS','old')
    warning('different other MODELS have already been trained, new models are appended')
  end
  clear LL old
end
num_not_eval = SETs.num_partitions*sum(num_models_tested) ;

fprintf(fid,'total number of jobs for training: %i\n',sum(SETs.partition_train_on)*sum(num_models_tested));
fprintf(fid,'total number of jobs for evaluation: %i\n',num_not_eval);


P.SETs = SETs;
P.Signal = Signal;
P.fn_filter_settings = fn_filter_settings;
P.fn_examples = fn_examples;
P.train_error_on = tasks.get_train_error;
P.pred_from_seq = 0;

P.use_preproc = 1;

num_crashed = 1;
num_crashed_1 = 0;
rep = 1;

P.check_overlap = 1;
while num_crashed + num_crashed_1 + num_not_eval>0 & rep<5
  q = 0 ; nq = 0 ;
  jobinfo = rproc_empty(0) ;
  jobinfo_1 = rproc_empty(0) ;
  num_not_eval = SETs.num_partitions*sum(num_models_tested) ;
  for p = 1:size(SETs.partitions,1) %% loop over partitions  
    fprintf(fid,'\n\n-----------\n');
    fprintf(fid,'PARTITION %i\n',p);
    fprintf(fid,'-----------\n');
    
    if tasks.run_all
      Train_data=[];
      Test_data=[];
    end
    
    P.partition = p;
    P.train_on = SETs.partition_train_on(p);
    P.SETs.which_train = find(ismember(SETs.partitions(p,:),SETs.info.train));
    P.SETs.which_val = find(ismember(SETs.partitions(p,:),SETs.info.eval));
    P.SETs.which_pred = find(ismember(SETs.partitions(p,:),SETs.info.pred));
    
    fn_save = [sprintf('%sms_set=[',output_fn.fn_SVMs),...
               sprintf('%i',P.SETs.which_train)];
    fn_save_pred  = [fn_save '_' sprintf('%i',P.SETs.which_val)];
    
    for num_models = 1:size(MODELS,2)
      %--------------------------------------------------------------%
      % assign parameters of model number num_models to the current parameter
      % structure
      %--------------------------------------------------------------%
      
      P.Signal.method = assign_model(method, MODELS, param_names,num_models); 
      
      %--------------------------------------------------------------%
      %% generate filename for SVMs and predictions
      %--------------------------------------------------------------%
      
      P.fn_SVMs = sprintf('%s]_model=%i.mat',fn_save,num_models) ;
      P.fn_eval = sprintf('%s]_model=%i.mat',fn_save_pred,num_models) ;
      
      %--------------------------------------------------------------%
      %%%%% TRAIN
      %--------------------------------------------------------------%
      
      %if ~fexist(P.fn_SVMs) & P.train_on
      %  fprintf(fid,'waiting for file %s...\n', P.fn_SVMs);
      %pause(1)
      %end      

      if ~fexist(P.fn_SVMs) & P.train_on
        q = q+1 ;
        fprintf(fid,'\n start training on set ');
        fprintf(fid,'%i',P.SETs.which_train);
        fprintf(fid,'\n')
        
        train_fct = 'load_and_train';
        
        if tasks.run_all
          Data = load_data(Signal, fn_filter_settings, fn_examples, P.SETs.which_train, 'train', SETs.train_subsample_neg, SETs.train_subsample_pos);
          Train_data.(sprintf('partition_%i',p))= Data ; clear Data 
          feval(train_fct,P,Train_data.(sprintf('partition_%i',p)));
        elseif tasks.run_locally.train==0,
          fprintf('creating job (training) %i/%i  \r', q,sum(SETs.partition_train_on)*sum(num_models_tested)) ;
          RPROC.options.identifier = sprintf('Tr%i_%s%s',num_models, Signal_name, upper(organism)) ;

          num_data = count_data(Signal, fn_filter_settings, fn_examples, P.SETs.which_train, 'train', SETs.train_subsample_neg, SETs.train_subsample_pos);

          [mem_req, time_req, opts] = rproc_memtime_policy(train_fct,  num_data, RPROC.options) ;

          jobinfo(q) = rproc_create(train_fct, P, mem_req, opts, time_req) ;
        else
          feval(train_fct,P);
        end 
      	
      %--------------------------------------------------------------%
      %%%%% EVAL 
      %--------------------------------------------------------------%
      elseif fexist(P.fn_SVMs) 
        fprintf(fid,'\nSVM already trained on set ');
        fprintf(fid,'%i',P.SETs.which_train);
	fprintf(fid,'\n')
       
	if ~fexist(P.fn_eval)
          fprintf(fid,'waiting for file %s...\n', P.fn_eval);
	  pause(1)
        end
 
        if ~fexist(P.fn_eval) 
          P.eval_on = 1;
          fprintf(fid,'\n\nstart evaluation on set ');
          fprintf(fid,'%i',P.SETs.which_val);
          fprintf(fid,'\n')
          nq = nq+1;
          
          if isfield(method,'pred_fct')&& strcmp(method.pred_fct,'load_content_and_predict')
            pred_fct = 'load_content_and_predict';
          else
            pred_fct = 'load_and_predict';
          end
  
          if tasks.run_all
            Data = load_data(Signal,fn_filter_settings,fn_examples, P.SETs.which_val, 'test',SETs.test_subsample_neg,SETs.test_subsample_pos);
            Test_data.(sprintf('partition_%i',p)) = Data; clear Data;
            feval(pred_fct,P,Test_data.(sprintf('partition_%i',p)));
          elseif tasks.run_locally.eval==0,
            fprintf('creating job (evaluation) %i/%i  \r', nq,num_not_eval) ;
            RPROC.options.identifier = sprintf('Ev%i_%s%s',num_models,Signal_name,upper(organism)) ;

            num_data = count_data(Signal,fn_filter_settings,fn_examples, P.SETs.which_val, 'test', SETs.test_subsample_neg, SETs.test_subsample_pos);

            [mem_req, time_req, opts] = rproc_memtime_policy(pred_fct,  num_data, RPROC.options) ;

            jobinfo_1(nq) = rproc_create(pred_fct, P, mem_req, opts, time_req) ;
          else
            feval(pred_fct,P);
          end
        
        %--------------------------------------------------------------%
        %%%%% Collext Results 
        %--------------------------------------------------------------%
        elseif fexist(P.fn_eval) 
          fprintf(fid,'\n model already evaluated on set ');
          fprintf(fid,'%i',P.SETs.which_val);
          
          L = load(P.fn_eval, 'RSV', 'RFV');%,'fn_SVMs') ;
          num_not_eval = num_not_eval-1;
              
          assert(isnumeric(L.RFV) && prod(size(L.RFV))==1) ;
          assert(isnumeric(L.RSV) && prod(size(L.RSV))==1) ;
          RFV_eval(p,num_models)= L.RFV ;
          RSV_eval(p,num_models)= L.RSV ;

        else
          fprintf(fid,'\n only training, no evaluation ');
          num_not_eval = num_not_eval-1;
        end
      end   %% if trained eval
    end   %% loop over num_models      
    
    fprintf(fid,'partition %i: DONE\n', P.partition);
  end  %% loop over partitions
  fprintf(fid,'experiment %s: DONE\n',Signal_name );
      
  
  %--------------------------------------------------------------%
  %%%%% RPROC STUFF
  %--------------------------------------------------------------%

  %if (~isempty(jobinfo) || ~isempty(jobinfo_1)) && isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs,
  %  jobinfo = [jobinfo jobinfo_1] ;
  %  return ;
  %end ;

  if ~isempty(jobinfo),
    for ii=1:length(jobinfo), jobinfo(ii) = rproc_resubmit(jobinfo(ii)); end ;
  else
    num_crashed = 0 ;
  end
  if ~isempty(jobinfo_1),
    %[jobinfo_1, meta_jobinfo_1] = rproc_submit_batch(jobinfo_1, RPROC.exm_per_batch) ;
    for ii=1:length(jobinfo_1), jobinfo_1(ii) = rproc_resubmit(jobinfo_1(ii)); end ;
  else
    num_crashed_1 = 0;
  end ;
  if ~isempty(jobinfo),
    [jobinfo,num_crashed] = rproc_wait(jobinfo, 60, 1, -1);
    if num_crashed==length(jobinfo)
      fprintf('all jobs seem to be crashed. Waiting 2min.\n') ;
      pause(120) ;
      [jobinfo,num_crashed] = rproc_wait(jobinfo, 60, 1, -1);
      if num_crashed==length(jobinfo),
        error('all jobs crashed')
      end ;
    end
    if num_crashed == 0,
      rproc_cleanup(jobinfo) ;
    end ;
  end
  if ~isempty(jobinfo_1),
    [jobinfo_1,num_crashed_1] = rproc_wait(jobinfo_1, 60, 1, -1);
    if num_crashed_1==length(jobinfo_1)
      fprintf('all jobs seem to be crashed. Waiting 2min.\n') ;
      pause(120) ;
      [jobinfo,num_crashed] = rproc_wait(jobinfo, 60, 1,-1);
      if num_crashed==length(jobinfo),
        error('all jobs crashed') ;
      end ;
    end
    if num_crashed_1 == 0,
      rproc_cleanup(jobinfo_1) ;
    end ;
  end ;
  
  %--------------------------------------------------------------%
  %%%%% FINAL STUFF
  %--------------------------------------------------------------%
  
  try
    L=load(output_fn.fn_models, 'RSV_eval', 'RFV_eval') ;
  catch
    L=struct ;
  end ;
  if ~isfield(L, 'RSV_eval') || ~isfield(L, 'RFV_eval') || ~isequal(L.RSV_eval, RSV_eval) || ~isequal(L.RFV_eval, RFV_eval),
    %save(output_fn.fn_models,'-append','RSV_eval','RFV_eval')
    save_append(output_fn.fn_models, 1,'RSV_eval',RSV_eval, 'RFV_eval',RFV_eval)
    % save_append(output_fn.fn_models, 1,'RSV_eval',RSV_eval, 'RFV_eval',RFV_eval)
  end 
  if num_crashed+num_crashed_1>0
    rep = rep+1;
  end
end  %% loop over crashed jobs

%fclose(fid)




return

function par_ms= collect_ms_parameters(kernels, par_ms)
%par_ms= collect_model_parameters(kernels, par_ms)

nofKernels = length(kernels);
assert(nofKernels>=1)
for k = 1:nofKernels
  kernel = kernels{k};
  for f =fieldnames(kernel)'
    if isnumeric(kernel.(f{1}))&~isempty(kernel.(f{1}))
      par_ms.(sprintf('kernel_%i_%s',k,f{1})) = kernel.(f{1});
    end
  end
end


return

function save_matrix(filen, M, varname, split_size)
%save_matrix(filen, M, varname, split_size)
%split matrix into parts with size of about split_size 
%bytes (1e+8 default) and save them in one file

if nargin<4
  split_size=1e+8;
end

info = whos('M');
num_splits = ceil(info.bytes/split_size);
num_cols = ceil(size(M, 2)/num_splits);

  
  meta.size=size(M);
  meta.num_splits = num_splits;
  mata.name = varname;
  if fexist(filen)
    save_append(filen, 1, 'meta', meta);
  else
    save(filen, 'meta');
  end
  for j=1:num_splits-1
    %eval(sprintf('%s_split_%i = M(:, 1:num_cols);',varname, j));
    var_split__ = M(:, 1:num_cols);
    M(:, 1:num_cols)=[];
    %save(filen, '-append', sprintf('%s_split_%i',varname,j));
    save_append(filen, 1, sprintf('%s_split_%i',varname,j), var_split__);
    clear var_split__
  end
  if isempty(j), j=0; end;
  var_split__ = M; 
  save_append(filen, 1, sprintf('%s_split_%i',varname,j+1), var_split__);

return

function var=load_matrix(filename, varname)
% var=load_matrix(filename, varname)

load(filename, 'meta')
if ~exist('meta')
  load(filename, varname);
  eval(sprintf('var = %s;',varname))
  return
end

var = [] ;
col = 1;
for j=1:meta.num_splits
  %if col>1
  %  assert(~all(var(:,col-1)==-1* ones(meta.size(1),1)))
  %end
  %assert(all(var(:,col)==-1*ones(meta.size(1),1)))
   
  load(filename, sprintf('%s_split_%i',varname,j));
  eval(sprintf('temp = %s_split_%i;',varname,j));

  if j==1,
    if isnumeric(temp)
      var = zeros(meta.size(1), meta.size(2), class(temp));
    else
      var = zeros(meta.size(1), meta.size(2),'uint8');
      var = char(var);
    end
  end ;

  var(:,col:col + size(temp, 2)-1)=temp;
  col = col + size(temp, 2);
end
%assert(~all(var(:,end)==-1* zeros(meta.size(1),1)))

return

function method = assign_model(method, models, par_names,  model_number)

%function method = assign_model(method, models, par_names, model_number)  

  
for num_param = 1:size(models,1)
  method.par_ms.(par_names{num_param}) = models(num_param,model_number) ;
end

if isfield(method, 'kernels') && ~isempty(method.kernels{1})
  nofKernels = length(method.kernels);
  for k =1:nofKernels
    kernel = method.kernels{k};
    for f =fieldnames(kernel)'
      if isnumeric(kernel.(f{1}))&~isempty(kernel.(f{1}))
        kernel.(f{1})= method.par_ms.(sprintf('kernel_%i_%s',k,f{1}));
      end
    end
    method.kernels{k}=kernel;
  end
end

return

function x = load_and_train(P, Train_Data)

% load_and_train(P)
%
% INPUT: P with fields:   
%         - fn_set,             file name of sets to be loaded 
%         - SETs.which_svm,     specifies the sets on which training is
%                               performed (e.g [1 2 3])
%         - SETs.train_subsample_neg  number between 0-1: for subsampling of negative examples, 
%                                0: setting label of examples labeled with 0 to -1   
%                               -1: removing examples labeled with 0 
%                               default(1) no subsampling 
%         - SETs.train_subsample_pos  number between 0-1: for subsampling of positive examples,                             
%                               default(1) no subsampling 
%         - par,                window around consensus to be used  
%         - use_cons            specifies if conservation is to be used
%         - fn_SVMs             file nmame to save results  
%  
%   
% OUTPUT: Train_Data with fields:
%  (XT,LT,SC,pos,reg,str,P)
%  saved to file P.fn_SVMs
  
%if( ~P.method.submit_batch )
%  paths;
%end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% load data of training set
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SETs = P.SETs;
Signal = P.Signal;
fn_filter_settings = P.fn_filter_settings;
fn_examples = P.fn_examples;

which_train = SETs.which_train;
train_fct = Signal.method.train_fct;

if ~isfield(SETs,'train_subsample_pos') && isfield(SETs,'subsample_pos')
  subsample_pos = SETs.subsample_pos ;
  error('warning: please rename field from subsample_pos to train_subsample_pos') ;
else
  subsample_pos = SETs.train_subsample_pos ;
end

if ~isfield(SETs,'train_subsample_neg') && isfield(SETs,'subsample_neg')
  subsample_neg = subsample_neg ;
  error('warning: please rename field from subsample_neg to train_subsample_neg') ;
else
  subsample_neg = SETs.train_subsample_neg ;
end

if ~exist('Train_Data')
  Train_Data = feval(Signal.load_fct,Signal,fn_filter_settings,fn_examples, which_train, 'train',subsample_neg, subsample_pos);
end
Train_Data.P = P;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% TRAIN 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

t = cputime;
feval(train_fct,Train_Data);
fprintf('\ntraining took %.2f min\n',(cputime-t)/60);
% if PAR.method.name=='SVM'
%   tssTrain(Train_Data)
% elseif PARn.method.name=='PWM'
%   hmm_train(Train_Data)
% else 
%   error('unknown method');
% end

% dummy return argument
x=0;

return

function [XT,LT,POS,SC] = subsample(XT, LT, POS, SC, subsample_neg, subsample_pos)
% [XT,LT,POS,SC] = subsample(XT, LT, POS, SC, subsample_neg, subsample_pos)

if subsample_neg<1 & subsample_neg>0 & subsample_pos<=1 & subsample_pos>0 % & ~isempty(LT) 

  fprintf('subsampling %i %% of negative examples \n',subsample_neg*100);
  fprintf('subsampling %i %% of positive examples \n',subsample_pos*100);
  posidx = find(LT==1);
  negidx = find(LT==-1);
  rand('seed', pi) ;
  idx2 = randperm(length(negidx));
  idx3 = randperm(length(posidx));
  num_neg = round(subsample_neg*sum(LT==-1));
  num_pos = round(subsample_pos*sum(LT==1));
  
  XT = [XT(:,posidx(idx3(1:num_pos))) XT(:,negidx(idx2(1:num_neg)))] ;
  
  if ~isempty(SC)
    SC = [SC(:,posidx(idx3(1:num_pos))) SC(:,negidx(idx2(1:num_neg)))] ;
  end
  LT = [LT(posidx(idx3(1:num_pos))) LT(negidx(idx2(1:num_neg)))] ; 
  POS.pos = [POS.pos(posidx(idx3(1:num_pos))) POS.pos(negidx(idx2(1:num_neg)))] ; 
  POS.region_id = [POS.region_id(posidx(idx3(1:num_pos))) POS.region_id(negidx(idx2(1:num_neg)))] ; 
  fprintf('%i examples remaining (%.2f %% positives)\n',length(LT),sum(LT==1)/length(LT)*100);

end

return

function [ alphas, b, betas, PAR1 ] = svm_train(Data);

P = Data.P;

method  = P.Signal.method;
fn_SVMs = P.fn_SVMs;
pred_from_seq = P.pred_from_seq;
use_preproc = P.use_preproc;


%------------------------------------------------------------------------------%
% SVM already trained?
%------------------------------------------------------------------------------%

%if( ~isfield(PAR,'conservWeighting') )
method.conservWeighting = 0;
%end;
if( ~exist('testDataPar','var') )
  testDataPar = [];
end;


%------------------------------------------------------------------------------%
% train SVM
%------------------------------------------------------------------------------%


if( method.conservWeighting == 0 )
  Data.W = [];
else
  Data.W = res.W;
  assert( ~isempty(W) );
end;

% keyboard
init_svm_and_kernels(method,pred_from_seq);


if isequal(method.feature_type,'double') && method.normalize_features
  idx_rm = find(std(Data.XT')<1e-8);
  Data.XT(idx_rm,:)  = [];
  mean_tr = mean(Data.XT');
  Data.XT = Data.XT - repmat(mean_tr,size(Data.XT,2),1)';
  std_tr = std(Data.XT');
  Data.XT = Data.XT./repmat(std_tr,size(Data.XT,2),1)';
end

setFeatures(method, Data,'TRAIN',pred_from_seq,use_preproc);

m = length( Data.LT );
% PAR1 = P;
% PAR1.POS = Data.Pos;
% PAR1.lenTrain = size( Data.XT, 1 );

sg_ver=sg('get_version');

% === train
nofKernels = length(method.kernels);
t0 = cputime;
sg('set_kernel_optimization_type', 'FASTBUTMEMHUNGRY' );
%sg( 'set_kernel_optimization_type', 'SLOWBUTMEMEFFICIENT' );
if( method.use_mkl )
  sg('use_mkl', true );
  sg('mkl_parameters', 1e-3, 0 );
  betas = sg( 'get_subkernel_weights' );
  assert( size(betas) == [nofKernels,1] );
end;
sg('svm_train' );
fprintf( '\n' );
t = cputime - t0;

%------------------------------------------------------------------------------%
% get alphas and betas
%------------------------------------------------------------------------------%

% === alphas
[ b, alphas ] = sg( 'get_svm' );


% === betas (kernel weights)
if(method.use_mkl)
  betas = sg( 'get_subkernel_weights' );
  assert( size(betas) == [nofKernels,1] );
  betas
else
  betas = ones( nofKernels, 1 );
end;


%------------------------------------------------------------------------------%
% save SVM
%------------------------------------------------------------------------------%
Train.alphas = alphas;
Train.b = b;
Train.PAR = P;
Train.POS = Data.Pos;
sup_vec_idx = alphas(:,2)+1;
Train.XT = Data.XT(:,sup_vec_idx);
Train.LT = Data.LT(sup_vec_idx);

w = zeros(size(Train.XT,1),1);
for a=1:length(Train.LT)
  w = w+Train.alphas(a,1).*Train.XT(:,a);
end
Train.w = w;

Train.POS.pos = Data.Pos.pos(sup_vec_idx);
Train.POS.region_id = Data.Pos.region_id(sup_vec_idx);

if isequal(method.feature_type,'double') && method.normalize_features
  Train.idx_rm = idx_rm
  Train.mean = mean_tr;
  Train.std = std_tr;
end

clear P Data
sg( 'clean_features', 'TRAIN' );
sg( 'clean_features', 'TEST' );

Train.time=t;
Train.sg_ver = sg_ver ; 
fprintf('save: %s',fn_SVMs)
save(fn_SVMs, '-V7', 'Train')






return

function [models,param_names] = generate_all_models(Model_param)
% models = generate_all_models(Model_param) 
  
param_names = fieldnames(Model_param); 
if isempty(param_names)
  models =[];param_names=[];
  return
end

for j = 1:length(param_names)
  param_nums(j) = length(getfield(Model_param,param_names{j}));
end
models = zeros(length(param_names),prod(param_nums));
for j = 1:length(param_names)
  par_tmp  = getfield(Model_param,param_names{j});
  par1 = [];
  for k = 1:length(par_tmp)
    par1 = [par1 repmat(k,1,prod(param_nums(j+1:end)))];
  end
  models(j,:) = par_tmp(repmat(par1,1,prod(param_nums(1:j-1))));
end

if prod(param_nums)<1
  assert(length(unique(models','rows'))==prod(param_nums))
else
  assert(~isempty(models))
end

return

function s = calcrocscore(oute,LTE)
% rocscore = calcrocscore(output,LT)
%
% computes the area under the ROC curve

[a,b]=calcroc(oute,LTE) ;

s=0 ;
for i=1:length(a)-1,
  s = s + (a(i)-a(i+1))*(b(i)+b(i+1))/2 ;
end ;


return

function [false_alarms,hits] = calcroc(output,LTE)

assert(all(size(output)==size(LTE))) ;

[ld,idx] = sort(output); 

i=0 ;

hits=1-cumsum(LTE(idx)>0)/sum(LTE > 0) ;
false_alarms=1-cumsum(LTE(idx)<0)/sum(LTE < 0) ;

% Da die Kurve nie alle Negativen als positiven zaehlt, hat
% Falsealarms nie 
% den Wert 1. Cumsum zaehlt das erste Element als negatives, diese
% fehlt bei
%  den False_alarms.
% Daher wird vorne an den Vektor noch eine 1 gehaengt

hits = [1 hits ];
false_alarms = [1 false_alarms];

if nargout<2,
  false_alarms = [false_alarms;hits];
end ;

return

function [score,tp,tdr]=calcrfcscore(output,LT)
% score=calcrfcscore(output,LT)
map=(LT==1) ;
[s,idx]=sort(-output) ; s=-s ;
map=map(idx) ;

tp=cumsum(map)/sum(LT==1) ;
tdr=cumsum(map)./[1:length(map)] ;

t=tp(2:end)-tp(1:end-1) ;
score = sum(0.5*[tdr(1:end-1)+tdr(2:end)].*t) ;



return

function ret = predict_testplif_geterror(PAR)

  
SETs = PAR.SETs;
RPROC = PAR.RPROC;
FN = PAR.FN;

if ~isempty(PAR.Signal_name)
  Signal_name = PAR.Signal_name;
  Signal = PAR.Signals.(Signal_name);
  output_fn = FN.output_sig.(Signal_name);
  tasks = PAR.tasks.signals;
else
  Signal_name = PAR.Content_name;
  Signal = PAR.Contents.(Signal_name);
  output_fn = FN.output_cont.(Signal_name);
  tasks = PAR.tasks.contents;
end


fid = 1;%fopen(PAR.FN.fn_log ,'a+');
fprintf(fid,'\n\n starting test on left out set \n');
fprintf(fid,'date: %s\n',date);


%PAR = setfield(PAR,PAR.method,Model_param);


load(output_fn.fn_models,'RSV_eval','RFV_eval','MODELS','Model_param')
param_names = fieldnames(Model_param); 

fprintf(fid,'\n\n-----------\n');

RSV_test = zeros(SETs.num_partitions,1);
RFV_test = zeros(SETs.num_partitions,1);
best_models = zeros(SETs.num_partitions,size(MODELS,1));

options = RPROC.options;
options.identifier = [ Signal_name '_' Signal.method.name];

jobinfo = rproc_empty(0) ;
jobinfo_1 = rproc_empty(0) ;  


for p = 1:size(SETs.partitions,1) %% loop over partitions  
  fprintf(fid,'\n\n-----------\n');
  fprintf(fid,'PARTITION %i\n',p);
  fprintf(fid,'-----------\n');
  fprintf(fid,'\nSITE: %s\n',Signal_name);
  
  which_train = find(ismember(SETs.partitions(p,:),SETs.info.train));
  which_val = find(ismember(SETs.partitions(p,:),SETs.info.eval));
  which_pred = find(ismember(SETs.partitions(p,:),SETs.info.pred));
  which_plif = find(ismember(SETs.partitions(p,:),SETs.info.plif));

  fn_best_SVM = sprintf('%s_best_partition=%i.mat', output_fn.fn_SVMs,p) ;
  fn_SVM_train = [sprintf('%sms_set=[',output_fn.fn_SVMs),...
                  sprintf('%i',which_train)];
  fn_SVM_eval  = [fn_SVM_train '_' sprintf('%i',which_val)];
  fn_pred = sprintf('%spartition=%i.mat',output_fn.fn_SVMs,p);
  fn_plifs = sprintf('%spartition=%i_plifs.mat',output_fn.fn_SVMs,p);
  
  %--------------------------------------------------------------%
  %%% select for each partition best model:
  %--------------------------------------------------------------%
  if ~all(RFV_eval(p,:)~=0) 
    warning('some models not evaluated')
  end
  [tmp,idx] = max(RFV_eval(p,:));
  fn_SVM_eval = sprintf('%s]_model=%i.mat',fn_SVM_eval,idx);
  fn_SVM_train = sprintf('%s]_model=%i.mat',fn_SVM_train,idx)
  load(fn_SVM_train,'Train');
  assert(isequal(Train.PAR.SETs.which_train,which_train))
  best_models(p,:) = MODELS(:,idx) ;
  for ii = 1:length(param_names)
    assert(isequal(getfield(Train.PAR.Signal.method.par_ms,param_names{ii}), MODELS(ii,idx)))
  end
  
  fprintf(fid,'\nbest model (%s,partition=%i): ',Signal_name,p);
  fprintf('\nbest model (%s,partition=%i): \n',Signal_name,p);
  for num_param = 1:size(MODELS,1)
    fprintf(1,'%s = %d; \t',param_names{num_param},getfield(Train.PAR.Signal.method.par_ms,param_names{num_param}));
    fprintf(fid,'%s = %d; \t',param_names{num_param},getfield(Train.PAR.Signal.method.par_ms,param_names{num_param}));
  end
  L = load(fn_SVM_eval) ;
  if ~nan(L.RFV)
    assert(L.RFV == RFV_eval(p,idx)) ;
    assert(L.RSV == RSV_eval(p,idx)) ;
  end
  fprintf(fid,'\nvalidation error: RFV:%.4f \t RSV:%.4f\n',L.RFV,L.RSV);
  %fprintf('\nvalidation error: RFV:%.4f \t RSV:%.4f\n',L.RFV,L.RSV);
  
  %--------------------------------------------------------------%
  %%%%% Predict on TEST SET
  %--------------------------------------------------------------%  
  
  Train.PAR.partition = p;
  Train.PAR.SETs.which_val = which_val;
  Train.PAR.SETs.which_pred = which_pred ;
  Train.PAR.SETs.which_plif = which_plif ;
  Train.PAR.fn_plifs = fn_plifs;
  Train.PAR.fn_SVM_eval = fn_SVM_eval;
  Train.PAR.fn_pred = fn_pred;
  
  PAR_new = Train.PAR;
  PAR_new.eval_on = 0;
  PAR_new.Signal.filter_label.test = Signal.filter_label.test;

  if ~fexist(PAR_new.fn_pred),
    predict = 1;
  else
    pr=load(PAR_new.fn_pred);
    if ~isequal(PAR_new.Signal.method.par_ms, pr.Train.PAR.Signal.method.par_ms);
      predict=1;
      fprintf(1,'prediction file is out of date; it will be overwritten now\n')
    else
      predict=0;
      fprintf(1,'prediction file exists and was created with par_ms of best model\n')
    end
  end
  if predict
    if ~isfield(tasks, 'run_locally') || tasks.run_locally.predict==0,

      [mem_req, time_req, opts] = rproc_memtime_policy('load_and_predict',  0, options) ;%RPROC.MEMREQ ;

      jobinfo(p) = rproc('load_and_predict',PAR_new, mem_req, opts, time_req);
      % load_and_predict(PAR_new)
    else
      %D=load_data(PAR_new,which_pred,'test',PAR.SETs.test_subsample_neg,PAR.SETs.test_subsample_pos); 
      load_and_predict(PAR_new);%, D);
      %clear D
    end
  end ;
  %--------------------------------------------------------------%
  %%%%% Predict on PLIF SET
  %--------------------------------------------------------------%  
  
  PAR_new.eval_on = 0;
  PAR_new.plif_on = 1;
  
  
  
  if ~fexist(PAR_new.fn_plifs)
    predict = 1;
  else
    pr=load(PAR_new.fn_plifs);
    if ~isequal(PAR_new.Signal.method.par_ms, pr.Train.PAR.Signal.method.par_ms);
      predict=1;
      fprintf(1,'plif file is out of date; it will be overwritten now')
    else
      predict=0;
      fprintf(1,'plif file exists and was created with par_ms of best model')
    end
  end
  if predict
    if ~isfield(tasks, 'run_locally') || tasks.run_locally.predict==0,
      [mem_req, time_req, opts] = rproc_memtime_policy('load_and_predict',  0, options) ;

      jobinfo_1(p) = rproc('load_and_predict', PAR_new, mem_req, opts, time_req);
      % load_and_predict(PAR_new)
    else
      %D=load_data(PAR,which_plif,'test',PAR.SETs.test_subsample_neg,PAR.SETs.test_subsample_pos); 
      load_and_predict(PAR_new);%, D);
      %clear D
    end
  end ;
end  %% loop over partitions
if ~isempty(jobinfo),
  [jobinfo,num_crashed] = rproc_wait(jobinfo, 20, 1, -1) ;
end ;
if ~isempty(jobinfo),
  [jobinfo_1,num_crashed] = rproc_wait(jobinfo, 20, 1, -1) ;
end ;

if ~isempty(jobinfo),
  unix('sync')
  pause(30)
  unix('sync')
end ;

for p = 1:size(SETs.partitions,1) %% loop over partitions 
  fprintf(fid,'\n\n-----------\n');
  fprintf(fid,'PARTITION %i\n',p);
  fprintf(fid,'-----------\n');
  fprintf(fid,'\nSITE: %s\n',Signal_name);
    
  fn_best_SVM = sprintf('%sbest_partition=%i.mat', output_fn.fn_SVMs,p) ;
  fn_pred = sprintf('%spartition=%i.mat',output_fn.fn_SVMs,p);
  L = load(fn_pred,'RSV','RFV','svmFileName','PAR') ;
  load(L.svmFileName,'Train')
  Train.PAR = L.PAR;
  if ~fexist(fn_best_SVM)
    %save(fn_best_SVM,'-struct', 'L')
    %save(fn_best_SVM, '-append' ,  'Train')
    save_append(fn_best_SVM, 0, L)
    save_append(fn_best_SVM, 2, 'Train', Train)
  else
    old = load(fn_best_SVM,'Train')
    if isequal(old.Train,Train)
      fprintf('file %s already exists\n',fn_best_SVM),
      fprintf('Train identical\n')
    else
      fprintf('file %s already exists\n',fn_best_SVM),
      error('Train not identical\n')
    end
  end
  fprintf(fid,'\ntest error: RFV:%.4f \t RSV:%.4f\n',L.RFV,L.RSV);
  fprintf('\ntest error: RFV:%.4f \t RSV:%.4f\n',L.RFV,L.RSV);
  
  RFV_test(p)= L.RFV ;
  RSV_test(p)= L.RSV ;
  
  clear PAR_new
end  %% loop over partitions

L=load(output_fn.fn_models, 'best_models','Model_param','RSV_test','RFV_test') ;
if ~isfield(L, 'best_models') || ~isfield(L, 'Model_param') || ~isfield(L, 'RSV_test') || ~isfield(L, 'RFV_test') || ...
      ~isequal(L.best_models, best_models) || ~isequal(L.Model_param, Model_param) || ~isequal(L.RSV_test, RSV_test) || ~isequal(L.RFV_test, RFV_test)
  %save(output_fn.fn_models,'-append','best_models','Model_param','RSV_test','RFV_test')
  save_append(output_fn.fn_models, 1, 'best_models', best_models, ...
              'Model_param', Model_param, 'RSV_test', RSV_test, ...
              'RFV_test', RFV_test)
end ;

fprintf(fid,'\n%s\n',Signal_name);
fprintf(fid,'RSV (mean and std over %i partitions): %.4f +/- %.4f\n',length(RSV_test),mean(RSV_test),std(RSV_test));
fprintf(fid,'RFV (mean and std over %i partitions): %.4f +/- %.4f\n',length(RSV_test),mean(RFV_test),std(RFV_test));
ret = [];
%fclose(fid)





return

function calc_plifs(PAR_1)
fid=1;
q=0;

fn_SVMs = PAR_1.FN.output_sig.(PAR_1.Signal_name).fn_SVMs;
method = PAR_1.Signals.(PAR_1.Signal_name).method;
num_partitions = PAR_1.SETs.num_partitions;

for p = 1:num_partitions
  fprintf(fid,'\n\n-----------\n');
  fprintf(fid,'PARTITION %i\n',p);
  fprintf(fid,'-----------\n');
  fn_SVM = sprintf('%sbest_partition=%i.mat', fn_SVMs,p) ;

  %--------------------------------------------------------------------------------%
  % load parameters from SVM file
  %--------------------------------------------------------------------------------%
  load(fn_SVM,'Train');
  PAR = Train.PAR;
  PAR.fn_SVMs=fn_SVM;
  clear Train
  
  %--------------------------------------------------------------------------------%
  % learn PLIFS on evaluation set and convert output on test set
  %--------------------------------------------------------------------------------%
  PAR.fn_pred = sprintf('%spartition=%i.mat',fn_SVMs,p);
  PAR.fn_plifs = sprintf('%spartition=%i_plifs.mat',fn_SVMs,p);
  assert(fexist(PAR.fn_pred)==1)
  assert(fexist(PAR.fn_plifs)==1)
  fprintf(fid,'\n already predicted on set ');
  fprintf(fid,'%i',PAR.SETs.which_pred);
        
  %--------------------------------------------------------------------------------%
  %%% learn PLIFS on evaluation set
  %--------------------------------------------------------------------------------%
  clear  prob prob_cum limits Conf Conf_cum
  fprintf('\nload fn_plif\n')
  load(PAR.fn_plifs,'LT', 'output');
  if ~all(ismember({'Conf','Conf_cum'},who_file(PAR.fn_plifs)))
    if ~all(ismember({'prob','prob_cum', 'limits'},who_file(PAR.fn_SVMs)))
      disp('learn PLIFs')
      assert(length(output)==length(LT))
      [prob, prob_cum,limits] = learn_plifs(output,LT,method.plif.bins,1,method.plif.min_incr );
      %save(PAR.fn_SVMs,'-append','prob', 'prob_cum','limits');
      save_append(PAR.fn_SVMs, 1, 'prob', prob, 'prob_cum', prob_cum, ...
                  'limits', limits);
    else
      load(PAR.fn_SVMs,'prob', 'prob_cum','limits');
    end
    disp('Convert SVM Output to Confidence on fn_plif')
    [Conf, Conf_cum] = Out2Conf(output,prob, prob_cum,limits);
    %save(PAR.fn_plifs,'-append','Conf','Conf_cum');
    save_append(PAR.fn_plifs,1, 'Conf', Conf, 'Conf_cum', Conf_cum);
    clear output Conf Conf_cum LT
  end
  %--------------------------------------------------------------------------------%
  %%% Convert SVM Outputs on test set 
  %--------------------------------------------------------------------------------%
  load(PAR.fn_pred,'output');
  if ~all(ismember({'Conf','Conf_cum'},who_file(PAR.fn_pred)))
    disp('Convert SVM Output to Confidence on fn_pred')
    [Conf, Conf_cum] = Out2Conf(output,prob, prob_cum,limits);
    %save(PAR.fn_pred,'-append','Conf','Conf_cum');
    save_append(PAR.fn_pred, 1,'Conf', Conf, 'Conf_cum', Conf_cum);
  end
end%loop over partitions
% [jobinfo,num_crashed] = rproc_wait(jobinfo,20,1,0);

return

function  [prob, prob_cum,limits,prob_unsort,prob_cum_unsort] = learn_plifs(Out,L,bins,do_balancing,min_inc);
% [prob, prob_cum,limits,prob_unsort,prob_cum_unsort] = learn_plifs_qp(Output,Labels,numberofbins,do_balancing,min_inc);
% 
% learn piecewise linear functions (PLIF), with which outputs of an SVM
% can be transformed to probability and cumulative
% probability values
% 
% INPUT    Out                vector of SVM Outputs on an evaluation set
%
%          L                  vector of corresponding labels (-1,1)  
%
%          bins               number of bins used for interpolation (default 50)
%                             (limits are choosen such that the number of
%                             examples in every bin is similar)  
%
%          do_balancing       should be set to 1 if negative class is
%                             much bigger than positive class,
%                             (negatives are downsampled for
%                             determining the bins) 
%
%          min_inc            probabilities have to be monotonically
%                             increasing (higher SVM output value
%                             corresponds to higher probability of true positive) 
%                             (default 1e-5)
%
%
% OUTPUT   prob               vector of length 'bins'. prob_cum(i) is the
%                             estimated probability of examples with 
%                             Output = limits(i) to be true positive
% 
%          prob_cum           vector of length 'bins'. prob_cum(i) is the
%                             estimated probability of examples with 
%                             Output> limits(i) to be true positive
%
%          limits             vector of length 'bins' with supporting
%                             points (SVM Output values)
%
% see also: Out2Conf  
  
if nargin<5, min_inc = 1e-5 ; end ;
if nargin<4, do_balancing = 1; end ;
if nargin<3, bins = 50; end ;

if do_balancing
  s1 = [min(Out) max(Out)];
  s1 = [s1 Out(L==1)];
  idx = find(L==-1);
  num = min(sum(L==-1),sum(L==1));
  % ii = randperm(length(idx));
  idx = idx(randperm(length(idx)));
  s1 = sort([s1 Out(idx(1:num))]);
  limits = [s1(round(linspace(1,length(s1),bins-1))) inf] ;
else
  s1 = sort(Out) ;
  limits = [s1(round(linspace(1,length(s1),bins-1))) inf] ;
end
  
prob = zeros(bins,1);
prob_cum = zeros(bins,1);
num = zeros(bins,1) ;
num_cum = zeros(bins,1) ;
for i=1:bins-1, 
  num_cum(i) = sum(Out>=limits(i)) ;
  prob_cum(i) = (sum(Out>=limits(i) & L==1 )+1e-10)/(sum(Out>=limits(i))+1e-10) ;
  num(i) = sum(Out>=limits(i) & Out<limits(i+1)) ;
  prob(i) = (sum(Out>=limits(i) & Out<limits(i+1) & L==1 )+1e-10)/(sum(Out>=limits(i) & Out<limits(i+1))+1e-10) ;
end 
prob_cum(bins) = 1;
prob(bins) = 1;

prob_unsort=prob ;
prob_cum_unsort=prob_cum ;

prob_cum = sort(prob_cum) ;
prob = sort(prob) ;

return ;

return

function list=who_file(fname) ;
% list=who_file(fname) ;
  
engine = determine_engine() ;
if isequal(engine, 'matlab')
  list=who('-file', fname) ;
else
  L=load(fname) ;
  list=fieldnames(L) ;
end ;

return

function [engine, environment, basedir, mccdir, license_type] = determine_engine() ;
% [engine, environment, basedir, mccdir, license_type] = determine_engine() ;
% 
% returns either 'matlab' or 'octave' in variable engine
% and 'internal' or 'galaxy' in variable environment
% basedir is the directory of the matlab or octave instance
% mccdir is the directory of the matlab compiler (does not yet exist for octave)

global g_license_type ;

if isempty(g_license_type),
  g_license_type = 'academic' ;
end ;
license_type = g_license_type ;

lic=license ;
if ~isequal(lic, 'GNU General Public License'),
  engine='matlab' ;
else
  engine='octave' ;
end ;

environment='internal' ;
if isequal(whoami, 'galaxy'),
  environment='galaxy' ;
end ;

if isequal(engine, 'matlab') && isequal(environment,  'internal'),
  basedir = '/fml/ag-raetsch/share/software/matlab/' ;
elseif isequal(engine, 'matlab') && isequal(environment,  'galaxy'),
  basedir = '/home/galaxy/matlab-7.6/' ;
elseif isequal(engine, 'octave') && isequal(environment,  'internal'),
  basedir = '/fml/ag-raetsch/share/software/octave-3.0.3/' ;
elseif isequal(engine, 'octave') && isequal(environment,  'galaxy'),
  basedir = '/home/galaxy/octave-3.0.4' ;
end ;

if isequal(environment,  'internal'),
  mccdir = '/fml/ag-raetsch/share/software/matlab/' ;
elseif isequal(environment,  'galaxy'),
  mccdir = '/home/galaxy/matlab-7.6/' ;
end ;

return

return

function user_name = whoami
% user_name = whoami
  
global whoami_user_name  
  
if isempty(whoami_user_name)
  
  [ret, whoami_user_name]=unix('whoami') ; 
  if ret~=0,
    if keyboard_allowed(),
      keyboard ;
    end ;
    whoami_user_name='unknown' ;
  end ;
end ;

user_name= whoami_user_name ;
idx=find(user_name<30, 1, 'first') ;
user_name = user_name(1:idx-1) ;

return

function A=spzeros(m,n,szmax) ;
% A=spzeros(m,n) ;

if nargin<=1
  A=sparse([],[],[],m,m) ;
  return ;
end 
if nargin<=2
  A=sparse([],[],[],m,n) ;
  return ;
end 
A=sparse([],[],[],m,n,szmax) ;

return

function PAR=exp_galaxy(PAR_config)
% PAR=exp_galaxy(PAR_config)

PAR.SETs.num_partitions=5 ;
PAR.SETs.num_splits=5 ;

PAR.FN.input_dir=PAR_config.dir ;
PAR.FN.output_dir=PAR_config.dir;
PAR.FN.genome_dir=PAR_config.dir ;
PAR.FN.input.fn_genome_config = PAR_config.genome_config ;

% preparing RPROC settings
PAR.RPROC.MEMREQ = 6000 ;
PAR.RPROC.collect_jobs = 0 ;
PAR.RPROC.exm_per_batch = 1 ;
if 0
PAR.RPROC.options.addpaths = {'~/svn/projects/genefinding/sensors',
                    '~/svn/projects/genefinding/data_processing_sensors',
                    '~/svn/projects/genefinding/contents',
                    '~/svn/projects/genefinding/galaxy/tools/',
                    '~/svn/projects/genefinding/utils',
                    '~/svn/projects/genefinding/shogun',
                    '~/svn/projects/genefinding/data_processing_lsl',
                    '~/svn/tools/genomes',
                    '~/svn/tools/utils'} ;

end

PAR.RPROC.options.addpaths = {} ;

signals = {'acc','don','tis','cdsStop','tss','cleave'};
for i=1:length(signals)
  PAR.label_source.(signals{i}).from_fn_candsites = 1 ;
  PAR.Signals.(signals{i}).filter_label.train.USE_ALL = 1;
  PAR.Signals.(signals{i}).filter_label.test.USE_ALL = 1;
  PAR.Signals.(signals{i}).filter_label.eval.USE_ALL = 1;
  PAR.Signals.(signals{i}).Conf_names = {} ;
  PAR.Signals.(signals{i}).method.threads = 1;
end


if 0,
  PAR.Signals.acc.order = 10 ;
  PAR.Signals.acc.shift = 0 ;

  PAR.Signals.don.order = 10 ;
  PAR.Signals.don.shift = 0 ;
  
  PAR.Signals.tis.order =  {[],[5],[]};
  PAR.Signals.tis.shift =  {[],[0],[]};
  
  PAR.Signals.cdsStop.order = {[],[5],[]};
  PAR.Signals.cdsStop.shift = {[],[0],[]};
  
  PAR.Signals.tss.order = {[],[5],[]};
  PAR.Signals.tss.shift = {[],[0],[]};
  
  PAR.Signals.cleave.order = {[],[5],[]};
  PAR.Signals.cleave.shift = {[],[0],[]};

end


contents = {'intergenic','utr5exon','cds_exon','utr3exon','polya_tail','intron','frame0'};

for i=1:length(contents)
  PAR.label_source.(contents{i}).from_fn_candsites = 1 ;
  PAR.Contents.(contents{i}).filter_label.train.USE_ALL = 1;
  PAR.Contents.(contents{i}).filter_label.test.USE_ALL = 1;
  PAR.Contents.(contents{i}).filter_label.eval.USE_ALL = 1;
  PAR.Contents.(contents{i}).Conf_names = {} ;
  PAR.Contents.(contents{i}).method.threads = 1;
end

PAR.LSL.method.use.signals.polya = 0 ;
PAR.LSL.method.exm_per_solve = 100 ;

PAR.regions.max_length = 30000 ;


if 1,

%----- REGULARIZATION
method=PAR.LSL.method ;

%C = 1 ;
C = 1e-3;
%%% linear regularization
method.par_ms.C_regul.lengths = C ;        % linear smoothness for len_hist
method.par_ms.C_regul.signals = C ;        % for sig_hist
method.par_ms.C_regul.contents = C ;       % for content_hist
method.par_ms.C_regul.transitions = C ;    % for

method.par_ms.C_regul.tiling = C ;  
method.par_ms.C_regul.rna_seq = C ;  


%%% quadratic regularization
C_sq = 1e-3;

method.par_ms.C_regul.transitions_sq = C_sq;
method.par_ms.C_regul.plif_ys_sq = C_sq*0.1 ;  % small_plif_ys_penalty
method.par_ms.C_regul.smoothness_sq = C_sq ;

PAR.LSL.method=method ;

end

return

function [organism label_source] = dummy_organism(label_source_orig,signal_names)

[label_source] = organism_template(label_source_orig,signal_names);
  
organism.clade = [];
organism.name = '';
organism.full_name = '';
organism.release = '';
organism.version = '';
organism.genebuilt = '';


return

function Content = polya_tail

name = 'polya_tail';

Content = content_template(name);

Content.wordlen = { [3:5]};

return

function FN=init_FN(signal_names,content_names)
 
  
if nargin<1
  signal_names = {'acc','don','tis','cdsStop','tss','cleave','polya','transacc'};
end
if nargin<2
  content_names = {'intergenic','utr5exon','cds_exon','utr3exon','polya_tail','intron','frame0'};
end
 
%--------------------------------------------------------------------------
% Directories
% -------------------------------------------------------------------------------------

FN.genome_dir= '/fml/ag-raetsch/share/databases/genomes/';
FN.input_dir= '/fml/ag-raetsch/share/projects/genefinding/';
FN.output_dir = '/fml/ag-raetsch/share/projects/genefinding/';

FN.dir_name = '';
FN.exp_name = '';
FN.lsl_exp_name = '';
%--------------------------------------------------------------------------
% INPUT FILE NAMES
% -------------------------------------------------------------------------------------

FN.genome.fn_confirmed_sequences = 'confirmed_sequences.mat';
FN.genome.fn_genome_config_orig  = 'genome_uncondensed.config';
FN.genome.fn_genome_config  = 'genome.config';
FN.genome.fn_map = 'condense_map.txt';
FN.genome.fn_genes_anno = 'annotated_genes.mat';
FN.genome.fn_genes_merged = '';
FN.genome.fn_genes_conf = 'confirmed_sequences.mat';
FN.genome.fn_name_to_contig_mapping = 'sequences/contig_orig_new_mapping';


FN.input.fn_genome_config = 'genome.config';
FN.input.fn_trainings_regions = 'training_regions.mat';
FN.input.fn_test_regions = 'test_regions.mat';
FN.input.fn_genes_anno_orig ='annotated_genes.mat';
FN.input.fn_genes_conf_orig = 'confirmed_sequences.mat';
FN.input.fn_ESTs_matches = {'est_matches.mat'};

%--------------------------------------------------------------------------
% OUTPUT FILE NAMES
% -------------------------------------------------------------------------------------

FN.output.fn_genes_anno ='anno_genes.mat';
FN.output.fn_genes_conf = 'conf_genes.mat';
FN.output.fn_genes_merged = 'genes_all.mat';
FN.output.fn_genes_signals = 'genes_signals.mat';
FN.output.fn_genes_contents = 'annotated_genes.mat';
FN.output.fn_genes_lsl = 'annotated_genes.mat';
FN.output.fn_ESTs_matches = [];
FN.output.fn_trivial_regions = 'trivial_regions.mat';
FN.output.fn_blocks = 'blocks.mat';
FN.output.fn_training_blocks ='training_blocks.mat';
FN.output.fn_test_blocks = 'test_blocks.mat';

%--------------------------------------------------------------------------
% SIGNAL FILE NAMES
% -------------------------------------------------------------------------------------

for j=1:length(signal_names)
  sig_name = signal_names{j};
  FN.input_sig.(sig_name).fn_candsites = 'cands/' ;
  FN.input_sig.(sig_name).fn_examples = 'examples';
  FN.input_sig.(sig_name).fn_pos = 'pos/';
  FN.input_sig.(sig_name).fn_filter_settings= 'filter_settings.mat';
  FN.input_sig.(sig_name).fn_example_statistics = 'example_statistics.mat';

  FN.output_sig.(sig_name).fn_models='MS/models';
  FN.output_sig.(sig_name).fn_SVMs='MS/';
  FN.output_sig.(sig_name).fn_pred='pred/';
  FN.output_sig.(sig_name).fn_motifs='MS/motifs';
end


if any(strcmp(signal_names,'tss'))
  FN.input_sig.tss.fn_est_cluster = 'tss_est_clusters.mat';
  FN.input_sig.tss.fn_fulllength  = 'tss_fulllength.mat';
  FN.input_sig.tss.fn_db = 'tss_db.mat' ;
end

if any(strcmp(signal_names,'cleave'))
  FN.input_sig.cleave.fn_est_cluster = 'cleave_est_clusters.mat';
  FN.input_sig.cleave.fn_fulllength = 'cleave_fulllength.mat';
  FN.input_sig.cleave.fn_db  = 'cleave_db.mat';
  FN.input_sig.cleave.fn_anno  = 'cleave_anno.mat';
end
if any(strcmp(signal_names,'polya'))
  FN.input_sig.polya.fn_polya = 'truesites_and_consensus.mat';
  FN.input_sig.polya.fn_log = 'log';
end
if any(strcmp(signal_names,'tis'))
  FN.input_sig.tis.fn_anno = 'tis_anno.mat';
end
if any(strcmp(signal_names,'cdsStop'))
  FN.input_sig.cdsStop.fn_anno = 'cdsStop_anno.mat';
end

if any(strcmp(signal_names,'transacc'))
  FN.input_sig.transacc.fn_anno_SL1 = 'transacc_anno_SL1' ;
  FN.input_sig.transacc.fn_anno_SL2 = 'transacc_anno_SL2' ;
  FN.input_sig.transacc.fn_pred = 'transacc_pred';
  FN.input_sig.transacc.fn_db = []; % only for C_elegans
end

%--------------------------------------------------------------------------
% CONTENT FILE NAMES
% -------------------------------------------------------------------------------------

for j=1:length(content_names)
  content_name = content_names{j};
  FN.input_cont.(content_name) = struct;
  FN.input_cont.(content_name).fn_candsites = 'cands/' ;
  FN.input_cont.(content_name).fn_examples = 'examples';
  FN.input_cont.(content_name).fn_pos = 'pos/';
  FN.input_cont.(content_name).fn_filter_settings= 'filter_settings.mat';
  FN.input_cont.(content_name).fn_example_statistics = 'example_statistics.mat';

  FN.output_cont.(content_name).fn_models = 'MS/models';
  FN.output_cont.(content_name).fn_SVMs = 'MS/';
  FN.output_cont.(content_name).fn_pred = 'pred/';
  FN.output_cont.(content_name).fn_motifs = 'MS/motifs';
end


%--------------------------------------------------------------------------
% LSL FILE NAMES
% -------------------------------------------------------------------------------------

FN.input_lsl.fn_test_blocks = 'data/test_blocks';
FN.input_lsl.fn_training_blocks =  'data/training_blocks.mat';
FN.input_lsl.fn_training_split =  'data/blocks_split.mat';

FN.input_lsl.fn_val_blocks =  'data/validation_blocks.mat';
FN.input_lsl.fn_blocks_all =  'data/blocks_all.mat';
FN.input_lsl.fn_boundary_model = 'data/boundary_model.mat';


FN.output_lsl.fn_init =  'train/init.mat';
FN.output_lsl.fn_train_block_preds = 'train/single_block_predictions/';
FN.output_lsl.fn_train = 'train/train';
FN.output_lsl.fn_pred = 'predictions/';
FN.output_lsl.fn_blocks_pred = 'predictions/blocks_pred';
FN.output_lsl.fn_pred_genome = 'genomewide_predictions/';
FN.output_lsl.fn_log = 'log_files/log_';



return

function [jobinfo,fn_pred]=masterscript_sensors_test(P, check_files, run_locally, do_gzip, save_as, keep_all_files)
% [jobinfo ,all_done]=masterscript_sensors_test(P, check_files, run_locally, do_gzip, save_as, keep_all_files)

num_splits = P.SETs.num_splits ; 
Signal_name = P.Signal_name;

FN = P.FN;  
fn_genome_config = FN.input.fn_genome_config;
fn_training_blocks = FN.output.fn_training_blocks;  
fn_test_blocks = FN.output.fn_test_blocks;

fn_pos =  FN.input_sig.(Signal_name).fn_pos;
fn_pred = FN.output_sig.(Signal_name).fn_pred;
fn_svms = FN.output_sig.(Signal_name).fn_SVMs ;
% fn_svms = sprintf('%sbest_partition=', FN.output_sig.(Signal.name).fn_SVMs) ;

L = load([fn_svms '1.mat'], 'Train');
Signal =  L.Train.PAR.Signal;
clear L 

if nargin<2 
  check_files = 1;
end

if nargin<3 
  run_locally.pos = 1 ;
  run_locally.predict = 1 ;
  run_locally.conf = 1 ;
  run_locally.concat = 1 ;
  run_locally.save = 1 ;
end

if nargin<4 
  do_gzip = 0;
end

if nargin<5 
  save_as.spf_ascii = 1 ;
  save_as.spf_binary = 1 ;
  save_as.wiggle = 0 ;
end

if nargin<6 
  keep_all_files = 0 ;
end

%if ~run_locally  
RPROC = P.RPROC;
%else
%  RPROC = struct;
%end

fid =1;
fprintf(fid,'\n\n-----------\n');
fprintf(fid,'SENSOR :  %s\n',Signal.name);
fprintf(fid,'-----------\n');

all_done = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% --- Generate Positions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

jobinfo=struct ;

%[temp, num_files] = unix(sprintf('ls %scontig_*.pos | wc -l',fn_pos));
d=dir(sprintf('%scontig_*.pos',fn_pos));
num_files = length(d) ;
%[temp, num_files1] = unix(sprintf('ls %scontig_*.svm | wc -l',fn_pos));
d=dir(sprintf('%scontig_*.svm',fn_pos));
num_files1=length(d) ;

genome_info = init_genome(fn_genome_config);
if isempty(num_files)||(num_files<length(genome_info.contig_names)*2)||...
      isempty(num_files1)||(num_files1<length(genome_info.contig_names)*2)
  jobinfo = gen_all_positions(fn_genome_config,fn_pos,Signal,fn_training_blocks,fn_test_blocks,num_splits,run_locally.pos,RPROC);
  if ~isempty(jobinfo),
    fprintf('generating genome position files\n')
    jobinfo = rproc_resubmit(jobinfo);
    jobinfo = rproc_wait(jobinfo, 5, 1, -1); 
  end ; 
else
  fprintf('genome position files already exist\n')
end

avg_all_svms = 0;
subtract_diff = 0;
jobinfo = genomewide_predictions(fn_genome_config,fn_pos,fn_pred,fn_svms,num_splits,avg_all_svms,run_locally.predict, RPROC);
fprintf('all %s predictions done\n',Signal.name)

jobinfo = convert_predictions2confs(fn_genome_config,fn_pred,fn_svms,num_splits, run_locally.conf, RPROC);
fprintf('conversion done\n')


jobinfo = concat_genomewide_predictons(fn_genome_config,fn_pos,fn_pred, num_splits,avg_all_svms,subtract_diff,run_locally.concat, RPROC);
fprintf('concatination done\n')


resolution = Signal.export_settings.resolution ;


conf_cum_thresh = Signal.export_settings.conf_cum_thresh;

inventory = {'output', 'pos', 'Conf', 'Conf_cum'} ;
[jobinfo] = save_predictions(fn_genome_config, fn_pred, ...
                             Signal_name, resolution, ...
                             conf_cum_thresh, '+-', ...
                             run_locally.save, RPROC, ...
                             do_gzip, save_as, inventory);
fprintf('saving done\n')


if check_files
  % fprintf('checking files \n')
  % check_genomewide_output(P) % this fct compares prediction results with test results from training (cross_validation)
  % all_good = check_predictions(fn_genome_config,fn_pred,Signal_name,resolution)
else
  fprintf('WARNING: files not checked\n')
end


fprintf('%s COMPLETE !\n\n',Signal.name)


if ~keep_all_files
 %  unix(sprintf('rm -rf %s*_split=*', FN.output_sig.(Signal_name).fn_pred));
end

return

function all_good = check_predictions(fn_genome_config,fn_pred,Signal_name,resolution)
% check_predictions(fn_pred,fn_genome_config)



P.fn_genome_config = fn_genome_config;
P.fn_pred = fn_pred;
P.resolution = resolution;
P.Signal_name = Signal_name;

genome_info = init_genome(fn_genome_config);

all_good = zeros(2, length(genome_info.contig_names));
for c = 1:length(genome_info.contig_names)
	  num = 0;
   for s = '+-'
     num = num+1;
     filename = sprintf('%scontig_%i%s', fn_pred, c, s);
     if fexist([filename '.pos'])&& fexist([filename '.output'])
       P.chrom = c;
       P.strand = s; 
       all_good(num,c) = check_predictions_helper(P) ;
       fprintf('check_predictions: strand: %s, chrom: %i, ok: %i',s, c,all_good);
     else
       fprintf('file missing: %s',filename)
     end 
   end  
end 

return

function blocks = add_signals2blocks(blocks, PAR, signal_names, which_dir, offset,which_fields)
% blocks = add_signals2blocks(blocks, PAR, signal_names, which_dir, offset)

if nargin<5
  warning('I think this should be a parameter')
  offset = 100;  
end
if nargin<4 || isempty(which_dir)
  which_dir = 'pred_dir' ;
end
genome_info = init_genome(PAR.FN.input.fn_genome_config);
which_pos.local = 1;
which_pos.contig = 1;
base_dirs    = {};

for i=1:length(signal_names),
  if isequal(which_dir,'pred_dir')'
    tmp = PAR.FN.output_sig.(signal_names{i});
    base_dirs{i} = tmp.fn_pred;
    fields = {'Conf_cum'};
  elseif isequal(which_dir,'cand_dir')'
    tmp = PAR.FN.input_sig.(signal_names{i});
    base_dirs{i} = tmp.fn_candsites;
    fields = {'label','altgenic','alt_intron','est_conf','est_conf_skip','cDNA_conf','cDNA_conf_skip'};
  end
end
if nargin>5
  fields = which_fields;
end

if ~isfield(blocks,'Signals')
  blocks(1).Signals=struct;
end

% blocks = load_sequence(blocks);
fprintf('Processing %i blocks: ', length(blocks));
for r=1:length(blocks)
  block=blocks(r);
  if mod(r,50)==0, fprintf('%i .. ', r); end ;
  % fill the field genestr
  for s=1:length(signal_names),
    f_name = sprintf('%scontig_%i%s.pos', base_dirs{s},block.chr_num,block.strand);
    d = dir(f_name) ;
    if isempty(d), error('file not found: %s',f_name);end
    if d.bytes>0
      block = retrieve_signals(block, base_dirs{s},signal_names{s}, fields, which_pos, offset);
      % block = retrieve_signals(block, base_dirs{s},signal_names{s}, fields,which_pos,0);
    else
      warning('hack!\n')
      block.Signals.(signal_names{s})=struct ;
      block.Signals.(signal_names{s}).pos = [];
      block.Signals.(signal_names{s}).contig_pos = [];
      for k=1:length(fields)
        block.Signals.(signal_names{s}).(fields{k}) =  [];
      end ;
    end
  end
  blocks(r)=block;
end
fprintf('Done.\n') ;

% load /fml/ag-raetsch/share/projects/nGASP/LSL/test_data/cat_1/blocks_2006-12-19.mat

% eof

return

function save_struct(outfile_name, array, arrayname, MAXSIZE)
% save_struct(outfile_name, array[, arrayname])

if nargin<3
  arrayname='array' ;
end ;

if nargin<4, 
  MAXSIZE = 1e9; % about 1Gb
end ;

whosinfo = whos('array');
if whosinfo.bytes > MAXSIZE
  % linearize first, store original size
  dimensions = size(array) ;
  array = reshape(array, 1, prod(dimensions)) ;

  elementsize = whosinfo.bytes/length(array);
  chunklength = round(MAXSIZE/elementsize);
  numvars = ceil(length(array)/chunklength);
  %fprintf('arraysave2 processes %i chunks:', numvars) ;
  for ix = 1:numvars-1
    eval(sprintf('%s%d = array(%d:%d);',arrayname, ix,(ix-1)*chunklength+1, ...
                 ix*chunklength));
    %fprintf('.') ;
  end
  %fprintf('\n') ;
  eval(sprintf('%s%d = array(%d:end);',arrayname, numvars,(numvars-1)*chunklength+1));
  clear array ix ;
  fprintf('save_struct saves %i chunks:', numvars) ;
  save(outfile_name, '-V7', 'chunklength', 'elementsize', 'numvars', 'dimensions') ;
  for ix = 1:numvars
    save(sprintf('%s_%i.mat', outfile_name, ix), '-V7', sprintf('%s%d', arrayname, ix)) ;
    fprintf('.') ;
  end ;
  fprintf('\n') ;
else
  %eval(sprintf('%s=array;', arrayname)) ;
  save_append(outfile_name, 0, arrayname, array) ;
end

return

function array = load_struct(infile_name, arrayname)
% array = load_struct(infile_name, arrayname)
  
if nargin<2,
  arrayname='array' ;
end ;

L_=load(infile_name) ;
if isfield(L_,'numvars')
  fprintf('arrayload2 loads %i chunks:', L_.numvars) ;
  ix=1;
  L=load(sprintf('%s_%i.mat', infile_name, ix), sprintf('%s%d', arrayname, ix)) ;
  eval(sprintf('array = L.%s1;', arrayname)) ;
  fprintf('.') ;
  for ix = 2:L_.numvars
    L=load(sprintf('%s_%i.mat', infile_name, ix), sprintf('%s%d', arrayname, ix)) ;
    if size(array,1) == 1
      array = [array, eval(sprintf('L.%s%d', arrayname, ix))];
    elseif size(array,2) == 1
      array = [array; eval(sprintf('L.%s%d', arrayname, ix))];
    end
    fprintf('.') ;
  end
  fprintf('\n') ;
  if isfield(L_, 'dimensions'),
    array = reshape(array, L_.dimensions) ;
  end ;
else
  eval(sprintf('array=L_.%s;', arrayname)) ;
end

return

function found = issubset(A,B)
% function found = issubset(A,B)
%
% returns true if A is a subset of B, where 
% both A and B are vectors with 1 where elements exists and 0 otherwise.
found = isequal(find(A),intersect(find(A),find(B)));
return

function ret=keyboard_allowed()
% ret=keyboard_allowed()
%
% returns 1, if a keyboard command would be allowed

global g_ignore_keyboard
global THIS_IS_A_RPROC_PROCESS  

if isequal(g_ignore_keyboard, 1),
  ret=0 ;
  return ;
end ;

if isequal(THIS_IS_A_RPROC_PROCESS, 1),
  ret=0 ;
  return ;
end ;

ret=1 ;


return

function f=bigunique(s) ;
% f=bigunique(s) ;

f=[] ; step=ceil(length(s)/10) ;
for i=1:10,
  idx=1+(i-1)*step:min(length(s), i*step) ;
  f=unique([f unique(s(idx))]) ;
end ;


return

function [mask hist] = compute_mers(string, order, stepping, offset)
%function [mask hist] = compute_mers(string, order, stepping)

mask    = zeros(1,4^order) ;
hist    = zeros(1,4^order) ;

%if nargin<3,
%  stepping = 1;
%end;
%
%if nargin<4,
%  offset = 0;
%end;
%
%if isempty(string), 
%  return; 
%end;
string = lower(char(string)) ;

if ~isempty(find(~ismember(string,'acgt')))
  string(find(~ismember(string,'acgt'))) = 'a';
end;

assert(order<=8);

if length(string)<order,
  return ;
end
t = sg('translate_string', double(string), order, order-1) ;
if length(t)<order
  return ;
end ;

% transform t to matlab coordinates
t = t(1:end-order+1)+1 ;

% throw away kmers if stepping > 1
assert(stepping>0);
%t = t(1:stepping:end);
t = t(end-offset:-stepping:1);

% reduce t to the information whether or not a certain kmer occurs 
u = unique(t) ;

% get histogram counts
t = sort(t);
[dummy loc] = ismember(u, t);
occ = loc - [0 loc(1:end-1)];

% return histogram
hist(u) = occ/sum(occ);

% return "binary histogram" 
% telling whether or not a certain kmer is present
mask(u) = 1/sqrt(length(u)) ;

return

