%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% Written (W) 2005-2009 Gunnar Raetsch, Gabriele Schweikert, Jonas Behr, Soeren Sonnenburg, Alexander Zien, Georg Zeller, Andre Noll, Cheng Soon Ong, Petra Philips
% Copyright (C) 2005-2009 Max Planck Society and Fraunhofer Institute FIRST 
%
function mgene_content_train(genome_config_dir, anno_dir, cpf_label_fname, classifier_dir)
% mgene_content_train(genome_config_dir, anno_dir, cpf_label_fname, classifier_dir)

global MGENE_TMP_PATH

if nargin~=4,
  first_arg=[] ;
  if nargin>=1, first_arg=genome_config_dir; end ;

  if show_usage('mgene_content_train', '<genome-dir>, <annotation-dir>, <label-CPF-file>, <classifier-dir>', ...
     nargin, first_arg),
     return
  end ;
end ;

if isdir(cpf_label_fname)
  cpf_label_dir=cpf_label_fname ;
  cpf_label_fname='' ;
else
  cpf_label_dir='' ;
end ;

classifier_info=tempname(MGENE_TMP_PATH) ;

content_train(cpf_label_fname, cpf_label_dir, 'auto', genome_config_dir, anno_dir, classifier_info, classifier_dir, -1) ;
fprintf('\n\n') ;

system(sprintf('cat %s && rm %s', classifier_info, classifier_info)) ;

return

function PAR = content_train(cpf_label_fname, cpf_label_dir, content_name, genome_config_dir, anno_dir, classifier_info, classifier_dir, run_locally_par, options)
% PAR = content_train(cpf_label_fname, cpf_label_dir, content_name, genome_config_dir, classifier_info, classifier_dir, run_locally)

global USE_SGE

[ret, timedate] = unix('date') ;
timedate(timedate==sprintf('\n')) = [] ;
fprintf('started content_train at %s\n', timedate) ;

fprintf('Results are written to: %s\n\n', classifier_info) ;

%extra_path = SHOGUN_PATH ;
%shogun_envstr = ['export LD_LIBRARY_PATH=' SHOGUN_PATH] ;
%addpath(extra_path)

genome_config = [genome_config_dir '/genome.config'] ;
genome_info = init_genome(genome_config) ;

if ~exist('run_locally_par', 'var'),
  run_locally_par = -1 ;
end ;

if run_locally_par==-1 && ~USE_SGE
  run_locally_par = 1 ;
end ;

if run_locally_par==-1,
  run_locally.pos = 1 ; % rproc_policy('content_train:pos',  genome_info) ; % not implemented 
  run_locally.train = rproc_policy('content_train:train',  genome_info) ;
  run_locally.eval = 1 ; % rproc_policy('content_train:eval',  genome_info) ; % not implemented
  run_locally.predict = 1 ; %rproc_policy('content_train:predict',  genome_info) ; % not implemented
  run_locally.save = 1;%rproc_policy('content_train:save',  genome_info) ; % not implemented
else
  run_locally.pos = 1 ;%run_locally_par ; % not implemented 
  run_locally.train = run_locally_par ;
  run_locally.eval = 1 ; %run_locally_par ; % not implemented 
  run_locally.predict = 1 ; %run_locally_par ; % not implemented
  run_locally.save = 1 ; % run_locally_par ; % not implementede
end ;


fid = 1;
  
disp('------------------------------------') ;
fprintf('Processing content: %s\n', content_name );
disp('------------------------------------') ;


disp('------------------------------------') ;
disp('Step 1: Setting up datastructures...') ;
disp('------------------------------------') ;



fprintf(fid,'\n\nstart training for sensor %s\n\n',content_name);

galaxy_PAR.dir = classifier_dir ;
galaxy_PAR.genome_config = genome_config ;
PAR = create_PAR('exp_galaxy', galaxy_PAR);

PAR.tasks.contents.prepare_labels = 1;
PAR.tasks.contents.train = 1;
PAR.tasks.contents.modelsel = 1;
PAR.tasks.contents.learn_plifs = 0;
PAR.tasks.contents.get_train_error = 1;
PAR.tasks.contents.pred_on_genome = 0;
PAR.tasks.contents.run_locally = run_locally ;

%PAR.RPROC.MEMREQ = 2000;

% keyboard
create_directories(PAR.FN,PAR.organism,0,content_name) ;

genome_config = PAR.FN.input.fn_genome_config;
organism = PAR.organism.name;

if ~isempty(anno_dir),

  genes = load_struct(sprintf('%s/genes.mat', anno_dir), 'genes') ;

  fprintf(fid,'\n\nloaded %i annotated genes \n\n', length(genes));
else
  genes = [] ;
end ;

fprintf(fid,'init_regions\n')
trivial_regions = init_regions(genome_config);

genome_info =  init_genome(genome_config) ;

% it is necessary to do this again here since the genes are not saved 
% by anno2contentlabel, where the filtering was already done
%genes = filter_invalid_genes(genes, genome_info);

[training, test] = assign_subsets(trivial_regions, PAR.FN.input.fn_trainings_regions,PAR.FN.input.fn_test_regions,genes,PAR.SETs,PAR.regions,PAR.tasks.contents) ;

% training_regions = init_regions(genome_config);
% save(PAR.FN.input.fn_trainings_regions,'training_regions');

% test_regions=training_regions(1);
% test_regions.stop = 0;
% test_regions.start = 0;
% save(PAR.FN.input.fn_test_regions, 'test_regions');

% trivial_regions = training_regions;
% save(PAR.FN.output.fn_trivial_regions,'trivial_regions');
% clear trivial_regions

fprintf('Saving blocks:\n\n') ;
blocks = training.blocks;
split = training.split;
save_struct(PAR.FN.output.fn_training_blocks, blocks, 'blocks') ;
save_append(PAR.FN.output.fn_training_blocks, 1, 'PAR', PAR, 'split', split);
fprintf('Done.\n\n') ;


disp('-----------------------------------------') ;
disp('Step 2: Reading and filter label files...') ;
disp('-----------------------------------------') ;

cpf_label_mat_fname = sprintf('%s/label_cpf.mat', cpf_label_dir) ;
if fexist(cpf_label_mat_fname),
  disp('Reading binary CPF label files') ;
  [chr, strand, pos, score, content_names, score_names, chr_dic, content_names_dic, score_names_dic, success] = read_contpred_bin(cpf_label_mat_fname) ;
  assert(success) ;

  %[chr_, strand_, pos_, score_, content_names_, score_names_, chr_dic_, content_names_dic_, score_names_dic_]=read_contpred(cpf_label_fname) ;
  %keyboard
  %assert(isequal(chr,chr_)) ;
  %assert(isequal(strand,strand_)) ;
  %assert(isequal(pos,pos_)) ;
  %assert(isequal(score,score_)) ;
  %assert(isequal(score_names,score_names_)) ;
  %assert(isequal(content_names_dic,content_names_dic)) ;
  %assert(isequal(score_names_dic,score_names_dic)) ;
else
  disp('Reading CPF label files') ;
  [chr, strand, pos, score, content_names, score_names, chr_dic, content_names_dic, score_names_dic]=read_sigpred(cpf_label_fname) ;
end ;

if isequal(content_name, 'auto') && length(content_names_dic)==1,
  content_name = content_names_dic{1} ;
end ;

% filter out spurious entries
idx_ = strmatch(content_name, content_names_dic, 'exact') ;
idx = find(content_names == idx_) ;
if length(idx)~=length(content_names),
  warning('dropping %i/%i entries from other contents', length(content_names)-length(idx), length(content_names)) ;
  chr = chr(idx) ;
  strand = strand(idx) ;
  pos = pos(idx,:) ;
  score = score(idx) ;
  content_names = content_names(idx) ;
  score_names = score_names(idx) ;
end 

idx_ = strmatch('label', score_names_dic, 'exact') ;
idx = find(score_names == idx_) ;
if length(idx)~=length(score_names),
  warning('dropping %i/%i non-label entries', length(score_names)-length(idx), length(score_names)) ;
  chr = chr(idx) ;
  strand = strand(idx) ;
  pos = pos(idx,:) ;
  score = score(idx) ;
  content_names = content_names(idx) ;
  score_names = score_names(idx) ;
end 

idx = find(abs(abs(score)-1)<=1e-3) ;
if length(idx)~=length(score),
  warning('dropping %i/%i entries with label not +/-1', length(score)-length(idx), length(score)) ;
  chr = chr(idx) ;
  strand = strand(idx) ;
  pos = pos(idx,:) ;
  score = score(idx) ;
  content_names = content_names(idx) ;
  score_names = score_names(idx) ;
end 

unix(sprintf('mkdir -p %s', PAR.FN.input_cont.(content_name).fn_candsites)) ;
num_pos = 0 ;
num_neg = 0 ;
for i=1:length(genome_info.contig_names),
  for s='+-',
    idx_ = strmatch(lower(genome_info.contig_names{i}), lower(chr_dic), 'exact') ;
    if ~isempty(idx_)
      idx = find(chr==idx_) ;
      idx2 = find(strand(idx)==s) ;
      idx = idx(idx2) ;
    else
      idx = [];
    end
    save_append(sprintf('%s/contig_%i%c.mat', PAR.FN.input_cont.(content_name).fn_candsites, i, s), 0, 'pos', pos(idx,:), 'label', score(idx)) ;
    num_pos = num_pos + sum(sign(score(idx))==1) ;
    num_neg = num_neg + sum(sign(score(idx))==-1) ;
  end ;
end 
fprintf('Number of pos: %i \n',num_pos) ;
fprintf('Number of neg: %i \n',num_neg) ;

fprintf('Done.\n\n') ;

unix(sprintf('mkdir -p %s', PAR.FN.output_cont.(content_name).fn_SVMs)) ;
unix(sprintf('mkdir -p %s', PAR.FN.output_cont.(content_name).fn_pred)) ;

disp('----------------------------') ;
disp('Step 3: Starting Training...') ;
disp('----------------------------') ;
PAR.Content_name = content_name ;
all_done = 0 ; iter = 0 ;
while ~all_done,
  fprintf('\n\n start masterscript_contents: iter:%i \n\n', iter)
  [PAR, jobinfo, all_done] = masterscript_contents(PAR, 1) ;
  iter = iter + 1 ;
  if iter>10, % this should not normally happen
    error('Computations aborted') ;
  end ;
end ;
fprintf('Done.\n\n') ;

disp('-------------------------------') ;
disp('Step 4: Writing output files...') ;
disp('-------------------------------') ;

PAR.Content_name = content_name ;
save([classifier_dir '/PAR.mat'], 'PAR', '-v7') ;

L=load(PAR.FN.output_cont.(content_name).fn_models) ;
num_pos = sum(score>0) ;
num_neg = sum(score<0) ;

fi = fopen(classifier_info, 'w+') ;
fprintf(fi, 'Trained "%s" classifier\n', content_name) ;
fprintf(fi, ' * based on %i labeled examples (%i positive, %i negative)\n', num_pos+num_neg, num_pos, num_neg) ;
fprintf(fi, ' * using %i-fold cross-validation for model-selection from %i models (inner cv loop)\n', PAR.SETs.num_splits, size(L.MODELS,2)) ;
fprintf(fi, ' * using %i-fold cross-validation for obtaining unbiased predictions (outer cv loop)\n\n', PAR.SETs.num_splits) ;
fprintf(fi, 'Performance\n') ;
fprintf(fi, ' * Average area under ROC curve on test splits: %1.3f\n', mean(L.RSV_test)) ;
fprintf(fi, ' * Average area under PRC curve on test splits: %1.3f\n', mean(L.RFV_test)) ;

fclose(fi) ;

fprintf('Done.\n\n') ;

%rmpath(extra_path) 

[ret, timedate] = unix('date') ;
timedate(timedate==sprintf('\n')) = [] ;
fprintf('finished content_train at %s\n', timedate) ;

return

return

function content_predict(PAR_dir, genome_config_dir, fn_out, fn_out_dir, run_locally_par, save_as,options)
% signal_predict(PAR_dir, genome_config_dir, fn_out, fn_out_dir, run_locally_par, save_as)
%
% predicts signals and writes the predictions to the file fn_out
% temporary files are stored in a directory fn_out which is deleted

global USE_SGE

[ret, timedate] = unix('date') ;
timedate(timedate==sprintf('\n')) = [] ;
fprintf('started content_predict at %s\n', timedate) ;

fprintf('results are written to: %s\n', fn_out) ;

%extra_path = SHOGUN_PATH ;
%shogun_envstr = ['export LD_LIBRARY_PATH=' SHOGUN_PATH] ;
%addpath(extra_path)

genome_config = [genome_config_dir '/genome.config'] ;
genome_info = init_genome(genome_config) ;

if ~exist('run_locally_par', 'var'),
  run_locally_par = -1 ;
end ;

if run_locally_par==-1 && ~USE_SGE
  run_locally_par = 1 ;
end ;

if run_locally_par==-1,
  run_locally.pos = rproc_policy('content_predict:pos',  genome_info) ; % not implemented
  run_locally.predict = rproc_policy('content_predict:predict',  genome_info) ;
  run_locally.concat = rproc_policy('content_predict:concat',  genome_info) ;
  run_locally.save = rproc_policy('content_save:save',  genome_info) ;
else
  run_locally.pos = run_locally_par ; % not implemented 
  run_locally.predict = run_locally_par ;
  run_locally.concat = run_locally_par ;
  run_locally.save = run_locally_par ;
end ;

if ~exist('save_as', 'var'),
  save_as.spf_ascii = 1 ;
  save_as.spf_binary = 1 ;
  save_as.wiggle = 0 ;
end ;
if ~exist('options', 'var'),
  options = [] ;
end ;


disp('------------------------------------') ;
disp('Step 1: Setting up datastructures...') ;
disp('------------------------------------') ;

L=load([PAR_dir '/PAR.mat'], 'PAR') ;
PAR=L.PAR ;
content = PAR.Content_name ;
P.Content_name = content ;
P.RPROC = PAR.RPROC ;
P.RPROC.options = options;
if ~isfield(P.RPROC.options, 'addpaths')
  P.RPROC.options.addpaths={} ;
end ;
%P.RPROC.options.addpaths{end+1}=extra_path ;
%P.RPROC.options.envstr = shogun_envstr ;

genome_config = [genome_config_dir '/genome.config'] ;

% make sure there is no file with the same name
%system(['rm -f ' fn_out]) ;

% create the working directory
%dir_orig = fn_out_dir ;
%dir_ = [fn_out_dir '/galaxy_content_test'] ;
%system(['mkdir -p ' dir_]) ;

fprintf(1,'\n\nstart testing for content %s\n\n', content);

% prepare data structures
fid = 1; % stdout

P.SETs.num_splits = PAR.SETs.num_splits ; 
P.FN.output_cont.(content).fn_SVMs = sprintf('%sbest_partition=', PAR.FN.output_cont.(content).fn_SVMs);
if isequal(genome_config, PAR.FN.input.fn_genome_config)
  P.FN.output.fn_training_blocks = PAR.FN.output.fn_training_blocks ;
  P.FN.output.fn_test_blocks = PAR.FN.output.fn_test_blocks ;
else
  P.FN.output.fn_training_blocks = [];
  P.FN.output.fn_test_blocks = [];
end

P.FN.input.fn_genome_config = genome_config;
P.FN.input_cont.(content).fn_pos =  sprintf('%s/pos/',fn_out_dir);
P.FN.output_cont.(content).fn_pred =  sprintf('%s/pred/',fn_out_dir) ;

% create output directories
system(sprintf('mkdir -p %s',fn_out_dir));
system(sprintf('mkdir -p %s/pos',fn_out_dir));
system(sprintf('mkdir -p %s/pred',fn_out_dir));

fprintf('Done.\n\n') ;

disp('------------------------------') ;
disp('Step 2: Starting Prediction...') ;
disp('------------------------------') ;

% start the actual work

[jobinfo,fn_pred] = masterscript_contents_test(P, 0, run_locally, 0, save_as) ;

fprintf('Done.\n\n') ;

disp('-------------------------------') ;
disp('Step 3: Writing output files...') ;
disp('-------------------------------') ;

% collect the output
genome_info = init_genome(genome_config) ;
fclose(fopen(fn_out, 'w+')) ;
cnt = 0 ;
spf_info = [] ;
for c = 1:length(genome_info.contig_names)
  for s = '+-'
    fprintf('processing contig_%i%s\r', c, s);

    P.chrom = c;
    P.strand = s;
    
    filename = sprintf('%scontig_%i%s', fn_pred, c, s);
    if save_as.spf_ascii,
      fn_in=[filename '_output.spf'];
      for j=1:20
        try
          tmp = fopen(fn_in, 'r');
          assert(tmp>-1) ;
          fclose(tmp);
        catch
  	  fprintf('unable to load file %s\nwaiting 30 secs\n', fn_in)
          pause(30)
        end
      end
      system(['cat ' fn_in ' >> ' fn_out]) ;
      %assert(ret==0) ;
      system(['rm -f ' filename '_output.spf ']) ;
      %assert(ret==0) ;
    end ;

    if save_as.spf_binary,
      fn_mat = sprintf('%s/contig_%i%s_output_spf.mat', fn_pred, c, s) ;
      fnall_mat = sprintf('%s/output_spf.mat', fn_out_dir) ;

      try %if file exists, but writing is not completed
        SPF=load(fn_mat);
      catch
        fprintf('unable to load %s\nwaiting 300sec\n', fn_mat) ;
	pause(10)
        SPF=load(fn_mat);
      end
      spf_info(end+1).contig_name = SPF.contig_name ;
      spf_info(end).strand = SPF.strand ;
      spf_info(end).score_name = SPF.score_name ;
      spf_info(end).signal_name = SPF.signal_name ;

      save_append(fnall_mat, (cnt>0), sprintf('SPF_%i', cnt), SPF) ;
      cnt = cnt + 1 ;

      % cleanup
      system(['rm -f ' filename '_output_spf.mat ']) ;
      %assert(ret==0) ;
    end ;
    system(['rm -f ' filename '_split=?.mat ' filename '_all.mat']) ;
    %assert(ret==0) ;
  end ;
end ;

if save_as.spf_binary,
  save_append(fnall_mat, (cnt>0), 'SPF_info', spf_info) ;
end ;

fprintf('Done.\n\n') ;

%rmpath(extra_path) 

[ret, timedate] = unix('date') ;
timedate(timedate==sprintf('\n')) = [] ;
fprintf('finished content_predict at %s\n', timedate) ;

return

function genes = filter_invalid_genes(genes, genome_info,log_fname)
%  genes = filter_invalid_genes(genes, genome_info,log_fname)
  
if exist('log_fname')
  fd=fopen(log_fname, 'a') ;
end

ignore_missing_referenced_entries = 1;  
ignore_genes_without_transcripts = 1;
ignore_genes_without_exons = 1;
ignore_genes_without_chr = 1;

take_map.is_valid = ones(1,length(genes)) ;
take_map.no_parents = ones(1,length(genes)) ;
take_map.no_transcripts = zeros(1,length(genes)) ;
take_map.no_exons = ones(1,length(genes)) ;
take_map.no_chr = ones(1,length(genes)) ;

%--------------

if length(genes)==1 && isequal(genes.id, []) && isequal(genes.transcripts, [])
  genes(1)=[] ;
end ;
assert(length([genes.is_valid]) == length(genes)) ;
take_map.is_valid(find(~[genes.is_valid]))=0 ;

assert(length([genes.is_correctly_gff3_referenced]) == length(genes)) ;
take_map.no_parents(find(~[genes.is_correctly_gff3_referenced]))=0 ;

num_invalid_transcripts = 0 ;

for i=1:length(genes),
  % find genes with no transcript
  take_map.no_transcripts(i) = ~isempty(genes(i).transcripts) ;
  % find genes with no exons 
  empty = 1;
  for j=1:length(genes(i).exons),
    if ~isempty(genes(i).exons{j})
      empty=0 ;
      break
    end ;
  end ;
  if empty,
    take_map.no_exons(i) = 0 ;
  end ;
  % find genes of unknown contig
  chr_num = find(ismember(lower(genome_info.contig_names), lower(genes(i).chr))) ;
  take_map.no_chr(i) = length(chr_num)==1 ;
  
  % remove invalid transcripts
  num_invalid_transcripts = num_invalid_transcripts + sum(~genes(i).transcript_valid) ;
  idx = find(genes(i).transcript_valid) ;
  genes(i).exons = genes(i).exons(idx) ;
  genes(i).exons_confirmed = genes(i).exons_confirmed(idx);
  genes(i).cds_exons = genes(i).cds_exons(idx) ;
  genes(i).transcripts = genes(i).transcripts(idx) ;
  genes(i).transcript_valid = genes(i).transcript_valid(idx) ;
  try
    genes(i).transcript_complete = genes(i).transcript_complete(idx) ; 
  catch
  end
  if ~isempty(genes(i).transcript_status)
    genes(i).transcript_status = genes(i).transcript_status(idx) ;
  end  
  genes(i).utr5_exons = genes(i).utr5_exons(idx) ;
  genes(i).utr3_exons = genes(i).utr3_exons(idx) ;
  if ~isempty(genes(i).tis),
    genes(i).tis = genes(i).tis(idx) ;
  end ;
  if ~isempty(genes(i).cdsStop),
    genes(i).cdsStop = genes(i).cdsStop(idx) ;
  end ;
  try,
    genes(i).tss = genes(i).tss(idx) ;
  catch
    % warning('tss elements missing') ;
  end ;
  try,
    genes(i).cleave = genes(i).cleave(idx) ;
  catch
    % warning('cleave elements missing') ;
  end ;
  try,
    genes(i).polya = genes(i).polya(idx) ;
  catch
    % warning('polya elements missing') ;
  end ;
end ;

fprintf(1, ' %i valid genes.\n', sum(take_map.is_valid==1)) ;
fprintf(1, ' %i invalid genes.\n', sum(take_map.is_valid==0)) ;
fprintf(1, '      %i genes with no exon segments.\n', sum(take_map.no_exons==0)) ;
fprintf(1, '      %i genes with no transcripts.\n', sum(take_map.no_transcripts==0)) ;
fprintf(1, '      %i genes incorrectly referenced (parent gene missing).\n', sum(take_map.no_parents==0)) ;
fprintf(1, '      %i genes on unknown contigs/chromosomes.\n', sum(take_map.no_chr==0)) ;
fprintf(1, '      %i invalid transcripts found.\n',num_invalid_transcripts);
if exist('fd')
fprintf(fd,'\n')
fprintf(fd, ' %i valid genes.\n', sum(take_map.is_valid==1)) ;
fprintf(fd, ' %i invalid genes.\n', sum(take_map.is_valid==0)) ;
fprintf(fd, '      %i genes with no exon segments.\n', sum(take_map.no_exons==0)) ;
fprintf(fd, '      %i genes with no transcripts.\n', sum(take_map.no_transcripts==0)) ;
fprintf(fd, '      %i genes incorrectly referenced (parent gene missing).\n', sum(take_map.no_parents==0)) ;
fprintf(fd, '      %i genes on unknown contigs/chromosomes.\n', sum(take_map.no_chr==0)) ;
fprintf(fd, '      %i invalid transcripts found.\n',num_invalid_transcripts);
fclose(fd);
end

take_idx = ones(1,length(genes));
take_idx = take_idx&take_map.is_valid;

if ignore_missing_referenced_entries,
  take_idx = take_idx&take_map.no_parents;
end ;
if ignore_genes_without_transcripts
  take_idx = take_idx&take_map.no_transcripts;
end ;
if ignore_genes_without_exons
  take_idx = take_idx&take_map.no_exons;
end ;
if ignore_genes_without_chr
  take_idx = take_idx&take_map.no_chr;
end ;
  
genes = genes(take_idx) ;
  
      
fprintf('Done.\n\n') ;


return

function genes2content_helper(genes, genome_config, cpf_fname, cpf_dir, content_name, vs_content_names,intergenic_dist_to_genes) ;
% genes2content_heper(genes, genome_config, cpf_fname, cpf_dir, content_name,  vs_content_names, intergenic_dist_to_genes) ;

disp('-------------------------------------------------------') ;
disp('Step 5: Extracting labeled segments from the genome...') ;
disp('-------------------------------------------------------') ;

if ~isempty(cpf_dir),
  galaxy_PAR.dir = cpf_dir ;
else
  galaxy_PAR.dir = tempname ;
end ;
galaxy_PAR.genome_config = genome_config ;
PAR = create_PAR('exp_galaxy', galaxy_PAR);
Content = PAR.Contents.(content_name);
segment_ids = PAR.model.segments;
ids = [];
for i=fieldnames(segment_ids)'
  ids = [ids segment_ids.(i{1})]; 
end
% for the moment contents must be one of segment types. 
if isfield( segment_ids,content_name)
  which_segments{1} = segment_ids.(content_name);
elseif isequal(content_name,'frame0')
  which_segments{1} = segment_ids.cds_exon;
end
if isempty(vs_content_names)
    which_segments{2} = setdiff(ids,which_segments{1});% one versus rest
else
  which_segments{2} = segment_ids.(vs_content_names);
end

if ~isempty(cpf_dir),
  out_dir = [cpf_dir '/tmp'] ;
else
  out_dir = tempname ;
end ;
unix(sprintf('mkdir -p %s', out_dir)) ;

if isempty(intergenic_dist_to_genes)
  intergenic_dist_to_genes = [100 20000];
end

if isempty(strfind(content_name,'frame0'))
  use_frame=0;
else
  use_frame=1;
end

genome_info = init_genome(genome_config) ;
contig_names = genome_info.contig_names ;
for c=1:length(contig_names)
  for s='+-',
    cs_idx = find(ismember({genes.chr}, contig_names{c}) & [genes.strand]==s) ;
    %if isempty(cs_idx)
    %  continue
    %end
    cs_genes = genes(cs_idx) ;
    P_ = struct ;
    P_.genes = cs_genes;
    P_.segment_ids = segment_ids;
    cs_genes = gen_gene_segmentation(P_);
    clear P_
    
    if isempty(cs_genes),
      segments=[] ;
      label=[] ;
    else
      % generate positive and negative candidate segments;
      P_ = struct ;
      P_.genes = cs_genes;
      P_.intergenic_dist_to_genes = intergenic_dist_to_genes;
      P_.use_frame = use_frame;
      P_.which_segments = which_segments;
      P_.segment_ids = segment_ids;
      P_.genome_info = genome_info;
      [segments, label] = feval(Content.label_fct, P_);
      clear P_ ;
    end ;

    % save them to a file
    filename_prefix = sprintf('%s/contig_%i%c', out_dir, c, s) ;
    fn_in=[filename_prefix '_label.cpf'];
    % unix(sprintf('rm -f %s', fn_in)) ;
    save_predictions_as_contpred(filename_prefix, segments, label, 'label', content_name, contig_names{c}, s, 0) ;
    save_predictions_as_contpred_bin(filename_prefix, segments, label, 'label', content_name, contig_names{c}, s) ;
  end ;
end ;
fprintf('Done.\n\n') ;

disp('----------------------------------') ;
disp('Step 6: Generating output files...') ;
disp('----------------------------------') ;
unix(sprintf('rm -f %s', cpf_fname)) ;
cnt = 0 ;
for c = 1:length(contig_names)
  fprintf('Processing outputs for contig %s\n', contig_names{c}) ;
  for s = '+-'
    P.chrom = c;
    P.strand = s;
    filename = sprintf('%s/contig_%i%s', out_dir, c, s);
    fn_in = [filename '_label.cpf'];
    ret = unix(['cat ' fn_in ' >> ' cpf_fname ' && rm -f ' fn_in]) ;
    assert(ret==0) ;

    fn_mat = sprintf('%s/contig_%i%s_label_cpf.mat', out_dir, c, s);
    fnall_mat = sprintf('%s/label_cpf.mat', cpf_dir);
    if ~fexist(fn_mat)
      fprintf('binary content prediction file %s not found\n', fn_mat) ;
      continue
    end
    CPF=load(fn_mat);
    fieldnames(CPF);

    save_append(fnall_mat, 1, sprintf('CPF_%i', cnt), CPF) ;
    cnt = cnt + 1 ;
    ret = unix(['rm -f ' fn_mat]) ;
    assert(ret==0) ;
  end ;
end ;
if length(cpf_fname)>=3 && isequal(cpf_fname(end-2:end), '.gz'),
  unix(sprintf('mv %s %s; gzip %s', cpf_fname, cpf_fname(1:end-3), cpf_fname(1:end-3))) ;
end ;

fprintf('Done.\n\n') ;
unix(sprintf('rmdir %s', out_dir)) ;

return

function x = load_and_train(P, Train_Data)

% load_and_train(P)
%
% INPUT: P with fields:   
%         - fn_set,             file name of sets to be loaded 
%         - SETs.which_svm,     specifies the sets on which training is
%                               performed (e.g [1 2 3])
%         - SETs.train_subsample_neg  number between 0-1: for subsampling of negative examples, 
%                                0: setting label of examples labeled with 0 to -1   
%                               -1: removing examples labeled with 0 
%                               default(1) no subsampling 
%         - SETs.train_subsample_pos  number between 0-1: for subsampling of positive examples,                             
%                               default(1) no subsampling 
%         - par,                window around consensus to be used  
%         - use_cons            specifies if conservation is to be used
%         - fn_SVMs             file nmame to save results  
%  
%   
% OUTPUT: Train_Data with fields:
%  (XT,LT,SC,pos,reg,str,P)
%  saved to file P.fn_SVMs
  
%if( ~P.method.submit_batch )
%  paths;
%end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% load data of training set
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SETs = P.SETs;
Signal = P.Signal;
fn_filter_settings = P.fn_filter_settings;
fn_examples = P.fn_examples;

which_train = SETs.which_train;
train_fct = Signal.method.train_fct;

if ~isfield(SETs,'train_subsample_pos') && isfield(SETs,'subsample_pos')
  subsample_pos = SETs.subsample_pos ;
  error('warning: please rename field from subsample_pos to train_subsample_pos') ;
else
  subsample_pos = SETs.train_subsample_pos ;
end

if ~isfield(SETs,'train_subsample_neg') && isfield(SETs,'subsample_neg')
  subsample_neg = subsample_neg ;
  error('warning: please rename field from subsample_neg to train_subsample_neg') ;
else
  subsample_neg = SETs.train_subsample_neg ;
end

if ~exist('Train_Data')
  Train_Data = feval(Signal.load_fct,Signal,fn_filter_settings,fn_examples, which_train, 'train',subsample_neg, subsample_pos);
end
Train_Data.P = P;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% TRAIN 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

t = cputime;
feval(train_fct,Train_Data);
fprintf('\ntraining took %.2f min\n',(cputime-t)/60);
% if PAR.method.name=='SVM'
%   tssTrain(Train_Data)
% elseif PARn.method.name=='PWM'
%   hmm_train(Train_Data)
% else 
%   error('unknown method');
% end

% dummy return argument
x=0;

return

function PAR=A_thaliana_acc_SVM_WD(PAR,create_dir)
  
if nargin<1
  PAR=init_PAR;
elseif nargin<2
  create_dir=0;
end
PAR = A_thaliana(PAR);
PAR = acceptor(PAR);
PAR = SVM_WD(PAR);

PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = make_partitions(PAR);
PAR = create_filenames(PAR);
if create_dir
  create_directories(PAR);
end

return

function PAR=A_thaliana_don_SVM_WD(PAR,create_dir)

if nargin<1
  PAR = init_PAR;
end
if nargin<2
  create_dir=0;
end
  
PAR = A_thaliana(PAR);
PAR = donor(PAR);
PAR = SVM_WD(PAR);

PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = make_partitions(PAR);
PAR = create_filenames(PAR);
if create_dir
  create_directories(PAR);
end


return

function PAR=C_elegans_acc_SVM_WD(PAR,create_dir)
  
if nargin<1
  PAR=[];
end
if nargin<2
  create_dir=0;
end
  
PAR = C_elegans(PAR);
PAR = acceptor(PAR);
PAR = SVM_WD(PAR);

PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = make_partitions(PAR);
PAR = create_filenames(PAR);

if create_dir
  create_directories(PAR);
end

return

function PAR=C_elegans_don_SVM_WD(PAR,create_dir)
  
if nargin<1
  PAR=[];
end
if nargin<2
  create_dir=0;
end
  
PAR = C_elegans(PAR);
PAR = donor(PAR);
PAR = SVM_WD(PAR);

PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = make_partitions(PAR);
PAR = create_filenames(PAR);

if create_dir
  create_directories(PAR);
end

return

function PAR=D_melanogaster_acc_SVM_WD(PAR)
  
if nargin<1
  PAR=[];
end
  
PAR = D_melanogaster(PAR);
PAR = acceptor(PAR);
PAR = SVM_WD(PAR);

PAR = make_partitions(PAR);

PAR.date=date;
PAR.user='Jonas';
PAR.exp_name='acceptor';

%PAR.method.kernels{1}.lwin=PAR.Signals.(PAR.Signal_name).lwin;
%PAR.method.kernels{1}.rwin=PAR.Signals.(PAR.Signal_name).rwin;
PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = create_filenames(PAR);

PAR.tasks.modelsel_on = 1;
PAR.tasks.learn_plifs_on = 1;
PAR.tasks.pred_from_seq = 0;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;

% this is the setting for the splice paper
PAR.regions.cluster_paralogs = 0;

PAR.RPROC.MEMREQ = 8000 ;

PAR.method.kernels{1}.order=22 ;
PAR.method.par_ms.C = 3 ;

return

function PAR=D_melanogaster_don_SVM_WD(PAR)
  
if nargin<1
  PAR=[];
end
  
PAR = D_melanogaster(PAR);
PAR = donor(PAR);
PAR = SVM_WD(PAR);

PAR = make_partitions(PAR);

PAR.date=date;
PAR.user='Jonas';
PAR.exp_name='donor';

%PAR.method.kernels{1}.lwin=PAR.Signals.(PAR.Signal_name).lwin;
%PAR.method.kernels{1}.rwin=PAR.Signals.(PAR.Signal_name).rwin;
PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = create_filenames(PAR);

PAR.tasks.modelsel_on = 1;
PAR.tasks.learn_plifs_on =1;
PAR.tasks.pred_from_seq=0;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;

% this is the setting for the splice paper
PAR.regions.cluster_paralogs = 0;

PAR.RPROC.MEMREQ = 8000 ;

PAR.method.kernels{1}.order=22 ;
PAR.method.par_ms.C = 3 ;

return

function PAR=D_rerio_acc_SVM_WD(PAR)
  
if nargin<1
  PAR=[];
end
  
PAR = D_rerio(PAR);
PAR = acceptor(PAR);

% test only one model ... 
PAR.Signals.(PAR.Signal_name).order = 22;
PAR.Signals.(PAR.Signal_name).shift = 0;
PAR.Signals.(PAR.Signal_name).C = 3; 

PAR = SVM_WD(PAR);

PAR = make_partitions(PAR);

PAR.date=date;
PAR.user='Jonas';
PAR.exp_name='acceptor';

PAR.method.kernels{1}.lwin=PAR.Signals.(PAR.Signal_name).lwin;
PAR.method.kernels{1}.rwin=PAR.Signals.(PAR.Signal_name).rwin;
PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = create_filenames(PAR);

PAR.tasks.modelsel_on = 1;
PAR.tasks.learn_plifs_on =1;
PAR.tasks.pred_from_seq=0;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;

PAR.SETs.train_subsample_pos = 1 ;
PAR.SETs.train_subsample_neg = 0.33 ;

PAR.RPROC.options.ncpus = 12 ; 

return

function PAR=D_rerio_don_SVM_WD(PAR)
  
if nargin<1
  PAR=[];
end
  
PAR = D_rerio(PAR);
PAR = donor(PAR);

% test only one model ... 
PAR.Signals.(PAR.Signal_name).order = 22;
PAR.Signals.(PAR.Signal_name).shift = 0;
PAR.Signals.(PAR.Signal_name).C = 3; 

PAR = SVM_WD(PAR);

PAR = make_partitions(PAR);

PAR.date=date;
PAR.user='Jonas';
PAR.exp_name='donor';

PAR.method.kernels{1}.lwin=PAR.Signals.(PAR.Signal_name).lwin;
PAR.method.kernels{1}.rwin=PAR.Signals.(PAR.Signal_name).rwin;
PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = create_filenames(PAR);

PAR.tasks.modelsel_on = 1;
PAR.tasks.learn_plifs_on =1;
PAR.tasks.pred_from_seq=0;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;

PAR.SETs.train_subsample_pos = 1 ;
PAR.SETs.train_subsample_neg = 0.33 ;

PAR.RPROC.options.ncpus = 12 ; 

return

function blocks = add_signals2blocks(blocks, PAR, signal_names, which_dir, offset,which_fields)
% blocks = add_signals2blocks(blocks, PAR, signal_names, which_dir, offset)

if nargin<5
  warning('I think this should be a parameter')
  offset = 100;  
end
if nargin<4 || isempty(which_dir)
  which_dir = 'pred_dir' ;
end
genome_info = init_genome(PAR.FN.input.fn_genome_config);
which_pos.local = 1;
which_pos.contig = 1;
base_dirs    = {};

for i=1:length(signal_names),
  if isequal(which_dir,'pred_dir')'
    tmp = PAR.FN.output_sig.(signal_names{i});
    base_dirs{i} = tmp.fn_pred;
    fields = {'Conf_cum'};
  elseif isequal(which_dir,'cand_dir')'
    tmp = PAR.FN.input_sig.(signal_names{i});
    base_dirs{i} = tmp.fn_candsites;
    fields = {'label','altgenic','alt_intron','est_conf','est_conf_skip','cDNA_conf','cDNA_conf_skip'};
  end
end
if nargin>5
  fields = which_fields;
end

if ~isfield(blocks,'Signals')
  blocks(1).Signals=struct;
end

% blocks = load_sequence(blocks);
fprintf('Processing %i blocks: ', length(blocks));
for r=1:length(blocks)
  block=blocks(r);
  if mod(r,50)==0, fprintf('%i .. ', r); end ;
  % fill the field genestr
  for s=1:length(signal_names),
    f_name = sprintf('%scontig_%i%s.pos', base_dirs{s},block.chr_num,block.strand);
    d = dir(f_name) ;
    if isempty(d), error('file not found: %s',f_name);end
    if d.bytes>0
      block = retrieve_signals(block, base_dirs{s},signal_names{s}, fields, which_pos, offset);
      % block = retrieve_signals(block, base_dirs{s},signal_names{s}, fields,which_pos,0);
    else
      warning('hack!\n')
      block.Signals.(signal_names{s})=struct ;
      block.Signals.(signal_names{s}).pos = [];
      block.Signals.(signal_names{s}).contig_pos = [];
      for k=1:length(fields)
        block.Signals.(signal_names{s}).(fields{k}) =  [];
      end ;
    end
  end
  blocks(r)=block;
end
fprintf('Done.\n') ;

% load /fml/ag-raetsch/share/projects/nGASP/LSL/test_data/cat_1/blocks_2006-12-19.mat

% eof

return

function create_directories(FN,organism,verb,signal)

% create_directories(PAR)
%   
% function creates all necessary directories for a gene finding
% experiment as defined by PAR.
  
  
if nargin<3
  verb=0;
end

if  nargin>3
  signalnames{1} = signal;
else
  signalnames = fieldnames(FN.input_sig);
end


%%%%%%%%%%%%%%%
%%% GENERAL INPUT FILES 
%%%%%%%%%%%%%%%

exp_base_in = sprintf('%s/%s/%s',FN.input_dir,organism.name,FN.dir_name);

input_dir = sprintf('%s/orig_data/',exp_base_in);
if verb
  fprintf(1, 'dir for original data: \n%s\n\n', input_dir)
end
nice_mkdir(input_dir);


%%%%%%%%%%%%%%%%%%%%%%%%%%
%% OUTPUT DIRs 
%%%%%%%%%%%%%%%%%%%%%%%%%

exp_base_out = sprintf('%s/%s/%s',FN.output_dir,organism.name,FN.dir_name);
output_dir = sprintf('%s/genome_data/',exp_base_out);
if verb
  fprintf(1, 'dir for genes, split and regions: \n%s\n\n', output_dir)
end
nice_mkdir(output_dir);

nice_mkdir(FN.exp_dir);


%%%%%%%%%%%%%%%
%%% SIGNAL SPECIFIC Dirs
%%%%%%%%%%%%%%%


for s=1:length(signalnames)
  if ~isfield( FN.input_sig, signalnames{s})
    continue
  end
  signals = FN.input_sig.(signalnames{s});
  signal_fns = fieldnames(signals);
  for i=1:length(signal_fns)
    name = signals.(signal_fns{i}) ;
    iii = max(strfind(name,'/'));
    if ~isempty(iii)
      nice_mkdir(name(1:iii));
    end 
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%
%% SIGNAL SPECIFIC OUTPUT DIRs 
%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin<4
  signalnames = fieldnames(FN.output_sig);
end
  
for s=1:length(signalnames)
  if ~isfield( FN.output_sig, signalnames{s})
    continue
  end
  signals = FN.output_sig.(signalnames{s});
  signal_fns = fieldnames(signals);
  for i=1:length(signal_fns)
    name = signals.(signal_fns{i}) ;
    if isempty(strmatch(FN.output_dir,name)) &verb
      fprintf(['incomplete signal output file name, signal output directories not generated; '...
               'run config for %s\n'],signalnames{s})
      break
    end
    iii = max(strfind(name,'/'));
    if ~isempty(iii)
      nice_mkdir(name(1:iii));
    end 
  end
end

%%%%%%%%%%%%%%%
%%% CONTENT SPECIFIC Dirs
%%%%%%%%%%%%%%%

if isfield(FN, 'input_cont')
  if nargin<4
    signalnames = fieldnames(FN.input_cont);
  end
  for s=1:length(signalnames)
    if ~isfield( FN.input_cont, signalnames{s})
      continue
    end
    signals = FN.input_cont.(signalnames{s});
    signal_fns = fieldnames(signals);
    for i=1:length(signal_fns)
      name = signals.(signal_fns{i}) ;
      iii = max(strfind(name,'/'));
      if ~isempty(iii)
        nice_mkdir(name(1:iii));
      end 
    end
  end
end
%%%%%%%%%%%%%%%%%%%%%%%%%
%% CONTENT SPECIFIC OUTPUT DIRs 
%%%%%%%%%%%%%%%%%%%%%%%%%

if isfield(FN, 'output_cont')
  if nargin<4
    signalnames = fieldnames(FN.output_cont);
  end
  for s=1:length(signalnames)
    if ~isfield( FN.output_cont, signalnames{s})
      continue
    end
    signals = FN.output_cont.(signalnames{s});
    signal_fns = fieldnames(signals);
    for i=1:length(signal_fns)
      name = signals.(signal_fns{i}) ;
      if isempty(strmatch(FN.output_dir,name)) &verb
        fprintf(['incomplete signal output file name, signal output directories not generated; '...
                 'run create_filenames first for\n'])
        break
      end
      iii = max(strfind(name,'/'));
      if ~isempty(iii)
        nice_mkdir(name(1:iii));
      end 
    end
  end
end
%%%%%%%%%%%%%%%%%%%%%%%%%
%% LSL SPECIFIC
%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin<4
  
  if isfield(FN, 'output_lsl')
    fns = fieldnames(FN.output_lsl);
    for i=1:length(fns)
      name = FN.output_lsl.(fns{i}) ;
      if isempty(strmatch(FN.output_dir,name)) &verb
        fprintf('incomplete signal output file name, signal output directories not generated;')
        break
      end
      iii = max(strfind(name,'/'));
      if ~isempty(iii)
        nice_mkdir(name(1:iii));
      end 
    end
  end
  if isfield(FN, 'input_lsl')
    fns = fieldnames(FN.input_lsl);
    for i=1:length(fns)
      if ~iscell(FN.input_lsl.(fns{i}))
        name = FN.input_lsl.(fns{i}) ;
        if isempty(strmatch(FN.input_dir,name)) &verb
          fprintf('incomplete signal input file name, signal input directories not generated;')
          break
        end
        iii = max(strfind(name,'/'));
        if ~isempty(iii)
          nice_mkdir(name(1:iii));
        end 
      else
        for j=1:length(FN.input_lsl.(fns{i}))
          name = FN.input_lsl.(fns{i}){j} ;
          if isempty(strmatch(FN.input_dir,name)) &verb
            fprintf('incomplete signal input file name, signal input directories not generated;')
            break
          end
          iii = max(strfind(name,'/'));
          if ~isempty(iii)
            nice_mkdir(name(1:iii));
          end 
        end
      end
    end
  end
  
  fn = sprintf('%s/%s/current_predictions', FN.input_dir, organism.name);
  nice_mkdir(fn);

end
return

function FN = create_filenames(FN,organism,Signals,Contents,verb)

%
% FN requires fields :
% 	- input_dirC_
% 	- output_dir
%	- exp_name
%	- FN.fn_genes_train
%	- date
%	- user
%	- organism
%

if nargin<2
  verb=0;
end

if ~isfield(FN,'dir_name_ext')
  dir_name_ext = '';
else
  dir_name_ext = [ '_', FN.dir_name_ext ];
end;

if isempty(FN.dir_name)
  FN.dir_name ='';
end
if isempty(FN.exp_name)
  FN.exp_name = '';
end
if isempty(FN.lsl_exp_name)
  FN.lsl_exp_name = '';
end

assert(isfield(FN,'exp_name'))
assert(isfield(FN,'dir_name'))

%%%%%%%%%%%%%%%
%%% GENERAL INPUT GENOME FILES 
%%%%%%%%%%%%%%%
exp_base_in = sprintf('%s/%s/%s/%s',FN.genome_dir,organism.name,organism.release,organism.genebuilt);

input_fns = fieldnames(FN.genome);
for i=1:length(input_fns)
  old_name = getfield(FN.genome,input_fns{i});
  if iscell(old_name)
    new_name ={};
    for j=1:length(old_name)
      if isempty(strmatch(FN.genome_dir,old_name{j}))
        new_name{j}  = sprintf('%s/%s',exp_base_in,old_name{j} );
      else
        new_name{j}  = old_name{j} ;
      end
    end
  elseif length(strfind(old_name,'/'))==1
    exp_base_in_ = sprintf('%s/%s/%s',FN.genome_dir,organism.name,organism.release);
    new_name  = sprintf('%s/%s',exp_base_in_,old_name );
  elseif isempty(strmatch(FN.genome_dir,old_name)) 
    new_name  = sprintf('%s/%s',exp_base_in,old_name );
  else
    new_name  = old_name ;
  end
  FN.genome = setfield(FN.genome,input_fns{i},new_name);
end  
if verb
  fprintf(1, 'genebuild from dir: \n%s\n\n', exp_base_in)
end

%%%%%%%%%%%%%%%
%%% GENERAL INPUT FILES 
%%%%%%%%%%%%%%%

exp_base_in = sprintf('%s/%s/%s',FN.input_dir,organism.name,FN.dir_name);

input_fns = fieldnames(FN.input);
for i=1:length(input_fns)
  old_name = getfield(FN.input,input_fns{i});
  if iscell(old_name)
    new_name ={};
    for j=1:length(old_name)
      if isempty(strmatch(FN.input_dir,old_name{j}))
        new_name{j}  = sprintf('%s/orig_data/%s',exp_base_in,old_name{j} );
      else
        new_name{j}  = old_name{j} ;
      end
    end
  else
    if isempty(strmatch(FN.input_dir,old_name)) 
      new_name  = sprintf('%s/orig_data/%s',exp_base_in,old_name );
    else
      new_name  = old_name ;
    end
  end
  FN.input = setfield(FN.input,input_fns{i},new_name);
end  
input_dir=sprintf('%s/orig_data/',exp_base_in);
if verb
  fprintf(1, 'dir for original data: \n%s\n\n', input_dir)
end


%%%%%%%%%%%%%%%%%%%%%%%%%%
%% OUTPUT DIRs AND FNs  
%%%%%%%%%%%%%%%%%%%%%%%%%
exp_base_out = sprintf('%s/%s/%s',FN.output_dir,organism.name,FN.dir_name);

output_fns = fieldnames(FN.output);
for i=1:length(output_fns)
  old_name = getfield(FN.output,output_fns{i});
  if isempty(old_name)
    old_name = '';
  end 
  if iscell(old_name)
    new_name ={};
    for j=1:length(old_name)
      if isempty(strmatch(FN.output_dir,old_name{j})) 
        new_name{j}  = sprintf('%s/genome_data/%s',exp_base_out,old_name{j} );
      else
        new_name{j}  = old_name{j} ;
      end
    end
  else
    if  isempty(strmatch(FN.output_dir,old_name)) 
      new_name  = sprintf('%s/genome_data/%s',exp_base_out,old_name );
    else
      new_name  = old_name ;
    end
  end
  FN.output = setfield(FN.output,output_fns{i},new_name);
end  

output_dir=sprintf('%s/genome_data/',exp_base_out);
if verb
  fprintf(1, 'dir for genes, split and regions: \n%s\n\n', output_dir)
end
if ~isfield( FN,'exp_dir')||isempty(FN.exp_dir)|| isempty(strmatch(FN.input_dir,FN.exp_dir)) 
  FN.exp_dir = sprintf('%s/%s',exp_base_out,FN.exp_name);
end

%%%%%%%%%%%%%%%
%%% SIGNAL SPECIFIC INPUT FILES 
%%%%%%%%%%%%%%%

signalnames = fieldnames(FN.input_sig);
for s=1:length(signalnames)
  signals = getfield(FN.input_sig,signalnames{s});
  signal_fns = fieldnames(signals);
  for i=1:length(signal_fns)
    old_name = getfield(signals,signal_fns{i}) ;
    if isempty(old_name)
       old_name = '';
    end 
    if isempty(strmatch(FN.input_dir,old_name)) 
      new_name  = sprintf('%s/%s/%s/%s',exp_base_in,FN.exp_name,dir_name_ext,old_name );
      % new_name  = sprintf('%s/%s/sensors/%s%s/%s',exp_base_in,FN.exp_name,signalnames{s},dir_name_ext,old_name );
    else
      new_name  = old_name ; 
    end
    signals = setfield(signals,signal_fns{i},new_name);
  end 
  FN.input_sig = setfield(FN.input_sig,signalnames{s},signals);
  
  % sig_input = sprintf('%s/%s/sensors/%s%s',exp_base_in,FN.exp_name,signalnames{s},dir_name_ext,old_name);
  if verb
    fprintf(1, 'dir for label_files and examples: \n%s\n\n', new_name)
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%
%% SIGNAL SPECIFIC OUTPUT DIRs AND FNs  
%%%%%%%%%%%%%%%%%%%%%%%%%



%output_fns = fieldnames(FN.output);
%for i=1:length(output_fns)
%  old_name = getfield(FN.output,output_fns{i});
%  new_name  = sprintf('%s/%s_%s/%s',FN.output_dir,FN.exp_name,...
%                      organism.name,old_name );
%  FN.output = setfield(FN.output,output_fns{i},new_name);
%end  

signalnames = fieldnames(FN.output_sig);
for s=1:length(signalnames)
  signals = FN.output_sig.(signalnames{s});
  signal_fns = fieldnames(signals);
  method = Signals.(signalnames{s}).method;
  for i=1:length(signal_fns)
    old_name = getfield(signals,signal_fns{i}) ;
    dir_name = sprintf('%s/%s/%s',exp_base_out,FN.exp_name,dir_name_ext);
    % dir_name = sprintf('%s/%s/sensors/%s%s',exp_base_out,FN.exp_name,signalnames{s},dir_name_ext);
    if isfield(FN, 'custom_output')
      dir_name = sprintf('%s/%s',dir_name,FN.custom_output);
    else
      dir_name = sprintf('%s/output_%s',dir_name,method.name);
      if isequal(method.name, 'SVM')
        if length(method.kernels)==1&&~isempty(method.kernels{1})
          if isfield(method.kernels{1},'name')
            kernel_names = method.kernels{1}.name; 
          else
            kernel_names = method.kernels{1}.kernel_name;
          end
        else
          kernel_names = 'COMBINED';
        end
        dir_name = sprintf('%s%s',dir_name,kernel_names);
      end
    end
    if isempty(strmatch(FN.output_dir,old_name)) 
      new_name  = sprintf('%s/%s',dir_name, old_name );
    else
      new_name = old_name;
    end
    signals = setfield(signals,signal_fns{i},new_name);
  end
  FN.output_sig = setfield( FN.output_sig, signalnames{s}, signals );
  if verb
    fprintf(1, 'dir for training and test data: \n%s\n\n', dir_name) 
  end
end


%%%%%%%%%%%%%%%
%%% CONTENT SPECIFIC INPUT FILES 
%%%%%%%%%%%%%%%

if isfield(FN, 'input_cont')
  cont_names = fieldnames(FN.input_cont);
  for s=1:length(cont_names)
    content = getfield(FN.input_cont,cont_names{s});
    cont_fns = fieldnames(content);
    for i=1:length(cont_fns)
      old_name = getfield(content,cont_fns{i}) ;
      if isempty(strmatch(FN.input_dir,old_name)) 
        new_name  = sprintf('%s/%s/%s/%s',exp_base_in,FN.exp_name,dir_name_ext,old_name );
        % new_name  = sprintf('%s/%s/contents/%s%s/%s',exp_base_in,FN.exp_name, cont_names{s},dir_name_ext,old_name );
      else
        new_name  = old_name ; 
      end
      content = setfield(content,cont_fns{i},new_name);
    end 
    FN.input_cont = setfield(FN.input_cont,cont_names{s},content);
  end

  %cont_input = sprintf('%s/%s/contents/%s',exp_base_in,FN.exp_name,signal_dir_name);
  if verb
    fprintf(1, 'dir for content specific input: \n%s\n\n', cont_input)
  end
end

%%%%%%%%%%%%%%%
%%% CONTENT SPECIFIC OUTPUT FILES 
%%%%%%%%%%%%%%%

if isfield(FN, 'output_cont')
  cont_names = fieldnames(FN.output_cont);
  for s=1:length(cont_names)
    content = getfield(FN.output_cont,cont_names{s});
    cont_fns = fieldnames(content);
    method = Contents.(cont_names{s}).method;
    for i=1:length(cont_fns)
      old_name = getfield(content,cont_fns{i}) ;
      dir_name = sprintf('%s/%s/%s',exp_base_out,FN.exp_name,dir_name_ext);
      if isfield(FN, 'custom_output')
        dir_name = sprintf('%s/%s',dir_name,FN.custom_output);
      else
        dir_name = sprintf('%s/output_%s',dir_name,method.name);
        if isequal(method.name, 'SVM')
          if length(method.kernels)==1&&~isempty(method.kernels{1})
            if isfield(method.kernels{1},'name')
              kernel_names = method.kernels{1}.name; 
            else
              kernel_names = method.kernels{1}.kernel_name;
            end
          else
            kernel_names = 'COMBINED';
          end
          dir_name = sprintf('%s%s',dir_name,kernel_names);
        end
      end
      if isempty(strmatch(FN.output_dir,old_name)) 
        new_name  = sprintf('%s/%s',dir_name, old_name );
      else
        new_name = old_name;
      end
      content = setfield(content,cont_fns{i},new_name);
    end 
    FN.output_cont = setfield(FN.output_cont, cont_names{s},content);
  end

  %cont_output = sprintf('%s/%s/contents/%s',exp_base_out,FN.exp_name,signal_dir_name);
  if verb
    fprintf(1, 'dir for content specific output: \n%s\n\n', cont_output)
  end
end

%%%%%%%%%%%%%%%
%%% LSL STUFF
%%%%%%%%%%%%%%%

if isfield(FN, 'input_lsl')
  fns = fieldnames(FN.input_lsl);
  FN.input_lsl.dir = sprintf('%s/%s/%s/%s/lsl/%s/', ...
			     exp_base_out,organism.name,FN.dir_name,FN.exp_name,FN.lsl_exp_name ); 
  for i=1:length(fns)
    if ~iscell(FN.input_lsl.(fns{i}))
      old_name = FN.input_lsl.(fns{i}) ;
      if isempty(strmatch(FN.input_lsl.dir,old_name))&~isequal(fns{i},'fn_old_training_blocks')...
            &~isequal(fns{i},'fn_old_test_blocks')&~isequal(fns{i},'fn_old_split')
        new_name  = sprintf('%s/%s',FN.input_lsl.dir,old_name);
      else
        new_name  = old_name ; 
      end
      FN.input_lsl.(fns{i}) = new_name;
    else
      for j=1:length(FN.input_lsl.(fns{i}))
        old_name = FN.input_lsl.(fns{i}){j} ;
        if isempty(strmatch(FN.input_lsl.dir,old_name))&~isequal(fns{i},'fn_old_training_blocks')...
              &~isequal(fns{i},'fn_old_test_blocks')&~isequal(fns{i},'fn_old_split')
          new_name  = sprintf('%s/%s',FN.input_lsl.dir,old_name);
        else
          new_name  = old_name ; 
        end
        FN.input_lsl.(fns{i}){j} = new_name;
      end    
    end
  end
end
%cont_output = sprintf('%s/%s/contents/%s',exp_base_out,FN.exp_name,signal_dir_name);
if verb
  fprintf(1, 'dir for lsl specific input: \n%s\n\n', cont_output)
end

if isfield(FN, 'output_lsl')
  fns = fieldnames(FN.output_lsl);
  FN.output_lsl.dir = sprintf('%s/%s/%s/%s/lsl/%s/', ...
			      exp_base_out, organism.name,FN.dir_name,FN.exp_name,FN.lsl_exp_name ); 
  for i=1:length(fns)
    old_name = FN.output_lsl.(fns{i}) ;
    if isempty(strmatch(FN.output_lsl.dir,old_name))
      new_name  = sprintf('%s/%s',FN.output_lsl.dir,old_name);
    else
      new_name  = old_name ; 
    end
    FN.output_lsl.(fns{i}) = new_name;
  end 
end
%cont_output = sprintf('%s/%s/contents/%s',exp_base_out,FN.exp_name,signal_dir_name);
if verb
  fprintf(1, 'dir for lsl specific output: \n%s\n\n', cont_output)
end


return

function PAR=create_PAR(Exp, PAR_config, verb)
%  PAR=create_PAR(Exp,PAR_config,verb)
%
% creates PAR with all parameters for one genefinding
% experiment. For a new Organism X a new template file must be added to /template_files/Organisms/X.m. 
% (e.g. C_elegans.m) 
% For each signal a standard method is applied (usually SVM with one or
% more spectrum kernels)
%  
% INPUT  - Exp: filename of current experiment  (e.g. exp_C_elegans_nGASP)
%        - verb: verbose
%
% OUTPUT - PAR. struct with all information needed for genefinding experiment  
  
Signals = {'acceptor','donor','tis','cdsStop','tss','cleave','polya','transacceptor'};
sig_names = {'acc','don','tis','cdsStop','tss','cleave','polya','transacc'};
Methods = {'SVM_WD','SVM_WD','SVM_COMBINED','SVM_COMBINED','SVM_COMBINED','SVM_COMBINED','SVM_COMBINED','SVM_WD'};

content_names = {'intergenic','utr5exon','cds_exon','utr3exon','polya_tail','intron','frame0'};
Methodc = 'SVM_Cont';
changeable_fields = {'organism','label_source','Signals','Contents','LSL','RPROC','SETs','regions','tasks','Source_organisms','FN'} ;

if nargin<3,
  verb=0;
end


PAR = init_PAR;

if nargin>0
  fprintf('parameters from exp file: %s \n',Exp)
  if ~exist('PAR_config') || isempty(PAR_config),
    PAR_exp = feval(Exp) ;
  else
    try,
      PAR_exp = feval(Exp, PAR_config) ;
    catch
      PAR_exp = feval(Exp) ;
    end 
    clear PAR_config ;
  end ;
  assert(all(ismember(fieldnames(PAR_exp),changeable_fields)))
  if ~isfield(PAR_exp,'Contents')
    PAR_exp.Contents = struct;
  end
  if ~isfield(PAR_exp,'Signals')
    PAR_exp.Signals = struct;
  end
  PAR.organism = set_default(PAR_exp,'organism',PAR.organism)  ;  
  PAR.label_source = set_default(PAR_exp,'label_source',PAR.label_source) ;
  PAR.RPROC = set_default(PAR_exp,'RPROC',PAR.RPROC)  ; 
  PAR.SETs = set_default(PAR_exp,'SETs',PAR.SETs)  ; 
  PAR.SETs = make_partitions(PAR.SETs);
  PAR.regions = set_default(PAR_exp,'regions',PAR.regions)  ; 
  PAR.tasks = set_default(PAR_exp,'tasks',PAR.tasks)  ; 
  PAR.Source_organisms = set_default(PAR_exp,'Source_organisms',PAR.Source_organisms)  ; 
end

if isempty(PAR.organism.name),
  [organism, label_source] = dummy_organism(PAR.label_source, sig_names);
else
  [organism, label_source] = feval(PAR.organism.name,PAR.label_source,sig_names);
end
PAR.organism = set_default(PAR,'organism',organism)  ;
PAR.label_source = set_default(PAR,'label_source',label_source)  ;

%-----------------------------
%%% prepare all signals
%-----------------------------

for i=1:length(Signals)
  Signal = feval(Signals{i},PAR.label_source);
  if exist('PAR_exp')
    if isfield(PAR_exp.Signals, sig_names{i}) && isfield(PAR_exp.Signals.(sig_names{i}),'method')
      me = PAR_exp.Signals.(sig_names{i}).method;
      PAR_exp.Signals.(sig_names{i}) = rmfield(PAR_exp.Signals.(sig_names{i}),'method');
    end
    Signal = set_default(PAR_exp.Signals,sig_names{i},Signal)  ;
  end
  Signal.method = feval(Methods{i},Signal);
  if exist('me')
    PAR_exp.Signals.(sig_names{i}).method = me;
    Signal = set_default(PAR_exp.Signals,sig_names{i},Signal)  ;
    clear me
  end
  % Signal = Signal.domain_adapt.transfer_fct(Signal);
  PAR.Signals.(sig_names{i}) = Signal;
end

%-----------------------------
%%% prepare all contents
%-----------------------------
for i=1:length(content_names)
  Content = feval(content_names{i});
  if exist('PAR_exp')
    if isfield(PAR_exp.Contents, content_names{i}) && isfield(PAR_exp.Contents.(content_names{i}),'method')
      me = PAR_exp.Contents.(content_names{i}).method;
      PAR_exp.Contents.(content_names{i}) = rmfield(PAR_exp.Contents.(content_names{i}),'method');
    end
    Content = set_default(PAR_exp.Contents,content_names{i},Content)  ;
  end
  Content.method = feval(Methodc,Content);
  if exist('me')
    PAR_exp.Contents.(content_names{i}).method = me;
    Content = set_default(PAR_exp.Contents,content_names{i},Content)  ;
    clear me
  end
  % Content = Content.domain_adapt.transfer_fct(Content);
  PAR.Contents.(content_names{i}) = Content;
end

%-----------------------------
%%% prepare LSL
%-----------------------------

LSL = lsl_template; 
% LSL = LSL.domain_adapt.transfer_fct(LSL);
if exist('PAR_exp')
  LSL = set_default(PAR_exp,'LSL',LSL)  ; 
end
PAR.LSL=LSL;

%-----------------------------
%%% prepare filenames
%-----------------------------
FN = init_FN;
if  exist('PAR_exp')
  FN = set_default(PAR_exp,'FN',FN)  ;
end

FN = create_filenames(FN,PAR.organism,PAR.Signals,PAR.Contents,verb);
if  exist('PAR_exp')
  FN = set_default(PAR_exp,'FN',FN)  ;
end
PAR.FN=FN;

%-----------------------------
%%% Add polya consensus if file available
%-----------------------------

if fexist(PAR.FN.input_sig.polya.fn_polya)
  load(PAR.FN.input_sig.polya.fn_polya,'consensus')
  PAR.Signals.polya.consensus = consensus;
end

%PAR.FN.source_directory = '~/svn/projects/genefinding/';
%[tmp,PAR.FN.svn_info]=unix('svn info');

PAR.FN.svn_info = '' ;

fprintf('loading model from %s\n', PAR.LSL.method.model_fct)
PAR.model = feval(PAR.LSL.method.model_fct,PAR.organism,PAR.LSL, ...
                  PAR.Signals);


return

function [fn, exists]=gen_example_filename(Signal,fn_filter_settings,fn_examples,which,split)

% [fn, exists]=gen_example_filename(Signal,fn_filter_settings,fn_examples,which,split)
% [fn, exists]=gen_example_filename(PAR,which,split)
% which is 'train','eval','test'

% sig = PAR.Signal_name;
% Signal = PAR.Signals.(sig); 
% fn_filter_settings =  PAR.FN.input_sig.(sig).input_sig;
% fn_examples = PAR.FN.input_sig.(sig).fn_examples;

whichs = fieldnames(Signal.filter_label);


if isnumeric(which)
  assert(which<=length(whichs))
  which = whichs{which};
else
  assert(ismember(which,whichs))
end
clear whichs

current_settings = Signal.filter_label.(which);

if fexist(fn_filter_settings)
  load(fn_filter_settings, 'filter_settings')
  idx = find([filter_settings.split]==split);
else
  filter_settings = [];
  idx = [];
end
idx_setting = [];is_equal=0;
for i=idx
  temp = rmfield(filter_settings(i),'split');
  temp = rmfield(temp,'filename');
  if isequal(temp, current_settings)
    idx_setting = [idx_setting i];
    is_equal=1;
  else
    assert(length(fieldnames(temp))==length(fieldnames(current_settings))&&...
           all(strcmp(sort(fieldnames(temp)),sort(fieldnames(current_settings)))))
  end
end
assert(length(idx_setting)<=1);
if isempty(idx_setting)
  temp = struct;
end
assert(length(setdiff(fieldnames(temp),fieldnames(current_settings)))==0)

if ~isempty(idx_setting)
  fn = filter_settings(idx_setting).filename;
  if fexist(fn)
    exists = 1;
  else
    exists = 0;
  end
else 
  settings_names = fieldnames(current_settings);
  s = length(filter_settings)+1;
  for j = 1:length(settings_names)
    filter_settings(s).(settings_names{j}) = current_settings.(settings_names{j});
  end
  filter_settings(s).split = split;
  filter_settings(s).filename = sprintf('%s_settings=%i_split=%i.mat',fn_examples, s, split);
  fn = filter_settings(s).filename;
  if fexist(fn)
    error('I do not know where that file comes from!!!')
  else
    exists = 0;
  end 
  save(fn_filter_settings, 'filter_settings')
end



       
%fn = sprintf('%s_%s_pc=%i_pa=%i_nc=%i_na=%i_split=%i.mat',...
%	input_sig.fn_examples,PAR.Signal_name,pos_conf,...
%	pos_anno,neg_conf,neg_anno, s);

return

function regions=init_regions(fn_genome_config, organism)
% regions=init_regions(fn_genome_config,organism)

  
regions(1).chr = [];
regions(1).chr_num = [];
regions(1).strand = [];
regions(1).start = [];
regions(1).stop = [];
regions(1).offset =[];
regions(1).id = [];
regions(1).organism = [];
regions(1).config = [];

if ~exist('fn_genome_config')
  return ;
else
  regions = regions([]) ;
  genome_config = init_genome(fn_genome_config);
  strands = '+-';
  for c=1:length(genome_config.contig_names)
    %+strand
    d = dir(genome_config.flat_fnames{c}) ;
    for s=1:length(strands)
      region = [];
      region.chr=genome_config.contig_names{c};
      region.chr_num = c;
      region.strand=strands(s);
      region.start = 1;
      region.stop = d.bytes ;
      switch strands(s)
       case '+',
        region.offset      = region.start - 1;
       case '-',
        region.offset      = region.stop + 1;
      end
      region.id = (c-1)*2+s;
      region.organism = [];
      if exist('organism')
        region.organism = organism;
      end
      region.config = fn_genome_config;
      regions(region.id) = region;
    end
  end
end


return

function SETs = make_partitions(SETs)
 

if SETs.num_partitions==1
  return
end
  
SETs.partitions = eye(SETs.num_partitions,SETs.num_partitions)+1;  
for i = 1:2:SETs.num_partitions-1
  SETs.partitions(i:i+1,i:i+1) = SETs.partitions(i:i+1,i:i+1)+1;
end
if mod(SETs.num_partitions,2)
  SETs.partitions(end,end-1:end) =  SETs.partitions(end,end-1:end)+1;
end
if SETs.num_splits~=SETs.num_partitions
  assert(mod(SETs.num_splits,SETs.num_partitions)==0)
  SETs.partitions = repmat(SETs.partitions,1,SETs.num_splits/SETs.num_partitions);
end

if ~isfield(SETs,'partition_train_on') 
  [tmp,idx] = unique(SETs.partitions==1,'rows');
  SETs.partition_train_on = zeros(1,SETs.num_partitions);
  SETs.partition_train_on(idx) = 1;
end



SETs.info.train = 1;
SETs.info.eval = 2;
SETs.info.plif = 2;
SETs.info.pred = 3;

for i=1:SETs.num_partitions
  assert(isequal(unique(SETs.partitions(i,:)),unique([SETs.info.train ...
                      SETs.info.eval SETs.info.plif SETs.info.pred ])))
end

assert(all(sum(SETs.partitions==SETs.info.pred)==1))
assert(length(unique(sum(SETs.partitions'==SETs.info.train)))==1)
assert(length(unique(sum(SETs.partitions'==SETs.info.eval)))==1)
assert(length(unique(sum(SETs.partitions'==SETs.info.plif)))==1)
assert(length(unique(sum(SETs.partitions'==SETs.info.pred)))==1)




return

function PAR=masterscript_data(PAR,first)
  
%PAR=masterscript_data(PAR,first)

if nargin<2
  first =1 ;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% PREPARE COMPLETE GENE STRUCTURE 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if PAR.tasks.prepare_labels
  if first==1 
    fprintf('start preparing gene structure for organism: %s\n\n',PAR.organism.name)
    PAR.add_signals.tss = 1; 
    PAR.add_signals.cleave = 1 ; 
    PAR.add_signals.polya = 1;
    PAR.add_signals.tis = 1;
    PAR.add_signals.cdsStop = 1;
    PAR.add_signals.transacc = 0; 
  elseif first==2
    fprintf('adding transacc sites to gene structure for organism: %s\n\n',PAR.organism.name)
    PAR.add_signals.tss = 0; 
    PAR.add_signals.cleave = 0 ; 
    PAR.add_signals.polya = 0;
    PAR.add_signals.tis = 0;
    PAR.add_signals.cdsStop = 0;
    PAR.add_signals.transacc = 1; 
  end
  
  [genes_sensors,genes_merged] = prepare_complete_genes(PAR);
  if first==2
    PAR.add_signals.tss = 1; 
    PAR.add_signals.cleave = 1 ; 
    PAR.add_signals.polya = 1;
    PAR.add_signals.tis = 1;
    PAR.add_signals.cdsStop = 1;
    PAR.add_signals.transacc = 1; 
  end
  genes = genes_sensors;
  if fexist(PAR.FN.output.fn_genes_signals) || fexist([PAR.FN.output.fn_genes_signals '.mat']),
    warning(sprintf('do not save file as it already exists: %s', PAR.FN.output.fn_genes_signals)) ;
    L=load_struct(PAR.FN.output.fn_genes_signals, 'genes') ;
    assert(isequal(L, genes)) ;
    clear L
  else
    save_struct(PAR.FN.output.fn_genes_signals,genes,'genes')
    %save(PAR.FN.output.fn_genes_signals,'-append','PAR')
    save_append(PAR.FN.output.fn_genes_signals, 1, 'PAR', PAR)
  end ;
  genes = genes_merged;
  if fexist(PAR.FN.output.fn_genes_merged) || fexist([PAR.FN.output.fn_genes_merged '.mat']),
    warning(sprintf('do not save file as it already exists: %s', PAR.FN.output.fn_genes_signals)) ;
    L=load_struct(PAR.FN.output.fn_genes_merged, 'genes') ;
    assert(isequal(L, genes)) ;
    clear L
  else
    save_struct(PAR.FN.output.fn_genes_merged,genes,'genes')
    %save(PAR.FN.output.fn_genes_signals,'-append','PAR')
    save_append(PAR.FN.output.fn_genes_signals, 1, 'PAR', PAR)
  end 
  clear genes_merged, genes_sensors
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% PREPARE REGIONS AND BLOCKS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% keyboard
if first==1 &...
         (~fexist(PAR.FN.output.fn_trivial_regions) |...
          ~fexist(PAR.FN.output.fn_blocks)|...
          ~fexist(PAR.FN.output.fn_test_blocks))
  [trivial_regions,blocks,training,test] = prepare_regions(PAR);
  if ~fexist(PAR.FN.output.fn_trivial_regions)| ~fexist(PAR.FN.output.fn_blocks)  
    save(PAR.FN.output.fn_trivial_regions,'trivial_regions','PAR');
    save(PAR.FN.output.fn_blocks, 'blocks','PAR');
  end
  if 1 % ~isempty(training.blocks)&~fexist(PAR.FN.output.fn_training_blocks)
    %save(PAR.FN.output.fn_training_blocks,'-struct','training');
    save_append(PAR.FN.output.fn_training_blocks, 0, training);
    %save(PAR.FN.output.fn_training_blocks,'-append' ,'PAR');
    save_append(PAR.FN.output.fn_training_blocks, 1, 'PAR', PAR);
    PAR.regions.offset=PAR.regions.offset_train;
    save_splits2bed(training.blocks,training.split,PAR.FN.output.fn_training_blocks(1:end-4),PAR);
  end
  if 1 % ~isempty(test.blocks)&~fexist(PAR.FN.output.fn_test_blocks)
    %save(PAR.FN.output.fn_test_blocks,'-struct','test');
    save_append(PAR.FN.output.fn_test_blocks, 0, test);
    %save(PAR.FN.output.fn_test_blocks,'-append','PAR');
    save_append(PAR.FN.output.fn_test_blocks, 1, 'PAR', PAR);
    PAR.regions.offset=PAR.regions.offset_test;
    save_splits2bed(test.blocks,test.split,PAR.FN.output.fn_test_blocks(1:end-4),PAR);
  end
end

return


function [ ok ] = nice_mkdir( dir );
if ~isdir(dir)
  ok = ( 0 == unix( [ 'mkdir -p ', dir ] ) );
  ok = ok & ( 0 == unix( [ 'chmod g+t ', dir ] ) );
end


return

function [trivial_regions,blocks,training,test] = prepare_regions(PAR);
%  [trivial_regions,training_split,training_blocks,test_split,test_blocks] = prepare_regions(PAR,genes);


% generate trivial regions
if fexist(PAR.FN.output.fn_trivial_regions)
  fprintf(1,'loading trivial regions from file %s\n',PAR.FN.output.fn_trivial_regions)
  load(PAR.FN.output.fn_trivial_regions,'trivial_regions');
  assert(isequal(trivial_regions(1).config,PAR.FN.input.fn_genome_config));
else
  fprintf(1,'initializing trivial regions\n')
  trivial_regions = init_regions(PAR);
  save(PAR.FN.output.fn_trivial_regions,'trivial_regions');
end

% generate training and test blocks and splits, 
% considering training and test region
if fexist(PAR.FN.output.fn_training_blocks)& fexist(PAR.FN.output.fn_test_blocks)
  Old = load(PAR.FN.output.fn_training_blocks, 'PAR');
  assert(isequal(Old.PAR.FN.output.fn_trivial_regions,PAR.FN.output.fn_trivial_regions));
  assert(isequal(Old.PAR.FN.input.fn_genome_config,PAR.FN.input.fn_genome_config));
  diff_fields = compare_PAR(PAR,Old.PAR) ;
  if ismember('regions',diff_fields)
    warning('regions exist, Parameters are different ')
    %fprintf('regions exist, Parameters are different ')
  else
    fprintf(1,'loading split and blocks from file %s \n',PAR.FN.output.fn_training_blocks)
    training = load(PAR.FN.output.fn_training_blocks)
    assert(length(fieldnames(training.split))==PAR.SETs.num_splits)
  end
  Old = load(PAR.FN.output.fn_test_blocks, 'PAR');
  assert(isequal(Old.PAR.FN.output.fn_trivial_regions,PAR.FN.output.fn_trivial_regions));
  assert(isequal(Old.PAR.FN.input.fn_genome_config,PAR.FN.input.fn_genome_config));
  diff_fields = compare_PAR(PAR,Old.PAR) ;
  if ismember('regions',diff_fields)
    warning('regions exist, Parameters are different ')
    %fprintf('regions exist, Parameters are different ')
  else
    fprintf(1,'loading split and blocks from file %s \n',PAR.FN.output.fn_test_blocks)
    test = load(PAR.FN.output.fn_test_blocks)
    assert(length(fieldnames(test.split))==PAR.SETs.num_splits)
  end
else
  if isfield(PAR.FN.output,'fn_genes_merged') && fexist(PAR.FN.output.fn_genes_merged)
    load(PAR.FN.output.fn_genes_merged,'genes');
  else
    genes=[];
  end
  [training, test] = assign_subsets(trivial_regions,PAR.FN.input.fn_trainings_regions,PAR.FN.input.fn_test_regions,genes,PAR.SETs,PAR.regions,PAR.tasks)
end

if 1
  if isfield(PAR.FN.output,'fn_genes_merged') && fexist(PAR.FN.output.fn_genes_merged)
    load(PAR.FN.output.fn_genes_merged,'genes');
  else
    genes=[];
  end
  load(PAR.FN.input.fn_trainings_regions,'training_regions');
  load(PAR.FN.input.fn_test_regions,'test_regions');
  load(PAR.FN.output.fn_trivial_regions,'trivial_regions');
  if  ~exist('training')
    training = load(PAR.FN.output.fn_training_blocks);
  end
  if  ~exist('test')
    test = load(PAR.FN.output.fn_test_blocks);
  end
  % load(PAR.FN.output.fn_blocks,'blocks');
  blocks =test.blocks;
  check_and_plot_splits(trivial_regions(1:min(10,length(trivial_regions))),training_regions,test_regions,genes,blocks,training, test)
end
  

return

function model = define_addContentFeatures(model)
  
% plifs for tiling array data
if model.use.addfeatures.tiling
  tiling_start =  model.cnt_contents + model.cnt_lengths + model.cnt_signals;
  content_names = fieldnames(model.contents);
  model.tiling_plif_ids = [];
  for s = 1:length(content_names)
    model.tiling.(content_names{s}) = tiling_start+s ;
    model.tiling_plif_ids = [model.tiling_plif_ids tiling_start+s];
  end
  model.cnt_tiling = length(fieldnames(model.tiling));
  % declare range
  model.tiling_range = struct;
  tiling_names = fieldnames(model.tiling) ;
  for s = 1:length(tiling_names)
    model.tiling_range = setfield(model.tiling_range, tiling_names{s},[6,11]);% ca. 2%-98% Percentile
  end
else
  model.cnt_tiling = 0;
end

return

function model = define_contents(model)
  
% declare content types

fid = 1;  
model.mod_words = [];
% the first contents consider all k-mers. the last three
% frame-content sensors only consider codons, with an offset from
% the end of the segment

content_start =  1 ;
cnt=0;

model.contents.intergenic = content_start+cnt ;cnt=cnt+1;
model.mod_words = [model.mod_words; 1 0] ;
model.contents.utr_exon   = content_start+cnt ;cnt=cnt+1;
model.mod_words = [model.mod_words; 1 0] ;
model.contents.cds_exon   = content_start+cnt ;cnt=cnt+1;
model.mod_words = [model.mod_words; 1 0] ;
model.contents.intron     = content_start+cnt ;cnt=cnt+1;
model.mod_words = [model.mod_words; 1 0] ;
if model.use.segments.intercistronic
  model.contents.intercistronic = content_start+cnt;cnt=cnt+1;
  model.mod_words = [model.mod_words; 1 0] ;
end
if model.use.contents.frame
  model.contents.cds_frame0   = content_start+cnt ;cnt=cnt+1;
  model.mod_words = [model.mod_words; 3 0] ;
  model.contents.cds_frame1   = content_start+cnt ;cnt=cnt+1;
  model.mod_words = [model.mod_words; 3 1] ;
  model.contents.cds_frame2   = content_start+cnt ;cnt=cnt+1;
  model.mod_words = [model.mod_words; 3 2] ;
end
model.cnt_contents = length(fieldnames(model.contents));

%hack
if model.cnt_contents<8
  fprintf(fid,'introducing dummy contents\n')
  for i= 1:8-model.cnt_contents
    if i==1
      model.contents.('dummy')= content_start+cnt;cnt=cnt+1;
    else
      model.contents.(['dummy' num2str(i)])= content_start+cnt;cnt=cnt+1;
    end
  end
  model.cnt_contents = length(fieldnames(model.contents));
  model.mod_words = [model.mod_words; 1 0] ;
end

% declare content range
model.contents_range = struct;
content_names = fieldnames(model.contents) ;
for s = 1:length(content_names)
  model.contents_range = setfield(model.contents_range, content_names{s},[-10,10]);
end


return

function model = define_lengths(model,organism)
  
fid = 1; 
% declare length types
length_start = model.cnt_contents+1 ;
cnt = 0;

model.lengths.intergenic = length_start+cnt; cnt = cnt+1;
model.lengths.utr5exon = length_start+cnt ;cnt = cnt+1;
if model.use.segments.transexon 
  model.lengths.transexon = length_start+cnt ;cnt = cnt+1;
end
model.lengths.single_cds_exon = length_start+cnt ;cnt = cnt+1;
model.lengths.first_cds_exon = length_start+cnt ;cnt = cnt+1;
model.lengths.middle_cds_exon = length_start+cnt ;cnt = cnt+1;
model.lengths.last_cds_exon = length_start+cnt ;cnt = cnt+1;
model.lengths.utr3exon = length_start+cnt;cnt = cnt+1;
if model.use.segments.polya_tail 
  model.lengths.polya_tail = length_start+cnt;cnt = cnt+1;
end
model.lengths.intron = length_start+cnt ;cnt = cnt+1;
if model.use.segments.intercistronic
  model.lengths.intercistronic = length_start+cnt ;cnt = cnt+1;
  model.lengths.intergenictrans = length_start+cnt ;cnt = cnt+1;
end
model.lengths.intergenic_long = length_start+cnt ;cnt = cnt+1;

model.cnt_lengths = length(fieldnames(model.lengths));


% declare length range
s1=which([organism.name '_lengths']);
s2=which([organism.clade '_lengths']);
if ~isempty(s1)
  fprintf(fid,'taking lengths range from file: %s_lengths.m \n',organism.name)
  model.lengths_range = feval([organism.name '_lengths']);
elseif ~isempty(s2)
  fprintf(fid,'taking lengths range from file: %s_lengths.m \n',organism.clade)
  model.lengths_range = feval([organism.clade '_lengths']);
else
  fprintf(fid,'no lengths range specified, using default settings\n')
  model.lengths_range = feval('default_lengths');
end

return

function model = define_loss(model)
  

% create loss matrices

equiv = [];
  
different = [model.segments.intergenic, model.segments.cds_exon, 3 ;
             model.segments.intergenic, model.segments.intron, 3;
             model.segments.intron, model.segments.cds_exon, 2];

%--------------------------------
if ~model.use.loss.orf_only
  different = [different;
               model.segments.intergenic, model.segments.utr5exon, 2;
               model.segments.intergenic, model.segments.utr3exon, 2];
  if model.use.segments.intercistronic
    different = [different;
                 model.segments.intercistronic, model.segments.utr5exon, 2;
                 model.segments.intercistronic, model.segments.utr3exon, 2;
                 model.segments.intergenictrans, model.segments.utr5exon, 2;
                 model.segments.intergenictrans, model.segments.utr3exon, 2];
    
  end
else
  equiv = [equiv;
           model.segments.intergenic model.segments.utr3exon ;
           model.segments.intergenic model.segments.utr5exon] ;
  if model.use.segments.intercistronic
    equiv = [equiv;
             model.segments.intercistronic model.segments.utr3exon ; 
             model.segments.intercistronic model.segments.utr5exon ;
             model.segments.intergenictrans model.segments.utr3exon ;
             model.segments.intergenictrans model.segments.utr5exon] ;
  end
end
%--------------------------------
if model.use.segments.polya_tail
  equiv = [equiv;
           model.segments.utr3exon, model.segments.polya_tail];
  different = [different;
               model.segments.intergenic, model.segments.polya_tail, 1];    
end
 %-------------------------------- 
if model.use.segments.intercistronic
  
  different = [different;
               model.segments.intercistronic, model.segments.cds_exon, 3 ;
               model.segments.intercistronic, model.segments.intron, 3;
               model.segments.intercistronic, model.segments.transexon, 2;
               model.segments.intergenic, model.segments.intercistronic, 0.1;
               model.segments.intergenictrans, model.segments.intercistronic, 0.1;
               model.segments.intergenictrans, model.segments.cds_exon, 3 ;
               model.segments.intergenictrans, model.segments.intron, 3;
               model.segments.intergenictrans, model.segments.transexon, 2;
               model.segments.intergenic, model.segments.intergenictrans, 0.1];

  if model.use.segments.polya_tail
    different = [different;
                 model.segments.intercistronic, model.segments.polya_tail,1;
                 model.segments.intergenictrans, model.segments.polya_tail, 1];
  end
end
%--------------------------------
if model.use.segments.transexon
  equiv = [equiv;
           model.segments.utr5exon, model.segments.transexon] ;     
  different = [different;
               model.segments.intergenic, model.segments.transexon, 1] ;
  if isfield(model.segments, 'intergenictrans'),
    different = [different;
                 model.segments.intergenictrans, model.segments.transexon, 1];    
  end ;
end
%--------------------------------

model.loss.segments = ones(model.cnt_segments) - eye(model.cnt_segments);
for i=1:size(different,1)
  model.loss.segments(different(i,1)+1, different(i,2)+1) = different(i,3) ;
  % model.loss.segments(different(i,1)+1, different(i,2)+1) = different(i,3)+1 ;warning('make loss matrix asymetric to enhance sensitivity')
  model.loss.segments(different(i,2)+1, different(i,1)+1) = different(i,3) ;
end ;

for i=1:size(equiv,1)
  model.loss.segments(equiv(i,1)+1, equiv(i,2)+1) = 0 ;
  model.loss.segments(equiv(i,2)+1, equiv(i,1)+1) = 0 ;
end ;


% tripple loss for coding
%model.loss.segments(model.segments.cds_exon+1, :) = 3*model.loss.segments(model.segments.cds_exon+1, :) ;
%model.loss.segments(:, model.segments.cds_exon+1) = 3*model.loss.segments(:, model.segments.cds_exon+1) ;

% first part is for segments, the second for nucleotides
model.loss.segments = [model.loss.segments 0.0*model.loss.segments] ;

if model.use.signals.tss
  model.loss.zero_range.tss = 10 ;
end
model.loss.zero_range.tis = 0;
model.loss.zero_range.don = 0;
model.loss.zero_range.acc = 0;
if model.use.signals.trans
  model.loss.zero_range.trans = 0;
end
model.loss.zero_range.cdsStop = 0;
if model.use.signals.polya
  model.loss.zero_range.polya = 10 ;
end
if model.use.signals.cleave
  model.loss.zero_range.cleave = 10 ;
end

model.loss.confirmed = 1 ;
model.loss.confirmed_tol = 1000 ; % region around confirmed gene
model.loss.unconfirmed = 1 ;

% model.loss.mustbe_corr.intergenic = 0 ;
% model.loss.mustbe_corr.utr5exon   = 0;
% model.loss.mustbe_corr.transexon  = 0 ;
% model.loss.mustbe_corr.cds_exon   = 1 ;
% model.loss.mustbe_corr.utr3exon   = 0 ;
% model.loss.mustbe_corr.polya_tail = 0 ;
% model.loss.mustbe_corr.intercistronic  = 0 ;
% model.loss.mustbe_corr.intron     = 1 ;
return ;

return

function model = define_mappings(model)
  

%%mappings between length and contents
  
model.plif_links = [] ;
model.plif_links(end+1,:) = [model.lengths.intergenic      model.contents.intergenic] ;
model.plif_links(end+1,:) = [model.lengths.first_cds_exon  model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.last_cds_exon   model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.middle_cds_exon model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.single_cds_exon model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.intron          model.contents.intron] ;
if model.use.segments.transexon
  model.plif_links(end+1,:) = [model.lengths.transexon       model.contents.utr_exon] ;
end
model.plif_links(end+1,:) = [model.lengths.utr5exon        model.contents.utr_exon] ;
model.plif_links(end+1,:) = [model.lengths.utr3exon        model.contents.utr_exon] ;
if model.use.segments.polya_tail
  model.plif_links(end+1,:) = [model.lengths.polya_tail      model.contents.utr_exon] ;
end
if model.use.segments.intercistronic
  model.plif_links(end+1,:) = [model.lengths.intercistronic  model.contents.intercistronic] ;
  model.plif_links(end+1,:) = [model.lengths.intergenictrans  model.contents.intergenic] ;
end
model.plif_links(end+1,:) = [model.lengths.intergenic_long model.contents.intergenic] ;


if isfield(model.state_ids, 'intron_long') 
  model.plif_links(end+1,:) = [model.lengths.intron_long model.contents.intron] ;  
end

%%mappings between length and additional contents (tiling)

if model.use.addfeatures.tiling
  model.tiling_links = [] ;   
  model.tiling_links(end+1,:) = [model.lengths.intergenic      model.tiling.intergenic] ;
  model.tiling_links(end+1,:) = [model.lengths.first_cds_exon  model.tiling.cds_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.last_cds_exon   model.tiling.cds_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.middle_cds_exon model.tiling.cds_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.single_cds_exon model.tiling.cds_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.intron          model.tiling.intron] ;
  if model.use.segments.transexon
    model.tiling_links(end+1,:) = [model.lengths.transexon       model.tiling.transexon] ;
  end
  model.tiling_links(end+1,:) = [model.lengths.utr5exon        model.tiling.utr_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.utr3exon        model.tiling.utr_exon] ;
  if model.use.segments.polya_tail
    model.tiling_links(end+1,:) = [model.lengths.polya_tail      model.tiling.utr_exon] ;
  end
  model.tiling_links(end+1,:) = [model.lengths.intergenic_long model.tiling.intergenic] ;
  if model.use.segments.intercistronic
    model.tiling_links(end+1,:) = [model.lengths.intercistronic  model.tiling.intercistronic] ;
  end
end

%%mappings between length and segments

model.seg_links = [] ;
model.seg_links(end+1,:) = [model.lengths.intergenic      model.segments.intergenic] ;
model.seg_links(end+1,:) = [model.lengths.first_cds_exon  model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.last_cds_exon   model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.middle_cds_exon model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.single_cds_exon model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.intron          model.segments.intron] ;
if model.use.segments.transexon
  model.seg_links(end+1,:) = [model.lengths.transexon       model.segments.transexon] ;
end
model.seg_links(end+1,:) = [model.lengths.utr5exon        model.segments.utr5exon] ;
model.seg_links(end+1,:) = [model.lengths.utr3exon        model.segments.utr3exon] ;
if model.use.segments.polya_tail
  model.seg_links(end+1,:) = [model.lengths.polya_tail      model.segments.polya_tail] ;
end
if model.use.segments.intercistronic
  model.seg_links(end+1,:) = [model.lengths.intergenictrans  model.segments.intergenictrans] ;
  model.seg_links(end+1,:) = [model.lengths.intercistronic  model.segments.intercistronic] ;
end
model.seg_links(end+1,:) = [model.lengths.intergenic_long model.segments.intergenic] ;
%

return

function model = define_segments(model)
% declare segment types

  cnt=0;  
model.segments.intergenic = cnt ;cnt=cnt+1;
model.segments.utr5exon   = cnt ;cnt=cnt+1;
if model.use.segments.transexon 
  model.segments.transexon  = cnt ;cnt=cnt+1;
end
model.segments.cds_exon   = cnt ;cnt=cnt+1;
model.segments.utr3exon   = cnt ;cnt=cnt+1;
if model.use.segments.polya_tail
  model.segments.polya_tail = cnt ;cnt=cnt+1;
end
if model.use.segments.intercistronic
  model.segments.intercistronic  = cnt ;cnt=cnt+1;
  model.segments.intergenictrans  = cnt ;cnt=cnt+1;
end
model.segments.intron     = cnt ;cnt=cnt+1;

model.cnt_segments = length(fieldnames(model.segments));


return

function model = define_signals(model,polya_sixmers)
  
% declare signal types
cnt=1;
signal_start = model.cnt_contents + model.cnt_lengths ;

if model.use.signals.tss
  model.signals.tss = signal_start+cnt;
  model.consensus.tss = {} ;
  model.consensus_pos.tss = [];
  cnt=cnt+1;
end

model.signals.tis = signal_start+cnt;
model.consensus.tis = {'atg'} ;
model.consensus_pos.tis = [0];
cnt=cnt+1;

model.signals.don = signal_start+cnt;
model.consensus.don = {'gt','gc'} ;
model.consensus_pos.don = [0,0];
cnt=cnt+1;

model.signals.acc = signal_start+cnt;
model.consensus.acc = {'ag'} ;
model.consensus_pos.acc = [2];
cnt=cnt+1;

model.signals.cdsStop = signal_start+cnt ;
model.consensus.cdsStop = {'taa','tga','tag'} ;
model.consensus_pos.cdsStop = [0,0,0];
cnt=cnt+1;

if model.use.signals.polya
  model.signals.polya = signal_start+cnt ;
  model.consensus.polya = polya_sixmers;
  model.consensus_pos.polya = zeros(1,length(polya_sixmers)) ;
  cnt=cnt+1;
end
if model.use.signals.cleave
  model.signals.cleave = signal_start+cnt ;
  model.consensus.cleave = {} ;
  model.consensus_pos.cleave = [];
  cnt=cnt+1;
end

if model.use.signals.trans
  model.signals.trans = signal_start+cnt;
  model.consensus.trans = {'ag'} ;
  model.consensus_pos.trans = [2];
  cnt=cnt+1;
end
if model.use.signals.transacc
  model.signals.transacc = signal_start+cnt;
  model.consensus.transacc = {'ag'} ;
  model.consensus_pos.transacc = [2];
  cnt=cnt+1;
end

model.cnt_signals = length(fieldnames(model.signals));


% declare signal ranges for PLiFs

if model.use.signals.tss
  model.signals_range.tss = [0,1] ;
end
model.signals_range.tis = [0,1] ;
model.signals_range.don = [0,1] ;
model.signals_range.acc = [0,1] ;
model.signals_range.cdsStop = [0,1] ;
if model.use.signals.polya
  model.signals_range.polya = [0,1] ;
end
if model.use.signals.cleave
  model.signals_range.cleave = [0,1] ;
end
if model.use.signals.trans
  model.signals_range.trans = [0,1] ;
end
if model.use.signals.transacc
  model.signals_range.transacc = [0,1] ;
end



return

function model = define_states(model)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% define states
model.states(1) = struct ;
cnt=0 ;

% sequence start 
cnt=cnt+1 ;
model.states(cnt).consensus={} ;
model.states(cnt).consensus_pos=[] ;
model.states(cnt).non_consensus={} ;
model.states(cnt).non_consensus_pos=[] ;
model.states(cnt).name='seq_start' ;
model.states(cnt).signal=[] ;
model.state_ids.seq_start=cnt ;

signal_names = fieldnames(model.signals) ;
for i=1:model.cnt_signals
  if strcmp(signal_names{i}, 'transacc'), continue; end ;
  if ~strcmp(signal_names{i},'acc') && ~strcmp(signal_names{i},'don') ...
        && ~strcmp(signal_names{i}, 'tis') && ~strcmp(signal_names{i}, 'cdsStop')...
        && ~strcmp(signal_names{i}, 'trans') && ~strcmp(signal_names{i}, 'cleave') 
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
    model.state_ids=setfield(model.state_ids, signal_names{i},cnt) ;
  elseif strcmp(signal_names{i},'cleave') 
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
    model.state_ids=setfield(model.state_ids, signal_names{i},[cnt]) ;
    if model.use.segments.intercistronic
      cnt=cnt+1 ;
      model.states(cnt).name = signal_names{i};
      model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
      model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
      model.states(cnt).non_consensus = {} ;
      model.states(cnt).non_consensus_pos = [] ;
      model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
      model.state_ids=setfield(model.state_ids, signal_names{i},[cnt-1 cnt]) ;
    end
  elseif strcmp(signal_names{i},'tis') | strcmp(signal_names{i},'cdsStop')
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.state_ids=setfield(model.state_ids, signal_names{i},[cnt-1 cnt]) ;
    if strcmp(signal_names{i}, 'tis')
    model.states(cnt).name = 'tis_acc' ;
      model.states(cnt).consensus = {'agatg'} ;
      model.states(cnt).consensus_pos = 2 ;
      model.states(cnt).signal = [model.signals.acc model.signals.tis] ;%%%% !!! 2 Zahlen
    else
      model.states(cnt).name = 'cdsStop_acc';
      model.states(cnt).consensus = {'agtaa', 'agtag', 'agtga'} ;
      model.states(cnt).consensus_pos = [2 2 2] ;
      model.states(cnt).signal = [model.signals.acc model.signals.cdsStop] ;%%%% !!! 2 Zahlen
    end
  elseif strcmp(signal_names{i},'trans') & model.use.states.SL1 & model.use.states.SL2
    cnt=cnt+1 ;
    model.states(cnt).name = 'SL1';
    model.states(cnt).consensus = getfield(model.consensus,'trans');
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,'trans');
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc]  ;
    cnt=cnt+1 ;
    model.states(cnt).name = 'SL1';
    model.states(cnt).consensus = {'agatg'} ; 
    model.states(cnt).consensus_pos = 2;
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc model.signals.tis]  ;
    model.state_ids = setfield(model.state_ids, 'SL1', [cnt-1 cnt]) ;
    cnt=cnt+1 ;
    model.states(cnt).name = 'SL2';
    model.states(cnt).consensus = getfield(model.consensus,'trans');
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,'trans');
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc]  ;
    cnt=cnt+1 ;
    model.states(cnt).name = 'SL2';
    model.states(cnt).consensus = {'agatg'} ; 
    model.states(cnt).consensus_pos = 2;
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc model.signals.tis]  ;%%%% !!! 3 Zahlen
    model.state_ids = setfield(model.state_ids, 'SL2', [cnt-1 cnt]) ;
  elseif strcmp(signal_names{i},'trans') & ~model.use.states.SL1 & ~model.use.states.SL2
    cnt=cnt+1 ;
    model.states(cnt).name = 'trans';
    model.states(cnt).consensus = getfield(model.consensus,'trans');
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,'trans');
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc]  ;
    cnt=cnt+1 ;
    model.states(cnt).name = 'trans';
    model.states(cnt).consensus = {'agatg'} ; 
    model.states(cnt).consensus_pos = 2;
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc model.signals.tis]  ;
    model.state_ids = setfield(model.state_ids, 'SL1', [cnt-1 cnt]) ;
  elseif strcmp(signal_names{i},'don')
    cnt_don = cnt ;
    cnt=cnt+8 ;
    for id = cnt_don+1:cnt_don+8 ;
      model.states(id).consensus = getfield(model.consensus,signal_names{i});
      model.states(id).consensus_pos = getfield(model.consensus_pos,signal_names{i});
      model.states(id).non_consensus = {} ;
      model.states(id).non_consensus_pos = [] ;
      model.states(id).signal=model.signals.don ;
    end ;
    model.states(cnt_don+1).name='UTR5_don' ;
    model.states(cnt_don+2).name='CDS_don_0' ;
    model.states(cnt_don+3).name='CDS_don_1a' ;
    model.states(cnt_don+3).non_consensus = {'t'} ;
    model.states(cnt_don+3).non_consensus_pos= [1] ;
    model.states(cnt_don+4).name='CDS_don_1b' ;
    model.states(cnt_don+4).consensus = {'tgt','tgc'} ;
    model.states(cnt_don+4).consensus_pos= [1,1] ;
    model.states(cnt_don+5).name='CDS_don_2a' ;
    model.states(cnt_don+5).non_consensus = {'ta', 'tg'} ;
    model.states(cnt_don+5).non_consensus_pos= [2,2] ;
    model.states(cnt_don+6).name='CDS_don_2b' ;
    model.states(cnt_don+6).consensus = {'tagt', 'tagc'} ;
    model.states(cnt_don+6).consensus_pos= [2,2] ;
    model.states(cnt_don+7).name='CDS_don_2c' ;
    model.states(cnt_don+7).consensus = {'tggt', 'tggc'} ;
    model.states(cnt_don+7).consensus_pos= [2,2] ;
    model.states(cnt_don+8).name='UTR3_don' ;
    
    model.state_ids.don=cnt_don+1:cnt_don+8 ;
  elseif strcmp(signal_names{i},'acc')
    cnt_acc = cnt ;
    cnt=cnt+8 ;
    for id = cnt_acc+1:cnt_acc+8 ;
      model.states(id).consensus = {'ag'} ;
      model.states(id).consensus_pos= [2] ;
      model.states(id).non_consensus={} ;
      model.states(id).non_consensus_pos=[] ;
      model.states(id).signal=model.signals.acc ;
    end ;
    model.states(cnt_acc+1).name='UTR5_acc' ;
    model.states(cnt_acc+2).name='CDS_acc_0' ;
    model.states(cnt_acc+3).name='CDS_acc_1a' ;
    model.states(cnt_acc+4).name='CDS_acc_1b' ;
    model.states(cnt_acc+4).non_consensus = {'aa','ag','ga'} ;
    model.states(cnt_acc+4).non_consensus_pos= [0,0,0] ;
    model.states(cnt_acc+5).name='CDS_acc_2a' ;
    model.states(cnt_acc+6).name='CDS_acc_2b' ;
    model.states(cnt_acc+6).consensus = {'agt', 'agc'} ;
    model.states(cnt_acc+6).consensus_pos= [2, 2] ;
    model.states(cnt_acc+7).name='CDS_acc_2c' ;
    model.states(cnt_acc+7).non_consensus = {'a'} ;
    model.states(cnt_acc+7).non_consensus_pos= [0] ;
    model.states(cnt_acc+8).name='UTR3_acc' ;
 
    model.state_ids.acc=cnt_acc+1:cnt_acc+8 ;

    if isfield(model.use, 'intron_long_state')&& model.use.intron_long_state
      cnt_intron_long = cnt;
      cnt = cnt+6;
      model.states(cnt_intron_long+1).consensus={} ;
      model.states(cnt_intron_long+1).consensus_pos=[] ;
      model.states(cnt_intron_long+1).non_consensus={} ;
      model.states(cnt_intron_long+1).non_consensus_pos=[] ;
      model.states(cnt_intron_long+1).name='intron_long_0' ;
      model.states(cnt_intron_long+1).signal=[] ;

      model.states(cnt_intron_long+2).consensus={} ;
      model.states(cnt_intron_long+2).consensus_pos=[] ;
      model.states(cnt_intron_long+2).non_consensus={} ;
      model.states(cnt_intron_long+2).non_consensus_pos=[] ;
      model.states(cnt_intron_long+2).name='intron_long_1a' ;
      model.states(cnt_intron_long+2).signal=[] ;
    
      model.states(cnt_intron_long+3).consensus={} ;
      model.states(cnt_intron_long+3).consensus_pos=[] ;
      model.states(cnt_intron_long+3).non_consensus={} ;
      model.states(cnt_intron_long+3).non_consensus_pos=[] ;
      model.states(cnt_intron_long+3).name='intron_long_1b' ;
      model.states(cnt_intron_long+3).signal=[] ;

      model.states(cnt_intron_long+4).consensus={} ;
      model.states(cnt_intron_long+4).consensus_pos=[] ;
      model.states(cnt_intron_long+4).non_consensus={} ;
      model.states(cnt_intron_long+4).non_consensus_pos=[] ;
      model.states(cnt_intron_long+4).name='intron_long_2a' ;
      model.states(cnt_intron_long+4).signal=[] ;

      model.states(cnt_intron_long+5).consensus={} ;
      model.states(cnt_intron_long+5).consensus_pos=[] ;
      model.states(cnt_intron_long+5).non_consensus={} ;
      model.states(cnt_intron_long+5).non_consensus_pos=[] ;
      model.states(cnt_intron_long+5).name='intron_long_2b' ;
      model.states(cnt_intron_long+5).signal=[] ;

      model.states(cnt_intron_long+6).consensus={} ;
      model.states(cnt_intron_long+6).consensus_pos=[] ;
      model.states(cnt_intron_long+6).non_consensus={} ;
      model.states(cnt_intron_long+6).non_consensus_pos=[] ;
      model.states(cnt_intron_long+6).name='intron_long_1a' ;
      model.states(cnt_intron_long+6).signal=[] ;

      model.state_ids.intergenic_long= cnt_intron_long+1:cnt_intron_long+6;
    end
%    cnt
  end
end

% intergenic_long
cnt=cnt+1 ;
model.states(cnt).consensus={} ;
model.states(cnt).consensus_pos=[] ;
model.states(cnt).non_consensus={} ;
model.states(cnt).non_consensus_pos=[] ;
model.states(cnt).name='intergenic_long' ;
model.states(cnt).signal=[] ;
model.state_ids.intergenic_long=cnt ;

% sequence end
cnt=cnt+1 ;
model.states(cnt).consensus={} ;
model.states(cnt).consensus_pos=[] ;
model.states(cnt).non_consensus={} ;
model.states(cnt).non_consensus_pos=[] ;
model.states(cnt).name='seq_end' ;
model.states(cnt).signal=[] ;
model.state_ids.seq_end = cnt ;

model.cnt_states = cnt ; 

return

function model = define_transition_pointers(model)
  
A = model.A;
mRNA_start_states_out=model.mRNA_start_states_out;
%%% transition_pointers contain for each allowed transition pointers to lengths plifs
transition_pointers = zeros(size(A)) ;
transition_pointers(~isinf(A))=A(~isinf(A)) ;
transition_pointers = transition_pointers' ;

%%% add pointers to content PLIFs
transition_pointers(:,:,2) = zeros(size(transition_pointers,1), size(transition_pointers,2), 1) ;
for i=1:size(model.plif_links,1)
  [idx1,idx2] = find(transition_pointers(:,:,1)==model.plif_links(i,1));
  for j=1:length(idx1)
    transition_pointers(idx1(j),idx2(j),2) = model.plif_links(i,2);
  end ;
end


%%% add pointers to tiling PLIFs
if model.use.addfeatures.tiling
  dim3 = size(transition_pointers,3);
  transition_pointers(:,:,dim3+1) = zeros(size(transition_pointers,1), size(transition_pointers,2), 1) ;
  for i=1:size(model.tiling_links,1)
    [idx1,idx2] = find(transition_pointers(:,:,1)==model.tiling_links(i,1));
    for j=1:length(idx1)
      transition_pointers(idx1(j),idx2(j),3) = model.tiling_links(i,2);
    end ;
  end
end

%%% add pointers to frame PLIFs
tmp = zeros(size(transition_pointers(:,:,1))) ;
tmp(mRNA_start_states_out, model.state_ids.cdsStop(1)) = model.contents.cds_frame0 ;% single cds exon  

if ~isfield(model.use.contents, 'pre_comp') || ~model.use.contents.pre_comp,
  tmp(mRNA_start_states_out, model.state_ids.don(2)) = model.contents.cds_frame0 ;% first cds exon 
  tmp(mRNA_start_states_out, model.state_ids.don(3:4)) = model.contents.cds_frame1 ;% first cds exon  
  tmp(mRNA_start_states_out, model.state_ids.don(5:7)) = model.contents.cds_frame2 ;% first cds exon 
  
  tmp(model.state_ids.acc(2:7), model.state_ids.don(2)) = model.contents.cds_frame0 ;% middle cds exon 
  tmp(model.state_ids.acc(2:7), model.state_ids.don(3:4)) = model.contents.cds_frame1 ;% middle cds exon 
  tmp(model.state_ids.acc(2:7), model.state_ids.don(5:7)) = model.contents.cds_frame2 ;% middle cds exon 
  tmp(model.state_ids.acc(2:7), model.state_ids.cdsStop(1)) = model.contents.cds_frame0 ;% last cds exon 
else
  tmp(mRNA_start_states_out, model.state_ids.don(2:7)) = model.contents.cds_frame0 ;% first cds exon 
  tmp(model.state_ids.acc(2),model.state_ids.don(2:7)) = model.contents.cds_frame0 ;% middle cds exon 
  tmp(model.state_ids.acc(3:4),model.state_ids.don(2:7)) = model.contents.cds_frame1 ;% middle cds exon 
  tmp(model.state_ids.acc(5:7),model.state_ids.don(2:7)) = model.contents.cds_frame2 ;% middle cds exon 
  tmp(model.state_ids.acc(2), model.state_ids.cdsStop(1)) = model.contents.cds_frame0 ;% last cds exon 
  tmp(model.state_ids.acc(3:4), model.state_ids.cdsStop(1)) = model.contents.cds_frame1 ;% last cds exon 
  tmp(model.state_ids.acc(5:7), model.state_ids.cdsStop(1)) = model.contents.cds_frame2 ;% last cds exon 
end ;


dim3 = size(transition_pointers,3);
transition_pointers(:,:,dim3+1) = tmp' ; % don't touch this---tmp has to be transposed!

a_trans = zeros(4,sum(~isinf(A(:)))) ;
% [from,to,val,segment]
k = 0 ;
for i=1:size(A,1)
  idx = find(~isinf(A(i,:))) ;
  val = A(i,idx) ;
  pen = transition_pointers(idx,i,1) ;
  a_trans(1,k+1:k+length(idx))=i-1 ;
  a_trans(2,k+1:k+length(idx))=idx-1 ;  
  a_trans(3,k+1:k+length(idx))=val ;    
  for q=1:length(pen) 
    idx_=find(model.seg_links(:,1)==pen(q)) ;
    if isempty(idx_), 
      assert(pen(q)==0) ; 
    else
      assert(length(idx_)==1) ;
      pen(q) = model.seg_links(idx_,2) ;
    end ;
  end ;
  a_trans(4,k+1:k+length(idx))= pen;
  k=k+length(idx) ;
end ;
%a_trans = a_trans' ;
%[tmp,idx] = sort(a_trans(:,2)) ;
%a_trans = a_trans(idx,:) ;

model.transition_pointers = transition_pointers ;
model.a_trans = a_trans;

return

function model = gen_transition_matrix(model,PAR)

use=model.use;  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define transitions

A = -inf(model.cnt_states, model.cnt_states);
p = -inf(model.cnt_states,1);
q = -inf(model.cnt_states,1) ;
p(model.state_ids.seq_start) = 0 ;
q(model.state_ids.seq_end) = 0 ;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% start constructing transition matrix A

orf_from = -1*ones(model.cnt_states,1) ;
orf_to = -1*ones(model.cnt_states,1) ;

orf_from(model.state_ids.acc(2:7))= [ 0 1 1 2 2 2] ;
orf_from(model.state_ids.tis)     = 0 ; % AGAGT case
if model.use.states.SL1 & model.use.states.SL2
  orf_from(model.state_ids.SL1(2)) = 0 ; % AGAGT case
  orf_from(model.state_ids.SL2(2)) = 0 ; % AGAGT case
elseif model.use.signals.trans
  orf_from(model.state_ids.trans(2)) = 0 ; % AGAGT case
end
orf_to(model.state_ids.don(2:7))  = [0 1 1 2 2 2] ;
orf_to(model.state_ids.cdsStop(1))   = 0 ;
orf_info=[orf_from, orf_to] ;

%%%%%%%%%%%
%% no gene / intergenic stuff
%%%%%%%%%%%
if ~use.model.one_gene,
  A(model.state_ids.seq_start,  model.state_ids.seq_end) = model.lengths.intergenic ;
end
A(model.state_ids.intergenic_long, model.state_ids.intergenic_long) = 0; % no penalty et al
A(model.state_ids.seq_start, model.state_ids.intergenic_long) = model.lengths.intergenic_long ;
A(model.state_ids.intergenic_long, model.state_ids.seq_end) = model.lengths.intergenic_long ;

%%%%%%%%%%%
% 5' UTR 
%%%%%%%%%%%
gene_start_states_in = [];
gene_start_states_out = [];
mRNA_start_states_out = [model.state_ids.tis];

if use.signals.tss,
  gene_start_states_in = [gene_start_states_in model.state_ids.tss] ;
  gene_start_states_out = [gene_start_states_out model.state_ids.tss] ;
  if use.model.allow_missing_tss,
    assert(false) ;
    gene_start_states_in = [gene_start_states_in model.state_ids.tis] ;
  end ;
else
  assert(false)
  gene_start_states_in = [gene_start_states_in model.state_ids.tis] ;
end ;

if use.signals.trans,
  gene_start_states_in = [gene_start_states_in model.state_ids.SL1] ;
  gene_start_states_out = [gene_start_states_out model.state_ids.SL1(1)] ;
  gene_start_states_out = [gene_start_states_out model.state_ids.SL2(1)] ;
  mRNA_start_states_out = [mRNA_start_states_out model.state_ids.SL1(2)] ;%'AGAGT cases'
  mRNA_start_states_out = [mRNA_start_states_out model.state_ids.SL2(2)] ;%'AGAGT cases'
  if use.signals.tss, 
    A(model.state_ids.tss, model.state_ids.SL1)    = model.lengths.transexon ;  
  end ;
  if use.segments.intercistronic & use.signals.cleave ,
    A(model.state_ids.cleave(1), model.state_ids.SL1) = model.lengths.intergenictrans ;
    A(model.state_ids.cleave(2), model.state_ids.SL1) = model.lengths.intercistronic ;
    A(model.state_ids.cleave(2), model.state_ids.SL2) = model.lengths.intercistronic ;
  end ;
end ;

% gene start
A(model.state_ids.seq_start,gene_start_states_in) = model.lengths.intergenic ;
if use.model.use_multi
  A(model.state_ids.intergenic_long, gene_start_states_in) = model.lengths.intergenic_long ; 
end
    
A(gene_start_states_out,model.state_ids.tis(1)) = model.lengths.utr5exon ;
A(gene_start_states_out,model.state_ids.don(1)) = model.lengths.utr5exon ;
% spliced 5'utr
A(model.state_ids.acc(1), model.state_ids.don(1)) = model.lengths.utr5exon ;
A(model.state_ids.don(1), model.state_ids.acc(1)) = model.lengths.intron ;
% last 5'utr exon
A(model.state_ids.acc(1),model.state_ids.tis(1)) = model.lengths.utr5exon ;
A(model.state_ids.don(1), model.state_ids.tis(2)) = model.lengths.intron ; 


%%%%%%%%%%%
% CDS STUFF
%%%%%%%%%%%
A(mRNA_start_states_out, model.state_ids.cdsStop(1)) = model.lengths.single_cds_exon ; % trans(2) ok 
A(mRNA_start_states_out, model.state_ids.don(2:7)) = model.lengths.first_cds_exon ; % trans(2) ok
for i=2:7,
  A(model.state_ids.don(i), model.state_ids.acc(i)) = model.lengths.intron ;
  if isfield(model.state_ids, 'intron_long')
    A(model.state_ids.don(i), model.state_ids.intron_long(i-1)) = model.lengths.intron;
    A(model.state_ids.intron_long(i-1), model.state_ids.acc(i)) = model.lengths.intron_long;
    A(model.state_ids.intron_long(i-1),model.state_ids.intron_long(i-1)) = model.lengths.intron_long;
  end
end ;
A(model.state_ids.acc(2:7), model.state_ids.don(2:7)) = model.lengths.middle_cds_exon ;
A(model.state_ids.acc(2:7), model.state_ids.cdsStop(1)) = model.lengths.last_cds_exon ;
% agstop 
A(model.state_ids.don(2), model.state_ids.cdsStop(2)) = model.lengths.intron ;



%%%%%%%%%%%
% 3' UTR
%%%%%%%%%%%
A(model.state_ids.cdsStop, model.state_ids.don(8)) = model.lengths.utr3exon ;
A(model.state_ids.don(8), model.state_ids.acc(8)) = model.lengths.intron ;
A(model.state_ids.acc(8), model.state_ids.don(8)) = model.lengths.utr3exon ;

if use.signals.polya,
  % polyA has to be in last 3'UTR exon
  A(model.state_ids.cdsStop, model.state_ids.polya) = model.lengths.utr3exon ;
  A(model.state_ids.acc(8), model.state_ids.polya)=model.lengths.utr3exon ;
end ;

gene_stop_states = [];    
if use.signals.cleave,
  gene_stop_states = [gene_stop_states model.state_ids.cleave(1)];
  A(model.state_ids.cdsStop, model.state_ids.cleave) = model.lengths.utr3exon ;
  A(model.state_ids.acc(8), model.state_ids.cleave) = model.lengths.utr3exon ;
  if use.signals.polya,
    A(model.state_ids.polya, model.state_ids.cleave) = model.lengths.polya_tail ;
  end
else
  gene_stop_states = [gene_stop_states model.state_ids.cdsStop];
  if use.signals.polya,
    gene_stop_states = [gene_stop_states model.state_ids.polya];
  end
end ;
A(gene_stop_states, model.state_ids.seq_end)    = model.lengths.intergenic ; 
A(gene_stop_states, model.state_ids.intergenic_long) = model.lengths.intergenic_long ; 
if use.model.use_multi,
  A(gene_stop_states, gene_start_states_in) = model.lengths.intergenic ;    
end ;

if use.segments.intercistronic 
  for k=model.state_ids.SL1,
    idx = find(A(:,k)==model.lengths.intergenic) ;
    A(idx,k) = model.lengths.intergenictrans ;   
  end ;
end
%%%%%%%%%%%
% end transition definition
%%%%%%%%%%%


%%% technical stuff for intergenic_long

transitions = [];
for i=1:length(gene_start_states_in)
  transitions = [transitions; model.state_ids.seq_start gene_start_states_in(i)];
  for j=1:length(gene_stop_states)
    transitions = [transitions; gene_stop_states(j) gene_start_states_in(i)];
    transitions = [transitions; gene_stop_states(j) model.state_ids.seq_end] ;
  end
end

model.intergenic_long.transitions = unique(transitions,'rows');
model.intergenic_long.max_short_length = model.lengths_range.intron(2) ;
model.intergenic_long.step = model.lengths_range.intron(2); % use the intron length as maximal step size
model.intergenic_long.long_segment_parts = model.lengths_range.intron(2)/2-25 ;
%model.intergenic_long.step = 30 ; % makes about 30 (step) * 40 (grid)=1200nt per step

%%% 

q(gene_stop_states) = 0 ;

model.gene_start_states_in = gene_start_states_in;
model.gene_start_states_out = gene_start_states_out;
model.mRNA_start_states_out = mRNA_start_states_out;
model.gene_stop_states = gene_stop_states;

model.A = A;
model.p = p;
model.q = q;
model.orf_info = orf_info;
model.active_transitions = find(~isinf(A)) ;
model.cnt_transitions = length(model.active_transitions) ;



return

function weights=init_weights(model) ;
% weights=init_weights(model) ;

weights = struct;

%%% content detectors
content_names = fieldnames(model.contents) ;
for s = 1:model.cnt_contents,
  weights = setfield(weights, ['contents_' content_names{s}], zeros(1,model.bins));
end

%%% length detectors
length_names = fieldnames(model.lengths) ;
for s = 1:model.cnt_lengths,
  weights = setfield(weights, ['lengths_' length_names{s}], zeros(1,model.bins));
end

%%% signal detectors
signal_names = fieldnames(model.signals) ;
for s = 1:model.cnt_signals,
  weights = setfield(weights, ['signals_' signal_names{s}], zeros(1,model.bins));
end

%%% tiling detectors
if isfield(model, 'tiling')
  tiling_names = fieldnames(model.tiling) ;
  for s = 1:model.cnt_tiling,
    weights = setfield(weights, ['tiling_' tiling_names{s}], zeros(1,model.bins));
  end
end
%%% rna_seq detectors
if isfield(model, 'rna_seq')
  rna_seq_names = fieldnames(model.rna_seq) ;
  for s = 1:model.cnt_rna_seq,
    weights = setfield(weights, ['rna_seq_' rna_seq_names{s}], zeros(1,model.bins));
  end
end
%%% rna_seq_intron_track detectors
if isfield(model, 'rna_seq_intron_track')
  rna_seq_intron_track_names = fieldnames(model.rna_seq_intron_track) ;
  for s = 1:model.cnt_rna_seq_intron_track,
    weights = setfield(weights, ['rna_seq_intron_track_' rna_seq_intron_track_names{s}], zeros(1,model.bins));
  end
end

%%% transitions
weights.transitions = zeros(1,model.cnt_transitions) ;

return

function [model] = make_model(organism,LSL,Signals,old)
% [model] = make_model(bins)

if nargin<4
  old=0;
end
  
model.bins = LSL.method.plif_bins ;
% model.use_gc =  LSL.method.use_gc;
model.use = LSL.method.use;
polya_sixmers = Signals.polya.consensus;

model = define_segments(model);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%define features

model = define_contents(model);
model = define_lengths(model,organism);
model = define_signals(model,polya_sixmers);
model = define_addContentFeatures(model);

model.cnt_plifs = model.cnt_contents+model.cnt_lengths+model.cnt_signals+model.cnt_tiling;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% remember the ordering of these fields
model.segments_fields = fieldnames(model.segments) ;
model.contents_fields = fieldnames(model.contents) ;
model.lengths_fields = fieldnames(model.lengths) ;
model.signals_fields = fieldnames(model.signals) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%define states

model = define_states(model);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define mappings between length, contents and segments

model = define_mappings(model);
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

model = gen_transition_matrix(model) ;
% transition_pointers used to be called penalties
model = define_transition_pointers(model);
if old
  model.penalties = model.transition_pointers;
end
model.A(~isinf(model.A))=0 ;



model.cnt_parameters = model.cnt_transitions + model.cnt_plifs*model.bins ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% define loss 
model =  define_loss(model);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% mappings of PLIFS to parameters

if ~old
  map_plif2param = init_weights(model);
  map_plif2param = rmfield(map_plif2param,'transitions');
  weights_name = fieldnames(map_plif2param);
  start=0;
  for w=1:length(weights_name)
    len=length(getfield(map_plif2param, weights_name{w}));
    model.plif2param{w} = start+(1:len);
    start = start + len;
  end
  model.transitions2param = start+1:model.cnt_parameters ;
  
  assert(model.cnt_parameters-model.cnt_transitions==start)
end

return

function method = assign_model(method, models, par_names,  model_number)

%function method = assign_model(method, models, par_names, model_number)  

  
for num_param = 1:size(models,1)
  method.par_ms.(par_names{num_param}) = models(num_param,model_number) ;
end

if isfield(method, 'kernels') && ~isempty(method.kernels{1})
  nofKernels = length(method.kernels);
  for k =1:nofKernels
    kernel = method.kernels{k};
    for f =fieldnames(kernel)'
      if isnumeric(kernel.(f{1}))&~isempty(kernel.(f{1}))
        kernel.(f{1})= method.par_ms.(sprintf('kernel_%i_%s',k,f{1}));
      end
    end
    method.kernels{k}=kernel;
  end
end

return

function [ALL,is_invalid] = basic_cand_stuff(gene,ALL,sig,debug,signal_name)
% [ALL,is_invalid] = basic_cand_stuff(gene,ALL,sig,debug,signal_name)

if nargin<5
  signal_name =[];
end

is_invalid=0; 

genic_idx = find(ALL.POS>gene.start&ALL.POS<gene.stop);
ALL.GENE_ID(genic_idx) = gene.id;

if isfield(gene,'transacc') && ~isempty(gene.transacc)
  ALL.TRANS(genic_idx) = 1;
end
if ~isempty(gene.tss)
  ALL.TSS(genic_idx) = 1;
end

if ~isempty(signal_name)&&~isempty(gene.(signal_name))
  ALL.HAS_SIG(genic_idx) = 1;
end
  
if ~gene.is_valid
  is_invalid=1; 
  ALL.GENE_ID(genic_idx) =  - gene.id;
  return
end

if isequal(sig,'other')
  if gene.is_alt 
    ALL.ALTGENIC(genic_idx) = 1;
  end
else
  if any(gene.exon_in_alt_region)
    alt_reg = get_connected_regions(gene.exon_in_alt_region);  
    
    for i=1:length(unique(alt_reg))-1
      temp=find(alt_reg==i);
      alt_start = gene.splicegraph{1}(1,temp(1));
      alt_stop = gene.splicegraph{1}(2,temp(end));
      alt_genic_idx = find(ALL.POS>alt_start&ALL.POS<alt_stop);
      ALL.ALTGENIC(alt_genic_idx) = 1;
      if debug
        viewsplicegraph(gene,1)
        hold on 
        plot([alt_start alt_start],[1 10],'b')
        plot([alt_stop alt_stop],[1 10],'b')  
      end
    end
  end
end

if gene.is_alt 
  ALL.ALTGENIC(genic_idx) = 1;
end
return

function calc_plifs(PAR_1)
fid=1;
q=0;

fn_SVMs = PAR_1.FN.output_sig.(PAR_1.Signal_name).fn_SVMs;
method = PAR_1.Signals.(PAR_1.Signal_name).method;
num_partitions = PAR_1.SETs.num_partitions;

for p = 1:num_partitions
  fprintf(fid,'\n\n-----------\n');
  fprintf(fid,'PARTITION %i\n',p);
  fprintf(fid,'-----------\n');
  fn_SVM = sprintf('%sbest_partition=%i.mat', fn_SVMs,p) ;

  %--------------------------------------------------------------------------------%
  % load parameters from SVM file
  %--------------------------------------------------------------------------------%
  load(fn_SVM,'Train');
  PAR = Train.PAR;
  PAR.fn_SVMs=fn_SVM;
  clear Train
  
  %--------------------------------------------------------------------------------%
  % learn PLIFS on evaluation set and convert output on test set
  %--------------------------------------------------------------------------------%
  PAR.fn_pred = sprintf('%spartition=%i.mat',fn_SVMs,p);
  PAR.fn_plifs = sprintf('%spartition=%i_plifs.mat',fn_SVMs,p);
  assert(fexist(PAR.fn_pred)==1)
  assert(fexist(PAR.fn_plifs)==1)
  fprintf(fid,'\n already predicted on set ');
  fprintf(fid,'%i',PAR.SETs.which_pred);
        
  %--------------------------------------------------------------------------------%
  %%% learn PLIFS on evaluation set
  %--------------------------------------------------------------------------------%
  clear  prob prob_cum limits Conf Conf_cum
  fprintf('\nload fn_plif\n')
  load(PAR.fn_plifs,'LT', 'output');
  if ~all(ismember({'Conf','Conf_cum'},who_file(PAR.fn_plifs)))
    if ~all(ismember({'prob','prob_cum', 'limits'},who_file(PAR.fn_SVMs)))
      disp('learn PLIFs')
      assert(length(output)==length(LT))
      [prob, prob_cum,limits] = learn_plifs(output,LT,method.plif.bins,1,method.plif.min_incr );
      %save(PAR.fn_SVMs,'-append','prob', 'prob_cum','limits');
      save_append(PAR.fn_SVMs, 1, 'prob', prob, 'prob_cum', prob_cum, ...
                  'limits', limits);
    else
      load(PAR.fn_SVMs,'prob', 'prob_cum','limits');
    end
    disp('Convert SVM Output to Confidence on fn_plif')
    [Conf, Conf_cum] = Out2Conf(output,prob, prob_cum,limits);
    %save(PAR.fn_plifs,'-append','Conf','Conf_cum');
    save_append(PAR.fn_plifs,1, 'Conf', Conf, 'Conf_cum', Conf_cum);
    clear output Conf Conf_cum LT
  end
  %--------------------------------------------------------------------------------%
  %%% Convert SVM Outputs on test set 
  %--------------------------------------------------------------------------------%
  load(PAR.fn_pred,'output');
  if ~all(ismember({'Conf','Conf_cum'},who_file(PAR.fn_pred)))
    disp('Convert SVM Output to Confidence on fn_pred')
    [Conf, Conf_cum] = Out2Conf(output,prob, prob_cum,limits);
    %save(PAR.fn_pred,'-append','Conf','Conf_cum');
    save_append(PAR.fn_pred, 1,'Conf', Conf, 'Conf_cum', Conf_cum);
  end
end%loop over partitions
% [jobinfo,num_crashed] = rproc_wait(jobinfo,20,1,0);

return

function all_good = check_predictions(fn_genome_config,fn_pred,Signal_name,resolution)
% check_predictions(fn_pred,fn_genome_config)



P.fn_genome_config = fn_genome_config;
P.fn_pred = fn_pred;
P.resolution = resolution;
P.Signal_name = Signal_name;

genome_info = init_genome(fn_genome_config);

all_good = zeros(2, length(genome_info.contig_names));
for c = 1:length(genome_info.contig_names)
	  num = 0;
   for s = '+-'
     num = num+1;
     filename = sprintf('%scontig_%i%s', fn_pred, c, s);
     if fexist([filename '.pos'])&& fexist([filename '.output'])
       P.chrom = c;
       P.strand = s; 
       all_good(num,c) = check_predictions_helper(P) ;
       fprintf('check_predictions: strand: %s, chrom: %i, ok: %i',s, c,all_good);
     else
       fprintf('file missing: %s',filename)
     end 
   end  
end 

return

function par_ms= collect_ms_parameters(kernels, par_ms)
%par_ms= collect_model_parameters(kernels, par_ms)

nofKernels = length(kernels);
assert(nofKernels>=1)
for k = 1:nofKernels
  kernel = kernels{k};
  for f =fieldnames(kernel)'
    if isnumeric(kernel.(f{1}))&~isempty(kernel.(f{1}))
      par_ms.(sprintf('kernel_%i_%s',k,f{1})) = kernel.(f{1});
    end
  end
end


return

function [dummy1, dummy2]=concat_genomewide_predictons_helper(P)
% concat_genomewide_predictons_helper(P)

try
  pause('on')
  warning('off', 'MATLAB:typeaheadBufferOverflow');
catch
  % works only for matlab
end


dummy1=[] ;
dummy2=[] ;

fn_pred_all = P.fn_pred_all ;
fn_pred_split = P.fn_pred_split;
num_splits = P.num_splits;
avg_all_svms = P.avg_all_svms;
subtract_diff = P.subtract_diff;

% for H_sapiens_nohomologs:
% prefix = '/fml/ag-raetsch/share/projects/genefinding/H_sapiens_nohomologs/sensors/don/output/pred';
% fn_pred_all = sprintf('%scontig_%i%s_all.mat',prefix,c,s);

contig_pos = [];
contig_out = [];
contig_conf= [];
contig_cnfc= [];
contig_cnt=[] ;
contig_svm = [];
for p=1:num_splits
  fn_pred = sprintf('%s%i.mat', fn_pred_split, p);
  % for H_sapiens_nohomologs:
  % prefix = '/fml/ag-raetsch/share/projects/genefinding/H_sapiens_nohomologs/sensors/don/output/pred';
  % fn_pred = sprintf('%scontig_%i%s%s_%i.mat',prefix,c,s,PAR.Signal_name,p);

  for kk=1:10
    try
      clear pos output Conf Conf_cum
      load(fn_pred, 'inventory');
      %inventory
      for j=1:length(inventory)
        if kk>1,
          fprintf('trying to load variable "%s" from %s (trial %i)\n', inventory{j}, fn_pred, kk);
        end ;
        load(fn_pred,inventory{j})
        assert(exist(inventory{j},'var')==1);	 
      end
      break ;
    catch
      if kk==10      
        fprintf('giving up waiting for file %s\n', fn_pred)
      else
        fprintf('waiting for file: %s \n', fn_pred);    
        pause(30)
      end	
    end
  end

  if ~avg_all_svms
    if ~isempty(intersect(contig_pos, pos))
      warning('positions found in multiple splits');
      %% hack
      [tmp,idx1,idx2] = intersect(contig_pos, pos);
      pos(idx2)=[];
      output(idx2)=[];
      if exist('Conf') && exist('Conf_cum')
        Conf(idx2)=[];
        Conf_cum(idx2)=[];
      %else
      %  warning('no confs found')
      end
      %%hack
    end
    % assert(isempty(intersect(contig_pos, pos)));
    contig_pos = [contig_pos pos];
    contig_out = [contig_out output];
    contig_svm = [contig_svm repmat(p,size(pos))];
    if exist('Conf') && exist('Conf_cum')
      contig_conf = [contig_conf Conf];
      contig_cnfc = [contig_cnfc Conf_cum];
    end
    
  else
    [tmp,idx1,idx2] = intersect(contig_pos, pos);
    assert(length(idx1)==length(contig_pos))
    if isempty(contig_pos)
      contig_pos = pos;
      contig_out = output;
      if exist('Conf') && exist('Conf_cum')
        contig_conf = Conf(idx2);
        contig_cnfc = Conf_cum(idx2) ;
        contig_cnt = 1 ;
      else
        clear contig_conf contig_cnfc contig_cnt ;
      end
    else
      % otherwise the following will be wrong
      assert(isequal(sort(contig_pos), contig_pos)) ;
      assert(isequal(idx1, 1:length(idx1))) ;

      contig_out(idx1) = contig_out(idx1)+output(idx2);
      if exist('Conf') && exist('Conf_cum')
        contig_conf(idx1) = contig_conf(idx1) + Conf(idx2) ;
        contig_cnfc(idx1) = contig_cnfc(idx1) + Conf_cum(idx2) ;
        contig_cnt(idx1) = contig_cnt(idx1) + 1 ;
      else
        clear contig_conf contig_cnfc contig_cnt;
      end
    end
  end
  clear output pos Conf Conf_cum
end 
[pos sort_idx] = sort(contig_pos);
output = contig_out(sort_idx);
svm = contig_svm(sort_idx);

if avg_all_svms
  output = output./contig_cnt ;
end

% for contents: make sure there are smooth transitions at split boundaries
if subtract_diff
  ii = find(svm(1:end-1)~=svm(2:end));
  fprintf('number of blocks: %i\n',length(ii));
  for j=ii
    output(j+1:end) = output(j+1:end) - (output(j+1)-output(j));
  end
end

if exist('contig_conf') && exist('contig_cnfc') && isequal(size(contig_conf), size(contig_pos)) && isequal(size(contig_cnfc), size(contig_pos))
  if avg_all_svms
    contig_conf = contig_conf./(contig_cnt+1e-16) ;
    contig_cnfc = contig_cnfc./(contig_cnt+1e-16) ;
  end ;
  Conf = contig_conf(sort_idx);
  Conf_cum = contig_cnfc(sort_idx);
  
  assert(all(Conf>=0-1e-8&Conf<=1+1e-8));
  assert(all(Conf_cum>=0-1e-8&Conf_cum<=1+1e-8));
  fprintf('saving output, pos, Conf and Conf_cum\n')
  fprintf('save: %s\n',fn_pred_all)
  inventory = {'output', 'pos', 'Conf', 'Conf_cum'};
  save(fn_pred_all, '-V7','inventory', 'output', 'pos', 'Conf', 'Conf_cum')
else
  inventory = {'output', 'pos'};
  fprintf('saving output and pos without Conf and Conf_cum\n')
  fprintf('save: %s\n',fn_pred_all)
  save(fn_pred_all, '-V7','inventory', 'output', 'pos')
end

return

function jobinfo =  concat_genomewide_predictons(fn_genome_config,fn_pos,fn_pred,num_splits,avg_all_svms,subtract_diff,run_locally, RPROC)

% concat_genomewide_predictons(fn_genome_config,fn_pos,fn_pred,num_splits,run_locally, RPROC);

jobinfo = rproc_empty(0) ;
q=0;
% options.ncpus = 1;
% options.identifier = sprintf('%sconcat%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;


genome_info = init_genome(fn_genome_config);
P.avg_all_svms = avg_all_svms;
P.subtract_diff = subtract_diff;
P.num_splits = num_splits;
for c = 1:length(genome_info.contig_names)
  for s = '+-'
    P.fn_pred_all = sprintf('%scontig_%i%s_all.mat',fn_pred,c,s);
    P.fn_pred_split = sprintf('%scontig_%i%s_split=',fn_pred,c,s);
    if fexist(P.fn_pred_all)
      continue
    else
      q=q+1;
      if ~run_locally
        [mem_req, time_req, opts] = rproc_memtime_policy('concat_genomewide_predictons_helper',  0, RPROC.options) ;

        if isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs 
          jobinfo(q) = rproc_create('concat_genomewide_predictons_helper', P, mem_req, opts, time_req) ;
        else
          jobinfo(q) = rproc('concat_genomewide_predictons_helper', P, mem_req, opts, time_req);
        end
      else
        concat_genomewide_predictons_helper(P)
      end
    end
  end
end

if ~run_locally
   rproc_wait(jobinfo, 20, 1, -1);
end

return

function jobinfo = convert_predictions2confs(fn_genome_config,fn_pred,fn_svms,num_splits,run_locally, RPROC)
% jobinfo = convert_predictions2confs(fn_genome_config,fn_pred,fn_svms,num_splits,run_locally, RPROC)

jobinfo = rproc_empty(0) ;
% options.ncpus = 1;
% options.identifier = sprintf('%sconf%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;

fid=1;q=0;
for p = 1:num_splits
  fprintf(fid,'partition %i\n',p);
  
  fn_SVM = sprintf('%s%i.mat',fn_svms,p) ;
 
  %--------------------------------------------------------------------------------%
  % load parameters from SVM file
  %--------------------------------------------------------------------------------%
  
  genome_info = init_genome(fn_genome_config);
  clear  prob prob_cum limits Conf Conf_cum

  if ~(all(ismember({'prob', 'prob_cum','limits'}, who_file(fn_SVM))))
    warning('Variables prob, prob_cum and or limits not found in %s',fn_SVM)
    error('calculate PLIFS first: execute this function with PAR.tasks.pred_from_seq=0; to calculate PLIFS')
  else
    load(fn_SVM, 'Train');
    PAR_new = load(fn_SVM, 'prob', 'prob_cum', 'limits');
    if ~isfield(Train.PAR.SETs, 'which_pred')
      Train.PAR.SETs.which_pred = Train.PAR.SETs.which_pred_svm;
    end
  end
  assert(Train.PAR.SETs.which_pred == p)
  %------------------------------------------------------------------------------------%
  % perform transformations
  %------------------------------------------------------------------------------------%
  for c = 1:length(genome_info.contig_names)
    for s = '+-'
      PAR_new.fn_pred = sprintf('%scontig_%i%s_split=%i.mat',fn_pred,c,s,p);
      for j=1:20
        if ~fexist(PAR_new.fn_pred)
	  fprintf(fid,'\n waiting for file: %s\n',PAR_new.fn_pred);
          pause(30) 
	end
      end
      if ~fexist(PAR_new.fn_pred)
       fprintf(fid,'\n file not found: %s', PAR_new.fn_pred);
       fprintf(fid,'\n run genomewide_prediction first\n');
      %elseif all(ismember({'Conf', 'Conf_cum'}, who_file(PAR_new.fn_pred))),
      %  continue
      else
        fprintf(fid,'\n start transforming on contig %i%s\n',c,s);
        %%% Convert SVM Outputs on test set `
        q=q+1;
        if ~run_locally

          [mem_req, time_req, opts] = rproc_memtime_policy('Out2Confhelper',  0, RPROC.options) ;

          if isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs 
            jobinfo(q) = rproc_create('Out2Confhelper', PAR_new, mem_req, opts, time_req) ;
          else
            jobinfo(q) = rproc('Out2Confhelper', PAR_new, mem_req, opts, time_req);
          end
        else
          Out2Confhelper(PAR_new);
        end
      end
    end%loop over strands
  end%loop over contigs
end %% loop over partitions

rproc_wait(jobinfo, 20, 1, -1);

return

function POS=  find_consensus(seq,Signal,strand)
%  POS=  find_consensus(seq,Signal,strand) 

consensus = Signal.consensus ; 
lwin_big = Signal.lwin_big;
rwin_big = Signal.rwin_big;
offset = Signal.offset;



POS = zeros(1,floor(length(seq)/10));
num = 0;
seq=upper(seq);
for m = 1:length(consensus)  
  cons = upper(consensus{m});
  pos_tmp = findstr(seq(lwin_big+1:end-rwin_big),cons)+lwin_big;
  POS(num+[1:length(pos_tmp)]) = pos_tmp;
  num = num + length(pos_tmp);
end
POS(num+1:end)=[];
clear num 
POS = POS + offset;
%if strand=='-' & strcmp(Signal.name,'polya')
%  POS = -POS + length(seq)+1-length(cons)+1;
%elseif strand=='-' 
if strand=='-'
  POS = -POS + length(seq)+1;
end
POS = unique(sort(POS));

return

function  jobinfo = gen_all_positions(fn_genome_config,fn_pos,Signal,fn_training_blocks,fn_test_blocks, num_splits,run_locally,RPROC) 
  
jobinfo = rproc_empty(0);
if ~run_locally,  
  jobinfo = rproc_empty(0) ;
  options = RPROC.options;
  % options.ncpus = 1;
  % options.identifier = sprintf('%spos%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;
  % options.identifier = sprintf('pos%s_%s',PAR.Signal_name,PAR.method.name) ;
end

strands = '+-';
genome_info = init_genome(fn_genome_config);

% keyboard

blocks = [];
if num_splits~=1 
  % load(FN.output.fn_regions, 'regions')
  % load(FN.output.fn_split,'split')
  if ~isempty(fn_training_blocks) && fexist(fn_training_blocks)

    % load training splits
    TR = [] ;
    TR.blocks = load_struct(fn_training_blocks, 'blocks');
    tmp = load(fn_training_blocks, 'split') ;
    TR.split = tmp.split ;
    if ~isempty(TR.split)& ~isempty(TR.blocks)
      has_split = isfield(TR.blocks,'split')&&~isempty(TR.blocks(1).split);
      splits = fieldnames(TR.split);
      for f=1:length(splits)
        regids = getfield(TR.split, splits{f});
        [temp i1 i2] = intersect(regids, [TR.blocks.id]);
        if ~has_split
          for ii2=i2, 
            [TR.blocks(ii2).split] = deal(f);
          end 
          %[TR.blocks(i2).split] = deal(f);
        else
          assert(all([TR.blocks(i2).split] == f))      
        end
      end
      clear temp i1 i2 splits regids 
      blocks = [blocks TR.blocks];
      clear TR
    end
  end
  if ~isempty(fn_test_blocks) && fexist(fn_test_blocks)
    TE=[] ;
    TE.blocks = load_struct(fn_test_blocks, 'blocks') ;
    tmp = load(fn_test_blocks, 'split');
    TE.split = tmp.split ;
    has_split = isfield(TE.blocks,'split');  
    splits = fields(TE.split);
    for f=1:length(splits)
      regids = getfield(TE.split, splits{f});
      [temp i1 i2] = intersect(regids, [TE.blocks.id]);
      if ~has_split
        for ii2=i2, 
          [TE.blocks(ii2).split] = deal(f);
        end 
        %[TE.blocks(i2).split] = deal(f);
      else
        assert(all([TE.blocks(i2).split] == f))      
      end
    end
    if isfield(TE.blocks,'right_dist')
      TE.blocks = rmfield(TE.blocks,'right_dist');
    end
    if isfield(TE.blocks,'right_dist')
      TE.blocks = rmfield(TE.blocks,'right_dist');
    end
    blocks = [blocks TE.blocks];
    clear TE;
  end
end

%[temp, num_files] = unix(sprintf('ls %scontig_*.pos | wc -l',fn_pos));
%d=dir(sprintf('%scontig_*.pos',fn_pos));
%num_files = length(d) ;
%[temp, num_files1] = unix(sprintf('ls %scontig_*.svm | wc -l',fn_pos));
%d=dir(sprintf('%scontig_*.svm',fn_pos));
%num_files1=length(d) ;

%----------
% generate positions
%-----------
trivial_regions = init_regions(fn_genome_config);

% comp_region  = ones(1,length(trivial_regions));
num_jobs=0;


for t=1:length(trivial_regions)
  if fexist(sprintf('%scontig_%i%s.pos',fn_pos,trivial_regions(t).chr_num,trivial_regions(t).strand))&...
        fexist(sprintf('%scontig_%i%s.svm',fn_pos,trivial_regions(t).chr_num,trivial_regions(t).strand))
    % comp_region(t)  = 0;
    continue
  end
  num_jobs = num_jobs+1;
  
  P.region = trivial_regions(t); 
  P.blocks = blocks;
  P.Signal = Signal;
  P.fn_pos = fn_pos;
  P.num_splits = num_splits;

  if ~run_locally
    [mem_req, time_req, opts] = rproc_memtime_policy('gen_all_positions_rproc',  0, options) ;
    jobinfo(num_jobs) = rproc_create('gen_all_positions_rproc', P, mem_req, opts, time_req) ;
  else
    gen_all_positions_rproc(P);
  end
end

return

return

function [dummy1, dummy2]=gen_all_positions_rproc(P)

% gen_all_positions_rproc(P)
% P has fields:  
%   - region;  
%   - blocks 
%   - Signal 
%   - fn_pos 
%   - num_splits 
%   
  

dummy1=[] ;
dummy2=[] ;
  
% use the rproc.addpath way to add paths in client processes!!
%
%paths  

region = P.region;  
blocks = P.blocks;
Signal = P.Signal;
fn_pos = P.fn_pos;
num_splits =  P.num_splits;


fprintf('start loading sequence for region %i%s\r',region.chr_num,region.strand)
tt = load_sequence(region);
if ~isempty(Signal.consensus)
  pos = find_consensus(tt.seq,Signal,region.strand);
elseif region.strand=='+'
  pos = Signal.lwin_big+1:length(tt.seq)-Signal.rwin_big; 
elseif region.strand=='-'
  pos = Signal.rwin_big+1:length(tt.seq)-Signal.lwin_big; 
end    

clear tt;

%-------------
% Determine SVM
%-------------
svm_num = zeros(size(pos));

% faster if there are many blocks and few splits
% needs more memory
if ~isempty(blocks),
  splits=[blocks.split] ; 
else
  splits=[] ;
end ;
assert(length(splits)==length(blocks)) ;
for split = unique(splits),
  fprintf('split %i\r',split)
  
  split_idx = find(split==splits) ;

  split_pos = {} ; p=length(split_idx); 
  for b_idx=split_idx,
    assert(~isfield(blocks, 'num_fragments') || blocks(b_idx).num_fragments<=1) ;
    if region.strand==blocks(b_idx).strand(1) && region.chr_num==blocks(b_idx).chr_num(1),
      if isfield(blocks(b_idx),'reg_no_overlap') && ~isempty(blocks(b_idx).reg_no_overlap)
        split_pos{p} = blocks(b_idx).start+[blocks(b_idx).reg_no_overlap(1):blocks(b_idx).reg_no_overlap(2)];
      else
        split_pos{p} = blocks(b_idx).start:blocks(b_idx).stop ;
      end
    end ;
    p=p-1 ;
  end ;
  split_pos = [split_pos{:}] ;
  [tmp,idx]=intersect(pos, split_pos); 
  assert(all(svm_num(idx)==0 | svm_num(idx)==split))
  svm_num(idx) = split ;
end ;

if 0, % slow version
  for b_idx = 1:length(blocks)
    fprintf('block %i\r',b_idx)
    block = blocks(b_idx);
    if ~isfield(block, 'num_fragments'),
      block.num_fragments = 1 ;
    end ;
    if ~isfield(block, 'reg_no_overlap'),
      block.reg_no_overlap = [block.start block.stop] ;
    end ;
    for f_idx = 1:block.num_fragments
      if f_idx>1
        error('block.reg_no_overlap not implemented for more than one fragment')
      end
      if region.strand==block.strand(f_idx) && region.chr_num==block.chr_num(f_idx);
        % idx =
        % find(pos>=block.start(f_idx)+PAR.regions.offset&pos<=block.stop(f_idx)-PAR.regions.offset);
        
        idx = find(pos>=block.reg_no_overlap(1) & pos<=block.reg_no_overlap(2));
        assert(all(svm_num(idx)==0 | svm_num(idx)==block.split))
        svm_num(idx) = block.split;
        % keyboard
      end
    end    
  end
end ;

filename = sprintf('%scontig_%i%s',fn_pos,region.chr_num,region.strand)
if isequal(filename(1),'~')
  filename = sprintf('%s/%s', getenv('HOME'), filename(2:end));
end

if ~isempty(find(svm_num==0))

  idx = find(svm_num==0);
  % nums = repmat([1:num_splits],1,ceil(length(idx)/num_splits));
  % nums = nums(randperm(length(nums)));
  % make blocks instead (faster computing for contents)

  num_blocks = round(max(length(idx)/10000, num_splits));
  nums = zeros(1, num_blocks*ceil(length(idx)/num_blocks)) ;
  p=1 ;
  for i=1:num_blocks
    nums( p : p+ceil(length(idx)/num_blocks)-1 ) = repmat( mod(num_blocks,num_splits) + 1, 1, ceil(length(idx)/num_blocks)) ;
    p = p + ceil( length(idx)/num_blocks ) ;
  end
  assert(p-1==length(nums));
  svm_num(idx) = nums(1:length(idx));

end

if ~isempty(pos)
  save_score_pos(pos,svm_num, filename ,{'svm'});
else
  unix(sprintf('touch %s.pos',filename));
  unix(sprintf('touch %s.svm',filename));
  warning('no position found')
end

return

function jobinfo = genomewide_predictions(fn_genome_config,fn_pos,fn_pred,fn_svms,num_splits,avg_all_svms,run_locally, RPROC, verb)
% jobinfo = genomewide_predictions(fn_pos,fn_pred,fn_genome_config,verb)


if nargin<9
  verb=0;
end
if nargin<7
  run_locally=1;
end
if nargin<6
  avg_all_svms = 0;
end

jobinfo = rproc_empty(0) ;
if ~run_locally
  jobinfo = rproc_empty(0) ;
  options = RPROC.options;
  options.ncpus = 1;
  % options.identifier = sprintf('%spred%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;
  MEMREQ = RPROC.MEMREQ;
end

    
fid=1;
q=0;

genome_info = init_genome(fn_genome_config);
num_pred = num_splits * length(genome_info.contig_names)*2;
fprintf(fid,'total number of jobs for prediction (num_svms*num_contigs*num_strands): %i\n',num_pred);

trivial_regions = init_regions(fn_genome_config);

for p = 1:num_splits
  fprintf(fid,'partition %i\n',p);
  
  fn_SVM = sprintf('%s%i.mat',fn_svms,p) ;
  %--------------------------------------------------------------------------------%
  % load parameters from SVM file
  %--------------------------------------------------------------------------------%
  
  load(fn_SVM,'Train');
  assert(Train.PAR.SETs.which_pred == p)
  
  PAR_new = Train.PAR;
  
  if isfield(PAR_new.Signal.method,'genomewide_predict_fct') && ~isempty(PAR_new.Signal.method.genomewide_predict_fct)
    PAR_new.Signal.method.predict_fct =  PAR_new.Signal.method.genomewide_predict_fct;
  end
  PAR_new.fn_SVMs = fn_SVM;
  PAR_new.pred_from_seq=1;
  PAR_new.avg_all_svms = avg_all_svms ; 
  if ~isfield(PAR_new.SETs,'which_pred')
    PAR_new.SETs.which_pred=PAR_new.SETs.which_pred_svm;
  end  
  assert(Train.PAR.SETs.which_pred == p)
  clear Train
  
  %------------------------------------------------------------------------------------%
  % perform genome-wide predictions
  %------------------------------------------------------------------------------------%
  for r_idx = 1:length(trivial_regions) 
    c = trivial_regions(r_idx).chr_num;
    s = trivial_regions(r_idx).strand;
    PAR_new.fn_pos = fn_pos;
    PAR_new.fn_pred = sprintf('%scontig_%i%s_split=%i.mat',fn_pred,c,s,p);
    if verb
      fprintf(fid,'\n start predicting on contig %i%s\n',c,s);
    end
    if ~fexist(PAR_new.fn_pred) 
      PAR_new.region = trivial_regions(r_idx);
      PAR_new.plif_on = 0;
      PAR_new.eval_on = 0;

      q=q+1; 
      if ~run_locally


        if ~isfield(RPROC,'divide4pred') || ~RPROC.divide4pred

          [mem_req, time_req, opts] = rproc_memtime_policy('load_and_predict',  0, options) ;

          if isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs 
            jobinfo(q) = rproc_create('load_and_predict', PAR_new, mem_req, opts, time_req) ;
          else
            jobinfo(q) = rproc('load_and_predict', PAR_new, mem_req, opts, time_req) ;
          end
        else
          q=q-1;
          
          output = [];
          pos = [];
          all_done=1;Temp_old =[];
          for chunk_idx=1:RPROC.numchunks
            PAR_new.chunk_idx = chunk_idx;PAR_new.RPROC.numchunks = RPROC.numchunks ;
            PAR_new.RPROC.divide4pred = RPROC.divide4pred;

            [mem_req, time_req, opts] = rproc_memtime_policy('load_and_predict',  0, options) ;

            if  ~fexist(sprintf('%s%i.mat',PAR_new.fn_pred, chunk_idx)) && isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs 
              q=q+1;
              jobinfo(q) = rproc_create('load_and_predict', PAR_new, mem_req, opts, time_req);
              all_done=0;
            elseif  ~fexist(sprintf('%s%i.mat',PAR_new.fn_pred, chunk_idx))
              q=q+1;
              jobinfo(q) = rproc('load_and_predict', PAR_new, mem_req, opts, time_req);
              all_done=0;
            else 
              Temp = load(sprintf('%s%i.mat',PAR_new.fn_pred, chunk_idx) )
              Temp.PAR = rmfield(Temp.PAR,'RPROC');
              Temp.PAR = rmfield(Temp.PAR,'chunk_idx');
              if ~isempty(Temp_old)
                assert(isequal(Temp_old.svmFileName,Temp.svmFileName));
                assert(isequal(Temp_old.PAR,Temp.PAR));
                assert(isequal(Temp_old.sg_ver,Temp.sg_ver));
              end
              Temp_old = Temp;
              output = [output Temp.output];
              pos = [pos Temp.pos];
            end
          end
          if all_done
            Temp = rmfield(Temp,'output');Temp = rmfield(Temp,'pos');
            %save(PAR_new.fn_pred,'struct','Temp');
            save_append(PAR_new.fn_pred, 0, Temp);
            %save(PAR_new.fn_pred,'-append','output','pos');
            inventory = {'output', 'pos'};
	    assert(isequal(size(pos),size(output)))
            save_append(PAR_new.fn_pred, 1, 'inventory', inventory,  'output', output, 'pos', pos);
          end
        end
      else
        load_and_predict(PAR_new)
      end
    else
      if verb
        fprintf(fid,'\n file already exists ');
      end
    end
  end%loop over contigs
end %% loop over partitions

if ~exist('jobinfo')| isempty(jobinfo),
  jobinfo = rproc_empty(0) ;
  return 
end

% [jobinfo, meta_jobinfo] = rproc_submit_batch(jobinfo, PAR.RPROC.exm_per_batch) ;
if ~(isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs),
%   [jobinfo,num_crashed] = rproc_wait(jobinfo,20,1,0);
end ;

[jobinfo,num_crashed] = rproc_wait(jobinfo, 30, 1, -1);


return

function jobinfo = gen_training_examples4signal(PAR)
% gen_labels_and_train(PAR)

%paths
  

Signal_name = PAR.Signal_name;
Signal = PAR.Signals.(PAR.Signal_name);
num_splits = PAR.SETs.num_splits;
label_source  = PAR.label_source.(Signal_name);
info = PAR.info_genes.(PAR.Signal_name);


FN = PAR.FN;
fn_genome_config = FN.input.fn_genome_config ;
fn_training_blocks = FN.output.fn_training_blocks;
fn_genes_signals = FN.output.fn_genes_signals ;

input_sig = FN.input_sig.(Signal_name);
fn_candsites =  input_sig.fn_candsites;
fn_filter_settings = input_sig.fn_filter_settings;
fn_examples = input_sig.fn_examples;
fn_example_statistics = input_sig.fn_example_statistics;

run_locally = PAR.tasks.signals.run_locally.pos ;

if ~run_locally
  RPROC = PAR.RPROC;
end



jobinfo = [] ;
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% load  trivial_regions, blocks, splits
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

trivial_regions = init_regions(fn_genome_config);


if ~fexist(fn_training_blocks)
  error('training block file does not exist; create it first with prepare_regions')
else
  blocks = load_struct(fn_training_blocks, 'blocks') ;
  load(fn_training_blocks, 'split') ;
  assert(length(fieldnames(split))==num_splits);
end


generate_examples=1;
if fexist([fn_candsites 'PAR.mat'])
  fprintf(1,'files with candidate pos already exists\n')
  Old = load([fn_candsites 'PAR.mat'],'P');
  diff_fields = compare_PAR(Signal,Old.P.Signal) ;
  if ismember('Signal',Signal_name)
    warning('label file exists, Signal parameters are differnt!')  
    % fprintf('label file exists, Signal parameters are differnt!')  
  end
  %[temp, num_files] = unix(sprintf('ls %scontig_*.pos | wc -l',fn_candsites));
  d=dir(sprintf('%scontig_*.pos',fn_candsites));
  num_files = length(d) ;

  if (num_files==length(trivial_regions))
    generate_examples=0;
    fprintf(1,'all cand files found; no label generation\n')
  else
    fprintf(1,'%i/%i cand files missing; start label generation\n',length(trivial_regions)-num_files,length(trivial_regions))
  end
  clear Old
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% generating .pos .label files
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if generate_examples
  fprintf(1,'generating labels\n')
  jobinfo = rproc_empty(0) ;
  
  P.fn_genes_signals = fn_genes_signals ;
  P.fn_candsites = fn_candsites;
  P.label_source = label_source;
  P.Signal = Signal; 
  P.info = info;
  
  D.P = P ;
  for j=1:length(trivial_regions)
    D.t_region = trivial_regions(j);
    %keyboard
    filename = sprintf('%scontig_%i%s',fn_candsites,trivial_regions(j).chr_num,trivial_regions(j).strand);
    if fexist(filename)
      continue
    end
    if run_locally
      save_label_files_starter(D);
    else
      [mem_req, time_req, opts] = rproc_mem_policy('save_label_files_starter',  0, RPROC.options) ;%RPROC.MEMREQ ;

      jobinfo(j) = rproc('save_label_files_starter', D, mem_req, opts, time_req) ;
      % save_label_files_starter(D)
    end;
    % save_label_files(t_region,PAR,genes)
  end
  if ~isempty(jobinfo),
    [jobinfo_1, num_crashed]=rproc_wait(jobinfo, 10, 1, -1);
  else
    num_crashed = 0 ;
    jobinfo_1 = rproc_empty(0) ;
  end ;
  %[temp, num_files] = unix(sprintf('ls %scontig_*.pos | wc -l',fn_candsites));
  d=dir(sprintf('%scontig_*.pos',fn_candsites));
  num_files = length(d) ;
  assert(num_files==length(trivial_regions))
  if num_crashed>0, 
    num_crashed
    %return
  end 
  if ~isempty(jobinfo_1)
    rproc_cleanup(jobinfo_1) ;
  end ;
end
clear P

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% generating example files
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

modes = {'train','test'};
for mode=modes
  [filename, exists]=gen_example_filename(Signal,fn_filter_settings,fn_examples,mode{1},num_splits);
  if ~exists
   
    fprintf(1, 'SAVE EXAMPLES ....');
    
    P.Signal = Signal;
    P.num_splits = num_splits;
    P.fn_training_blocks = fn_training_blocks;
    P.fn_candsites = fn_candsites;
    P.fn_filter_settings = fn_filter_settings ;
    P.fn_examples = fn_examples;
    P.fn_example_statistics = fn_example_statistics;
    if 0% ~run_locally
      D.blocks = blocks;
      D.split = split;
      D.P = P;

      [mem_req, time_req, opts] = rproc_mem_policy('save_examples',  0, RPROC.options) ;%RPROC.MEMREQ ;

      jobinfo = rproc('save_examples', D, mem_req, opts, time_req);
      [jobinfo_1, num_crashed]=rproc_wait(jobinfo, 60, 1, -1);
    else
      save_examples(blocks,split,P) ; num_crashed = 0 ;
    end
    if num_crashed>0
      return
    end
  end
end


return

function [POS,LABEL,CONF] = get_cand_others(P)
% [POS,LABEL,CONF] = get_cand_others(P)

debug = 0;  

genes = P.genes ;
seq = P.seq ;
Signal = P.Signal ;
info = P.info ;
strand = P.strand ;
signal_name = P.signal_name ;
%clear P

%[Signal,info,info_names,sig] = initialize_cand_search(PAR,genes,strand,signal_name,debug);
[Signal,info_names,sig] = initialize_cand_search(Signal, info, genes, strand, signal_name, debug);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% GET ALL CONSENSUS SITES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isempty(Signal.consensus)
  fprintf(1,'getting all consensus sites for: %s\n',Signal.consensus{:})
  POS =  find_consensus(seq,Signal,strand);
else
  fprintf(1,'no consensus given, all positions are potential candidates\n')
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GET ALL POSITIVE SITES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

num = 0;
pos = zeros(1,length(seq));
conf = -ones(length(info_names),length(seq))*200;
gene_idx = zeros(1,length(seq));
for g_idx = 1:length(genes)
  gene = genes(g_idx) ;
  if ~isempty(gene.(signal_name))
    unique_pos = unique([gene.(signal_name){:}]);
    unique_pos2 = setdiff(unique_pos, pos(1:num)) ;
    if ~isequal(unique_pos2, unique_pos),
      warning('identified other genes with same signal positions. Positions ignored')
      unique_pos = unique_pos2 ;
    end ;
    pos(num+[1:length(unique_pos)]) = unique_pos;
    gene_idx(num+[1:length(unique_pos)]) = repmat(g_idx,length(unique_pos),1);
    % === assertion fails for A. thaliana TSS !!! ===
    assert(length(unique(pos(1:num+length(unique_pos))))==length(pos(1:num+length(unique_pos))))
    conf_temp = zeros(length(info_names),length(unique_pos));
    for i=1:length(info_names)
      idx = find(gene.([signal_name '_info'])== ...
                 PAR.info_genes.(signal_name).(info_names{i}));
      pos_temp = [gene.(signal_name){idx}];
      [temp,idx1,idx2] = intersect(unique_pos,pos_temp);
      assert(length(idx) == length(pos_temp))
      conf_temp(i,idx1) = gene.([signal_name '_conf'])(idx(idx2));
    end
    conf(:,num+[1:length(unique_pos)]) = conf_temp; 
    num = num+length(unique_pos);
  end
end; %%%loop over genes
fprintf( 'found %d positive %s signals\n', num, signal_name );
pos(num+1:end) = [];
conf(:,num+1:end) = [];

if isempty(Signal.consensus)
  invalid_true = 0;
  neg_pos = find(~ismember([1:length(seq)],pos));
else
  invalid_true = length(setdiff(pos,POS));
  if invalid_true>10
    warning('%i positions did not exhibit the defined consensus', invalid_true) ;
  end
  pos = intersect(pos,POS); 
  neg_pos = setdiff(POS,pos);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SAMPLE DOWN
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
randn('state',24593854);
rand('state',24593854);

%neg_pos = sort(neg_pos) ;
fprintf(1,'Sampling down negatives:%d\n',Signal.sampling.downsample_neg);
if Signal.sampling.downsample_neg~=1
  var_ = 1/Signal.sampling.downsample_neg*1/5;
  %mean_ = Signal.sampling.mindist+1/Signal.sampling.downsample_neg;
  mean_ = 1/Signal.sampling.downsample_neg;
  num_new_pos = ceil((length(neg_pos)/mean_)*1.1);
  dist = abs(ceil(randn(num_new_pos,1).*var_+ mean_));
  %dist(dist<Signal.sampling.mindist)=dist(dist<Signal.sampling.mindist)+Signal.sampling.mindist;
  dist(dist==0) = deal(Signal.sampling.mindist);
  dd = cumsum(dist);
  neg_pos = intersect(neg_pos,dd);
  %keep = ceil(rand(1,length(neg_pos)*Signal.sampling.downsample_neg)*length(neg_pos));
  %keep = unique(keep);
  %neg_pos = neg_pos(keep);
elseif Signal.sampling.downsample_neg==1 & Signal.sampling.mindist>1
  remove = 1;
  while remove,
    dist = neg_pos(2:end)-neg_pos(1:end-1) ;
    idx = find(dist<Signal.sampling.mindist)+1;
    neg_pos(idx) = [];
    if isempty(idx)
      remove = 0;
    end
  end
end

POS = [pos neg_pos];
LABEL = [ones(1,length(pos)),-ones(1,length(neg_pos))]; 
COV1 = [conf zeros(size(conf,1),length(neg_pos))]; 


[temp,idx] = unique(POS);
assert(length(temp)==length(POS));
POS = POS(idx);
LABEL = LABEL(idx);
COV1 = COV1(:,idx);
if ~isequal(determine_engine, 'octave'), 
  assert(issorted(POS));
end ;


ALL = initialize_allconfs(POS,info_names);
ALL.COV1 = COV1;

  
fprintf('processing: ') ;
num_invalid_genes = 0;
for g_idx = 1:length(genes)
  if mod(g_idx, 100)==0,
    fprintf('%i/%i .. ', num, length(genes));
  end ;
  num = num+1;
  gene = genes(g_idx) ;
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % GET BASIC STUFF, 
  % if candidate pos is in gene, get gene id, if gene is valid, if
  % candidate pos is in alternative region 
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  [ALL,is_invalid] = basic_cand_stuff(gene,ALL,sig,debug,signal_name);
  num_invalid_genes  = num_invalid_genes + is_invalid;
end %%%loop over genes
fprintf('Done\n') ;


if nargout>=3,
  CONF = set_CONF(Signal, ALL, sig, info_names, signal_name);
end ;


% assert(all(sum(Conf(1:3,LABEL==1),1)))
% assert(size(Conf,1)==length(Signal.Conf_names));
fprintf(1,'number of invalid genes: \t\t\t%i\n',num_invalid_genes)
fprintf(1,'number of true %s sites: \t\t\t%i\n',signal_name,sum(LABEL==1))
fprintf(1,'number of invalid true sites (no consensus): \t%i\n',invalid_true)
fprintf(1,'number of decoy sites:\t\t\t\t%i\n',length(LABEL))
% fprintf(1,'mean confidence:\t\t\t\t%.2f\n',mean(sum(Conf,1)));

return

function [POS,LABEL,CONF] = get_cand_splicesites(P)
  
% [pos,label,Conf] = get_cand_splicesites(genes,seq,PAR)
% WARNING IMPLENENTATION OF COV_SKIPPED NOT FINISHED!!!

debug = 0;  
  
genes = P.genes ;
seq = P.seq ;
Signal = P.Signal ;
info = P.info ;
strand = P.strand ;
%clear P
  
% info = PAR.info_genes.(signal_name);
[Signal,info_names,sig,alt_names] = initialize_cand_search(Signal,info,genes,strand,'transcript',debug);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% GET ALL CONSENSUS SITES / INITIALIZE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

POS = find_consensus(seq,Signal,strand);

ALL = initialize_allconfs(POS, info_names);
ALL.LABEL =  -ones(1,length(POS)); 
clear POS;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% GET TRANSSPLICE SPECIFIC STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isequal(Signal.name,'transacc')
  ALL = get_cand_transacc(genes,ALL,Signal,info_tran);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% detecting exons in regions of alternative splicing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('detecting exons in regions of alternative splicing') ;
for i=1:length(genes)
  genes(i).alt_regions = detectaltregions(genes(i)) ;
  map = zeros(1,size(genes(i).splicegraph{1},2)) ;
  for j=1:size(genes(i).alt_regions,2),
    map = map | segment_overlap(genes(i).alt_regions(1,j)+1, ...
                                genes(i).alt_regions(2,j)-1, ...
                                genes(i).splicegraph{1}(1,:), ...
                                genes(i).splicegraph{1}(2,:)) ;
  end ;
  if ~isempty(genes(i).alt_regions),
    assert(any(map==1)) ;
  end ;
  
  %extent alt_region to neigbouring exons
  map = [0 map 0];
  idx_temp = find(map);
  map(idx_temp+1)=1;
  map(idx_temp-1)=1;
  genes(i).exon_in_alt_region=map(2:end-1) ;
  %% Version 4225 works without extension
end ;

genes = annotate_splicegraph(genes);
% genes = cover_splicegraph(genes);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% START LOOP OVER GENES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

num_invalid_genes = 0;
invalid_true = 0;

for g_idx = 1:length(genes)
  
  
  %fprintf(1,'%i\r',g_idx);
  gene = genes(g_idx) ;
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % GET BASIC STUFF, 
  % if candidate pos is in gene, get gene id, if gene is valid, if
  % candidate pos is in alternative region 
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  [ALL,is_invalid] = basic_cand_stuff(gene,ALL,sig,debug);
  num_invalid_genes  = num_invalid_genes + is_invalid;
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  % GET TRUE SPLICE SITES
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  [don_exons, acc_exons] = ind2sub(size(gene.splicegraph{2}), find(triu(gene.splicegraph{2}))) ;
  true_splice = [];
  
  if (isequal(sig,'acc') && strand=='+')||...
        (isequal(sig,'don') && strand=='-'),
    true_splice = unique(gene.splicegraph{1}(1,acc_exons)) ;
  elseif (isequal(sig,'don')&& strand=='+') || ...
        (isequal(sig,'acc')&& strand=='-')
    true_splice = unique(gene.splicegraph{1}(2,don_exons)) ;
  end
   
  [tmp,ii1,ii2] = intersect(ALL.POS,true_splice);
  
  if length(tmp)<length(true_splice)
    invalid_true = invalid_true+length(true_splice)-length(tmp) ;
  end
  if ~isequal(Signal.name,'transacc')
    ALL.LABEL(ii1) = 1;
  else
    ALL.ISSPLICE(ii1) = 1;
  end
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  % GET TRUE ALTERNATIVE SPLICE SITES
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

  if ~isempty(alt_names)
    [ALL,label_2] = get_true_alternative_sites(ALL,gene,true_splice,sig,debug);
  else
    label_2 = -ones(3,length(true_splice)); 
  end 
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  %%% CHECK COVERAGE 
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  
  if ~isempty(info_names) 
    ALL = get_coverage(ALL, gene, true_splice, label_2, info, info_names);
  end
      
  % keyboard
end %%%loop over genes

if isequal(Signal.name,'transacc')
  ALL.COV1 = ALL.CONF_TRANS;
end
if nargout>=3,
  CONF = set_CONF(Signal, ALL, sig, info_names, '');
end ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SAMPLE DOWN
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

pos=ALL.POS(ALL.LABEL==1);
neg_pos=ALL.POS(ALL.LABEL==-1);

% pos contains positive examples, neg_pos contains negative examples

%if Signal.sampling.downsample_neg~=1
%  fprintf(1,'Sampling down negatives:%d',Signal.sampling.downsample_neg);
%  randn('state',24593854);
%  dist = abs(ceil(randn(length(neg_pos),1)./Signal.sampling.downsample_neg/5+ Signal.sampling.mindist+1/Signal.sampling.downsample_neg));
%  dist(dist<Signal.sampling.mindist)=dist(dist<Signal.sampling.mindist)+Signal.sampling.mindist;
%  dd = cumsum(dist);
%  neg_pos = intersect(neg_pos,dd);
%elseif Signal.sampling.downsample_neg==1 & Signal.sampling.mindist>1
%  remove = 1;
%  while remove
%    dist = neg_pos(2:end)-neg_pos(1:end-1)
%    idx = find(dist<Signal.sampling.mindist)+1;
%    neg_pos(idx) = [];
%    if isempty(idx)
%      remove = 0;
%    end
%  end
%end

POS=[pos neg_pos];
LABEL=[ones(1,length(pos)), -ones(1,length(neg_pos))];

% keyboard
% assert(all(sum(Conf(1:3,LABEL==1),1)))
% assert(size(Conf,1)==length(Signal.Conf_names));
fprintf(1,'number of invalid genes: \t\t\t%i\n',num_invalid_genes)
fprintf(1,'number of true %s sites: \t\t\t%i\n',Signal.name,sum(LABEL==1))
fprintf(1,'number of invalid true sites (no consensus): \t%i\n',invalid_true)
fprintf(1,'number of decoy sites:\t\t\t\t%i\n',length(LABEL))
% fprintf(1,'mean confidence:\t\t\t\t%.2f\n',mean(sum(Conf,1)));

if invalid_true>0
  % keyboard
end

return

function ALL = get_coverage(ALL,gene,true_splice,label_2,info,info_names)
% [COV1,COV2] = get_coverage(ALL,gene,true_splice,info_names)
% 
% 
% get sum over all coverage for each true_splicesite and certain transcript()
% 
% INPUT  - gene:  gene structure of length 1;  
%          required fields:transcript_info,transcript_valid,exons,exons_confirmed, is_alt
%        - true_splice_site:    vector of all true splice sites (either
%          donor or acceptor, not both) that occur in the
%          splicegraph of the gene.
%        - info_names: cell which tells, you which transcripts to look
%          at ('anno','cDNA',...)
%   
% OUTPUT - cov1  number of coverage by given transcript type (according to info_names) for each true splice site. 
%        - cov2  number of times skipped by given transcript type (according to info_names) for each true splice site. 

  
COV1 = ALL.COV1 ;
COV2 = ALL.COV2;
POS = ALL.POS;
  
genic_idx = find(POS>gene.start&POS<gene.stop);
genic_pos = POS(genic_idx); 
assert(all(all(COV2(:,genic_idx)==0)))
assert(all(all(COV1(:,genic_idx)==0)))

cov1 = zeros(size(info_names,1),length(true_splice));
cov2 = zeros(size(info_names,1),length(true_splice));

% idx_const = find(sum(label_2==1)==0);
const_exons = gene.splicegraph{1}(:,find(~gene.exon_in_alt_region));
[tmp,idx_const,tmp] = intersect(true_splice,const_exons(:));
assert(all(sum(label_2(:,idx_const)==1)==0))
for type = 1:size(info_names,1) 
  if isequal(info_names{type,1},'anno')
    cov1(type,:) = -inf;
    cov2(type,:) = -inf;
  end
  t_idx = [];
  for tt1 = 1:size(info_names,2)
    if ~isempty(info_names{type,tt1})
      t_idx = [t_idx find(gene.transcript_info==info.(info_names{type,tt1}))];  
    end
  end
  if isempty(t_idx)
    continue
  end
  for t = t_idx
    if ~gene.transcript_valid(t)
      continue
    end
    [cov1_t, cov2_t] = get_coverage4transcript(gene,t,true_splice,info);
    if gene.transcript_info(t)==info.anno
      cov1(type,:) = max(cov1(type,:),cov1_t);
      cov2(type,:) = max(cov2(type,:),cov2_t); 
      if ~isempty(idx_const)&&~all(isinf(cov2_t(idx_const)))
        warning('here seems to be bug that should be fixed ... ') ;
        if keyboard_allowed()
          keyboard
        end
      end
    else
      cov1(type,:) = cov1(type,:) +cov1_t;
      cov2(type,:) = cov2(type,:) +cov2_t;
      if ~isempty(idx_const)&&~all(cov2_t(idx_const)==0)
        if keyboard_allowed()
          keyboard
        end
      end
    end
  end %%% loop over transcripts
end %%% loop over transcript types
idx_anno = find(strcmp(info_names,'anno')) ;
not_anno = setdiff(idx_anno,[1:size(info_names,1)]);

ii_alt2 = find(any(label_2==1));
ii_ir2 = find(label_2(2,:)==1);
if ~isempty(ii_alt2) && ~all(sum(cov2(not_anno,ii_alt2))>0|any(~isinf(cov2(idx_anno,ii_alt2))))
  warning('no cov2')
  % hold off;
  % viewsplicegraph(gene);
  if ~isempty(ii_ir2)
    fprintf('intron retention')
    % for a=1:length(true_alt_ir)
    %    hold on ; plot([true_alt_ir(a) true_alt_ir(a)],[1 10],'b')
    % end
  else
    %keyboard
  end
  gene.id
  warning('keyboard')
  %keyboard
end


[ii,ii1,ii2] = intersect(POS,true_splice);
assert(all(ismember(ii1,genic_idx)))

COV1(:,ii1) = cov1(:,ii2);
COV2(:,ii1) = cov2(:,ii2);

%%% SET COV2 OF NEGATIVE GENIC POSITIONS to min over coverage of
% neigbouring truesplicesites

true_splice = true_splice(ii2);
cov1 = cov1(:,ii2);


genic_neg_pos = setdiff(genic_pos,true_splice);
genic_cov2  =   zeros(size(info_names,1),length(genic_neg_pos));
[temp,ii1,ii2] = intersect(POS,genic_neg_pos);
assert(length(temp)==length(genic_neg_pos));
genic_neg_pos = genic_neg_pos(ii2);
if ~isequal(determine_engine, 'octave'),
  assert(issorted(true_splice))
end ;
for a=2:length(true_splice)
  idx = find(genic_neg_pos<true_splice(a)&genic_neg_pos>true_splice(a-1));
  genic_cov2(:,idx) = repmat(min([cov1(:,a),cov1(:,a-1)]'),length(idx),1)';
end
assert(all(all(COV2(:,ii1)==0)))
assert(all(all(COV1(:,ii1)==0)))
COV2(:,ii1) =genic_cov2;
    






function [cov1, cov2] = get_coverage4transcript(gene,transcript_idx,true_splice,info)

% [cov1, cov2] = get_coverage(gene,transcrit_idx,true_splice)
%
% for a given transcript transcrit_idx of the gene counts number of coverage for each
% of the true splice sites. 
% returned is cov1 the number of times a splice site is used in one
% evidence (e.g. transcript). In the case of ESTs this can be more than
% one, because a single transcript is produced out of a cluster of
% several ESTs.This information is stored in the
% field exons_confirmed{transcript_idx}, which is a Nx2 matrix, with the
% first number giving the confirmation for the exon and the second number
% giving the confirmation for the succeding
% intron. (exons_confirmed{transcript_idx}(end,2) shoeld always be 0, as
% there is no intron after the last exon.)
% The second returned value cov2 gives the number of evidences that
% disagree with the splicesite. e.i. the number of times a splice site is skipped.
   
  

exons = gene.exons{transcript_idx}(:,1:2);
exons_conf = gene.exons_confirmed{transcript_idx};

cov1 = -inf(1,length(true_splice));
cov2 = -inf(1,length(true_splice));

for a = 1:length(true_splice)
  %%% CHECK COVERAGE OF INCLUDED FORM     
  [idx1,idx2] = find(exons==true_splice(a));
  if ~isempty(idx1)
    assert(length(idx1)==1)
    cov1(a) = exons_conf(idx1,1); 
  end
  %%% CHECK EXON COVERAGE OF SKIPPED FORM    
  idx  = find(exons(:,1)<true_splice(a)&exons(:,2)>true_splice(a)) ;
  if ~isempty(idx)   
    assert(length(idx)==1)
    % cov2(a) = exons_conf(idx,2); 
    if gene.transcript_info(transcript_idx)==info.est|gene.transcript_info(transcript_idx)==info.cDNA &&...
          (idx==1|idx==size(exons,1))
      if all(label_2(:,a)==-1)
        continue
      else
        % alternative 3' or 5' or first or last exon; est ends
        % are usually unconfirmed 
      end
    elseif gene.transcript_info(transcript_idx)==info.anno
      if min(abs(true_splice(a)-exons(idx,:)))<3 
        % keyboard
        continue
      end
    end
    cov2(a) = exons_conf(idx,1);
  end  
  %%% CHECK INTRON COVERAGE OF SKIPPED FORM
  idx  = find(exons(1:end-1,2)<true_splice(a)&exons(2:end,1)>true_splice(a)) ;
  if ~isempty(idx)            
    assert(length(idx)==1)           
    if gene.transcript_info(transcript_idx)==info.anno
      cov2(a) = max(cov2(a),exons_conf(idx,2));      
    else
      cov2(a) = cov2(a)+exons_conf(idx,2);
    end
  end 
end %% loop over true_splice sites

assert(~any(~isinf(cov1)&~isinf(cov2)))





return

function  [ALL,label_2] = get_true_alternative_sites(ALL,gene,true_splice,sig,debug)

strand = gene.strand;
POS=ALL.POS;
LABEL_2=ALL.LABEL_2;

if (isequal(sig,'acc') & strand=='+')|...
      (isequal(sig,'don') & strand=='-'),
  which_end = 1;
  other_end = 2;
elseif (isequal(sig,'don')& strand=='+') | ...
      (isequal(sig,'acc')& strand=='-')
  which_end = 2;
  other_end = 1;
end
   
  
label_2 = -ones(3,length(true_splice)); 
% GET TRUE ALTERNATIVE SPLICE SITES

ii_es2 = []; ii_o2 = [];  ii_ir2 = [];  ii_3p2 = [];  ii_5p2 = []; 

if gene.alt_exon %% EXON SKIP
  alt_exons = gene.alt_exon_details.exons;
  true_alt_es = gene.splicegraph{1}(which_end,alt_exons(2,:)) ;
  [temp,ii1,ii_es2] = intersect(true_alt_es,true_splice);
  assert(length(temp)==length(unique(true_alt_es)))
  true_alt_es = true_alt_es(ii1);
  % splice_before_es = gene.splicegraph{1}(don_end,alt_exons(1,ii1)) ;
  label_2(1,ii_es2) = 1;
  if debug
    fprintf(1,'exon skip')
    viewsplicegraph(gene,1)
    hold on
    for jj=1:length(true_alt_es)
      plot([true_alt_es(jj) true_alt_es(jj)],[0 10],'b');
    end
    if keyboard_allowed(),
      keyboard
    end ;
  end
end %if gene.alt_exon

if gene.alt_intron
  alt_introns = gene.alt_intron_details.exons;
  true_alt_ir = gene.splicegraph{1}(which_end,alt_introns(other_end,:));
  [temp,ii1,ii_ir2] = intersect(true_alt_ir,true_splice);
  assert(length(temp)==length(unique(true_alt_ir)))
  true_alt_ir = true_alt_ir(ii1);
  % splice_before_ir = gene.splicegraph{1}(acc_end,alt_introns(1,ii1));
  % intron_ret(1,:) = gene.splicegraph{1}(which_end,alt_introns(other_end,ii1));
  % intron_ret(2,:) = gene.splicegraph{1}(other_end,alt_introns(which_end,ii1));
  label_2(2,ii_ir2) = 1;
  if debug
    fprintf(1,'intron retention')
    viewsplicegraph(gene,1)
    hold on
    for jj=1:length(true_alt_ir)
      plot([true_alt_ir(jj) true_alt_ir(jj)],[0 10],'b');
    end
    if keyboard_allowed(),
      keyboard
    end 
  end
end

if gene.alt_5prime&isequal(sig, 'don') 
  %if ~isempty(gene.alt_5prime_details), keyboard; end ;
  if gene.strand=='+',
    alt_5p = gene.alt_5prime_details.exons(1,:);
  else
    alt_5p = gene.alt_5prime_details.exons(2,:);
  end ;
  alt_5p = unique([alt_5p gene.alt_5prime_details.exons(3,:)]);
  true_alt_5p = gene.splicegraph{1}(which_end,alt_5p);
  [temp,ii1,ii_5p2] = intersect(true_alt_5p,true_splice);
  assert(length(temp)==length(unique(true_alt_5p)))
  true_alt_5p = true_alt_5p(ii1);
  % splice_before_5p = gene.splicegraph{1}(acc_end,);
  % don_after = gene.splicegraph{1}(other_end,alt_3p);
  label_2(3,ii_5p2) = 1;
  if debug
    fprintf(1,'alt 5prime')
    viewsplicegraph(gene,1)
    hold on
    for jj=1:length(true_alt_5p)
      plot([true_alt_5p(jj) true_alt_5p(jj)],[0 10],'b');
    end
    if keyboard_allowed(),
      keyboard
    end 
  end
end
if gene.alt_3prime&isequal(sig,'acc')
  %if ~isempty(gene.alt_3prime_details), keyboard; end ;
  if gene.strand=='+',
    alt_3p = gene.alt_3prime_details.exons(2,:);
  else
    alt_3p = gene.alt_3prime_details.exons(1,:);
  end ;
  alt_3p = unique([alt_3p gene.alt_3prime_details.exons(3,:)]);
  true_alt_3p = gene.splicegraph{1}(which_end,alt_3p);
  [temp,ii1,ii_3p2] = intersect(true_alt_3p,true_splice);
  assert(length(temp)==length(unique(true_alt_3p)))
  true_alt_3p = true_alt_3p(ii1);
  % splice_before_3p = gene.splicegraph{1}(don_end,[exon_alt_3prime(find(idx_alt_3prime==g_idx)).fiveprimesite]);
  % don_before = gene.splicegraph{1}(other_end,[exon_alt_3prime(find(idx_alt_3prime==g_idx)).fiveprimesite]);
  % don_after = gene.splicegraph{1}(other_end,alt_3p);
  label_2(3,ii_3p2) = 1;
  if debug
    fprintf(1,'alt 3prime')
    viewsplicegraph(gene,1)
    hold on
    for jj=1:length(true_alt_3p)
      plot([true_alt_3p(jj) true_alt_3p(jj)],[0 10],'b');
    end
    if keyboard_allowed(),
      keyboard 
    end 
  end
end
[ii,ii1,ii2] = intersect(POS,true_splice);
LABEL_2(:,ii1) = label_2(:,ii2);





return

function ALL = initialize_allconfs(POS,info_names)

ALL.POS = POS;  
  
ALL.GENE_ID = zeros(1,length(POS)); 
ALL.ALTGENIC = zeros(1,length(POS)); 
ALL.COV1 = zeros(length(info_names),length(POS)); 
ALL.COV2 = zeros(length(info_names),length(POS)); 
ALL.LABEL_2 =  -ones(3,length(POS)); 
ALL.ISSPLICE = zeros(1,length(POS)); 

ALL.TRANS = zeros(1,length(POS)); 
ALL.TSS = zeros(1,length(POS)); 
ALL.HAS_SIG = zeros(1,length(POS));
ALL.DIST = zeros(1,length(POS)); 
ALL.CONF_TRANS = zeros(1,length(POS)); 

return

function [Signal,info_names,sig,alt_names] = initialize_cand_search(Signal,info,genes,strand,signal_name,debug)
% [Signal,info,info_names,sig,alt_names] = initialize_cand_search(PAR,genes,strand,signal_name,debug)

rand('seed',1234);

% info = PAR.info_genes.(signal_name);
% Signal = PAR.Signals.(PAR.Signal_name);

if isempty(Signal.consensus) && isfield(Signal, 'consensus_fn') && ~isempty(Signal.consensus_fn)
  load(Signal.consensus_fn,'consensus')
  Signal.consensus = consensus;
end

info_names = fieldnames(info);
use = [];
fprintf(1,'getting conf for ');
for i=1:length(info_names)
  if ismember([info_names{i} '_conf'],Signal.Conf_names)
    use = [use i];
    if isequal(signal_name,'transcript')&~isequal(Signal.name,'transacc')
      assert(ismember([info_names{i} '_conf_skip'],Signal.Conf_names))
    end
    fprintf(1,'%s, ',info_names{i})
  end
end
fprintf('\n')
info_names = info_names(use);

% if ismember('cDNA',info_names)
%   use = strmatch('cDNA',info_names,'exact');
%   info_names{use,2} = 'fulllength';
%   fprintf(1,'\nfulllength cDNAs added to cDNAs\n')
% end

fprintf(1,'\n');

if ~isempty(genes)
  strands = unique([genes.strand]);
  assert(strand==strands(1))
  assert(length(strands)==1);
  assert(length(unique([genes.chr_num]))==1);
end


sig='';
switch Signal.name
 case {'acc','transacc','alt_exon_acc'}
  sig = 'acc';
 case {'don','alt_exon_don'}
  sig = 'don';
 otherwise
  sig = 'other';
end

if isequal(sig,'other') || isequal(Signal.name,'transacc') || ...
      isequal(Signal.name,'acc') || isequal(Signal.name,'don') 
  alt_names ={};
  return
end

alt_names = {'alt_exon', 'alt_intron', 'alt_5prime', 'alt_3prime'};
use = []; 
fprintf(1,'getting label 2 for ');
for i=1:length(alt_names)
  if ismember(alt_names{i},Signal.Conf_names)
    fprintf(1,'%s, ',alt_names{i})
  end
end

return

function init_svm_and_kernels(method,pred_from_seq)
% init_svm_and_kernels(method,pred_from_seq)
  
  
  conservWdsWeighting =0;
  %------------------------------------------------------------------------------%
  % init svm
  %------------------------------------------------------------------------------%
  
  % === cleaning
  sg( 'clean_features', 'TRAIN' );
  sg( 'clean_preproc' );
  sg( 'clean_kernel');
  %clear sg;
  
  %sg( 'loglevel', 'ALL');   % DEBUG
  if ~isfield(method, 'svm_name')
    warning( 'no svm specified, set to default: LIGHT')
    method.svm_name='LIGHT'
  end
  
  % === SVM
  sg( 'new_svm', method.svm_name);
  %sg( 'svm_epsilon', 1e-4 );
  %%%sg( 'svm_epsilon', PAR.svmEps);
  sg('svm_epsilon', 1e-4);
  %%%sg( 'send_command', sprintf( 'use_linadd %d', PAR.useLinadd ) );
  sg('use_linadd', true);
  %%%sg( 'svm_qpsize', PAR.qpSize );
  if ~isfield(method, 'qpsize')
    warning( 'qpsize not specified, set to default: 50')
    method.qpsize=50;
  end
  sg('svm_qpsize', method.qpsize) ; 
  if( conservWdsWeighting > 0 )
    sg('use_batch_computation', false);
  else
    sg('use_batch_computation', true);
  end
  
  % === parallel training?
  if isfield(method,'threads')
    sg('threads', method.threads);
  else
    fprintf('number of threads not specified, set to default: 16')
    sg('threads', 16);
  end;
  
  
  %------------------------------------------------------------------------------%
  % init kernels
  %------------------------------------------------------------------------------%
  
  nofKernels = length(method.kernels);
  if nofKernels>1 && ~isfield(method.kernels{2},'name') && ~isfield(method.kernels{2}, 'kernel_name')
    % for content sensors; several orders are trained simulatuously.
    % but this is done by concatination of the feature vector directly.
    nofKernels = 1;
  end
  
  assert( nofKernels >= 1 );
  if( nofKernels == 1 )
    addKernel = {'set_kernel'};
  else
    addKernel = {'add_kernel', 1};
    sg( 'set_kernel', 'COMBINED', 100);
  end;
  
  for k = 1:nofKernels
    kernel = method.kernels{k};
    % for compatibility with older versions
    if ~isfield(kernel,'kernel_name') && isfield(kernel, 'name')
      kernel.kernel_name = kernel.name;
    end
    fprintf(1, '%s: %s\n', sprintf('%s %1.1f', addKernel{:}), kernel.kernel_name)
    
    switch kernel.kernel_name
     case 'WD'
      if ~isfield(kernel, 'mismatch')
        kernel.mismatch=0;
      end
      assert(length(kernel.order)==1)
      assert(length(kernel.mismatch)==1)
      sg( addKernel{:}, 'WEIGHTEDDEGREE', 'CHAR', 10, kernel.order, kernel.mismatch, true, 1) ;
      
     case 'WDS'
      if ~isfield(kernel, 'mismatch')
        kernel.mismatch=0;
      end
      if ~isfield(kernel, 'shift')
        kernel.shift=0;
      end
      if ~isfield(kernel, 'shift_const')
        kernel.shift_const=0;
      end
      assert(length(kernel.order)==1)
      assert(length(kernel.mismatch)==1)
      len = kernel.rwin-kernel.lwin+1;
      shifts = ceil(kernel.shift * abs( (kernel.lwin:kernel.rwin)+1 ));
      shifts = shifts + kernel.shift_const;
      %shifts = sprintf( '%i ', shifts );
      sg( addKernel{:}, 'WEIGHTEDDEGREEPOS3', 'CHAR', 10, kernel.order, kernel.mismatch, len, 1, int32(shifts)) ;
      clear shifts len
      
     case 'SP'
      use_sign = false;
      % use_sign and prediction from position lists does not work
      %assert(~use_sign|~PAR.tasks.pred_from_seq)
      
      %addKernel_ = {'add_kernel', 0};
      
      sg( addKernel{:}, 'COMMSTRING', 'WORD', 10, use_sign);
      if pred_from_seq
        %sg('loglevel','ALL')
        sg( 'use_diagonal_speedup',true)
      end
     case 'RBF'
      if ~isfield(kernel, 'cache')
        kernel.cache = 40;
      end
      assert(length(kernel.sigma)==1)
      sg( addKernel{:}, 'GAUSSIAN', 'REAL', kernel.cache, kernel.sigma) ;
      
     case 'POLY'
      if ~isfield(kernel, 'cache')
        kernel.cache = 40;
      end
      if ~isfield(kernel, 'type')
        kernel.type = 'REAL';
      end 
      if ~isfield(kernel, 'norm')
        kernel.type = 'R';
      end
      assert(length(kernel.degree)==1)
      sg( addKernel{:}, 'POLY', Kernel.type, kernel.cache, kernel.degree, kernel.norm) ;
      
     case 'LINEAR'
      if ~isfield(kernel, 'cache')
        kernel.cache = 40;
      end
      if ~isfield(kernel, 'type')
        kernel.type = 'REAL';
      end
      sg( addKernel{:}, 'LINEAR', kernel.type, kernel.cache) ;
      
     otherwise
      fprintf(1, 'not yet implemented: kernel_name: %s\n', kernel.kernel_name)
    end;
    
  end;%loop over kernels
  
  if isfield(method, 'subkernel_weights') 
    assert(length(method.subkernel_weights)==nofKernels)
    sg('set_subkernel_weights', method.subkernel_weights);
  end
  
  % for compatibility with older versions
  if ~isfield(method.par_ms,'C') && isfield(method.kernels{1}, 'C')
    method.par_ms.C = method.kernels{1}.C;
  end
  sg('c', method.par_ms.C) ;
  
  disp('done with kernel setup');

return

function  [dummy1, dummy2]=load_and_predict(P, Test_Data)
% load_and_predict(P, Test_Data)

% used for both evaluation on the leftout set of the training region and
% prediction on test regions
% 
% * IF PAR.eval_on is set to 1 prediction is performed on the evaluation
%   set specified by PAR.SETs.which_val. The model saved in PAR.fn_SVMs is used for
%   prediction. Additionally RSV and RFV are determined.
%  
% * IF PAR.eval_on is set to 0  only prediction is performed on the test set
%   specified by PAR.SETs.which_pred.
% 
% 

  dummy1 = [] ;
  dummy2 = [] ;

fn_SVMs = P.fn_SVMs ;
SETs = P.SETs;
pred_from_seq = P.pred_from_seq;
Signal = P.Signal;
fn_examples = P.fn_examples;
fn_filter_settings = P.fn_filter_settings;

if isfield(P,'avg_all_svms') 
  avg_all_svms = P.avg_all_svms;
else
  avg_all_svms = 0;
end

if isfield(P,'check_overlap')
  check_overlap = P.check_overlap;
else
  check_overlap = 1;
end

if isfield(P,'eval_on') && P.eval_on
  which_set = SETs.which_val;
  if isfield(P,'fn_eval')
    fn_pred = P.fn_eval;
  else
    fn_pred = [];
  end
elseif isfield(P,'plif_on') && P.plif_on
  which_set = SETs.which_plif;
  if isfield(P,'fn_plifs')
    fn_pred = P.fn_plifs;
  else
    fn_pred = [];
  end
else
  which_set = SETs.which_pred;
  if isfield(P,'fn_pred')
    fn_pred = P.fn_pred;
  else
    fn_pred = [];
  end
end

if pred_from_seq
  fn_pos = P.fn_pos;
  if isequal(fn_pos(1),'~')
    fn_pos = sprintf('%s/%s', getenv('HOME'), fn_pos(2:end));
  end
  region = P.region;
  fn_pred = P.fn_pred;
  chunk_idx = 1;% P.chunk_idx;
  RPROC = struct;
end

load(fn_SVMs, 'Train') ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% load data 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isfield(SETs,'test_subsample_pos')
  subsample_pos = 1;
else
  subsample_pos = SETs.test_subsample_pos;
end
if ~isfield(SETs,'test_subsample_neg')
  subsample_neg = 1;
else
  subsample_neg= SETs.test_subsample_neg;
end


if ~pred_from_seq
  pred_from_seq=0;
  if ~exist('Test_Data')
    Test_Data = feval(Signal.load_fct,Signal,fn_filter_settings,fn_examples, which_set, 'test',subsample_neg, subsample_pos);
  end
  
  %  if isequal(PAR.method.name,'SVM')
    % [RFV RSV]=tssTest(Train, Test_Data, fn_SVMs, fn_pred);
    % varargout{1} = RFV;
    % varargout{2} = RSV;
 %  elseif isequal(PAR.method.name,'PWM')
    % hmm_test(Train, Test_Data, fn_SVMs,fn_pred);
 %  end
else %genomweite Vorhersage
  
  region = load_sequence(region);
  region = retrieve_signals(region,fn_pos,Signal.name,{'svm'});
  Test_Data.seq = region.seq;
  Test_Data.pos = region.Signals.(Signal.name).pos';
  Test_Data.svm = region.Signals.(Signal.name).svm';
  idx = find((Test_Data.pos<Signal.lwin_big+1)|(Test_Data.pos>length(Test_Data.seq)-Signal.rwin_big-1));
  if length(idx)==length(Test_Data.pos), 
    % error('no examples in valid region found'); 
  end
  Test_Data.pos(idx)=[];
  Test_Data.svm(idx)=[];
  % load(PAR.fn_pos,'pos','seq','region_id','svm_num') ;
  if ~all(ismember(unique(Test_Data.seq),['ACGT']))
    warning('sequence alphabet different from ACGT; setting all Ns to A')
    Test_Data.seq(~ismember(Test_Data.seq,['ACGT']))='A';
  end
  if ~avg_all_svms
    idx = find(Test_Data.svm==which_set);
  else
    idx = 1:length(Test_Data.svm);
  end
  if isequal(Signal.method.predict_fct,'svm_content_predict')
    idx = [idx length(Test_Data.svm)];
  end
  if length(idx)==0, warning('no examples for specified svm found'); end
  Test_Data.pos = Test_Data.pos(idx);
  Test_Data.svm = Test_Data.svm(idx);
  if isfield(RPROC, 'divide4pred') && RPROC.divide4pred
    junk_length = ceil(length(Test_Data.pos)/RPROC.numjunks) ;
    idx_sub = (chunk_idx-1)*junk_length+1:min(chunk_idx*junk_length,length(Test_Data.pos));
    Test_Data.pos = Test_Data.pos(idx_sub);
    Test_Data.svm = Test_Data.svm(idx_sub);
    fn_pred = sprintf('%s%i.mat',fn_pred, chunk_idx);
  end
  %   Test_Data.region_id = Test_Data.region_id(idx);
end

Test_Data.PAR = P;

feval(Signal.method.predict_fct, Train, Test_Data,fn_SVMs,fn_pred,pred_from_seq, check_overlap);

return

function [Data,par] = load_data(Signal,fn_filter_settings,fn_examples, which, filter_mode,subsample_neg,subsample_pos)
% [Data,par] = load_data(Signal,fn_filter_settings,fn_examples, which, filter_mode,subsample_neg,subsample_pos)
% [Data,par] = load_data(PAR, which, filter_mode,subsample_neg,subsample_pos)
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% load data of training set
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% method = PAR.method;
% Signal = PAR.Signals.(PAR.Signal_name);
%  FN = PAR.FN;
% input_sig = FN.input_sig.(Signal_name);
% fn_filter_settings = input_sig.fn_filter_settings;
% fn_examples = input_sig.fn_examples;

method = Signal.method;

XT = []; LT = [] ; POS.region_id = []; POS.pos=[]; POS_tmp = []; 
SC = []; use_sc = [];
if nargin<4
  subsample_neg=1;
end
if nargin<5
  subsample_pos=1;
end

use_cons = Signal.method.use_cons;
% if nargin<6
%   use_cons = 0;
% end
for w = which
  % create filename of input file
  filename = gen_example_filename(Signal,fn_filter_settings,fn_examples,filter_mode,w);
  
  fprintf('loading XT, LT and POS from file %s \n',filename)
  L = load(filename,'LT');
  L.XT = load_matrix(filename, 'XT');
  P = load(filename, 'POS');
  %if isfield(signal,'center')
  %  par.center = signal.center;
  %else
  %  par.center = size(L.XT,1)./2;
  %end
  
  have_pos = isfield(P, 'POS') ;
  have_lt = isfield(L, 'LT') ;
  if ~have_lt, L.LT=[] ; end ;
  have_sc = isfield(L, 'SC_') ;
  if ~have_sc, L.SC_=[] ; end ;  

  [L.XT, L.LT, P.POS, L.SC_] = subsample(L.XT, L.LT, P.POS, L.SC_, subsample_neg, subsample_pos) ;

  if ~have_lt, L=rmfield(L, 'LT') ; end ;
  if ~have_sc, L=rmfield(L, 'SC_') ; end ;
  
  if ~isfield(method,'feature_type') || isempty(method.feature_type) ||isequal(method.feature_type, 'char')
    L.XT_char = char(L.XT) ;
    for i=1:size(L.XT,1)
      L.XT_char(i,:) = char(double(L.XT(i,:))) ;
    end ;
    XT = [XT L.XT_char];
  elseif isequal(method.feature_type, 'uint8')
    L.XT_char = uint8(L.XT) ;
    for i=1:size(L.XT,1)
      L.XT_char(i,:) = uint8(double(L.XT(i,:))) ;
    end ;
    XT = [XT L.XT_char];
  elseif isequal(method.feature_type, 'double')
    XT = [XT L.XT];
  else
    error('unknown feature type; should be "char" or "double"')
  end
  if isfield(L,'LT')
    if size(L.LT,1)>size(L.LT,2)
      L.LT = L.LT';
    end
    LT = [LT L.LT] ;
  end
  
  if (isfield(L,'SC_')&& isfield(L.use_sc)) || use_cons
    fprintf('using consensus information\n');
    SC = [SC L.SC_] ;
    use_sc = [use_sc L.use_sc];
  end

  POS.region_id =[POS.region_id P.POS.region_id];
  POS.pos = [POS.pos P.POS.pos];
end ;
%%% just for compatibility with older versions 
  
% check whether everything is ok (matlab bug successfully avoided?)
for i=1:size(XT,1)
  if all(XT(i,:)==0)
    % i
    % keyboard
  end
  if  ~isfield(method,'feature_type') || isequal(method.feature_type, 'char')
    assert(all(XT(i,:)~=0)) ;
  end
end ;

clear L
if ~isempty(LT)
  fprintf('%i positive, %i negative examples (%.2f %% positives), sequences of length %i \n',sum(LT==1),sum(LT==-1),sum(LT==1)/length(LT)*100, size(XT,1));
end

if ~isempty(SC)
  XT(:,~use_sc) = [] ;
  POS(~use_sc) =[];
  LT(~use_sc) = [];
  SC(:,~use_sc) = [];
  if ~isempty(LT)
    fprintf('%i positive, %i negative examples (%.2f %% positives) \n',sum(LT==1),sum(LT==-1),sum(LT==1)/sum(LT==1|LT==-1)*100);
  end
end

Data.LT = LT;
% Data.XT = char(XT) ;
Data.XT = XT;
Data.Pos = POS;
clear XT LT POS
%this assert is very mem consuming
%assert(~any(XT(:)~='A' & XT(:)~='G' & XT(:)~='C' & XT(:)~='T')) ;
% XT = to_upper_case(char(XT)) ;
if ~isempty(SC)
  Data.SC = char(SC) ;
  assert(~any(SC(:)~='A' & SC(:)~='G' & SC(:)~='C' & SC(:)~='T')) ;
  
  % SC = to_upper_case(char(SC)) ;
end


assert(size(Data.XT,2)==length(Data.LT))
assert(length(Data.LT)==length(Data.Pos.pos))
assert(length(Data.LT)==length(Data.Pos.region_id))

return

function [num_data, num_pos, num_neg] = count_data(Signal, fn_filter_settings, fn_examples, which, filter_mode, subsample_neg, subsample_pos)
% [num_data, num_pos, num_neg] = count_data(Signal, fn_filter_settings, fn_examples, which, filter_mode,subsample_neg,subsample_pos)
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% count data of training set
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

method = Signal.method;

LT = [] ; POS.pos=[]; 
if nargin<4
  subsample_neg=1;
end
if nargin<5
  subsample_pos=1;
end

for w = which
  % create filename of input file
  filename = gen_example_filename(Signal,fn_filter_settings,fn_examples,filter_mode,w);
  
  fprintf('loading LT and POS from file %s \n',filename)
  L = load(filename, 'LT');
  P = load(filename, 'POS');
  
  have_pos = isfield(P, 'POS') ;
  have_lt = isfield(L, 'LT') ;
  if ~have_lt, L.LT=[] ; end ;

  [tmp, L.LT, P.POS] = subsample(L.LT, L.LT, P.POS, [], subsample_neg, subsample_pos) ;

  if ~have_lt, L=rmfield(L, 'LT') ; end ;
  
  if isfield(L,'LT')
    if size(L.LT,1)>size(L.LT,2)
      L.LT = L.LT';
    end
    LT = [LT L.LT] ;
  end
  
  POS.pos = [POS.pos P.POS.pos];
end ;
%%% just for compatibility with older versions 
  
clear L
if ~isempty(LT)
  fprintf('%i positive, %i negative examples (%.2f %% positives)\n',sum(LT==1),sum(LT==-1),sum(LT==1)/length(LT)*100);
  num_data = length(LT) ;
  num_pos = sum(LT==1) ;
  num_neg = sum(LT==-1) ;
else
  num_data = length(POS.pos) ;
  num_pos = nan ;
  num_neg = nan ;
end

return

function regions=load_sequence(regions);

% regions=load_sequence(regions);
  
genome_info = init_genome(regions(1).config);


for i=1:length(regions)
  if isfield(regions(i),'num_fragments')&&~isempty(regions(i).num_fragments)
    num_fragments = regions(i).num_fragments;
  else
    num_fragments = 1;
  end
  for p=1:num_fragments
    if mod(i,100)==0,
      fprintf(1,'region %i of %i\r',i,length(regions))
    end ;
    chr = regions(i).chr_num(p) ;
    str = regions(i).strand(p) ;
    start = regions(i).start(p) ;
    stop = regions(i).stop(p) ;
    seq = upper(load_genomic(genome_info.contig_names{chr},str,start,stop,genome_info));
    assert(ischar(seq)) ;
    idx = (~ismember(seq,'ACGT')) ;
    seq(idx)='N';
    if p==1
      regions(i).seq = seq;
    else
      regions(i).seq = [regions(i).seq ,seq];
    end
  end
end

return

function [PAR, jobinfo ,all_done]=masterscript_sensors(PAR,check_files,keep_all_files,verb)
% [PAR, jobinfo ,all_done] = masterscript_sensors(PAR,check_files,verb)

%cd ~/svn/projects/genefinding/sensors
%paths

tasks = PAR.tasks.signals;  
%run_locally = tasks.run_locally;

FN = PAR.FN;  
fn_genome_config = PAR.FN.input.fn_genome_config;
 
if nargin<2 
  check_files =0;
end
if nargin<3 
  keep_all_files = 0;
end
if nargin<4 
  verb = 0;
end

fid =1;
fprintf(fid,'\n\n-----------\n');
fprintf(fid,'SENSOR :  %s\n',PAR.Signal_name);
fprintf(fid,'-----------\n');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% --- Train SENSORS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PAR.method.submit_batch=1;
all_done=0;
%train_ok = 0;

jobinfo=struct ;

if tasks.prepare_labels
  %fprintf('start training for sensor %s on organism %s with method %s: %s\n\n',sensor,organism,method)
  gen_training_examples4signal(PAR) ;
end
if tasks.modelsel | tasks.train
  jobinfo = ms_gfwd_starter(PAR);
  if ~isempty(jobinfo), return ; end ;
end
if tasks.get_train_error
  jobinfo = predict_testplif_geterror(PAR)
  if ~isempty(jobinfo), return ; end ;
end
if tasks.learn_plifs
  calc_plifs(PAR);
end

%if ~train_ok
%  fprintf('something wrong with training files, check first!!  \n')
%end


if ~keep_all_files
  unix(sprintf('rm -rf %s', FN.input_sig.(PAR.Signal_name).fn_candsites));
  unix(sprintf('rm -rf %s*', FN.input_sig.(PAR.Signal_name).fn_examples));
  unix(sprintf('rm -rf %s*', FN.input_sig.(PAR.Signal_name).fn_example_statistics(1:end-4)));  
  unix(sprintf('rm -rf %s', FN.input_sig.(PAR.Signal_name).fn_pos));
  unix(sprintf('rm -rf %s', FN.input_sig.(PAR.Signal_name).fn_filter_settings));

  unix(sprintf('rm -rf %s', FN.output_sig.(PAR.Signal_name).fn_pred));
  unix(sprintf('rm -rf %s/ms_set*', FN.output_sig.(PAR.Signal_name).fn_SVMs));
  unix(sprintf('rm -rf %s/partition=*', FN.output_sig.(PAR.Signal_name).fn_SVMs));  
end

if ~tasks.pred_on_genome
  all_done = 1 ;%train_ok ;
  return
end

return

function [jobinfo,fn_pred]=masterscript_sensors_test(P, check_files, run_locally, do_gzip, save_as, keep_all_files)
% [jobinfo ,all_done]=masterscript_sensors_test(P, check_files, run_locally, do_gzip, save_as, keep_all_files)

num_splits = P.SETs.num_splits ; 
Signal_name = P.Signal_name;

FN = P.FN;  
fn_genome_config = FN.input.fn_genome_config;
fn_training_blocks = FN.output.fn_training_blocks;  
fn_test_blocks = FN.output.fn_test_blocks;

fn_pos =  FN.input_sig.(Signal_name).fn_pos;
fn_pred = FN.output_sig.(Signal_name).fn_pred;
fn_svms = FN.output_sig.(Signal_name).fn_SVMs ;
% fn_svms = sprintf('%sbest_partition=', FN.output_sig.(Signal.name).fn_SVMs) ;

L = load([fn_svms '1.mat'], 'Train');
Signal =  L.Train.PAR.Signal;
clear L 

if nargin<2 
  check_files = 1;
end

if nargin<3 
  run_locally.pos = 1 ;
  run_locally.predict = 1 ;
  run_locally.conf = 1 ;
  run_locally.concat = 1 ;
  run_locally.save = 1 ;
end

if nargin<4 
  do_gzip = 0;
end

if nargin<5 
  save_as.spf_ascii = 1 ;
  save_as.spf_binary = 1 ;
  save_as.wiggle = 0 ;
end

if nargin<6 
  keep_all_files = 0 ;
end

%if ~run_locally  
RPROC = P.RPROC;
%else
%  RPROC = struct;
%end

fid =1;
fprintf(fid,'\n\n-----------\n');
fprintf(fid,'SENSOR :  %s\n',Signal.name);
fprintf(fid,'-----------\n');

all_done = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% --- Generate Positions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

jobinfo=struct ;

%[temp, num_files] = unix(sprintf('ls %scontig_*.pos | wc -l',fn_pos));
d=dir(sprintf('%scontig_*.pos',fn_pos));
num_files = length(d) ;
%[temp, num_files1] = unix(sprintf('ls %scontig_*.svm | wc -l',fn_pos));
d=dir(sprintf('%scontig_*.svm',fn_pos));
num_files1=length(d) ;

genome_info = init_genome(fn_genome_config);
if isempty(num_files)||(num_files<length(genome_info.contig_names)*2)||...
      isempty(num_files1)||(num_files1<length(genome_info.contig_names)*2)
  jobinfo = gen_all_positions(fn_genome_config,fn_pos,Signal,fn_training_blocks,fn_test_blocks,num_splits,run_locally.pos,RPROC);
  if ~isempty(jobinfo),
    fprintf('generating genome position files\n')
    jobinfo = rproc_resubmit(jobinfo);
    jobinfo = rproc_wait(jobinfo, 5, 1, -1); 
  end ; 
else
  fprintf('genome position files already exist\n')
end

avg_all_svms = 0;
subtract_diff = 0;
jobinfo = genomewide_predictions(fn_genome_config,fn_pos,fn_pred,fn_svms,num_splits,avg_all_svms,run_locally.predict, RPROC);
fprintf('all %s predictions done\n',Signal.name)

jobinfo = convert_predictions2confs(fn_genome_config,fn_pred,fn_svms,num_splits, run_locally.conf, RPROC);
fprintf('conversion done\n')


jobinfo = concat_genomewide_predictons(fn_genome_config,fn_pos,fn_pred, num_splits,avg_all_svms,subtract_diff,run_locally.concat, RPROC);
fprintf('concatination done\n')


resolution = Signal.export_settings.resolution ;


conf_cum_thresh = Signal.export_settings.conf_cum_thresh;

inventory = {'output', 'pos', 'Conf', 'Conf_cum'} ;
[jobinfo] = save_predictions(fn_genome_config, fn_pred, ...
                             Signal_name, resolution, ...
                             conf_cum_thresh, '+-', ...
                             run_locally.save, RPROC, ...
                             do_gzip, save_as, inventory);
fprintf('saving done\n')


if check_files
  % fprintf('checking files \n')
  % check_genomewide_output(P) % this fct compares prediction results with test results from training (cross_validation)
  % all_good = check_predictions(fn_genome_config,fn_pred,Signal_name,resolution)
else
  fprintf('WARNING: files not checked\n')
end


fprintf('%s COMPLETE !\n\n',Signal.name)


if ~keep_all_files
 %  unix(sprintf('rm -rf %s*_split=*', FN.output_sig.(Signal_name).fn_pred));
end

return

function jobinfo = ms_gfwd_starter(PAR)

% ms_gfwd_starter(PAR)
%
% used for modelselection as well as simple training and evaluation 
% INPUT : struct PAR as generated by create_PAR
%         fields used: 
%         - Signal_name: string specifying the signal that is actually computed
%           (e.g. 'acc','don') 
% 
%         - Signals.(Signal_name) : struct containing all necessary information to
%           compute the signal,
% 
%         - SETs : struct with information about split and partitions, 
%   
%         - FN : struct containing all used filenames including: 
%           FN.input_sig.(Signal_name).fn_filter_settings; 
%           FN.input_sig.(Signal_name).fn_examples; 
%           FN.output_sig.(Signal_name).fn_models;
%           FN.output_sig.(Signal_name).fn_SVMs;
%           FN.output_sig.(Signal_name).fn_motifs;
%   
%         - tasks.signals struct specifying what will be done
%           fields include run_locally (default=1), run_all,
%  
%         - RPROC (only necessary when tasks.signals.run_locally=0)
  
  
fid = 1;%fopen(PAR_1.FN.fn_log ,'a+');
FN = PAR.FN;
if ~isempty(PAR.Signal_name)
  Signal_name = PAR.Signal_name;
  Signal = PAR.Signals.(Signal_name);
  input_fn = FN.input_sig.(Signal_name);
  output_fn = FN.output_sig.(Signal_name);
  tasks = PAR.tasks.signals;
else
  Signal_name = PAR.Content_name;
  Signal = PAR.Contents.(Signal_name);
  input_fn = FN.input_cont.(Signal_name);
  output_fn = FN.output_cont.(Signal_name);
  tasks = PAR.tasks.contents;
end
fn_filter_settings = input_fn.fn_filter_settings;
fn_examples = input_fn.fn_examples;


method = Signal.method;
SETs = PAR.SETs;

if ~isfield(tasks,'run_locally')
  tasks.run_locally = 1;
end
if ~isfield(tasks,'run_all')
  tasks.run_all = 0;
end 

if ~tasks.run_locally.train
  RPROC = PAR.RPROC;
  organism = PAR.organism.name;
  RPROC.options.identifier = sprintf('%s%s',Signal_name,upper(organism)) ;
  RPROC.options.ncpus = 1;
  %if isfield(RPROC.options, 'addpaths') && iscell(RPROC.options.addpaths)
  %  RPROC.options.addpaths = {RPROC.options.addpaths{:}, fileparts(which(sg))};
  %else
  %  RPROC.options.addpaths = {fileparts(which(sg))};
  %end
  %path
end  

fprintf(fid,'\nstarting ms_gfwd_starter \n');
fprintf(fid,'date: %s\n',date);
fprintf(fid,'\nSITE: %s\n',Signal_name);


%------------------------------------------------------------------------%
% write numeric kernelpars into field for model selection
%------------------------------------------------------------------------%

if isfield(method, 'kernels') && ~isempty(method.kernels{1})
  method.par_ms = collect_ms_parameters(method.kernels, method.par_ms);
end

%------------------------------------------------------------------------%
% initiate parameters
%------------------------------------------------------------------------%
Model_param = getfield(method,'par_ms');
Model_param = orderfields(Model_param) ;

[MODELS,param_names] = generate_all_models(orderfields(method.par_ms));
if ~tasks.modelsel
  fprintf('no model selection. taking first model;\n')
  MODELS = MODELS(:,1);
end

fprintf(fid,'number of models to be tested %s: %i\n',Signal_name,size(MODELS,2));
num_models_tested = size(MODELS,2);
RSV_eval = zeros(SETs.num_partitions,size(MODELS,2));
RFV_eval = zeros(SETs.num_partitions,size(MODELS,2));

%------------------------------------------------------------------------%
% check for saved model files and save models if non are found
%------------------------------------------------------------------------%
if ~fexist(output_fn.fn_models) && ~fexist([output_fn.fn_models '.mat']) 
  save(output_fn.fn_models,'MODELS','Model_param')
elseif (fexist(output_fn.fn_models) || fexist([output_fn.fn_models '.mat']) ) 
  LL = load(output_fn.fn_models);
  assert(isequal(LL.Model_param,Model_param))
  if ~isequal(LL.MODELS,MODELS)
    MODELS=append_new_models(MODELS, LL.MODELS);
    old=LL;
    save(output_fn.fn_models,'MODELS','old')
    warning('different other MODELS have already been trained, new models are appended')
  end
  clear LL old
end
num_not_eval = SETs.num_partitions*sum(num_models_tested) ;

fprintf(fid,'total number of jobs for training: %i\n',sum(SETs.partition_train_on)*sum(num_models_tested));
fprintf(fid,'total number of jobs for evaluation: %i\n',num_not_eval);


P.SETs = SETs;
P.Signal = Signal;
P.fn_filter_settings = fn_filter_settings;
P.fn_examples = fn_examples;
P.train_error_on = tasks.get_train_error;
P.pred_from_seq = 0;

P.use_preproc = 1;

num_crashed = 1;
num_crashed_1 = 0;
rep = 1;

P.check_overlap = 1;
while num_crashed + num_crashed_1 + num_not_eval>0 & rep<5
  q = 0 ; nq = 0 ;
  jobinfo = rproc_empty(0) ;
  jobinfo_1 = rproc_empty(0) ;
  num_not_eval = SETs.num_partitions*sum(num_models_tested) ;
  for p = 1:size(SETs.partitions,1) %% loop over partitions  
    fprintf(fid,'\n\n-----------\n');
    fprintf(fid,'PARTITION %i\n',p);
    fprintf(fid,'-----------\n');
    
    if tasks.run_all
      Train_data=[];
      Test_data=[];
    end
    
    P.partition = p;
    P.train_on = SETs.partition_train_on(p);
    P.SETs.which_train = find(ismember(SETs.partitions(p,:),SETs.info.train));
    P.SETs.which_val = find(ismember(SETs.partitions(p,:),SETs.info.eval));
    P.SETs.which_pred = find(ismember(SETs.partitions(p,:),SETs.info.pred));
    
    fn_save = [sprintf('%sms_set=[',output_fn.fn_SVMs),...
               sprintf('%i',P.SETs.which_train)];
    fn_save_pred  = [fn_save '_' sprintf('%i',P.SETs.which_val)];
    
    for num_models = 1:size(MODELS,2)
      %--------------------------------------------------------------%
      % assign parameters of model number num_models to the current parameter
      % structure
      %--------------------------------------------------------------%
      
      P.Signal.method = assign_model(method, MODELS, param_names,num_models); 
      
      %--------------------------------------------------------------%
      %% generate filename for SVMs and predictions
      %--------------------------------------------------------------%
      
      P.fn_SVMs = sprintf('%s]_model=%i.mat',fn_save,num_models) ;
      P.fn_eval = sprintf('%s]_model=%i.mat',fn_save_pred,num_models) ;
      
      %--------------------------------------------------------------%
      %%%%% TRAIN
      %--------------------------------------------------------------%
      
      %if ~fexist(P.fn_SVMs) & P.train_on
      %  fprintf(fid,'waiting for file %s...\n', P.fn_SVMs);
      %pause(1)
      %end      

      if ~fexist(P.fn_SVMs) & P.train_on
        q = q+1 ;
        fprintf(fid,'\n start training on set ');
        fprintf(fid,'%i',P.SETs.which_train);
        fprintf(fid,'\n')
        
        train_fct = 'load_and_train';
        
        if tasks.run_all
          Data = load_data(Signal, fn_filter_settings, fn_examples, P.SETs.which_train, 'train', SETs.train_subsample_neg, SETs.train_subsample_pos);
          Train_data.(sprintf('partition_%i',p))= Data ; clear Data 
          feval(train_fct,P,Train_data.(sprintf('partition_%i',p)));
        elseif tasks.run_locally.train==0,
          fprintf('creating job (training) %i/%i  \r', q,sum(SETs.partition_train_on)*sum(num_models_tested)) ;
          RPROC.options.identifier = sprintf('Tr%i_%s%s',num_models, Signal_name, upper(organism)) ;

          num_data = count_data(Signal, fn_filter_settings, fn_examples, P.SETs.which_train, 'train', SETs.train_subsample_neg, SETs.train_subsample_pos);

          [mem_req, time_req, opts] = rproc_memtime_policy(train_fct,  num_data, RPROC.options) ;

          jobinfo(q) = rproc_create(train_fct, P, mem_req, opts, time_req) ;
        else
          feval(train_fct,P);
        end 
      	
      %--------------------------------------------------------------%
      %%%%% EVAL 
      %--------------------------------------------------------------%
      elseif fexist(P.fn_SVMs) 
        fprintf(fid,'\nSVM already trained on set ');
        fprintf(fid,'%i',P.SETs.which_train);
	fprintf(fid,'\n')
       
	if ~fexist(P.fn_eval)
          fprintf(fid,'waiting for file %s...\n', P.fn_eval);
	  pause(1)
        end
 
        if ~fexist(P.fn_eval) 
          P.eval_on = 1;
          fprintf(fid,'\n\nstart evaluation on set ');
          fprintf(fid,'%i',P.SETs.which_val);
          fprintf(fid,'\n')
          nq = nq+1;
          
          if isfield(method,'pred_fct')&& strcmp(method.pred_fct,'load_content_and_predict')
            pred_fct = 'load_content_and_predict';
          else
            pred_fct = 'load_and_predict';
          end
  
          if tasks.run_all
            Data = load_data(Signal,fn_filter_settings,fn_examples, P.SETs.which_val, 'test',SETs.test_subsample_neg,SETs.test_subsample_pos);
            Test_data.(sprintf('partition_%i',p)) = Data; clear Data;
            feval(pred_fct,P,Test_data.(sprintf('partition_%i',p)));
          elseif tasks.run_locally.eval==0,
            fprintf('creating job (evaluation) %i/%i  \r', nq,num_not_eval) ;
            RPROC.options.identifier = sprintf('Ev%i_%s%s',num_models,Signal_name,upper(organism)) ;

            num_data = count_data(Signal,fn_filter_settings,fn_examples, P.SETs.which_val, 'test', SETs.test_subsample_neg, SETs.test_subsample_pos);

            [mem_req, time_req, opts] = rproc_memtime_policy(pred_fct,  num_data, RPROC.options) ;

            jobinfo_1(nq) = rproc_create(pred_fct, P, mem_req, opts, time_req) ;
          else
            feval(pred_fct,P);
          end
        
        %--------------------------------------------------------------%
        %%%%% Collext Results 
        %--------------------------------------------------------------%
        elseif fexist(P.fn_eval) 
          fprintf(fid,'\n model already evaluated on set ');
          fprintf(fid,'%i',P.SETs.which_val);
          
          L = load(P.fn_eval, 'RSV', 'RFV');%,'fn_SVMs') ;
          num_not_eval = num_not_eval-1;
              
          assert(isnumeric(L.RFV) && prod(size(L.RFV))==1) ;
          assert(isnumeric(L.RSV) && prod(size(L.RSV))==1) ;
          RFV_eval(p,num_models)= L.RFV ;
          RSV_eval(p,num_models)= L.RSV ;

        else
          fprintf(fid,'\n only training, no evaluation ');
          num_not_eval = num_not_eval-1;
        end
      end   %% if trained eval
    end   %% loop over num_models      
    
    fprintf(fid,'partition %i: DONE\n', P.partition);
  end  %% loop over partitions
  fprintf(fid,'experiment %s: DONE\n',Signal_name );
      
  
  %--------------------------------------------------------------%
  %%%%% RPROC STUFF
  %--------------------------------------------------------------%

  %if (~isempty(jobinfo) || ~isempty(jobinfo_1)) && isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs,
  %  jobinfo = [jobinfo jobinfo_1] ;
  %  return ;
  %end ;

  if ~isempty(jobinfo),
    for ii=1:length(jobinfo), jobinfo(ii) = rproc_resubmit(jobinfo(ii)); end ;
  else
    num_crashed = 0 ;
  end
  if ~isempty(jobinfo_1),
    %[jobinfo_1, meta_jobinfo_1] = rproc_submit_batch(jobinfo_1, RPROC.exm_per_batch) ;
    for ii=1:length(jobinfo_1), jobinfo_1(ii) = rproc_resubmit(jobinfo_1(ii)); end ;
  else
    num_crashed_1 = 0;
  end ;
  if ~isempty(jobinfo),
    [jobinfo,num_crashed] = rproc_wait(jobinfo, 60, 1, -1);
    if num_crashed==length(jobinfo)
      fprintf('all jobs seem to be crashed. Waiting 2min.\n') ;
      pause(120) ;
      [jobinfo,num_crashed] = rproc_wait(jobinfo, 60, 1, -1);
      if num_crashed==length(jobinfo),
        error('all jobs crashed')
      end ;
    end
    if num_crashed == 0,
      rproc_cleanup(jobinfo) ;
    end ;
  end
  if ~isempty(jobinfo_1),
    [jobinfo_1,num_crashed_1] = rproc_wait(jobinfo_1, 60, 1, -1);
    if num_crashed_1==length(jobinfo_1)
      fprintf('all jobs seem to be crashed. Waiting 2min.\n') ;
      pause(120) ;
      [jobinfo,num_crashed] = rproc_wait(jobinfo, 60, 1,-1);
      if num_crashed==length(jobinfo),
        error('all jobs crashed') ;
      end ;
    end
    if num_crashed_1 == 0,
      rproc_cleanup(jobinfo_1) ;
    end ;
  end ;
  
  %--------------------------------------------------------------%
  %%%%% FINAL STUFF
  %--------------------------------------------------------------%
  
  try
    L=load(output_fn.fn_models, 'RSV_eval', 'RFV_eval') ;
  catch
    L=struct ;
  end ;
  if ~isfield(L, 'RSV_eval') || ~isfield(L, 'RFV_eval') || ~isequal(L.RSV_eval, RSV_eval) || ~isequal(L.RFV_eval, RFV_eval),
    %save(output_fn.fn_models,'-append','RSV_eval','RFV_eval')
    save_append(output_fn.fn_models, 1,'RSV_eval',RSV_eval, 'RFV_eval',RFV_eval)
    % save_append(output_fn.fn_models, 1,'RSV_eval',RSV_eval, 'RFV_eval',RFV_eval)
  end 
  if num_crashed+num_crashed_1>0
    rep = rep+1;
  end
end  %% loop over crashed jobs

%fclose(fid)




return

function [dummy1, dummy2]=Out2Confhelper(PAR)
% Out2Confhelper(PAR)
try
  pause('on')
  warning('off', 'MATLAB:typeaheadBufferOverflow');  
catch
  % works only for matlab
end

dummy1=[] ;
dummy2=[] ;

try
  load(PAR.fn_pred,'output') ;
catch
  pause(60)
  load(PAR.fn_pred,'output') ;
end
disp('Convert SVM Output to Confidence on fn_pred')
[Conf, Conf_cum] = Out2Conf(output, PAR.prob, PAR.prob_cum, PAR.limits);

if ~(all(Conf>=0-1e-8&Conf<=1+1e-8))
  save('~/tmp/ws_Out2Confhelper_error');
  error('Conf out of range [0, 1]');
end
if ~(all(Conf_cum>=0-1e-8&Conf_cum<=1+1e-8))
  save('~/tmp/ws_Out2Confhelper_error');
  error('Conf_cum out of range [0, 1]');
end

fprintf('append Conf and Conf_cum to file %s ', PAR.fn_pred)
%save(PAR.fn_pred,'-append','Conf','Conf_cum');
inventory = {'output', 'pos', 'Conf', 'Conf_cum'};
save_append(PAR.fn_pred, 1, 'inventory', inventory,'Conf', Conf, 'Conf_cum', Conf_cum);

return

function ret = predict_testplif_geterror(PAR)

  
SETs = PAR.SETs;
RPROC = PAR.RPROC;
FN = PAR.FN;

if ~isempty(PAR.Signal_name)
  Signal_name = PAR.Signal_name;
  Signal = PAR.Signals.(Signal_name);
  output_fn = FN.output_sig.(Signal_name);
  tasks = PAR.tasks.signals;
else
  Signal_name = PAR.Content_name;
  Signal = PAR.Contents.(Signal_name);
  output_fn = FN.output_cont.(Signal_name);
  tasks = PAR.tasks.contents;
end


fid = 1;%fopen(PAR.FN.fn_log ,'a+');
fprintf(fid,'\n\n starting test on left out set \n');
fprintf(fid,'date: %s\n',date);


%PAR = setfield(PAR,PAR.method,Model_param);


load(output_fn.fn_models,'RSV_eval','RFV_eval','MODELS','Model_param')
param_names = fieldnames(Model_param); 

fprintf(fid,'\n\n-----------\n');

RSV_test = zeros(SETs.num_partitions,1);
RFV_test = zeros(SETs.num_partitions,1);
best_models = zeros(SETs.num_partitions,size(MODELS,1));

options = RPROC.options;
options.identifier = [ Signal_name '_' Signal.method.name];

jobinfo = rproc_empty(0) ;
jobinfo_1 = rproc_empty(0) ;  


for p = 1:size(SETs.partitions,1) %% loop over partitions  
  fprintf(fid,'\n\n-----------\n');
  fprintf(fid,'PARTITION %i\n',p);
  fprintf(fid,'-----------\n');
  fprintf(fid,'\nSITE: %s\n',Signal_name);
  
  which_train = find(ismember(SETs.partitions(p,:),SETs.info.train));
  which_val = find(ismember(SETs.partitions(p,:),SETs.info.eval));
  which_pred = find(ismember(SETs.partitions(p,:),SETs.info.pred));
  which_plif = find(ismember(SETs.partitions(p,:),SETs.info.plif));

  fn_best_SVM = sprintf('%s_best_partition=%i.mat', output_fn.fn_SVMs,p) ;
  fn_SVM_train = [sprintf('%sms_set=[',output_fn.fn_SVMs),...
                  sprintf('%i',which_train)];
  fn_SVM_eval  = [fn_SVM_train '_' sprintf('%i',which_val)];
  fn_pred = sprintf('%spartition=%i.mat',output_fn.fn_SVMs,p);
  fn_plifs = sprintf('%spartition=%i_plifs.mat',output_fn.fn_SVMs,p);
  
  %--------------------------------------------------------------%
  %%% select for each partition best model:
  %--------------------------------------------------------------%
  if ~all(RFV_eval(p,:)~=0) 
    warning('some models not evaluated')
  end
  [tmp,idx] = max(RFV_eval(p,:));
  fn_SVM_eval = sprintf('%s]_model=%i.mat',fn_SVM_eval,idx);
  fn_SVM_train = sprintf('%s]_model=%i.mat',fn_SVM_train,idx)
  load(fn_SVM_train,'Train');
  assert(isequal(Train.PAR.SETs.which_train,which_train))
  best_models(p,:) = MODELS(:,idx) ;
  for ii = 1:length(param_names)
    assert(isequal(getfield(Train.PAR.Signal.method.par_ms,param_names{ii}), MODELS(ii,idx)))
  end
  
  fprintf(fid,'\nbest model (%s,partition=%i): ',Signal_name,p);
  fprintf('\nbest model (%s,partition=%i): \n',Signal_name,p);
  for num_param = 1:size(MODELS,1)
    fprintf(1,'%s = %d; \t',param_names{num_param},getfield(Train.PAR.Signal.method.par_ms,param_names{num_param}));
    fprintf(fid,'%s = %d; \t',param_names{num_param},getfield(Train.PAR.Signal.method.par_ms,param_names{num_param}));
  end
  L = load(fn_SVM_eval) ;
  if ~nan(L.RFV)
    assert(L.RFV == RFV_eval(p,idx)) ;
    assert(L.RSV == RSV_eval(p,idx)) ;
  end
  fprintf(fid,'\nvalidation error: RFV:%.4f \t RSV:%.4f\n',L.RFV,L.RSV);
  %fprintf('\nvalidation error: RFV:%.4f \t RSV:%.4f\n',L.RFV,L.RSV);
  
  %--------------------------------------------------------------%
  %%%%% Predict on TEST SET
  %--------------------------------------------------------------%  
  
  Train.PAR.partition = p;
  Train.PAR.SETs.which_val = which_val;
  Train.PAR.SETs.which_pred = which_pred ;
  Train.PAR.SETs.which_plif = which_plif ;
  Train.PAR.fn_plifs = fn_plifs;
  Train.PAR.fn_SVM_eval = fn_SVM_eval;
  Train.PAR.fn_pred = fn_pred;
  
  PAR_new = Train.PAR;
  PAR_new.eval_on = 0;
  PAR_new.Signal.filter_label.test = Signal.filter_label.test;

  if ~fexist(PAR_new.fn_pred),
    predict = 1;
  else
    pr=load(PAR_new.fn_pred);
    if ~isequal(PAR_new.Signal.method.par_ms, pr.Train.PAR.Signal.method.par_ms);
      predict=1;
      fprintf(1,'prediction file is out of date; it will be overwritten now\n')
    else
      predict=0;
      fprintf(1,'prediction file exists and was created with par_ms of best model\n')
    end
  end
  if predict
    if ~isfield(tasks, 'run_locally') || tasks.run_locally.predict==0,

      [mem_req, time_req, opts] = rproc_memtime_policy('load_and_predict',  0, options) ;%RPROC.MEMREQ ;

      jobinfo(p) = rproc('load_and_predict',PAR_new, mem_req, opts, time_req);
      % load_and_predict(PAR_new)
    else
      %D=load_data(PAR_new,which_pred,'test',PAR.SETs.test_subsample_neg,PAR.SETs.test_subsample_pos); 
      load_and_predict(PAR_new);%, D);
      %clear D
    end
  end ;
  %--------------------------------------------------------------%
  %%%%% Predict on PLIF SET
  %--------------------------------------------------------------%  
  
  PAR_new.eval_on = 0;
  PAR_new.plif_on = 1;
  
  
  
  if ~fexist(PAR_new.fn_plifs)
    predict = 1;
  else
    pr=load(PAR_new.fn_plifs);
    if ~isequal(PAR_new.Signal.method.par_ms, pr.Train.PAR.Signal.method.par_ms);
      predict=1;
      fprintf(1,'plif file is out of date; it will be overwritten now')
    else
      predict=0;
      fprintf(1,'plif file exists and was created with par_ms of best model')
    end
  end
  if predict
    if ~isfield(tasks, 'run_locally') || tasks.run_locally.predict==0,
      [mem_req, time_req, opts] = rproc_memtime_policy('load_and_predict',  0, options) ;

      jobinfo_1(p) = rproc('load_and_predict', PAR_new, mem_req, opts, time_req);
      % load_and_predict(PAR_new)
    else
      %D=load_data(PAR,which_plif,'test',PAR.SETs.test_subsample_neg,PAR.SETs.test_subsample_pos); 
      load_and_predict(PAR_new);%, D);
      %clear D
    end
  end ;
end  %% loop over partitions
if ~isempty(jobinfo),
  [jobinfo,num_crashed] = rproc_wait(jobinfo, 20, 1, -1) ;
end ;
if ~isempty(jobinfo),
  [jobinfo_1,num_crashed] = rproc_wait(jobinfo, 20, 1, -1) ;
end ;

if ~isempty(jobinfo),
  unix('sync')
  pause(30)
  unix('sync')
end ;

for p = 1:size(SETs.partitions,1) %% loop over partitions 
  fprintf(fid,'\n\n-----------\n');
  fprintf(fid,'PARTITION %i\n',p);
  fprintf(fid,'-----------\n');
  fprintf(fid,'\nSITE: %s\n',Signal_name);
    
  fn_best_SVM = sprintf('%sbest_partition=%i.mat', output_fn.fn_SVMs,p) ;
  fn_pred = sprintf('%spartition=%i.mat',output_fn.fn_SVMs,p);
  L = load(fn_pred,'RSV','RFV','svmFileName','PAR') ;
  load(L.svmFileName,'Train')
  Train.PAR = L.PAR;
  if ~fexist(fn_best_SVM)
    %save(fn_best_SVM,'-struct', 'L')
    %save(fn_best_SVM, '-append' ,  'Train')
    save_append(fn_best_SVM, 0, L)
    save_append(fn_best_SVM, 2, 'Train', Train)
  else
    old = load(fn_best_SVM,'Train')
    if isequal(old.Train,Train)
      fprintf('file %s already exists\n',fn_best_SVM),
      fprintf('Train identical\n')
    else
      fprintf('file %s already exists\n',fn_best_SVM),
      error('Train not identical\n')
    end
  end
  fprintf(fid,'\ntest error: RFV:%.4f \t RSV:%.4f\n',L.RFV,L.RSV);
  fprintf('\ntest error: RFV:%.4f \t RSV:%.4f\n',L.RFV,L.RSV);
  
  RFV_test(p)= L.RFV ;
  RSV_test(p)= L.RSV ;
  
  clear PAR_new
end  %% loop over partitions

L=load(output_fn.fn_models, 'best_models','Model_param','RSV_test','RFV_test') ;
if ~isfield(L, 'best_models') || ~isfield(L, 'Model_param') || ~isfield(L, 'RSV_test') || ~isfield(L, 'RFV_test') || ...
      ~isequal(L.best_models, best_models) || ~isequal(L.Model_param, Model_param) || ~isequal(L.RSV_test, RSV_test) || ~isequal(L.RFV_test, RFV_test)
  %save(output_fn.fn_models,'-append','best_models','Model_param','RSV_test','RFV_test')
  save_append(output_fn.fn_models, 1, 'best_models', best_models, ...
              'Model_param', Model_param, 'RSV_test', RSV_test, ...
              'RFV_test', RFV_test)
end ;

fprintf(fid,'\n%s\n',Signal_name);
fprintf(fid,'RSV (mean and std over %i partitions): %.4f +/- %.4f\n',length(RSV_test),mean(RSV_test),std(RSV_test));
fprintf(fid,'RFV (mean and std over %i partitions): %.4f +/- %.4f\n',length(RSV_test),mean(RFV_test),std(RFV_test));
ret = [];
%fclose(fid)





return

function [chr, strand, pos, score, signal_name, score_name, chr_name_dic, signal_name_dic, score_name_dic, success]=read_sigpred_bin(sigpred_fname)
% [chr, strand, pos, score, signal_name, score_name, chr_name_dic, signal_name_dic, score_name_dic, success]=read_sigpred_bin(sigpred_fname)

chr_name_dic = {} ;
score_name_dic = {} ;
signal_name_dic = {} ;

chr=[] ; 
signal_name = [] ;
score_name = [] ;
strand = '' ;
pos = [] ;
score = [] ;

if ~fexist(sigpred_fname),
  success=false ;
  return ;
end ;

cnt = 0; last=0;
while ~last,
  try
    SPF=load(sigpred_fname, sprintf('SPF_%i', cnt)) ;
    if ~isfield(SPF, sprintf('SPF_%i', cnt)),
      break
    end ;
  catch
    break ;
  end 
  if ~isempty(SPF)
    eval(sprintf('SPF=SPF.SPF_%i;', cnt)) ;
    cnt=cnt+1 ;
  %else
  %  SPF=load(sigpred_fname) ;
  %  last = 1 ;
  end ;

  if any(ismember(chr_name_dic, SPF.contig_name)),
    contig_idx = find(ismember(chr_name_dic, SPF.contig_name)) ;
  else
    chr_name_dic{end+1} = SPF.contig_name ;
    contig_idx = length(chr_name_dic) ;
  end ;

  if any(ismember(score_name_dic, SPF.score_name)),
    score_idx = find(ismember(score_name_dic, SPF.score_name)) ;
  else
    score_name_dic{end+1} = SPF.score_name ;
    score_idx = length(score_name_dic) ;
  end ;

  if any(ismember(signal_name_dic, SPF.signal_name)),
    signal_idx = find(ismember(signal_name_dic, SPF.signal_name)) ;
  else
    signal_name_dic{end+1} = SPF.signal_name ;
    signal_idx = length(signal_name_dic) ;
  end ;

  chr(end+1:end+length(SPF.pos))=contig_idx ;
  score_name(end+1:end+length(SPF.pos))=score_idx ;
  signal_name(end+1:end+length(SPF.pos))=signal_idx ;
  pos(end+1:end+length(SPF.pos))=SPF.pos ;
  score(end+1:end+length(SPF.pos))=SPF.score ;
  strand(end+1:end+length(SPF.pos))=SPF.strand ;
end ;

success = true ;

return

function [chr, strand, pos, score, signal_name, score_name, chr_name_dic, signal_name_dic, score_name_dic]=read_sigpred(sigpred_fname)
% [chr, strand, pos, score, signal_name, score_name]=read_sigpred(sigpred_fname)

if fexist([sigpred_fname '.gz']) && ~fexist(sigpred_fname),
  ret = unix(sprintf('gzip -d %s',sigpred_fname)) ;
  assert(ret==0) ;
end ;


chr_name_dic = {} ;
score_name_dic = {} ;
signal_name_dic = {} ;

chr=[] ; 
signal_name = [] ;
score_name = [] ;
strand = '' ;
pos = [] ;
score = [] ;

fd = fopen(sigpred_fname, 'r') ;

rest_text = '' ; part = 0 ; part_offset = 0 ;
line_idx = 0 ;
while ~feof(fd),
  part = part + 1 ;
  text=fread(fd, 100000000, 'uint8=>char') ;
  text=[rest_text text'] ;
  rest_text = '' ;
  if length(text)==0, 
    break ; 
  end ;
  last_idx = find(text==sprintf('\n'), 1, 'last') ;
  if last_idx~=length(text) && ~feof(fd),
    rest_text = text(last_idx+1:end) ;
    text(last_idx:end) = [] ;
  end ;
  text = [text sprintf('\n')] ;
  line_idx = find(text==sprintf('\n')) ;

  if length(line_idx)>2, 
    chr(length(chr)+1 : length(chr) + length(line_idx) -2 ) = nan ; 
    signal_name(length(signal_name)+1:length(signal_name) + length(line_idx)-2) = nan ;
    score_name(length(score_name)+1 : length(score_name) + length(line_idx)-2) = nan ;
    strand(length(strand)+1 : length(strand) + length(line_idx)-2) = ' ' ;
    pos(length(pos)+1 : length(pos) + length(line_idx)-2) = nan ;
    score(length(score)+1 : length(score) + length(line_idx)-2) = nan ;
  end ;

  chr_idx = 0 ;
  signal_name_idx = 0 ;
  score_name_idx = 0 ;
  progress = 0 ;
  for i=1:length(line_idx)-1,
    line=text(line_idx(i)+1:line_idx(i+1)-1) ;
    if isempty(line), 
      chr(i) = nan ;
      score(i) = nan ;
      score_name(i) = nan ;
      signal_name(i) = nan ;
      pos(i) = nan ;
      strand(i) = ' ' ;
      continue; 
    end ;
    if isequal(line(1),'#')
      continue
    end

    elem_idx = find(line==sprintf('\t')) ;
    
    if (i-1)/length(line_idx)>=progress,
      fprintf('\rPart %i: Read line %i/%i (%2.0f%%)     ', part, (i-1), length(line_idx), progress*100);
      progress = progress+0.05 ;
    end ;
    if chr_idx==0 || ~isequal(line(1:elem_idx(1)-1), chr_name_dic{chr_idx}),
      chr_idx = strmatch(line(1:elem_idx(1)-1), chr_name_dic, 'exact') ;
      if isempty(chr_idx),
        chr_name_dic{end+1} = line(1:elem_idx(1)-1) ;
        chr_idx = length(chr_name_dic) ;
      end ;
    end ;
    
    if signal_name_idx==0 || ~isequal(line(elem_idx(1)+1:elem_idx(2)-1), signal_name_dic{signal_name_idx}),
      signal_name_idx = strmatch(line(elem_idx(1)+1:elem_idx(2)-1), signal_name_dic, 'exact') ;
      if isempty(signal_name_idx),
        signal_name_dic{end+1} = line(elem_idx(1)+1:elem_idx(2)-1) ;
        signal_name_idx = length(signal_name_dic) ;
      end ;
    end ;
    
    if score_name_idx==0 || ~isequal(line(elem_idx(2)+1:elem_idx(3)-1), score_name_dic{score_name_idx}),
      score_name_idx = strmatch(line(elem_idx(2)+1:elem_idx(3)-1), score_name_dic, 'exact') ;
      if isempty(score_name_idx),
        score_name_dic{end+1} = line(elem_idx(2)+1:elem_idx(3)-1) ;
        score_name_idx = length(score_name_dic) ;
      end ;
    end ;
    
    chr(i+part_offset) = chr_idx ;
    signal_name(i+part_offset) = signal_name_idx ;
    score_name(i+part_offset) = score_name_idx ;
    strand(i+part_offset) = line(elem_idx(4)+1:elem_idx(5)-1) ;
    pos(i+part_offset) = str2double(line(elem_idx(3)+1:elem_idx(4)-1)) ;
    score(i+part_offset) = str2double(line(elem_idx(5)+1:end)) ;
  end

  part_offset = part_offset + length(line_idx)-1 ;
end ;
fclose(fd) ;

fprintf('\rPart %i: Read line %i/%i (%2.0f%%)              \n', part, ...
        length(line_idx), length(line_idx), 100);
signal_name(isnan(chr)) = [] ;
score_name(isnan(chr)) = [] ;
strand(isnan(chr)) = [] ;
pos(isnan(chr)) = [] ;
score(isnan(chr)) = [] ;
chr(isnan(chr)) = [] ;

assert(all(chr~=0)) ;


return

function [total_num_pos, num_pos] = retrieve_num_pos(regions,base_dir)
% num_pos = retrieve_num_pos(regions,base_dir)
%

num_pos = zeros(1,length(regions)) ;
for r=1:length(regions)
  f_name = sprintf('%scontig_%i%s',base_dir,regions(r).chr_num,regions(r).strand);
  d=dir([f_name '.pos']) ;
  if d(1).bytes==0,
    warning('empty pos file') ;
    num_pos(r) = 0 ;
  else
    num_pos(r) = get_num_pos(f_name, [regions(r).start; regions(r).stop]) ;
  end ;
end
total_num_pos = sum(num_pos) ;

return

function regions = retrieve_signals(regions,base_dir,signal_name,fields,which_pos,offset)

% regions = retrieve_signals(regions,base_dir,signal_name,fields,which_pos)
%
% loads all candidate positions and addional informations as specified in
% fields,(i.e.'label','anno_cover','est_cover','cdna_cover','Conf_cum' )
% from binary fiels and adds them to the field Signals.signal_name of the
% structure regions 
%
% see also save_label_files
if nargin<6
  offset = 0;
end
  
if nargin<5
  which_pos.local = 1;
  which_pos.contig = 1;
end

genome_info = init_genome(regions(1).config);

f_name = sprintf('%scontig_%i%s.pos', base_dir,regions(1).chr_num,regions(1).strand );
if ~fexist(f_name),
  fprintf('file %s does not exist\n', f_name);
end

if ~isempty(fields)
  f_idx = [];
  for f=1:length(fields),
    f_name = sprintf('%scontig_%i%s.%s', base_dir,regions(1).chr_num,regions(1).strand, fields{f});
    if fexist(f_name),
      f_idx = [f_idx f];
    else
      fprintf('file %s does not exist\n', f_name);
    end
  end
  if ~isempty(f_idx),
    fields = fields(f_idx);
  else
    error('no files available')
  end
end

if ~isfield(regions,'Signals')
  regions(1).Signals = [];
end


for r=1:length(regions)
  if isfield(regions(r),'num_fragments')&&~isempty(regions(r).num_fragments)
    num_fragments = regions(r).num_fragments;
  else
    num_fragments = 1;
  end
  prefix = 0;
  
  SIGNALS.pos = [];
  SIGNALS.contig_pos = [];
  for f=1:length(fields)
    SIGNALS.(fields{f})= [];
  end
  if ismember('pos2',fields)
    SIGNALS.contig_pos2 = [];
  end
  for p=1:num_fragments
    contig_length = dir(genome_info.flat_fnames{regions(r).chr_num(p)}) ;
    contig_length = contig_length.bytes;
    %if 1%r==5145
    %  fprintf(1, 'regions.stop: %i, regions.start: %ir: %i',regions(r).start,regions(r).stop, r)
    %end
    if regions(r).start(p)<1
      warning('regions.start<1; set to 1!')
      regions(r).start(p) = 1;
    end
    if regions(r).stop(p)>contig_length
      warning('regions.stop>contig length; set to contig_length!')
      regions(r).stop(p) = contig_length;
    end
    f_name = sprintf('%scontig_%i%s',base_dir, regions(r).chr_num(p), ...
                     regions(r).strand(p));
    if fexist([f_name '_sha1sum.mat'])
      [tmp,sha1sum] = unix(sprintf('sha1sum %s.pos',f_name));
      sha1sum = separate(sha1sum,' ');
      Old = load([f_name '_sha1sum'],'sha1sum');
      Old.sha1sum = separate(Old.sha1sum,' ');
      if ~isequal(Old.sha1sum{1},sha1sum{1})
        warning('sha1sum different')
      end
      %  else
      %    warning('no sha1sum check performed')
    end
    
    if isempty(fields)
      fields{1} ='pos';
    end
    if regions(r).start(p)+offset>regions(r).stop(p)-offset
      regions(r).Signals.(signal_name) = SIGNALS;
      continue
    end
    d=dir([f_name '.pos']) ;
    if d(1).bytes==0,
      contig_pos = [] ;
      score = zeros(0, length(fields)) ;
    else
      [contig_pos,score] = interval_query(f_name,fields,[regions(r).start(p)+offset;regions(r).stop(p)-offset]);
    end ;
    if ismember('pos2',fields)
      if ~isempty(contig_pos)
        idx_p = find(strcmp('pos2',fields));
        contig_pos2 = score(:,idx_p);
      else
        contig_pos2 =[];
      end
    end
    assert(isequal(contig_pos, sort(contig_pos))) ;
    %assert(issorted(contig_pos))
    if ~isempty(contig_pos)
      if ~(contig_pos(1)>=regions(r).start(p)&contig_pos(end)<=regions(r).stop(p))
        if keyboard_allowed()
          keyboard
        end;
      end
    end
    if which_pos.local
      if regions(r).strand(p)=='+'
        pos = contig_pos - regions(r).start(p) + 1+prefix;
        if exist('contig_pos2')
          pos2 = contig_pos2 - regions(r).start(p) + 1+prefix;
        end
        SIGNALS.pos = [SIGNALS.pos pos];
      else
        pos = regions(r).stop(p) - contig_pos + 1;
        [pos,idx] = sort(pos);
        score = score(idx,:);
        contig_pos = contig_pos(idx);
        if exist('contig_pos2')
          contig_pos2 = contig_pos2(idx);
          pos2 = regions(r).stop(p) - contig_pos2 + 1;
          SIGNALS.pos = [SIGNALS.pos pos2];
        else
          SIGNALS.pos = [SIGNALS.pos pos];
        end
      end
    end
    if which_pos.contig
      SIGNALS.contig_pos = [SIGNALS.contig_pos contig_pos];
      if exist('contig_pos2')
        SIGNALS.contig_pos2 = [SIGNALS.contig_pos2 contig_pos2];
        if regions(r).strand(p)=='+'
          SIGNALS.pos2 = [SIGNALS.pos2 pos2];
        else
          SIGNALS.pos2 = [SIGNALS.pos2 pos];
        end
      end
    end
    for f=1:length(fields)
      if strmatch(fields{f},'label') ix=f; end
      if ~isequal(fields{f},'pos')&~isequal(fields{f},'pos2')
        SIGNALS.(fields{f})= [SIGNALS.(fields{f}), score(:,f)];
      end
    end
    if exist('ix')
      u = unique(score(:,ix));
      if length(u)==1 assert(u==-1|u==1),
      elseif length(u)==2 assert(all(u==[-1;1])),
      elseif isempty(u),
      else error('unique(label) is not [-1;1]');
      end
    end
    regions(r).Signals.(signal_name) = SIGNALS;
    prefix = prefix +regions(r).stop(p)- regions(r).start(p);
  end %%loop over fragments
end %% loop over blocks

return

function save_examples(blocks,split,P)
%save_examples(blocks,split,P)
%
% INPUT 	regions with field Signals with field `Signal.name` with fields:
%			-pos
%			-anno_cover
%			-est_cover
%			-cdna_cover
%	PAR with fields
%			-organism
%			-Signal.filter_label
%			-Signal.name
%			-Signal.consensus
%			-FN.input_sig.fn_examples
%			-FN.input.fn_split
%			-FN.input.fn_regions


%only for RPROC
if nargin<3
  %addpath ..
  % use the rproc.addpath way to add paths in client processes!!
  %paths
  P = blocks.P;
  split = blocks.split;
  blocks = blocks.blocks;
end

Signal = P.Signal ; 
num_splits = P.num_splits;

fn_training_blocks = P.fn_training_blocks;
fn_candsites = P.fn_candsites;
fn_filter_settings = P.fn_filter_settings;
fn_examples = P.fn_examples;
fn_example_statistics = P.fn_example_statistics;

Signal_name = Signal.name;
lwin = Signal.lwin_big;
rwin = Signal.rwin_big;

fields = fieldnames(Signal.filter_label);
fprintf( 1, 'save_examples... (%d fields, %d splits)\n', length(fields), num_splits );
for ff = 1:length(fields)
   for ss=1:num_splits
    %---------------------------------------------
    %CREATE OUTPUT FILENAME
    %----------------------------------------------  
    filename = gen_example_filename(Signal,fn_filter_settings,fn_examples,ff,ss);
    if ~fexist(filename)
      num_examples = 0;
      POS.fn_training_blocks = fn_training_blocks;	 
      split_reg_ids = getfield(split,sprintf('split_%i',ss));
      blocks_idx = find(ismember([blocks.id],split_reg_ids));
      split_regs = blocks(blocks_idx);
      clear blocks_idx split_reg_ids
      
      %if ~isfield(split_regs,'seq') 
      %  fprintf(1,'loading sequence...\n')
      %  split_regs = load_sequence(split_regs);
      %  fprintf(1,'\n')
      %end
      
      %---------------------------------------------
      % retreive label information
      %---------------------------------------------
      
      % if ~isfield(split_regs, 'Signals')
      %   temp_region = retrieve_signals(split_regs(1),fn_candsites,Signal_name,{'label',Sig.Conf_names{:}});
      % end
      
      
      %---------------------------------------------
      %----heuristic--->estimate size of chararray; 
      % -some positions are filtered out by the filerfunction: Signal.filter_fct
      % -if the size is to small then this function is very time-consuming
      % -otherwise it is very space-consuming
      %---------------------------------------------
      %total_num_pos = retrieve_num_pos(split_regs, fn_candsites) ;
      %num_pos = ceil(total_num_pos) ;
     
      %--------------------------------------------------
      % determine the maximal number of training examples
      % for the given sensor
      %--------------------------------------------------
      t = cputime;
      num_pos_ex = 0;
      num_neg_ex = 0;
      for r=1:length(split_regs)
        if ~isfield(split_regs, 'Signals')
          temp_region = retrieve_signals(split_regs(r),fn_candsites,Signal_name,{'label'});
	  num_pos_ex = num_pos_ex+sum(temp_region.Signals.(Signal_name).label == 1);
	  num_neg_ex = num_neg_ex+sum(temp_region.Signals.(Signal_name).label == -1);
        end
      end
      fprintf(1,'it took %fs to determine the true number of positive and negative examples\n', cputime-t);
      vec=subsample_policy([num_pos_ex num_neg_ex], Signal_name);
      max_num_pos = vec(1);
      max_num_neg = vec(2);
      reduction_factor_pos = max_num_pos/num_pos_ex;
      reduction_factor_neg = max_num_neg/num_neg_ex;
      num_pos = max_num_pos+max_num_neg;

      XT=zeros([lwin+rwin,num_pos],'int8');
      LT=zeros(1,num_pos);
      POS.pos=zeros(1,num_pos);
      POS.region_id=zeros(1,num_pos);

      if isfield(Signal.filter_label.(fields{ff}), 'USE_ALL')&&...
            Signal.filter_label.(fields{ff}).USE_ALL
        Signal.Conf_names = {} ;
      end ;
      
      t = cputime;
      fprintf(1,'starting with split %i\n',ss)
      for r=1:length(split_regs)
        if mod(r,100)==0,
          fprintf('region %i/%i\r', r, length(split_regs)) ;
        end ;
        
        if ~isfield(split_regs, 'Signals')
          temp_region = retrieve_signals(split_regs(r),fn_candsites,Signal_name,{'label',Signal.Conf_names{:}});
        end
        
        sig_field = getfield(temp_region.Signals,Signal_name);
        if isfield(Signal.filter_label.(fields{ff}),'use_label')
          sig_field.label = sig_field.(Signal.filter_label.(fields{ff}).use_label);
        end
        
        if ~isfield(split_regs,'seq') 		
          if isfield(Signal.filter_label.(fields{ff}), 'USE_ALL')&&...
                Signal.filter_label.(fields{ff}).USE_ALL
            if temp_region.strand == '+',
              loffset=0 ; % remember the offset for later
              tmp = max(1,temp_region.start-lwin) ;
              loffset = temp_region.start - tmp ;
              temp_region.start = tmp ;
              temp_region.stop = temp_region.stop + rwin ;
            else
              tmp = temp_region.start-rwin ;
              temp_region.start = tmp ;
              temp_region.stop = temp_region.stop + lwin ;
              loffset = lwin ;
            end ;
          end ;
          temp_region = load_sequence(temp_region);
        end
        seq = temp_region.seq;
        
        %---------------------
        %FILTER EXAMPLES
        %---------------------
        if ~isfield(Signal.filter_label.(fields{ff}), 'USE_ALL')||...
              ~Signal.filter_label.(fields{ff}).USE_ALL
          if ~isfield(Signal.filter_label.(fields{ff}),'conf_pos') ...
                && isfield(Signal.filter_label.(fields{ff}),'conf')
            error('fix this: filtering will not work')
          end
          idx = feval(Signal.filter_fct, temp_region, Signal, fields{ff},1);
        else
          sig_field.pos = sig_field.pos + loffset ;
          idx = find(sig_field.pos>lwin&sig_field.pos<length(seq)-rwin+1);
	  pos_idx = find(sig_field.label==1);
          if reduction_factor_pos<1
	    keep = randperm(length(pos_idx));
	    pos_idx = pos_idx(keep(1:floor(length(pos_idx)*reduction_factor_pos)));
	  end
	  neg_idx = find(sig_field.label==-1);
	  if reduction_factor_neg<1
	    keep = randperm(length(neg_idx));
	    neg_idx = neg_idx(keep(1:floor(length(neg_idx)*reduction_factor_neg)));
	  end
	  if reduction_factor_pos<1||reduction_factor_neg<1
            if ~isempty(pos_idx) && ~isempty(neg_idx),
              un=[pos_idx', neg_idx'] ;
            elseif isempty(pos_idx),
              un = neg_idx' ;
            else
              un = pos_idx' ;
            end ;
	    idx = intersect(idx, un);
	  end
        end
        if isempty(idx); continue; end
        
        %--------------------
        %CREATE XTs
        %--------------------
        XT_temp = zeros(lwin+rwin,length(idx),'int8');
        rid_temp= zeros(1,length(idx))+temp_region.id;
        for row=1:lwin+rwin
          XT_temp(row,:)= seq(sig_field.pos(idx)-lwin+row-1);
        end%loop over rows
        LT_temp= sig_field.label(idx)';
        pos_temp = sig_field.pos(idx)';
        assert(size(LT,2)==num_examples||LT(num_examples+1)==0)
        assert(num_examples==0||LT(num_examples)~=0)
        
        %[size(XT,2) num_examples+length(idx) ceil(size(XT,2)*r/length(split_regs))]
        if size(XT,2)<num_examples+length(idx), 
          warning('extending XT') ;
          POS.pos(ceil(size(XT,2)*1.2)) = 0 ;
          POS.region_id(ceil(size(XT,2)*1.2)) = 0 ;
          LT(ceil(size(XT,2)*1.2)) = 0 ;
          XT(1,ceil(size(XT,2)*1.2)) = 0 ;
        end ;
        
        XT(:, num_examples+1:num_examples+length(idx))= XT_temp;	
        LT(num_examples+1:num_examples+length(idx))= LT_temp;
        POS.pos(num_examples+1:num_examples+length(idx))= pos_temp;
        POS.region_id(num_examples+1:num_examples+length(idx))=rid_temp;
        num_examples = num_examples+length(idx);
      end%loop over blocks
      fprintf(1, 'time required for one split: %.2f\n',cputime-t)
      fprintf(1,'save file %s\n',filename)
      %=== remove zeros
      assert(all(LT(num_examples+1:end)==0))
      LT = LT(1:num_examples);
      XT = XT(:,1:num_examples);
      POS.pos = POS.pos(1:num_examples);
      POS.region_id = POS.region_id(1:num_examples);
      
      %----------------------------------------------- 
      % remove examples that appear more than once
      %----------------------------------------------- 
      if 0
        [tmp1 idx tmp2] = unique(XT','rows');
        idx = idx(randperm(length(idx)));
        XT = XT(:,idx);
        LT = LT(idx);
        POS.pos = POS.pos(idx);
        POS.region_id = POS.region_id(idx);
      end
      %-----------------------------------------------
      % exclude examples containing non ACGT-Symbols 
      %-----------------------------------------------
      %xtpos=XT~='A'&XT~='C'&XT~='G'&XT~='T';
      %save space
      xtpos=XT~='A';
      for s='CGT'
        xtpos=xtpos&XT~=s;
      end
      cols = find(sum(xtpos));
      clear xtpos
      if length(cols)>0
        warning(sprintf('%i examples containing non ACGT symbols excluded',length(cols)))
        XT(:,cols)=[];
        LT(cols)=[];
        POS.pos(cols)=[];
        POS.region_id(cols)=[];
      end
      %----------------------------------------------- 
      % save matrix in pieces into one file
      %----------------------------------------------- 
      assert(sum(LT==1)>0);
      fprintf( 'trying to save %s\n', filename );
      save_matrix(filename, XT, 'XT');
      %save(filename,'-append','LT','POS','PAR');
      save_append(filename, 1,'LT', LT, 'POS', POS, 'P', P);
      % save_append(filename, 1,'LT', LT, 'POS', POS, 'PAR', PAR);
    end%if ~fexist(filename)
   end%loop over splits
   
   stat = save_example_statistics(Signal, fn_filter_settings, ...
                                  fn_examples, fn_example_statistics, ...
                                  num_splits);
end%loop over filter_label fields


return

function stat=save_example_statistics(Signal,fn_filter_settings,fn_examples,fn_example_statistics,num_splits)
% stat = save_example_statistics(Signal,fn_filter_settings,fn_examples,fn_example_statistics,num_splits)


pos = 0;
neg = 0;
for j=1:num_splits
  f = gen_example_filename(Signal,fn_filter_settings,fn_examples,1,j) ;
  l=load(f, 'LT');
  pos = pos + sum(l.LT==1);
  neg = neg + sum(l.LT==-1);
  stat.(sprintf('split_%i_num_pos',j))=sum(l.LT==1);
  stat.(sprintf('split_%i_num_neg',j))=sum(l.LT==-1);
  stat.(sprintf('split_%i_frac_pos',j))=sum(l.LT==1)/sum(l.LT==-1)*100;
end

stat.all_num_pos = pos;
stat.all_num_neg = neg;
stat.all_frac_pos=pos/neg*100;

assert(pos>0);
assert(neg>0);


fn = fn_example_statistics(1:end-4);
num = 1;
while fexist(sprintf('%s_%i.mat',fn,num))
  num=num+1;
end

save(sprintf('%s_%i.mat',fn,num), 'stat');

return

function save_predictions_as_sigpred_bin(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand)
% save_predictions_as_sigpred_bin(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand)

%assert(issorted(pos))
assert(isequal(pos, sort(pos)))
assert(length(pos)==length(score))

fn_out=[out_fn_prefix '_' score_name '_spf.mat'];

if fexist(fn_out) 
  fprintf(1, 'file exists, it will NOT be overwritten\n')
  return
end
fprintf(1,'save in binary signal prediction format (spf): %s\n', fn_out);
save(fn_out, '-V7', 'contig_name', 'signal_name', 'score_name', 'pos', 'strand', 'score') ;

return

function save_predictions_as_sigpred(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)
% save_predictions_as_wiggle(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)

%assert(issorted(pos))
assert(isequal(pos, sort(pos)))
assert(length(pos)==length(score))

if ~exist('do_gzip', 'var'),
  do_gzip = 1 ;
end ;

fn_out=[out_fn_prefix '_' score_name '.spf'];

if fexist(fn_out) | fexist([fn_out '.gz'])
  fprintf(1, 'file exists, it will NOT be overwritten\n')
  return
end
fprintf(1,'save in signal prediction format (spf): %s\n', fn_out);
[fd msg] = fopen(fn_out, 'w+');

fprintf(fd, '##Seqid\tType\tSource\tStart\tStrand\tScore\n') ;
for j=1:length(pos)
  fprintf(fd, '%s\t%s\t%s\t%i\t%c\t%.3f\n', contig_name, signal_name, score_name, pos(j), strand, score(j)) ;
end

fclose(fd);

if do_gzip,
  ret = unix(sprintf('gzip %s',fn_out)) ;
  if ret~=0
    warning('gzip failed') ;
  end
end 


return

function fn_out = save_predictions_as_wiggle(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)
% fn_out = save_predictions_as_wiggle(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)

%assert(issorted(pos))
assert(isequal(pos, sort(pos)))
assert(length(pos)==length(score))

if ~exist('do_gzip', 'var'),
  do_gzip = 1 ;
end ;

fn_out=[out_fn_prefix '_' score_name '.wiggle'];

if fexist(fn_out)|fexist([fn_out '.gz'])
  fprintf(1, 'file exists, it will NOT be overwritten\n')
%  keyboard
  return
end

fprintf(1,'save as wiggle: %s\n', fn_out);
[fd msg] = fopen(fn_out, 'w+');
disp(msg)

fprintf(fd,'browser position chr%s:1-50,000\n', contig_name);
fprintf(fd,'browser hide all\n');
fprintf(fd,'browser pack Genes\n');
fprintf(fd,'browser full altGraph\n');
fprintf(fd,'browser viewLimits=%1.2f:%1.2f\n', min(score), max(score));


fprintf(fd,['track type=wiggle_0 name="%s_%s%s" description="%s chr%s %s" ' ...
            '\n'],signal_name, score_name, strand, score_name, ...
        contig_name, strand);
fprintf(fd,'variableStep chrom=chr%s\n',contig_name);

for j=1:length(pos)
  fprintf(fd,'%i\t%.3f\n', pos(j), score(j));
end

fclose(fd);

if do_gzip,
  ret = unix(sprintf('gzip %s',fn_out));
  if ret~=0
    warning('gzip failed')
  end
end ;

return

function [dummy1, dummy2]=save_predictions_helper(P)
% save_predictions_helper(P) 
try
  pause('on')
  warning('off', 'MATLAB:typeaheadBufferOverflow');  
catch
  % works only for matlab
end


dummy1=[] ;
dummy2=[] ;

fn_genome_config = P.fn_genome_config;
chrom = P.chrom;
strand = P.strand;
fn_pred = P.fn_pred;
Signal_name = P.Signal_name;
resolution = P.resolution;
conf_cum_thresh = P.conf_cum_thresh;
do_gzip = P.do_gzip ;
save_as = P.save_as ;

genome_info = init_genome(fn_genome_config); 
S = dir(genome_info.flat_fnames{chrom}) ;
contig_length = S.bytes;
CHR_NAME = genome_info.contig_names{chrom};
fprintf('contig: %i%s\n', chrom,strand);

%----------------------------
%% GET ALL PREDICTIONS from mat file
%----------------------------

filename = sprintf('%scontig_%i%s_all.mat', fn_pred, chrom, strand) ;
for kk=1:30 % it can take easily several minutes until the file is visible
   try 
     clear pos output Conf Conf_cum
     load(filename, 'inventory');
     for j=1:length(inventory)
        if kk>1,
           fprintf('trying to load variable "%s" from %s (trial %i)\n', inventory{j}, filename, kk);
        end ;
        load(filename,inventory{j})
        assert(exist(inventory{j}, 'var')==1);	 
     end
     break ;
   catch
     if kk==30      
       fprintf('giving up waiting for file %s\n', filename)
     else
       fprintf('waiting for file: %s \n', filename);    
       pause(30)
     end	
  end
end


% always save global coordinates on forward strand
if isequal(strand, '-'),
  pos = contig_length - pos + 1;
end
assert(all(pos>=1));

[pos sort_idx] = sort(pos);
output = output(sort_idx);
if exist('Conf') && exist('Conf_cum')
  Conf = Conf(sort_idx);
  Conf_cum = Conf_cum(sort_idx);
  
  %----------------------------
  %% downsample if demanded
  %----------------------------
  
  assert(all(Conf>=0-1e-8&Conf<=1+1e-8));
  Conf(Conf<0)=0 ;
  Conf(Conf>1)=1 ;
end

side = floor(resolution/2) ;
if (resolution~=1), 
  fprintf('Reducing resolution to one prediction per %i nucleotides\n', resolution) ;
  idx = 1:resolution:length(pos) ;
  cc = zeros(1,length(idx)) ;
  co = zeros(1,length(idx)) ;
  oo = zeros(1,length(idx)) ;
  pp = zeros(1,length(idx)) ;
  
  for j=1:length(idx),
    %if mod(j,1000)==0, fprintf(' %i  \r', j), end ;
    win = max(1,idx(j)-side):min(idx(j)+side,length(pos)) ;
    neighbour = find((pos(win)>=pos(idx(j))-side) & (pos(win)<pos(idx(j))+side)) ;
    %pos(win(neighbour))
    if exist('Conf') && exist('Conf_cum')
      [mv,mi] = max(Conf_cum(win(neighbour))) ;
      assert(Conf_cum(win(neighbour(mi)))==mv) ;
      cc(j) = Conf_cum(win(neighbour(mi))) ;
      co(j) = Conf(win(neighbour(mi))) ;
    else
      [mv,mi] = max(output(win(neighbour))) ;
    end
    pp(j) = pos(win(neighbour(mi))) ;
    oo(j) = output(win(neighbour(mi))) ;
  end;
  pos = pp ;
  Conf_cum = cc ;
  Conf = co ;
  output = oo ;
end ;

%----------------------------
%%check if files exist
%----------------------------
filename = sprintf('%scontig_%i%s', fn_pred, chrom, strand);
if fexist([filename '.pos']),
  fprintf('removing position file (%s). \n', filename);
  unix(sprintf('rm %s', [filename '.pos']));
end
if fexist([filename '.output']),
  fprintf('removing output file (%s)\n', filename);
  unix(sprintf('rm %s', [filename '.output']));
end
if fexist([filename '.Conf']),
  fprintf('removing Conf file (%s)\n', filename);
  unix(sprintf('rm %s', [filename '.Conf']));
end
if fexist([filename '.Conf_cum']),
  fprintf('removing Conf_cum file (%s)\n', filename);
  unix(sprintf('rm %s', [filename '.Conf_cum']));
end

%Conf=round(Conf*1000)/1000;
%output = round(output*1000)/1000;
%Conf_cum=round(Conf_cum*1000)/1000; 
if exist('Conf') && exist('Conf_cum') %&& ~all(Conf_cum==0)&&~all(Conf==0)
  save_score_pos(pos, [output; Conf; Conf_cum], filename, {'output', 'Conf', 'Conf_cum'});
else
  save_score_pos(pos, [output], filename, {'output'});
end

if isequal(Signal_name, 'transacc'),
  disp('Using acceptor predictions in combination with transacc predictions (for wiggle files only)') ;
  FN2.fn_pred = strrep(fn_pred,'transacc', 'acc') ;
  
  filename2 = sprintf('%scontig_%i%s_all.mat', FN2.fn_pred,chrom,strand);
  L=load(filename2, 'pos', 'Conf', 'Conf_cum', 'output');
  % always save global coordinates on forward strand
  if isequal(strand, '-'),
    L.pos = contig_length - L.pos + 1;
  end
  assert(all(pos>=1));
  if ~isequal(sort(L.pos),pos)
    warning('position lists not identical') ;
    setdiff(L.pos, pos)
    setdiff(pos, L.pos)
    
    [tmp,idx1,idx2]=intersect(pos,L.pos) ;
    
    L.Conf = L.Conf(idx2) ;
    L.Conf_cum = L.Conf_cum(idx2) ;
    L.output = L.output(idx2) ;
    L.pos = L.pos(idx2) ;
    
    Conf = Conf(idx1) ;
    Conf_cum = Conf_cum(idx1) ;
    output = output(idx1) ;
    pos = pos(idx1) ;
  else
    [L.pos sort_idx] = sort(L.pos);
    L.Conf = L.Conf(sort_idx);
    L.Conf_cum = L.Conf_cum(sort_idx);
    L.output = L.output(sort_idx);
  end ;
  
  Conf_cum = Conf_cum .* L.Conf_cum ;
  Conf = Conf .* L.Conf ;
  output = output + L.output ;
  clear L ;
end ;

%----------------------------
%% SAVE PREDICTIONS as SPF files
%---------------------------- 

% saving in signal prediction format (without filtering)
if save_as.spf_ascii,
  save_predictions_as_sigpred(filename, pos, output, 'output', Signal_name, CHR_NAME, strand, do_gzip);
end ;
if save_as.spf_binary,
  save_predictions_as_sigpred_bin(filename, pos, output, 'output', Signal_name, CHR_NAME, strand);
end ;

if exist('Conf') %&& ~all(Conf==0)
  if save_as.spf_ascii,
    save_predictions_as_sigpred(filename, pos, Conf, 'Conf', Signal_name, CHR_NAME, strand, do_gzip);
  end ;
  if save_as.spf_binary,
    save_predictions_as_sigpred_bin(filename, pos, Conf, 'Conf', Signal_name, CHR_NAME, strand);
  end ;
end ;

if exist('Conf_cum') %&&  ~all(Conf_cum==0)
  if save_as.spf_ascii,
    save_predictions_as_sigpred(filename, pos, Conf_cum, 'Conf_cum', Signal_name, CHR_NAME, strand, do_gzip);
  end ;
  if save_as.spf_binary,
    save_predictions_as_sigpred_bin(filename, pos, Conf_cum, 'Conf_cum', Signal_name, CHR_NAME, strand);
  end
end ;

%----------------------------
%% SAVE PREDICTIONS as wiggle files
%---------------------------- 

% rounding for wiggle format output
output = round(output*1000)/1000;

%----------------------------
% REMOVE PREDICTIOND BELOW THRESHOLD
%----------------------------


if exist('Conf') && exist('Conf_cum') && save_as.wiggle

  fprintf('Removing all predictions below %.2f before saving wiggle file',conf_cum_thresh) 
  idx = find(Conf > conf_cum_thresh);
  output   = output(idx);
  pos = pos(idx);
  Conf = round(Conf*1000)/1000;
  Conf_cum = round(Conf_cum*1000)/1000; 
  Conf     = Conf(idx); 
  Conf_cum = Conf_cum(idx);
  if save_as.wiggle
    save_predictions_as_wiggle(filename, pos, Conf, 'Conf', Signal_name, CHR_NAME, strand, do_gzip);
    save_predictions_as_wiggle(filename, pos, Conf_cum, 'Conf_cum', Signal_name, CHR_NAME, strand, do_gzip);
  end ;

end

if save_as.wiggle
  save_predictions_as_wiggle(filename, pos, output, 'output', Signal_name, CHR_NAME, strand, do_gzip);
end

return

function [jobinfo] = save_predictions(fn_genome_config, fn_pred, ...
                                      Signal_name, resolution, conf_cum_thresh, strands, run_locally, RPROC, do_gzip, save_as, inventory)
%  [jobinfo] =
%  save_predictions(fn_genome_config,fn_pred,strands,run_locally,RPROC,do_gzip,save_as,inventory)
%
%  Loads predictions corresponding to each contig for a signal
%  defined in PAR.Signal_name.
%  Saves them to binary files which can be read by 
%  interval_query.
%

%
%  see retrieve_signals, interval_query 
if nargin<2
  strands='+-';
end

if ~exist('do_gzip', 'var'),
  do_gzip = 1 ;
end ;
if ~exist('save_as', 'var'),
  save_as.spf_ascii = 1 ;
  save_as.spf_binary = 1 ;
  save_as.wiggle = 0 ;
end ;

jobinfo = rproc_empty(0) ;
% options = RPROC.options;
% options.ncpus = 1;
% options.identifier = sprintf('%scheck%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;
% MEMREQ = RPROC.MEMREQ;

fid=1;q=0;

genome_info = init_genome(fn_genome_config);

P.fn_genome_config = fn_genome_config;
P.Signal_name = Signal_name;
P.fn_pred = fn_pred;
P.conf_cum_thresh = conf_cum_thresh;
P.resolution = resolution ;
P.save_as = save_as ;


% options.identifier = sprintf('%ssave%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;
for c = 1:length(genome_info.contig_names)
  for s = strands
    P.chrom = c;
    P.strand = s;
    P.do_gzip = do_gzip ;
    filename = sprintf('%scontig_%i%s', fn_pred, c, s);

    one_missing=0 ;
    for k=1:length(inventory)
      if ~fexist([filename '.' inventory{k}]) || ~fexist([filename '_' inventory{k} '.spf']) || ~fexist([filename '_' inventory{k} '_spf.mat'])
        one_missing = 1 ;
      end ;
    end ;
    if one_missing,
      if ~run_locally
        q=q+1;

        [mem_req, time_req, opts] = rproc_memtime_policy('save_predictions_helper',  0, RPROC.options) ;%RPROC.MEMREQ ;

        if isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs 
          jobinfo(q) = rproc_create('save_predictions_helper', P, mem_req, opts, time_req) ;
        else
          jobinfo(q) = rproc('save_predictions_helper', P, mem_req, opts, time_req);
        end
      else
        save_predictions_helper(P)
      end
    end
  end %%% loop over strand
end %%% loop over contigs 


rproc_wait(jobinfo, 20, 1, -1);



return

function flag = segment_overlap(start,stop,starts,stops)
% flag = segment_overlap(start,stop,starts,stops)  

flag = ((start >= starts) & (start<=stops)) | ...
       ((stop >= starts) & (stop<=stops)) | ...
       ((start <=starts) & (stop>=stops)) | ...
       ((start >=starts) & (stop<=stops)) ;
  
return

function CONF = set_CONF(Signal,ALL,sig,info_names,signal_name)
% CONF = set_CONF(Signal,ALL,sig,info_names)

%%% GENERAL 
idx = find(strcmp(Signal.Conf_names,'altgenic'));
if ~isempty(idx)
  CONF(idx,:) = ALL.ALTGENIC;
end
idx = find(strcmp(Signal.Conf_names,'gene_id'));
if ~isempty(idx)
  CONF(idx,:) = ALL.GENE_ID;
end

for i=1:length(info_names)
  idx = find(strcmp(Signal.Conf_names,[info_names{i} '_conf']));
  CONF(idx,:) = ALL.COV1(i,:);
  if ~isequal(sig,'other')&~isequal(Signal.name,'transacc')
    idx = find(strcmp(Signal.Conf_names,[info_names{i} '_conf_skip']));
    CONF(idx,:) = ALL.COV2(i,:);
  end
end

%%% ALT SPLICE
idx = find(strcmp(Signal.Conf_names,'alt_exon'));
if ~isempty(idx)
  CONF(idx,:) = ALL.LABEL_2(1,:);
end
idx = find(strcmp(Signal.Conf_names,'alt_intron'));
if ~isempty(idx)
  CONF(idx,:) = ALL.LABEL_2(2,:);
end
idx = find(strcmp(Signal.Conf_names,'alt_3prime'));
if ~isempty(idx)
  CONF(idx,:) = ALL.LABEL_2(3,:);
end
idx = find(strcmp(Signal.Conf_names,'alt_5prime'));
if ~isempty(idx)
  CONF(idx,:) = ALL.LABEL_2(3,:);
end

idx = find(strcmp(Signal.Conf_names,['is' sig]));
if ~isempty(idx)
  CONF(idx,:) = ALL.ISSPLICE;
end

%%% SPECIAL THINGS
idx = find(strcmp(Signal.Conf_names,['has_' signal_name]));
if ~isempty(idx)
  CONF(idx,:) = ALL.HAS_SIG;
end
idx = find(strcmp(Signal.Conf_names,'has_trans'));
if ~isempty(idx)
  CONF(idx,:) = ALL.TRANS;
end


idx = find(strcmp(Signal.Conf_names,'dist'));
if ~isempty(idx)
  CONF(idx,:) = ALL.DIST;
end

if exist('CONF', 'var') && ~(size(CONF,1)==length(Signal.Conf_names))
  warning('this was an assert') ;
end ;

return

function setFeatures(method, Data, svmMode, pred_from_seq, use_preproc);
% setFeatures(method,Data, svmMode,pred_from_seq,use_preproc);


% method = PAR.method;
% tasks = PAR.tasks;
% pred_from_seq = tasks.pred_from_seq;

  
%------------------------------------------------------------------------------%
% check
%------------------------------------------------------------------------------%

assert( strcmp(svmMode,'TRAIN') || strcmp(svmMode,'TEST') );

%assert( isscalar(Data.pos0) );
%assert( size(XT,2) > 0 );
%assert( ischar( XT ) );

if nargin<5
  use_preproc=1;
end
if ~isfield(method,'conservWdsWeighting')
  conservWdsWeighting=0;
else
  conservWdsWeighting= method.conservWdsWeighting;
end
%------------------------------------------------------------------------------%
% pass data
%------------------------------------------------------------------------------%

%sg('loglevel', 'ALL');
if pred_from_seq
  seq = Data.seq;
  idx = find(seq~='A'&seq~='C'&seq~='G'&seq~='T');
  if ~isempty(idx)
    warning('non AGCT-Symbols found; changed to As')
    seq(idx)='A';
  end
  pos = Data.pos;
else
  XT = Data.XT;
  %pos0=Data.pos0;
  if isfield(Data, 'LT')
    LT = Data.LT;
  end
  if isfield(Data, 'W')
    W = Data.W;
  end
end

%==============================================================================%
% set labels
%==============================================================================%
if strcmp(svmMode,'TRAIN')
  sg( 'set_labels', 'TRAIN', LT ); 
end
%==============================================================================%
% set features
%==============================================================================%



nofKernels = length(method.kernels);
if nofKernels>1 && ~isfield(method.kernels{2},'name') && ~isfield(method.kernels{2}, 'kernel_name')
  % for content sensors; several orders are trained simulatuously.
  % but this is done by concatination of the feature vector directly.
  nofKernels = 1;
end
assert( nofKernels >= 1 );
if( nofKernels == 1 )
  addFeats = 'set_features';
else
  addFeats = 'add_features';
end;


for k = 1:nofKernels
  kernel = method.kernels{k};
  % for compatibility with older versions
  if ~isfield(kernel, 'kernel_name') && isfield(kernel,'name')
    kernel.kernel_name = kernel.name;
  end
  if ~pred_from_seq   
    if ~isfield(method,'center') || isempty(method.center)
      select= [1:size(XT,1)];
    else
      if ~isfield(kernel,'lwin') 
        warning('no lwin specified for kernel')
        kernel.lwin = -method.center;
      end
      if ~isfield(kernel,'rwin') 
        warning('no rwin specified for kernel')
        kernel.rwin = -method.center+size(XT,1)-1;
      end
      select = (kernel.lwin:kernel.rwin)+method.center+1;
    end
    if ~isequal(select,[1:size(XT,1)])
      XT_temp = XT(select,:);
    else
      XT_temp = XT;
    end
    %keyboard;
  else
    %positiones should be a 0-based index
    pos_temp=pos+kernel.lwin-1;
    winsize_temp=kernel.rwin-kernel.lwin+1;
    assert(length(pos_temp)>0)
    assert(length(seq)>max(pos_temp)+winsize_temp)
    assert(winsize_temp>0)

    % work-around for from_position_list bug
    if length(pos_temp)==1,
      pos_temp = [pos_temp pos_temp] ;
    end ;
  end
  %------------------------------------------------------------------------------%
  % kernel: WD or WDS
  %------------------------------------------------------------------------------%

  if strcmp(kernel.kernel_name,'WD') || strcmp(kernel.kernel_name,'WDS')
    if ~pred_from_seq
      fprintf( '--> XT[%d:%d], num_examples=%i,  order=%d\n', min(select), max(select),size(XT,2), kernel.order );
      sg( addFeats, svmMode, XT_temp, 'DNA' );
    else
      sg( addFeats, svmMode,{seq}, 'DNA' );
      sg('from_position_list','TEST', winsize_temp, int32(pos_temp));
      % NEW
      %sg( addFeats, svmMode, {seq}, 'DNA', 'from_position_list', winsize_temp, int32(pos_temp), 1);
    end
  end;
  

  %------------------------------------------------------------------------------%
  % kernel: Spectrum
  %------------------------------------------------------------------------------%

  if strcmp(kernel.kernel_name,'SP' )
    if~pred_from_seq 
      fprintf( '--> XT[%d:%d], wordlen=%d\n', min(select), max(select), kernel.wordlen );
      sg( addFeats, svmMode, XT_temp, 'DNA');
    
      if( ~isfield(kernel,'gaps') )
        sg( 'convert', svmMode, 'STRING', 'CHAR', 'STRING', 'WORD', kernel.wordlen, kernel.wordlen-1 ) ;
      else
        fprintf('--> convert\n')
        sg( 'convert', svmMode, 'STRING', 'CHAR', 'STRING', 'WORD', kernel.wordlen, kernel.wordlen-1, kernel.gaps );
      end;
      
      if( strcmp(svmMode,'TRAIN') && use_preproc )
        fprintf('--> clean_preproc\n')
        sg( 'clean_preproc' );
        fprintf('--> add_preproc SORTWORDSTRING\n')
        sg( 'add_preproc', 'SORTWORDSTRING' );
        fprintf('--> attach_preproc %s\n', svmMode)
        sg( 'attach_preproc', svmMode );
      end;
    else
      % from_position_list doesnot work for training for spectrum kernel  
      assert(strcmp(svmMode,'TEST')) ;
      sg( addFeats, svmMode, {seq}, 'DNA' );
      % window is cut out from sequence if vonvertion is made after calling 'from_position_list
      % if convertion is done at this place the window size must be reduced by -ordnung-1;
      % (not yet tested) this woud save a lot of time and space. 
      % 
      if( ~isfield(kernel,'gaps') )
        sg( 'convert', svmMode, 'STRING', 'CHAR', 'STRING', 'WORD', kernel.wordlen, 0);
      else
        fprintf('--> convert\n')
        sg( 'convert', svmMode, 'STRING', 'CHAR', 'STRING', 'WORD', kernel.wordlen, 0, kernel.gaps );
      end;
      sg('from_position_list','TEST', winsize_temp- kernel.wordlen+1, int32(pos_temp+kernel.wordlen-1));
    end
  end;
 
  %------------------------------------------------------------------------------%
  % kernel:  RBF 
  %------------------------------------------------------------------------------%
 
  if strcmp(kernel.kernel_name,'RBF')
    fprintf( '--> XT[%d:%d], num_examples=%i,  sigma=%d\n', min(select), max(select),size(XT,2), kernel.sigma );
    sg( addFeats, svmMode, XT_temp);    
  end;
  
  %------------------------------------------------------------------------------%
  % kernel:  POLYNOMIAL
  %------------------------------------------------------------------------------%
 
  if strcmp(kernel.kernel_name,'POLY')
    fprintf( '--> XT[%d:%d], num_examples=%i,  degree=%d\n', min(select), max(select),size(XT,2), kernel.degree );
    sg( addFeats, svmMode, XT_temp);    
  end;
  
  %------------------------------------------------------------------------------%
  % kernel:  LINEAR
  %------------------------------------------------------------------------------%
  
  if strcmp(kernel.kernel_name,'LINEAR')
    fprintf( '--> XT[%d:%d], num_examples=%i,\n', min(select), max(select),size(XT,2) );
    sg( addFeats, svmMode, XT_temp);    
  end;
end%loop over kernels
%------------------------------------------------------------------------------%
% done
%------------------------------------------------------------------------------%

if( use_preproc & strcmp(svmMode,'TEST')  & ~pred_from_seq )
   fprintf('--> attach_preproc %s', svmMode)
  sg( 'attach_preproc', svmMode );
end;

clear XT;
sg( 'init_kernel', svmMode );

% --- set feature weights
if( conservWdsWeighting > 0 )
  W1 = W( select1, : );
  if( conservWdsWeightNorming )
    fprintf( '--> conserv(WDS): using normalized feature weights!\n' );
    meanW = mean( W1, 1 );
    W1 = W1 .* repmat( 1./meanW, size(W1,1), 1 );
  else
    fprintf( '--> conserv(WDS): using raw feature weights!\n' );
    W1 = 3/size(W1,1) * W1;
  end;
  sg( 'set_subkernel_weights_combined', W1, 1 );
  clear W1;
end;



return

function [XT,LT,POS,SC] = subsample(XT, LT, POS, SC, subsample_neg, subsample_pos)
% [XT,LT,POS,SC] = subsample(XT, LT, POS, SC, subsample_neg, subsample_pos)

if subsample_neg<1 & subsample_neg>0 & subsample_pos<=1 & subsample_pos>0 % & ~isempty(LT) 

  fprintf('subsampling %i %% of negative examples \n',subsample_neg*100);
  fprintf('subsampling %i %% of positive examples \n',subsample_pos*100);
  posidx = find(LT==1);
  negidx = find(LT==-1);
  rand('seed', pi) ;
  idx2 = randperm(length(negidx));
  idx3 = randperm(length(posidx));
  num_neg = round(subsample_neg*sum(LT==-1));
  num_pos = round(subsample_pos*sum(LT==1));
  
  XT = [XT(:,posidx(idx3(1:num_pos))) XT(:,negidx(idx2(1:num_neg)))] ;
  
  if ~isempty(SC)
    SC = [SC(:,posidx(idx3(1:num_pos))) SC(:,negidx(idx2(1:num_neg)))] ;
  end
  LT = [LT(posidx(idx3(1:num_pos))) LT(negidx(idx2(1:num_neg)))] ; 
  POS.pos = [POS.pos(posidx(idx3(1:num_pos))) POS.pos(negidx(idx2(1:num_neg)))] ; 
  POS.region_id = [POS.region_id(posidx(idx3(1:num_pos))) POS.region_id(negidx(idx2(1:num_neg)))] ; 
  fprintf('%i examples remaining (%.2f %% positives)\n',length(LT),sum(LT==1)/length(LT)*100);

end

return

function [RFV, RSV, train_test_overlap] = svm_predict(Train, Data, svmFileName, outFileName, pred_from_seq, check_overlap);
% [RFV, RSV] = svm_predict(Train, Data, svmFileName,outFileName);
%
%------------------------------------------------------------------------------%
% deal with parameters 
%------------------------------------------------------------------------------%
% for usage with rproc

  if nargin==1
    Data  = Train.Data;
    svmFileName = Train.svmFileName;
    outFileName = Train.outFileName;
    Train = Train.Train;
  end
  
  fprintf('working for outputs to be written to %s\n', outFileName);
  
  method = Train.PAR.Signal.method ;
  
  if ~exist('check_overlap', 'var'),
    check_overlap=1 ;
  end ;
  
  if( ~isfield(method,'conservWeighting') )
    method.conservWeighting = 0;
  end;
  if( method.conservWeighting == 0 )
    W1 = [];
    meanCons = [];
  else
    W1 = res.W;
    assert( ~isempty(W1) );
    meanCons = mean( W1, 1 );
  end;
  
  %------------------------------------------------------------------------------%
  % init svm and set kernel parameters
  %------------------------------------------------------------------------------%
  if (pred_from_seq && ~isempty(Data.pos)) || (~pred_from_seq && ~isempty(Data.XT))
    init_svm_and_kernels(method,pred_from_seq);
    sg_ver = sg('get_version');
  else
    sg_ver = 'empty_prediction' ;
  end ;
  %------------------------------------------------------------------------------%
  % init training features with labels and examples that only contain those of 
  % support vectors
  %------------------------------------------------------------------------------%

  if (pred_from_seq && ~isempty(Data.pos)) || (~pred_from_seq && ~isempty(Data.XT))
    sg('clean_features', 'TRAIN');
    % D.pos0 = method.center;
    D.XT = Train.XT;
    D.LT = Train.LT;
    
    setFeatures(method, D,'TRAIN',0,0);
  end 
  
  clear D 
  
  %------------------------------------------------------------------------------%
  % init test features with examples if pred_from_seq==0 or with seq and positions
  % in the case of genome wide predictions
  %------------------------------------------------------------------------------%

  if (pred_from_seq && ~isempty(Data.pos)) || (~pred_from_seq && ~isempty(Data.XT))
    sg('clean_features', 'TEST' );
    if ~pred_from_seq
      D.XT = Data.XT;
      if isfield(method, 'feature_type') && isequal(method.feature_type,'double') && method.normalize_features
        D.XT(Train.idx_rm,:)  = [];
        D.XT = D.XT - repmat(Train.mean,size(D.XT,2),1)';
        D.XT = D.XT./repmat(Train.std,size(D.XT,2),1)';
      end
      % D.pos0 = method.center;
      D.pos = Data.Pos;
    else
      D.seq = Data.seq;
      assert(size(Data.pos,1)==1|size(Data.pos,2)==2|isempty(Data.pos))
      if size(Data.pos,1)~=1
        D.pos = Data.pos';
      else
        D.pos = Data.pos;
      end
    end
    
    if ~isempty(D.pos)
      setFeatures(method, D, 'TEST', pred_from_seq, 0);
      D_pos = D.pos ;
      clear D
      %------------------------------------------------------------------------------%
      % setup svm with trainingdata
      %------------------------------------------------------------------------------%
      %Train.XT should contain only support vector examples
      assert(size(Train.alphas, 1)==size(Train.XT,2))
      % alphas are in sparse format col_1=value; col_2=0-based index
      % change index to 0:size(alphas,2)-1 consistent with XTs  
      Train.alphas(:,2)= (0:size(Train.alphas, 1)-1)';
      sg('set_svm', Train.b, Train.alphas);
      
      %------------------------------------------------------------------------------%
      % classify
      %------------------------------------------------------------------------------%
      if( method.use_mkl )
        sg( 'set_subkernel_weights', betas );
      end;
      output = sg( 'svm_classify' );
      
      % work-around for from_position_list bug
      if length(D_pos) == 1 && isequal(class(D_pos), 'double') && length(output)==2,
        output=output(1) ;
      end ;
      clear D_pos
    else
      output = [];
    end
  else
    output = [] ;
  end 
  
  % check for overlaps between training and testing data
  
  if check_overlap && ~isequal(determine_engine, 'octave'),
    
    train_subset = randperm(size(Train.XT, 2)) ;
    train_subset = train_subset(1:min(length(train_subset), 100)) ;
    
    if ~pred_from_seq
      [tmp, idx, idx_] = intersect(Train.XT(:, train_subset)', Data.XT', 'rows') ;
    else
      P=[] ; 
      for i=1:length(train_subset), 
        P = [P strfind(Data.seq, Train.XT(:,train_subset(i))') + method.center] ; 
      end ;
      [tmp,idx,idx_]=intersect(P, Data.pos);
    end ;
    if length(idx)>0,
      warning('Overlap between Train and Test sets discovered (%i/%i sequences)\n', length(idx), length(train_subset)) ;
      if length(idx)/length(train_subset)>0.1,
        warning('Large overlap between Train and Test sets discovered (%i/%i sequences)\n', length(idx), length(train_subset)) ;
      end ;
    end ;
  end 
      
      
      
  %------------------------------------------------------------------------------%
  % save output
  %------------------------------------------------------------------------------%
  
  PAR = Data.PAR;
  PAR.Signal = Train.PAR.Signal;
  
  if isfield(Data, 'LT')
    RSV = calcrocscore(output, Data.LT); 
    if isnan(RSV), 
      RSV=0.5 ;
    end ;
    RFV= calcrfcscore(output, Data.LT);
    if isnan(RFV),
      RFV=0 ;
    end;
    LT = Data.LT;
    %   save( outFileName, 'output','LT','PAR', 'svmFileName','RSV','RFV' );
    POS = Data.Pos;
    fprintf( 'save output to %s\n',outFileName );
    inventory = {'output', 'POS'};
    %save( outFileName, '-V7', 'inventory', 'output','LT','POS', 'Train', 'svmFileName','RSV','RFV','sg_ver','PAR' );
    save( outFileName, '-V7', 'inventory', 'output','LT','POS', 'svmFileName','RSV','RFV','sg_ver','PAR' );
  else
    pos = Data.pos;
    fprintf( 'save output to %s\n',outFileName );
    %save( outFileName, '-V7', 'output', 'pos', 'Train', 'svmFileName','sg_ver','PAR' );
    if isfield(PAR.Signal, 'export_settings') && isfield(PAR.Signal.export_settings, 'resolution') && PAR.Signal.export_settings.resolution>1
      pos_short = zeros(1,floor(length(pos)/2));
      output_short = zeros(1,floor(length(pos)/2));
      % reduce number of predictions (take the better one out of two) 
      for j=2:2:length(pos)
        if output(j-1)>output(j)
          output_short(j/2) = output(j-1);
          pos_short(j/2) = pos(j-1);
        else
          output_short(j/2) = output(j);
          pos_short(j/2) = pos(j);
        end
      end
      output = output_short;
      pos = pos_short;
    end
    
    inventory = {'output', 'pos'};
    assert(isequal(size(pos),size(output)) || (isempty(pos) && isempty(output)));
    save( outFileName, '-V7', 'inventory', 'output', 'pos', 'svmFileName','sg_ver','PAR' );
  end
      
      
return

function [ alphas, b, betas, PAR1 ] = svm_train(Data);

P = Data.P;

method  = P.Signal.method;
fn_SVMs = P.fn_SVMs;
pred_from_seq = P.pred_from_seq;
use_preproc = P.use_preproc;


%------------------------------------------------------------------------------%
% SVM already trained?
%------------------------------------------------------------------------------%

%if( ~isfield(PAR,'conservWeighting') )
method.conservWeighting = 0;
%end;
if( ~exist('testDataPar','var') )
  testDataPar = [];
end;


%------------------------------------------------------------------------------%
% train SVM
%------------------------------------------------------------------------------%


if( method.conservWeighting == 0 )
  Data.W = [];
else
  Data.W = res.W;
  assert( ~isempty(W) );
end;

% keyboard
init_svm_and_kernels(method,pred_from_seq);


if isequal(method.feature_type,'double') && method.normalize_features
  idx_rm = find(std(Data.XT')<1e-8);
  Data.XT(idx_rm,:)  = [];
  mean_tr = mean(Data.XT');
  Data.XT = Data.XT - repmat(mean_tr,size(Data.XT,2),1)';
  std_tr = std(Data.XT');
  Data.XT = Data.XT./repmat(std_tr,size(Data.XT,2),1)';
end

setFeatures(method, Data,'TRAIN',pred_from_seq,use_preproc);

m = length( Data.LT );
% PAR1 = P;
% PAR1.POS = Data.Pos;
% PAR1.lenTrain = size( Data.XT, 1 );

sg_ver=sg('get_version');

% === train
nofKernels = length(method.kernels);
t0 = cputime;
sg('set_kernel_optimization_type', 'FASTBUTMEMHUNGRY' );
%sg( 'set_kernel_optimization_type', 'SLOWBUTMEMEFFICIENT' );
if( method.use_mkl )
  sg('use_mkl', true );
  sg('mkl_parameters', 1e-3, 0 );
  betas = sg( 'get_subkernel_weights' );
  assert( size(betas) == [nofKernels,1] );
end;
sg('svm_train' );
fprintf( '\n' );
t = cputime - t0;

%------------------------------------------------------------------------------%
% get alphas and betas
%------------------------------------------------------------------------------%

% === alphas
[ b, alphas ] = sg( 'get_svm' );


% === betas (kernel weights)
if(method.use_mkl)
  betas = sg( 'get_subkernel_weights' );
  assert( size(betas) == [nofKernels,1] );
  betas
else
  betas = ones( nofKernels, 1 );
end;


%------------------------------------------------------------------------------%
% save SVM
%------------------------------------------------------------------------------%
Train.alphas = alphas;
Train.b = b;
Train.PAR = P;
Train.POS = Data.Pos;
sup_vec_idx = alphas(:,2)+1;
Train.XT = Data.XT(:,sup_vec_idx);
Train.LT = Data.LT(sup_vec_idx);

w = zeros(size(Train.XT,1),1);
for a=1:length(Train.LT)
  w = w+Train.alphas(a,1).*Train.XT(:,a);
end
Train.w = w;

Train.POS.pos = Data.Pos.pos(sup_vec_idx);
Train.POS.region_id = Data.Pos.region_id(sup_vec_idx);

if isequal(method.feature_type,'double') && method.normalize_features
  Train.idx_rm = idx_rm
  Train.mean = mean_tr;
  Train.std = std_tr;
end

clear P Data
sg( 'clean_features', 'TRAIN' );
sg( 'clean_features', 'TEST' );

Train.time=t;
Train.sg_ver = sg_ver ; 
fprintf('save: %s',fn_SVMs)
save(fn_SVMs, '-V7', 'Train')






return

function Content = cds_exon

name = 'cds_exon';

Content = content_template(name);

Content.wordlen = {[3],[4],[5]};
Content.stepping = 1; 
Content.offset = 0;
return

function Content=content_template(name)

Content.name = name;
Content.consensus = [];
Content.lwin_big = 0;
Content.rwin_big = 0;
%Content.order = [3 4 5];
Content.label_fct = 'get_cand_contents';
Content.load_fct = 'load_content_data';

Content.filter_label.train.USE_ALL = 1;
Content.filter_label.test.USE_ALL = 1;
Content.filter_label.eval.USE_ALL = 1;
Content.Conf_names = {} ;
Content.C = [1]; 

Content.wordlen = { };
Content.stepping = 1;
Content.offset = 0;

Content.export_settings.conf_cum_thresh = 0.5 ;
Content.export_settings.resolution = 1 ;


%PAR.Contents.type_descrs = { 'intergenic', 'utr_exon', 'cds_exon','intron', 'intercistronic' };
%PAR.Contents.type_abbrs = { 'ige' 'utr' 'cex' 'ino' 'ics' };
%PAR.Contents.num_types = length(PAR.Contents.type_descrs); 



return

function Content = frame0(info)

name = 'frame0';

Content = content_template(name);
Content.wordlen = {[3],[4],[5],[6]};
Content.stepping = 3;

Content.offset = mod(3-mod(Content.wordlen{1},3),3);
 

return

function Content = intergenic

name = 'intergenic';

Content = content_template(name);

Content.wordlen = {[3],[4],[5]};
Content.stepping = 1; 
Content.offset = 0;
return

function Content=intron

name = 'intron';

Content = content_template(name);

Content.wordlen = {[3],[4],[5]};
Content.stepping = 1; 
Content.offset = 0;
return

function Content = polya_tail

name = 'polya_tail';

Content = content_template(name);

Content.wordlen = { [3:5]};

return

function Content = utr3exon

name = 'utr3exon';

Content = content_template(name);

Content.wordlen = {[3],[4],[5]};
Content.stepping = 1; 
Content.offset = 0;
return

function Content=utr5exon

name = 'utr5exon';

Content = content_template(name);

Content.wordlen = {[3],[4],[5],[6]};
Content.stepping = 1; 
Content.offset = 0;
return

function PAR=exp_galaxy_A_thaliana(PAR)

if nargin<1 
  PAR=init_PAR;
end

PAR.date=date;
PAR.exp_name='';
PAR.dir_name ='';

PAR.organism.name = 'A_thaliana';  

PAR.SETs.num_partitions=5;
PAR.SETs.num_splits=5;
%PAR.SETs.which_pred_svm = 1;

PAR.tasks.modelsel_on = 1;
PAR.tasks.learn_plifs_on =1;
PAR.tasks.pred_from_seq=1;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;
PAR.tasks.prepare_labels=0;
PAR.tasks.pred_on_genome = 0 ;
PAR.tasks.train_on = 1;
PAR.tasks.run_locally=1;

PAR.FN.input_dir='' ;
PAR.FN.output_dir='' ;

PAR.RPROC.collect_jobs = 1;
PAR.RPROC.options.ncpus = 1;
PAR.RPROC.exm_per_batch = 1;
PAR.RPROC.time_req = 3600;
PAR.RPROC.MEMREQ = 3000;

return

function PAR=exp_galaxy_C_elegans(PAR)

if nargin<1 
  PAR=init_PAR;
end

PAR.date=date;
PAR.exp_name='';
PAR.dir_name ='';

PAR.organism.name = 'C_elegans'; 

PAR.SETs.num_partitions=1;
PAR.SETs.num_splits=1;
PAR.SETs.which_pred_svm = 1;

PAR.tasks.modelsel_on = 0;
PAR.tasks.learn_plifs_on =0;
PAR.tasks.pred_from_seq=1;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;
PAR.tasks.prepare_labels=0;
PAR.tasks.pred_on_genome = 1;
PAR.tasks.train_on = 0;
PAR.tasks.run_locally=1;

PAR.FN.input_dir='/tmp';
PAR.FN.output_dir='/tmp';

PAR.RPROC.collect_jobs = 1;
PAR.RPROC.options.ncpus = 1;
PAR.RPROC.exm_per_batch = 1;
PAR.RPROC.time_req = 3600;
PAR.RPROC.MEMREQ = 3000;


return

function PAR=exp_galaxy_D_melanogaster(PAR)

if nargin<1 
  PAR=init_PAR;
end

PAR.date=date;
PAR.exp_name='';
PAR.dir_name ='';

PAR.organism.name = 'D_melanogaster';  

PAR.SETs.num_partitions=1;
PAR.SETs.num_splits=1;
PAR.SETs.which_pred_svm = 1;

PAR.tasks.modelsel_on = 0;
PAR.tasks.learn_plifs_on =0;
PAR.tasks.pred_from_seq=1;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;
PAR.tasks.prepare_labels=0;
PAR.tasks.pred_on_genome = 1;
PAR.tasks.train_on = 0;
PAR.tasks.run_locally=1;

PAR.FN.input_dir='/tmp';
PAR.FN.output_dir='/tmp';

PAR.RPROC.collect_jobs = 1;
PAR.RPROC.options.ncpus = 1;
PAR.RPROC.exm_per_batch = 1;
PAR.RPROC.time_req = 3600;
PAR.RPROC.MEMREQ = 3000;


return

function PAR=exp_galaxy_D_rerio(PAR)

if nargin<1 
  PAR=init_PAR;
end

PAR.date=date;
PAR.exp_name='';
PAR.dir_name ='';

PAR.organism.name = 'D_rerio';  

PAR.SETs.num_partitions=1;
PAR.SETs.num_splits=1;
PAR.SETs.which_pred_svm = 1;

PAR.tasks.modelsel_on = 0;
PAR.tasks.learn_plifs_on =0;
PAR.tasks.pred_from_seq=1;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;
PAR.tasks.prepare_labels=0;
PAR.tasks.pred_on_genome = 1;
PAR.tasks.train_on = 0;
PAR.tasks.run_locally=1;

PAR.FN.input_dir='/tmp';
PAR.FN.output_dir='/tmp';

PAR.RPROC.collect_jobs = 1;
PAR.RPROC.options.ncpus = 1;
PAR.RPROC.exm_per_batch = 1;
PAR.RPROC.time_req = 3600;
PAR.RPROC.MEMREQ = 3000;


return

function PAR=exp_galaxy(PAR_config)
% PAR=exp_galaxy(PAR_config)

PAR.SETs.num_partitions=5 ;
PAR.SETs.num_splits=5 ;

PAR.FN.input_dir=PAR_config.dir ;
PAR.FN.output_dir=PAR_config.dir;
PAR.FN.genome_dir=PAR_config.dir ;
PAR.FN.input.fn_genome_config = PAR_config.genome_config ;

% preparing RPROC settings
PAR.RPROC.MEMREQ = 6000 ;
PAR.RPROC.collect_jobs = 0 ;
PAR.RPROC.exm_per_batch = 1 ;
if 0
PAR.RPROC.options.addpaths = {'~/svn/projects/genefinding/sensors',
                    '~/svn/projects/genefinding/data_processing_sensors',
                    '~/svn/projects/genefinding/contents',
                    '~/svn/projects/genefinding/galaxy/tools/',
                    '~/svn/projects/genefinding/utils',
                    '~/svn/projects/genefinding/shogun',
                    '~/svn/projects/genefinding/data_processing_lsl',
                    '~/svn/tools/genomes',
                    '~/svn/tools/utils'} ;

end

PAR.RPROC.options.addpaths = {} ;

signals = {'acc','don','tis','cdsStop','tss','cleave'};
for i=1:length(signals)
  PAR.label_source.(signals{i}).from_fn_candsites = 1 ;
  PAR.Signals.(signals{i}).filter_label.train.USE_ALL = 1;
  PAR.Signals.(signals{i}).filter_label.test.USE_ALL = 1;
  PAR.Signals.(signals{i}).filter_label.eval.USE_ALL = 1;
  PAR.Signals.(signals{i}).Conf_names = {} ;
  PAR.Signals.(signals{i}).method.threads = 1;
end


if 0,
  PAR.Signals.acc.order = 10 ;
  PAR.Signals.acc.shift = 0 ;

  PAR.Signals.don.order = 10 ;
  PAR.Signals.don.shift = 0 ;
  
  PAR.Signals.tis.order =  {[],[5],[]};
  PAR.Signals.tis.shift =  {[],[0],[]};
  
  PAR.Signals.cdsStop.order = {[],[5],[]};
  PAR.Signals.cdsStop.shift = {[],[0],[]};
  
  PAR.Signals.tss.order = {[],[5],[]};
  PAR.Signals.tss.shift = {[],[0],[]};
  
  PAR.Signals.cleave.order = {[],[5],[]};
  PAR.Signals.cleave.shift = {[],[0],[]};

end


contents = {'intergenic','utr5exon','cds_exon','utr3exon','polya_tail','intron','frame0'};

for i=1:length(contents)
  PAR.label_source.(contents{i}).from_fn_candsites = 1 ;
  PAR.Contents.(contents{i}).filter_label.train.USE_ALL = 1;
  PAR.Contents.(contents{i}).filter_label.test.USE_ALL = 1;
  PAR.Contents.(contents{i}).filter_label.eval.USE_ALL = 1;
  PAR.Contents.(contents{i}).Conf_names = {} ;
  PAR.Contents.(contents{i}).method.threads = 1;
end

PAR.LSL.method.use.signals.polya = 0 ;
PAR.LSL.method.exm_per_solve = 100 ;

PAR.regions.max_length = 30000 ;


if 1,

%----- REGULARIZATION
method=PAR.LSL.method ;

%C = 1 ;
C = 1e-3;
%%% linear regularization
method.par_ms.C_regul.lengths = C ;        % linear smoothness for len_hist
method.par_ms.C_regul.signals = C ;        % for sig_hist
method.par_ms.C_regul.contents = C ;       % for content_hist
method.par_ms.C_regul.transitions = C ;    % for

method.par_ms.C_regul.tiling = C ;  
method.par_ms.C_regul.rna_seq = C ;  


%%% quadratic regularization
C_sq = 1e-3;

method.par_ms.C_regul.transitions_sq = C_sq;
method.par_ms.C_regul.plif_ys_sq = C_sq*0.1 ;  % small_plif_ys_penalty
method.par_ms.C_regul.smoothness_sq = C_sq ;

PAR.LSL.method=method ;

end

return

function FN=init_FN(signal_names,content_names)
 
  
if nargin<1
  signal_names = {'acc','don','tis','cdsStop','tss','cleave','polya','transacc'};
end
if nargin<2
  content_names = {'intergenic','utr5exon','cds_exon','utr3exon','polya_tail','intron','frame0'};
end
 
%--------------------------------------------------------------------------
% Directories
% -------------------------------------------------------------------------------------

FN.genome_dir= '/fml/ag-raetsch/share/databases/genomes/';
FN.input_dir= '/fml/ag-raetsch/share/projects/genefinding/';
FN.output_dir = '/fml/ag-raetsch/share/projects/genefinding/';

FN.dir_name = '';
FN.exp_name = '';
FN.lsl_exp_name = '';
%--------------------------------------------------------------------------
% INPUT FILE NAMES
% -------------------------------------------------------------------------------------

FN.genome.fn_confirmed_sequences = 'confirmed_sequences.mat';
FN.genome.fn_genome_config_orig  = 'genome_uncondensed.config';
FN.genome.fn_genome_config  = 'genome.config';
FN.genome.fn_map = 'condense_map.txt';
FN.genome.fn_genes_anno = 'annotated_genes.mat';
FN.genome.fn_genes_merged = '';
FN.genome.fn_genes_conf = 'confirmed_sequences.mat';
FN.genome.fn_name_to_contig_mapping = 'sequences/contig_orig_new_mapping';


FN.input.fn_genome_config = 'genome.config';
FN.input.fn_trainings_regions = 'training_regions.mat';
FN.input.fn_test_regions = 'test_regions.mat';
FN.input.fn_genes_anno_orig ='annotated_genes.mat';
FN.input.fn_genes_conf_orig = 'confirmed_sequences.mat';
FN.input.fn_ESTs_matches = {'est_matches.mat'};

%--------------------------------------------------------------------------
% OUTPUT FILE NAMES
% -------------------------------------------------------------------------------------

FN.output.fn_genes_anno ='anno_genes.mat';
FN.output.fn_genes_conf = 'conf_genes.mat';
FN.output.fn_genes_merged = 'genes_all.mat';
FN.output.fn_genes_signals = 'genes_signals.mat';
FN.output.fn_genes_contents = 'annotated_genes.mat';
FN.output.fn_genes_lsl = 'annotated_genes.mat';
FN.output.fn_ESTs_matches = [];
FN.output.fn_trivial_regions = 'trivial_regions.mat';
FN.output.fn_blocks = 'blocks.mat';
FN.output.fn_training_blocks ='training_blocks.mat';
FN.output.fn_test_blocks = 'test_blocks.mat';

%--------------------------------------------------------------------------
% SIGNAL FILE NAMES
% -------------------------------------------------------------------------------------

for j=1:length(signal_names)
  sig_name = signal_names{j};
  FN.input_sig.(sig_name).fn_candsites = 'cands/' ;
  FN.input_sig.(sig_name).fn_examples = 'examples';
  FN.input_sig.(sig_name).fn_pos = 'pos/';
  FN.input_sig.(sig_name).fn_filter_settings= 'filter_settings.mat';
  FN.input_sig.(sig_name).fn_example_statistics = 'example_statistics.mat';

  FN.output_sig.(sig_name).fn_models='MS/models';
  FN.output_sig.(sig_name).fn_SVMs='MS/';
  FN.output_sig.(sig_name).fn_pred='pred/';
  FN.output_sig.(sig_name).fn_motifs='MS/motifs';
end


if any(strcmp(signal_names,'tss'))
  FN.input_sig.tss.fn_est_cluster = 'tss_est_clusters.mat';
  FN.input_sig.tss.fn_fulllength  = 'tss_fulllength.mat';
  FN.input_sig.tss.fn_db = 'tss_db.mat' ;
end

if any(strcmp(signal_names,'cleave'))
  FN.input_sig.cleave.fn_est_cluster = 'cleave_est_clusters.mat';
  FN.input_sig.cleave.fn_fulllength = 'cleave_fulllength.mat';
  FN.input_sig.cleave.fn_db  = 'cleave_db.mat';
  FN.input_sig.cleave.fn_anno  = 'cleave_anno.mat';
end
if any(strcmp(signal_names,'polya'))
  FN.input_sig.polya.fn_polya = 'truesites_and_consensus.mat';
  FN.input_sig.polya.fn_log = 'log';
end
if any(strcmp(signal_names,'tis'))
  FN.input_sig.tis.fn_anno = 'tis_anno.mat';
end
if any(strcmp(signal_names,'cdsStop'))
  FN.input_sig.cdsStop.fn_anno = 'cdsStop_anno.mat';
end

if any(strcmp(signal_names,'transacc'))
  FN.input_sig.transacc.fn_anno_SL1 = 'transacc_anno_SL1' ;
  FN.input_sig.transacc.fn_anno_SL2 = 'transacc_anno_SL2' ;
  FN.input_sig.transacc.fn_pred = 'transacc_pred';
  FN.input_sig.transacc.fn_db = []; % only for C_elegans
end

%--------------------------------------------------------------------------
% CONTENT FILE NAMES
% -------------------------------------------------------------------------------------

for j=1:length(content_names)
  content_name = content_names{j};
  FN.input_cont.(content_name) = struct;
  FN.input_cont.(content_name).fn_candsites = 'cands/' ;
  FN.input_cont.(content_name).fn_examples = 'examples';
  FN.input_cont.(content_name).fn_pos = 'pos/';
  FN.input_cont.(content_name).fn_filter_settings= 'filter_settings.mat';
  FN.input_cont.(content_name).fn_example_statistics = 'example_statistics.mat';

  FN.output_cont.(content_name).fn_models = 'MS/models';
  FN.output_cont.(content_name).fn_SVMs = 'MS/';
  FN.output_cont.(content_name).fn_pred = 'pred/';
  FN.output_cont.(content_name).fn_motifs = 'MS/motifs';
end


%--------------------------------------------------------------------------
% LSL FILE NAMES
% -------------------------------------------------------------------------------------

FN.input_lsl.fn_test_blocks = 'data/test_blocks';
FN.input_lsl.fn_training_blocks =  'data/training_blocks.mat';
FN.input_lsl.fn_training_split =  'data/blocks_split.mat';

FN.input_lsl.fn_val_blocks =  'data/validation_blocks.mat';
FN.input_lsl.fn_blocks_all =  'data/blocks_all.mat';
FN.input_lsl.fn_boundary_model = 'data/boundary_model.mat';


FN.output_lsl.fn_init =  'train/init.mat';
FN.output_lsl.fn_train_block_preds = 'train/single_block_predictions/';
FN.output_lsl.fn_train = 'train/train';
FN.output_lsl.fn_pred = 'predictions/';
FN.output_lsl.fn_blocks_pred = 'predictions/blocks_pred';
FN.output_lsl.fn_pred_genome = 'genomewide_predictions/';
FN.output_lsl.fn_log = 'log_files/log_';



return

function PAR=init_PAR(signal_names,content_names);
 
  
% -------------------------------------------------------------------------------------
% Experiments
% -------------------------------------------------------------------------------------
if nargin<1
  signal_names = {'acc','don','tis','cdsStop','tss','cleave','polya','transacc'};
end
if nargin<2
  content_names = {'intergenic','utr5exon','cds_exon','utr3exon','polya_tail','intron','frame0'};
end




PAR.date = date;
PAR.user = whoami;
  
PAR.organism.clade = [];
PAR.organism.name = '';
PAR.organism.full_name = [];
PAR.organism.release = [];
PAR.organism.version = [];
PAR.organism.genebuilt = [];

% -------------------------------------------------------------------------------------
% info regarding original gene structure
% -------------------------------------------------------------------------------------


info_genes.transcript.anno = -1;
info_genes.transcript.est = 0;
info_genes.transcript.cDNA = 1;
info_genes.transcript.fulllength = 2;

info_genes.transcript_status.confirmed = 1;
info_genes.transcript_status.partially_confirmed = 0;
info_genes.transcript_status.predicted = -1;


info_genes.pseudo.dummy = [];

% -------------------------------------------------------------------------------------
% info regarding label source
% -------------------------------------------------------------------------------------

label_source.use_anno = [];
label_source.use_confirmed = [];
for j=1:length(signal_names)
  signal_name = signal_names{j};
  label_source.(signal_name).anno = [];
  label_source.(signal_name).est = [];
  label_source.(signal_name).cDNA = [];
  label_source.(signal_name).fulllength = [];
  label_source.(signal_name).db = [];
  label_source.(signal_name).from_fn_candsites = [];
  
  info_genes.(signal_name).anno = -1;
  info_genes.(signal_name).est = 0;
  info_genes.(signal_name).cDNA = 1;
  info_genes.(signal_name).fulllength = 2;
  info_genes.(signal_name).db = 3;
  info_genes.(signal_name).from_fn_candsites = 4;
end
for j=1:length(content_names)
  content_name = content_names{j};
  label_source.(content_name).from_fn_candsites = [];
end

label_source.tis.maxorf = [];
info_genes.tis.maxorf = 5;
label_source.cdsStop.maxorf = [];
info_genes.cdsStop.maxorf = 5;

label_source.transacc.anno_SL1 = [];
label_source.transacc.anno_SL2 = [];
label_source.transacc.pred = [];
info_genes.transacc.anno_SL1 = 5;
info_genes.transacc.anno_SL2 = 6;
info_genes.transacc.pred = 7;

label_source.polya.from_cleave = [];
info_genes.polya.from_cleave = 5;


PAR.label_source = label_source;
PAR.info_genes = info_genes;


PAR.Signal_name = [];
for j=1:length(signal_names)
  signal_name = signal_names{j};
  PAR.Signals.(signal_name) = struct;
end

cnt=0;
PAR.Content_name = [];
for j=1:length(content_names)
  PAR.Contents.(content_names{j}) = struct;
  PAR.Contents.(content_names{j}).name = content_names{j};
  PAR.Contents.(content_names{j}).type = cnt;
  cnt=cnt+1;
end

PAR.LSL.method = struct;

% -------------------------------------------------------------------------------------
% memory requirements
% -------------------------------------------------------------------------------------

PAR.RPROC.collect_jobs = 0 ; 
PAR.RPROC.options.ncpus = 1 ; 
PAR.RPROC.options.waitonfull = [];
PAR.RPROC.options.maxjobs = [] ;
PAR.RPROC.options.immediately = 0;
PAR.RPROC.options.immediately_bg = 0;
PAR.RPROC.options.addpaths = [];
PAR.RPROC.options.rmpaths = [];

PAR.RPROC.MEMREQ = [];
PAR.RPROC.time_req = [];
PAR.RPROC.exm_per_batch = 1 ;


% -------------------------------------------------------------------------------------
% SPLITS AND PARTITIONS
% -------------------------------------------------------------------------------------

PAR.SETs.num_splits = 5;
PAR.SETs.num_partitions = 5;
PAR.SETs.train_subsample_neg = 1;
PAR.SETs.test_subsample_neg = 1;
PAR.SETs.train_subsample_pos = 1;
PAR.SETs.test_subsample_pos = 1;
%for lsl
PAR.SETs.num_train = [] ; 
PAR.SETs.ignore_incomplete = 1 ; 


% -------------------------------------------------------------------------------------
% original data processing stuff
% -------------------------------------------------------------------------------------

PAR.regions.max_length = 1000000;
PAR.regions.min_genes_per_region = 1;
PAR.regions.offset_test = 20000;
PAR.regions.offset_train = 1000;
PAR.regions.minCutDist = 250;
PAR.regions.cluster_paralogs = 0;
PAR.regions.merge_confirmed = 0;

% -------------------------------------------------------------------------------------
% TASKS
% -------------------------------------------------------------------------------------

PAR.tasks.signals.prepare_labels = 0;
PAR.tasks.signals.train = 0;
PAR.tasks.signals.modelsel = 0;
PAR.tasks.signals.learn_plifs = 0;
PAR.tasks.signals.get_train_error = 0;

PAR.tasks.signals.pred_on_genome = 0; 
PAR.tasks.signals.verbose = 0; 

PAR.tasks.signals.run_locally = 0;

% -------------------------------------------------------------------------------------
% transfer from other organisms
% -------------------------------------------------------------------------------------

PAR.Source_organisms.names = {};
PAR.Source_organisms.dir_names = {};
PAR.Source_organisms.exp_names = {};


% -------------------------------------------------------------------------------------
% FN
% -------------------------------------------------------------------------------------
PAR.FN = struct;


return

function lengths_range = default_lengths

lengths_range.intergenic = [0, 21000] ;
%lengths_range.intergenictrans = [0, 21000] ;
lengths_range.utr5exon   = [0, 8000] ;
% lengths_range.transexon  = [1, 1000] ;
lengths_range.single_cds_exon = [100, 8000] ; %%%100, from don => acc+stop
lengths_range.first_cds_exon  = [3, 8000] ;
lengths_range.middle_cds_exon   = [3, 8000] ;
lengths_range.last_cds_exon   = [1, 8000] ;
lengths_range.utr3exon    = [1, 8000] ;
lengths_range.polya_tail  = [0, 250] ;
% lengths_range.intercistronic = [1, 5000] ;
lengths_range.intergenic_long = [10500-50, 10500+50] ; % due to irregular spacing 
lengths_range.intron      = [30, 21000] ;%%%30, from don => acc+stop

return

function LSL = lsl_template
  

method.name = 'lsl' ;
method.model_fct = 'make_model';
method.train_fct = 'lsl_train';
method.predict_fct = 'lsl_predict';

method.dt = datestr(now,'dd-mm-yyyy');
method.user = 'GS';
method.plif_bins   = 20 ;   % number of plif bins

%%%

%%Signals
use.signals.trans = 0 ;
use.signals.transacc = 0 ;
use.signals.tss = 1;
use.signals.polya = 1;
use.signals.cleave =  1;

%%States
use.states.SL1 = 0 ;
use.states.SL2 = 0;

%%transitions
use.transitions.allow_missing_tss = 0;
use.transitions.one_gene = 0 ; % force exactly one gene
use.transitions.use_multi = 1 ; % allow zero or more than one gene 

%%loss
use.loss.orf_only = 0;
use.model.one_gene = 0;
use.model.allow_missing_tss = 0;
use.model.use_multi = 1;

%%Segments & lengths
use.segments.transexon = 0 ;
use.segments.intercistronic = 0 ;
use.segments.intergenictrans = 0;
use.segments.polya_tail = 1;

%%Content 

use.contents.frame = 1;
use.contents.pre_comp = 1;

%%additional features
use.addfeatures.conservation = 0 ;
use.addfeatures.ests = 0 ;
use.addfeatures.proteins = 0 ;
use.addfeatures.tiling = 0 ;
use.addfeatures.rna_seq = 0 ;

% keyboard
method.use = use;



%----- REGULARIZATION
%C = 1 ;
C = 1e-1;
%%% linear regularization
method.par_ms.C_regul.lengths = C ;        % linear smoothness for len_hist
method.par_ms.C_regul.signals = C ;        % for sig_hist
method.par_ms.C_regul.contents = C ;       % for content_hist
method.par_ms.C_regul.transitions = C ;    % for

method.par_ms.C_regul.tiling = C ;  
method.par_ms.C_regul.rna_seq = C ;  


%%% quadratic regularization
C_sq = 1e-1;

method.par_ms.C_regul.transitions_sq = C_sq;
method.par_ms.C_regul.plif_ys_sq = C_sq*0.1 ;  % small_plif_ys_penalty
method.par_ms.C_regul.smoothness_sq = C_sq ;


%----------- for predictions only
method.par_ms.cleave_offset = 6;
%-----------

method.solver = 'dual';
method.ignore_init_file = 1;

method.opt_step = 1000 ;
method.max_neg = 2 ;
method.margin = 1 ;

method.INF = 1e20;
method.max_num_iterations = 100;

method.exm_per_solve = 400 ;
% method.exm_per_batch = 5 ;

%method.cleave_offset = 6 ;
% -------
% -------
LSL.method = method;


return

function method = method_template

% Signals = method_template(Signal,signal_name)
% Signals is a struct with a field for each Signal. In this function only
% the field Signal.(signal_name).method is added. Default values are set.

method.name = [];
method.train_fct = [];
method.predict_fct = [];

% PAR.method.svm_name = 'GPBTSVM';PAR.method.qpsize=500;
% use conservation information
% method.use_cons = 0;
% load training examples only once and calculate all models in one job
% method.submit_batch=0;

method.plif.bins = 50;
method.plif.min_incr = 1e-6;

method.use_cons = 0;
method.feature_type = [];
%assert(max(abs([PAR.Signals.(PAR.Signal_name).lwin{:}]))<PAR.Signals.(PAR.Signal_name).lwin_big)

%assert(max(abs([PAR.Signals.(PAR.Signal_name).lwin{:}]))<PAR.Signals.(PAR.Signal_name).rwin_big)




return

function method = SVM_COMBINED(Signal)

% Signals = SVM_COMBINED(Signals,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. In this function only
%          the field (signal_name) is modified.
%          Particuarly a field method is added.
%        - signal_name can be something in {'acc','don','tss','cleave','polya','tis','cdsStop'};
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: SVM_COMBINMED -> SVM -> method_template


method = SVM(Signal) ;

nofKernels = length(Signal.kernel_name);

assert(length(Signal.lwin)==nofKernels)
assert(length(Signal.rwin)==nofKernels)


for k=1:nofKernels
  for n = Signal.kernel_par_names 
    method.kernels{k}.(n{1}) = Signal.(n{1}){k};
  end
end
if isfield(Signal,'subkernel_weights')
  method.subkernel_weights = Signal.subkernel_weights;
end


return

function method = SVM_Cont(Content)

  
method = SVM(Content);
method.feature_type = 'char';
method.center = [];
method.kernels{1}.name = 'LINEAR';
method.genomewide_predict_fct = 'svm_content_predict';

for k=1:length(Content.wordlen)
  method.kernels{k}.wordlen = Content.wordlen{k};
  method.kernels{k}.stepping = Content.stepping ;
  method.kernels{k}.offset = Content.offset ;
end



return

function method = SVM(Signal)
  
method = method_template;

% set parameters, model selection 

method.name = 'SVM';
method.train_fct = 'svm_train';
method.predict_fct = 'svm_predict';
method.kernels = {[]};
method.poim_maxOrder = 8;
method.svm_name = 'LIGHT';
method.qpsize = 50;
method.threads = 16;
method.center = Signal.lwin_big;
method.par_ms.C = Signal.C;

%PAR.method.svm_name = 'GPBTSVM';PAR.method.qpsize=500;

% use multi kernel learning
method.use_mkl = 0;





return

function method = SVM_WD(Signal)

method = SVM(Signal);
method.feature_type = 'char';

kernel.name = 'WD';
kernel.order = Signal.order;
kernel.shift = 0 ;
kernel.lwin = Signal.lwin{1};
kernel.rwin = Signal.rwin{1};

method.kernels{1} = kernel; 


return

function method = SVM_WDS(Signal)

% Signals = SVM_WDS(Signals,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. In this function only
%          the field (signal_name) is modified.
%          Particuarly a field method is added.
%        - signal_name can be something in {'acc','don','tss','cleave','polya','tis','cdsStop'};
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: SVM_WDS -> SVM -> method_template

  
method = SVM(Signal) ;
method.feature_type = 'char';


kernel.name = 'WDS';
kernel.order = Signal.order;
kernel.shift = Signal.shift;
kernel.lwin =  Signal.lwin{1};
kernel.rwin = Signal.rwin{1};

method.kernels{1} = kernel; 

return

function PAR=A_thaliana(PAR)

PAR = organism_template(PAR);

PAR.organism.name = 'A_thaliana';

PAR.label_source.use_anno = 0;
PAR.label_source.use_confirmed = 1;

PAR.label_source.tss.est_clusters = 1;
PAR.label_source.tss.fulllength = 0;
PAR.label_source.tss.db = 0;

PAR.label_source.cleave.est_clusters = 1;
PAR.label_source.cleave.fulllength = 0;
PAR.label_source.cleave.db = 0;

PAR.label_source.cds.anno = 0;
PAR.label_source.cds.maxorf = 0;


PAR.regions.cluster_paralogs = 1;

return

function [organism label_source] = C_elegans(label_source_orig,signal_names)

[organism label_source] = Nematodes(label_source_orig,signal_names);

organism.name = 'C_elegans';
organism.full_name = 'Caenorhabditis_elegans';
organism.release = '';
organism.version = '';
organism.genebuilt = '';
return

function PAR = D_melanogaster(PAR)


PAR = organism_template(PAR);

PAR.organism.clade = 'insect';
PAR.organism.name = 'D_melanogaster';

PAR.label_source.use_anno = 0;
PAR.label_source.use_confirmed = 1;

PAR.label_source.cleave.est_clusters = 1;
PAR.label_source.cleave.fulllength = 1;
PAR.label_source.cleave.db = 0;

PAR.label_source.tss.est_clusters  = 1;
PAR.label_source.tss.fulllength   = 0;
PAR.label_source.tss.db = 0;

return

function PAR=D_rerio(PAR)
  
PAR = organism_template(PAR);

PAR.organism.name = 'D_rerio';

PAR.label_source.use_anno = 0;
PAR.label_source.use_confirmed = 1;

PAR.label_source.tss.est_clusters  = 1;
PAR.label_source.tss.fulllength   = 1;
PAR.label_source.tss.db = 0;

PAR.label_source.cleave.est_clusters = 1;
PAR.label_source.cleave.fulllength = 1;
PAR.label_source.cleave.db = 0;

return

function [organism label_source] = dummy_organism(label_source_orig,signal_names)

[label_source] = organism_template(label_source_orig,signal_names);
  
organism.clade = [];
organism.name = '';
organism.full_name = '';
organism.release = '';
organism.version = '';
organism.genebuilt = '';


return

function [label_source] = organism_template(label_source_orig,signal_names)


% PAR = set_label_source(PAR)
% 
% sets defaults for all label_sources (don't use anno, use EST confirmed only.)
% values that are already assigned will NOT be overriden.
  
  
%% default values for general use of annotation and EST confrimed sequences
label_source = label_source_orig;

if isempty(label_source.use_anno)
  label_source.use_anno = 0;
end
if isempty(label_source.use_confirmed)
  label_source.use_confirmed = 1;
end


%% default values for individual signals


for j=1:length(signal_names)
  signal_name = signal_names{j};
  if label_source.use_anno
    label_source.(signal_name).anno = 1;
  else
    label_source.(signal_name).anno = 0;
  end
  if label_source.use_confirmed & ~isequal(signal_name,'transacc')
    label_source.(signal_name).est_clusters = 1;
    label_source.(signal_name).cDNA = 1;
    label_source.(signal_name).fulllength = 1;
  elseif ~label_source.use_confirmed & ~isequal(signal_name,'transacc')
    label_source.(signal_name).est_clusters = 0;
    label_source.(signal_name).cDNA = 0;
    label_source.(signal_name).fulllength = 0;
  end
  label_source.(signal_name).db = 0;
  label_source.(signal_name).from_fn_candsites = 1 ;
end

label_source.tis.maxorf = 0;
label_source.cdsStop.maxorf = 0;

label_source.transacc.anno_SL1 = 0;
label_source.transacc.anno_SL2 = 0;
label_source.transacc.pred = 1;
label_source.transacc.db = 0;



return

function Signal = acceptor(label_source,signal_name)

% Signal = acceptor(label_source,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. In this function only
%          the field acc (transcc) is modified.  
%        - label_source is a struct that should contain a field
%          (signal_name). this is used in the called function splicesite_template.
%        - name can either be 'acc' or 'transacc'  
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: acceptor/donor -> splicesite_template -> signal_template
% (function acceptor is also called from transacc)
  
  
if nargin<3 
  signal_name = 'acc';
end

Signal = splicesites_template(label_source,signal_name);

%%%% SET ACCEPTOR STUFF

Signal.consensus = {'AG'};
Signal.offset = 2;

Signal.lwin = {[-60]} ;
Signal.rwin = {[80]} ;

Signal.order = [ 22];
Signal.shift = 0.5;
Signal.C = [3]; 

Signal.Conf_names = {Signal.Conf_names{:},'alt_3prime'};

Signals.(signal_name) = Signal;



return

function Signal = cdsStop(label_source)
  
% Signals = cdsStop(Signals,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field cdsStop is modified.
%        - label_source is a struct that should contain a field cdsStop. 
%          this is used in the called function signal_template.
%       
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tis/cdsStop -> translatends_template -> signal_template
   
signal_name = 'cdsStop';

Signal = translatends_template(label_source,signal_name);

%%%% SET cdsStop STUFF

Signal.consensus = {'TAA','TAG','TGA'};


Signal.kernel_name= {'SP','WDS','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};
Signal.lwin = {[-200] [-110] [0]} ; 
Signal.rwin = {[0] [10] [180]};
Signal.shift={[],[0],[]};
Signal.shift_const={[],[0],[]};
Signal.order={[],[20],[]};  
Signal.wordlen = {[2] [] [4]}; 
Signal.C = 2.8284; % logC=1.5



return

function Signal = cleave(label_source)

% Signals = cleave(Signals,signal_name,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field cleave is modified.
%        - label_source is a struct that should contain a field cleave. 
%          this is used in the called function signal_template.
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tss/cleave -> transcriptends_template -> signal_template

  
signal_name = 'cleave';

Signal = transcriptends_template(label_source,signal_name);

%%%% SET CLEAVE STUFF

Signal.lwin_big = 410;
Signal.rwin_big = 810;

Signal.kernel_name= {'SP','WDS','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};

Signal.lwin = {[-400], [-60], [0]};
Signal.rwin = {[100], [60], [800]};
Signal.wordlen = {[5], [], [3]}; 

Signal.order = {[],[18],[]};
Signal.shift = {[],[0.1],[]};
Signal.shift_const = {[],[0],[]};
Signal.C = 1; % log2(C)=0



return

function Signal = donor(label_source)

% Signals = donor(Signals,signal_name,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field don is modified.
%        - label_source is a struct that should contain a field don. 
%          this is used in the called function signal_template.
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: acceptor/donor -> splicesite_template -> signal_template

 
signal_name = 'don';

Signal = splicesites_template(label_source,signal_name);

%%%% SET DONOR STUFF

Signal.consensus = {'GC' 'GT'};
Signal.offset = 0;

Signal.lwin = {[-80]};
Signal.rwin = {[60]};

Signal.order = [22];
Signal.shift = 0.5;
Signal.C = [3]; 

Signal.Conf_names = {Signal.Conf_names{:},'alt_5prime'};




return

function Signal = polya(label_source,fn_polya)
  
% Signals = polya(Signals,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field polya is modified.
%        - label_source is a struct that should contain a field polya. 
%          this is used in the called function signal_template.
%       
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: polya -> signal_template

signal_name = 'polya' ; 
Signal = signal_template(label_source,signal_name);

%%%% SET POLYA STUFF

Signal.name = signal_name;
Signal.type = signal_name; 
Signal.label_fct = 'get_cand_others';
Signal.filter_fct = 'filter_cands';

Signal.offset = 0;

Signal.lwin_big = 310;
Signal.rwin_big = 310;

Signal.kernel_name= {'WDS','SP','SP','SP','SP','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};
Signal.subkernel_weights = [1 0.2 0.2 0.2 0.2 0.2];

Signal.lwin = {-300 -300 -149 1 151 -300} ;
Signal.rwin = {306 -150 0 150 306 306};
Signal.shift={[0 0.1 0.4],[],[],[],[],[]};
%Signal.shift={[0.1],[],[],[],[],[]};
Signal.shift_const={[0 10 20],[],[],[],[],[]};
%Signal.shift_const={[0],[],[],[],[],[]};
Signal.wordlen={[],[4],[4],[4],[4],[4]};
Signal.order={[18],[],[],[],[],[]};
Signal.C = [0.01 0.5 10];

Signal.consensus = {''};
if exist('fn_polya')
  Signal.consensus_fn = fn_polya;
  if fexist(Signal.consensus_fn)
    load(Signal.consensus_fn,'consensus')
    Signal.consensus = consensus;
  end
end

Signal.filter_label.train.use_intergenic=0;
Signal.filter_label.train.use_alt=0;
%Signal.filter_label.eval.use_intergenic=0;
%Signal.filter_label.eval.use_alt=0;
Signal.filter_label.test.use_intergenic=0;
Signal.filter_label.test.use_alt=0;

Signal.filter_label.train.conf_pos.anno = 0;
Signal.filter_label.train.conf_pos.from_cleave = [201 301 401];
Signal.filter_label.train.conf_pos.db = 0;
Signal.filter_label.train.conf_pos.fulllength = 0;
Signal.filter_label.train.onlywithpos = 0; 

%Signal.filter_label.eval.conf_pos.anno = 0;
%Signal.filter_label.eval.conf_pos.from_cleave = [201 301 401];
%Signal.filter_label.eval.conf_pos.db = 0;
%Signal.filter_label.eval.conf_pos.fulllength = 0;
%Signal.filter_label.eval.onlywithpos = 0; 

Signal.filter_label.test.conf_pos.anno = 0;
Signal.filter_label.test.conf_pos.from_cleave = [201 301 401];
Signal.filter_label.test.conf_pos.db = 0;
Signal.filter_label.test.conf_pos.fulllength = 0;
Signal.filter_label.test.onlywithpos = 0; 


Signal.Conf_names = {Signal.Conf_names{:},['has_' signal_name]};



Signal.label_source_par.polya.wl = 1;
Signal.label_source_par.polya.POSconstr =1;
Signal.label_source_par.polya.maxpcCov = 95;
Signal.label_source_par.polya.logdir = [];

return

function Signal = signal_template(label_source,signal_name)

% Signals = signal_template(Signal,label_source,signal_name)
% Signals is a struct with a field for each Signal. In this function only
% the field signal_name is added. Default values are set.


if nargin<2
  signal_name = 'dummy';
end

%%%%%

Signal.type = []; 
Signal.name = signal_name;
Signal.consensus = {[]};
Signal.offset = [];

Signal.lwin_big = [];
Signal.rwin_big = [];

Signal.label_fct = [];
Signal.filter_fct = 'filter_cands';
Signal.load_fct = 'load_data';
Signal.score_fct = [];

Signal.Conf_names = { 'altgenic','gene_id' };

Signal.sampling.downsample_neg = 1; % <=1 (for no downsampling: 1)  
Signal.sampling.mindist = 1; % >=1 (for all: 1)  
Signal.sampling.pos_neg_ratio = 1/10;

Signal.resolution   = 1; % for lsl

conf.USE_ALL = 0;
conf.conf_pos = struct;
conf.conf_pos_skip = struct;
conf.any_conf_pos = [];
conf.any_conf_pos_skip = [];
conf.conf_neg = struct;
conf.conf_neg_skip = struct;
conf.any_conf_neg = [];
conf.any_conf_neg_skip = [];
conf.use_intergenic=[];
conf.use_alt = [];
conf.use_label = 'label';

Signal.filter_label.train = conf ; 
Signal.filter_label.test = conf ;
Signal.filter_label.eval = conf ;

Signal.export_settings.conf_cum_thresh = 0 ;
Signal.export_settings.resolution = 1 ;

Signal.domain_adapt.name = 'target_only';
Signal.domain_adapt.Source_organisms = [];

Signal.method = struct;

label_source = label_source.(signal_name);
sources = fieldnames(label_source);
for i=1:length(sources)
  if label_source.(sources{i})
    Signal.Conf_names = {Signal.Conf_names{:},[sources{i} '_conf']};
    if isequal(signal_name,'acc')|isequal(signal_name,'don')
      Signal.Conf_names = {Signal.Conf_names{:},[sources{i} '_conf_skip']};
    end
  end
end




return

function Signal = splicesites_template(label_source,signal_name)

% Signals = splicesites_template(Signals,label_source,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field signal_name is added or
%          modified. 
%          Default values are set.
%        - label_source is a struct that should contain a field (signal_name). 
%          this is used in the called function signal_template.
%        - signal_name: 'don', 'acc','transacc'
%  
% OUTPUT Signals with modified field signal_name; 
%  
% HIERACHY: acceptor/donor -> splicesite_template -> signal_template
%
% see also transcriptends_template/translat_template
  
  
INF=1e6;

Signal = signal_template(label_source,signal_name);

%%%% SET SPLICE SITE STUFF

Signal.type = 'splicesites'; 
Signal.label_fct = 'get_cand_splicesites';

Signal.lwin_big = 199;
Signal.rwin_big = 199;

Signal.filter_label.train.use_intergenic=0;
Signal.filter_label.train.use_alt=0;
Signal.filter_label.train.use_alt_neg=0;
Signal.filter_label.test.use_intergenic=0;
Signal.filter_label.test.use_alt=0;
Signal.filter_label.test.use_alt_neg=0;

Signal.filter_label.train.use_label = 'label';
Signal.filter_label.test.use_label = 'label';


Signal.filter_label.train.use_decoys = 1;
Signal.filter_label.test.use_decoys = 1;


Signal.filter_label.train.conf_pos.est_clusters = [0 INF];
Signal.filter_label.train.conf_pos.cDNA = [0 INF];
Signal.filter_label.train.conf_pos.fulllength = [0 INF];
Signal.filter_label.train.conf_pos.anno = [0:1] ;

Signal.filter_label.train.conf_pos_skip.est_clusters = [0 INF];
Signal.filter_label.train.conf_pos_skip.cDNA = [0 INF];
Signal.filter_label.train.conf_pos_skip.fulllength = [0 INF];
Signal.filter_label.train.conf_pos_skip.anno = [-inf] ;

Signal.filter_label.train.conf_neg.est_clusters = [0 INF];
Signal.filter_label.train.conf_neg.cDNA = [0 INF];
Signal.filter_label.train.conf_neg.fulllength = [0 INF];
Signal.filter_label.train.conf_neg.anno = [-1:1] ;

Signal.filter_label.train.conf_neg_skip.est_clusters = [0 INF];
Signal.filter_label.train.conf_neg_skip.cDNA = [0 INF];
Signal.filter_label.train.conf_neg_skip.fulllength = [0 INF];
Signal.filter_label.train.conf_neg_skip.anno = [-1:1] ;

Signal.filter_label.test = Signal.filter_label.train;
Signal.filter_label.eval = Signal.filter_label.train;

% Signal.save_confs = 1;
% Signal.save_alts = 1;


Signal.export_settings.conf_cum_thresh = 0.5 ;
Signal.export_settings.resolution = 1 ;


Signal.Conf_names = {Signal.Conf_names{:},'alt_exon','alt_intron'};





return

function Signal = tis(label_source)
  
% Signals = tis(Signals,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field tis is modified.
%        - label_source is a struct that should contain a field tis. 
%          this is used in the called function signal_template.
%       
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tis/cdsStop -> translatends_template -> signal_template

    
signal_name = 'tis';

Signal = translatends_template(label_source,signal_name);

%%%% SET TIS STUFF

Signal.consensus = {'ATG'};


Signal.kernel_name= {'SP','WDS','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};
Signal.lwin = {[-200] [-30] [0]} ; 
Signal.rwin = {[0] [110] [180]}; 
Signal.shift={[],[0],[]};
Signal.shift_const={[],[0],[]};
Signal.order={[],[24],[]};  
Signal.wordlen = {[3] [] [5]}; 
Signal.C = 1; % log2(C)=1.0










return

function Signal = transacceptor(label_source)

% Signals = transacceptor(Signals,label_source)
% INPUT  - Signals is a struct with a field for each Signal. In this function only
%          the field transcc is modified.  
%        - label_source is a struct that should contain a field
%          transacc. this is used in the called function splicesite_template.
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tansacceptor -> acceptor -> splicesite_template -> signal_template

INF = 1e6;  
signal_name = 'transacc';

Signal = acceptor(label_source,signal_name);

%%%% SET transacc STUFF

Signal.lwin = {[-190]} ;
Signal.rwin = {[80]};


Signal.filter_label.train.conf.anno_SL1 = 0;
Signal.filter_label.train.conf.anno_SL2 = [0 INF];
Signal.filter_label.train.conf.pred = [0 INF];

Signal.filter_label.eval.conf.anno_SL1 = 0;
Signal.filter_label.eval.conf.anno_SL2 = [0 INF];
Signal.filter_label.eval.conf.pred = [0 INF];


Signal.filter_label.test.conf.anno_SL1 = 0;
Signal.filter_label.test.conf.anno_SL2 = [0 INF];
Signal.filter_label.test.conf.pred = [0 INF];


Signal.filter_label.train.use_intergenic = 1;
Signal.filter_label.train.use_alt = 0;
Signal.filter_label.eval.use_intergenic = 1;
Signal.filter_label.eval.use_alt = 0;
Signal.filter_label.test.use_intergenic = 1;
Signal.filter_label.test.use_alt = 0;
 
Signal.filter_label.train.use_pred = 1;
Signal.filter_label.eval.use_pred = 1;
Signal.filter_label.test.use_pred = 1;

Signal.filter_label.train.only_acc = 1;
Signal.filter_label.eval.only_acc = 1;
Signal.filter_label.test.only_acc = 1;

Signal.filter_label.train.pos_dist = 20;
Signal.filter_label.eval.pos_dist = 20;
Signal.filter_label.test.pos_dist = 20;

Signal.export_settings.conf_cum_thresh = 0.05 ;
Signal.export_settings.resolution = 1 ;

Signal.Conf_names = {Signal.Conf_names{:},'isacc','has_tss','has_trans','dist'};


Signal.label_source_par.transacc.win_anno_SL1 = 50;
Signal.label_source_par.transacc.win_anno_SL2 = 50;
Signal.label_source_par.transacc.win_pred = 5;
Signal.label_source_par.transacc.threshold_pred = 0.5;


return

function Signal = transcriptends_template(label_source,signal_name)

% Signals = transcriptends_template(Signals,label_source,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field signal_name is added or
%          modified. 
%          Default values are set.
%        - label_source is a struct that should contain a field (signal_name). 
%          this is used in the called function signal_template.
%        - signal_name:'tss', 'cleave'
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tss/cleave -> transcriptends_template -> signal_template
%
% see also splicesite_template/translat_template
  
  
  
Signal = signal_template(label_source,signal_name);

%%%% SET TRANSCRIPT END STUFF

Signal.type = 'transcriptends'; 
Signal.label_fct = 'get_cand_others';


Signal.consensus = [];
Signal.offset = 0;

Signal.sampling.downsample_neg = 1/10; %% take randomly every 10th porsitions 
Signal.sampling.mindist = 10 ;


Signal.resolution   = 20; % for lsl

Signal.filter_label.train.conf_pos.anno = [0 1];
Signal.filter_label.train.conf_pos.est_clusters = [2:3];
Signal.filter_label.train.conf_pos.db = 0;
Signal.filter_label.train.conf_pos.fulllength = 0;
Signal.filter_label.train.onlywithpos=0; 

Signal.filter_label.train.use_intergenic=1;
Signal.filter_label.train.use_alt=0;

Signal.filter_label.test = Signal.filter_label.train;
Signal.filter_label.eval = Signal.filter_label.train;

Signal.export_settings.conf_cum_thresh = 0.01 ;
Signal.export_settings.resolution = 10 ;

Signal.Conf_names = {Signal.Conf_names{:},['has_' signal_name]};



return

function Signal = translatends_template(label_source,signal_name)

% Signals = translatends_template(Signals,label_source,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field signal_name is added or
%          modified. 
%          Default values are set.
%        - label_source is a struct that should contain a field (signal_name). 
%          this is used in the called function signal_template.
%        - signal_name:'tis', 'cdsStop'
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tis/cdsStop -> translatends_template -> signal_template
%
% see also splicesite_template/transcriptends_template
  
INF=1e6;
  
Signal = signal_template(label_source,signal_name);

%%%% SET TRANSLAT END STUFF

Signal.type = 'translatends' ;
Signal.label_fct = 'get_cand_others';

Signal.offset = 0;
Signal.lwin_big = 210;
Signal.rwin_big = 200;



Signal.filter_label.train.conf_pos.anno = [0 1] ;
Signal.filter_label.train.conf_pos.maxorf = INF;
Signal.filter_label.test.conf_pos.anno = [0 1] ;
Signal.filter_label.test.conf_pos.maxorf =INF;

Signal.filter_label.train.use_intergenic=0;
Signal.filter_label.train.use_alt=0;
Signal.filter_label.test.use_intergenic=0;
Signal.filter_label.test.use_alt=0;

Signal.filter_label.eval = Signal.filter_label.test;

% PAR.label_source.cds.par.max_nof_paths = 100;
% PAR.label_source.cds.par.nof_best_kept = 2;


return

function Signal = tss(label_source,has_transacc)
  
% Signals = tss(Signals,label_source,has_transacc)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field tss is modified.
%        - label_source is a struct that should contain a field tss. 
%          this is used in the called function signal_template.
%        - has_transacc: is per default set to zero, only 1 for nematodes. 
%       
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tss/cleave -> transcriptends_template -> signal_template

  
if nargin<3
  has_transacc=0; 
end
   
signal_name = 'tss';

Signal = transcriptends_template(label_source,signal_name);

%%%% SET TSS STUFF


Signal.lwin_big = 601;
Signal.rwin_big = 901;

Signal.kernel_name= {'SP','WDS','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};

Signal.lwin = { [-600] [-70] [   0] };
Signal.rwin = { [+100] [+70] [+900] };
Signal.wordlen = { [4] [] [4] };

Signal.order ={[] [16] []};
Signal.shift ={[] [0] []};
Signal.shift_const ={[] [28] []};
Signal.C = 2; % log2(C)=1.0

if has_transacc
  Signal.Conf_names = {Signal.Conf_names{:},'has_trans'};
  Signal.filter_label.train.rm_trans = 1;
  Signal.filter_label.eval.rm_trans = 1;
  Signal.filter_label.test.rm_trans = 1;
else
  Signal.filter_label.train.rm_trans = 0;
  Signal.filter_label.eval.rm_trans = 0;
  Signal.filter_label.test.rm_trans = 0;
end

%%%%%% for human
%Signal.sampling.downsample_neg = 1/160; % <=1 (for no downsampling: 1)



return

function diff_fields = compare_PAR(PAR_1,PAR_2,silent)

if nargin<3
  silent=0;
end

diff_fields ={};
if isequal(PAR_1,PAR_2)
  fprintf(1,'\t parameters identical\n')
else
  fields_1 = fieldnames(PAR_1);
  fields_2 = fieldnames(PAR_2);
  common_fields = intersect(fields_1,fields_2);
  for i=1:length(common_fields)
    par_1 = getfield(PAR_1,common_fields{i});
    par_2 = getfield(PAR_2,common_fields{i});
    if ~isequal(par_1,par_2)
      diff_fields{end+1} = common_fields{i};
      if ~silent
        fprintf(1,'\t field "%s" not identical\n',common_fields{i})
        fprintf(1,'\t %s, 1:\n',common_fields{i});disp(par_1)
        fprintf(1,'\t %s, 2:\n',common_fields{i});disp(par_2)
      end
    end
  end
  fields_x1= setdiff(fields_1,fields_2);
  if ~isempty(fields_x1)
    if ~silent
      fprintf(1,'\t additional fields in PAR_1: \n')
    end
    diff_fields = {diff_fields{:},fields_x1{:}};
  end
  fields_x2 = setdiff(fields_2,fields_1);
  if ~isempty(fields_x2)
    if ~silent
      fprintf(1,'\t additional fields in PAR_2: \n')
    end
    diff_fields = {diff_fields{:},fields_x2{:}};
  end
end
   
return

function [regions,N]=get_connected_regions(bw);
  
 % similar to bw_label just for 1xm matrix 

reg_starts = find(bw(1:end)-[0 bw(1:end-1)]==1);
reg_ends = find(bw(1:end)-[bw(2:end) 0]==1);
assert(length(reg_starts)==length(reg_ends))

N = length(reg_starts);

regions = zeros(size(bw));
for i=1:N
  assert(reg_starts(i)<=reg_ends(i))
  regions(reg_starts(i):reg_ends(i))=i;
end
return

function new_field = set_default(old_struct,field,defaults,silent)  ;

% new_struct = set_default(old_struct,defaults,overwrite)  ;
% sets all values of all fields in old_struct to the default values specified in
% defaults IF field is empty in old_struct. If overwrite is set all entries
% will be overwritten.
% defaults must be a struct.
%
% e.g. 
% default.use_cons = 0;
% default.plif.bins = 50;
% method.use_cons = 0;
% method = set_default(method,default,1) 

if ~isfield(old_struct,field)
  new_field = defaults;
  return
end  

old_struct = old_struct.(field);

if nargin<4
  silent = 1 ;
end

names = fieldnames(defaults);
for j=1:length(names)
  if ~isfield(old_struct,names{j}) || isempty(old_struct.(names{j}))||...
        (iscell(old_struct.(names{j}))&&length(old_struct.(names{j}))==1&&isempty(old_struct.(names{j}){1}))
    if ~isfield(old_struct,names{j}) && ~silent
      warning('new field added')
      names{j}
    end
    
    old_struct.(names{j}) = defaults.(names{j});
  elseif isstruct(defaults.(names{j}))
    old_struct.(names{j}) = set_default(old_struct,names{j},defaults.(names{j}));
    %names2 =  fieldnames(defaults.(names{j}));
    %for k= 1:length(names2)
    %  if ~isfield(old_struct.(names{j}),names2{k})||isempty(old_struct.(names{j}).(names2{k}))||overwrite
    %    old_struct.(names{j}).(names2{k}) = defaults.(names{j}).(names2{k});
    %  end
    %end
  end
end
% if length(fieldnames(PAR))>length(names)
%   keyboard
% end
% PAR = orderfields(PAR,defaults);

new_field = orderfields(old_struct,defaults);

return

function genes = build_splice_graph_caller(genes) ;
% genes = build_splice_graph_caller(genes) ;
  
for gene_idx = 1:length(genes)
  % construct a new empty splice graph, based on
  % http://proline.bic.nus.edu.sg/dedb/
  vertices =  [];
  edges = [];
%  fprintf(1,'gene index:\t%d\n',gene_idx);
  if (mod(gene_idx,100)==0)
    fprintf(1,'.');
  end

  for transcript_idx = 1:length(genes(gene_idx).transcripts)
    exon_start_end = genes(gene_idx).exons{transcript_idx};
    
    if (size(exon_start_end,1) == 1)
      exon1_start = exon_start_end(1,1);
      exon1_end = exon_start_end(1,2);

      
      if isempty(vertices)
	vertices(1,1) = exon1_start;
	vertices(2,1) = exon1_end;
	edges = 0;
	num_exons = 1;
      else
	vertices(1,num_exons+1) = exon1_start;
	vertices(2,num_exons+1) = exon1_end;
	edges(1,num_exons+1) = 0;
	edges(num_exons+1,1) = 0;
	num_exons = num_exons + 1;
      end

      
    else
      

      for exon_idx = 1:(size(exon_start_end,1)-1)
	exon1_start = exon_start_end(exon_idx,1);
	exon1_end = exon_start_end(exon_idx,2);
	exon2_start = exon_start_end(exon_idx+1,1);
	exon2_end = exon_start_end(exon_idx+1,2);


	
	if isempty(vertices)
	  vertices(1,1) = exon1_start;
	  vertices(2,1) = exon1_end;
	  vertices(1,2) = exon2_start;
	  vertices(2,2) = exon2_end;
	  edges = zeros(2);
	  edges(1,2) = 1;
	  edges(2,1) = 1;
	  num_exons = 2;
	else
	  exon1_idx = 0;
	  exon2_idx = 0;
	  idx = 1;
	  while idx <= num_exons
%	    if ((vertices(1,idx)==exon1_start)||...
%		(vertices(1,idx)+1==exon1_start)||...
%		(vertices(1,idx)-1==exon1_start))...
%		  && ((vertices(2,idx)==exon1_end)||...
%		      (vertices(2,idx)+1==exon1_end)||...
%		      (vertices(2,idx)-1==exon1_end))
	    if ((vertices(1,idx)==exon1_start) &&...
		(vertices(2,idx)==exon1_end))
	      exon1_idx = idx;
	    end
	    idx = idx+1;
	  end
	  idx = 1;
	  while idx <= num_exons
%	    if ((vertices(1,idx)==exon2_start)||...
%		(vertices(1,idx)+1==exon2_start)||...
%		(vertices(1,idx)-1==exon2_start))...
%		  && ((vertices(2,idx)==exon2_end)||...
%		      (vertices(2,idx)+1==exon2_end)||...
%		      (vertices(2,idx)-1==exon2_end))
	    if ((vertices(1,idx)==exon2_start) &&... 
		(vertices(2,idx)==exon2_end))
	      exon2_idx = idx;
	    end
	    idx = idx+1;
	  end
	  if (exon1_idx~=0) && (exon2_idx~=0)
	    edges(exon1_idx,exon2_idx) = 1;
	    edges(exon2_idx,exon1_idx) = 1;
	  else
	    if ((exon1_idx==0) && (exon2_idx~=0))
	      vertices(1,num_exons+1) = exon1_start;
	      vertices(2,num_exons+1) = exon1_end;
	      edges(exon2_idx,num_exons+1) = 1;
	      edges(num_exons+1,exon2_idx) = 1;
	      num_exons = num_exons + 1;
	    elseif ((exon2_idx==0) && (exon1_idx~=0))
	      vertices(1,num_exons+1) = exon2_start;
	      vertices(2,num_exons+1) = exon2_end;
	      edges(exon1_idx,num_exons+1) = 1;
	      edges(num_exons+1,exon1_idx) = 1;
	      num_exons = num_exons + 1;
	    else
	      assert((exon1_idx==0)&&(exon2_idx==0));
	      vertices(1,num_exons+1) = exon1_start;
	      vertices(2,num_exons+1) = exon1_end;
	      num_exons = num_exons + 1;
	      vertices(1,num_exons+1) = exon2_start;
	      vertices(2,num_exons+1) = exon2_end;
	      num_exons = num_exons + 1;	  
	      
	      edges(num_exons-1,num_exons) = 1;
	      edges(num_exons,num_exons-1) = 1;
	    end
	  end
	end
      end
    end
  end




  genes(gene_idx).splicegraph = {vertices,edges};
end


fprintf(1,'\n');


return






return

function [regions,level] = detectaltregions(gene)
% regions = detectaltregions(gene)
  
[max_exon_level,max_intron_level,exon_level,intron_level,level] = ...
    detectsplicegraph(gene) ;

if all(level(3,:)<=1), 
  regions=[] ;
  return ;
else
  regions = [] ;
  idx=find(level(3,:)>1) ;
  idx=unique([idx(1) find(level(3,2:end)>1 & level(3,1:end-1)<=1)+1]);

  
  for i=1:length(idx)
    %level(4,idx(i))
    %if level(4,idx(i))==0, keyboard; end ;

  %    if idx(i)==1
      start = level(1,idx(i)) ;
%    else
%      start = floor(mean([level(1,idx(i)-1) level(1,idx(i))])) ;
%    end ;
%    if idx(i)==size(level,2)
      stop = level(2,idx(i)) ;
%    else
%      stop = floor(mean([level(2,idx(i)) level(2,idx(i)+1)]))-1 ;
%    end ;
    depth = level(3,idx(i)) ;
    for j=idx(i):size(level,2),
      if level(3,j)>1 
%        if j==size(level,2)
          stop = level(2,j) ;
%        else
%          stop = floor(mean([level(2,j) level(2,j+1)]))-1 ;
%        end ;
        depth = max(depth, level(3,j)) ;
      else
        %if level(4,j)==0, keyboard; end ;
        break ;
      end ;
    end ;
    regions=[regions, [start; stop; depth]] ;
  end ;
end ;
return

function genes= alt_const(genes) ;
%function genes = alt_const(genes) ;

tot_exons = 0;
for ix=1:length(genes)
  if (mod(ix,1000)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2) ;
  tot_exons = tot_exons + num_exons ;
  vertices = genes(ix).splicegraph{1} ;
  edges = genes(ix).splicegraph{2} ;

  if isempty(edges)
    genes(ix).is_alt_spliced=0 ;
    genes(ix).is_alt=0 ;
    continue ;
  end 
  
  init_alt_idx = [] ;
  term_alt_idx = [] ;
  for i=1:num_exons,
    if ~any(edges(1:i-1,i)) % initial
      idx=setdiff(find(vertices(2,i)==vertices(2,:)),i) ;
      if ~isempty(idx) % other exon with same end
        is_simple=1 ; 
        for j=idx,
          if ~isequal(find(edges(j,j+1:end))+j,find(edges(i,i+1:end))+i)
            is_simple = 0 ;
            break ;
          end ;
          idx_prev = find(edges(1:j-1,j)) ;
          for k=idx_prev,
            if vertices(2,k)>vertices(1,i)
              is_simple = 0 ;
              break ;
            end ;
          end ;
        end ;
        if ~is_simple, continue ; end ;
        init_alt_idx(end+1) = i ;
      end
    end ;
    if ~any(edges(i,i+1:num_exons)) % terminal
      idx=setdiff(find(vertices(1,i)==vertices(1,:)),i) ;
      if ~isempty(idx) % other exon with same start
        is_simple=1 ; 
        for j=idx,
          if ~isequal(find(edges(1:j-1,j)),find(edges(1:i-1,i)))
            is_simple = 0 ;
            break ;
          end ;
          idx_next = find(edges(j,j+1:end))+j ;
          for k=idx_next,
            if vertices(1,k)<vertices(2,i)
              is_simple = 0 ;
              break ;
            end ;
          end ;
        end ;
        if ~is_simple, continue ; end ;
        term_alt_idx(end+1) = i ;
      end ;
    end ;
  end ;
  take_idx = setdiff(1:num_exons, [init_alt_idx,term_alt_idx]) ;
  vertices = genes(ix).splicegraph{1}(:,take_idx) ;
  edges = genes(ix).splicegraph{2}(take_idx,take_idx) ;

  start=min(vertices(1,:)-50) ;
  stop=max(vertices(2,:)+50) ;
  
  exon_loc = zeros(1,stop-start);

  for i=1:size(edges,1),
    for j=i+1:size(edges,1),
      if edges(i,j)==1,
        cur_edge = [vertices(2,i)+1-start, vertices(1,j)-1-start] ;
        exon_loc(cur_edge(1):cur_edge(2)) = ...
            exon_loc(cur_edge(1):cur_edge(2)) + 1;
      end ;
    end ;
  end ;

  for i=1:size(vertices,2)
    cur_vertex = vertices(:,i) - [start;start];
    exon_loc(cur_vertex(1):cur_vertex(2)) = ...
        exon_loc(cur_vertex(1):cur_vertex(2)) + 1;
  end ;

  if max(exon_loc)>1
    genes(ix).is_alt_spliced=1 ;
    genes(ix).is_alt=1 ;
  else
    genes(ix).is_alt_spliced=0 ;
    if isempty([init_alt_idx,term_alt_idx])
      genes(ix).is_alt=0 ;
    else
      genes(ix).is_alt=1 ;
    end ;
  end
end

fprintf(1,'\n\nTotal genes:\t\t\t\t\t\t\t%d\n',...
        length(genes));
fprintf(1,'Total exons:\t\t\t\t\t\t\t%d\n',...
        tot_exons);
fprintf(1,'Total genes with alternative isoforms:\t\t\t\t%d\n',...
        sum([genes(:).is_alt]));
fprintf(1,'Total genes alternatively spliced:\t\t\t\t%d\n',...
        sum([genes(:).is_alt_spliced]));
fprintf(1,'Total constitutively spliced:\t\t\t\t\t%d\n',...
        sum(~[genes(:).is_alt_spliced]));


return

function genes = annotate_splicegraph(genes)
% function annotate_splicegraph(info,origpath)
%
% annotate the genes structure with counts for:
% - exon skipping
% - intron retention
% - alternative donor (5 prime)
% - alternative acceptor (3 prime)
% - alternative transcription start
% - alternative transcription end
%
% Written by: Cheng Soon Ong, 14 May 2007
% Modified on: 31 May 2007 to include transcription starts and stops


%addpath(sprintf('%s/utils', origpath)) ;
%addpath(sprintf('%s/splicegraphs', origpath)) ;
%addpath(sprintf('%s/splicegraphs/detect_altsplice', origpath)) ;
%addpath(sprintf('%s/gff', origpath)) ;
%addpath(sprintf('%s/gff/read_sequences', origpath)) ;
%addpath /fml/ag-raetsch/share/software/matlab_tools/utils

%genome_info = init_genome(info) ;
%fprintf('directory: %s\n', genome_info.basedir) ;
%load(sprintf('%s/confirmed_sequences.mat', genome_info.basedir), 'genes') ;
%fprintf('loaded confirmed_sequences.mat\n\n') ;

if ~isfield(genes, 'strands'),
  for i=1:length(genes),
    genes(i).strands = repmat(genes(i).strand, 1, length(genes(i).transcripts)) ;
  end ;
end ;

if isfield(genes,'alt_3prime_details') || isfield(genes,'alt_5prime_details')
  warning('annotate_splicegraph adds to alt_3prime_details and alt_5prime_details,\nrunning more than one creates inconsistencies');
  return;
end

idx_alt=[] ;
for i=1:length(genes)
  if genes(i).is_alt_spliced
    idx_alt=[idx_alt, i] ;
  end
  if ~isfield(genes(i),'strands') || isempty(genes(i).strands)
    genes(i).strands(1) = genes(i).strand;
  end
end

[idx_exon_skips, exon_exon_skips] = detect_exonskips(genes, idx_alt);
[idx_alt_5prime,exon_alt_5prime, idx_alt_3prime,exon_alt_3prime] = ...
    detect_altprime_pair(genes, idx_alt);
[idx_intron_reten,intron_intron_reten] = detect_intronreten(genes, idx_alt) ;
[idx_xor_exons, exon_xor_exons] = detect_xorexons(genes, idx_alt) ;
[idx_alt_tstart, exon_alt_tstart] = detect_alttstart(genes) ;
idx_alt_tstart = [idx_alt_tstart{:}];
[idx_alt_tend, exon_alt_tend] = detect_alttend(genes) ;
idx_alt_tend = [idx_alt_tend{:}];

%keyboard
for ix = 1:length(genes)
    
  if mod(ix,1000)==0
    fprintf('.');
  end

  exons_is_alt = zeros(1,size(genes(ix).splicegraph{1},2));

  % exon skips
  if ismember(ix,idx_exon_skips)
    tix = find(idx_exon_skips==ix);
    genes(ix).alt_exon = length(tix);
    genes(ix).alt_exon_details.exons = exon_exon_skips(:,tix);
    exons_is_alt(exon_exon_skips(:,tix)) = exons_is_alt(exon_exon_skips(:,tix)) + 1;
  else
    genes(ix).alt_exon = 0;
  end

  % intron retentions
  if ismember(ix,idx_intron_reten)
    tix = find(idx_intron_reten==ix);
    genes(ix).alt_intron = length(tix);
    genes(ix).alt_intron_details.exons = intron_intron_reten(:,tix);
    exons_is_alt(intron_intron_reten(:,tix)) = exons_is_alt(intron_intron_reten(:,tix)) + 1;
  else
    genes(ix).alt_intron = 0;
  end

  % alternative 5prime
  if ismember(ix,idx_alt_5prime)
    genes(ix).alt_5prime_details.exons = zeros(3,0);
    tix = find(idx_alt_5prime==ix);
    genes(ix).alt_5prime = length(tix);
    for ixt = tix
      if genes(ix).strands(1) == '+'
	idx_exon_l = exon_alt_5prime(ixt).fiveprimesites(1);
	idx_exon_r = exon_alt_5prime(ixt).threeprimesite;
	idx_exon_alt = exon_alt_5prime(ixt).fiveprimesites(2);
      else
	idx_exon_r = exon_alt_5prime(ixt).fiveprimesites(1);
	idx_exon_l = exon_alt_5prime(ixt).threeprimesite;
	idx_exon_alt = exon_alt_5prime(ixt).fiveprimesites(2);
      end
      genes(ix).alt_5prime_details.exons(:,end+1) = [idx_exon_l;idx_exon_r;idx_exon_alt];
      exons_is_alt([idx_exon_l;idx_exon_r;idx_exon_alt]) = exons_is_alt([idx_exon_l;idx_exon_r;idx_exon_alt]) + 1;
    end
  else
    genes(ix).alt_5prime = 0;
  end

  % alternative 3prime
  if ismember(ix,idx_alt_3prime)
    genes(ix).alt_3prime_details.exons = zeros(3,0);
    tix = find(idx_alt_3prime==ix);
    genes(ix).alt_3prime = length(tix);
    for ixt = tix
      if genes(ix).strands(1) == '+'
	idx_exon_l = exon_alt_3prime(ixt).fiveprimesite;
	idx_exon_r = exon_alt_3prime(ixt).threeprimesites(1);
	idx_exon_alt = exon_alt_3prime(ixt).threeprimesites(2);
      else
	idx_exon_r = exon_alt_3prime(ixt).fiveprimesite;
	idx_exon_l = exon_alt_3prime(ixt).threeprimesites(1);
	idx_exon_alt = exon_alt_3prime(ixt).threeprimesites(2);
      end
      genes(ix).alt_3prime_details.exons(:,end+1) = [idx_exon_l;idx_exon_r;idx_exon_alt];
      exons_is_alt([idx_exon_l;idx_exon_r;idx_exon_alt]) = exons_is_alt([idx_exon_l;idx_exon_r;idx_exon_alt]) + 1;
    end
  else
    genes(ix).alt_3prime = 0;
  end

  % xor exons
  if ismember(ix,idx_xor_exons)
    tix = find(idx_xor_exons==ix);
    genes(ix).alt_xor_exon = length(tix);
    for ixt = tix
      exons_is_alt(exon_xor_exons(:,tix)) = exons_is_alt(exon_xor_exons(:,tix)) + 1;
    end
  else
    genes(ix).alt_xor_exon = 0;
  end

  % alternative transcriptions
  if ismember(ix,idx_alt_tstart)
    tix = find([idx_alt_tstart == ix]);
    % there is only one entry per gene.
    assert(all(size(tix) == [1,1]))
    genes(ix).alt_tstart = length(exon_alt_tstart{tix}.exons);
    genes(ix).alt_tstart_details.exons = exon_alt_tstart{tix}.exons;
  else
    genes(ix).alt_tstart = 0;
  end

  if ismember(ix,idx_alt_tend)
    tix = find([idx_alt_tend == ix]);
    % there is only one entry per gene.
    assert(all(size(tix) == [1,1]))
    genes(ix).alt_tend = length(exon_alt_tend{tix}.exons);
    genes(ix).alt_tend_details.exons = exon_alt_tend{tix}.exons;
  else
    genes(ix).alt_tend = 0;
  end

  genes(ix).exons_is_alt = exons_is_alt;
end


if 0,
  fid = fopen(sprintf('%s/alt_splice_type.tsv', genome_info.basedir),'w');
  fprintf(fid,'Clust\tExon Skip\tIntron Reten\tAlt 5prime\tAlt 3prime\n');
  for ix = idx_alt  
    fprintf(fid,'%d\t%d\t%d\t%d\t%d\n',ix,...
	    genes(ix).alt_exon,genes(ix).alt_intron,genes(ix).alt_5prime,genes(ix).alt_3prime);
  end
  fclose(fid);
end

return

function [idx_alt_5prime,exon_alt_5prime, idx_alt_3prime,exon_alt_3prime] ...
    = detect_altprime_pair(genes, idx_alt);
% function [idx_alt_5prime,exon_alt_5prime, idx_alt_3prime,exon_alt_3prime] ...
%    = detect_altprime_pair(genes, idx_alt);
%
% Detect the alternative 5 and 3 prime ends of the intron. Note that 5 prime refers to the left
% and 3 prime to the right for a positive strand
% The difference between this function and detect_altprime.m is that the other function collects
% all the alternative sites corresponding to an acceptor or donor.
% note that the 'overlap' relationship is not transitive

MIN_OVERLAP = 11; % two exons that are alternative should overlap by at least the BLAT seed

idx_alt_5prime = [];
idx_alt_3prime = [];
exon_alt_5prime = {};
exon_alt_3prime = {};

if ~isfield(genes, 'strands'),
  for i=1:length(genes),
    genes(i).strands = repmat(genes(i).strand, 1, length(genes(i).transcripts)) ;
  end ;
end ;

for ix=idx_alt
  if (mod(ix,50)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2) ;
  vertices = genes(ix).splicegraph{1} ;
  edges = genes(ix).splicegraph{2} ;
  
  % Find alternative sites on the right of the intron,
  % same site on the left of the intron.
  for exon_idx = 1:num_exons-2
    nr_exons=sum(edges(exon_idx,exon_idx+1:num_exons));
    if nr_exons>=2,
      which_exons = find([zeros(1,exon_idx),edges(exon_idx,exon_idx+1:num_exons)]);
      exons=[vertices(:, which_exons)] ;
      for i=1:nr_exons-1
	for j=i+1:nr_exons
	  % check that the left splice site of the exons are different
	  % make sure exons overlap - either:
	  % - left splice site of exon(i) is in exon(j)
	  % - left splice site of exon(j) is in exon(i)
	  
	  if ((exons(1,i) ~= exons(1,j)) && ...
	       (( (exons(1,i) >= exons(1,j)) && (exons(1,i) <= exons(2,j)) ) ...
		|| ( (exons(1,j) >= exons(1,i)) && (exons(1,j) <= exons(2,i)) )) && ...
	       (min(exons(2,i),exons(2,j))-max(exons(1,i),exons(1,j)) >= MIN_OVERLAP ) )
	    assert(~((exons(1,i) == exons(1,j)) && (exons(2,i) == exons(2,j))));
	    assert(exons(1,i) ~= exons(1,j));
	    % construct the output
	    if (genes(ix).strands(1)=='+'),
	      exon_alt_3prime(end+1).fiveprimesite=exon_idx ;
	      if exons(1,i) > exons(1,j)
		exon_alt_3prime(end).threeprimesites=which_exons([i j]);
	      else
		exon_alt_3prime(end).threeprimesites=which_exons([j i]);
	      end
	      idx_alt_3prime(end+1) = ix;
	    end
	    if (genes(ix).strands(1)=='-'),
	      exon_alt_5prime(end+1).threeprimesite=exon_idx ;
	      if exons(1,i) > exons(1,j)
		exon_alt_5prime(end).fiveprimesites=which_exons([i j]);
	      else
		exon_alt_5prime(end).fiveprimesites=which_exons([j i]);
	      end
	      idx_alt_5prime(end+1) = ix;
	    end
	  end % if altsplice
	  
	end % for j=i+1:nr_exons
      end % i=1:nr_exons-1
    end
  end % for exon_idx

  
  % Find alternative sites on the left of the intron,
  % same site on the right of the intron.
  for exon_idx = 3:num_exons
    nr_exons=sum(edges(1:exon_idx-1,exon_idx)) ;
    if nr_exons>=2,
      which_exons = find([edges(1:exon_idx-1,exon_idx)', zeros(1,num_exons-exon_idx+1)]);
      exons=[vertices(:, which_exons)] ;
      for i=1:nr_exons-1
	for j=i+1:nr_exons
	  % check that the 5prime sites are different
	  % make sure exons overlap - either:
	  % - right splice site of exon(i) is in exon(j)
	  % - right splice site of exon(j) is in exon(i)
	  if ((exons(2,i) ~= exons(2,j)) &&...
	      (( (exons(2,i) <= exons(2,j)) && (exons(2,i) >= exons(1,j)) )...
	      || ( (exons(2,j) <= exons(2,i)) && (exons(2,j) >= exons(1,i)) )) && ...
	      (min(exons(2,i),exons(2,j))-max(exons(1,i),exons(1,j)) >= MIN_OVERLAP ) )
	    assert(~((exons(1,i) == exons(1,j)) && (exons(2,i) == exons(2,j))));
	    assert(exons(2,i) ~= exons(2,j));

	    % construct the output
	    if (genes(ix).strands(1)=='+'),
	      exon_alt_5prime(end+1).threeprimesite=exon_idx ;
	      if exons(2,i) < exons(2,j)
		exon_alt_5prime(end).fiveprimesites=which_exons([i j]);
	      else
		exon_alt_5prime(end).fiveprimesites=which_exons([j i]);
	      end
	      idx_alt_5prime(end+1) = ix;
	    end
	    if (genes(ix).strands(1)=='-'),
	      exon_alt_3prime(end+1).fiveprimesite=exon_idx ;
	      if exons(2,i) < exons(2,j)
		exon_alt_3prime(end).threeprimesites=which_exons([i j]);
	      else
		exon_alt_3prime(end).threeprimesites=which_exons([j i]);
	      end
	      idx_alt_3prime(end+1) = ix;
	    end
	  end % if altsplice
	  
	end % for j=i+1:nr_exons
      end % for i=1:nr_exons-1
    end

  end % construct output
end % for exon_idx
  



fprintf(1,'\n\nNumber of alternative 5 prime sites:\t\t\t\t%d\n',...
	length(idx_alt_5prime));

fprintf(1,'Number of alternative 3 prime sites:\t\t\t\t%d\n',...
	length(idx_alt_3prime));






return

function [idx_alt_tend, exon_alt_tend] = detect_alttend(genes) ;

idx_alt_tend = {};
exon_alt_tend = {};

for ix=1:length(genes)
  if (mod(ix,1000)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2) ;
  vertices = genes(ix).splicegraph{1} ;
  edges = triu(genes(ix).splicegraph{2}) ;
  
  term_alt_idx = [] ;
  for i=1:num_exons,
    if sum(edges(i,:),2)==0,
      term_alt_idx(end+1) = i ;
    end ;
  end ;
  if length(term_alt_idx)>1
    idx_alt_tend{end+1} = ix;
    exon_alt_tend{end+1}.exons = term_alt_idx;
  end
end

fprintf(1,'\nTotal alternative transcription ends:\t\t\t\t%d\n',...
        length(idx_alt_tend));

return

function [idx_alt_tstart, exon_alt_tstart] = detect_alttstart(genes) ;

idx_alt_tstart = {};
exon_alt_tstart = {};

for ix=1:length(genes)
  if (mod(ix,1000)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2) ;
  vertices = genes(ix).splicegraph{1} ;
  edges = triu(genes(ix).splicegraph{2}) ;
  
  init_alt_idx = [] ;
  for i=1:num_exons,
    if sum(edges(:,i),1)==0, % initial
      init_alt_idx(end+1) = i ;
    end ;
  end ;

  if length(init_alt_idx)>1,
    idx_alt_tstart{end+1} = ix;
    exon_alt_tstart{end+1}.exons = init_alt_idx;
  end
end

fprintf(1,'\nTotal alternative transcription starts:\t\t\t\t%d\n',...
        length(idx_alt_tstart));

return

function [idx_exon_skips, exon_exon_skips] = detect_exonskips(genes, idx_alt) ;
% [idx_exon_skips, exon_exon_skips] = detect_exonskips(genes, idx_alt) ;

idx_exon_skips = [];
exon_exon_skips = [];
for ix=idx_alt
  if (mod(ix,50)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2);
  edges = genes(ix).splicegraph{2};
  for exon_idx = 1:num_exons-2
    for exon_idx1 = (exon_idx+1):(num_exons-1)
      for exon_idx2 = (exon_idx1+1):num_exons
        if (edges(exon_idx,exon_idx1))&&...
              (edges(exon_idx,exon_idx2))&&...
              (edges(exon_idx1,exon_idx2))
          idx_exon_skips = [idx_exon_skips,ix];
          exon_exon_skips = [exon_exon_skips,[exon_idx; exon_idx1; exon_idx2]];
        end
      end
    end
  end
end
sum_skips=length(idx_exon_skips) ;
fprintf(1,'\n\nNumber of single exon skips:\t\t\t\t\t%d\n', sum_skips);



return

function [idx_intron_reten,intron_intron_reten] = detect_intronreten(genes,idx_alt) ;

idx_intron_reten = [];
intron_intron_reten = [];
for ix=idx_alt
  %viewsplicegraph(genes(ix)) ;
  if (mod(ix,50)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2);
  vertices = genes(ix).splicegraph{1};
  edges = genes(ix).splicegraph{2};
  introns=[] ;
  for exon_idx = 1:num_exons-1 %start of intron
    %exon_idx
    %edges(exon_idx,exon_idx+1:length(edges))
    idx=find(edges(exon_idx,exon_idx+1:length(edges))==1) ;
    if isempty(idx), continue; end
    idx=idx+exon_idx ;
    for exon_idx2=idx, %end of intron
      is_intron_reten=0 ;
      for exon_idx1=1:num_exons %exon
	% check that the exon covers the intron
        if (vertices(2,exon_idx) >= vertices(1,exon_idx1)) && (vertices(1,exon_idx2) <= vertices(2,exon_idx1))
	
	% check that the exon covers the intron, but doesn't extend beyond
        %if (vertices(2,exon_idx) > vertices(1,exon_idx1)) && (vertices(1,exon_idx2) < vertices(2,exon_idx1))...
	%      && (vertices(1,exon_idx) <= vertices(1,exon_idx1)) && (vertices(2,exon_idx2) >= vertices(2,exon_idx1))
          is_intron_reten=1 ;
          long_exon=exon_idx1 ;
          for len=1:size(introns,2),
            if (vertices(2,exon_idx)==introns(1,len)) && (vertices(1,exon_idx2)==introns(2,len))
              is_intron_reten=0 ;
            end  
          end
        end
      end
      if is_intron_reten==1,
        idx_intron_reten = [idx_intron_reten, ix];
        intron_intron_reten = [intron_intron_reten, [exon_idx ; exon_idx2 ...
                   ; long_exon]] ;
        introns=[introns, [vertices(2,exon_idx) ; vertices(1,exon_idx2)]] ;
      end
    end
    %pause ;
  end
  
  %pause ;
end
fprintf(1,'\n\nNumber of intron retentions:\t\t\t\t\t%d\n', ...
	length(idx_intron_reten));


return

function [idx_xor_exons, exon_xor_exons,id_xor_exons] = detect_xorexons(genes, idx_alt);

idx_xor_exons = zeros(1,0); 
id_xor_exons = zeros(1,0); 
exon_xor_exons = zeros(4,0); %%%% 5primesite of first exon, the 2 skipped
                     %exons, 3primesite of last exon %%%
id = 0 ;
for ix=idx_alt
  if (mod(ix,50)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2);
  edges = genes(ix).splicegraph{2};
  vertices = genes(ix).splicegraph{1};
  for exon_idx1 = 1:num_exons-3
    id=id+1 ;
    for exon_idx2 = (exon_idx1+1):(num_exons-2)
      if edges(exon_idx1, exon_idx2)
        for exon_idx3=(exon_idx2+1):(num_exons-1)
          if edges(exon_idx1, exon_idx3) & edges(exon_idx2, exon_idx3)==0 & ...
             (vertices(1,exon_idx3) > vertices(2,exon_idx2)),
            for exon_idx4=(exon_idx3+1):num_exons
              if edges(exon_idx2, exon_idx4) & edges(exon_idx3, exon_idx4) & ...
                    edges(exon_idx1, exon_idx4)==0
                idx_xor_exons = [idx_xor_exons, ix];
                id_xor_exons = [id_xor_exons, id];
                exon_xor_exons = [exon_xor_exons,[exon_idx1; exon_idx2; exon_idx3; exon_idx4]] ;
              end
            end
          end
        end
      end
    end
  end
end
fprintf(1,'\n\nNumber of XOR exons:\t\t\t\t\t\t%d\n',...
	length(idx_xor_exons));



return

function [max_exon_level,max_intron_level,exon_level,intron_level,level] = detectsplicegraph(gene)

  [max_exon_level,max_intron_level,exon_level,intron_level,level] = countlevel(gene.exons,gene.name,...
                                                    gene.transcripts,[], ...
                                                    gene.splicegraph{1},...
                                                    gene.splicegraph{2});
  return


function [max_level,max_leveli,exon_level,intron_level,level] = countlevel(truepos,name,labels,colors,...
                                                    vertices,edges)
  
  
  start=inf ;
  stop=-inf ;
  for k=1:length(truepos)
    start=min(start,truepos{k}(1,1)-50) ;
    stop=max(stop,truepos{k}(end,2)+50) ;
  end ;
  
  
  max_level = 0;
  max_leveli = 0;

  splice = unique(vertices(:)) ;
  
  exon_loc = zeros(1,stop-start);
  exon_level = zeros(3,size(vertices,2));
  for i=1:size(vertices,2)
    cur_vertex = vertices(:,i) - [start;start];
    exon_loc(cur_vertex(1):cur_vertex(2)-1) = ...
        exon_loc(cur_vertex(1):cur_vertex(2)-1) + 1;
    %exon_loc(cur_vertex(1):cur_vertex(2)-1) = ...
    %    max(exon_loc(cur_vertex(1):cur_vertex(2)-1));
    
    level = max(exon_loc(cur_vertex(1):cur_vertex(2)-1));
    if isempty(level), level=0 ; end ;
    max_level = max(level,max_level);
    exon_level(1,i) = vertices(1,i) ;  
    exon_level(2,i) = vertices(2,i) ;  
    exon_level(3,i) = level;  
  end
  
  intron_loc = zeros(1,stop-start);
  intron_level = zeros(3,0) ;
  if (size(edges,1) > 1)
    for i=1:size(vertices,2)
      for j=i+1:size(vertices,2)
        if edges(i,j)
          if (vertices(1,i) < vertices(1,j))
            istart = vertices(2,i);
            istop =  vertices(1,j);
            %level1 = exon_level(i);
            %level2 = exon_level(j);
          else
            istart = vertices(2,j);
            istop =  vertices(1,i);
            %level1 = exon_level(j);
            %level2 = exon_level(i);
          end
          
          cur_intron = [istart-start,istop-start];
          intron_loc(cur_intron(1):cur_intron(2)-1) = ...
              intron_loc(cur_intron(1):cur_intron(2)-1) + 1;
          intron_level(3,end+1) = max(intron_loc(cur_intron(1):cur_intron(2)-1));
          intron_level(1,end) = istart ;
          intron_level(2,end) = istop ;

          max_leveli = max(intron_level(3,end),max_leveli);
        end
      end
    end
  end
  
  splice_loc = exon_loc+intron_loc ;
  level=zeros(5,length(splice)-1) ;
  for i=1:length(splice)-1,
    level(:,i)=[splice(i);
                splice(i+1);
                max(splice_loc([splice(i)-start:splice(i+1)-start-1]));
                max(exon_loc([splice(i)-start:splice(i+1)-start-1]));
                max(intron_loc([splice(i)-start:splice(i+1)-start-1]));
               ] ;
  end ;

return


return

function genes = infer_splice_graph_caller(genes) ;
% genes = infer_splice_graph_caller(genes) ;
  
%infer_splice_graph

MISALIGN = 5;

fprintf(1,'Number of genes:%d\n',length(genes));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Some simple merging to reduce exons
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf(1,'Merging some genes\n');
genes = reduce_splice_graph_caller(genes) ;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% infer_splice_graph infers the relevant splice graph given a set of ESTs. 
%
% It is based on two assumptions, neither which are true in reality, but
% are necessary for any reasonable processing.
% 1. All splicing events are independent. That is a splice site
%    at location A in ESTA does not depend on splice site B in ESTA, and
%    neither does it depend on splice site C in ESTB.
% 2. There exists a splicing event at a particular location if and only if
%    there exists an EST that is spliced at this point.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


fprintf(1,'performing inference\n');

for gene_idx = 1:length(genes)

  if (mod(gene_idx,1000)==0)
    fprintf(1,'%d\n',gene_idx);
  end
  

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % For each gene, we consider each pair of exons in the splicegraph.
  % After sorting the exons by the location of the left side, we make
  % a local copy of the splicegraph.
  %
  % Note that 'left' means the 5 prime end of a positive strand, but
  % the 3 prime end of a negative strand.
  %
  % The two exons considered are indexed by exon_idx, and test_exon_idx.
  % The general idea of the algorithm is as follows:
  % For each pair:
  %   check which case it is in (more on this later)
  %   if it is possible to merge:
  %     form a new exon
  %   end merge.
  %   if new exon is not in current set
  %     add to splicegraph
  %   end new exon insert
  % end for
  % 
  % In words:
  % Each time two exons are considered, and they can be merged to form
  % a new exon, they are added to the list of exons. Then, each of the
  % original parent exons are considered for deletion (variable to_keep).
  % All the exons are kept till the very end, when all exons labelled
  % to_keep == 0 are removed.
  %
  % Hence the number of exons could potentially grow quadratically.
  % Since we have to also consider merging the pairs of exons which are
  % newly created, the number of pairs considered can be larger than
  % m^2 where m is the number of original exons. However, I believe that
  % the algorithm should terminate because there are only finitely many
  % splice sites and hence at worst, we have all possible left and right
  % exon ends used in our splicegraph.
  %
  % I am not sure what the best termination condition for the loops are.
  %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if isempty(genes(gene_idx).splicegraph{1}),
    continue ; 
  end ;
  [dummy,exon_order] = sort(genes(gene_idx).splicegraph{1}(1,:),2,'ascend');
  vertices = genes(gene_idx).splicegraph{1}(:,exon_order);
  edges = genes(gene_idx).splicegraph{2}(exon_order,exon_order);
  to_keep = ones(1,size(vertices,2));

  changed = 0;
  exon_idx = 1;
  first_merge_exon = 1;
  test_exon_idx = 0;
  while exon_idx <= size(vertices,2)

    if changed
      changed = 0;
      exon_idx = first_merge_exon;
      continue;
    end
    

    num_exons = size(vertices,2);
    [dummy,exon_order] = sort(vertices(1,:),2,'ascend');
    if ~isequal(vertices(1,:),dummy)
      vertices = vertices(:,exon_order);
      edges = edges(exon_order,exon_order);
      to_keep = to_keep(exon_order);
    end ;
    
    
    exon_take_idx = find(((vertices(1,exon_idx)-2*MISALIGN)<vertices(2,:))&...
			 ((vertices(2,exon_idx)+2*MISALIGN)>vertices(1,:)));
    if ~isempty(exon_take_idx)
      first_merge_exon = exon_take_idx(1);
    end
    exon_take_idx = exon_take_idx(exon_take_idx>exon_idx);
    internal_idx = 1;


    while ((internal_idx <= length(exon_take_idx)) && ~changed)
      test_exon_idx = exon_take_idx(internal_idx);
      if (test_exon_idx < exon_idx) && keyboard_allowed(), keyboard;end;
      cur_edge_left = sum(edges(1:exon_idx,exon_idx));
      test_edge_left = sum(edges(1:test_exon_idx,test_exon_idx));
      cur_edge_right = sum(edges(exon_idx:end,exon_idx));
      test_edge_right = sum(edges(test_exon_idx:end,test_exon_idx));
	  

      new_vertex = zeros(2,1);
	
      
      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      % In the following, the cases are labelled with a binary string
      % [cur_edge_left, cur_edge_right, test_edge_left, test_edge_right]
      %
      % cur refers to exon_idx
      % test refers to test_exon_idx
      % edge means introns.
      % e.g. cur_edge_left == 1 means that there exists an intron on
      % the left of exon_idx
      %
      % Hence there are 16 cases, and we treat each one individually.
      % For each case, the two logical tests are:
      % 1. should we merge?
      % 2. is the new exon known?
      %
      % Note that a splice site is defined by an intron-exon boundary.
      % Furthermore, we allow misalignment by MISALIGN nucleotides,
      % that is we shorten an exonAendA if it doesn't have a splice site
      % and there exists a splice site exonBendB within MISALIGN nucleotides.
      % 
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
      
      
      % 0000
      if (~cur_edge_left&&~cur_edge_right&&~test_edge_left&&~test_edge_right)

	if ((~isequal(vertices(1,exon_idx),vertices(1,test_exon_idx)))||...
	    (~isequal(vertices(2,exon_idx),vertices(2,test_exon_idx))))&&...
	      ((to_keep(test_exon_idx)~=0)||(to_keep(exon_idx)~=0))&&...
	      (vertices(2,exon_idx)>=vertices(1,test_exon_idx))
	  
	  
	  new_vertex(1) = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	  new_vertex(2) = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	  new_edges = zeros(1,num_exons);
	
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix))
	      known = ix;
	      break;
	    end
	  end
	
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  else
	    if (known ~= exon_idx) && to_keep(exon_idx)
	      to_keep(exon_idx) = 0;
	      changed = 1;
	    end
	    if (known ~= test_exon_idx) && to_keep(test_exon_idx)
	      to_keep(test_exon_idx) = 0;
	      changed = 1;
	    end
	  end
	end
	      
	      

	% 0001
      elseif (~cur_edge_left&&~cur_edge_right&&~test_edge_left&&test_edge_right)
	if (vertices(2,exon_idx)-MISALIGN<=vertices(2,test_exon_idx))&&...
	      (vertices(2,exon_idx)>=vertices(1,test_exon_idx))
	
	  new_vertex(1) = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	  new_vertex(2) = vertices(2,test_exon_idx);
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	  
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix)) && issubset(new_edges,edges(ix,:)) 
	      known = ix;
	      break;
	    end
	  end
	  assert(known~=exon_idx);
	  
	  if to_keep(exon_idx)
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif (known ~= test_exon_idx) && to_keep(test_exon_idx)
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	% 0100
      elseif (~cur_edge_left&&cur_edge_right&&~test_edge_left&&~test_edge_right)
	if (vertices(2,exon_idx)>=vertices(2,test_exon_idx)-MISALIGN)&&...
	     (vertices(1,exon_idx)<=vertices(2,test_exon_idx))
	
	  new_vertex(1) = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	  new_vertex(2) = vertices(2,exon_idx);
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix)) && issubset(new_edges,edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert(known~=test_exon_idx);
	
	  if to_keep(test_exon_idx)
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif (known ~= exon_idx) && to_keep(exon_idx)
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	
	% 0010
      elseif (~cur_edge_left&&~cur_edge_right&&test_edge_left&&~test_edge_right)
	if (vertices(1,exon_idx)+MISALIGN>=vertices(1,test_exon_idx))&&...
	      (vertices(1,exon_idx)<=vertices(2,test_exon_idx))
	    
	    
	  new_vertex(1) = vertices(1,test_exon_idx);
	  new_vertex(2) = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	  
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix)) && issubset(new_edges,edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert(known~=exon_idx);
	  
	  if to_keep(exon_idx)
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif (known ~= test_exon_idx) && to_keep(test_exon_idx)
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	% 1000
      elseif (cur_edge_left&&~cur_edge_right&&~test_edge_left&&~test_edge_right)
	if (vertices(1,exon_idx)<=vertices(1,test_exon_idx)+MISALIGN)&&...
	      (vertices(2,exon_idx)>=vertices(1,test_exon_idx))
	
	    
	  new_vertex(1) = vertices(1,exon_idx);
	  new_vertex(2) = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	  
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix)) && issubset(new_edges,edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert(known~=test_exon_idx);
	  
	  if to_keep(test_exon_idx)
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif (known ~= exon_idx) && to_keep(exon_idx)
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	end
	    
	% 0011
      elseif (~cur_edge_left&&~cur_edge_right&&test_edge_left&&test_edge_right)
	if (vertices(1,exon_idx)+MISALIGN>=vertices(1,test_exon_idx))&&...
	     (vertices(2,exon_idx)-MISALIGN<=vertices(2,test_exon_idx))
	
	  new_vertex = vertices(:,test_exon_idx);
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	  
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix)) && issubset(new_edges,edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert(known~=exon_idx);
	  
	  if to_keep(exon_idx)
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	  
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif (known ~= test_exon_idx) && to_keep(test_exon_idx)
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	% 1100
      elseif (cur_edge_left&&cur_edge_right&&~test_edge_left&&~test_edge_right)
	if (vertices(1,exon_idx)<=vertices(1,test_exon_idx)+MISALIGN)&&...
	      (vertices(2,exon_idx)>=vertices(2,test_exon_idx)-MISALIGN)
	
	  new_vertex = vertices(:,exon_idx);
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	  
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix)) && issubset(new_edges,edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert(known~=test_exon_idx);
	  
	  if to_keep(test_exon_idx)
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif (known ~= exon_idx) && to_keep(exon_idx)
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	end
	    
	% 0101
      elseif (~cur_edge_left&&cur_edge_right&&~test_edge_left&&test_edge_right)
	if (vertices(2,exon_idx)==vertices(2,test_exon_idx))

	  if (~isequal(edges(exon_idx,:),edges(test_exon_idx,:)))
	    edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	    edges(:,exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	    edges(test_exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	    edges(:,test_exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	      
	    if (vertices(1,exon_idx) > vertices(1,test_exon_idx))
	      to_keep(exon_idx) = 0;
	    else
	      to_keep(test_exon_idx) = 0;
	    end
	  
	  
	    changed = 1;
	  else
	    if (vertices(1,exon_idx) > vertices(1,test_exon_idx)) && (to_keep(exon_idx))
	      to_keep(exon_idx) = 0;
	      changed = 1;
	    end
	    if (vertices(1,exon_idx) <= vertices(1,test_exon_idx)) && (to_keep(test_exon_idx))
	      to_keep(test_exon_idx) = 0;
	      changed = 1;
	    end
	  end
	end
	    

	    
	    
	% 1010
      elseif (cur_edge_left&&~cur_edge_right&&test_edge_left&&~test_edge_right)
	if (vertices(1,exon_idx)==vertices(1,test_exon_idx))
	
	  if (~isequal(edges(exon_idx,:),edges(test_exon_idx,:)))
	    edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	    edges(:,exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	    edges(test_exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	    edges(:,test_exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	    
	    if (vertices(2,exon_idx) < vertices(2,test_exon_idx))
	      to_keep(exon_idx) = 0;
	    else
	      to_keep(test_exon_idx) = 0;
	    end
	  
	  
	    changed = 1;
	  else
	    if (vertices(2,exon_idx) < vertices(2,test_exon_idx)) && (to_keep(exon_idx))
	      to_keep(exon_idx) = 0;
	      changed = 1;
	    end
	    if (vertices(2,exon_idx) >= vertices(2,test_exon_idx)) && (to_keep(test_exon_idx))
	      to_keep(test_exon_idx) = 0;
	      changed = 1;
	    end
	  end
	end
	    
	    
	    
	% 0110  
      elseif (~cur_edge_left&&cur_edge_right&&test_edge_left&&~test_edge_right)
	if (vertices(1,exon_idx)+MISALIGN>=vertices(1,test_exon_idx))&&...
	      (vertices(2,exon_idx)>=vertices(2,test_exon_idx)-MISALIGN)&&...
	      (vertices(1,exon_idx)<=vertices(2,test_exon_idx))

	  new_vertex = [vertices(1,test_exon_idx);vertices(2,exon_idx)];
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	
	  known = 0;
	  for ix = 1:size(vertices,2)
	    % assert(isequal(find(new_edges),intersect(find(new_edges),find(edges(ix,:)))) == ~any(new_edges & ~edges(ix,:)));
	    if isequal(new_vertex,vertices(:,ix)) && ~any(new_edges & ~edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert((known~=exon_idx)&&(known~=test_exon_idx));
	  
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif to_keep(exon_idx) || to_keep(test_exon_idx)
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end

	% 1001
      elseif (cur_edge_left&&~cur_edge_right&&~test_edge_left&&test_edge_right)
	if (vertices(1,exon_idx)<=vertices(1,test_exon_idx)+MISALIGN)&&...
	      (vertices(2,exon_idx)-MISALIGN<=vertices(2,test_exon_idx))&&...
	      (vertices(2,exon_idx)>=vertices(1,test_exon_idx))

	
	  new_vertex = [vertices(1,exon_idx);vertices(2,test_exon_idx)];
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	  known = 0;
	  for ix = 1:size(vertices,2)
	    % assert(isequal(find(new_edges),intersect(find(new_edges),find(edges(ix,:))))==~any(new_edges & ~edges(ix,:)));
	    if isequal(new_vertex,vertices(:,ix)) && ~any(new_edges & ~edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert((known~=exon_idx)&&(known~=test_exon_idx));
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif to_keep(exon_idx) || to_keep(test_exon_idx)
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end
	  

	    
	  
	    
	% 0111 
      elseif (~cur_edge_left&&cur_edge_right&&test_edge_left&&test_edge_right)
	if (vertices(2,exon_idx)==vertices(2,test_exon_idx))
	  
	  %    (vertices(1,exon_idx)+MISALIGN>=vertices(1,test_exon_idx))
	  
	  % index to the right
	  right_edge = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	  idx = find(vertices(1,:)>right_edge, 1, 'first');
	  
	  
	  if (~isequal(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end)))
	    
	    edges(exon_idx,idx:end) = or(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end));
	    edges(test_exon_idx,idx:end) = edges(exon_idx,idx:end);
	    edges(idx:end,exon_idx) = or(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end));
	    edges(idx:end,test_exon_idx) = edges(idx:end,exon_idx);
	    
	    
	    %to_keep(exon_idx) = 0;
	    
	    changed = 1;
	  end
	  if (vertices(1,exon_idx)+MISALIGN >= vertices(1,test_exon_idx)) && (to_keep(exon_idx))
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	end
	    
	% 1101
      elseif (cur_edge_left&&cur_edge_right&&~test_edge_left&&test_edge_right)
	if (vertices(2,exon_idx)==vertices(2,test_exon_idx))
	  
	  %    (vertices(1,exon_idx)-MISALIGN<=vertices(1,test_exon_idx))
	  
	  % index to the right
	  right_edge = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	  idx = find(vertices(1,:)>right_edge,1,'first');
	
	
	  if (~isequal(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end)))
	    
	    edges(exon_idx,idx:end) = or(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end));
	    edges(test_exon_idx,idx:end) = edges(exon_idx,idx:end);
	    edges(idx:end,exon_idx) = or(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end));
	    edges(idx:end,test_exon_idx) = edges(idx:end,exon_idx);
	    
	    
	    %to_keep(test_exon_idx) = 0;
	    
	    changed = 1;
	  end
	  if (vertices(1,exon_idx)-MISALIGN <= vertices(1,test_exon_idx)) && (to_keep(test_exon_idx))
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	% 1011  
      elseif (cur_edge_left&&~cur_edge_right&&test_edge_left&&test_edge_right)
	if (vertices(1,exon_idx)==vertices(1,test_exon_idx))
	  
	  %(vertices(2,exon_idx)-MISALIGN<=vertices(2,test_exon_idx))
	  % index to the left
	  left_edge = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	  idx = find(vertices(2,:)<left_edge,1,'last');
	  
	  
	  if (~isequal(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx)))
	    
	    edges(exon_idx,1:idx) = or(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx));
	    edges(test_exon_idx,1:idx) = edges(exon_idx,1:idx);
	    edges(1:idx,exon_idx) = or(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx));
	    edges(1:idx,test_exon_idx) = edges(1:idx,exon_idx);
	    
	    
	    %to_keep(exon_idx) = 0;
	      
	    changed = 1;
	  end
	  
	  if (vertices(2,exon_idx)-MISALIGN <= vertices(2,test_exon_idx)) && (to_keep(exon_idx))
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	
	% 1110
      elseif (cur_edge_left&&cur_edge_right&&test_edge_left&&~test_edge_right)
	if (vertices(1,exon_idx)==vertices(1,test_exon_idx))
	  
	  %    (vertices(2,exon_idx)+MISALIGN>=vertices(2,test_exon_idx))
	  
	  % index to the left
	  left_edge = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	  idx = find(vertices(2,:)<left_edge,1,'last') ;

	
	  if (~isequal(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx)))
	    edges(exon_idx,1:idx) = or(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx));
	    edges(test_exon_idx,1:idx) = edges(exon_idx,1:idx);
	    edges(1:idx,exon_idx) = or(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx));
	    edges(1:idx,test_exon_idx) = edges(1:idx,exon_idx);
	  
	    
	    %to_keep(test_exon_idx) = 0;
	    
	    changed = 1;
	  end
	  
	  if (vertices(2,exon_idx)+MISALIGN >= vertices(2,test_exon_idx)) && (to_keep(test_exon_idx))
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	% 1111
      elseif (cur_edge_left&&cur_edge_right&&test_edge_left&&test_edge_right)
	
	if (vertices(1,exon_idx)==vertices(1,test_exon_idx))&&...
	      (vertices(2,exon_idx)==vertices(2,test_exon_idx))
	  if (~isequal(edges(exon_idx,:),edges(test_exon_idx,:)))
	    edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	    edges(test_exon_idx,:) = edges(exon_idx,:);
	    edges(:,exon_idx) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	    edges(:,test_exon_idx) = edges(:,exon_idx);
	  
	    
	    to_keep(test_exon_idx) = 0;
	    
	    changed = 1;
	  elseif (to_keep(test_exon_idx))
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	elseif (vertices(1,exon_idx)==vertices(1,test_exon_idx))&&...
	      (vertices(2,exon_idx)~=vertices(2,test_exon_idx))&&...
	      (to_keep(exon_idx)||to_keep(test_exon_idx))
	  % index to the left
	  %left_edge = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	  %idx = find(vertices(2,:)<left_edge, 1, 'last');
	  idx = find(vertices(2,:)<vertices(1,exon_idx), 1, 'last');

	  if (~isequal(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx)))
	    edges(exon_idx,1:idx) = or(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx));
	    edges(test_exon_idx,1:idx) = edges(exon_idx,1:idx);
	    edges(1:idx,exon_idx) = or(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx));
	    edges(1:idx,test_exon_idx) = edges(1:idx,exon_idx);
	    	    
	    changed = 1;
	  end

	elseif (vertices(1,exon_idx)~=vertices(1,test_exon_idx))&&...
	      (vertices(2,exon_idx)==vertices(2,test_exon_idx))&&...
	      (to_keep(exon_idx)||to_keep(test_exon_idx))
	  % index to the right
	  %right_edge = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	  %idx = find(vertices(1,:)>right_edge,1,'first');
	  idx = find(vertices(1,:)>vertices(2,exon_idx),1,'first');
	
          
	  if (~isequal(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end)))
	  
	    edges(exon_idx,idx:end) = or(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end));
	    edges(test_exon_idx,idx:end) = edges(exon_idx,idx:end);
	    edges(idx:end,exon_idx) = or(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end));
	    edges(idx:end,test_exon_idx) = edges(idx:end,exon_idx);
	    
	    changed = 1;
	  end
	  
	end
	
      else
	fprintf(1,'Unknown case!!\n');
        if keyboard_allowed(), keyboard; end ;
	
      end  % cases
      

      %test_exon_idx = test_exon_idx+1;
      internal_idx = internal_idx + 1;

    end
    exon_idx = exon_idx + 1;
  end


  
  
  

    
  genes(gene_idx).splicegraph = {};
  to_keep = find(to_keep);
  genes(gene_idx).splicegraph = {vertices(:,to_keep),edges(to_keep,to_keep)};
end



fprintf(1,'\n');












return

function genes=reduce_splice_graph_caller(genes) ;
% genes=reduce_splice_graph_caller(genes) ;

for gene_idx = 1:length(genes)
%gene_idx
%genes(gene_idx)
  vertices = genes(gene_idx).splicegraph{1};
  edges = genes(gene_idx).splicegraph{2};
  %fprintf(1,'gene index:\t%d\n',gene_idx);
  if (mod(gene_idx,1000)==0)
    fprintf(1,'%d\n',gene_idx);
  end
  if isempty(vertices), continue ; end ;

  % find all the intron locations
  [dummy,exon_order] = sort(vertices(1,:),2,'ascend');
  vertices = vertices(:,exon_order);
  edges = edges(exon_order,exon_order);
  intron_loc = [];
  for ix1 = 1:size(vertices,2)-1
    for ix2 = 2:size(vertices,2)
      if edges(ix1,ix2)
	intron_loc = union(intron_loc,[vertices(2,ix1)+1:vertices(1,ix2)-1]);
      end
    end
  end
  
  
  
  if (size(edges,1) < 2)
    changed = 1;
    while changed
      changed = 0;
      exon_idx = 1;
      while exon_idx <= size(vertices,2)
	
	test_exon_idx = exon_idx+1;
	while test_exon_idx <= size(vertices,2)
	  if (vertices(1,exon_idx)<=vertices(1,test_exon_idx)) &&...
		(vertices(2,exon_idx)>=vertices(2,test_exon_idx))
	    vertices(1,exon_idx) = min(vertices(1,exon_idx),...
				       vertices(1,test_exon_idx));
	    vertices(2,exon_idx) = max(vertices(2,exon_idx),...
				       vertices(2,test_exon_idx));
	    new_index = [1:test_exon_idx-1,test_exon_idx+1:size(vertices,2)];
	    
	    
	    vertices = vertices(:,new_index);
	  
	    %disp(vertices)
	    
	    %test_exon_idx = exon_idx+1;
	    changed = 1;
	    
	  end
	  test_exon_idx = test_exon_idx+1;
	end
	exon_idx = exon_idx + 1;
      end
    end
    
  else
    changed = 1;
    while changed
      changed = 0;
      exon_idx = 1;
      while exon_idx <= size(vertices,2)

	[dummy,exon_order] = sort(vertices(1,:),2,'ascend');
	if ~isequal(vertices(1,:),dummy)
	  vertices = vertices(:,exon_order);
	  edges = edges(exon_order,exon_order);
	end
	
	test_exon_idx = exon_idx + 1;
	while test_exon_idx <= size(vertices,2)
	  reduce_now = 0;
	  
          if (test_exon_idx < exon_idx) && keyboard_allowed(), keyboard; end;
	  
	  cur_edge_left = sum(edges(1:exon_idx,exon_idx));
	  test_edge_left = sum(edges(1:test_exon_idx,test_exon_idx));
	  cur_edge_right = sum(edges(exon_idx:end,exon_idx));
	  test_edge_right = sum(edges(test_exon_idx:end,test_exon_idx));
	  
	  

	  
	  % 0000
	  if (~cur_edge_left&&~cur_edge_right&&~test_edge_left&&~test_edge_right)
	    
	    if ((vertices(2,exon_idx)>=vertices(1,test_exon_idx))&&...
		(vertices(1,exon_idx)<=vertices(1,test_exon_idx)))||...
		  ((vertices(2,test_exon_idx)>=vertices(1,exon_idx))&&...
		   (vertices(1,test_exon_idx)<=vertices(1,exon_idx)))&&...
		  (sum(ismember([min(vertices(1,exon_idx),vertices(1,test_exon_idx)):...
				 max(vertices(2,exon_idx),vertices(2,test_exon_idx))],intron_loc))==0)
	      
	      vertices(1,exon_idx) = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	      vertices(2,exon_idx) = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	      new_index = [1:test_exon_idx-1,test_exon_idx+1:size(vertices,2)];
	    
	    
	      vertices = vertices(:,new_index);
	      edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	      edges(:,exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	      edges = edges(new_index,new_index);
	    
	  
	    
	      reduce_now = 1;
	      changed = 1;
	    end
	    
	    
	  % 0101
	  elseif (~cur_edge_left&&cur_edge_right&&~test_edge_left&&test_edge_right)

	    if (vertices(2,exon_idx)==vertices(2,test_exon_idx))&&...
		  (sum(ismember([min(vertices(1,exon_idx),vertices(1,test_exon_idx)):...
				 vertices(2,exon_idx)],intron_loc))==0)
	      
	      vertices(1,exon_idx) = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	      new_index = [1:test_exon_idx-1,test_exon_idx+1:size(vertices,2)];
	    
	    
	      vertices = vertices(:,new_index);
	      edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	      edges(:,exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	      edges = edges(new_index,new_index);
	    
	  
	    
	      reduce_now = 1;
	      changed = 1;
	    end

	    
	  % 1010
	  elseif (cur_edge_left&&~cur_edge_right&&test_edge_left&&~test_edge_right)
	    if (vertices(1,exon_idx)==vertices(1,test_exon_idx))&&...
		  (sum(ismember([vertices(1,exon_idx):...
				 max(vertices(2,exon_idx),vertices(2,test_exon_idx))],intron_loc))==0)
	      
	      vertices(2,exon_idx) = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	      new_index = [1:test_exon_idx-1,test_exon_idx+1:size(vertices,2)];
	    
	    
	      vertices = vertices(:,new_index);
	      edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	      edges(:,exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	      edges = edges(new_index,new_index);
	    
	  
	    
	      reduce_now = 1;
	      changed = 1;
	    end

	  
	  % 1111
	  elseif (cur_edge_left&&cur_edge_right&&test_edge_left&&test_edge_right)
	    if (vertices(1,exon_idx)==vertices(1,test_exon_idx))&&...
		 (vertices(2,exon_idx)==vertices(2,test_exon_idx))
	    
	      new_index = [1:test_exon_idx-1,test_exon_idx+1:size(vertices,2)];
	    
	    
	      vertices = vertices(:,new_index);
	      edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	      edges(:,exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	      edges = edges(new_index,new_index);
	      
	      reduce_now = 1;
	      changed = 1;
	  
	    end
	    
	  end % cases
	  

          if ~reduce_now
	    test_exon_idx = test_exon_idx+1;
	  end
	  

	end
	exon_idx = exon_idx + 1;
      end %while exon_idx <= size(vertices,2)
    end %while changed

  
  
  
  end 
    
  genes(gene_idx).splicegraph = {};
  genes(gene_idx).splicegraph = {vertices,edges};
end


fprintf(1,'\n');

return





return

function [tmp_fname,STATS] = gff3_statistics(gff_fname,thesources, contig)
% [tmp_fname,STATS] = gff3_statistics(gff_fname,thesources, contig)
  

if nargin<2
  thesources = [];
end

                                                                                                                                                       
if nargin<3                                                                                     
  contig = [];                                                                                    
end 
                                                                                    

%%% SOFA mapping
SOFA_short_names = {'gene','mRNA','exon','CDS','intron','polyA_sequence','polyA_site','five_prime_UTR','three_prime_UTR'};
SOFA_IDs = {'SO:0000704','SO:0000234','SO:0000147','SO:0000316','SO:0000188','SO:0000610','SO:0000553','SO:0000204','SO:0000205'};

%%% GFF file summary

[fd msg] = fopen(gff_fname, 'r');
disp(msg);

tmp_fname = tempname;
if ~isempty(thesources) 
  fprintf('filtering for source :\n');
  unix(sprintf('echo > %s', tmp_fname)) ;
  for i=1:length(thesources),
    fprintf('%s\n',thesources{i});
    unix(sprintf('grep -e "[[:space:]]%s[[:space:]]" %s |grep -v "#" >> %s', thesources{i}, gff_fname, tmp_fname));
  end 
else
  unix(sprintf('grep -v "#" %s > %s', gff_fname, tmp_fname));
end

fprintf('\n------FILE SUMMARY-----\n');
[tmp,seqids] = unix(sprintf('sed -e "1d" %s | cut -f 1 |sort -u', tmp_fname));
if isempty(contig) 
   fprintf('gff file containes following %i seqids in column 1 :\n',length(seqids));
   if length(seqids)<100
      fprintf('%s\n',seqids)
   end
end

[tmp, sources]=unix(sprintf('sed -e "1d" %s | cut -f 2 |sort -u', tmp_fname));
if isempty(sources)
  error('specified sources not contained in file')
end
fprintf('gff file containes following sources in column 2 :\n');
sources = separate(sources,sprintf('\n'));
sources(end)=[];
for t=1:length(sources)
  [tmp,num_sources]=unix(sprintf('cut -f 2 %s |grep -e "^%s$" | wc -l',tmp_fname,sources{t}));
  fprintf('%s\t%i\n',sources{t},str2num(num_sources));
end
  
if ~isempty(contig) 
  fprintf('filtering for contig :\n');
  fprintf('^%s[[:space:]]\n',contig);

  seqids_array = separate(upper(seqids), sprintf('\n')) ;
  if ~isempty(seqids_array) && isempty(seqids_array{end})
    seqids_array(end)=[] ;
  end ;

  cnamepos = strmatch(upper(contig), seqids_array, 'exact');
  % contig = seqids(cnamepos:cnamepos+length(contig)-1); 
  if isempty(cnamepos)%all(isempty(strfind(upper(seqids),upper(contig))))
    fprintf('contig not found in file\n')
    return
  end
  tmp_fname1 = tempname;
  unix(sprintf('grep -e "^%s[[:space:]]" %s > %s', contig, tmp_fname, tmp_fname1));
  [tmp,num_seqs] = unix(sprintf('sed -e "1d" %s | cut -f 1 |sort -u | wc -l',tmp_fname1));
  if (str2num(num_seqs)~=1)
    error('grep for contig doesnt work, repeat with by_contig=0')
  end
else
  tmp_fname1 = tmp_fname;
end
[tmp,types_]=unix(sprintf('sed -e "1d" %s | cut -f 3 |sort -u',tmp_fname1));
fprintf('\nfiltered gff file containes following types in column 3 :\n');
types = separate(types_,sprintf('\n')) ;
idx = find(cellfun('isempty',types));
types(idx)=[];


for t=1:length(types)
  [tmp,num_types]=unix(sprintf('cut -f 3 %s | grep -e "^%s$" |wc -l',tmp_fname1,types{t}));
  fprintf('%s\t%i\n',types{t},str2num(num_types));
  STATS.(types{t})=str2num(num_types);
end
% assert(all(ismember(lower(types),lower(SOFA_short_names)))|all(ismember(types,SOFA_IDs)))

tmp_fname= tmp_fname1;

return

function genes = init_genes(num_genes,PAR,genes)
% genes = init_genes(num_genes,PAR,genes)
  
if nargin<1
  num_genes=0;
end
if nargin<2 || isempty(PAR),
  PAR.organism.name =[];
end
    
genes.id = [];
genes.anno_id = [];
genes.confgenes_id = [];
genes.name = '' ;
genes.alias = [];
genes.name2 = []; 
genes.strand = '';
genes.chr = '';
genes.chr_num = [];
genes.paralogs = [];

genes.start = [];
genes.stop = [];

genes.transcripts = {};
genes.transcript_info = [];
genes.transcript_status = [];
genes.transcript_valid = [];
genes.exons = {}; 
genes.exons_confirmed = {};
genes.cds_exons = {}; 
genes.utr5_exons = {}; 
genes.utr3_exons = {}; 
genes.tis = {};
genes.tis_conf = [];
genes.tis_info = [];

genes.cdsStop = {};
genes.cdsStop_conf = [];
genes.cdsStop_info = [];

if isequal(PAR.organism.name,'C_elegans')||(isfield(PAR.organism,'clade')&&isequal(PAR.organism.clade,'nematode'))
  genes.transacc = {};
  genes.transacc_info = [];
  genes.transacc_conf = [];
end
genes.tss = {};
genes.tss_info = [];
genes.tss_conf = [];
genes.cleave = {};
genes.cleave_info = [];
genes.cleave_conf = [];
genes.polya = {};
genes.polya_info = [];
genes.polya_conf = [];

genes.is_alt = [];
genes.is_alt_spliced = [];
genes.is_valid = [];
genes.transcript_complete = [];
genes.is_complete = [];
genes.is_correctly_gff3_referenced = [];

genes.splicegraph =[];

if isequal(PAR.organism.name,'C_elegans')||(isfield(PAR.organism,'clade')&&isequal(PAR.organism.clade,'nematode'))
  genes.in_operon=[];
end

if num_genes==0
  genes=genes([]);
elseif num_genes>1,
  genes(num_genes) = genes ;
end

return

function genome_info=init_genome(config_fname)
% genome_info=init_genome(config_fname)  
%
% global g_genome_info ;

global g_genome_info genome_strings;

genome_strings = [];
g_est_fnames={} ;
g_prot_fnames={} ;
g_cdna_fnames={} ;
g_flcdna_fnames={} ;
g_contig_names={} ;
g_flat_fnames={} ;
g_fasta_fnames={} ;
g_annotation_fnames={} ;
g_alphabet='acgt' ;
g_basedir='' ;
g_max_intron_len=1000000 ;
g_min_intron_len=10 ;
g_merge_est_transcripts=1 ;

fd=fopen(config_fname, 'r') ;
while ~feof(fd),
  line = fgetl(fd) ;
  if ~ischar(line), break ; end ;
  if isempty(line), continue ; end ;
  if line(1)=='#', continue ; end ;
  if length(line)>8 && isequal(line(1:7),'BASEDIR')
    g_basedir=deblank(line(9:end)) ;
  elseif length(line)>21 && isequal(line(1:21), 'MERGE_EST_TRANSCRIPTS')
    g_merge_est_transcripts = str2num(line(22:end)) ; 
  elseif length(line)>15 && isequal(line(1:15),'MAX_DOUBLE_QGAP')
  elseif length(line)>15 && isequal(line(1:15),'MAX_DOUBLE_TGAP')
  elseif length(line)>17 && isequal(line(1:17),'MAX_CDNA_DELETION')
  elseif length(line)>18 && isequal(line(1:18),'MAX_CDNA_INSERTION')
  elseif length(line)>21 && isequal(line(1:21),'TERMINAL_EXON_END_TOL')
  elseif length(line)>14 && isequal(line(1:14),'MIN_INTRON_LEN')
    g_min_intron_len = str2num(line(15:end)) ; 
    assert(~isempty(g_min_intron_len) & g_min_intron_len<1000) ;
  elseif length(line)>14 && isequal(line(1:14),'MAX_INTRON_LEN')
    g_max_intron_len = str2num(line(15:end)) ; 
    assert(~isempty(g_max_intron_len) & g_max_intron_len>1000) ;
  elseif length(line)>18 && isequal(line(1:18),'MIN_EST_COVER_FRAC')

  elseif length(line)>19 && isequal(line(1:19),'MIN_PROT_COVER_FRAC')

  elseif length(line)>19 && isequal(line(1:19),'MIN_CDNA_COVER_FRAC')
  
  elseif length(line)>18 && isequal(line(1:18),'BLAT_BEST_HIT_ONLY')
  
  elseif length(line)>20 && isequal(line(1:20),'BLAT_BEST_HIT_MARGIN')

  elseif length(line)>8 && isequal(line(1:8),'ALPHABET')
    g_alphabet=deblank(line(10:end)) ;
  elseif length(line)>7 && isequal(line(1:7),'CONTIGS'),
    for i=1:str2num(line(8:end))
      l2=fgetl(fd) ; 
      idx=strfind(l2,'  ') ;
      while ~isempty(idx), 
        l2(idx)=[] ;
        idx=strfind(l2,'  ') ;
      end ;
      l2(l2==' ')=sprintf('\t') ;
      elems=separate(l2) ; 
      assert(length(elems)==3) ;
      g_contig_names{i}=elems{1} ;
      g_flat_fnames{i}=path_with_basedir(elems{2}, g_basedir) ;
      g_fasta_fnames{i}=path_with_basedir(elems{3}, g_basedir) ; 
    end ;
  elseif length(line)>8 && isequal(line(1:8),'ESTFILES'),
    for i=1:str2num(line(9:end))
      g_est_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>9 && isequal(line(1:9),'PROTFILES'),
    for i=1:str2num(line(10:end))
      g_prot_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>9 && isequal(line(1:9),'CDNAFILES'),
    for i=1:str2num(line(10:end))
      g_cdna_fnames{end+1}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>11 && isequal(line(1:11),'FLCDNAFILES'),
    for i=1:str2num(line(12:end))
      g_flcdna_fnames{end+1}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>15 && isequal(line(1:15),'ANNOTATIONFILES'),
    for i=1:str2num(line(16:end))
      g_annotation_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif ~isempty(line),
    error(sprintf('cannot understand "%s" in config file', line)) ;
  end ;
end ;
fclose(fd) ;

genome_info=[] ;
genome_info.est_fnames=g_est_fnames ;
genome_info.prot_fnames=g_prot_fnames ;
genome_info.cdna_fnames=g_cdna_fnames ;
genome_info.flcdna_fnames=g_flcdna_fnames ;
genome_info.contig_names=g_contig_names ;
genome_info.flat_fnames=g_flat_fnames ;
genome_info.fasta_fnames=g_fasta_fnames ;
genome_info.annotation_fnames=g_annotation_fnames ;
genome_info.alphabet=g_alphabet ;
genome_info.basedir=g_basedir ;
genome_info.max_intron_len = g_max_intron_len ;
genome_info.min_intron_len = g_min_intron_len ;
genome_info.merge_est_transcripts = g_merge_est_transcripts ;
g_genome_info=genome_info ;

return ;


function p=path_with_basedir(p1,basedir)
  
  if p1(1)=='/',
    p=p1; 
    return ;
  else
    if basedir(end)=='/',
      p=[basedir p1] ;
    else
      p=[basedir '/' p1] ;
    end ;
  end ;
  
  

return

function [str,valid_splice,ok] = load_genomic(contig,strand,start,stop, genome_info,check_splice)
% function [str,valid_splice,ok]=load_genomic(contig,strand,start,stop, genome_info,check_splice)

if nargin<6
  check_splice=0;
end ;
ok=1 ;

if ~check_splice 
  valid_splice=[];
  % fprintf(1,'no splice site consensus check!')
end ;


if length(start)>1 | length(stop)>1,
  if nargout<3
    assert(length(start)==length(stop)) ;
    assert(all([start(2:end)-stop(1:end-1)]>=0));
  else
    if ~(length(start)==length(stop)) || ~(all([start(2:end)-stop(1:end-1)]>=0)),
      ok=0 ;
      str='' ;
      valid_splice = [] ;
      return ;
    end ;
  end ;
  valid_splice = zeros(length(start),2); 
  if strand == '+', 
    idx = 1:length(start) ; 
  else 
    idx = length(start):-1:1 ; 
  end ;
  str = '' ;
  for i=idx  %1:length(start)
    seq = load_genomic(contig, strand, start(i)-5, stop(i)+5, genome_info) ;
    str = [str seq(6:end-5)] ;

    % check internal splicing sites
    if check_splice
      % if (i~=1 && strand == '+') || (i~=length(start) && strand == '-')
      if (strand == '+') || (i~=length(start) && strand == '-')
        valid_splice(i,1) = isequal(seq(4:5),'ag') ;
        %if ~isequal(seq(4:5),'ag'), fprintf('acc: %s %c\n', seq(3:6), strand); end ;
        % assert(isequal(seq(4:5),'ag'))
      end ;
      %if i~=idx(end)
      % if (i~=1 && strand == '-') || (i~=length(start) && strand == '+')
      if (strand == '-') || (i~=length(start) && strand == '+')        
        valid_splice(i,2) = ismember(seq(end-5:end-4), {'gt','gc'}) ;
        %if ~isequal(seq(end-4:end-3),'gt')||isequal(seq(end-4:end-3),'gc'), fprintf('don: %s %c\n', seq(end-5:end-2), strand); end ;
        % assert(isequal(seq(end-4:end-3),'gt')||isequal(seq(end-4:end-3),'gc'))
      end ; 
    end ;
  end
  return ;
else
  valid_splice=[0 0];
end ;


if ischar(contig),
  %contig_idx = find(ismember(upper(genome_info.contig_names),upper(contig))) ;
  contig_idx = strmatch(upper(contig), upper(genome_info.contig_names), 'exact') ;
  assert(all(size(contig_idx)==1) & ~isempty(contig_idx)) ;
else
  contig_idx = contig ;
end ;
  
fname=genome_info.flat_fnames{contig_idx} ;
if genome_info.alphabet(1)=='a',
  NN='n' ;
else
  NN='N' ;
end ;

d=dir(fname) ; left_n = 0 ; right_n=0 ;
if isinf(stop), stop=d.bytes; end ;

if start<1 | stop>d.bytes,
  warning('load_genomic:contig_boundary', 'boundary of contig reached (start: %i, stop: %i, d.bytes: %i), padding with "n"', start, stop,d.bytes) ;

  if start<1,
    left_n = -(start-1) ;
    start = 1 ;
  end ;
  if stop>d.bytes,
    right_n = stop-d.bytes ;
  end ;
  if start>d.bytes && stop>d.bytes,
    str = char(NN*ones(1,stop-start)) ;
    return ;
  end ;
end ;

fd=fopen(fname,'r') ;
fseek(fd, start-1, -1) ;
str=char(fread(fd,stop-start+1, 'char=>char'))' ;
fclose(fd) ;
if left_n>0,
  str=char([NN*ones(1,left_n) str]) ;
end ;
if right_n>0,
  str=char([str NN*ones(1,right_n)]) ;
end ;

assert(length(str)==stop-start+left_n+1)
%let = bigunique(str) ;
let = unique(str) ;
if genome_info.alphabet(1)=='a' && ~isempty(setdiff(let,'acgtn')),
  % warning(sprintf('invalid letter in genome! ("%s")', setdiff(let,'acgtn')));
  let = setdiff(let, 'acgtn') ;
  for c=let,
    str(str==c) = 'n';
  end ;
elseif genome_info.alphabet(1)=='A' && ~isempty(setdiff(let,'ACGTN')),
  warning(sprintf('invalid letter in genome! ("%s")', setdiff(let,'ACGTN')));
  let = setdiff(let, 'ACGTN') ;
  for c=let,
    str(str==c) = 'N';
  end ;
end

if strand=='-',
  str=reverse_complement(str) ;
end ;

return ;

return

function [seqs,names]=read_fasta_file(fname, name_only)
% [seqs,names]=read_fasta_file(fname)
% 
% reads all entries in a fasta file
  
if nargin<2, name_only=0; end ; 

fd=fopen(fname,'r') ;
str=fread(fd) ;

% fix new lines
num13=sum(str==13) ;
num10=sum(str==10) ;
if num10==0 && num13>0,
  str(str==13)=10 ;
end ;
if num10>0 && num13>0,
  assert(num10==num13) ;
  str(str==13)=[] ;
end ;

newseq=find(str=='>') ;
newseq(end+1) = length(str)+1 ;

seqs={} ;
names={} ;

for i=1:length(newseq)-1,
  seq=char(str(newseq(i):newseq(i+1)-1)) ;
  idx=find(seq==10, 1, 'first') ;
  assert(length(idx)==1) ;
  name = seq(2:idx-1)' ;
  names{end+1}=name ;
  if ~name_only,
    seq  = seq(idx+1:end)' ;
    seq(seq==10) = [] ;
    seqs{end+1} = seq ;
  end ;
end ;

fclose(fd) ;

return

function x=reverse_complement(y)

if length(y)>200*1024*1024
  x=uint8(y) ;
  for i=1:length(y),
    x(i)=y(end-i+1) ;
  end ;
  y=x ;
else
  y=y(end:-1:1) ;
  x=y ;
end ;

x(y=='a')='t' ;
x(y=='c')='g' ;
x(y=='g')='c' ;
x(y=='t')='a' ;
x(y=='n')='n' ;
x(y=='A')='T' ;
x(y=='C')='G' ;
x(y=='G')='C' ;
x(y=='T')='A' ;
x(y=='N')='N' ;

x=char(x) ;

return

function  [prob, prob_cum,limits,prob_unsort,prob_cum_unsort] = learn_plifs(Out,L,bins,do_balancing,min_inc);
% [prob, prob_cum,limits,prob_unsort,prob_cum_unsort] = learn_plifs_qp(Output,Labels,numberofbins,do_balancing,min_inc);
% 
% learn piecewise linear functions (PLIF), with which outputs of an SVM
% can be transformed to probability and cumulative
% probability values
% 
% INPUT    Out                vector of SVM Outputs on an evaluation set
%
%          L                  vector of corresponding labels (-1,1)  
%
%          bins               number of bins used for interpolation (default 50)
%                             (limits are choosen such that the number of
%                             examples in every bin is similar)  
%
%          do_balancing       should be set to 1 if negative class is
%                             much bigger than positive class,
%                             (negatives are downsampled for
%                             determining the bins) 
%
%          min_inc            probabilities have to be monotonically
%                             increasing (higher SVM output value
%                             corresponds to higher probability of true positive) 
%                             (default 1e-5)
%
%
% OUTPUT   prob               vector of length 'bins'. prob_cum(i) is the
%                             estimated probability of examples with 
%                             Output = limits(i) to be true positive
% 
%          prob_cum           vector of length 'bins'. prob_cum(i) is the
%                             estimated probability of examples with 
%                             Output> limits(i) to be true positive
%
%          limits             vector of length 'bins' with supporting
%                             points (SVM Output values)
%
% see also: Out2Conf  
  
if nargin<5, min_inc = 1e-5 ; end ;
if nargin<4, do_balancing = 1; end ;
if nargin<3, bins = 50; end ;

if do_balancing
  s1 = [min(Out) max(Out)];
  s1 = [s1 Out(L==1)];
  idx = find(L==-1);
  num = min(sum(L==-1),sum(L==1));
  % ii = randperm(length(idx));
  idx = idx(randperm(length(idx)));
  s1 = sort([s1 Out(idx(1:num))]);
  limits = [s1(round(linspace(1,length(s1),bins-1))) inf] ;
else
  s1 = sort(Out) ;
  limits = [s1(round(linspace(1,length(s1),bins-1))) inf] ;
end
  
prob = zeros(bins,1);
prob_cum = zeros(bins,1);
num = zeros(bins,1) ;
num_cum = zeros(bins,1) ;
for i=1:bins-1, 
  num_cum(i) = sum(Out>=limits(i)) ;
  prob_cum(i) = (sum(Out>=limits(i) & L==1 )+1e-10)/(sum(Out>=limits(i))+1e-10) ;
  num(i) = sum(Out>=limits(i) & Out<limits(i+1)) ;
  prob(i) = (sum(Out>=limits(i) & Out<limits(i+1) & L==1 )+1e-10)/(sum(Out>=limits(i) & Out<limits(i+1))+1e-10) ;
end 
prob_cum(bins) = 1;
prob(bins) = 1;

prob_unsort=prob ;
prob_cum_unsort=prob_cum ;

prob_cum = sort(prob_cum) ;
prob = sort(prob) ;

return ;

return

function  [Conf, Conf_cum] = Out2Conf(Out,prob, prob_cum,limits);

% [Conf, Conf_cum] = Out2Conf(Output ,prob, prob_cum,limits);
%  
% function transforms outputs of an SVM to probability and cumulative
% probability values by means of piecewise linear functions (PLIF). 
% Use learn_plifs to determine the function (it will return supporting
% points 'limits' and corresponding y-values, 'prob' and 'prob_cum')
%
% INPUT   Out       vector of SVM Output to be transformed
%         prob
%         prob_cum
%         limits    
%
% OUTPUT  Conf      vector (same length as Out) with estimated
%                   probability for true positive
%         Conf_cum  vector (same length as Out) with estimated 
%                   cumulative probability true positive
% 
% Functions called: -> pwl_add
%
% see also: learn_plifs

bins = length(prob); 
Conf = zeros(1, length(Out)) ;
Conf_cum = zeros(1, length(Out)) ;
for i=1:length(Out)
  x(:,1) = pwl_add(zeros(bins, 1), Out(i), limits) ;
  Conf_cum(i) = x'*prob_cum;
  Conf(i) = x'*prob;
end ;

return

function vec = pwl_add(vec, value, limits)
% vec = pwl_add(vec, value, limits)
  
idx = sum(limits<=value) ;

if idx==0,
  vec(1)=vec(1)+1 ;
elseif idx==length(limits)-1,
  vec(end)=vec(end)+1 ;
else
  vec(idx+1) = vec(idx+1) + (value-limits(idx))/(limits(idx+1)-limits(idx)) ;  
  vec(idx)   = vec(idx)   + (limits(idx+1)-value)/(limits(idx+1)-limits(idx)) ;
end ;
return

function [score,tp,tdr]=calcrfcscore(output,LT)
% score=calcrfcscore(output,LT)
map=(LT==1) ;
[s,idx]=sort(-output) ; s=-s ;
map=map(idx) ;

tp=cumsum(map)/sum(LT==1) ;
tdr=cumsum(map)./[1:length(map)] ;

t=tp(2:end)-tp(1:end-1) ;
score = sum(0.5*[tdr(1:end-1)+tdr(2:end)].*t) ;



return

function [false_alarms,hits] = calcroc(output,LTE)

assert(all(size(output)==size(LTE))) ;

[ld,idx] = sort(output); 

i=0 ;

hits=1-cumsum(LTE(idx)>0)/sum(LTE > 0) ;
false_alarms=1-cumsum(LTE(idx)<0)/sum(LTE < 0) ;

% Da die Kurve nie alle Negativen als positiven zaehlt, hat
% Falsealarms nie 
% den Wert 1. Cumsum zaehlt das erste Element als negatives, diese
% fehlt bei
%  den False_alarms.
% Daher wird vorne an den Vektor noch eine 1 gehaengt

hits = [1 hits ];
false_alarms = [1 false_alarms];

if nargout<2,
  false_alarms = [false_alarms;hits];
end ;

return

function s = calcrocscore(oute,LTE)
% rocscore = calcrocscore(output,LT)
%
% computes the area under the ROC curve

[a,b]=calcroc(oute,LTE) ;

s=0 ;
for i=1:length(a)-1,
  s = s + (a(i)-a(i+1))*(b(i)+b(i+1))/2 ;
end ;


return

function [engine, environment, basedir, mccdir, license_type] = determine_engine() ;
% [engine, environment, basedir, mccdir, license_type] = determine_engine() ;
% 
% returns either 'matlab' or 'octave' in variable engine
% and 'internal' or 'galaxy' in variable environment
% basedir is the directory of the matlab or octave instance
% mccdir is the directory of the matlab compiler (does not yet exist for octave)

global g_license_type ;

if isempty(g_license_type),
  g_license_type = 'academic' ;
end ;
license_type = g_license_type ;

lic=license ;
if ~isequal(lic, 'GNU General Public License'),
  engine='matlab' ;
else
  engine='octave' ;
end ;

environment='internal' ;
if isequal(whoami, 'galaxy'),
  environment='galaxy' ;
end ;

if isequal(engine, 'matlab') && isequal(environment,  'internal'),
  basedir = '/fml/ag-raetsch/share/software/matlab/' ;
elseif isequal(engine, 'matlab') && isequal(environment,  'galaxy'),
  basedir = '/home/galaxy/matlab-7.6/' ;
elseif isequal(engine, 'octave') && isequal(environment,  'internal'),
  basedir = '/fml/ag-raetsch/share/software/octave-3.0.3/' ;
elseif isequal(engine, 'octave') && isequal(environment,  'galaxy'),
  basedir = '/home/galaxy/octave-3.0.4' ;
end ;

if isequal(environment,  'internal'),
  mccdir = '/fml/ag-raetsch/share/software/matlab/' ;
elseif isequal(environment,  'galaxy'),
  mccdir = '/home/galaxy/matlab-7.6/' ;
end ;

return

return

function x = equal(M1, M2)
% EQUAL(M1, M2) tests M1 and M2 for equality
% Function to test whether two matrices are identical
%
% returns 1 if true and 0 if false

x=isequal(M1, M2) 

return

function x = fexist(filename,gz)
% FEXIST Check if file exists
%        FEXIST('filename') returns:
%        0 if filename does not exist
%        1 if filename exists in the given path

if nargin<2,
  gz=0;
end ;

fd = fopen(filename);
if fd ~= -1
  x = 1;
  fclose(fd);
elseif gz,
  fd = fopen([filename '.gz']);
  if fd~=-1,
    x=1 ;
    fclose(fd) ;
  else
    x=0 ;
  end ;
else
  x = 0;
end

return

function [models,param_names] = generate_all_models(Model_param)
% models = generate_all_models(Model_param) 
  
param_names = fieldnames(Model_param); 
if isempty(param_names)
  models =[];param_names=[];
  return
end

for j = 1:length(param_names)
  param_nums(j) = length(getfield(Model_param,param_names{j}));
end
models = zeros(length(param_names),prod(param_nums));
for j = 1:length(param_names)
  par_tmp  = getfield(Model_param,param_names{j});
  par1 = [];
  for k = 1:length(par_tmp)
    par1 = [par1 repmat(k,1,prod(param_nums(j+1:end)))];
  end
  models(j,:) = par_tmp(repmat(par1,1,prod(param_nums(1:j-1))));
end

if prod(param_nums)<1
  assert(length(unique(models','rows'))==prod(param_nums))
else
  assert(~isempty(models))
end

return

function var=load_matrix(filename, varname)
% var=load_matrix(filename, varname)

load(filename, 'meta')
if ~exist('meta')
  load(filename, varname);
  eval(sprintf('var = %s;',varname))
  return
end

var = [] ;
col = 1;
for j=1:meta.num_splits
  %if col>1
  %  assert(~all(var(:,col-1)==-1* ones(meta.size(1),1)))
  %end
  %assert(all(var(:,col)==-1*ones(meta.size(1),1)))
   
  load(filename, sprintf('%s_split_%i',varname,j));
  eval(sprintf('temp = %s_split_%i;',varname,j));

  if j==1,
    if isnumeric(temp)
      var = zeros(meta.size(1), meta.size(2), class(temp));
    else
      var = zeros(meta.size(1), meta.size(2),'uint8');
      var = char(var);
    end
  end ;

  var(:,col:col + size(temp, 2)-1)=temp;
  col = col + size(temp, 2);
end
%assert(~all(var(:,end)==-1* zeros(meta.size(1),1)))

return

function array = load_struct(infile_name, arrayname)
% array = load_struct(infile_name, arrayname)
  
if nargin<2,
  arrayname='array' ;
end ;

L_=load(infile_name) ;
if isfield(L_,'numvars')
  fprintf('arrayload2 loads %i chunks:', L_.numvars) ;
  ix=1;
  L=load(sprintf('%s_%i.mat', infile_name, ix), sprintf('%s%d', arrayname, ix)) ;
  eval(sprintf('array = L.%s1;', arrayname)) ;
  fprintf('.') ;
  for ix = 2:L_.numvars
    L=load(sprintf('%s_%i.mat', infile_name, ix), sprintf('%s%d', arrayname, ix)) ;
    if size(array,1) == 1
      array = [array, eval(sprintf('L.%s%d', arrayname, ix))];
    elseif size(array,2) == 1
      array = [array; eval(sprintf('L.%s%d', arrayname, ix))];
    end
    fprintf('.') ;
  end
  fprintf('\n') ;
  if isfield(L_, 'dimensions'),
    array = reshape(array, L_.dimensions) ;
  end ;
else
  eval(sprintf('array=L_.%s;', arrayname)) ;
end

return

function save_append(fname__, append_flag, varargin)
% save_append(fname__, append_flag, varargin)
%
% octave compatible version of save -append

try,
  if append_flag
    L__=load(fname__) ;
  else
    L__=struct ;
  end ;
catch 
  L__=struct ;
end 

if length(varargin)>1,
  
  % assume varargin to have the following structure: varname1, var1, ...
  
  assert(mod(length(varargin),2)==0) ;
  for i=1:2:length(varargin),
    L__.(varargin{i}) = varargin{i+1} ;
  end ;
  
  f=fieldnames(L__) ;
  for i=1:length(f),
    eval(sprintf('%s=L__.%s;', f{i}, f{i})) ;
  end ;
  
  eval(sprintf('save -V7 %s %s', fname__, sprintf('%s ', f{:}))) ;
else
  
  % assume the second argument is a struct
  assert(length(varargin)==1) ;
  
  f = fieldnames(varargin{1}) ;
  for i=1:2:length(f),
    L__.(f{i}) = varargin{1}.(f{i}) ;
  end ;
  
  f=fieldnames(L__) ;
  for i=1:length(f),
    eval(sprintf('%s=L__.%s;', f{i}, f{i})) ;
  end ;
  
  eval(sprintf('save -V7 %s %s', fname__, sprintf('%s ', f{:}))) ;
  
end ;




return

function save_matrix(filen, M, varname, split_size)
%save_matrix(filen, M, varname, split_size)
%split matrix into parts with size of about split_size 
%bytes (1e+8 default) and save them in one file

if nargin<4
  split_size=1e+8;
end

info = whos('M');
num_splits = ceil(info.bytes/split_size);
num_cols = ceil(size(M, 2)/num_splits);

  
  meta.size=size(M);
  meta.num_splits = num_splits;
  mata.name = varname;
  if fexist(filen)
    save_append(filen, 1, 'meta', meta);
  else
    save(filen, 'meta');
  end
  for j=1:num_splits-1
    %eval(sprintf('%s_split_%i = M(:, 1:num_cols);',varname, j));
    var_split__ = M(:, 1:num_cols);
    M(:, 1:num_cols)=[];
    %save(filen, '-append', sprintf('%s_split_%i',varname,j));
    save_append(filen, 1, sprintf('%s_split_%i',varname,j), var_split__);
    clear var_split__
  end
  if isempty(j), j=0; end;
  var_split__ = M; 
  save_append(filen, 1, sprintf('%s_split_%i',varname,j+1), var_split__);

return

function save_struct(outfile_name, array, arrayname, MAXSIZE)
% save_struct(outfile_name, array[, arrayname])

if nargin<3
  arrayname='array' ;
end ;

if nargin<4, 
  MAXSIZE = 1e9; % about 1Gb
end ;

whosinfo = whos('array');
if whosinfo.bytes > MAXSIZE
  % linearize first, store original size
  dimensions = size(array) ;
  array = reshape(array, 1, prod(dimensions)) ;

  elementsize = whosinfo.bytes/length(array);
  chunklength = round(MAXSIZE/elementsize);
  numvars = ceil(length(array)/chunklength);
  %fprintf('arraysave2 processes %i chunks:', numvars) ;
  for ix = 1:numvars-1
    eval(sprintf('%s%d = array(%d:%d);',arrayname, ix,(ix-1)*chunklength+1, ...
                 ix*chunklength));
    %fprintf('.') ;
  end
  %fprintf('\n') ;
  eval(sprintf('%s%d = array(%d:end);',arrayname, numvars,(numvars-1)*chunklength+1));
  clear array ix ;
  fprintf('save_struct saves %i chunks:', numvars) ;
  save(outfile_name, '-V7', 'chunklength', 'elementsize', 'numvars', 'dimensions') ;
  for ix = 1:numvars
    save(sprintf('%s_%i.mat', outfile_name, ix), '-V7', sprintf('%s%d', arrayname, ix)) ;
    fprintf('.') ;
  end ;
  fprintf('\n') ;
else
  %eval(sprintf('%s=array;', arrayname)) ;
  save_append(outfile_name, 0, arrayname, array) ;
end

return

function f=separate(str, delim, merge_multiple) ;
% f=separate(str, delimiter, merge_multiple_delimiters) ;
  
if nargin<2, delim=sprintf('\t') ; end ;
if nargin<3, merge_multiple=0 ; end ;
f={} ;
idx=[0 find(str==delim) length(str)+1] ;
if merge_multiple,
  idx2=find(idx(1:end-1)~=idx(2:end)-1) ;
  idx=[idx(idx2) idx(end)] ;
end 
for i=1:length(idx)-1
  f{i}=deblank(str(idx(i)+1:idx(i+1)-1)) ;
end ;

return

function A=spdiag(diagonal) ;
% A=spdiag(diagonal) ;

if size(diagonal,1)>1
  A=spdiags(diagonal, 0, length(diagonal),length(diagonal)) ;
else
  A=spdiags(diagonal', 0, length(diagonal),length(diagonal)) ;
end ;

return

function A=spzeros(m,n,szmax) ;
% A=spzeros(m,n) ;

if nargin<=1
  A=sparse([],[],[],m,m) ;
  return ;
end 
if nargin<=2
  A=sparse([],[],[],m,n) ;
  return ;
end 
A=sparse([],[],[],m,n,szmax) ;

return

function list=who_file(fname) ;
% list=who_file(fname) ;
  
engine = determine_engine() ;
if isequal(engine, 'matlab')
  list=who('-file', fname) ;
else
  L=load(fname) ;
  list=fieldnames(L) ;
end ;

return

function genes = gen_gene_segmentation(P)
% genes = gen_gene_segmentation(P)
% INPUT
% P.genes
% P.segment_ids  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% generate segmentation for all valid transcripts of a gene %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
genes = P.genes;
segment_ids = P.segment_ids;  
  
for g=1:length(genes)
  if ~genes(g).is_complete
    continue;
  end
  exons = genes(g).exons;
  for j=1:length(exons),
    if ~genes(g).transcript_valid(j)
      genes(g).segments{j}=[];
      continue;
    end
    if ~genes(g).transcript_complete(j)
      continue;
    end
    if genes(g).strand=='+'
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    % INSERT SEGMENTS FOR POSITIVE STRAND %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
      isoform = [];
      % insert a row for each 5'utr exon and each intron between
      %----------------------------------------------------
      for k=1:size(genes(g).utr5_exons{j},1)
        isoform = [isoform; [genes(g).utr5_exons{j}(k,1:2) segment_ids.utr5exon]];
        if k<size(genes(g).utr5_exons{j},1)
          isoform = [isoform; [genes(g).utr5_exons{j}(k,2) genes(g).utr5_exons{j}(k+1,1) segment_ids.intron]];
        end
      end
      % if there is an intron directly before the tis site (AGATG):
      %----------------------------------------------------
      if  ~isempty(genes(g).utr5_exons{j})
        if genes(g).utr5_exons{j}(end,2)<genes(g).cds_exons{j}(1,1)
           isoform = [isoform; [genes(g).utr5_exons{j}(end,2) genes(g).cds_exons{j}(1,1) segment_ids.intron]];
        end
      end
      % insert a row for each cds_exon and each intron between
      %----------------------------------------------------
      for k=1:size(genes(g).cds_exons{j},1)
        isoform = [isoform; [genes(g).cds_exons{j}(k,1:2) segment_ids.cds_exon]];
        if k<size(genes(g).cds_exons{j},1)
          isoform = [isoform; [genes(g).cds_exons{j}(k,2) genes(g).cds_exons{j}(k+1,1) segment_ids.intron]];
        else
          % set end of last cds_exon to the beginning of the cdsStop-consensus
          isoform(end,2)=isoform(end,2)-3;
        end
      end   
      % insert an intron between last cds_exon and 3'utr if necessary
      %----------------------------------------------------   
      if  ~isempty(genes(g).utr3_exons{j}) 
        if  genes(g).cds_exons{j}(end,2)<genes(g).utr3_exons{j}(1,1)
          %isoform = [isoform; [genes(g).cds_exons{j}(end,2) genes(g).utr3_exons{j}(1,1) segment_ids.intron]];
          isoform = [isoform; [isoform(end,2) genes(g).utr3_exons{j}(1,1) segment_ids.intron]];
        end
      end
      % insert a row for each 3'utr exon and each intron between
      %----------------------------------------------------
      for k=1:size(genes(g).utr3_exons{j},1)
        if k==1 && isoform(end,3)==segment_ids.cds_exon
          % set end of first 3'utrexon to the beginning of the cdsStop-consensus
          isoform = [isoform; [genes(g).utr3_exons{j}(k,1)-3 genes(g).utr3_exons{j}(k,2) segment_ids.utr3exon]];
        % insert polya_tail if applicable
        elseif k==size(genes(g).utr3_exons{j},1)&&length(genes(g).polya)>=j&&...
               genes(g).polya(j)>genes(g).utr3_exons{j}(k,1)&&genes(g).polya(j)<genes(g).utr3_exons{j}(k,2)
          isoform = [isoform; [genes(g).utr3_exons{j}(k,1) genes(g).polya(j) segment_ids.utr3exon]];
          isoform = [isoform; [genes(g).polya(j) genes(g).utr3_exons{j}(k,2) segment_ids.polya_tail]];
        else
          isoform = [isoform; [genes(g).utr3_exons{j}(k,1:2) segment_ids.utr3exon]];
        end
        if k<size(genes(g).utr3_exons{j},1)
          isoform = [isoform; [genes(g).utr3_exons{j}(k,2) genes(g).utr3_exons{j}(k+1,1) segment_ids.intron]];
        end
      end   
    elseif genes(g).strand=='-'
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    % INSERT SEGMENTS FOR NEGATIVE STRAND %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
      isoform=[];
      % insert a row for each 3'utr exon and each intron between
      %----------------------------------------------------
      for k=1:size(genes(g).utr3_exons{j},1)
        % insert polya_tail if applicable
        if k==1&&length(genes(g).polya)>=j&&genes(g).polya(j)>genes(g).utr3_exons{j}(k,1)&&...
           genes(g).polya(j)<genes(g).utr3_exons{j}(k,2)
          isoform = [isoform; [genes(g).utr3_exons{j}(k,1) genes(g).polya(j) segment_ids.polya_tail]];
          isoform = [isoform; [genes(g).polya(j) genes(g).utr3_exons{j}(k,2) segment_ids.utr3exon]];
        else
          isoform = [isoform; [genes(g).utr3_exons{j}(k,1:2) segment_ids.utr3exon]];
        end
        if k<size(genes(g).utr3_exons{j},1)
          isoform = [isoform; [genes(g).utr3_exons{j}(k,2) genes(g).utr3_exons{j}(k+1,1) segment_ids.intron]];
        else
          isoform(end,2)=isoform(end,2)+3;
        end
      end
      % if there is an intron between cdsStop and first utr3_exon:
      %----------------------------------------------------
      if  ~isempty(genes(g).utr3_exons{j})
        if genes(g).utr3_exons{j}(end,2)<genes(g).cds_exons{j}(1,1)
          isoform(end,2) = isoform(end,2)-3;%undo moving the last utr3-exon position
          isoform = [isoform; [isoform(end,2) genes(g).cds_exons{j}(1,1)+3 segment_ids.intron]];
        end
      end
      % insert a row for each cds_exon and each intron between
      %----------------------------------------------------
      for k=1:size(genes(g).cds_exons{j},1)
        if k==1 %&& ~isempty(isoform)&&isoform(end,3)==segment_ids.utr3exon
          isoform = [isoform; [genes(g).cds_exons{j}(k,1)+3 genes(g).cds_exons{j}(k,2) segment_ids.cds_exon]];
        else
          isoform = [isoform; [genes(g).cds_exons{j}(k,1:2) segment_ids.cds_exon]];
        end  
        if k<size(genes(g).cds_exons{j},1)
          isoform = [isoform; [genes(g).cds_exons{j}(k,2) genes(g).cds_exons{j}(k+1,1) segment_ids.intron]];
        end
      end  
      % insert an intron between last cds_exon and 5'utr if necessary
      %----------------------------------------------------   
      if  ~isempty(genes(g).utr5_exons{j}) 
        if genes(g).cds_exons{j}(end,2)<genes(g).utr5_exons{j}(1,1)
          isoform = [isoform; [genes(g).cds_exons{j}(end,2) genes(g).utr5_exons{j}(1,1) segment_ids.intron]];
        end
      end
      % insert a row for each 5'utr exon and each intron between
      %----------------------------------------------------
      for k=1:size(genes(g).utr5_exons{j},1)
        isoform = [isoform; [genes(g).utr5_exons{j}(k,1:2) segment_ids.utr5exon]];
        if k<size(genes(g).utr5_exons{j},1)
          isoform = [isoform; [genes(g).utr5_exons{j}(k,2) genes(g).utr5_exons{j}(k+1,1) segment_ids.intron]];
        end
      end
    end %strand case
    % assert a tight segmentation
    assert(all(isoform(2:end, 1) == isoform(1:end-1, 2)));
    if ~(all(isoform(2:end, 1) == isoform(1:end-1, 2)))
       if genes(g).utr5_exons{j}(end,2)<genes(g).tis(j)
         fprintf('last utr5_exon stop: %i \n tis: \t%i \n first cds_exon_start: %i\n\n',...
                genes(g).utr5_exons{j}(end,2),genes(g).tis(j),genes(g).cds_exons{j}(1,1))
       else 
       end
    end

    % assert that there is an ORF
    assert(sum(isoform(:,3)==segment_ids.cds_exon ...
               | isoform(:,3)==segment_ids.utr5exon ...
               | isoform(:,3)==segment_ids.utr3exon)==size(genes(g).cds_exons{j},1)+size(genes(g).utr5_exons{j},1)+size(genes(g).utr3_exons{j},1));
    cds_idx = find(isoform(:,3)==segment_ids.cds_exon);
    assert(mod(sum(isoform(cds_idx,2)-isoform(cds_idx)),3)==0)
    %mod(sum(isoform(cds_idx,2)-isoform(cds_idx)),3)
    %keyboard

    %str = [];
    %for k=1:length(cds_idx),
    %  str = [str block.genestr(isoform(cds_idx(k),1):isoform(cds_idx(k),2)-1)];
    %end
    %f = find_open_frames(lower(str),1); 
    %assert(f(1)~=0);
    genes(g).segments{j} = isoform;
  end
end

return



return

function [SEG, LABEL] = get_cand_contents(P)
% [SEG, LABEL] = get_cand_contents(P)


genes = P.genes ;
dist_to_genes = P.intergenic_dist_to_genes;
use_frame = P.use_frame;
which_segments = P.which_segments;
segment_ids = P.segment_ids;
genome_info = P.genome_info;

clear P

%i=min(find([genes.transcript_complete]));
pos_segments_all = ones(1e6, 1) ;%size(genes(i).segments{1},2));
neg_segments_all = ones(1e6, 1) ;%size(genes(i).segments{1},2));
%i=min(find([genes.transcript_complete]));
%pos_segments_all = ones(1e6,size(genes(i).segments{1},2));
%neg_segments_all = ones(1e6,size(genes(i).segments{1},2));
num_pos = 0;
num_neg = 0;

last_gene_stop=1;
[tmp,idx] = sort([genes.start]);
genes = genes(idx);
for i=1:length(genes)
  gene = genes(i);
  if ~gene.is_valid || ~any(gene.transcript_complete)
    last_gene_stop = gene.stop;
    continue
  end
  % find common/consens segments from different truths
  %----------------------------------------------------------------
  if isempty(gene.segments)
    last_gene_stop = gene.stop;
    continue
  end
  if ~use_frame
    cons_segments = gene.segments{1};
    for t = 2:length(gene.segments)
      if isempty(cons_segments) 
        cons_segments=gene.segments{t} ;
        %break
      end
      if ~gene.transcript_complete(t),
        continue ;
      end ;
      if isempty(gene.segments{t})
        continue
      end
      [tmp,idx1,idx2] = intersect(cons_segments(:,1),gene.segments{t}(:,1) ) ;
      ii = find(cons_segments(idx1,2)==gene.segments{t}(idx2,2));
      ii2 = find(cons_segments(idx1(ii),3)==gene.segments{t}(idx2(ii),3));
      cons_segments = cons_segments(idx1(ii(ii2)),:);
      % doesn't work in octave
      % cons_segments = intersect(cons_segments,gene.segments{t},'rows');
    end
    if isempty(cons_segments)
      cons_segments=zeros(0,3);
    end
    
    % extract lines from common/consens segments 
    %----------------------------------------------------------------
    % if any(cons_segments(2:end,2)-cons_segments(1:end-1,2)<0)
    %   keyboard
    % end
    
    pos_segments = cons_segments(find(ismember(cons_segments(:,3), which_segments{1})), :);
    neg_segments = cons_segments(find(ismember(cons_segments(:,3), which_segments{2})), :);
    
    if ismember(segment_ids.intergenic,which_segments{1})
      assert(isempty(pos_segments))
      if gene.start - dist_to_genes(1)>last_gene_stop
        pos_segments = [pos_segments;  max(last_gene_stop,gene.start-dist_to_genes(2)), gene.start segment_ids.intergenic];
      end
    end

    if ismember(segment_ids.intergenic,which_segments{2})
      if gene.start - dist_to_genes(1)>last_gene_stop
        neg_segments = [neg_segments; max(last_gene_stop,gene.start-dist_to_genes(2)), gene.start segment_ids.intergenic];
      end
    end
  else % use_frame
    cds_exons = gene.cds_exons{1};
    % just checks
    if isempty(cds_exons), continue; end
    if genes(i).strand=='+'
      cds_exons(:,2) = cds_exons(:,2)-1;
    else
      cds_exons(:,1) = cds_exons(:,1)+1;
    end
    [cds,valid_splice] =  load_genomic(gene.chr, char(gene.strand),cds_exons(:,1),cds_exons(:,2),genome_info,1);
    f = find_open_frames(cds,1) ;
    too_short = cds_exons(1,2)-cds_exons(1,1)+1<3||cds_exons(end,2)-cds_exons(end,1)+1<3;
    if mod(length(cds),3)~=0 || f(1)~=1 || isempty(cds) || any(cds(1:3)~='atg') || ~any(ismember({'taa', 'tag', 'tga'}, cds(end-2:end)))|| too_short
      warning('invalid transcript') ;
      continue
    end ;
    % create positive examples
    cds_exons = gene.cds_exons{1};
    %% remove ATG and TAG
    cds_exons(1,1) = cds_exons(1,1)+3;
    cds_exons(end,2) = cds_exons(end,2)-3;
    if genes(i).strand=='+'
      cds_exons(cds_exons(:,1)==1,1) = cds_exons(cds_exons(:,1)==1,1)+2;
      cds_exons(cds_exons(:,1)==1,2) = cds_exons(cds_exons(:,2)==2,1)+1;
    else
      cds_exons(cds_exons(:,1)==1,2) = cds_exons(cds_exons(:,1)==1,2)-2;
      cds_exons(cds_exons(:,2)==2,2) = cds_exons(cds_exons(:,2)==2,2)-1;
    end
    pos_segments = cds_exons(:,1:2);
    %% create negative examples
    cds_exons = gene.cds_exons{1};
    %% remove ATG and TAG
    cds_exons(1,1) = cds_exons(1,1)+3;
    cds_exons(end,2) = cds_exons(end,2)-3; 
    neg_segments = cds_exons(cds_exons(:,3)~=0,1:2);
    if genes(i).strand=='+'
      neg_segments = [neg_segments;  [cds_exons(cds_exons(:,3)==0,1)+1 cds_exons(cds_exons(:,3)==0,2)]];
      neg_segments = [neg_segments;  [cds_exons(cds_exons(:,3)==0,1)+2 cds_exons(cds_exons(:,3)==0,2)]];
      neg_segments = [neg_segments;  [cds_exons(cds_exons(:,3)==1,1)+1 cds_exons(cds_exons(:,3)==1,2)]];
      neg_segments = [neg_segments;  [cds_exons(cds_exons(:,3)==2,1)+2 cds_exons(cds_exons(:,3)==2,2)]];
    else
      neg_segments = [neg_segments;  [cds_exons(cds_exons(:,3)==0,1) cds_exons(cds_exons(:,3)==0,2)-1]];
      neg_segments = [neg_segments;  [cds_exons(cds_exons(:,3)==0,1) cds_exons(cds_exons(:,3)==0,2)-2]];
      neg_segments = [neg_segments;  [cds_exons(cds_exons(:,3)==1,1) cds_exons(cds_exons(:,3)==1,2)-1]];
      neg_segments = [neg_segments;  [cds_exons(cds_exons(:,3)==2,1) cds_exons(cds_exons(:,3)==2,2)-2]];
    end
    idx = find(neg_segments(:,2)<=neg_segments(:,1));
    neg_segments(idx,:)=[];
    idx = find(pos_segments(:,2)<=pos_segments(:,1));
    pos_segments(idx,:)=[];
  end
  pos_segments_all(num_pos+1:num_pos+size(pos_segments,1),1:size(pos_segments,2))  = pos_segments;
  neg_segments_all(num_neg+1:num_neg+size(neg_segments,1),1:size(neg_segments,2))  = neg_segments;
  
  
  num_pos = num_pos+size(pos_segments,1);
  num_neg = num_neg+size(neg_segments,1);

  last_gene_stop = gene.stop;
end

% assert(num_pos>0)
% assert(num_neg>0)

pos_segments_all(num_pos+1:end,:)=[];
neg_segments_all(num_neg+1:end,:)=[];

SEG = [pos_segments_all; neg_segments_all];
LABEL = [ones(num_pos,1); -ones(num_neg,1)];
[tmp,idx] = sort(SEG(:,1));
SEG = SEG(idx,:);
LABEL = LABEL(idx);

assert(all(SEG(:,2)-SEG(:,1)>=0))
idx = find(SEG(:,2)-SEG(:,1)==0);
SEG(idx,:) = [];
LABEL(idx) = [];

if ~isempty(find(SEG(2:end,1)-SEG(1:end-1,2)<0)) && ~use_frame
  idx = find(SEG(2:end,1)-SEG(1:end-1,2)<0);
  fprintf(1, 'removing %i overlapping segments\n',length(idx)*2)
  SEG(idx,:)=[];
  LABEL(idx)=[]; 
  assert(all(SEG(2:end,1)-SEG(1:end-1,2)>=0))
end
SEG = SEG(:,1:2);
return

function save_predictions_as_contpred(out_fn_prefix, pos, score, score_name, content_name, contig_name, strand, do_gzip)
% save_predictions_as_wiggle(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)

%assert(issorted(pos))
  
assert(isempty(pos) || isequal(pos(:,1), sort(pos(:,1))))
assert(size(pos,1)==length(score))

if ~exist('do_gzip', 'var'),
  do_gzip = 1 ;
end ;

fn_out=[out_fn_prefix '_' score_name '.cpf'];

if fexist(fn_out) | fexist([fn_out '.gz'])
  fprintf(1, 'file exists, it will NOT be overwritten\n')
  return
end
fprintf(1,'save in content prediction format (cpf): %s\n', fn_out);
[fd msg] = fopen(fn_out, 'w+');

fprintf(fd, '##Seqid\tType\tSource\tStart\tEnd\tStrand\tScore\n') ;

for j=1:min(10000,length(pos))
  fprintf(fd, '%s\t%s\t%s\t%i\t%i\t%c\t%.3f\n', contig_name, content_name, score_name, pos(j,1),pos(j,2), strand, score(j)) ;
end
if length(pos)>10000,
  fprintf(fd, '##Attention: this file has been truncated.\n') ;
end ;
fclose(fd);

if do_gzip,
  ret = unix(sprintf('gzip %s',fn_out)) ;
  if ret~=0
    warning('gzip failed') ;
  end
end 


return

function save_predictions_as_contpred_bin(out_fn_prefix, pos, score, score_name, content_name, contig_name, strand)
% save_predictions_as_sigpred_bin(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand)

%assert(issorted(pos))

assert(isempty(pos) || isequal(pos(:,1), sort(pos(:,1))))
assert(size(pos,1)==length(score))

fn_out=[out_fn_prefix '_' score_name '_cpf.mat'];

if fexist(fn_out) 
  fprintf(1, 'file exists, it will NOT be overwritten\n')
  return
end
fprintf(1,'save in binary content prediction format (cpf): %s\n', fn_out);
save(fn_out, 'contig_name', 'content_name', 'score_name', 'pos', 'strand', 'score') ;

return

function [train,test] = assign_subsets(trivial_regions,fn_trainings_regions,fn_test_regions,genes,SETs,regions_par,tasks)
% [train,test] = assign_subsets(trivial_regions,fn_trainings_regions,fn_test_regions,genes,SETs,regions_par,tasks)


% label_source
% if fexist(fn_genes)&(label_source.use_confirmed|label_source.use_anno)
  
if ~isempty(fn_trainings_regions)&fexist(fn_trainings_regions)
  assert(ismember('training_regions',who('-file',fn_trainings_regions)))
  load(fn_trainings_regions,'training_regions');
else
  training_regions = trivial_regions;
  fprintf('no training region specified, training on whole genome except test region\n')
end

if ~isempty(fn_test_regions)&fexist(fn_test_regions)
  assert(ismember('test_regions',who('-file',fn_test_regions)))
  load(fn_test_regions,'test_regions');
else
  test_regions = trivial_regions(1);
  test_regions.stop = 0;
  test_regions.start = 0;
  fprintf('no test region specified, training on whole genome\n')
end
fprintf('make sure there is no overlap between training and test regions\n')  
if ~(tasks.train)
  training_regions = assert_noRegionOverlap(test_regions,training_regions);
else
  warning('region overlap not checked!! Training and Test region could overlap!\n')
end
rand('seed',1234);
idx = [];

fprintf('Splitting testing regions\n') ;
regions_par.offset = regions_par.offset_test;
test.blocks = split_regions(test_regions, [], regions_par, 0);
fprintf('Done\n') ;

fprintf('Splitting training regions\n') ;
regions_par.offset = regions_par.offset_train;
train.blocks = split_regions(training_regions, genes, regions_par,0);
fprintf('Done\n') ;

if ~isempty(test.blocks)
  test.blocks = rmfield(test.blocks, 'left_dist');
  test.blocks = rmfield(test.blocks, 'right_dist');
  % test.blocks = sort_regions(test.blocks) ;
  % for j=1:length(test.blocks)
  %   test.blocks(j).id = j;
  % end
  [test.blocks, test.split] = make_split(test.blocks,SETS)
else
  test.split = [];
end

% merge training blocks and generate training split
if ~isempty(train.blocks)
  % train.blocks = sort_regions(train.blocks) ; 
  % for j=1:length(train.blocks)
  %   train.blocks(j).id = j;
  % end
  if regions_par.merge_confirmed
    train.blocks = construct_merged_blocks(train.blocks,PAR);
  else
    train.blocks = rmfield(train.blocks,'left_dist');
    train.blocks = rmfield(train.blocks,'right_dist');
  end
  if regions_par.cluster_paralogs
    split_idx = splits_by_paralogs(train.blocks,SETs.num_splits);
    for s = 1:SETs.num_splits
      idx = setfield(idx,sprintf('split_%i',s),split_idx{s});
      error('something wron here; check implementation')
      train.blocks(idx.(sprintf('split_%i',s))) = s;
    end
  else
    [train.blocks, train.split] = make_split(train.blocks,SETs);
  end
else
  train.split = [];
end

return

function [blocks, split] = make_split(blocks,SETs)

blocks_per_split = ceil(length(blocks)/SETs.num_splits);
block_idx  = randperm(length(blocks));
for s = 1:SETs.num_splits
  IDX = (s-1)*blocks_per_split+[1:blocks_per_split];
  IDX(IDX>length(blocks)) = [];
  IDX = block_idx(IDX);
  split.(sprintf('split_%i',s)) = IDX;
% [blocks(IDX).split] = deal(s); % problematic piece of code
  for i=IDX
    blocks(i).split = s;
  end ;
end

y = [];
for s = 1:SETs.num_splits
  y = [y getfield(split,sprintf('split_%i',s))];
end
assert(length(y)==length(unique(y)))

return

function blocks = split_regions(REGS,genes,regions_par, do_assertion)

% blocks=split_regions(REGS,genes,regions_par, do_assertion)
% 
% REGS are splitted into short blocks containing only region_par.min_genes_per_region genes if 
% split criterion holds
% split criterion: splits must have minimal distance of region_par.minCutDist (default 250)
% to both genes
% If region_par.merge_confirmed is set, only 1 gene is put into region,
% regions will be merged later with only complete and confirmed genes

  
if nargin<4
  do_assertion=0;
end
fprintf(1,'Start split_regions...\n')

if ~isfield(regions_par,'minCutDist')
  regions_par.minCutDist = 250;
end

if ~isempty(genes)
  %   load(fn_genes,'genes'); 
  map = zeros(1,length(genes));

  if regions_par.cluster_paralogs
    
    % filter out genes from regions other than the given trivial regions
    % this is necessary for correct detection of paralog blocks
    assert(isequal([genes.id],1:length(genes)))
    
    all_genes_in_REGS_idx = [];
    fprintf(1,'sorting paralogs\n')
    for r=1:length(REGS)
      fprintf(1,'check region %i\r',r)
      contig_genes_idx = find(REGS(r).chr_num==[genes.chr_num]&[genes.strand]==REGS(r).strand);
      all_genes_in_REGS_idx = [all_genes_in_REGS_idx contig_genes_idx];
    end
    for j=1:length(genes)
      fprintf(1,'check genes %i\r',j)
      other_genes_idx = find(~ismember(genes(j).paralogs,all_genes_in_REGS_idx));
      genes(j).paralogs(other_genes_idx) = [];
    end
    all_genes_in_REGS_idx = unique(all_genes_in_REGS_idx);
    genes = genes(all_genes_in_REGS_idx);
  end
else
  warning('no genes: blocks are produced without looking at gene boundaries!!!')
  % keyboard
  genes=[];
  map  =[];
end

blocks=[];
for r=1:size(REGS,2)
  REGS(r).TR_idx = r;
  [blocks,map] = split_regions_helper(blocks,REGS(r),genes,regions_par,map);
end % loop over REGS

if ~isempty(blocks)&&~isempty([blocks.gene_ids])
  assert(isequal(find(map),sort([blocks.gene_ids])))
else
  assert(~any(map))
end
cluster = {};
covered = [];
if isfield(genes,'paralogs')
  idx1 = find(~cellfun('isempty',{genes.paralogs}));
  for i = idx1
    paralogs = unique([map(genes(i).paralogs)]);
    if isempty(intersect(paralogs,covered))
      covered = [covered paralogs];
      cluster{end+1} = paralogs;
      for j= paralogs
        assert(isempty(blocks(j).paralogs))
        blocks(j).paralogs = paralogs;
      end
    else
      idx_c = [];
      covered = unique([covered paralogs]);
      for j=1:length(cluster)
        if any(ismember(paralogs,cluster{j}))
          idx_c = [idx_c,j];
        end
      end
      cluster{idx_c(1)} = [cluster{idx_c(1)},paralogs];
      for j=2:length(idx_c)
        cluster{idx_c(1)} = ([cluster{idx_c(1)},cluster{idx_c(j)}]);
        cluster{idx_c(j)} = [];
      end
      cluster{idx_c(1)} = unique(cluster{idx_c(1)});
      for j= cluster{idx_c(1)}
        blocks(j).paralogs = cluster{idx_c(1)};
      end
      
    end
  end
end
 

if do_assertion
  %check split
  fprintf(1,'check pairwise overlap of blocks\n')
  sum1 = 0;
  %ids = zeros(1,length(blocks));
  for r1 = 1:length(blocks)
    fprintf(1,'%i of %i blocks checked \r',r1,length(blocks)) 
    sum1 = sum1 + blocks(r1).stop-blocks(r1).start+1;
    ids(r1) =  blocks(r1).id;
    num_genes(r1) = length(blocks(r1).gene_ids);
   % rgenes = [];
   % for g=1:num_genes(r1)
   %   gidx = find([genes.id]==blocks(r1).gene_ids(g));
   %   assert(length(gidx)==1)
   %   rgenes = [rgenes genes(gidx)];
   % end
   % assert(max([rgenes.stop])<=blocks(r1).stop-PAR.blocks.minCutDist)
   % assert(min([rgenes.start])>=blocks(r1).start+PAR.blocks.minCutDist)
   % for r2 = (r1+1):size(blocks,2)
   %   check_overlap(blocks(r1),blocks(r2));
   % end
  end
  sum2 = 0;
  for r = 1:size(REGS,2)
    sum2 = sum2 + REGS(r).stop-REGS(r).start+1;
  end
  assert(sum1==sum2)
  assert(all(sort(ids)==1:length(blocks)))
  assert(min(num_genes)>= regions_par.min_genes_per_region)
  assert(isequal(sort([blocks.ids]),1:length(genes)))%all genes are assigned to exactly one region
  fprintf(1,'created %i blocks with min %i and max %i genes\n',length(blocks),min(num_genes), max(num_genes))
  fprintf(1,'num of genes: %ii\n',sum(num_genes))
end



if 0% ~isempty(genes)
  assert(isempty(setdiff([genes.id],[blocks.gene_ids])))
end

return

function [regions,map] = split_regions_helper(regions,REG,GENES,regions_par,map)
% [regions,map] = split_regions_helper(regions,REG,GENES,regions_par,map)
  
new_tr=1;
g=0;
contigstart = REG.start;
contigstop = REG.stop;

if regions_par.merge_confirmed
  min_genes_per_region = 1;
  fpritnf('putting only 1 gene in region; regions will be merged later with only complete genes')
else 
  min_genes_per_region = regions_par.min_genes_per_region;
end


if ~isempty(GENES)
  GENES_chr_num=[GENES.chr_num] ;
  GENES_strand=[GENES.strand] ;
  GENES_start=[GENES.start]+5 ; % hack for some minor problems
  GENES_stop=[GENES.stop]-5 ;   % hack for some minor problems

  idx =  find(REG.chr_num==GENES_chr_num& GENES_strand==REG.strand & ...
              REG.start <= GENES_start & REG.stop >= GENES_stop);
  idx_overlap = find(REG.chr_num==GENES_chr_num & GENES_strand==REG.strand & ...
                     REG.start > GENES_start & REG.start < GENES_stop);
  assert(isempty(idx_overlap))
  idx_overlap = find(REG.chr_num==GENES_chr_num & GENES_strand==REG.strand & ...
                     REG.stop > GENES_start & REG.stop < GENES_stop);
  assert(isempty(idx_overlap))
else 
  idx=[];
end

fprintf(1,'split region %s%s, num of genes found: %i  \r',REG.chr,REG.strand, length(idx)) 
if ~isempty(idx)
  genes = GENES(idx);
  [tmp pos] = sort([genes.start]);
  genes = genes(pos);
  clear pos idx start_pos
else
  block_length = REG.stop-REG.start;
  if ~isfield(regions_par,'max_length')|isempty(regions_par.max_length)
    num_blocks = 1;
  else
    num_blocks = ceil(block_length/regions_par.max_length);
    if num_blocks>0,
      block_length = ceil(block_length/num_blocks);
    else
      block_length = 0 ;
    end ;
  end
  start = REG.start+2*regions_par.minCutDist;
  genes = [];
  %if keyboard_allowed(), keyboard ; end ;
  for j=1:num_blocks
    stop = min(start+block_length, REG.stop);
    genes(j).id = [];
    genes(j).transcript_status =[];
    genes(j).is_complete = [];
    genes(j).start = start+2*regions_par.minCutDist+10;
    genes(j).stop = stop-2*regions_par.minCutDist-10;
    start = stop;
  end
end

%genes
cnt=0 ;
while g<size(genes,2)
  cnt = cnt + 1 ;
  if cnt>g+20, fprintf('break') ; break ; end ;

  region = [];
  region.organism = REG.organism;
  region.chr = REG.chr;
  region.chr_num  = REG.chr_num;
  region.strand  = REG.strand;
  region.id = length(regions)+1;
  region.TR_idx = REG.TR_idx ;
  region.num_fragments = 1;
  region.config = REG.config;
  region.paralogs = [];
  r_genes = [];
  region.gene_ids=[];
  region.gene_status=[];
  region.gene_complete=[];
  genedist_too_short=0;
  if new_tr
    region.start = REG.start;
    region.stop = REG.start;
    new_tr=0;
  else
    region.start = region_end_stop;
  end
  %add genes to region as long as min gene num is not achieved, genedist is too short or more if you are at 
  %the end of the contig and last region would be to short
  while  g<size(genes,2)&&...
        (length(r_genes)<min_genes_per_region||...
         g+min_genes_per_region>size(genes,2)||...
         genedist_too_short ||...
         (length(r_genes)>0 && isfield(region, 'stop') && ...
          (region.stop+regions_par.offset>contigstop||region.stop-regions_par.offset<1))) 
    % is this the last gene of the contig?
    % enough genes for this region collected?
    % still enough genes to fill another region?
    % next gene closer than 2*minCutDist?
    g=g+1;
    r_genes = [r_genes genes(g)];
    region.gene_ids = [region.gene_ids genes(g).id];
    region.gene_status = [region.gene_status max(genes(g).transcript_status)];
    region.gene_complete = [region.gene_complete max(genes(g).is_complete)];
    map(genes(g).id) = region.id;
    gene_stop = max([r_genes.stop]);
    if g==size(genes,2)
      region.stop = REG.stop;
    else
      genedist = genes(g+1).start-gene_stop-1;
      rand_num = ceil(rand*(genedist-2*regions_par.minCutDist));
      region.stop = gene_stop+regions_par.minCutDist+rand_num;
      genedist_too_short = (genedist<=2*regions_par.minCutDist+1);
    end
  end%while add genes
  region.left_dist = min([r_genes.start])- region.start;
  region.right_dist = region.stop - gene_stop;
  %   assert(length(region.gene_ids)>=min_genes_per_region)
  region_end_stop = region.stop+1;
  
  %if ~isempty(GENES)
   % check_block_gene_ids(region, GENES);
  %end

  region.reg_no_overlap(1) = max([contigstart+regions_par.offset region.start]);
  region.reg_no_overlap(2) = min([contigstop-regions_par.offset region.stop]);
  region.start = max([contigstart region.start-regions_par.offset]);
  region.stop = min([contigstop region.stop+regions_par.offset]);
  region.reg_no_overlap = region.reg_no_overlap-region.start+1;
 
  switch region.strand
   case '+',
    region.offset = region.start - 1;
   case '-',
    region.offset = region.stop + 1;
  end
  % keyboard
  regions = [regions region];
end % loop over genes in contig


return

function check_block_gene_ids(blocks, genes)
% find all genes that overlap with a block and check if the gene_ids list of that block is correct and complete
% 
% this is also a check for the blocks being produced correctly without cutting any gene.


chrs = unique([blocks.chr_num]);
strands = unique([blocks.strand]);
wrong_idx = [];
cut_idx = [];

block_chr_num = [blocks.chr_num] ;
genes_chr_num = [genes.chr_num] ;
block_strand  = [blocks.strand] ;
genes_strand  = [genes.strand] ;

for c = chrs
  %cblocks = blocks(block_chr_num==c);
  %cgenes  = genes(genes_chr_num==c); 
  for s=strands 
    %sblocks = cblocks([cblocks.strand]==s);
    %sgenes  = cgenes([cgenes.strand]==s); 
    sblocks = blocks(block_chr_num==c & block_strand==s) ;
    sgenes = genes(genes_chr_num==c & genes_strand==s) ;
    for j=1:length(sblocks),
      if mod(j,100)==0, fprintf('block: %i (%i)\r',j,length(sblocks)), end 
      idx = find([sgenes.stop]>sblocks(j).start & [sgenes.start]<sblocks(j).stop);
      %assert(isequal([sgenes(idx).id],sblocks(j).gene_ids))
      if ~isequal(sort([sgenes(idx).id]),sort(sblocks(j).gene_ids)),
        if keyboard_allowed(),
          keyboard
        end ;
        wrong_idx = [wrong_idx sblocks(j).id]; 
        g = sgenes(idx);
        for k = 1:length(g)
          if g(k).stop>sblocks(j).stop|g(k).start<sblocks(j).start
            cut_idx = [cut_idx g(k).id];
          end
        end
        %keyboard
      end, 
    end%sblocks
  end%strand
end%chr
if length(wrong_idx)>0
  fprintf('%i blocks have incorrect lists of associated genes\n',length(wrong_idx))
end
if length(cut_idx)>0
  fprintf('%i genes have been cut by a block boundary\n',length(cut_idx))
end
return

function [chr, strand, pos, score, content_name, score_name, chr_name_dic, content_name_dic, score_name_dic, success]=read_contpred_bin(contpred_fname)
% [chr, strand, pos, score, content_name, score_name, chr_name_dic, content_name_dic, score_name_dic, success]=read_contpred_bin(contpred_fname)

chr_name_dic = {} ;
score_name_dic = {} ;
content_name_dic = {} ;

chr=[] ; 
content_name = [] ;
score_name = [] ;
strand = '' ;
pos = [] ;
score = [] ;

if ~fexist(contpred_fname),
  success=false ;
  return ;
end ;

cnt = 0; last=0;
while ~last,
  try
    CPF=load(contpred_fname, sprintf('CPF_%i', cnt)) ;
    if ~isfield(CPF, sprintf('CPF_%i', cnt)),
      break
    end ;
  catch
    break ;
  end 
  if ~isempty(CPF)
    eval(sprintf('CPF=CPF.CPF_%i;', cnt)) ;
    cnt=cnt+1 ;
  %else
  %  CPF=load(contpred_fname) ;
  %  last = 1 ;
  end ;

  if isempty(CPF.pos)
    assert(isempty(CPF.score))
    continue;
  end

  if any(ismember(chr_name_dic, CPF.contig_name)),
    contig_idx = find(ismember(chr_name_dic, CPF.contig_name)) ;
  else
    chr_name_dic{end+1} = CPF.contig_name ;
    contig_idx = length(chr_name_dic) ;
  end ;

  if any(ismember(score_name_dic, CPF.score_name)),
    score_idx = find(ismember(score_name_dic, CPF.score_name)) ;
  else
    score_name_dic{end+1} = CPF.score_name ;
    score_idx = length(score_name_dic) ;
  end ;

  if any(ismember(content_name_dic, CPF.content_name)),
    content_idx = find(ismember(content_name_dic, CPF.content_name)) ;
  else
    content_name_dic{end+1} = CPF.content_name ;
    content_idx = length(content_name_dic) ;
  end ;

  chr(end+1:end+size(CPF.pos,1))=contig_idx ;
  score_name(end+1:end+size(CPF.pos,1))=score_idx ;
  content_name(end+1:end+size(CPF.pos,1))=content_idx ;
  pos(end+1:end+size(CPF.pos,1),:)=CPF.pos ;
  score(end+1:end+size(CPF.pos,1))=CPF.score ;
  strand(end+1:end+size(CPF.pos,1))=CPF.strand ;
end ;

success = true ;

return

function [PAR, jobinfo ,all_done]=masterscript_contents(PAR,check_files,verb)
% [PAR, jobinfo ,all_done] = masterscript_contents(PAR,check_files,verb)

%cd ~/svn/projects/genefinding/sensors
%paths

tasks = PAR.tasks.contents;  
%run_locally = tasks.run_locally;
RPROC = PAR.RPROC;
SETs = PAR.SETs;

FN = PAR.FN;  
fn_genome_config = PAR.FN.input.fn_genome_config;
 
if nargin<2 
  check_files =0;
end
if nargin<3 
  verb = 0;
end

fid =1;
fprintf(fid,'\n\n-----------\n');
fprintf(fid,'CONTENT :  %s\n',PAR.Content_name);
fprintf(fid,'-----------\n');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% --- Train CONTENTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PAR.method.submit_batch=1;
all_done=0;
%train_ok = 0;

jobinfo=struct ;

if tasks.prepare_labels
  %fprintf('start training for sensor %s on organism %s with method %s: %s\n\n',sensor,organism,method)
  gen_training_examples4content(PAR) ;
end
if tasks.modelsel | tasks.train
  jobinfo = ms_gfwd_starter(PAR);
  if ~isempty(jobinfo), return ; end ;
end
if tasks.get_train_error
  jobinfo = predict_testplif_geterror(PAR)
  if ~isempty(jobinfo), return ; end ;
end
if tasks.learn_plifs
  calc_plifs(PAR);
end

%if ~train_ok
%  fprintf('something wrong with training files, check first!!  \n')
%end


if ~tasks.pred_on_genome
  all_done = 1 ;%train_ok ;
  return
end


return

function jobinfo = gen_training_examples4content(PAR)
% gen_labels_and_train(PAR)

%paths
  

Content_name = PAR.Content_name;
Content = PAR.Contents.(PAR.Content_name);
num_splits = PAR.SETs.num_splits;
label_source  = PAR.label_source.(Content_name);
info = PAR.info_genes.transcript;


FN = PAR.FN;
fn_genome_config = FN.input.fn_genome_config ;
fn_training_blocks = FN.output.fn_training_blocks;
fn_genes_contents = FN.output.fn_genes_contents ;

input_cont = FN.input_cont.(Content_name);
fn_candsites =  input_cont.fn_candsites;
fn_filter_settings = input_cont.fn_filter_settings;
fn_examples = input_cont.fn_examples;
fn_example_statistics = input_cont.fn_example_statistics;

run_locally = PAR.tasks.contents.run_locally.pos ;

if ~run_locally
  RPROC = PAR.RPROC;
end



jobinfo = [] ;
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% load  trivial_regions, blocks, splits
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

trivial_regions = init_regions(fn_genome_config);


if ~fexist(fn_training_blocks)
  error('training block file does not exist; create it first with prepare_regions')
else
  blocks = load_struct(fn_training_blocks, 'blocks') ;
  load(fn_training_blocks, 'split') ;
  assert(length(fieldnames(split))==num_splits);
end


generate_examples=1;
if fexist([fn_candsites 'PAR.mat'])
  fprintf(1,'files with candidate pos already exists\n')
  Old = load([fn_candsites 'PAR.mat'],'P');
  diff_fields = compare_PAR(Content,Old.P.Content) ;
  if ismember('Content',Content_name)
    warning('label file exists, Content parameters are differnt!')  
    % fprintf('label file exists, Content parameters are differnt!')  
  end
  %[temp, num_files] = unix(sprintf('ls %scontig_*.pos | wc -l',fn_candsites));
  d=dir(sprintf('%scontig_*.pos',fn_candsites));
  num_files = length(d) ;

  if (num_files==length(trivial_regions))
    generate_examples=0;
    fprintf(1,'all cand files found; no label generation\n')
  else
    fprintf(1,'%i/%i cand files missing; start label generation\n',length(trivial_regions)-num_files,length(trivial_regions))
  end
  clear Old
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% generating .pos .label files
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if generate_examples
  fprintf(1,'generating training examples\n')
  jobinfo = rproc_empty(0) ;
  
  P.fn_genes_contents = fn_genes_contents ;
  P.fn_candsites = fn_candsites;
  P.label_source = label_source;
  P.Content = Content; 
  P.info = info;
  
  D.P = P ;
  for j=1:length(trivial_regions)
    D.t_region = trivial_regions(j);
    %keyboard
    filename = sprintf('%scontig_%i%s',fn_candsites,trivial_regions(j).chr_num,trivial_regions(j).strand);
    if fexist(filename)
      continue
    end
    if run_locally
      save_label_files_starter(D);
    else
      [mem_req, time_req, opts] = rproc_memtime_policy('save_label_files_starter',  0, RPROC.options) ;%RPROC.MEMREQ ;

      jobinfo(j) = rproc('save_label_files_starter', D, mem_req, opts, time_req);
      % save_label_files_starter(D)
    end;
    % save_label_files(t_region,PAR,genes)
  end
  if ~isempty(jobinfo),
    [jobinfo_1, num_crashed]=rproc_wait(jobinfo, 10, 1, -1);
  else
    num_crashed = 0 ;
    jobinfo_1 = rproc_empty(0) ;
  end ;
  %[temp, num_files] = unix(sprintf('ls %scontig_*.pos | wc -l',fn_candsites));
  d=dir(sprintf('%scontig_*.pos',fn_candsites));
  num_files = length(d) ;
  if num_files~=length(trivial_regions)
    error('number of files (%i) not equal to number of trivial_regions (%i)',num_files, length(trivial_regions));
  end
  if num_crashed>0, 
    num_crashed
    %return
  end 
  if ~isempty(jobinfo_1)
    rproc_cleanup(jobinfo_1) ;
  end ;
end
clear P

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% generating example files
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

modes = {'train','test'};
for mode=modes
  [filename, exists]=gen_example_filename(Content,fn_filter_settings,fn_examples,mode{1},num_splits);
  if ~exists
        
    P.Content = Content;
    P.num_splits = num_splits;
    P.fn_training_blocks = fn_training_blocks;
    P.fn_candsites = fn_candsites;
    P.fn_filter_settings = fn_filter_settings ;
    P.fn_examples = fn_examples;
    P.fn_example_statistics = fn_example_statistics;

    if ~run_locally
      D.blocks = blocks;
      D.split = split;
      D.P = P;

      [mem_req, time_req, opts] = rproc_memtime_policy('save_content_examples',  0, RPROC.options) ;%RPROC.MEMREQ ;
      
      jobinfo = rproc('save_content_examples', D, mem_req, opts, time_req);

      [jobinfo_1, num_crashed]=rproc_wait(jobinfo, 60, 1, -1);
    else
      save_content_examples(blocks,split,P) ; num_crashed = 0 ;
    end
    if num_crashed>0
      return
    end
  end
end


return

function x = save_content_examples(blocks,split,P)
%save_examples(blocks,split,P)
%
% INPUT 	regions with field Signals with field `Signal.name` with fields:
%			-pos
%			-anno_cover
%			-est_cover
%			-cdna_cover
%	PAR with fields
%			-organism
%			-Content.filter_label
%			-Content.name
%			-Content.consensus
%			-FN.input_sig.fn_examples
%			-FN.input.fn_split
%			-FN.input.fn_regions

%dummy return argument
x=0;

%only for RPROC
if nargin<3
  %addpath ..
  % use the rproc.addpath way to add paths in client processes!!
  %paths
  P = blocks.P;
  split = blocks.split;
  blocks = blocks.blocks;
end

Content = P.Content ; 
num_splits = P.num_splits;

fn_training_blocks = P.fn_training_blocks;
fn_candsites = P.fn_candsites;
fn_filter_settings = P.fn_filter_settings;
fn_examples = P.fn_examples;
fn_example_statistics = P.fn_example_statistics;

Content_name = Content.name;

fields = fieldnames(Content.filter_label);
fprintf( 1, 'save_examples... (%d fields, %d splits)\n', length(fields), num_splits );

min_seg_length = 7;

for ff = 1:length(fields)
   for ss=1:num_splits
    %---------------------------------------------
    %CREATE OUTPUT FILENAME
    %----------------------------------------------  
    filename = gen_example_filename(Content,fn_filter_settings,fn_examples,ff,ss);
    if ~fexist(filename)
      num_examples = 0;
      POS.fn_training_blocks = fn_training_blocks;	 
      split_reg_ids = getfield(split,sprintf('split_%i',ss));
      blocks_idx = find(ismember([blocks.id],split_reg_ids));
      split_regs = blocks(blocks_idx);
      clear blocks_idx split_reg_ids
     

      %-------------------------------------------------- 
      % determine the number of positive and negative 
      % examples in this split
      %-------------------------------------------------- 
      t = cputime;
      num_pos_ex = 0;
      num_neg_ex = 0;
      for r=1:length(split_regs)
        temp_region = retrieve_signals(split_regs(r),fn_candsites,Content_name,{'label'});
	num_pos_ex = num_pos_ex+sum(temp_region.Signals.(Content_name).label == 1);
	num_neg_ex = num_neg_ex+sum(temp_region.Signals.(Content_name).label == -1);
      end
      fprintf(1,'it took %.2fs to determine the number of examples\n', cputime-t);
      vec=subsample_policy([num_pos_ex num_neg_ex], Content_name);
      max_num_pos = vec(1);
      max_num_neg = vec(2);
      reduction_factor_pos = max_num_pos/num_pos_ex;
      reduction_factor_neg = max_num_neg/num_neg_ex;
      num_pos = max_num_pos+max_num_neg;

 
      %-------------------------------------------------- 
      % initialize arrays 
      %-------------------------------------------------- 
      XT = cell(1,num_pos);
      LT = zeros(1,num_pos);
      POS.pos=zeros(1,num_pos);
      POS.region_id=zeros(1,num_pos);
      
      if isfield(Content.filter_label.(fields{ff}), 'USE_ALL')&&...
            Content.filter_label.(fields{ff}).USE_ALL
        Content.Conf_names = {} ;
      end ;
      
      t = cputime;
      fprintf(1,'starting with split %i\n',ss)
      for r=1:length(split_regs)
        if mod(r,100)==0,
          fprintf('region %i/%i\r', r, length(split_regs)) ;
        end ;

        
        if fn_candsites(1)=='~'
          fn_candsites = sprintf('%s/%s', getenv('HOME'), fn_candsites(2:end));
        end
        if ~isfield(split_regs, 'Contents')
          temp_region = retrieve_signals(split_regs(r),fn_candsites,Content_name,{'pos2','label',Content.Conf_names{:}});
        end
        
        
        sig_field = getfield(temp_region.Signals,Content_name);
        if isfield(Content.filter_label.(fields{ff}),'use_label')
          sig_field.label = sig_field.(Content.filter_label.(fields{ff}).use_label);
        end
        
        if ~isfield(split_regs,'seq') 		
          temp_region = load_sequence(temp_region);
        end
        seq = temp_region.seq;

        %---------------------
        %FILTER EXAMPLES
        %---------------------
        if ~isfield(Content.filter_label.(fields{ff}), 'USE_ALL')||...
              ~Content.filter_label.(fields{ff}).USE_ALL
          if ~isfield(Content.filter_label.(fields{ff}),'conf_pos') ...
                && isfield(Content.filter_label.(fields{ff}),'conf')
            error('fix this: filtering will not work')
          end
          idx = feval(Content.filter_fct, temp_region, Content, fields{ff},1);
        else
          idx = 1:length(sig_field.pos);
        end
        idx1 = find(sig_field.pos2<=length(seq));

        idx = intersect(idx,idx1);

	pos_idx = find(sig_field.label==1);
        if reduction_factor_pos<1
	  keep = randperm(length(pos_idx));
	  pos_idx = pos_idx(keep(1:floor(length(pos_idx)*reduction_factor_pos)));
	end
	neg_idx = find(sig_field.label==-1);
	if reduction_factor_neg<1
	  keep = randperm(length(neg_idx));
	  neg_idx = neg_idx(keep(1:floor(length(neg_idx)*reduction_factor_neg)));
	end
	if reduction_factor_pos<1||reduction_factor_neg<1
          if ~isempty(pos_idx) && ~isempty(neg_idx),
            un=[pos_idx', neg_idx'] ;
          elseif isempty(pos_idx),
            un = neg_idx' ;
          else
            un = pos_idx' ;
          end ;
	  idx = intersect(idx, un);
	end

        if isempty(idx); continue; end
        
        %--------------------
        %CREATE XTs
        %--------------------
        XT_temp = cell(1,length(idx));
        rid_temp = zeros(1,length(idx))+temp_region.id;
        for num=1:length(idx)
          if any(~ismember(seq(max(1,sig_field.pos(idx(num))):sig_field.pos2(idx(num))),'ACGT'))
            warning(sprintf(' examples containing non ACGT symbols excluded'))
            continue
          end
          if length(seq(max(1,sig_field.pos(idx(num))):sig_field.pos2(idx(num))))<min_seg_length
            warning(sprintf('very short segments are excluded (<5bp)'))
            continue
          end
          if isempty(seq(max(1,sig_field.pos(idx(num))):sig_field.pos2(idx(num))))
            warning(sprintf('empty range'))
          end
          XT_temp{num} = seq(max(1,sig_field.pos(idx(num))):sig_field.pos2(idx(num)));
        end%loop over rows
        LT_temp= sig_field.label(idx)';
        pos_temp = sig_field.pos(idx)';
        pos2_temp = sig_field.pos2(idx)';
        assert(size(LT,2)==num_examples||LT(num_examples+1)==0)
        assert(num_examples==0||LT(num_examples)~=0)
        
        %[size(XT,2) num_examples+length(idx) ceil(size(XT,2)*r/length(split_regs))]
        if size(XT)<num_examples+length(idx), 
          warning('extending XT') ;
          LT(ceil(size(XT,2)*1.2)) = 0 ;
          
          XT(1,ceil(size(XT,2)*1.2)) = 0 ;
        end ;
        XT(num_examples+1:num_examples+length(idx)) =  XT_temp(:);	
        LT(num_examples+1:num_examples+length(idx)) =  LT_temp;
        POS.pos(num_examples+1:num_examples+length(idx))= pos_temp;
        POS.region_id(num_examples+1:num_examples+length(idx))=rid_temp;
        num_examples = num_examples+length(idx);
      end%loop over blocks
      
      
     
      fprintf(1, 'time required for one split: %.2f\n',cputime-t)
      fprintf(1,'save file %s\n',filename)
      %=== remove zeros
      assert(all(LT(num_examples+1:end)==0))
      LT = LT(1:num_examples);
      XT = XT(1:num_examples);
      
      
      % balance length distribution of negative examples
      %---------------------------------------------------------------
      fprintf(1,'start fit_length_hist\n');
      bins = 100;
      reduce_pos=0;% throw away positive examples if necessary to achieve similar histograms
      
      neg_examples = XT(LT==-1);
      pos_examples = XT(LT==1);
      % if ~use_frame
      neg_examples = fit_length_hist(pos_examples,neg_examples,bins, reduce_pos);
      % end
      % keyboard
      LT = [-ones(size(neg_examples)) ones(size(pos_examples))];
      XT = {neg_examples{:} pos_examples{:}};
      idx = randperm(length(LT));
      LT = LT(idx);
      XT = XT(idx);
      %----------------------------------------------- 
      % remove examples that appear more than once
      %----------------------------------------------- 
      if 0
        [tmp1 idx tmp2] = unique(XT','rows');
        idx = idx(randperm(length(idx)));
        XT = XT(:,idx);
        LT = LT(idx);
        POS.pos = POS.pos(idx);
        POS.region_id = POS.region_id(idx);
      end
     
      %----------------------------------------------- 
      % save matrix in pieces into one file
      %----------------------------------------------- 
      % assert(sum(LT==1)>0);
      % fprintf( 'trying to save %s\n', filename );
      save(filename,'XT');
      % save_matrix(filename, XT, 'XT');
      % save(filename,'-append','LT','POS','PAR');
      save_append(filename, 1,'LT', LT, 'POS', POS, 'P', P);
      % save_append(filename, 1,'LT', LT, 'POS', POS, 'PAR', PAR);
    
    end%if ~fexist(filename)
    
   end%loop over splits
   
   stat = save_example_statistics(Content, fn_filter_settings, ...
                                  fn_examples, fn_example_statistics, ...
                                  num_splits);
end%loop over filter_label fields


return

function [neg pos] = fit_length_hist(pos, neg, steps, reduce_pos, make_plot)

% pos and neg need to be cell arrays of strings

if nargin<5
  make_plots=0;
end

if nargin<4
  reduce_pos=0;
end

% count lengths 
neg_lengths = zeros(1,length(neg));
for j=1:length(neg)
  neg_lengths(j)=length(neg{j});
end
pos_lengths = zeros(1,length(pos));
for j=1:length(pos)
  pos_lengths(j)=length(pos{j});
end

max_len = max([pos_lengths neg_lengths]);
step=ceil(max_len/steps);
bins = logspace(0,log10(max_len),steps);
bins(1)=0;
if max(bins)<max_len,
  bins = [bins max_len];
end

pos_hist = histc(pos_lengths,bins);
neg_hist = histc(neg_lengths,bins);
if make_plots
  figure;
  plot(1:length(bins),neg_hist,1:length(bins),pos_hist)
end

% scale histograms such that they are comparable
neg_hist_norm = neg_hist/length(neg);
pos_hist_norm = pos_hist/length(pos);

if make_plots
  figure;
  plot(1:length(bins),neg_hist_norm,1:length(bins),pos_hist_norm)
end

% calculate the number of sequences that has to be deleted for each length bin
overh = ceil(length(neg)*(neg_hist_norm-pos_hist_norm));

rm_idx=[];
for j = 1:length(neg)  
  len = length(neg{j});
  bin = sum(bins<=len);
  if overh(bin)>0
    overh(bin) = overh(bin)-1;
    rm_idx = [rm_idx j];
  end
end
waste = {neg{rm_idx}};
w_lens = neg_lengths(rm_idx);
neg(rm_idx)=[];
neg_lengths_2 = neg_lengths;
neg_lengths_2(rm_idx)=[];

% sort waste
[w_lens idx] = sort(w_lens,'descend');
waste = {waste{idx}};
clear idx

% recycle
new_neg = {}; 
j = 0;
while j<length(waste)
  j = j+1;
  %idcs = find(overh<0); 
  %idx = idxs(ceil(rand*length(idcs)));
  [tmp idx] = min(overh);
  if tmp>=0
    break;
  elseif w_lens(j)<bins(idx)
    continue;
  elseif idx == length(bins)
    continue;
  else
    len = floor(bins(idx)+rand*(bins(idx+1)-bins(idx)));
    p = min(length(waste{j}),len);
    new_neg = {new_neg{:} waste{j}(1:p)};
    if p<w_lens(j)-30 
      waste = {waste{:} waste{j}(p+1:end)};
      w_lens(end+1) = length(waste{end});
    end
    overh(idx) = overh(idx)+1;
  end
end

neg_hist_2 = histc(neg_lengths_2,bins);

if make_plots
  figure;
  plot(1:length(bins),neg_hist_2,1:length(bins),pos_hist)
end

neg = {neg{:} new_neg{:}};

if make_plots
  for j=1:length(neg)
    neg_lengths_3(j)=length(neg{j});
  end
  neg_hist_3 = histc(neg_lengths_3,bins);

  hist_opt = neg_hist_3-overh;
  figure;
  plot(1:length(bins),neg_hist_3/sum(neg_hist_3),1:length(bins),pos_hist/sum(pos_hist))
end

if reduce_pos
  rm_idx=[];
  for j = 1:length(pos)  
    len = length(pos{j});
    bin = sum(bins<=len);
    if overh(bin)<0
      overh(bin) = overh(bin)+1;
      rm_idx = [rm_idx j];
    end
  end
  pos(rm_idx) = [];
end
%handle = gca;
%prop = get(handle);
%prop.XTick = bins;
%set(handle, 'XTick',bins);

%keyboard 




return


function [ n, bin ] = histc( x, edges );

assert( all( edges == edges ) );

last = repmat( edges(end), size(x) );
bin = zeros( size(x) );
for( i = 1:length(edges) )
  edge = repmat( edges(i), size(x) );
  bin = bin + 1 * ( edge <= x & x <= last );
end;

% --- the following code fails due to
%  /fml/ag-raetsch/share/software/matlab-7.6/toolbox/matlab/datafun/hist.m
% ---
%n = hist( bin, 0:length(edges) );
%n(1) = [];
% --- end of failing code ---

n = zeros( size(edges) );
for( i = 1:length(edges) )
  n(i) = sum( bin == i );
end;


return

function [Data,par] = load_content_data(Content,fn_filter_settings,fn_examples, which, filter_mode,subsample_neg,subsample_pos)
% [Data,par] = load_content_data(Content,fn_filter_settings,fn_examples, which, filter_mode,subsample_neg,subsample_pos)


method = Content.method;

XT = []; LT = [] ;  POS.region_id = []; POS.pos=[]; POS_tmp = []; 

if nargin<4
  subsample_neg=1;
end
if nargin<5
  subsample_pos=1;
end

use_cons = method.use_cons;
% if nargin<6
%   use_cons = 0;
% end
for w = which
  % create filename of input file
  filename = gen_example_filename(Content,fn_filter_settings,fn_examples,filter_mode,w);
  
  fprintf('loading XT, LT and POS from file %s \n',filename)
  L = load(filename,'LT','XT','POS');
  %  L.XT = load_matrix(filename, 'XT');
  % [L.XT, L.LT, P.POS, L.SC_] = subsample(L.XT, L.LT, P.POS, L.SC_, subsample_neg, subsample_pos) ;

  if isfield(L,'LT')
    if size(L.LT,1)>1,
      L.LT = L.LT';
    end
    LT = [LT L.LT] ;
  end
  POS.region_id =[POS.region_id L.POS.region_id];
  POS.pos = [POS.pos L.POS.pos];
  
  % generate feature vectors
  %----------------------------------------------------------------
  dim = 0;
  for o=1:length(method.kernels)
    dim = dim + 4^method.kernels{o}.wordlen;
  end
  xt = zeros(dim,length(L.XT));
  for j=1:length(L.XT) 
    dim = 0;
    for o=1:length(method.kernels)
      [mask hist] = compute_mers(L.XT{j}, method.kernels{o}.wordlen, method.kernels{o}.stepping, method.kernels{o}.offset);
      xt(dim+[1:4^method.kernels{o}.wordlen],j) = hist';
      dim = dim+ 4^method.kernels{o}.wordlen;
    end
  end
  clear L
  XT=[XT xt];
  
end ;

if ~isempty(LT)
  fprintf('%i positive, %i negative examples (%.2f %% positives), sequences of length %i \n',sum(LT==1),sum(LT==-1),sum(LT==1)/length(LT)*100, size(XT,1));
end


Data.LT = LT;
% Data.XT = char(XT) ;
Data.XT = XT;
Data.Pos = POS;
clear XT LT POS

assert(size(Data.XT,2)==length(Data.LT))
% assert(length(Data.LT)==length(Data.Pos.pos))
% assert(length(Data.LT)==length(Data.Pos.region_id))







return

function [mask hist] = compute_mers(string, order, stepping, offset)
%function [mask hist] = compute_mers(string, order, stepping)

mask    = zeros(1,4^order) ;
hist    = zeros(1,4^order) ;

%if nargin<3,
%  stepping = 1;
%end;
%
%if nargin<4,
%  offset = 0;
%end;
%
%if isempty(string), 
%  return; 
%end;
string = lower(char(string)) ;

if ~isempty(find(~ismember(string,'acgt')))
  string(find(~ismember(string,'acgt'))) = 'a';
end;

assert(order<=8);

if length(string)<order,
  return ;
end
t = sg('translate_string', double(string), order, order-1) ;
if length(t)<order
  return ;
end ;

% transform t to matlab coordinates
t = t(1:end-order+1)+1 ;

% throw away kmers if stepping > 1
assert(stepping>0);
%t = t(1:stepping:end);
t = t(end-offset:-stepping:1);

% reduce t to the information whether or not a certain kmer occurs 
u = unique(t) ;

% get histogram counts
t = sort(t);
[dummy loc] = ismember(u, t);
occ = loc - [0 loc(1:end-1)];

% return histogram
hist(u) = occ/sum(occ);

% return "binary histogram" 
% telling whether or not a certain kmer is present
mask(u) = 1/sqrt(length(u)) ;

return

function [jobinfo, fn_pred]=masterscript_contents_test(P, check_files, run_locally, do_gzip, save_as, keep_all_files)
  % [jobinfo ,all_done]=masterscript_contents_test(P, check_files, run_locally, do_gzip, save_as, keep_all_files)

num_splits = P.SETs.num_splits ; 
Content_name = P.Content_name;

FN = P.FN;  
fn_genome_config = FN.input.fn_genome_config;
fn_training_blocks = FN.output.fn_training_blocks;  
fn_test_blocks = FN.output.fn_test_blocks;

fn_pos =  FN.input_cont.(Content_name).fn_pos;
fn_pred = FN.output_cont.(Content_name).fn_pred;
fn_svms = FN.output_cont.(Content_name).fn_SVMs ;
% fn_svms = sprintf('%sbest_partition=', FN.output_cont.(Content.name).fn_SVMs) ;

L = load([fn_svms '1.mat'], 'Train');
Content =  L.Train.PAR.Signal;
clear L 

if nargin<2 
  check_files = 1;
end

if nargin<3 
  run_locally.pos = 1;
  run_locally.predict = 1;
  run_locally.concat = 1;
  run_locally.save = 1;
end

if nargin<4 
  do_gzip = 0;
end

if nargin<5 
  save_as.spf_ascii = 1 ;
  save_as.spf_binary = 1 ;
  save_as.wiggle = 0 ;
end

if nargin<6 
  keep_all_files = 0 ;
end

%if ~run_locally  
  RPROC = P.RPROC;
%else
%  RPROC = struct;
%end

fid =1;
fprintf(fid,'\n\n-----------\n');
fprintf(fid,'CONTENT :  %s\n',Content.name);
fprintf(fid,'-----------\n');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% --- Generate Positions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

jobinfo=struct ;

%[temp, num_files] = unix(sprintf('ls %scontig_*.pos | wc -l',fn_pos));
d=dir(sprintf('%scontig_*.pos',fn_pos));
num_files = length(d) ;
%[temp, num_files1] = unix(sprintf('ls %scontig_*.svm | wc -l',fn_pos));
d=dir(sprintf('%scontig_*.svm',fn_pos));
num_files1=length(d) ;

genome_info = init_genome(fn_genome_config);
if isempty(num_files)||(num_files<length(genome_info.contig_names)*2)||...
      isempty(num_files1)||(num_files1<length(genome_info.contig_names)*2)
  jobinfo = gen_all_positions(fn_genome_config,fn_pos,Content,fn_training_blocks,fn_test_blocks,num_splits, run_locally.pos, RPROC);
  if ~isempty(jobinfo),
    fprintf('generating genome position files\n')
    jobinfo = rproc_resubmit(jobinfo);
    jobinfo = rproc_wait(jobinfo, 5, 1, -1); 
  end ; 
else
  fprintf('genome position files already exist\n')
end

avg_all_svms=0;
subtract_diff=1;
% keyboard
jobinfo = genomewide_predictions(fn_genome_config,fn_pos,fn_pred,fn_svms,num_splits,avg_all_svms,run_locally.predict, RPROC);
fprintf('all %s predictions done\n',Content.name)


% jobinfo = convert_predictions2confs(fn_genome_config,fn_pred,fn_svms,num_splits,run_locally, RPROC);
% fprintf('conversion done\n')

%run_locally_concat = run_locally ;
jobinfo = concat_genomewide_predictons(fn_genome_config,fn_pos,fn_pred, num_splits,avg_all_svms,subtract_diff, run_locally.concat, RPROC);
fprintf('concatination done\n')

resolution = Content.export_settings.resolution ;
conf_cum_thresh = Content.export_settings.conf_cum_thresh;

%run_locally_save = run_locally ;
inventory = {'output', 'pos'} ;
[jobinfo] = save_predictions(fn_genome_config, fn_pred, ...
                             Content_name, resolution, ...
                             conf_cum_thresh, '+-', ...
                             run_locally.save, RPROC, ...
                             do_gzip, save_as, inventory);
fprintf('saving done\n')


if check_files
  % fprintf('checking files \n')
  % check_genomewide_output(P) % this fct compares prediction results with test results from training (cross_validation)
  % all_good = check_predictions(fn_genome_config,fn_pred,Content_name,resolution)
else
  fprintf('WARNING: files not checked\n')
end


fprintf('%s COMPLETE !\n\n',Content.name)


if ~keep_all_files
 %  unix(sprintf('rm -rf %s*_split=*', FN.output_sig.(Content_name).fn_pred));
end

return

function [RFV, RSV, train_test_overlap] = svm_content_predict(Train, Data, svmFileName,outFileName,pred_from_seq, check_overlap);
% [RFV, RSV] = svm_predict(Train, Data, svmFileName,outFileName);
%
%------------------------------------------------------------------------------%
% deal with parameters 
%------------------------------------------------------------------------------%
% for usage with rproc

if nargin==1
  Data  = Train.Data;
  svmFileName = Train.svmFileName;
  outFileName = Train.outFileName;
  Train = Train.Train;
end

method = Train.PAR.Signal.method ;

if ~exist('check_overlap', 'var'),
  check_overlap=1 ;
end ;

sg_ver = sg('get_version');

assert(size(Data.pos,1)==1|size(Data.pos,2)==2|isempty(Data.pos))
if size(Data.pos,1)~=1
  Data.pos = Data.pos';
end

if length(Train.w)~=sum(4.^[3:6]),
  fprintf('Assumed degrees 3:6 are not used in trained content predictor (dim %i)\n', length(Train.w)) ;
  Train.w(sum(4.^[3:6]))=0 ;
end ;

assert(isequal(sort(Data.pos), Data.pos));
sg('init_dyn_prog', 1);
sg('precompute_content_svms', Data.seq, int32(Data.pos-1), Train.w);
output = sg('get_lin_feat');
% output = output - output(end)/Data.pos(end).*Data.pos;

%------------------------------------------------------------------------------%
% save output
%------------------------------------------------------------------------------%

PAR = Data.PAR;
PAR.Signal = Train.PAR.Signal;

if isfield(Data, 'LT')
  RSV = calcrocscore(output, Data.LT); 
  RFV= calcrfcscore(output, Data.LT);
  LT = Data.LT;
  %   save( outFileName, 'output','LT','PAR', 'svmFileName','RSV','RFV' );
  POS = Data.Pos;
  fprintf( 'save output to %s\n',outFileName );
  inventory = {'output', 'POS'};
  %save( outFileName, 'inventory', 'output','LT','POS', 'Train', 'svmFileName','RSV','RFV','sg_ver','PAR' );
  save( outFileName, 'inventory', 'output','LT','POS', 'svmFileName','RSV','RFV','sg_ver','PAR' );
else
  pos = Data.pos;
  fprintf( 'save output to %s\n',outFileName );
  inventory = {'output', 'pos'};
  %save( outFileName, 'inventory', 'output', 'pos', 'Train', 'svmFileName','sg_ver','PAR' );
  save( outFileName, 'inventory', 'output', 'pos', 'svmFileName','sg_ver','PAR' );
end

return
%%%%TEST



L = load(['galaxy/testit/content_cds_exon_classifier_files/examples_settings=2_split=1.mat']);
LL = load('galaxy/testit/content_cds_exon_classifier_files/output_SVMCOMBINED/MS/partition=1.mat')

for i=1:length(L.LT)
  sg('init_dyn_prog', 1);
  sg('precompute_content_svms', L.XT{i},int32(0:length(L.XT{i})-1),LL.Train.w);
  output{i} = sg('get_lin_feat');
  output1(i) = output{i}(end);
end

%%%


XT = zeros(5440,length(L.XT));
method = LL.Train.PAR.Signal.method;
for j=1:length(L.XT) 
  dim = 0;
  for o=1:length(method.kernels)
    [mask hist] = compute_mers(L.XT{j}, method.kernels{o}.wordlen, method.kernels{o}.stepping, method.kernels{o}.offset);
    XT(dim+[1:4^method.kernels{o}.wordlen],j) = hist';
    dim = dim+ 4^method.kernels{o}.wordlen;
  end
end
init_svm_and_kernels(method,0);
sg('clean_features', 'TRAIN');
D.XT = LL.Train.XT;
D.LT = LL.Train.LT;
setFeatures(method, D,'TRAIN',0,0);
clear D

sg('clean_features', 'TEST' );
D.XT = XT;
setFeatures(method, D,'TEST',0,0);

sg('set_svm',LL.Train.b,LL.Train.alphas);
output2 = sg( 'svm_classify' );

return

function found = issubset(A,B)
% function found = issubset(A,B)
%
% returns true if A is a subset of B, where 
% both A and B are vectors with 1 where elements exists and 0 otherwise.
found = isequal(find(A),intersect(find(A),find(B)));
return

function ret=keyboard_allowed()
% ret=keyboard_allowed()
%
% returns 1, if a keyboard command would be allowed

global g_ignore_keyboard
global THIS_IS_A_RPROC_PROCESS  

if isequal(g_ignore_keyboard, 1),
  ret=0 ;
  return ;
end ;

if isequal(THIS_IS_A_RPROC_PROCESS, 1),
  ret=0 ;
  return ;
end ;

ret=1 ;


return

function f=bigunique(s) ;
% f=bigunique(s) ;

f=[] ; step=ceil(length(s)/10) ;
for i=1:10,
  idx=1+(i-1)*step:min(length(s), i*step) ;
  f=unique([f unique(s(idx))]) ;
end ;


return

function x = load_and_train(P, Train_Data)

% load_and_train(P)
%
% INPUT: P with fields:   
%         - fn_set,             file name of sets to be loaded 
%         - SETs.which_svm,     specifies the sets on which training is
%                               performed (e.g [1 2 3])
%         - SETs.train_subsample_neg  number between 0-1: for subsampling of negative examples, 
%                                0: setting label of examples labeled with 0 to -1   
%                               -1: removing examples labeled with 0 
%                               default(1) no subsampling 
%         - SETs.train_subsample_pos  number between 0-1: for subsampling of positive examples,                             
%                               default(1) no subsampling 
%         - par,                window around consensus to be used  
%         - use_cons            specifies if conservation is to be used
%         - fn_SVMs             file nmame to save results  
%  
%   
% OUTPUT: Train_Data with fields:
%  (XT,LT,SC,pos,reg,str,P)
%  saved to file P.fn_SVMs
  
%if( ~P.method.submit_batch )
%  paths;
%end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% load data of training set
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SETs = P.SETs;
Signal = P.Signal;
fn_filter_settings = P.fn_filter_settings;
fn_examples = P.fn_examples;

which_train = SETs.which_train;
train_fct = Signal.method.train_fct;

if ~isfield(SETs,'train_subsample_pos') && isfield(SETs,'subsample_pos')
  subsample_pos = SETs.subsample_pos ;
  error('warning: please rename field from subsample_pos to train_subsample_pos') ;
else
  subsample_pos = SETs.train_subsample_pos ;
end

if ~isfield(SETs,'train_subsample_neg') && isfield(SETs,'subsample_neg')
  subsample_neg = subsample_neg ;
  error('warning: please rename field from subsample_neg to train_subsample_neg') ;
else
  subsample_neg = SETs.train_subsample_neg ;
end

if ~exist('Train_Data')
  Train_Data = feval(Signal.load_fct,Signal,fn_filter_settings,fn_examples, which_train, 'train',subsample_neg, subsample_pos);
end
Train_Data.P = P;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% TRAIN 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

t = cputime;
feval(train_fct,Train_Data);
fprintf('\ntraining took %.2f min\n',(cputime-t)/60);
% if PAR.method.name=='SVM'
%   tssTrain(Train_Data)
% elseif PARn.method.name=='PWM'
%   hmm_train(Train_Data)
% else 
%   error('unknown method');
% end

% dummy return argument
x=0;

return

function PAR=A_thaliana_acc_SVM_WD(PAR,create_dir)
  
if nargin<1
  PAR=init_PAR;
elseif nargin<2
  create_dir=0;
end
PAR = A_thaliana(PAR);
PAR = acceptor(PAR);
PAR = SVM_WD(PAR);

PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = make_partitions(PAR);
PAR = create_filenames(PAR);
if create_dir
  create_directories(PAR);
end

return

function PAR=A_thaliana_don_SVM_WD(PAR,create_dir)

if nargin<1
  PAR = init_PAR;
end
if nargin<2
  create_dir=0;
end
  
PAR = A_thaliana(PAR);
PAR = donor(PAR);
PAR = SVM_WD(PAR);

PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = make_partitions(PAR);
PAR = create_filenames(PAR);
if create_dir
  create_directories(PAR);
end


return

function PAR=C_elegans_acc_SVM_WD(PAR,create_dir)
  
if nargin<1
  PAR=[];
end
if nargin<2
  create_dir=0;
end
  
PAR = C_elegans(PAR);
PAR = acceptor(PAR);
PAR = SVM_WD(PAR);

PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = make_partitions(PAR);
PAR = create_filenames(PAR);

if create_dir
  create_directories(PAR);
end

return

function PAR=C_elegans_don_SVM_WD(PAR,create_dir)
  
if nargin<1
  PAR=[];
end
if nargin<2
  create_dir=0;
end
  
PAR = C_elegans(PAR);
PAR = donor(PAR);
PAR = SVM_WD(PAR);

PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = make_partitions(PAR);
PAR = create_filenames(PAR);

if create_dir
  create_directories(PAR);
end

return

function PAR=D_melanogaster_acc_SVM_WD(PAR)
  
if nargin<1
  PAR=[];
end
  
PAR = D_melanogaster(PAR);
PAR = acceptor(PAR);
PAR = SVM_WD(PAR);

PAR = make_partitions(PAR);

PAR.date=date;
PAR.user='Jonas';
PAR.exp_name='acceptor';

%PAR.method.kernels{1}.lwin=PAR.Signals.(PAR.Signal_name).lwin;
%PAR.method.kernels{1}.rwin=PAR.Signals.(PAR.Signal_name).rwin;
PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = create_filenames(PAR);

PAR.tasks.modelsel_on = 1;
PAR.tasks.learn_plifs_on = 1;
PAR.tasks.pred_from_seq = 0;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;

% this is the setting for the splice paper
PAR.regions.cluster_paralogs = 0;

PAR.RPROC.MEMREQ = 8000 ;

PAR.method.kernels{1}.order=22 ;
PAR.method.par_ms.C = 3 ;

return

function PAR=D_melanogaster_don_SVM_WD(PAR)
  
if nargin<1
  PAR=[];
end
  
PAR = D_melanogaster(PAR);
PAR = donor(PAR);
PAR = SVM_WD(PAR);

PAR = make_partitions(PAR);

PAR.date=date;
PAR.user='Jonas';
PAR.exp_name='donor';

%PAR.method.kernels{1}.lwin=PAR.Signals.(PAR.Signal_name).lwin;
%PAR.method.kernels{1}.rwin=PAR.Signals.(PAR.Signal_name).rwin;
PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = create_filenames(PAR);

PAR.tasks.modelsel_on = 1;
PAR.tasks.learn_plifs_on =1;
PAR.tasks.pred_from_seq=0;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;

% this is the setting for the splice paper
PAR.regions.cluster_paralogs = 0;

PAR.RPROC.MEMREQ = 8000 ;

PAR.method.kernels{1}.order=22 ;
PAR.method.par_ms.C = 3 ;

return

function PAR=D_rerio_acc_SVM_WD(PAR)
  
if nargin<1
  PAR=[];
end
  
PAR = D_rerio(PAR);
PAR = acceptor(PAR);

% test only one model ... 
PAR.Signals.(PAR.Signal_name).order = 22;
PAR.Signals.(PAR.Signal_name).shift = 0;
PAR.Signals.(PAR.Signal_name).C = 3; 

PAR = SVM_WD(PAR);

PAR = make_partitions(PAR);

PAR.date=date;
PAR.user='Jonas';
PAR.exp_name='acceptor';

PAR.method.kernels{1}.lwin=PAR.Signals.(PAR.Signal_name).lwin;
PAR.method.kernels{1}.rwin=PAR.Signals.(PAR.Signal_name).rwin;
PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = create_filenames(PAR);

PAR.tasks.modelsel_on = 1;
PAR.tasks.learn_plifs_on =1;
PAR.tasks.pred_from_seq=0;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;

PAR.SETs.train_subsample_pos = 1 ;
PAR.SETs.train_subsample_neg = 0.33 ;

PAR.RPROC.options.ncpus = 12 ; 

return

function PAR=D_rerio_don_SVM_WD(PAR)
  
if nargin<1
  PAR=[];
end
  
PAR = D_rerio(PAR);
PAR = donor(PAR);

% test only one model ... 
PAR.Signals.(PAR.Signal_name).order = 22;
PAR.Signals.(PAR.Signal_name).shift = 0;
PAR.Signals.(PAR.Signal_name).C = 3; 

PAR = SVM_WD(PAR);

PAR = make_partitions(PAR);

PAR.date=date;
PAR.user='Jonas';
PAR.exp_name='donor';

PAR.method.kernels{1}.lwin=PAR.Signals.(PAR.Signal_name).lwin;
PAR.method.kernels{1}.rwin=PAR.Signals.(PAR.Signal_name).rwin;
PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = create_filenames(PAR);

PAR.tasks.modelsel_on = 1;
PAR.tasks.learn_plifs_on =1;
PAR.tasks.pred_from_seq=0;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;

PAR.SETs.train_subsample_pos = 1 ;
PAR.SETs.train_subsample_neg = 0.33 ;

PAR.RPROC.options.ncpus = 12 ; 

return

function blocks = add_signals2blocks(blocks, PAR, signal_names, which_dir, offset,which_fields)
% blocks = add_signals2blocks(blocks, PAR, signal_names, which_dir, offset)

if nargin<5
  warning('I think this should be a parameter')
  offset = 100;  
end
if nargin<4 || isempty(which_dir)
  which_dir = 'pred_dir' ;
end
genome_info = init_genome(PAR.FN.input.fn_genome_config);
which_pos.local = 1;
which_pos.contig = 1;
base_dirs    = {};

for i=1:length(signal_names),
  if isequal(which_dir,'pred_dir')'
    tmp = PAR.FN.output_sig.(signal_names{i});
    base_dirs{i} = tmp.fn_pred;
    fields = {'Conf_cum'};
  elseif isequal(which_dir,'cand_dir')'
    tmp = PAR.FN.input_sig.(signal_names{i});
    base_dirs{i} = tmp.fn_candsites;
    fields = {'label','altgenic','alt_intron','est_conf','est_conf_skip','cDNA_conf','cDNA_conf_skip'};
  end
end
if nargin>5
  fields = which_fields;
end

if ~isfield(blocks,'Signals')
  blocks(1).Signals=struct;
end

% blocks = load_sequence(blocks);
fprintf('Processing %i blocks: ', length(blocks));
for r=1:length(blocks)
  block=blocks(r);
  if mod(r,50)==0, fprintf('%i .. ', r); end ;
  % fill the field genestr
  for s=1:length(signal_names),
    f_name = sprintf('%scontig_%i%s.pos', base_dirs{s},block.chr_num,block.strand);
    d = dir(f_name) ;
    if isempty(d), error('file not found: %s',f_name);end
    if d.bytes>0
      block = retrieve_signals(block, base_dirs{s},signal_names{s}, fields, which_pos, offset);
      % block = retrieve_signals(block, base_dirs{s},signal_names{s}, fields,which_pos,0);
    else
      warning('hack!\n')
      block.Signals.(signal_names{s})=struct ;
      block.Signals.(signal_names{s}).pos = [];
      block.Signals.(signal_names{s}).contig_pos = [];
      for k=1:length(fields)
        block.Signals.(signal_names{s}).(fields{k}) =  [];
      end ;
    end
  end
  blocks(r)=block;
end
fprintf('Done.\n') ;

% load /fml/ag-raetsch/share/projects/nGASP/LSL/test_data/cat_1/blocks_2006-12-19.mat

% eof

return

function genes = check_genes(genes, genome_info,which_checks) ;
% genes = check_genes(genes, genome_info,which_checks) ;

% TO DO CHECK SPLICE GRAPH: 
%  CHECK if splice graoh is connected. SHOULd WORK NOW
%  
%addpath ~/svn/tools/genomes/

min_boundary_dist=20 ;

if nargin<3 | isempty(which_checks), 
  fprintf('no checks performed\n')
  return
end

if ~isfield(which_checks,'exons_sorted')
  which_checks.exons_sorted = 1;
end ;
if ~isfield(which_checks,'intron_length')
  which_checks.intron_length = 0;
end
if ~isfield(which_checks,'exon_length')
  which_checks.exon_length = 0;
end
if ~isfield(which_checks,'splicesites');
  which_checks.splicesites = 0;
end
if ~isfield(which_checks,'orf')
  which_checks.orf = 0;
end
if ~isfield(which_checks,'gene_length')
   which_checks.gene_length = 0;
end
if ~isfield(which_checks,'graph')
  which_checks.graph = 0;
end ;
if ~isfield(which_checks,'transacc')
  which_checks.transacc = 0;
end ;
if ~isfield(which_checks,'complete')
  which_checks.complete = 0;
end ;
which_checks;


num_invalid_transcripts = 0 ;
num_invalid_genes = 0 ;
num_no_exons = 0;
num_no_trans = 0;
num_splice_not_ok = 0 ;
num_intron_not_ok = 0 ;
num_orf_not_ok = 0 ;
num_tis_not_ok = 0 ;
num_cds_exon_not_ok = 0 ;
num_stop_not_ok = 0 ;
num_graphs_not_ok = 0 ;
num_transacc_not_ok = 0;
num_at_boundary = 0;

genes_not_ok = [];

fprintf('checking %i genes: ', length(genes)) ;
for i=1:length(genes), 
  if mod(i,1000)==0||i==length(genes) 
    %fprintf(['gene=%i  invalid_genes=%i  invalid_transcripts=%i  no_exons=%i no_transcripts=%i ' ...
    %         'num_splice_not_ok=%i cds_problems=%i,%i,%i,%i num_graphs_not_ok =%i num_trans_not_ok =%i \r'],...
    %        i,num_invalid_genes,num_invalid_transcripts , num_no_exons,num_no_trans,num_splice_not_ok,...
    %        num_tis_not_ok, num_cds_exon_not_ok, num_orf_not_ok,
    %        num_stop_not_ok, num_graphs_not_ok,
    %        num_transacc_not_ok); 
    fprintf('%i ... ', i) ;
  end
  transcript_valid = [] ; 
  genes(i).is_valid = 0 ; % make sure it is set...
  is_valid = 1 ;
  
  if isempty(genes(i).exons)||all(cellfun('isempty',genes(i).exons))
    genes(i).is_valid = 0;
    num_invalid_genes = num_invalid_genes+1;
    num_no_exons = num_no_exons+1;
    continue
  end
  if isempty(genes(i).transcripts)||all(cellfun('isempty',genes(i).transcripts))
    genes(i).is_valid = 0;
    num_no_trans = num_no_trans+1;
    num_invalid_genes = num_invalid_genes+1;
    continue
  end
  
  
  % CHECK TRANSACC CONSENSUS
  trans_not_ok = 0;
  rm_trans=[];
  if which_checks.transacc 
    [transacc,idx] = unique(genes(i).transacc);
    for trans_=transacc'
      [mRNA, tmp, ok] =  load_genomic(genes(i).chr, char(genes(i).strand),...
                                      trans_-5, trans_+5,genome_info,1);
      if ~ok,
        warning('load genomic failed') ;
      else
        if ~isequal(mRNA(4:5),'ag')% &genes(i).strand=='-';
          mRNA(2:6)
          idx=find(ismember(genes(i).transacc,trans_));
          genes(i).transacc_info
          trans_not_ok = trans_not_ok+1;
          rm_trans=[rm_trans trans_];
        end
      end ;
    end
    idx=find(ismember(genes(i).transacc,rm_trans));
    if trans_not_ok
      warning('trans_not_ok');
      %keyboard
    end
    genes(i).transacc(idx)=[];
    genes(i).transacc_info(idx)=[];
    genes(i).transacc_conf(idx)=[];
    num_transacc_not_ok = num_transacc_not_ok+trans_not_ok; 
    
  end

  for j=1:length(genes(i).exons),


    [tmp,idx] = sort(genes(i).exons{j}(:,1)) ;
    
    genes(i).exons{j} = genes(i).exons{j}(idx,:) ;
    if ~isempty(genes(i).exons_confirmed) && ~isempty(genes(i).exons_confirmed{j})
      genes(i).exons_confirmed{j} = genes(i).exons_confirmed{j}(idx,:) ;
    end
    

    %%% CHECK BOUNDARY
    % keyboard
    boundary_not_ok=0;
    if isempty(genes(i).start) || isempty(genes(i).stop)
      boundary_not_ok=1;
    else
      exons = genes(i).exons{j};
      if any(any(exons(:,1:2)<genes(i).start-1|exons(:,1:2)>genes(i).stop+1))
        warning('overlaying transcript found');
        boundary_not_ok=1;
      end
    end ;
   
    %%% CHECK EXONS SORTED
    exons_not_ok = 0 ;
    if isempty(genes(i).exons{j}),
      exons_not_ok = 1 ;
    else
      if which_checks.exons_sorted
        [tmp,idx] = sort(genes(i).exons{j}(:,2)) ;
        if ~isequal(idx',1:length(idx)), 
          fprintf('exons not sorted\n') ;
          exons_not_ok = 1 ;
        end ;
      end
    end ;
    %%% CHECK EXON LENGTH
    if which_checks.exon_length
      exon_length = genes(i).exons{j}(:,2)-genes(i).exons{j}(:,1);
      if any(exon_length<10)
        exons_not_ok = 1;
      end ;
    end

    
    %%% SPLICE SITE CONSENSUS
    splice_not_ok = 0 ;  
    
    if which_checks.splicesites && size(genes(i).exons{j},1)>1 && isempty(genes(i).exons{j}),
      % if any(genes(i).exons{j}(:,1)>genes(i).exons{j}(:,2))
      %    valid_splice(1:2,1:2)=0
      % else
      [mRNA, valid_splice, ok] =  load_genomic(genes(i).chr, char(genes(i).strand),...
                                              genes(i).exons{j}(:,1), ...
                                              genes(i).exons{j}(:,2),genome_info,1);
      if ~ok,
        splice_not_ok = 1 ;
      else
        % end
        if genes(i).strand=='+'
          splice_not_ok = sum(sum(valid_splice(2:end-1,:)')<2) + ~valid_splice(1,2)+~valid_splice(end,1);
        else
          splice_not_ok = sum(sum(valid_splice(2:end-1,:)')<2) + ~valid_splice(1,1)+~valid_splice(end,2);
        end
      end ;
      if splice_not_ok
        genes_not_ok = [genes_not_ok i];
      end
      num_splice_not_ok = num_splice_not_ok+splice_not_ok; 
    end
    
    %%% CHECK INTRON LENGTH
    intron_not_ok = 0 ;
    if which_checks.intron_length
      intron_length = genes(i).exons{j}(2:end,1)-genes(i).exons{j}(1:end-1,2);
      if any(intron_length>genome_info.max_intron_len)|...      
            any(intron_length<genome_info.min_intron_len)
        
        % fprintf('\nintron lengths: %i - %i \n',min(intron_length),max(intron_length)) ;
        intron_not_ok = sum(intron_length>genome_info.max_intron_len |...
                            intron_length<genome_info.min_intron_len);
      end ;
      num_intron_not_ok = num_intron_not_ok+intron_not_ok; 
    end
    transcript_valid(j) = ~any([exons_not_ok splice_not_ok intron_not_ok boundary_not_ok]);
  end %%loop over exons
  
  %%% CHECK ORF
  for j=1:length(genes(i).cds_exons), 
    if which_checks.orf ,
      cds_exons = genes(i).cds_exons{j};
      if isempty(cds_exons), 
        transcript_valid(j)=0;
        continue; 
      end
      if all(cds_exons(:)==0),
         genes(i).cds_exons{j}=[] ;
         transcript_valid(j)=0;
         continue ;
      end ;
      if genes(i).strand=='+'
        if ~isempty(genes(i).tis)
          if ~(genes(i).tis{j}==cds_exons(1,1)),
            warning('this was an assert') ;
            transcript_valid(j) =0;
            %keyboard
            continue ;
          end ;
        end
        if ~isempty(genes(i).cdsStop)
          if ~(genes(i).cdsStop{j}==cds_exons(end,2)-3),
            warning('this was an assert') ;
            transcript_valid(j) =0;
            %keyboard ;
            continue ;
          end ;
        end
        cds_exons(:,2) = cds_exons(:,2)-1;
      else
        if ~isempty(genes(i).tis)
          assert(genes(i).tis{j}==cds_exons(end,2))
        end
        if ~isempty(genes(i).cdsStop) 
          assert(genes(i).cdsStop{j}==cds_exons(1,1)+3);
        end
        cds_exons(:,1) = cds_exons(:,1)+1;
      end

      [cds, valid_splice, ok] =  load_genomic(genes(i).chr, char(genes(i).strand), cds_exons(:,1), cds_exons(:,2), genome_info, 1);
      if ~ok,
        transcript_valid(j) =0;
        continue ;
      end ;
      f = find_open_frames(cds,1) ;

      % keyboard
      % if first (last) cds_exon is smaler than 3 the start (stop) codon is 
      % spliced together, thus we will not have predictions for it
      too_short = cds_exons(1,2)-cds_exons(1,1)+1<3 || cds_exons(end,2)-cds_exons(end,1)+1<3;

      if ~any(ismember({'taa', 'tag', 'tga'}, cds(end-2:end)))
        transcript_valid(j) =0;
        num_stop_not_ok = num_stop_not_ok + 1;
      end ;
      if any(cds(1:3)~='atg'),
        transcript_valid(j) =0;
        num_tis_not_ok = num_tis_not_ok + 1;
      end ;
      if too_short,
        num_cds_exon_ok = num_cds_exon_not_ok + 1 ;
        transcript_valid(j) =0;
      end ;
      if mod(length(cds),3)~=0 || f(1)~=1 || isempty(cds) 
        num_orf_not_ok  = num_orf_not_ok + 1 ;
        transcript_valid(j) =0;
      end ;
    end
  end ; %% loop over cds_exons

  num_invalid_transcripts = num_invalid_transcripts + sum(~transcript_valid);

  % if isfield(genes(i),'transcript_valid') && ~isempty(genes(i).transcript_valid)
  %   genes(i).transcript_valid = genes(i).transcript_valid.*transcript_valid ;  
  % else
  genes(i).transcript_valid = transcript_valid ;
  % end
    
  %%% check genes length (only the region of the valid transcripts)
  length_not_ok = 0;
  if which_checks.gene_length,
    starts = [] ;
    ends = [] ;
    for j=1:length(genes(i).exons),
      if genes(i).transcript_valid(j)&&~isempty(genes(i).exons{j})
        starts(end+1)= genes(i).exons{j}(1,1) ;
        ends(end+1)  = genes(i).exons{j}(end,2) ;
      end ;
    end ;
    if max(ends)-min(starts)>genome_info.max_intron_len*10,
      % fprintf('gene length: %i\n', max(ends)-min(starts)) ;
      length_not_ok = 1;
    end ;
  end
  
  %%% check splicegraph
  %% please document me ... (some more) !!
  graph_not_ok = 0;
  if which_checks.graph && isfield(genes, 'splicegraph'), 
    edges = genes(i).splicegraph{2} ;
    if ~isequal(edges,0)
      ss = 0 ;
      for e=1:size(edges,1), 
        ss = ss + edges^e ; 
      end ; 
      if any(ss==0)
        graph_not_ok = 1;
        % viewsplicegraph(genes(i))
        % keyboard
      end
      num_graphs_not_ok = num_graphs_not_ok+graph_not_ok ;
    end
  end
  % if ~isfield(genes(i),'is_valid') || isempty(genes(i).is_valid)
  %   genes(i).is_valid = 1;
  % end    
  genes(i).is_valid = any(genes(i).transcript_valid)&&~length_not_ok&&...
      ~graph_not_ok&& ~isempty(genes(i).exons);
  % genes(i).is_valid = genes(i).is_valid&any(genes(i).transcript_valid)&~length_not_ok&...
  %     ~graph_not_ok& ~isempty(genes(i).exons);
  
  num_invalid_genes = num_invalid_genes + ~genes(i).is_valid; 
  
end ; %%loop over genes
fprintf('Done.\n') ;

genes(1).transcript_complete = [];
if which_checks.complete
  for i=1:length(genes),
    if ~any(genes(i).transcript_valid==1),
      genes(i).is_complete = 0 ;
      continue ;
    end ;
    for k=1:length(genes(i).transcripts)
      complete = 1;

% does not work since exons can be split into a cds and a utr part      
%      intersection = intersect(genes(i).cds_exons,genes(i).exons,'rows');
%      if ~(size(intersection,1)==size(genes(i).cds_exons,1)
%        error('adsfadfasdf')
%      end
%      intersection = intersect(genes(i).utr3_exons,genes(i).exons,'rows');
%      if ~(size(intersection,1)==size(genes(i).utr3_exons,1)
%        error('adsfadfasdf')
%      end
%      intersection = intersect(genes(i).utr5_exons,genes(i).exons,'rows');
%      if ~(size(intersection,1)==size(genes(i).utr5_exons,1)
%        error('adsfadfasdf')
%      end
%      clear intersection

      if isempty(genes(i).exons)|| isempty(genes(i).exons{k})
        genes(i).transcript_complete(k) = 0;
        continue
      end
      if length(genes(i).cds_exons)<k||isempty(genes(i).cds_exons{k})
        complete = 0;
      end
      if length(genes(i).utr3_exons)<k||isempty(genes(i).utr3_exons{k})
        complete = 0;
      end
      if length(genes(i).utr5_exons)<k||isempty(genes(i).utr5_exons{k})
        complete = 0;
      end
      if ~isempty(genes(i).tis) && length(genes(i).tis)>=k && ~isempty(genes(i).tis{k})
        if genes(i).tis{k}<genes(i).start||genes(i).tis{k}>genes(i).stop
          complete = 0;
        end
      else
        complete = 0;
      end

      if ~isempty(genes(i).cdsStop)&&length(genes(i).cdsStop)>=k &&~isempty(genes(i).cdsStop{k})
        if genes(i).cdsStop{k}<genes(i).start||genes(i).cdsStop{k}>genes(i).stop
          complete = 0;
        end
      else
        complete = 0;
      end
    
      if ~isempty(genes(i).tss)&& length(genes(i).tss)>=k&&~isempty(genes(i).tss{k})
        if genes(i).tss{k}==genes(i).tis{k}||genes(i).tss{k}<genes(i).start||genes(i).tss{k}>genes(i).stop
          complete = 0;
        end
      else 
        complete = 0;
      end
      
      if ~isempty(genes(i).cleave)&& length(genes(i).cleave)>=k && ~isempty(genes(i).cleave{k})
        if genes(i).cleave{k}==genes(i).cdsStop{k} || genes(i).cleave{k}<genes(i).start || genes(i).cleave{k}>genes(i).stop
          complete = 0;
        end
      else 
        complete = 0;
      end
      
      for j=1:length(genes(i).exons),
        if ~(all(all(genes(i).exons{j}(:,1:2)>=genes(i).start& ...
                     genes(i).exons{j}(:,1:2)<=genes(i).stop+1)))
          genes(i).start = min(min(genes(i).exons{j}))
          genes(i).stop = max(max(genes(i).exons{j}))
        end
      end

      gene_contig_len=contig_len(genes(i).chr, genome_info) ;
      if genes(i).start<min_boundary_dist || genes(i).stop>gene_contig_len-min_boundary_dist,
        genes(i).is_valid=0 ;
        num_at_boundary = num_at_boundary + 1 ;
      end ;
      
      genes(i).transcript_complete(k) = complete & genes(i).transcript_valid(k);
    end
    if ~any(genes(i).transcript_complete)
      %keyboard
    end
    genes(i).is_complete = any(genes(i).transcript_complete);
  end
end

fprintf('\nProblems found:\n') ;
	    fprintf('\ttotal invalid genes\t\t%i\n',num_invalid_genes) ;
	    fprintf('\twithout transcripts\t\t%i\n', num_no_trans) ;
	    fprintf('\ttotal invalid transcripts\t%i\n', num_invalid_transcripts) ;
	    fprintf('\twithout exons\t\t\t%i\n', num_no_exons) ;
	    fprintf('\tsplice consensus missing\t%i\n', num_splice_not_ok) ;
	    fprintf('\ttis consensus missing\t\t%i\n', num_tis_not_ok) ;
	    fprintf('\tcds exon too short\t\t%i\n', num_cds_exon_not_ok) ;
	    fprintf('\tORF missing\t\t\t%i\n', num_orf_not_ok) ;
	    fprintf('\tstop consensus missing\t\t%i\n', num_stop_not_ok) ;
	    fprintf('\tgraphs not ok\t\t\t%i\n', num_graphs_not_ok) ;
	    fprintf('\ttransacceptor not ok\t\t%i\n', num_transacc_not_ok) ;
	    fprintf('\tgenes at contig boundary\t%i', num_at_boundary) ;

fprintf('\n\n')

return

function len = contig_len(contig, genome_info)
% len = contig_len(contig, genome_info)

if ischar(contig),
  %contig_idx = find(ismember(upper(genome_info.contig_names),upper(contig))) ;
  contig_idx = strmatch(upper(contig), upper(genome_info.contig_names), 'exact') ;
  assert(all(size(contig_idx)==1) & ~isempty(contig_idx)) ;
else
  contig_idx = contig ;
end ;
  
fname=genome_info.flat_fnames{contig_idx} ;

d=dir(fname) ; 
len=d.bytes;

return

function create_directories(FN,organism,verb,signal)

% create_directories(PAR)
%   
% function creates all necessary directories for a gene finding
% experiment as defined by PAR.
  
  
if nargin<3
  verb=0;
end

if  nargin>3
  signalnames{1} = signal;
else
  signalnames = fieldnames(FN.input_sig);
end


%%%%%%%%%%%%%%%
%%% GENERAL INPUT FILES 
%%%%%%%%%%%%%%%

exp_base_in = sprintf('%s/%s/%s',FN.input_dir,organism.name,FN.dir_name);

input_dir = sprintf('%s/orig_data/',exp_base_in);
if verb
  fprintf(1, 'dir for original data: \n%s\n\n', input_dir)
end
nice_mkdir(input_dir);


%%%%%%%%%%%%%%%%%%%%%%%%%%
%% OUTPUT DIRs 
%%%%%%%%%%%%%%%%%%%%%%%%%

exp_base_out = sprintf('%s/%s/%s',FN.output_dir,organism.name,FN.dir_name);
output_dir = sprintf('%s/genome_data/',exp_base_out);
if verb
  fprintf(1, 'dir for genes, split and regions: \n%s\n\n', output_dir)
end
nice_mkdir(output_dir);

nice_mkdir(FN.exp_dir);


%%%%%%%%%%%%%%%
%%% SIGNAL SPECIFIC Dirs
%%%%%%%%%%%%%%%


for s=1:length(signalnames)
  if ~isfield( FN.input_sig, signalnames{s})
    continue
  end
  signals = FN.input_sig.(signalnames{s});
  signal_fns = fieldnames(signals);
  for i=1:length(signal_fns)
    name = signals.(signal_fns{i}) ;
    iii = max(strfind(name,'/'));
    if ~isempty(iii)
      nice_mkdir(name(1:iii));
    end 
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%
%% SIGNAL SPECIFIC OUTPUT DIRs 
%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin<4
  signalnames = fieldnames(FN.output_sig);
end
  
for s=1:length(signalnames)
  if ~isfield( FN.output_sig, signalnames{s})
    continue
  end
  signals = FN.output_sig.(signalnames{s});
  signal_fns = fieldnames(signals);
  for i=1:length(signal_fns)
    name = signals.(signal_fns{i}) ;
    if isempty(strmatch(FN.output_dir,name)) &verb
      fprintf(['incomplete signal output file name, signal output directories not generated; '...
               'run config for %s\n'],signalnames{s})
      break
    end
    iii = max(strfind(name,'/'));
    if ~isempty(iii)
      nice_mkdir(name(1:iii));
    end 
  end
end

%%%%%%%%%%%%%%%
%%% CONTENT SPECIFIC Dirs
%%%%%%%%%%%%%%%

if isfield(FN, 'input_cont')
  if nargin<4
    signalnames = fieldnames(FN.input_cont);
  end
  for s=1:length(signalnames)
    if ~isfield( FN.input_cont, signalnames{s})
      continue
    end
    signals = FN.input_cont.(signalnames{s});
    signal_fns = fieldnames(signals);
    for i=1:length(signal_fns)
      name = signals.(signal_fns{i}) ;
      iii = max(strfind(name,'/'));
      if ~isempty(iii)
        nice_mkdir(name(1:iii));
      end 
    end
  end
end
%%%%%%%%%%%%%%%%%%%%%%%%%
%% CONTENT SPECIFIC OUTPUT DIRs 
%%%%%%%%%%%%%%%%%%%%%%%%%

if isfield(FN, 'output_cont')
  if nargin<4
    signalnames = fieldnames(FN.output_cont);
  end
  for s=1:length(signalnames)
    if ~isfield( FN.output_cont, signalnames{s})
      continue
    end
    signals = FN.output_cont.(signalnames{s});
    signal_fns = fieldnames(signals);
    for i=1:length(signal_fns)
      name = signals.(signal_fns{i}) ;
      if isempty(strmatch(FN.output_dir,name)) &verb
        fprintf(['incomplete signal output file name, signal output directories not generated; '...
                 'run create_filenames first for\n'])
        break
      end
      iii = max(strfind(name,'/'));
      if ~isempty(iii)
        nice_mkdir(name(1:iii));
      end 
    end
  end
end
%%%%%%%%%%%%%%%%%%%%%%%%%
%% LSL SPECIFIC
%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin<4
  
  if isfield(FN, 'output_lsl')
    fns = fieldnames(FN.output_lsl);
    for i=1:length(fns)
      name = FN.output_lsl.(fns{i}) ;
      if isempty(strmatch(FN.output_dir,name)) &verb
        fprintf('incomplete signal output file name, signal output directories not generated;')
        break
      end
      iii = max(strfind(name,'/'));
      if ~isempty(iii)
        nice_mkdir(name(1:iii));
      end 
    end
  end
  if isfield(FN, 'input_lsl')
    fns = fieldnames(FN.input_lsl);
    for i=1:length(fns)
      if ~iscell(FN.input_lsl.(fns{i}))
        name = FN.input_lsl.(fns{i}) ;
        if isempty(strmatch(FN.input_dir,name)) &verb
          fprintf('incomplete signal input file name, signal input directories not generated;')
          break
        end
        iii = max(strfind(name,'/'));
        if ~isempty(iii)
          nice_mkdir(name(1:iii));
        end 
      else
        for j=1:length(FN.input_lsl.(fns{i}))
          name = FN.input_lsl.(fns{i}){j} ;
          if isempty(strmatch(FN.input_dir,name)) &verb
            fprintf('incomplete signal input file name, signal input directories not generated;')
            break
          end
          iii = max(strfind(name,'/'));
          if ~isempty(iii)
            nice_mkdir(name(1:iii));
          end 
        end
      end
    end
  end
  
  fn = sprintf('%s/%s/current_predictions', FN.input_dir, organism.name);
  nice_mkdir(fn);

end
return

function FN = create_filenames(FN,organism,Signals,Contents,verb)

%
% FN requires fields :
% 	- input_dirC_
% 	- output_dir
%	- exp_name
%	- FN.fn_genes_train
%	- date
%	- user
%	- organism
%

if nargin<2
  verb=0;
end

if ~isfield(FN,'dir_name_ext')
  dir_name_ext = '';
else
  dir_name_ext = [ '_', FN.dir_name_ext ];
end;

if isempty(FN.dir_name)
  FN.dir_name ='';
end
if isempty(FN.exp_name)
  FN.exp_name = '';
end
if isempty(FN.lsl_exp_name)
  FN.lsl_exp_name = '';
end

assert(isfield(FN,'exp_name'))
assert(isfield(FN,'dir_name'))

%%%%%%%%%%%%%%%
%%% GENERAL INPUT GENOME FILES 
%%%%%%%%%%%%%%%
exp_base_in = sprintf('%s/%s/%s/%s',FN.genome_dir,organism.name,organism.release,organism.genebuilt);

input_fns = fieldnames(FN.genome);
for i=1:length(input_fns)
  old_name = getfield(FN.genome,input_fns{i});
  if iscell(old_name)
    new_name ={};
    for j=1:length(old_name)
      if isempty(strmatch(FN.genome_dir,old_name{j}))
        new_name{j}  = sprintf('%s/%s',exp_base_in,old_name{j} );
      else
        new_name{j}  = old_name{j} ;
      end
    end
  elseif length(strfind(old_name,'/'))==1
    exp_base_in_ = sprintf('%s/%s/%s',FN.genome_dir,organism.name,organism.release);
    new_name  = sprintf('%s/%s',exp_base_in_,old_name );
  elseif isempty(strmatch(FN.genome_dir,old_name)) 
    new_name  = sprintf('%s/%s',exp_base_in,old_name );
  else
    new_name  = old_name ;
  end
  FN.genome = setfield(FN.genome,input_fns{i},new_name);
end  
if verb
  fprintf(1, 'genebuild from dir: \n%s\n\n', exp_base_in)
end

%%%%%%%%%%%%%%%
%%% GENERAL INPUT FILES 
%%%%%%%%%%%%%%%

exp_base_in = sprintf('%s/%s/%s',FN.input_dir,organism.name,FN.dir_name);

input_fns = fieldnames(FN.input);
for i=1:length(input_fns)
  old_name = getfield(FN.input,input_fns{i});
  if iscell(old_name)
    new_name ={};
    for j=1:length(old_name)
      if isempty(strmatch(FN.input_dir,old_name{j}))
        new_name{j}  = sprintf('%s/orig_data/%s',exp_base_in,old_name{j} );
      else
        new_name{j}  = old_name{j} ;
      end
    end
  else
    if isempty(strmatch(FN.input_dir,old_name)) 
      new_name  = sprintf('%s/orig_data/%s',exp_base_in,old_name );
    else
      new_name  = old_name ;
    end
  end
  FN.input = setfield(FN.input,input_fns{i},new_name);
end  
input_dir=sprintf('%s/orig_data/',exp_base_in);
if verb
  fprintf(1, 'dir for original data: \n%s\n\n', input_dir)
end


%%%%%%%%%%%%%%%%%%%%%%%%%%
%% OUTPUT DIRs AND FNs  
%%%%%%%%%%%%%%%%%%%%%%%%%
exp_base_out = sprintf('%s/%s/%s',FN.output_dir,organism.name,FN.dir_name);

output_fns = fieldnames(FN.output);
for i=1:length(output_fns)
  old_name = getfield(FN.output,output_fns{i});
  if isempty(old_name)
    old_name = '';
  end 
  if iscell(old_name)
    new_name ={};
    for j=1:length(old_name)
      if isempty(strmatch(FN.output_dir,old_name{j})) 
        new_name{j}  = sprintf('%s/genome_data/%s',exp_base_out,old_name{j} );
      else
        new_name{j}  = old_name{j} ;
      end
    end
  else
    if  isempty(strmatch(FN.output_dir,old_name)) 
      new_name  = sprintf('%s/genome_data/%s',exp_base_out,old_name );
    else
      new_name  = old_name ;
    end
  end
  FN.output = setfield(FN.output,output_fns{i},new_name);
end  

output_dir=sprintf('%s/genome_data/',exp_base_out);
if verb
  fprintf(1, 'dir for genes, split and regions: \n%s\n\n', output_dir)
end
if ~isfield( FN,'exp_dir')||isempty(FN.exp_dir)|| isempty(strmatch(FN.input_dir,FN.exp_dir)) 
  FN.exp_dir = sprintf('%s/%s',exp_base_out,FN.exp_name);
end

%%%%%%%%%%%%%%%
%%% SIGNAL SPECIFIC INPUT FILES 
%%%%%%%%%%%%%%%

signalnames = fieldnames(FN.input_sig);
for s=1:length(signalnames)
  signals = getfield(FN.input_sig,signalnames{s});
  signal_fns = fieldnames(signals);
  for i=1:length(signal_fns)
    old_name = getfield(signals,signal_fns{i}) ;
    if isempty(old_name)
       old_name = '';
    end 
    if isempty(strmatch(FN.input_dir,old_name)) 
      new_name  = sprintf('%s/%s/%s/%s',exp_base_in,FN.exp_name,dir_name_ext,old_name );
      % new_name  = sprintf('%s/%s/sensors/%s%s/%s',exp_base_in,FN.exp_name,signalnames{s},dir_name_ext,old_name );
    else
      new_name  = old_name ; 
    end
    signals = setfield(signals,signal_fns{i},new_name);
  end 
  FN.input_sig = setfield(FN.input_sig,signalnames{s},signals);
  
  % sig_input = sprintf('%s/%s/sensors/%s%s',exp_base_in,FN.exp_name,signalnames{s},dir_name_ext,old_name);
  if verb
    fprintf(1, 'dir for label_files and examples: \n%s\n\n', new_name)
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%
%% SIGNAL SPECIFIC OUTPUT DIRs AND FNs  
%%%%%%%%%%%%%%%%%%%%%%%%%



%output_fns = fieldnames(FN.output);
%for i=1:length(output_fns)
%  old_name = getfield(FN.output,output_fns{i});
%  new_name  = sprintf('%s/%s_%s/%s',FN.output_dir,FN.exp_name,...
%                      organism.name,old_name );
%  FN.output = setfield(FN.output,output_fns{i},new_name);
%end  

signalnames = fieldnames(FN.output_sig);
for s=1:length(signalnames)
  signals = FN.output_sig.(signalnames{s});
  signal_fns = fieldnames(signals);
  method = Signals.(signalnames{s}).method;
  for i=1:length(signal_fns)
    old_name = getfield(signals,signal_fns{i}) ;
    dir_name = sprintf('%s/%s/%s',exp_base_out,FN.exp_name,dir_name_ext);
    % dir_name = sprintf('%s/%s/sensors/%s%s',exp_base_out,FN.exp_name,signalnames{s},dir_name_ext);
    if isfield(FN, 'custom_output')
      dir_name = sprintf('%s/%s',dir_name,FN.custom_output);
    else
      dir_name = sprintf('%s/output_%s',dir_name,method.name);
      if isequal(method.name, 'SVM')
        if length(method.kernels)==1&&~isempty(method.kernels{1})
          if isfield(method.kernels{1},'name')
            kernel_names = method.kernels{1}.name; 
          else
            kernel_names = method.kernels{1}.kernel_name;
          end
        else
          kernel_names = 'COMBINED';
        end
        dir_name = sprintf('%s%s',dir_name,kernel_names);
      end
    end
    if isempty(strmatch(FN.output_dir,old_name)) 
      new_name  = sprintf('%s/%s',dir_name, old_name );
    else
      new_name = old_name;
    end
    signals = setfield(signals,signal_fns{i},new_name);
  end
  FN.output_sig = setfield( FN.output_sig, signalnames{s}, signals );
  if verb
    fprintf(1, 'dir for training and test data: \n%s\n\n', dir_name) 
  end
end


%%%%%%%%%%%%%%%
%%% CONTENT SPECIFIC INPUT FILES 
%%%%%%%%%%%%%%%

if isfield(FN, 'input_cont')
  cont_names = fieldnames(FN.input_cont);
  for s=1:length(cont_names)
    content = getfield(FN.input_cont,cont_names{s});
    cont_fns = fieldnames(content);
    for i=1:length(cont_fns)
      old_name = getfield(content,cont_fns{i}) ;
      if isempty(strmatch(FN.input_dir,old_name)) 
        new_name  = sprintf('%s/%s/%s/%s',exp_base_in,FN.exp_name,dir_name_ext,old_name );
        % new_name  = sprintf('%s/%s/contents/%s%s/%s',exp_base_in,FN.exp_name, cont_names{s},dir_name_ext,old_name );
      else
        new_name  = old_name ; 
      end
      content = setfield(content,cont_fns{i},new_name);
    end 
    FN.input_cont = setfield(FN.input_cont,cont_names{s},content);
  end

  %cont_input = sprintf('%s/%s/contents/%s',exp_base_in,FN.exp_name,signal_dir_name);
  if verb
    fprintf(1, 'dir for content specific input: \n%s\n\n', cont_input)
  end
end

%%%%%%%%%%%%%%%
%%% CONTENT SPECIFIC OUTPUT FILES 
%%%%%%%%%%%%%%%

if isfield(FN, 'output_cont')
  cont_names = fieldnames(FN.output_cont);
  for s=1:length(cont_names)
    content = getfield(FN.output_cont,cont_names{s});
    cont_fns = fieldnames(content);
    method = Contents.(cont_names{s}).method;
    for i=1:length(cont_fns)
      old_name = getfield(content,cont_fns{i}) ;
      dir_name = sprintf('%s/%s/%s',exp_base_out,FN.exp_name,dir_name_ext);
      if isfield(FN, 'custom_output')
        dir_name = sprintf('%s/%s',dir_name,FN.custom_output);
      else
        dir_name = sprintf('%s/output_%s',dir_name,method.name);
        if isequal(method.name, 'SVM')
          if length(method.kernels)==1&&~isempty(method.kernels{1})
            if isfield(method.kernels{1},'name')
              kernel_names = method.kernels{1}.name; 
            else
              kernel_names = method.kernels{1}.kernel_name;
            end
          else
            kernel_names = 'COMBINED';
          end
          dir_name = sprintf('%s%s',dir_name,kernel_names);
        end
      end
      if isempty(strmatch(FN.output_dir,old_name)) 
        new_name  = sprintf('%s/%s',dir_name, old_name );
      else
        new_name = old_name;
      end
      content = setfield(content,cont_fns{i},new_name);
    end 
    FN.output_cont = setfield(FN.output_cont, cont_names{s},content);
  end

  %cont_output = sprintf('%s/%s/contents/%s',exp_base_out,FN.exp_name,signal_dir_name);
  if verb
    fprintf(1, 'dir for content specific output: \n%s\n\n', cont_output)
  end
end

%%%%%%%%%%%%%%%
%%% LSL STUFF
%%%%%%%%%%%%%%%

if isfield(FN, 'input_lsl')
  fns = fieldnames(FN.input_lsl);
  FN.input_lsl.dir = sprintf('%s/%s/%s/%s/lsl/%s/', ...
			     exp_base_out,organism.name,FN.dir_name,FN.exp_name,FN.lsl_exp_name ); 
  for i=1:length(fns)
    if ~iscell(FN.input_lsl.(fns{i}))
      old_name = FN.input_lsl.(fns{i}) ;
      if isempty(strmatch(FN.input_lsl.dir,old_name))&~isequal(fns{i},'fn_old_training_blocks')...
            &~isequal(fns{i},'fn_old_test_blocks')&~isequal(fns{i},'fn_old_split')
        new_name  = sprintf('%s/%s',FN.input_lsl.dir,old_name);
      else
        new_name  = old_name ; 
      end
      FN.input_lsl.(fns{i}) = new_name;
    else
      for j=1:length(FN.input_lsl.(fns{i}))
        old_name = FN.input_lsl.(fns{i}){j} ;
        if isempty(strmatch(FN.input_lsl.dir,old_name))&~isequal(fns{i},'fn_old_training_blocks')...
              &~isequal(fns{i},'fn_old_test_blocks')&~isequal(fns{i},'fn_old_split')
          new_name  = sprintf('%s/%s',FN.input_lsl.dir,old_name);
        else
          new_name  = old_name ; 
        end
        FN.input_lsl.(fns{i}){j} = new_name;
      end    
    end
  end
end
%cont_output = sprintf('%s/%s/contents/%s',exp_base_out,FN.exp_name,signal_dir_name);
if verb
  fprintf(1, 'dir for lsl specific input: \n%s\n\n', cont_output)
end

if isfield(FN, 'output_lsl')
  fns = fieldnames(FN.output_lsl);
  FN.output_lsl.dir = sprintf('%s/%s/%s/%s/lsl/%s/', ...
			      exp_base_out, organism.name,FN.dir_name,FN.exp_name,FN.lsl_exp_name ); 
  for i=1:length(fns)
    old_name = FN.output_lsl.(fns{i}) ;
    if isempty(strmatch(FN.output_lsl.dir,old_name))
      new_name  = sprintf('%s/%s',FN.output_lsl.dir,old_name);
    else
      new_name  = old_name ; 
    end
    FN.output_lsl.(fns{i}) = new_name;
  end 
end
%cont_output = sprintf('%s/%s/contents/%s',exp_base_out,FN.exp_name,signal_dir_name);
if verb
  fprintf(1, 'dir for lsl specific output: \n%s\n\n', cont_output)
end


return

function PAR=create_PAR(Exp, PAR_config, verb)
%  PAR=create_PAR(Exp,PAR_config,verb)
%
% creates PAR with all parameters for one genefinding
% experiment. For a new Organism X a new template file must be added to /template_files/Organisms/X.m. 
% (e.g. C_elegans.m) 
% For each signal a standard method is applied (usually SVM with one or
% more spectrum kernels)
%  
% INPUT  - Exp: filename of current experiment  (e.g. exp_C_elegans_nGASP)
%        - verb: verbose
%
% OUTPUT - PAR. struct with all information needed for genefinding experiment  
  
Signals = {'acceptor','donor','tis','cdsStop','tss','cleave','polya','transacceptor'};
sig_names = {'acc','don','tis','cdsStop','tss','cleave','polya','transacc'};
Methods = {'SVM_WD','SVM_WD','SVM_COMBINED','SVM_COMBINED','SVM_COMBINED','SVM_COMBINED','SVM_COMBINED','SVM_WD'};

content_names = {'intergenic','utr5exon','cds_exon','utr3exon','polya_tail','intron','frame0'};
Methodc = 'SVM_Cont';
changeable_fields = {'organism','label_source','Signals','Contents','LSL','RPROC','SETs','regions','tasks','Source_organisms','FN'} ;

if nargin<3,
  verb=0;
end


PAR = init_PAR;

if nargin>0
  fprintf('parameters from exp file: %s \n',Exp)
  if ~exist('PAR_config') || isempty(PAR_config),
    PAR_exp = feval(Exp) ;
  else
    try,
      PAR_exp = feval(Exp, PAR_config) ;
    catch
      PAR_exp = feval(Exp) ;
    end 
    clear PAR_config ;
  end ;
  assert(all(ismember(fieldnames(PAR_exp),changeable_fields)))
  if ~isfield(PAR_exp,'Contents')
    PAR_exp.Contents = struct;
  end
  if ~isfield(PAR_exp,'Signals')
    PAR_exp.Signals = struct;
  end
  PAR.organism = set_default(PAR_exp,'organism',PAR.organism)  ;  
  PAR.label_source = set_default(PAR_exp,'label_source',PAR.label_source) ;
  PAR.RPROC = set_default(PAR_exp,'RPROC',PAR.RPROC)  ; 
  PAR.SETs = set_default(PAR_exp,'SETs',PAR.SETs)  ; 
  PAR.SETs = make_partitions(PAR.SETs);
  PAR.regions = set_default(PAR_exp,'regions',PAR.regions)  ; 
  PAR.tasks = set_default(PAR_exp,'tasks',PAR.tasks)  ; 
  PAR.Source_organisms = set_default(PAR_exp,'Source_organisms',PAR.Source_organisms)  ; 
end

if isempty(PAR.organism.name),
  [organism, label_source] = dummy_organism(PAR.label_source, sig_names);
else
  [organism, label_source] = feval(PAR.organism.name,PAR.label_source,sig_names);
end
PAR.organism = set_default(PAR,'organism',organism)  ;
PAR.label_source = set_default(PAR,'label_source',label_source)  ;

%-----------------------------
%%% prepare all signals
%-----------------------------

for i=1:length(Signals)
  Signal = feval(Signals{i},PAR.label_source);
  if exist('PAR_exp')
    if isfield(PAR_exp.Signals, sig_names{i}) && isfield(PAR_exp.Signals.(sig_names{i}),'method')
      me = PAR_exp.Signals.(sig_names{i}).method;
      PAR_exp.Signals.(sig_names{i}) = rmfield(PAR_exp.Signals.(sig_names{i}),'method');
    end
    Signal = set_default(PAR_exp.Signals,sig_names{i},Signal)  ;
  end
  Signal.method = feval(Methods{i},Signal);
  if exist('me')
    PAR_exp.Signals.(sig_names{i}).method = me;
    Signal = set_default(PAR_exp.Signals,sig_names{i},Signal)  ;
    clear me
  end
  % Signal = Signal.domain_adapt.transfer_fct(Signal);
  PAR.Signals.(sig_names{i}) = Signal;
end

%-----------------------------
%%% prepare all contents
%-----------------------------
for i=1:length(content_names)
  Content = feval(content_names{i});
  if exist('PAR_exp')
    if isfield(PAR_exp.Contents, content_names{i}) && isfield(PAR_exp.Contents.(content_names{i}),'method')
      me = PAR_exp.Contents.(content_names{i}).method;
      PAR_exp.Contents.(content_names{i}) = rmfield(PAR_exp.Contents.(content_names{i}),'method');
    end
    Content = set_default(PAR_exp.Contents,content_names{i},Content)  ;
  end
  Content.method = feval(Methodc,Content);
  if exist('me')
    PAR_exp.Contents.(content_names{i}).method = me;
    Content = set_default(PAR_exp.Contents,content_names{i},Content)  ;
    clear me
  end
  % Content = Content.domain_adapt.transfer_fct(Content);
  PAR.Contents.(content_names{i}) = Content;
end

%-----------------------------
%%% prepare LSL
%-----------------------------

LSL = lsl_template; 
% LSL = LSL.domain_adapt.transfer_fct(LSL);
if exist('PAR_exp')
  LSL = set_default(PAR_exp,'LSL',LSL)  ; 
end
PAR.LSL=LSL;

%-----------------------------
%%% prepare filenames
%-----------------------------
FN = init_FN;
if  exist('PAR_exp')
  FN = set_default(PAR_exp,'FN',FN)  ;
end

FN = create_filenames(FN,PAR.organism,PAR.Signals,PAR.Contents,verb);
if  exist('PAR_exp')
  FN = set_default(PAR_exp,'FN',FN)  ;
end
PAR.FN=FN;

%-----------------------------
%%% Add polya consensus if file available
%-----------------------------

if fexist(PAR.FN.input_sig.polya.fn_polya)
  load(PAR.FN.input_sig.polya.fn_polya,'consensus')
  PAR.Signals.polya.consensus = consensus;
end

%PAR.FN.source_directory = '~/svn/projects/genefinding/';
%[tmp,PAR.FN.svn_info]=unix('svn info');

PAR.FN.svn_info = '' ;

fprintf('loading model from %s\n', PAR.LSL.method.model_fct)
PAR.model = feval(PAR.LSL.method.model_fct,PAR.organism,PAR.LSL, ...
                  PAR.Signals);


return

function [fn, exists]=gen_example_filename(Signal,fn_filter_settings,fn_examples,which,split)

% [fn, exists]=gen_example_filename(Signal,fn_filter_settings,fn_examples,which,split)
% [fn, exists]=gen_example_filename(PAR,which,split)
% which is 'train','eval','test'

% sig = PAR.Signal_name;
% Signal = PAR.Signals.(sig); 
% fn_filter_settings =  PAR.FN.input_sig.(sig).input_sig;
% fn_examples = PAR.FN.input_sig.(sig).fn_examples;

whichs = fieldnames(Signal.filter_label);


if isnumeric(which)
  assert(which<=length(whichs))
  which = whichs{which};
else
  assert(ismember(which,whichs))
end
clear whichs

current_settings = Signal.filter_label.(which);

if fexist(fn_filter_settings)
  load(fn_filter_settings, 'filter_settings')
  idx = find([filter_settings.split]==split);
else
  filter_settings = [];
  idx = [];
end
idx_setting = [];is_equal=0;
for i=idx
  temp = rmfield(filter_settings(i),'split');
  temp = rmfield(temp,'filename');
  if isequal(temp, current_settings)
    idx_setting = [idx_setting i];
    is_equal=1;
  else
    assert(length(fieldnames(temp))==length(fieldnames(current_settings))&&...
           all(strcmp(sort(fieldnames(temp)),sort(fieldnames(current_settings)))))
  end
end
assert(length(idx_setting)<=1);
if isempty(idx_setting)
  temp = struct;
end
assert(length(setdiff(fieldnames(temp),fieldnames(current_settings)))==0)

if ~isempty(idx_setting)
  fn = filter_settings(idx_setting).filename;
  if fexist(fn)
    exists = 1;
  else
    exists = 0;
  end
else 
  settings_names = fieldnames(current_settings);
  s = length(filter_settings)+1;
  for j = 1:length(settings_names)
    filter_settings(s).(settings_names{j}) = current_settings.(settings_names{j});
  end
  filter_settings(s).split = split;
  filter_settings(s).filename = sprintf('%s_settings=%i_split=%i.mat',fn_examples, s, split);
  fn = filter_settings(s).filename;
  if fexist(fn)
    error('I do not know where that file comes from!!!')
  else
    exists = 0;
  end 
  save(fn_filter_settings, 'filter_settings')
end



       
%fn = sprintf('%s_%s_pc=%i_pa=%i_nc=%i_na=%i_split=%i.mat',...
%	input_sig.fn_examples,PAR.Signal_name,pos_conf,...
%	pos_anno,neg_conf,neg_anno, s);

return

function regions=init_regions(fn_genome_config, organism)
% regions=init_regions(fn_genome_config,organism)

  
regions(1).chr = [];
regions(1).chr_num = [];
regions(1).strand = [];
regions(1).start = [];
regions(1).stop = [];
regions(1).offset =[];
regions(1).id = [];
regions(1).organism = [];
regions(1).config = [];

if ~exist('fn_genome_config')
  return ;
else
  regions = regions([]) ;
  genome_config = init_genome(fn_genome_config);
  strands = '+-';
  for c=1:length(genome_config.contig_names)
    %+strand
    d = dir(genome_config.flat_fnames{c}) ;
    for s=1:length(strands)
      region = [];
      region.chr=genome_config.contig_names{c};
      region.chr_num = c;
      region.strand=strands(s);
      region.start = 1;
      region.stop = d.bytes ;
      switch strands(s)
       case '+',
        region.offset      = region.start - 1;
       case '-',
        region.offset      = region.stop + 1;
      end
      region.id = (c-1)*2+s;
      region.organism = [];
      if exist('organism')
        region.organism = organism;
      end
      region.config = fn_genome_config;
      regions(region.id) = region;
    end
  end
end


return

function SETs = make_partitions(SETs)
 

if SETs.num_partitions==1
  return
end
  
SETs.partitions = eye(SETs.num_partitions,SETs.num_partitions)+1;  
for i = 1:2:SETs.num_partitions-1
  SETs.partitions(i:i+1,i:i+1) = SETs.partitions(i:i+1,i:i+1)+1;
end
if mod(SETs.num_partitions,2)
  SETs.partitions(end,end-1:end) =  SETs.partitions(end,end-1:end)+1;
end
if SETs.num_splits~=SETs.num_partitions
  assert(mod(SETs.num_splits,SETs.num_partitions)==0)
  SETs.partitions = repmat(SETs.partitions,1,SETs.num_splits/SETs.num_partitions);
end

if ~isfield(SETs,'partition_train_on') 
  [tmp,idx] = unique(SETs.partitions==1,'rows');
  SETs.partition_train_on = zeros(1,SETs.num_partitions);
  SETs.partition_train_on(idx) = 1;
end



SETs.info.train = 1;
SETs.info.eval = 2;
SETs.info.plif = 2;
SETs.info.pred = 3;

for i=1:SETs.num_partitions
  assert(isequal(unique(SETs.partitions(i,:)),unique([SETs.info.train ...
                      SETs.info.eval SETs.info.plif SETs.info.pred ])))
end

assert(all(sum(SETs.partitions==SETs.info.pred)==1))
assert(length(unique(sum(SETs.partitions'==SETs.info.train)))==1)
assert(length(unique(sum(SETs.partitions'==SETs.info.eval)))==1)
assert(length(unique(sum(SETs.partitions'==SETs.info.plif)))==1)
assert(length(unique(sum(SETs.partitions'==SETs.info.pred)))==1)




return

function PAR=masterscript_data(PAR,first)
  
%PAR=masterscript_data(PAR,first)

if nargin<2
  first =1 ;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% PREPARE COMPLETE GENE STRUCTURE 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if PAR.tasks.prepare_labels
  if first==1 
    fprintf('start preparing gene structure for organism: %s\n\n',PAR.organism.name)
    PAR.add_signals.tss = 1; 
    PAR.add_signals.cleave = 1 ; 
    PAR.add_signals.polya = 1;
    PAR.add_signals.tis = 1;
    PAR.add_signals.cdsStop = 1;
    PAR.add_signals.transacc = 0; 
  elseif first==2
    fprintf('adding transacc sites to gene structure for organism: %s\n\n',PAR.organism.name)
    PAR.add_signals.tss = 0; 
    PAR.add_signals.cleave = 0 ; 
    PAR.add_signals.polya = 0;
    PAR.add_signals.tis = 0;
    PAR.add_signals.cdsStop = 0;
    PAR.add_signals.transacc = 1; 
  end
  
  [genes_sensors,genes_merged] = prepare_complete_genes(PAR);
  if first==2
    PAR.add_signals.tss = 1; 
    PAR.add_signals.cleave = 1 ; 
    PAR.add_signals.polya = 1;
    PAR.add_signals.tis = 1;
    PAR.add_signals.cdsStop = 1;
    PAR.add_signals.transacc = 1; 
  end
  genes = genes_sensors;
  if fexist(PAR.FN.output.fn_genes_signals) || fexist([PAR.FN.output.fn_genes_signals '.mat']),
    warning(sprintf('do not save file as it already exists: %s', PAR.FN.output.fn_genes_signals)) ;
    L=load_struct(PAR.FN.output.fn_genes_signals, 'genes') ;
    assert(isequal(L, genes)) ;
    clear L
  else
    save_struct(PAR.FN.output.fn_genes_signals,genes,'genes')
    %save(PAR.FN.output.fn_genes_signals,'-append','PAR')
    save_append(PAR.FN.output.fn_genes_signals, 1, 'PAR', PAR)
  end ;
  genes = genes_merged;
  if fexist(PAR.FN.output.fn_genes_merged) || fexist([PAR.FN.output.fn_genes_merged '.mat']),
    warning(sprintf('do not save file as it already exists: %s', PAR.FN.output.fn_genes_signals)) ;
    L=load_struct(PAR.FN.output.fn_genes_merged, 'genes') ;
    assert(isequal(L, genes)) ;
    clear L
  else
    save_struct(PAR.FN.output.fn_genes_merged,genes,'genes')
    %save(PAR.FN.output.fn_genes_signals,'-append','PAR')
    save_append(PAR.FN.output.fn_genes_signals, 1, 'PAR', PAR)
  end 
  clear genes_merged, genes_sensors
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% PREPARE REGIONS AND BLOCKS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% keyboard
if first==1 &...
         (~fexist(PAR.FN.output.fn_trivial_regions) |...
          ~fexist(PAR.FN.output.fn_blocks)|...
          ~fexist(PAR.FN.output.fn_test_blocks))
  [trivial_regions,blocks,training,test] = prepare_regions(PAR);
  if ~fexist(PAR.FN.output.fn_trivial_regions)| ~fexist(PAR.FN.output.fn_blocks)  
    save(PAR.FN.output.fn_trivial_regions,'trivial_regions','PAR');
    save(PAR.FN.output.fn_blocks, 'blocks','PAR');
  end
  if 1 % ~isempty(training.blocks)&~fexist(PAR.FN.output.fn_training_blocks)
    %save(PAR.FN.output.fn_training_blocks,'-struct','training');
    save_append(PAR.FN.output.fn_training_blocks, 0, training);
    %save(PAR.FN.output.fn_training_blocks,'-append' ,'PAR');
    save_append(PAR.FN.output.fn_training_blocks, 1, 'PAR', PAR);
    PAR.regions.offset=PAR.regions.offset_train;
    save_splits2bed(training.blocks,training.split,PAR.FN.output.fn_training_blocks(1:end-4),PAR);
  end
  if 1 % ~isempty(test.blocks)&~fexist(PAR.FN.output.fn_test_blocks)
    %save(PAR.FN.output.fn_test_blocks,'-struct','test');
    save_append(PAR.FN.output.fn_test_blocks, 0, test);
    %save(PAR.FN.output.fn_test_blocks,'-append','PAR');
    save_append(PAR.FN.output.fn_test_blocks, 1, 'PAR', PAR);
    PAR.regions.offset=PAR.regions.offset_test;
    save_splits2bed(test.blocks,test.split,PAR.FN.output.fn_test_blocks(1:end-4),PAR);
  end
end

return


function [ ok ] = nice_mkdir( dir );
if ~isdir(dir)
  ok = ( 0 == unix( [ 'mkdir -p ', dir ] ) );
  ok = ok & ( 0 == unix( [ 'chmod g+t ', dir ] ) );
end


return

function [trivial_regions,blocks,training,test] = prepare_regions(PAR);
%  [trivial_regions,training_split,training_blocks,test_split,test_blocks] = prepare_regions(PAR,genes);


% generate trivial regions
if fexist(PAR.FN.output.fn_trivial_regions)
  fprintf(1,'loading trivial regions from file %s\n',PAR.FN.output.fn_trivial_regions)
  load(PAR.FN.output.fn_trivial_regions,'trivial_regions');
  assert(isequal(trivial_regions(1).config,PAR.FN.input.fn_genome_config));
else
  fprintf(1,'initializing trivial regions\n')
  trivial_regions = init_regions(PAR);
  save(PAR.FN.output.fn_trivial_regions,'trivial_regions');
end

% generate training and test blocks and splits, 
% considering training and test region
if fexist(PAR.FN.output.fn_training_blocks)& fexist(PAR.FN.output.fn_test_blocks)
  Old = load(PAR.FN.output.fn_training_blocks, 'PAR');
  assert(isequal(Old.PAR.FN.output.fn_trivial_regions,PAR.FN.output.fn_trivial_regions));
  assert(isequal(Old.PAR.FN.input.fn_genome_config,PAR.FN.input.fn_genome_config));
  diff_fields = compare_PAR(PAR,Old.PAR) ;
  if ismember('regions',diff_fields)
    warning('regions exist, Parameters are different ')
    %fprintf('regions exist, Parameters are different ')
  else
    fprintf(1,'loading split and blocks from file %s \n',PAR.FN.output.fn_training_blocks)
    training = load(PAR.FN.output.fn_training_blocks)
    assert(length(fieldnames(training.split))==PAR.SETs.num_splits)
  end
  Old = load(PAR.FN.output.fn_test_blocks, 'PAR');
  assert(isequal(Old.PAR.FN.output.fn_trivial_regions,PAR.FN.output.fn_trivial_regions));
  assert(isequal(Old.PAR.FN.input.fn_genome_config,PAR.FN.input.fn_genome_config));
  diff_fields = compare_PAR(PAR,Old.PAR) ;
  if ismember('regions',diff_fields)
    warning('regions exist, Parameters are different ')
    %fprintf('regions exist, Parameters are different ')
  else
    fprintf(1,'loading split and blocks from file %s \n',PAR.FN.output.fn_test_blocks)
    test = load(PAR.FN.output.fn_test_blocks)
    assert(length(fieldnames(test.split))==PAR.SETs.num_splits)
  end
else
  if isfield(PAR.FN.output,'fn_genes_merged') && fexist(PAR.FN.output.fn_genes_merged)
    load(PAR.FN.output.fn_genes_merged,'genes');
  else
    genes=[];
  end
  [training, test] = assign_subsets(trivial_regions,PAR.FN.input.fn_trainings_regions,PAR.FN.input.fn_test_regions,genes,PAR.SETs,PAR.regions,PAR.tasks)
end

if 1
  if isfield(PAR.FN.output,'fn_genes_merged') && fexist(PAR.FN.output.fn_genes_merged)
    load(PAR.FN.output.fn_genes_merged,'genes');
  else
    genes=[];
  end
  load(PAR.FN.input.fn_trainings_regions,'training_regions');
  load(PAR.FN.input.fn_test_regions,'test_regions');
  load(PAR.FN.output.fn_trivial_regions,'trivial_regions');
  if  ~exist('training')
    training = load(PAR.FN.output.fn_training_blocks);
  end
  if  ~exist('test')
    test = load(PAR.FN.output.fn_test_blocks);
  end
  % load(PAR.FN.output.fn_blocks,'blocks');
  blocks =test.blocks;
  check_and_plot_splits(trivial_regions(1:min(10,length(trivial_regions))),training_regions,test_regions,genes,blocks,training, test)
end
  

return

function [genes, FAULTS_id,FAULTS_name] = process_gff3_genes(genes, genome_info, name_of_CDS,model) 
% genes = process_gff3_genes(genes, genome_info, name_of_CDS)  


if nargin<3
  name_of_CDS = 'CDS';
end

if nargin<4
  m.exon = 0;
  m.transexon = 1;
  m.utr5exon = 2;
  m.utr3exon = 4; 
  m.cds_exon = 100;
  if any(ismember(name_of_CDS, {'CDS','coding_exon'}))
    m.cds_exon = 3;
  end
  if any(ismember(name_of_CDS, {'exon'}))
    m.cds_exon = [m.cds_exon 0];
  end
else
  if isfield(model.segments, 'exon')
    m.exon = model.segments.exon;
  else
    m.exon = -1e7;
  end
  if isfield(model.segments, 'trans_exon')
    m.transexon = model.segments.trans_exon;
  else
    m.transexon = -1e7; 
  end
  m.utr5exon = model.segments.utr5exon;
  m.utr3exon = model.segments.utr3exon;
  m.cds_exon = model.segments.cds_exon;
end

gff_contig_names = unique({genes.chr}) ;
unannotated_contigs = setdiff(genome_info.contig_names, gff_contig_names) ;
if ~isempty(unannotated_contigs),
  fprintf(1, '\nThe genome information object contains %i/%i contigs not annotated in the GFF3 file.\n', length(unannotated_contigs), length(genome_info.contig_names)) ;
  if length(unannotated_contigs)<=10,
    for i=1:length(unannotated_contigs), 
      fprintf(1, '  %s\n', unannotated_contigs{i}) ;
    end;
  end ;
end ;
unknown_contigs = setdiff(gff_contig_names, genome_info.contig_names) ;
if ~isempty(unknown_contigs),
  fprintf(2, '\nThe GFF3 file contains the following %i contigs not present in the genome information object:\n', length(unknown_contigs)) ;
  for i=1:length(unknown_contigs), 
    fprintf(2, '  %s\n', unknown_contigs{i}) ;
  end;
  error('Cannot continue.') ;
end ;


FAULTS_name.exons.overlapping = {};
FAULTS_name.exons.unknown_type = {};
FAULTS_name.exons.no_transcripts = [];
FAULTS_name.exons.no_exons = [];
FAULTS_name.exons.TAGT_cases = [];

FAULTS_name.cdsUTR5.ok = {};
FAULTS_name.cdsUTR5.TISdiff = {};
FAULTS_name.cdsUTR5.mult_overlap = {}; 
FAULTS_name.cdsUTR5.gap = {};
FAULTS_name.cdsUTR5.TSSdiff = {};
FAULTS_name.cdsUTR5.no_utr5 = {};

FAULTS_name.cdsUTR3.ok = {};
FAULTS_name.cdsUTR3.STOPdiff = {};
FAULTS_name.cdsUTR3.mult_overlap = {}; 
FAULTS_name.cdsUTR3.gap = {};
FAULTS_name.cdsUTR3.CLEAVEdiff = {};
FAULTS_name.cdsUTR3.no_utr3 = {};

FAULTS_name.exonUTR5.gap = {};
FAULTS_name.exonUTR5.TSSdiff = {};
FAULTS_name.exonUTR3.CLEAVEdiff = {};
FAULTS_name.exonUTR3.gap = {};
FAULTS_name.noCDS.noCDS = {};



FAULTS_id.exons.overlapping = [];
FAULTS_id.exons.unknown_type = [];
FAULTS_id.exons.no_transcripts = [];
FAULTS_id.exons.no_exons = [];
FAULTS_id.exons.TAGT_cases = [];

FAULTS_id.cdsUTR5.ok = [];
FAULTS_id.cdsUTR5.TISdiff = [];
FAULTS_id.cdsUTR5.mult_overlap = []; 
FAULTS_id.cdsUTR5.gap = [];
FAULTS_id.cdsUTR5.TSSdiff = [];
FAULTS_id.cdsUTR5.no_utr5 = [];

FAULTS_id.cdsUTR3.ok = [];
FAULTS_id.cdsUTR3.STOPdiff = [];
FAULTS_id.cdsUTR3.mult_overlap = []; 
FAULTS_id.cdsUTR3.gap = [];
FAULTS_id.cdsUTR3.CLEAVEdiff = [];
FAULTS_id.cdsUTR3.no_utr3 = [];

FAULTS_id.exonUTR5.gap = [];
FAULTS_id.exonUTR5.TSSdiff = [];
FAULTS_id.exonUTR3.CLEAVEdiff = [];
FAULTS_id.exonUTR3.gap = [];
FAULTS_id.noCDS.noCDS = [];

fprintf('processing genes\n')
for i=1:length(genes)
  if (mod(i,1000)==0) || i==length(genes), 
    fprintf('%i/%i\r',i,length(genes));
  end
  genes(i).id = i;
  if exist('genome_info','var') && ~isempty(genome_info) && (~isfield(genes,'chr_num') || isempty(genes(i).chr_num))
    genes(i).chr_num= strmatch(upper(genes(i).chr), upper(genome_info.contig_names), 'exact');
    if isempty(genes(i).chr_num)
      error('sequence ID in gff file (%s) does not match any sequence IDs in fasta file (%s)', upper(genes(i).chr), upper(genome_info.contig_names));
    end
  end
  genes(i).tis = {} ;
  genes(i).cdsStop = {} ;
  genes(i).tss = {} ;
  genes(i).cleave = {} ;
  if ~isfield(genes(i), 'cds_exons') || isempty(genes(i).cds_exons)
    genes(i).cds_exons = {} ;
  end ;
  if ~isfield(genes(i), 'utr5_exons') || isempty(genes(i).utr5_exons)
    genes(i).utr5_exons = {} ;
  end ;
  if ~isfield(genes(i), 'utr3_exons') || isempty(genes(i).utr3_exons)
    genes(i).utr3_exons = {} ;
  end ;
  gene = genes(i) ;
  if isempty(gene.exons) || (length(gene.exons)==1&&isempty(gene.exons{1}))
    FAULTS_id.exons.no_exons(end+1)=i;
    FAULTS_name.exons.no_exons{end+1}=gene.name;
    continue
  end
  if isempty(gene.transcripts) 
    FAULTS_id.exons.no_transcripts(end+1) = i;
    FAULTS_name.exons.no_transcripts{end+1} = gene.name;
    continue
  end
  
  for j=1:length(gene.transcripts)
    gene.exons_confirmed{j} =[]; 
    exons = gene.exons{j} ;
    if size(exons,2)<3, 
      %warning('no exon meta info, identifying as exons') ;
      exons(:,3)=m.exon ; 
    end ;
    [exons(:,1),idx] = sort(exons(:,1)) ;
    exons(:,2:end) = exons(idx,2:end) ;
    %assert(all((exons(:,2)-exons(:,1))>=0)) ;
    if ~all((exons(:,2)-exons(:,1))>=0),
      warning('exon_end<exon_start; gene invalid') ;
      continue ;
    end ;
      
    idx_trans = find(exons(:,3)==m.transexon); 
    if ~isempty(idx_trans)
      exons(idx_trans,:) = [];
      warning('transexon removed')
    end
    %remove redundant information 
    if all(ismember([m.exon,m.cds_exon],unique(exons(:,3))))
      idx_cds = find(exons(:,3)==m.cds_exon);
      idx_exon = find(exons(:,3)==m.exon);
      [tmp,tmp,idx2] = intersect(exons(idx_cds,1:2),exons(idx_exon,1:2),'rows');
      exons(idx_exon(idx2),:)=[];
    end
    %% overlapping exons are merged
    if ~all((exons(2:end,1)-exons(1:end-1,2))>=0)
      idx = find(exons(2:end,1)-exons(1:end-1,2)<0) ;
      if (exons(idx,3)==exons(idx+1,3))
        FAULTS_id.exons.overlapping(end+1) = i;
        FAULTS_name.exons.overlapping{end+1} = gene.name;
        % warning('overlapping exons found within one transcript\n')  
	if keyboard_allowed(),
          keyboard
        end ;
	exons(idx,2) =  exons(idx+1,2);
        exons(idx+1,:) =[];
        if ~all((exons(:,2)-exons(:,1))>=0)
	  error('corrupted exons found: length of exon is <= zero');
	end
	if ~all((exons(2:end,1)-exons(1:end-1,2))>=0)
	  error('overlapping exons found within a single transcript');
	end
      end
    end	
    
    transcript = [exons(1) exons(2)];
    
    all_exons = exons;
    cds_exons = exons(find(ismember(exons(:,3),m.cds_exon)),1:2) ;
    utr5_exons = exons(find(exons(:,3)==m.utr5exon), 1:2) ;
    utr3_exons = exons(find(exons(:,3)==m.utr3exon), 1:2) ;
    other = exons(find(~ismember(exons(:,3),[m.cds_exon m.utr5exon m.utr3exon m.exon])),:) ;
    exons = exons(find(exons(:,3)==m.exon),1:2) ;
    assert(size(all_exons,1)==size(cds_exons,1)+size(utr5_exons,1)+size(utr3_exons,1)+size(exons,1)+size(other,1))
    
    if size(other,1)>0
      FAULTS_name.exons.unknown_type{end+1} = gene.name;
      FAULTS_id.exons.unknown_type(end+1) = i;  
      warning('unknown childs ignored')
    end
    tis = [];
    Stop = [];
    tss = [];
    cleave = [];
    if gene.strand=='+',
      
      if ~isempty(cds_exons)  
        tis = cds_exons(1,1) ;
        Stop = cds_exons(end,2) ;
        if ~isempty(utr5_exons)  %% 
          if utr5_exons(end,2)==cds_exons(1,1)
            %% all good
            FAULTS_id.cdsUTR5.ok(end+1) = i;
            FAULTS_name.cdsUTR5.ok{end+1} = gene.name;
          else
            FAULTS_name.cdsUTR5.TISdiff{end+1} = gene.name;
            FAULTS_id.cdsUTR5.TISdiff(end+1) = i; 
            % warning(sprintf('last UTR-5 exon end (%i) is not equal to the start of the first coding exon (%i) (gene %s, contig %s)\nignoring 5''UTR\n', utr5_exons(end,2), cds_exons(1,1), gene.name, gene.chr));
            utr5_exons = [] ;
          end
	elseif ~isempty(exons) && any(exons(:,1)<tis) 
          idx = find(exons(:,1)<tis);
          utr5_exons = exons(idx,:);
          if sum(utr5_exons(:,2)>tis)>1
            FAULTS_name.cdsUTR5.mult_overlap{end+1} = gene.name;
            FAULTS_id.cdsUTR5.mult_overlap(end+1) = i; 
            % warning(sprintf('more than one exons overlaps start of the first coding exon (gene %s, contig %s) \nignoring 5''UTR \n', gene.name, gene.chr));
            utr5_exons = [];
          elseif ~any(utr5_exons(:,2)>tis)
            utr5_exons(end,2) = cds_exons(1,1);
            FAULTS_name.cdsUTR5.gap{end+1} = gene.name;
            FAULTS_id.cdsUTR5.gap(end+1) = i; 
            % warning(sprintf('extending 5'' UTR to start of CDS exon\n', gene.name, gene.chr));
          elseif utr5_exons(end,2)~=cds_exons(1,2)
            FAULTS_name.cdsUTR5.TSSdiff{end+1} = gene.name;
            FAULTS_id.cdsUTR5.TSSdiff(end+1) = i; 
            % warning(sprintf('TIS overlapping exon has different end than first cds_exon\n', gene.name, gene.chr));
            utr5_exons(end,2) = cds_exons(1,1);
          else
            FAULTS_id.cdsUTR5.ok(end+1) = i;
            FAULTS_name.cdsUTR5.ok{end+1} = gene.name;
            utr5_exons(end,2) = cds_exons(1,1);
          end
        else
          FAULTS_name.cdsUTR5.no_utr5{end+1} = gene.name;
          FAULTS_id.cdsUTR5.no_utr5(end+1) = i; 
          % warning(sprintf('cannot find 5'' UTR \n', gene.name, gene.chr));
        end ;
        
        if ~isempty(utr3_exons)  %% 
          if utr3_exons(1,1)==cds_exons(end,2)
            %% all good
            FAULTS_id.cdsUTR3.ok(end+1) = i;
            FAULTS_name.cdsUTR3.ok{end+1} = gene.name;
          else
            FAULTS_name.cdsUTR3.STOPdiff{end+1} = gene.name;
            FAULTS_id.cdsUTR3.STOPdiff(end+1) = i; 
            % warning(sprintf('first UTR-3 exon start (%i) is not equal to the end of the first coding exon (%i) (gene %s, contig %s)\nignoring 3''UTR\n', utr3_exons(end,2), cds_exons(1,1), gene.name, gene.chr));
            utr3_exons = [] ;
          end
        elseif ~isempty(exons) && any(exons(:,2)>Stop) 
          idx = find(exons(:,2)>Stop);
          utr3_exons = exons(idx,:);
          if sum(utr3_exons(:,1)<Stop)>1
            FAULTS_name.cdsUTR3.mult_overlap{end+1} = gene.name;
            FAULTS_id.cdsUTR3.mult_overlap(end+1) = i; 
            % warning(sprintf('more than one exons overlaps end of the first coding exon (gene %s, contig %s) \nignoring 3''UTR \n', gene.name, gene.chr));
            utr3_exons = [];
          elseif ~any(utr3_exons(:,1)<Stop)
            utr3_exons(1,1) = cds_exons(end,2);
            FAULTS_name.cdsUTR3.gap{end+1} = gene.name;
            FAULTS_id.cdsUTR3.gap(end+1) = i; 
            % warning(sprintf('extending 3'' UTR to end of CDS exon\n', gene.name, gene.chr));
          elseif utr3_exons(1,1)~=cds_exons(end,1)
            FAULTS_name.cdsUTR3.CLEAVEdiff{end+1} = gene.name;
            FAULTS_id.cdsUTR3.CLEAVEdiff(end+1) = i; 
            % warning(sprintf('TIS overlapping exon has different end than first cds_exon\n', gene.name, gene.chr));
            utr3_exons(1,1) = cds_exons(end,2);
          else
            FAULTS_id.cdsUTR3.ok(end+1) = i;
            FAULTS_name.cdsUTR3.ok{end+1} = gene.name;
            utr3_exons(1,1) = cds_exons(end,2);
          end
        else
          FAULTS_name.cdsUTR3.no_utr3{end+1} = gene.name;
          FAULTS_id.cdsUTR3.no_utr3(end+1) = i; 
          % warning(sprintf('cannot find 3'' UTR \n', gene.name, gene.chr));
        end ; % isempty(utr3_exons)
        
      % start no_cds_exons given
      elseif ~isempty(utr5_exons) & ~isempty(exons) & ~isempty(utr3_exons)
        % inferring cds_exons
        tis = utr5_exons(end,2) ;
        Stop = utr3_exons(1,1) ;
        idx = find(exons(:,2)>tis&exons(:,1)<Stop);
        cds_exons = exons(idx,:);
        if cds_exons(1,1)<tis 
          if cds_exons(1,1)~= utr5_exons(end,1) ;
            FAULTS_name.exonUTR5.TSSdiff{end+1} = gene.name;
            FAULTS_id.exonUTR5.TSSdiff(end+1) = i; 
            % warning(['first potential coding exon has different start than ' ...
            %          'last 5''UTR exon']) 
          end
          cds_exons(1,1)=tis;
        else
          FAULTS_name.exonUTR5.gap{end+1} = gene.name;
          FAULTS_id.exonUTR5.gap(end+1) = i; 
          % warning(['first potential coding exon starts downstream of tis']) 
          cds_exons(1,1)=tis;
        end
        if cds_exons(end,2)>Stop 
          if cds_exons(end,2)~= utr3_exons(1,2) ;
            FAULTS_name.exonUTR3.CLEAVEdiff{end+1} = gene.name;
            FAULTS_id.exonUTR3.CLEAVEdiff(end+1) = i; 
            % warning(['last potential coding exon has different end than ' ...
            %          'first 3''UTR exon']) 
          end
          cds_exons(end,2)=Stop;
        else
          FAULTS_name.exonUTR3.gap{end+1} = gene.name;
          FAULTS_id.exonUTR3.gap(end+1) = i; 
          % warning(['last potential coding exon ends upstream of cdsStop']) 
          cds_exons(end,2)=Stop;
        end 
      else
        tis = [] ;
        Stop = [] ;
        FAULTS_name.noCDS.noCDS{end+1} = gene.name;
        FAULTS_id.noCDS.noCDS(end+1) = i; 
        % warning(sprintf('cannot infer UTRs and coding regions \n', gene.name, gene.chr));
      end
      
      Stop=Stop-3 ;

      if size(utr3_exons,1)>1 && utr3_exons(1,2)-utr3_exons(1,1)<3
        utr3_exons = [utr3_exons(1,1) utr3_exons(end,2)];
        FAULTS_name.exons.TAGT_cases{end+1} = gene.name;
        FAULTS_id.exons.TAGT_cases(end+1) = i;
        % warning('TAGT case: invalid donor!')
      end
      % determine phase of cds exon
      cds_exons(1,3) = 0;
      cds_length = 0;
      for e=2:size(cds_exons,1)
        cds_length = cds_length + cds_exons(e-1,2)-cds_exons(e-1,1);
        phase = mod(cds_length,3);
        switch phase
         case 0
          cds_exons(e,3) = phase;
         case 1
          cds_exons(e,3) = 2;
         case 2
          cds_exons(e,3) = 1;
        end
      end
      exons = utr5_exons;
      if ~isempty(utr5_exons)
        tss =  utr5_exons(1,1);
      end ;
      if ~isempty(cds_exons)
        if ~isempty(exons)
          exons(end,2) = cds_exons(1,2) ;
          exons = [exons; cds_exons(2:end,1:2)] ;
        else
          exons = cds_exons(:,1:2) ;
        end ;
      end
      if ~isempty(utr3_exons)
        cleave =  utr3_exons(end,2);
        if ~isempty(exons)
          exons(end,2) = utr3_exons(1,2) ;
          exons = [exons; utr3_exons(2:end,:)] ;
        else
          exons = utr3_exons(:,1:2) ;
        end ;
      end
      
    else % gene.strand=='-',
      if ~isempty(cds_exons)  
        tis = cds_exons(end,2) ;
        Stop = cds_exons(1,1) ;
        if ~isempty(utr5_exons)  %% 
          if utr5_exons(1,1)==cds_exons(end,2)
            %% all good
            FAULTS_id.cdsUTR5.ok(end+1) = i;
            FAULTS_name.cdsUTR5.ok{end+1} = gene.name;
          else
            FAULTS_name.cdsUTR5.TISdiff{end+1} = gene.name;
            FAULTS_id.cdsUTR5.TISdiff(end+1) = i; 
            % warning(sprintf('last UTR-5 exon end (%i) is not equal to the start of the first coding exon (%i) (gene %s, contig %s)\nignoring 5''UTR\n', utr5_exons(end,2), cds_exons(1,1), gene.name, gene.chr));
            utr5_exons = [] ;
          end
	elseif ~isempty(exons) && any(exons(:,2)>tis) 
          idx = find(exons(:,2)>tis);
          utr5_exons = exons(idx,:);
          if sum(utr5_exons(:,1)<tis)>1
            FAULTS_name.cdsUTR5.mult_overlap{end+1} = gene.name;
            FAULTS_id.cdsUTR5.mult_overlap(end+1) = i; 
            % warning(sprintf('more than one exons overlaps start of the first coding exon (gene %s, contig %s) \nignoring 5''UTR \n', gene.name, gene.chr));
            utr5_exons = [];
          elseif ~any(utr5_exons(:,1)<tis)
            FAULTS_name.cdsUTR5.gap{end+1} = gene.name;
            FAULTS_id.cdsUTR5.gap(end+1) = i; 
            utr5_exons(1,1) = cds_exons(end,2);
            % warning(sprintf('extending 5'' UTR to start of CDS exon\n', gene.name, gene.chr));
          elseif utr5_exons(1,1)~=cds_exons(end,1)
            FAULTS_name.cdsUTR5.TSSdiff{end+1} = gene.name;
            FAULTS_id.cdsUTR5.TSSdiff(end+1) = i; 
            % warning(sprintf('TIS overlapping exon has different end than first cds_exon\n', gene.name, gene.chr));
            utr5_exons(1,1) = cds_exons(end,2);
          else
            utr5_exons(1,1) = cds_exons(end,2);
          end
        else
          FAULTS_name.cdsUTR5.no_utr5{end+1} = gene.name;
          FAULTS_id.cdsUTR5.no_utr5(end+1) = i; 
          % warning(sprintf('cannot find 5'' UTR \n', gene.name, gene.chr));
        end ;
        
        if ~isempty(utr3_exons)  %% 
          if utr3_exons(end,2)==cds_exons(1,1)
            %% all good
            FAULTS_id.cdsUTR3.ok(end+1) = i;
            FAULTS_name.cdsUTR3.ok{end+1} = gene.name;
          else
            FAULTS_name.cdsUTR3.STOPdiff{end+1} = gene.name;
            FAULTS_id.cdsUTR3.STOPdiff(end+1) = i; 
            %  warning(sprintf('first UTR-3 exon start (%i) is not equal to the end of the first coding exon (%i) (gene %s, contig %s)\nignoring 5''UTR\n', utr3_exons(end,2), cds_exons(1,1), gene.name, gene.chr));
            utr3_exons = [] ;
          end
        elseif ~isempty(exons) && any(exons(:,1)<Stop) 
          idx = find(exons(:,1)<Stop);
          utr3_exons = exons(idx,:);
          if sum(utr3_exons(:,2)>Stop)>1
            FAULTS_name.cdsUTR3.mult_overlap{end+1} = gene.name;
            FAULTS_id.cdsUTR3.mult_overlap(end+1) = i; 
            % warning(sprintf('more than one exons overlaps end of the first coding exon (gene %s, contig %s) \nignoring 5''UTR \n', gene.name, gene.chr));
            utr3_exons = [];
          elseif ~any(utr3_exons(:,2)>Stop)
            FAULTS_name.cdsUTR3.gap{end+1} = gene.name;
            FAULTS_id.cdsUTR3.gap(end+1) = i; 
            utr3_exons(end,2) = cds_exons(1,1);
            % warning(sprintf('extending 3'' UTR to end of CDS exon\n', gene.name, gene.chr));
          elseif utr3_exons(end,2)~=cds_exons(1,2)
            FAULTS_name.cdsUTR3.CLEAVEdiff{end+1} = gene.name;
            FAULTS_id.cdsUTR3.CLEAVEdiff(end+1) = i; 
            % warning(sprintf('TIS overlapping exon has different end than first cds_exon\n', gene.name, gene.chr));
            utr3_exons(end,2) = cds_exons(1,1);
          else
            utr3_exons(end,2) = cds_exons(1,1);
          end
        else
          FAULTS_name.cdsUTR3.no_utr3{end+1} = gene.name;
          FAULTS_id.cdsUTR3.no_utr3(end+1) = i; 
          % warning(sprintf('cannot find 3'' UTR \n', gene.name, gene.chr));
        end ;

      % start no_cds_exons given
      elseif ~isempty(utr5_exons) & ~isempty(exons) & ~isempty(utr3_exons)
        % inferring cds_exons
        tis = utr5_exons(1,1) ;
        Stop = utr3_exons(end,2) ;
        idx = find(exons(:,1)<tis&exons(:,2)>Stop);
        cds_exons = exons(idx,:);
        if cds_exons(end,2)>tis 
          if cds_exons(end,2)~= utr5_exons(1,2) ;
            FAULTS_name.exonUTR5.TSSdiff{end+1} = gene.name;
            FAULTS_id.exonUTR5.TSSdiff(end+1) = i; 
            % warning(['first potential coding exon has different start than ' ...
            %          'last 5''UTR exon']) 
          end
          cds_exons(end,2)=tis;
        else
          FAULTS_name.exonUTR5.gap{end+1} = gene.name;
          FAULTS_id.exonUTR5.gap(end+1) = i; 
          % warning(['first potential coding exon starts downstream of tis']) 
          cds_exons(end,2)=tis;
        end
        if cds_exons(1,1)<Stop 
          if cds_exons(1,1)~= utr3_exons(end,1) ;
            FAULTS_name.exonUTR3.CLEAVEdiff{end+1} = gene.name;
            FAULTS_id.exonUTR3.CLEAVEdiff(end+1) = i; 
            % warning(['last potential coding exon has different end than ' ...
            %          'first 3''UTR exon']) 
          end
          cds_exons(1,1)=Stop;
        else
          FAULTS_name.exonUTR3.gap{end+1} = gene.name;
          FAULTS_id.exonUTR3.gap(end+1) = i; 
          % warning(['last potential coding exon ends upstream of Stop']) 
          cds_exons(1,1)=Stop;
        end 
      else
        tis = [] ;
        Stop = [] ;
        FAULTS_name.noCDS.noCDS{end+1} = gene.name;
        FAULTS_id.noCDS.noCDS(end+1) = i; 
        % warning(sprintf('cannot infer UTRs and coding regions \n', gene.name, gene.chr));
      end
      
      Stop=Stop+3 ;
      
      if size(utr3_exons,1)>1 && utr3_exons(end,2)-utr3_exons(end,1)<3
        utr3_exons = [utr3_exons(1,1) utr3_exons(end,2)];
        FAULTS_name.exons.TAGT_cases{end+1} = gene.name;
        FAULTS_id.exons.TAGT_cases(end+1) = i;
        % warning('TAGT case: invalid donor!')
      end
      
      % determine phase of cds exon
      if ~isempty(cds_exons)
        cds_exons(end,3) = 0;
        cds_length = 0;
        for e=size(cds_exons,1)-1:-1:1
          cds_length = cds_length + cds_exons(e+1,2)-cds_exons(e+1,1);
          phase = mod(cds_length,3);
          switch phase
           case 0
            cds_exons(e,3) = phase;
           case 1
            cds_exons(e,3) = 2;
           case 2
            cds_exons(e,3) = 1;
          end
        end
      end    
      exons = utr3_exons;
      if ~isempty(utr3_exons)
        cleave = utr3_exons(1,1);
      end 
      if ~isempty(cds_exons)
        if ~isempty(exons),
          exons(end,2) = cds_exons(1,2) ;
          exons = [exons; cds_exons(2:end,1:2)] ;
        else
          exons = cds_exons(:,1:2) ;
        end ;
      end
      if ~isempty(utr5_exons)
        tss = utr5_exons(end,2);
        if ~isempty(exons)
          exons(end,2) = utr5_exons(1,2) ;
          exons = [exons; utr5_exons(2:end,:)] ;
        else
          exons = utr5_exons(:,1:2) ;
        end 
      end
      
    end ; %strand==-'

    gene.exons{j} = exons ;
    gene.cds_exons{j} = cds_exons;
    gene.utr5_exons{j} = utr5_exons;
    gene.utr3_exons{j} = utr3_exons;
    
    gene.tis{j}=tis ;
    gene.cdsStop{j}=Stop ;
    gene.tss{j} = tss;
    gene.cleave{j} = cleave;
    
  end ; %%loop over transcripts
  
  gene.start = min(gene.exons{1}(:)) ;
  gene.stop = max(gene.exons{1}(:)) ;
  for j=2:length(gene.exons)
    gene.start = min(gene.start, min(gene.exons{j}(:))) ;
    gene.stop = max(gene.stop, max(gene.exons{j}(:))) ;
  end ;
  
  genes(i)=gene ;
end ;%%loop over genes



F1=fieldnames(FAULTS_name);
for f=1:length(F1)
  F=fieldnames(FAULTS_name.(F1{f}));
  for i=1:length(F)
    [tmp,idx] = unique(FAULTS_id.(F1{f}).(F{i}));
    FAULTS_name.(F1{f}).(F{i}) = FAULTS_name.(F1{f}).(F{i})(idx);
    FAULTS_id.(F1{f}).(F{i})= FAULTS_id.(F1{f}).(F{i})(idx);
  end
end

fprintf('\n\n');
fprintf('number of genes with overlapping exons:\t\t\t\t%i \n',length(unique(FAULTS_id.exons.overlapping)));
fprintf('number of genes with exons of unknown type:\t\t\t%i \n',length(unique(FAULTS_id.exons.unknown_type)));
fprintf('number of genes with invalid donor (TAGT):\t\t\t%i \n' ,length(unique(FAULTS_name.exons.TAGT_cases)));
        
fprintf('number of genes with cds and UTR5 derived TIS different:\t%i \n',length(unique(FAULTS_id.cdsUTR5.TISdiff)));
fprintf('number of genes with multiple UTR5s overlapping TIS: \t\t%i \n',length(unique(FAULTS_id.cdsUTR5.mult_overlap)));
fprintf('number of genes with gap between 5'' UTR and CDS: \t\t%i \n',length(unique(FAULTS_id.cdsUTR5.gap)));
fprintf('number of genes with different end of first CDS exon:\t\t%i \n',length(unique(FAULTS_id.cdsUTR5.TSSdiff)));
fprintf('number of genes with coding region but no 5'' UTR:\t\t%i \n',length(unique(FAULTS_id.cdsUTR5.no_utr5)));

fprintf('number of genes with cds and UTR3 derived STOP different:\t%i \n',length(unique(FAULTS_id.cdsUTR3.STOPdiff)));
fprintf('number of genes with multiple UTR3s overlapping STOP:\t\t%i \n',length(unique(FAULTS_id.cdsUTR3.mult_overlap)));
fprintf('number of genes with gap between 3'' UTR and CDS:\t\t%i \n',length(unique(FAULTS_id.cdsUTR3.gap)));
fprintf('number of genes with different start of last CDS exon:\t\t%i \n',length(unique(FAULTS_id.cdsUTR3.CLEAVEdiff)));
fprintf('number of genes with coding region but no 3'' UTR:\t\t%i \n',length(unique(FAULTS_id.cdsUTR3.no_utr3)));

fprintf('number of genes with exon and UTR5 derived TSS different:\t%i \n',length(unique(FAULTS_id.exonUTR5.TSSdiff)));
fprintf('number of genes with gap between 5'' UTR and exon:\t\t%i \n',length(unique(FAULTS_id.exonUTR5.gap)));
fprintf('number of genes with exon and UTR3 derived CLEAVE different:\t%i \n',length(unique(FAULTS_id.exonUTR3.CLEAVEdiff)));
fprintf('number of genes with gap between 3'' UTR and exon:\t\t%i \n',length(unique(FAULTS_id.exonUTR3.gap)));
fprintf('number of genes without coding or UTR information:\t\t%i \n',length(unique(FAULTS_id.noCDS.noCDS)));



return

function model = define_addContentFeatures(model)
  
% plifs for tiling array data
if model.use.addfeatures.tiling
  tiling_start =  model.cnt_contents + model.cnt_lengths + model.cnt_signals;
  content_names = fieldnames(model.contents);
  model.tiling_plif_ids = [];
  for s = 1:length(content_names)
    model.tiling.(content_names{s}) = tiling_start+s ;
    model.tiling_plif_ids = [model.tiling_plif_ids tiling_start+s];
  end
  model.cnt_tiling = length(fieldnames(model.tiling));
  % declare range
  model.tiling_range = struct;
  tiling_names = fieldnames(model.tiling) ;
  for s = 1:length(tiling_names)
    model.tiling_range = setfield(model.tiling_range, tiling_names{s},[6,11]);% ca. 2%-98% Percentile
  end
else
  model.cnt_tiling = 0;
end

return

function model = define_contents(model)
  
% declare content types

fid = 1;  
model.mod_words = [];
% the first contents consider all k-mers. the last three
% frame-content sensors only consider codons, with an offset from
% the end of the segment

content_start =  1 ;
cnt=0;

model.contents.intergenic = content_start+cnt ;cnt=cnt+1;
model.mod_words = [model.mod_words; 1 0] ;
model.contents.utr_exon   = content_start+cnt ;cnt=cnt+1;
model.mod_words = [model.mod_words; 1 0] ;
model.contents.cds_exon   = content_start+cnt ;cnt=cnt+1;
model.mod_words = [model.mod_words; 1 0] ;
model.contents.intron     = content_start+cnt ;cnt=cnt+1;
model.mod_words = [model.mod_words; 1 0] ;
if model.use.segments.intercistronic
  model.contents.intercistronic = content_start+cnt;cnt=cnt+1;
  model.mod_words = [model.mod_words; 1 0] ;
end
if model.use.contents.frame
  model.contents.cds_frame0   = content_start+cnt ;cnt=cnt+1;
  model.mod_words = [model.mod_words; 3 0] ;
  model.contents.cds_frame1   = content_start+cnt ;cnt=cnt+1;
  model.mod_words = [model.mod_words; 3 1] ;
  model.contents.cds_frame2   = content_start+cnt ;cnt=cnt+1;
  model.mod_words = [model.mod_words; 3 2] ;
end
model.cnt_contents = length(fieldnames(model.contents));

%hack
if model.cnt_contents<8
  fprintf(fid,'introducing dummy contents\n')
  for i= 1:8-model.cnt_contents
    if i==1
      model.contents.('dummy')= content_start+cnt;cnt=cnt+1;
    else
      model.contents.(['dummy' num2str(i)])= content_start+cnt;cnt=cnt+1;
    end
  end
  model.cnt_contents = length(fieldnames(model.contents));
  model.mod_words = [model.mod_words; 1 0] ;
end

% declare content range
model.contents_range = struct;
content_names = fieldnames(model.contents) ;
for s = 1:length(content_names)
  model.contents_range = setfield(model.contents_range, content_names{s},[-10,10]);
end


return

function model = define_lengths(model,organism)
  
fid = 1; 
% declare length types
length_start = model.cnt_contents+1 ;
cnt = 0;

model.lengths.intergenic = length_start+cnt; cnt = cnt+1;
model.lengths.utr5exon = length_start+cnt ;cnt = cnt+1;
if model.use.segments.transexon 
  model.lengths.transexon = length_start+cnt ;cnt = cnt+1;
end
model.lengths.single_cds_exon = length_start+cnt ;cnt = cnt+1;
model.lengths.first_cds_exon = length_start+cnt ;cnt = cnt+1;
model.lengths.middle_cds_exon = length_start+cnt ;cnt = cnt+1;
model.lengths.last_cds_exon = length_start+cnt ;cnt = cnt+1;
model.lengths.utr3exon = length_start+cnt;cnt = cnt+1;
if model.use.segments.polya_tail 
  model.lengths.polya_tail = length_start+cnt;cnt = cnt+1;
end
model.lengths.intron = length_start+cnt ;cnt = cnt+1;
if model.use.segments.intercistronic
  model.lengths.intercistronic = length_start+cnt ;cnt = cnt+1;
  model.lengths.intergenictrans = length_start+cnt ;cnt = cnt+1;
end
model.lengths.intergenic_long = length_start+cnt ;cnt = cnt+1;

model.cnt_lengths = length(fieldnames(model.lengths));


% declare length range
s1=which([organism.name '_lengths']);
s2=which([organism.clade '_lengths']);
if ~isempty(s1)
  fprintf(fid,'taking lengths range from file: %s_lengths.m \n',organism.name)
  model.lengths_range = feval([organism.name '_lengths']);
elseif ~isempty(s2)
  fprintf(fid,'taking lengths range from file: %s_lengths.m \n',organism.clade)
  model.lengths_range = feval([organism.clade '_lengths']);
else
  fprintf(fid,'no lengths range specified, using default settings\n')
  model.lengths_range = feval('default_lengths');
end

return

function model = define_loss(model)
  

% create loss matrices

equiv = [];
  
different = [model.segments.intergenic, model.segments.cds_exon, 3 ;
             model.segments.intergenic, model.segments.intron, 3;
             model.segments.intron, model.segments.cds_exon, 2];

%--------------------------------
if ~model.use.loss.orf_only
  different = [different;
               model.segments.intergenic, model.segments.utr5exon, 2;
               model.segments.intergenic, model.segments.utr3exon, 2];
  if model.use.segments.intercistronic
    different = [different;
                 model.segments.intercistronic, model.segments.utr5exon, 2;
                 model.segments.intercistronic, model.segments.utr3exon, 2;
                 model.segments.intergenictrans, model.segments.utr5exon, 2;
                 model.segments.intergenictrans, model.segments.utr3exon, 2];
    
  end
else
  equiv = [equiv;
           model.segments.intergenic model.segments.utr3exon ;
           model.segments.intergenic model.segments.utr5exon] ;
  if model.use.segments.intercistronic
    equiv = [equiv;
             model.segments.intercistronic model.segments.utr3exon ; 
             model.segments.intercistronic model.segments.utr5exon ;
             model.segments.intergenictrans model.segments.utr3exon ;
             model.segments.intergenictrans model.segments.utr5exon] ;
  end
end
%--------------------------------
if model.use.segments.polya_tail
  equiv = [equiv;
           model.segments.utr3exon, model.segments.polya_tail];
  different = [different;
               model.segments.intergenic, model.segments.polya_tail, 1];    
end
 %-------------------------------- 
if model.use.segments.intercistronic
  
  different = [different;
               model.segments.intercistronic, model.segments.cds_exon, 3 ;
               model.segments.intercistronic, model.segments.intron, 3;
               model.segments.intercistronic, model.segments.transexon, 2;
               model.segments.intergenic, model.segments.intercistronic, 0.1;
               model.segments.intergenictrans, model.segments.intercistronic, 0.1;
               model.segments.intergenictrans, model.segments.cds_exon, 3 ;
               model.segments.intergenictrans, model.segments.intron, 3;
               model.segments.intergenictrans, model.segments.transexon, 2;
               model.segments.intergenic, model.segments.intergenictrans, 0.1];

  if model.use.segments.polya_tail
    different = [different;
                 model.segments.intercistronic, model.segments.polya_tail,1;
                 model.segments.intergenictrans, model.segments.polya_tail, 1];
  end
end
%--------------------------------
if model.use.segments.transexon
  equiv = [equiv;
           model.segments.utr5exon, model.segments.transexon] ;     
  different = [different;
               model.segments.intergenic, model.segments.transexon, 1] ;
  if isfield(model.segments, 'intergenictrans'),
    different = [different;
                 model.segments.intergenictrans, model.segments.transexon, 1];    
  end ;
end
%--------------------------------

model.loss.segments = ones(model.cnt_segments) - eye(model.cnt_segments);
for i=1:size(different,1)
  model.loss.segments(different(i,1)+1, different(i,2)+1) = different(i,3) ;
  % model.loss.segments(different(i,1)+1, different(i,2)+1) = different(i,3)+1 ;warning('make loss matrix asymetric to enhance sensitivity')
  model.loss.segments(different(i,2)+1, different(i,1)+1) = different(i,3) ;
end ;

for i=1:size(equiv,1)
  model.loss.segments(equiv(i,1)+1, equiv(i,2)+1) = 0 ;
  model.loss.segments(equiv(i,2)+1, equiv(i,1)+1) = 0 ;
end ;


% tripple loss for coding
%model.loss.segments(model.segments.cds_exon+1, :) = 3*model.loss.segments(model.segments.cds_exon+1, :) ;
%model.loss.segments(:, model.segments.cds_exon+1) = 3*model.loss.segments(:, model.segments.cds_exon+1) ;

% first part is for segments, the second for nucleotides
model.loss.segments = [model.loss.segments 0.0*model.loss.segments] ;

if model.use.signals.tss
  model.loss.zero_range.tss = 10 ;
end
model.loss.zero_range.tis = 0;
model.loss.zero_range.don = 0;
model.loss.zero_range.acc = 0;
if model.use.signals.trans
  model.loss.zero_range.trans = 0;
end
model.loss.zero_range.cdsStop = 0;
if model.use.signals.polya
  model.loss.zero_range.polya = 10 ;
end
if model.use.signals.cleave
  model.loss.zero_range.cleave = 10 ;
end

model.loss.confirmed = 1 ;
model.loss.confirmed_tol = 1000 ; % region around confirmed gene
model.loss.unconfirmed = 1 ;

% model.loss.mustbe_corr.intergenic = 0 ;
% model.loss.mustbe_corr.utr5exon   = 0;
% model.loss.mustbe_corr.transexon  = 0 ;
% model.loss.mustbe_corr.cds_exon   = 1 ;
% model.loss.mustbe_corr.utr3exon   = 0 ;
% model.loss.mustbe_corr.polya_tail = 0 ;
% model.loss.mustbe_corr.intercistronic  = 0 ;
% model.loss.mustbe_corr.intron     = 1 ;
return ;

return

function model = define_mappings(model)
  

%%mappings between length and contents
  
model.plif_links = [] ;
model.plif_links(end+1,:) = [model.lengths.intergenic      model.contents.intergenic] ;
model.plif_links(end+1,:) = [model.lengths.first_cds_exon  model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.last_cds_exon   model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.middle_cds_exon model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.single_cds_exon model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.intron          model.contents.intron] ;
if model.use.segments.transexon
  model.plif_links(end+1,:) = [model.lengths.transexon       model.contents.utr_exon] ;
end
model.plif_links(end+1,:) = [model.lengths.utr5exon        model.contents.utr_exon] ;
model.plif_links(end+1,:) = [model.lengths.utr3exon        model.contents.utr_exon] ;
if model.use.segments.polya_tail
  model.plif_links(end+1,:) = [model.lengths.polya_tail      model.contents.utr_exon] ;
end
if model.use.segments.intercistronic
  model.plif_links(end+1,:) = [model.lengths.intercistronic  model.contents.intercistronic] ;
  model.plif_links(end+1,:) = [model.lengths.intergenictrans  model.contents.intergenic] ;
end
model.plif_links(end+1,:) = [model.lengths.intergenic_long model.contents.intergenic] ;


if isfield(model.state_ids, 'intron_long') 
  model.plif_links(end+1,:) = [model.lengths.intron_long model.contents.intron] ;  
end

%%mappings between length and additional contents (tiling)

if model.use.addfeatures.tiling
  model.tiling_links = [] ;   
  model.tiling_links(end+1,:) = [model.lengths.intergenic      model.tiling.intergenic] ;
  model.tiling_links(end+1,:) = [model.lengths.first_cds_exon  model.tiling.cds_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.last_cds_exon   model.tiling.cds_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.middle_cds_exon model.tiling.cds_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.single_cds_exon model.tiling.cds_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.intron          model.tiling.intron] ;
  if model.use.segments.transexon
    model.tiling_links(end+1,:) = [model.lengths.transexon       model.tiling.transexon] ;
  end
  model.tiling_links(end+1,:) = [model.lengths.utr5exon        model.tiling.utr_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.utr3exon        model.tiling.utr_exon] ;
  if model.use.segments.polya_tail
    model.tiling_links(end+1,:) = [model.lengths.polya_tail      model.tiling.utr_exon] ;
  end
  model.tiling_links(end+1,:) = [model.lengths.intergenic_long model.tiling.intergenic] ;
  if model.use.segments.intercistronic
    model.tiling_links(end+1,:) = [model.lengths.intercistronic  model.tiling.intercistronic] ;
  end
end

%%mappings between length and segments

model.seg_links = [] ;
model.seg_links(end+1,:) = [model.lengths.intergenic      model.segments.intergenic] ;
model.seg_links(end+1,:) = [model.lengths.first_cds_exon  model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.last_cds_exon   model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.middle_cds_exon model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.single_cds_exon model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.intron          model.segments.intron] ;
if model.use.segments.transexon
  model.seg_links(end+1,:) = [model.lengths.transexon       model.segments.transexon] ;
end
model.seg_links(end+1,:) = [model.lengths.utr5exon        model.segments.utr5exon] ;
model.seg_links(end+1,:) = [model.lengths.utr3exon        model.segments.utr3exon] ;
if model.use.segments.polya_tail
  model.seg_links(end+1,:) = [model.lengths.polya_tail      model.segments.polya_tail] ;
end
if model.use.segments.intercistronic
  model.seg_links(end+1,:) = [model.lengths.intergenictrans  model.segments.intergenictrans] ;
  model.seg_links(end+1,:) = [model.lengths.intercistronic  model.segments.intercistronic] ;
end
model.seg_links(end+1,:) = [model.lengths.intergenic_long model.segments.intergenic] ;
%

return

function model = define_segments(model)
% declare segment types

  cnt=0;  
model.segments.intergenic = cnt ;cnt=cnt+1;
model.segments.utr5exon   = cnt ;cnt=cnt+1;
if model.use.segments.transexon 
  model.segments.transexon  = cnt ;cnt=cnt+1;
end
model.segments.cds_exon   = cnt ;cnt=cnt+1;
model.segments.utr3exon   = cnt ;cnt=cnt+1;
if model.use.segments.polya_tail
  model.segments.polya_tail = cnt ;cnt=cnt+1;
end
if model.use.segments.intercistronic
  model.segments.intercistronic  = cnt ;cnt=cnt+1;
  model.segments.intergenictrans  = cnt ;cnt=cnt+1;
end
model.segments.intron     = cnt ;cnt=cnt+1;

model.cnt_segments = length(fieldnames(model.segments));


return

function model = define_signals(model,polya_sixmers)
  
% declare signal types
cnt=1;
signal_start = model.cnt_contents + model.cnt_lengths ;

if model.use.signals.tss
  model.signals.tss = signal_start+cnt;
  model.consensus.tss = {} ;
  model.consensus_pos.tss = [];
  cnt=cnt+1;
end

model.signals.tis = signal_start+cnt;
model.consensus.tis = {'atg'} ;
model.consensus_pos.tis = [0];
cnt=cnt+1;

model.signals.don = signal_start+cnt;
model.consensus.don = {'gt','gc'} ;
model.consensus_pos.don = [0,0];
cnt=cnt+1;

model.signals.acc = signal_start+cnt;
model.consensus.acc = {'ag'} ;
model.consensus_pos.acc = [2];
cnt=cnt+1;

model.signals.cdsStop = signal_start+cnt ;
model.consensus.cdsStop = {'taa','tga','tag'} ;
model.consensus_pos.cdsStop = [0,0,0];
cnt=cnt+1;

if model.use.signals.polya
  model.signals.polya = signal_start+cnt ;
  model.consensus.polya = polya_sixmers;
  model.consensus_pos.polya = zeros(1,length(polya_sixmers)) ;
  cnt=cnt+1;
end
if model.use.signals.cleave
  model.signals.cleave = signal_start+cnt ;
  model.consensus.cleave = {} ;
  model.consensus_pos.cleave = [];
  cnt=cnt+1;
end

if model.use.signals.trans
  model.signals.trans = signal_start+cnt;
  model.consensus.trans = {'ag'} ;
  model.consensus_pos.trans = [2];
  cnt=cnt+1;
end
if model.use.signals.transacc
  model.signals.transacc = signal_start+cnt;
  model.consensus.transacc = {'ag'} ;
  model.consensus_pos.transacc = [2];
  cnt=cnt+1;
end

model.cnt_signals = length(fieldnames(model.signals));


% declare signal ranges for PLiFs

if model.use.signals.tss
  model.signals_range.tss = [0,1] ;
end
model.signals_range.tis = [0,1] ;
model.signals_range.don = [0,1] ;
model.signals_range.acc = [0,1] ;
model.signals_range.cdsStop = [0,1] ;
if model.use.signals.polya
  model.signals_range.polya = [0,1] ;
end
if model.use.signals.cleave
  model.signals_range.cleave = [0,1] ;
end
if model.use.signals.trans
  model.signals_range.trans = [0,1] ;
end
if model.use.signals.transacc
  model.signals_range.transacc = [0,1] ;
end



return

function model = define_states(model)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% define states
model.states(1) = struct ;
cnt=0 ;

% sequence start 
cnt=cnt+1 ;
model.states(cnt).consensus={} ;
model.states(cnt).consensus_pos=[] ;
model.states(cnt).non_consensus={} ;
model.states(cnt).non_consensus_pos=[] ;
model.states(cnt).name='seq_start' ;
model.states(cnt).signal=[] ;
model.state_ids.seq_start=cnt ;

signal_names = fieldnames(model.signals) ;
for i=1:model.cnt_signals
  if strcmp(signal_names{i}, 'transacc'), continue; end ;
  if ~strcmp(signal_names{i},'acc') && ~strcmp(signal_names{i},'don') ...
        && ~strcmp(signal_names{i}, 'tis') && ~strcmp(signal_names{i}, 'cdsStop')...
        && ~strcmp(signal_names{i}, 'trans') && ~strcmp(signal_names{i}, 'cleave') 
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
    model.state_ids=setfield(model.state_ids, signal_names{i},cnt) ;
  elseif strcmp(signal_names{i},'cleave') 
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
    model.state_ids=setfield(model.state_ids, signal_names{i},[cnt]) ;
    if model.use.segments.intercistronic
      cnt=cnt+1 ;
      model.states(cnt).name = signal_names{i};
      model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
      model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
      model.states(cnt).non_consensus = {} ;
      model.states(cnt).non_consensus_pos = [] ;
      model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
      model.state_ids=setfield(model.state_ids, signal_names{i},[cnt-1 cnt]) ;
    end
  elseif strcmp(signal_names{i},'tis') | strcmp(signal_names{i},'cdsStop')
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.state_ids=setfield(model.state_ids, signal_names{i},[cnt-1 cnt]) ;
    if strcmp(signal_names{i}, 'tis')
    model.states(cnt).name = 'tis_acc' ;
      model.states(cnt).consensus = {'agatg'} ;
      model.states(cnt).consensus_pos = 2 ;
      model.states(cnt).signal = [model.signals.acc model.signals.tis] ;%%%% !!! 2 Zahlen
    else
      model.states(cnt).name = 'cdsStop_acc';
      model.states(cnt).consensus = {'agtaa', 'agtag', 'agtga'} ;
      model.states(cnt).consensus_pos = [2 2 2] ;
      model.states(cnt).signal = [model.signals.acc model.signals.cdsStop] ;%%%% !!! 2 Zahlen
    end
  elseif strcmp(signal_names{i},'trans') & model.use.states.SL1 & model.use.states.SL2
    cnt=cnt+1 ;
    model.states(cnt).name = 'SL1';
    model.states(cnt).consensus = getfield(model.consensus,'trans');
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,'trans');
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc]  ;
    cnt=cnt+1 ;
    model.states(cnt).name = 'SL1';
    model.states(cnt).consensus = {'agatg'} ; 
    model.states(cnt).consensus_pos = 2;
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc model.signals.tis]  ;
    model.state_ids = setfield(model.state_ids, 'SL1', [cnt-1 cnt]) ;
    cnt=cnt+1 ;
    model.states(cnt).name = 'SL2';
    model.states(cnt).consensus = getfield(model.consensus,'trans');
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,'trans');
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc]  ;
    cnt=cnt+1 ;
    model.states(cnt).name = 'SL2';
    model.states(cnt).consensus = {'agatg'} ; 
    model.states(cnt).consensus_pos = 2;
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc model.signals.tis]  ;%%%% !!! 3 Zahlen
    model.state_ids = setfield(model.state_ids, 'SL2', [cnt-1 cnt]) ;
  elseif strcmp(signal_names{i},'trans') & ~model.use.states.SL1 & ~model.use.states.SL2
    cnt=cnt+1 ;
    model.states(cnt).name = 'trans';
    model.states(cnt).consensus = getfield(model.consensus,'trans');
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,'trans');
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc]  ;
    cnt=cnt+1 ;
    model.states(cnt).name = 'trans';
    model.states(cnt).consensus = {'agatg'} ; 
    model.states(cnt).consensus_pos = 2;
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc model.signals.tis]  ;
    model.state_ids = setfield(model.state_ids, 'SL1', [cnt-1 cnt]) ;
  elseif strcmp(signal_names{i},'don')
    cnt_don = cnt ;
    cnt=cnt+8 ;
    for id = cnt_don+1:cnt_don+8 ;
      model.states(id).consensus = getfield(model.consensus,signal_names{i});
      model.states(id).consensus_pos = getfield(model.consensus_pos,signal_names{i});
      model.states(id).non_consensus = {} ;
      model.states(id).non_consensus_pos = [] ;
      model.states(id).signal=model.signals.don ;
    end ;
    model.states(cnt_don+1).name='UTR5_don' ;
    model.states(cnt_don+2).name='CDS_don_0' ;
    model.states(cnt_don+3).name='CDS_don_1a' ;
    model.states(cnt_don+3).non_consensus = {'t'} ;
    model.states(cnt_don+3).non_consensus_pos= [1] ;
    model.states(cnt_don+4).name='CDS_don_1b' ;
    model.states(cnt_don+4).consensus = {'tgt','tgc'} ;
    model.states(cnt_don+4).consensus_pos= [1,1] ;
    model.states(cnt_don+5).name='CDS_don_2a' ;
    model.states(cnt_don+5).non_consensus = {'ta', 'tg'} ;
    model.states(cnt_don+5).non_consensus_pos= [2,2] ;
    model.states(cnt_don+6).name='CDS_don_2b' ;
    model.states(cnt_don+6).consensus = {'tagt', 'tagc'} ;
    model.states(cnt_don+6).consensus_pos= [2,2] ;
    model.states(cnt_don+7).name='CDS_don_2c' ;
    model.states(cnt_don+7).consensus = {'tggt', 'tggc'} ;
    model.states(cnt_don+7).consensus_pos= [2,2] ;
    model.states(cnt_don+8).name='UTR3_don' ;
    
    model.state_ids.don=cnt_don+1:cnt_don+8 ;
  elseif strcmp(signal_names{i},'acc')
    cnt_acc = cnt ;
    cnt=cnt+8 ;
    for id = cnt_acc+1:cnt_acc+8 ;
      model.states(id).consensus = {'ag'} ;
      model.states(id).consensus_pos= [2] ;
      model.states(id).non_consensus={} ;
      model.states(id).non_consensus_pos=[] ;
      model.states(id).signal=model.signals.acc ;
    end ;
    model.states(cnt_acc+1).name='UTR5_acc' ;
    model.states(cnt_acc+2).name='CDS_acc_0' ;
    model.states(cnt_acc+3).name='CDS_acc_1a' ;
    model.states(cnt_acc+4).name='CDS_acc_1b' ;
    model.states(cnt_acc+4).non_consensus = {'aa','ag','ga'} ;
    model.states(cnt_acc+4).non_consensus_pos= [0,0,0] ;
    model.states(cnt_acc+5).name='CDS_acc_2a' ;
    model.states(cnt_acc+6).name='CDS_acc_2b' ;
    model.states(cnt_acc+6).consensus = {'agt', 'agc'} ;
    model.states(cnt_acc+6).consensus_pos= [2, 2] ;
    model.states(cnt_acc+7).name='CDS_acc_2c' ;
    model.states(cnt_acc+7).non_consensus = {'a'} ;
    model.states(cnt_acc+7).non_consensus_pos= [0] ;
    model.states(cnt_acc+8).name='UTR3_acc' ;
 
    model.state_ids.acc=cnt_acc+1:cnt_acc+8 ;

    if isfield(model.use, 'intron_long_state')&& model.use.intron_long_state
      cnt_intron_long = cnt;
      cnt = cnt+6;
      model.states(cnt_intron_long+1).consensus={} ;
      model.states(cnt_intron_long+1).consensus_pos=[] ;
      model.states(cnt_intron_long+1).non_consensus={} ;
      model.states(cnt_intron_long+1).non_consensus_pos=[] ;
      model.states(cnt_intron_long+1).name='intron_long_0' ;
      model.states(cnt_intron_long+1).signal=[] ;

      model.states(cnt_intron_long+2).consensus={} ;
      model.states(cnt_intron_long+2).consensus_pos=[] ;
      model.states(cnt_intron_long+2).non_consensus={} ;
      model.states(cnt_intron_long+2).non_consensus_pos=[] ;
      model.states(cnt_intron_long+2).name='intron_long_1a' ;
      model.states(cnt_intron_long+2).signal=[] ;
    
      model.states(cnt_intron_long+3).consensus={} ;
      model.states(cnt_intron_long+3).consensus_pos=[] ;
      model.states(cnt_intron_long+3).non_consensus={} ;
      model.states(cnt_intron_long+3).non_consensus_pos=[] ;
      model.states(cnt_intron_long+3).name='intron_long_1b' ;
      model.states(cnt_intron_long+3).signal=[] ;

      model.states(cnt_intron_long+4).consensus={} ;
      model.states(cnt_intron_long+4).consensus_pos=[] ;
      model.states(cnt_intron_long+4).non_consensus={} ;
      model.states(cnt_intron_long+4).non_consensus_pos=[] ;
      model.states(cnt_intron_long+4).name='intron_long_2a' ;
      model.states(cnt_intron_long+4).signal=[] ;

      model.states(cnt_intron_long+5).consensus={} ;
      model.states(cnt_intron_long+5).consensus_pos=[] ;
      model.states(cnt_intron_long+5).non_consensus={} ;
      model.states(cnt_intron_long+5).non_consensus_pos=[] ;
      model.states(cnt_intron_long+5).name='intron_long_2b' ;
      model.states(cnt_intron_long+5).signal=[] ;

      model.states(cnt_intron_long+6).consensus={} ;
      model.states(cnt_intron_long+6).consensus_pos=[] ;
      model.states(cnt_intron_long+6).non_consensus={} ;
      model.states(cnt_intron_long+6).non_consensus_pos=[] ;
      model.states(cnt_intron_long+6).name='intron_long_1a' ;
      model.states(cnt_intron_long+6).signal=[] ;

      model.state_ids.intergenic_long= cnt_intron_long+1:cnt_intron_long+6;
    end
%    cnt
  end
end

% intergenic_long
cnt=cnt+1 ;
model.states(cnt).consensus={} ;
model.states(cnt).consensus_pos=[] ;
model.states(cnt).non_consensus={} ;
model.states(cnt).non_consensus_pos=[] ;
model.states(cnt).name='intergenic_long' ;
model.states(cnt).signal=[] ;
model.state_ids.intergenic_long=cnt ;

% sequence end
cnt=cnt+1 ;
model.states(cnt).consensus={} ;
model.states(cnt).consensus_pos=[] ;
model.states(cnt).non_consensus={} ;
model.states(cnt).non_consensus_pos=[] ;
model.states(cnt).name='seq_end' ;
model.states(cnt).signal=[] ;
model.state_ids.seq_end = cnt ;

model.cnt_states = cnt ; 

return

function model = define_transition_pointers(model)
  
A = model.A;
mRNA_start_states_out=model.mRNA_start_states_out;
%%% transition_pointers contain for each allowed transition pointers to lengths plifs
transition_pointers = zeros(size(A)) ;
transition_pointers(~isinf(A))=A(~isinf(A)) ;
transition_pointers = transition_pointers' ;

%%% add pointers to content PLIFs
transition_pointers(:,:,2) = zeros(size(transition_pointers,1), size(transition_pointers,2), 1) ;
for i=1:size(model.plif_links,1)
  [idx1,idx2] = find(transition_pointers(:,:,1)==model.plif_links(i,1));
  for j=1:length(idx1)
    transition_pointers(idx1(j),idx2(j),2) = model.plif_links(i,2);
  end ;
end


%%% add pointers to tiling PLIFs
if model.use.addfeatures.tiling
  dim3 = size(transition_pointers,3);
  transition_pointers(:,:,dim3+1) = zeros(size(transition_pointers,1), size(transition_pointers,2), 1) ;
  for i=1:size(model.tiling_links,1)
    [idx1,idx2] = find(transition_pointers(:,:,1)==model.tiling_links(i,1));
    for j=1:length(idx1)
      transition_pointers(idx1(j),idx2(j),3) = model.tiling_links(i,2);
    end ;
  end
end

%%% add pointers to frame PLIFs
tmp = zeros(size(transition_pointers(:,:,1))) ;
tmp(mRNA_start_states_out, model.state_ids.cdsStop(1)) = model.contents.cds_frame0 ;% single cds exon  

if ~isfield(model.use.contents, 'pre_comp') || ~model.use.contents.pre_comp,
  tmp(mRNA_start_states_out, model.state_ids.don(2)) = model.contents.cds_frame0 ;% first cds exon 
  tmp(mRNA_start_states_out, model.state_ids.don(3:4)) = model.contents.cds_frame1 ;% first cds exon  
  tmp(mRNA_start_states_out, model.state_ids.don(5:7)) = model.contents.cds_frame2 ;% first cds exon 
  
  tmp(model.state_ids.acc(2:7), model.state_ids.don(2)) = model.contents.cds_frame0 ;% middle cds exon 
  tmp(model.state_ids.acc(2:7), model.state_ids.don(3:4)) = model.contents.cds_frame1 ;% middle cds exon 
  tmp(model.state_ids.acc(2:7), model.state_ids.don(5:7)) = model.contents.cds_frame2 ;% middle cds exon 
  tmp(model.state_ids.acc(2:7), model.state_ids.cdsStop(1)) = model.contents.cds_frame0 ;% last cds exon 
else
  tmp(mRNA_start_states_out, model.state_ids.don(2:7)) = model.contents.cds_frame0 ;% first cds exon 
  tmp(model.state_ids.acc(2),model.state_ids.don(2:7)) = model.contents.cds_frame0 ;% middle cds exon 
  tmp(model.state_ids.acc(3:4),model.state_ids.don(2:7)) = model.contents.cds_frame1 ;% middle cds exon 
  tmp(model.state_ids.acc(5:7),model.state_ids.don(2:7)) = model.contents.cds_frame2 ;% middle cds exon 
  tmp(model.state_ids.acc(2), model.state_ids.cdsStop(1)) = model.contents.cds_frame0 ;% last cds exon 
  tmp(model.state_ids.acc(3:4), model.state_ids.cdsStop(1)) = model.contents.cds_frame1 ;% last cds exon 
  tmp(model.state_ids.acc(5:7), model.state_ids.cdsStop(1)) = model.contents.cds_frame2 ;% last cds exon 
end ;


dim3 = size(transition_pointers,3);
transition_pointers(:,:,dim3+1) = tmp' ; % don't touch this---tmp has to be transposed!

a_trans = zeros(4,sum(~isinf(A(:)))) ;
% [from,to,val,segment]
k = 0 ;
for i=1:size(A,1)
  idx = find(~isinf(A(i,:))) ;
  val = A(i,idx) ;
  pen = transition_pointers(idx,i,1) ;
  a_trans(1,k+1:k+length(idx))=i-1 ;
  a_trans(2,k+1:k+length(idx))=idx-1 ;  
  a_trans(3,k+1:k+length(idx))=val ;    
  for q=1:length(pen) 
    idx_=find(model.seg_links(:,1)==pen(q)) ;
    if isempty(idx_), 
      assert(pen(q)==0) ; 
    else
      assert(length(idx_)==1) ;
      pen(q) = model.seg_links(idx_,2) ;
    end ;
  end ;
  a_trans(4,k+1:k+length(idx))= pen;
  k=k+length(idx) ;
end ;
%a_trans = a_trans' ;
%[tmp,idx] = sort(a_trans(:,2)) ;
%a_trans = a_trans(idx,:) ;

model.transition_pointers = transition_pointers ;
model.a_trans = a_trans;

return

function model = gen_transition_matrix(model,PAR)

use=model.use;  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define transitions

A = -inf(model.cnt_states, model.cnt_states);
p = -inf(model.cnt_states,1);
q = -inf(model.cnt_states,1) ;
p(model.state_ids.seq_start) = 0 ;
q(model.state_ids.seq_end) = 0 ;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% start constructing transition matrix A

orf_from = -1*ones(model.cnt_states,1) ;
orf_to = -1*ones(model.cnt_states,1) ;

orf_from(model.state_ids.acc(2:7))= [ 0 1 1 2 2 2] ;
orf_from(model.state_ids.tis)     = 0 ; % AGAGT case
if model.use.states.SL1 & model.use.states.SL2
  orf_from(model.state_ids.SL1(2)) = 0 ; % AGAGT case
  orf_from(model.state_ids.SL2(2)) = 0 ; % AGAGT case
elseif model.use.signals.trans
  orf_from(model.state_ids.trans(2)) = 0 ; % AGAGT case
end
orf_to(model.state_ids.don(2:7))  = [0 1 1 2 2 2] ;
orf_to(model.state_ids.cdsStop(1))   = 0 ;
orf_info=[orf_from, orf_to] ;

%%%%%%%%%%%
%% no gene / intergenic stuff
%%%%%%%%%%%
if ~use.model.one_gene,
  A(model.state_ids.seq_start,  model.state_ids.seq_end) = model.lengths.intergenic ;
end
A(model.state_ids.intergenic_long, model.state_ids.intergenic_long) = 0; % no penalty et al
A(model.state_ids.seq_start, model.state_ids.intergenic_long) = model.lengths.intergenic_long ;
A(model.state_ids.intergenic_long, model.state_ids.seq_end) = model.lengths.intergenic_long ;

%%%%%%%%%%%
% 5' UTR 
%%%%%%%%%%%
gene_start_states_in = [];
gene_start_states_out = [];
mRNA_start_states_out = [model.state_ids.tis];

if use.signals.tss,
  gene_start_states_in = [gene_start_states_in model.state_ids.tss] ;
  gene_start_states_out = [gene_start_states_out model.state_ids.tss] ;
  if use.model.allow_missing_tss,
    assert(false) ;
    gene_start_states_in = [gene_start_states_in model.state_ids.tis] ;
  end ;
else
  assert(false)
  gene_start_states_in = [gene_start_states_in model.state_ids.tis] ;
end ;

if use.signals.trans,
  gene_start_states_in = [gene_start_states_in model.state_ids.SL1] ;
  gene_start_states_out = [gene_start_states_out model.state_ids.SL1(1)] ;
  gene_start_states_out = [gene_start_states_out model.state_ids.SL2(1)] ;
  mRNA_start_states_out = [mRNA_start_states_out model.state_ids.SL1(2)] ;%'AGAGT cases'
  mRNA_start_states_out = [mRNA_start_states_out model.state_ids.SL2(2)] ;%'AGAGT cases'
  if use.signals.tss, 
    A(model.state_ids.tss, model.state_ids.SL1)    = model.lengths.transexon ;  
  end ;
  if use.segments.intercistronic & use.signals.cleave ,
    A(model.state_ids.cleave(1), model.state_ids.SL1) = model.lengths.intergenictrans ;
    A(model.state_ids.cleave(2), model.state_ids.SL1) = model.lengths.intercistronic ;
    A(model.state_ids.cleave(2), model.state_ids.SL2) = model.lengths.intercistronic ;
  end ;
end ;

% gene start
A(model.state_ids.seq_start,gene_start_states_in) = model.lengths.intergenic ;
if use.model.use_multi
  A(model.state_ids.intergenic_long, gene_start_states_in) = model.lengths.intergenic_long ; 
end
    
A(gene_start_states_out,model.state_ids.tis(1)) = model.lengths.utr5exon ;
A(gene_start_states_out,model.state_ids.don(1)) = model.lengths.utr5exon ;
% spliced 5'utr
A(model.state_ids.acc(1), model.state_ids.don(1)) = model.lengths.utr5exon ;
A(model.state_ids.don(1), model.state_ids.acc(1)) = model.lengths.intron ;
% last 5'utr exon
A(model.state_ids.acc(1),model.state_ids.tis(1)) = model.lengths.utr5exon ;
A(model.state_ids.don(1), model.state_ids.tis(2)) = model.lengths.intron ; 


%%%%%%%%%%%
% CDS STUFF
%%%%%%%%%%%
A(mRNA_start_states_out, model.state_ids.cdsStop(1)) = model.lengths.single_cds_exon ; % trans(2) ok 
A(mRNA_start_states_out, model.state_ids.don(2:7)) = model.lengths.first_cds_exon ; % trans(2) ok
for i=2:7,
  A(model.state_ids.don(i), model.state_ids.acc(i)) = model.lengths.intron ;
  if isfield(model.state_ids, 'intron_long')
    A(model.state_ids.don(i), model.state_ids.intron_long(i-1)) = model.lengths.intron;
    A(model.state_ids.intron_long(i-1), model.state_ids.acc(i)) = model.lengths.intron_long;
    A(model.state_ids.intron_long(i-1),model.state_ids.intron_long(i-1)) = model.lengths.intron_long;
  end
end ;
A(model.state_ids.acc(2:7), model.state_ids.don(2:7)) = model.lengths.middle_cds_exon ;
A(model.state_ids.acc(2:7), model.state_ids.cdsStop(1)) = model.lengths.last_cds_exon ;
% agstop 
A(model.state_ids.don(2), model.state_ids.cdsStop(2)) = model.lengths.intron ;



%%%%%%%%%%%
% 3' UTR
%%%%%%%%%%%
A(model.state_ids.cdsStop, model.state_ids.don(8)) = model.lengths.utr3exon ;
A(model.state_ids.don(8), model.state_ids.acc(8)) = model.lengths.intron ;
A(model.state_ids.acc(8), model.state_ids.don(8)) = model.lengths.utr3exon ;

if use.signals.polya,
  % polyA has to be in last 3'UTR exon
  A(model.state_ids.cdsStop, model.state_ids.polya) = model.lengths.utr3exon ;
  A(model.state_ids.acc(8), model.state_ids.polya)=model.lengths.utr3exon ;
end ;

gene_stop_states = [];    
if use.signals.cleave,
  gene_stop_states = [gene_stop_states model.state_ids.cleave(1)];
  A(model.state_ids.cdsStop, model.state_ids.cleave) = model.lengths.utr3exon ;
  A(model.state_ids.acc(8), model.state_ids.cleave) = model.lengths.utr3exon ;
  if use.signals.polya,
    A(model.state_ids.polya, model.state_ids.cleave) = model.lengths.polya_tail ;
  end
else
  gene_stop_states = [gene_stop_states model.state_ids.cdsStop];
  if use.signals.polya,
    gene_stop_states = [gene_stop_states model.state_ids.polya];
  end
end ;
A(gene_stop_states, model.state_ids.seq_end)    = model.lengths.intergenic ; 
A(gene_stop_states, model.state_ids.intergenic_long) = model.lengths.intergenic_long ; 
if use.model.use_multi,
  A(gene_stop_states, gene_start_states_in) = model.lengths.intergenic ;    
end ;

if use.segments.intercistronic 
  for k=model.state_ids.SL1,
    idx = find(A(:,k)==model.lengths.intergenic) ;
    A(idx,k) = model.lengths.intergenictrans ;   
  end ;
end
%%%%%%%%%%%
% end transition definition
%%%%%%%%%%%


%%% technical stuff for intergenic_long

transitions = [];
for i=1:length(gene_start_states_in)
  transitions = [transitions; model.state_ids.seq_start gene_start_states_in(i)];
  for j=1:length(gene_stop_states)
    transitions = [transitions; gene_stop_states(j) gene_start_states_in(i)];
    transitions = [transitions; gene_stop_states(j) model.state_ids.seq_end] ;
  end
end

model.intergenic_long.transitions = unique(transitions,'rows');
model.intergenic_long.max_short_length = model.lengths_range.intron(2) ;
model.intergenic_long.step = model.lengths_range.intron(2); % use the intron length as maximal step size
model.intergenic_long.long_segment_parts = model.lengths_range.intron(2)/2-25 ;
%model.intergenic_long.step = 30 ; % makes about 30 (step) * 40 (grid)=1200nt per step

%%% 

q(gene_stop_states) = 0 ;

model.gene_start_states_in = gene_start_states_in;
model.gene_start_states_out = gene_start_states_out;
model.mRNA_start_states_out = mRNA_start_states_out;
model.gene_stop_states = gene_stop_states;

model.A = A;
model.p = p;
model.q = q;
model.orf_info = orf_info;
model.active_transitions = find(~isinf(A)) ;
model.cnt_transitions = length(model.active_transitions) ;



return

function weights=init_weights(model) ;
% weights=init_weights(model) ;

weights = struct;

%%% content detectors
content_names = fieldnames(model.contents) ;
for s = 1:model.cnt_contents,
  weights = setfield(weights, ['contents_' content_names{s}], zeros(1,model.bins));
end

%%% length detectors
length_names = fieldnames(model.lengths) ;
for s = 1:model.cnt_lengths,
  weights = setfield(weights, ['lengths_' length_names{s}], zeros(1,model.bins));
end

%%% signal detectors
signal_names = fieldnames(model.signals) ;
for s = 1:model.cnt_signals,
  weights = setfield(weights, ['signals_' signal_names{s}], zeros(1,model.bins));
end

%%% tiling detectors
if isfield(model, 'tiling')
  tiling_names = fieldnames(model.tiling) ;
  for s = 1:model.cnt_tiling,
    weights = setfield(weights, ['tiling_' tiling_names{s}], zeros(1,model.bins));
  end
end
%%% rna_seq detectors
if isfield(model, 'rna_seq')
  rna_seq_names = fieldnames(model.rna_seq) ;
  for s = 1:model.cnt_rna_seq,
    weights = setfield(weights, ['rna_seq_' rna_seq_names{s}], zeros(1,model.bins));
  end
end
%%% rna_seq_intron_track detectors
if isfield(model, 'rna_seq_intron_track')
  rna_seq_intron_track_names = fieldnames(model.rna_seq_intron_track) ;
  for s = 1:model.cnt_rna_seq_intron_track,
    weights = setfield(weights, ['rna_seq_intron_track_' rna_seq_intron_track_names{s}], zeros(1,model.bins));
  end
end

%%% transitions
weights.transitions = zeros(1,model.cnt_transitions) ;

return

function [model] = make_model(organism,LSL,Signals,old)
% [model] = make_model(bins)

if nargin<4
  old=0;
end
  
model.bins = LSL.method.plif_bins ;
% model.use_gc =  LSL.method.use_gc;
model.use = LSL.method.use;
polya_sixmers = Signals.polya.consensus;

model = define_segments(model);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%define features

model = define_contents(model);
model = define_lengths(model,organism);
model = define_signals(model,polya_sixmers);
model = define_addContentFeatures(model);

model.cnt_plifs = model.cnt_contents+model.cnt_lengths+model.cnt_signals+model.cnt_tiling;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% remember the ordering of these fields
model.segments_fields = fieldnames(model.segments) ;
model.contents_fields = fieldnames(model.contents) ;
model.lengths_fields = fieldnames(model.lengths) ;
model.signals_fields = fieldnames(model.signals) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%define states

model = define_states(model);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define mappings between length, contents and segments

model = define_mappings(model);
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

model = gen_transition_matrix(model) ;
% transition_pointers used to be called penalties
model = define_transition_pointers(model);
if old
  model.penalties = model.transition_pointers;
end
model.A(~isinf(model.A))=0 ;



model.cnt_parameters = model.cnt_transitions + model.cnt_plifs*model.bins ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% define loss 
model =  define_loss(model);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% mappings of PLIFS to parameters

if ~old
  map_plif2param = init_weights(model);
  map_plif2param = rmfield(map_plif2param,'transitions');
  weights_name = fieldnames(map_plif2param);
  start=0;
  for w=1:length(weights_name)
    len=length(getfield(map_plif2param, weights_name{w}));
    model.plif2param{w} = start+(1:len);
    start = start + len;
  end
  model.transitions2param = start+1:model.cnt_parameters ;
  
  assert(model.cnt_parameters-model.cnt_transitions==start)
end

return

function method = assign_model(method, models, par_names,  model_number)

%function method = assign_model(method, models, par_names, model_number)  

  
for num_param = 1:size(models,1)
  method.par_ms.(par_names{num_param}) = models(num_param,model_number) ;
end

if isfield(method, 'kernels') && ~isempty(method.kernels{1})
  nofKernels = length(method.kernels);
  for k =1:nofKernels
    kernel = method.kernels{k};
    for f =fieldnames(kernel)'
      if isnumeric(kernel.(f{1}))&~isempty(kernel.(f{1}))
        kernel.(f{1})= method.par_ms.(sprintf('kernel_%i_%s',k,f{1}));
      end
    end
    method.kernels{k}=kernel;
  end
end

return

function [ALL,is_invalid] = basic_cand_stuff(gene,ALL,sig,debug,signal_name)
% [ALL,is_invalid] = basic_cand_stuff(gene,ALL,sig,debug,signal_name)

if nargin<5
  signal_name =[];
end

is_invalid=0; 

genic_idx = find(ALL.POS>gene.start&ALL.POS<gene.stop);
ALL.GENE_ID(genic_idx) = gene.id;

if isfield(gene,'transacc') && ~isempty(gene.transacc)
  ALL.TRANS(genic_idx) = 1;
end
if ~isempty(gene.tss)
  ALL.TSS(genic_idx) = 1;
end

if ~isempty(signal_name)&&~isempty(gene.(signal_name))
  ALL.HAS_SIG(genic_idx) = 1;
end
  
if ~gene.is_valid
  is_invalid=1; 
  ALL.GENE_ID(genic_idx) =  - gene.id;
  return
end

if isequal(sig,'other')
  if gene.is_alt 
    ALL.ALTGENIC(genic_idx) = 1;
  end
else
  if any(gene.exon_in_alt_region)
    alt_reg = get_connected_regions(gene.exon_in_alt_region);  
    
    for i=1:length(unique(alt_reg))-1
      temp=find(alt_reg==i);
      alt_start = gene.splicegraph{1}(1,temp(1));
      alt_stop = gene.splicegraph{1}(2,temp(end));
      alt_genic_idx = find(ALL.POS>alt_start&ALL.POS<alt_stop);
      ALL.ALTGENIC(alt_genic_idx) = 1;
      if debug
        viewsplicegraph(gene,1)
        hold on 
        plot([alt_start alt_start],[1 10],'b')
        plot([alt_stop alt_stop],[1 10],'b')  
      end
    end
  end
end

if gene.is_alt 
  ALL.ALTGENIC(genic_idx) = 1;
end
return

function calc_plifs(PAR_1)
fid=1;
q=0;

fn_SVMs = PAR_1.FN.output_sig.(PAR_1.Signal_name).fn_SVMs;
method = PAR_1.Signals.(PAR_1.Signal_name).method;
num_partitions = PAR_1.SETs.num_partitions;

for p = 1:num_partitions
  fprintf(fid,'\n\n-----------\n');
  fprintf(fid,'PARTITION %i\n',p);
  fprintf(fid,'-----------\n');
  fn_SVM = sprintf('%sbest_partition=%i.mat', fn_SVMs,p) ;

  %--------------------------------------------------------------------------------%
  % load parameters from SVM file
  %--------------------------------------------------------------------------------%
  load(fn_SVM,'Train');
  PAR = Train.PAR;
  PAR.fn_SVMs=fn_SVM;
  clear Train
  
  %--------------------------------------------------------------------------------%
  % learn PLIFS on evaluation set and convert output on test set
  %--------------------------------------------------------------------------------%
  PAR.fn_pred = sprintf('%spartition=%i.mat',fn_SVMs,p);
  PAR.fn_plifs = sprintf('%spartition=%i_plifs.mat',fn_SVMs,p);
  assert(fexist(PAR.fn_pred)==1)
  assert(fexist(PAR.fn_plifs)==1)
  fprintf(fid,'\n already predicted on set ');
  fprintf(fid,'%i',PAR.SETs.which_pred);
        
  %--------------------------------------------------------------------------------%
  %%% learn PLIFS on evaluation set
  %--------------------------------------------------------------------------------%
  clear  prob prob_cum limits Conf Conf_cum
  fprintf('\nload fn_plif\n')
  load(PAR.fn_plifs,'LT', 'output');
  if ~all(ismember({'Conf','Conf_cum'},who_file(PAR.fn_plifs)))
    if ~all(ismember({'prob','prob_cum', 'limits'},who_file(PAR.fn_SVMs)))
      disp('learn PLIFs')
      assert(length(output)==length(LT))
      [prob, prob_cum,limits] = learn_plifs(output,LT,method.plif.bins,1,method.plif.min_incr );
      %save(PAR.fn_SVMs,'-append','prob', 'prob_cum','limits');
      save_append(PAR.fn_SVMs, 1, 'prob', prob, 'prob_cum', prob_cum, ...
                  'limits', limits);
    else
      load(PAR.fn_SVMs,'prob', 'prob_cum','limits');
    end
    disp('Convert SVM Output to Confidence on fn_plif')
    [Conf, Conf_cum] = Out2Conf(output,prob, prob_cum,limits);
    %save(PAR.fn_plifs,'-append','Conf','Conf_cum');
    save_append(PAR.fn_plifs,1, 'Conf', Conf, 'Conf_cum', Conf_cum);
    clear output Conf Conf_cum LT
  end
  %--------------------------------------------------------------------------------%
  %%% Convert SVM Outputs on test set 
  %--------------------------------------------------------------------------------%
  load(PAR.fn_pred,'output');
  if ~all(ismember({'Conf','Conf_cum'},who_file(PAR.fn_pred)))
    disp('Convert SVM Output to Confidence on fn_pred')
    [Conf, Conf_cum] = Out2Conf(output,prob, prob_cum,limits);
    %save(PAR.fn_pred,'-append','Conf','Conf_cum');
    save_append(PAR.fn_pred, 1,'Conf', Conf, 'Conf_cum', Conf_cum);
  end
end%loop over partitions
% [jobinfo,num_crashed] = rproc_wait(jobinfo,20,1,0);

return

function all_good = check_predictions(fn_genome_config,fn_pred,Signal_name,resolution)
% check_predictions(fn_pred,fn_genome_config)



P.fn_genome_config = fn_genome_config;
P.fn_pred = fn_pred;
P.resolution = resolution;
P.Signal_name = Signal_name;

genome_info = init_genome(fn_genome_config);

all_good = zeros(2, length(genome_info.contig_names));
for c = 1:length(genome_info.contig_names)
	  num = 0;
   for s = '+-'
     num = num+1;
     filename = sprintf('%scontig_%i%s', fn_pred, c, s);
     if fexist([filename '.pos'])&& fexist([filename '.output'])
       P.chrom = c;
       P.strand = s; 
       all_good(num,c) = check_predictions_helper(P) ;
       fprintf('check_predictions: strand: %s, chrom: %i, ok: %i',s, c,all_good);
     else
       fprintf('file missing: %s',filename)
     end 
   end  
end 

return

function par_ms= collect_ms_parameters(kernels, par_ms)
%par_ms= collect_model_parameters(kernels, par_ms)

nofKernels = length(kernels);
assert(nofKernels>=1)
for k = 1:nofKernels
  kernel = kernels{k};
  for f =fieldnames(kernel)'
    if isnumeric(kernel.(f{1}))&~isempty(kernel.(f{1}))
      par_ms.(sprintf('kernel_%i_%s',k,f{1})) = kernel.(f{1});
    end
  end
end


return

function [dummy1, dummy2]=concat_genomewide_predictons_helper(P)
% concat_genomewide_predictons_helper(P)

try
  pause('on')
  warning('off', 'MATLAB:typeaheadBufferOverflow');
catch
  % works only for matlab
end


dummy1=[] ;
dummy2=[] ;

fn_pred_all = P.fn_pred_all ;
fn_pred_split = P.fn_pred_split;
num_splits = P.num_splits;
avg_all_svms = P.avg_all_svms;
subtract_diff = P.subtract_diff;

% for H_sapiens_nohomologs:
% prefix = '/fml/ag-raetsch/share/projects/genefinding/H_sapiens_nohomologs/sensors/don/output/pred';
% fn_pred_all = sprintf('%scontig_%i%s_all.mat',prefix,c,s);

contig_pos = [];
contig_out = [];
contig_conf= [];
contig_cnfc= [];
contig_cnt=[] ;
contig_svm = [];
for p=1:num_splits
  fn_pred = sprintf('%s%i.mat', fn_pred_split, p);
  % for H_sapiens_nohomologs:
  % prefix = '/fml/ag-raetsch/share/projects/genefinding/H_sapiens_nohomologs/sensors/don/output/pred';
  % fn_pred = sprintf('%scontig_%i%s%s_%i.mat',prefix,c,s,PAR.Signal_name,p);

  for kk=1:10
    try
      clear pos output Conf Conf_cum
      load(fn_pred, 'inventory');
      %inventory
      for j=1:length(inventory)
        if kk>1,
          fprintf('trying to load variable "%s" from %s (trial %i)\n', inventory{j}, fn_pred, kk);
        end ;
        load(fn_pred,inventory{j})
        assert(exist(inventory{j},'var')==1);	 
      end
      break ;
    catch
      if kk==10      
        fprintf('giving up waiting for file %s\n', fn_pred)
      else
        fprintf('waiting for file: %s \n', fn_pred);    
        pause(30)
      end	
    end
  end

  if ~avg_all_svms
    if ~isempty(intersect(contig_pos, pos))
      warning('positions found in multiple splits');
      %% hack
      [tmp,idx1,idx2] = intersect(contig_pos, pos);
      pos(idx2)=[];
      output(idx2)=[];
      if exist('Conf') && exist('Conf_cum')
        Conf(idx2)=[];
        Conf_cum(idx2)=[];
      %else
      %  warning('no confs found')
      end
      %%hack
    end
    % assert(isempty(intersect(contig_pos, pos)));
    contig_pos = [contig_pos pos];
    contig_out = [contig_out output];
    contig_svm = [contig_svm repmat(p,size(pos))];
    if exist('Conf') && exist('Conf_cum')
      contig_conf = [contig_conf Conf];
      contig_cnfc = [contig_cnfc Conf_cum];
    end
    
  else
    [tmp,idx1,idx2] = intersect(contig_pos, pos);
    assert(length(idx1)==length(contig_pos))
    if isempty(contig_pos)
      contig_pos = pos;
      contig_out = output;
      if exist('Conf') && exist('Conf_cum')
        contig_conf = Conf(idx2);
        contig_cnfc = Conf_cum(idx2) ;
        contig_cnt = 1 ;
      else
        clear contig_conf contig_cnfc contig_cnt ;
      end
    else
      % otherwise the following will be wrong
      assert(isequal(sort(contig_pos), contig_pos)) ;
      assert(isequal(idx1, 1:length(idx1))) ;

      contig_out(idx1) = contig_out(idx1)+output(idx2);
      if exist('Conf') && exist('Conf_cum')
        contig_conf(idx1) = contig_conf(idx1) + Conf(idx2) ;
        contig_cnfc(idx1) = contig_cnfc(idx1) + Conf_cum(idx2) ;
        contig_cnt(idx1) = contig_cnt(idx1) + 1 ;
      else
        clear contig_conf contig_cnfc contig_cnt;
      end
    end
  end
  clear output pos Conf Conf_cum
end 
[pos sort_idx] = sort(contig_pos);
output = contig_out(sort_idx);
svm = contig_svm(sort_idx);

if avg_all_svms
  output = output./contig_cnt ;
end

% for contents: make sure there are smooth transitions at split boundaries
if subtract_diff
  ii = find(svm(1:end-1)~=svm(2:end));
  fprintf('number of blocks: %i\n',length(ii));
  for j=ii
    output(j+1:end) = output(j+1:end) - (output(j+1)-output(j));
  end
end

if exist('contig_conf') && exist('contig_cnfc') && isequal(size(contig_conf), size(contig_pos)) && isequal(size(contig_cnfc), size(contig_pos))
  if avg_all_svms
    contig_conf = contig_conf./(contig_cnt+1e-16) ;
    contig_cnfc = contig_cnfc./(contig_cnt+1e-16) ;
  end ;
  Conf = contig_conf(sort_idx);
  Conf_cum = contig_cnfc(sort_idx);
  
  assert(all(Conf>=0-1e-8&Conf<=1+1e-8));
  assert(all(Conf_cum>=0-1e-8&Conf_cum<=1+1e-8));
  fprintf('saving output, pos, Conf and Conf_cum\n')
  fprintf('save: %s\n',fn_pred_all)
  inventory = {'output', 'pos', 'Conf', 'Conf_cum'};
  save(fn_pred_all, '-V7','inventory', 'output', 'pos', 'Conf', 'Conf_cum')
else
  inventory = {'output', 'pos'};
  fprintf('saving output and pos without Conf and Conf_cum\n')
  fprintf('save: %s\n',fn_pred_all)
  save(fn_pred_all, '-V7','inventory', 'output', 'pos')
end

return

function jobinfo =  concat_genomewide_predictons(fn_genome_config,fn_pos,fn_pred,num_splits,avg_all_svms,subtract_diff,run_locally, RPROC)

% concat_genomewide_predictons(fn_genome_config,fn_pos,fn_pred,num_splits,run_locally, RPROC);

jobinfo = rproc_empty(0) ;
q=0;
% options.ncpus = 1;
% options.identifier = sprintf('%sconcat%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;


genome_info = init_genome(fn_genome_config);
P.avg_all_svms = avg_all_svms;
P.subtract_diff = subtract_diff;
P.num_splits = num_splits;
for c = 1:length(genome_info.contig_names)
  for s = '+-'
    P.fn_pred_all = sprintf('%scontig_%i%s_all.mat',fn_pred,c,s);
    P.fn_pred_split = sprintf('%scontig_%i%s_split=',fn_pred,c,s);
    if fexist(P.fn_pred_all)
      continue
    else
      q=q+1;
      if ~run_locally
        [mem_req, time_req, opts] = rproc_memtime_policy('concat_genomewide_predictons_helper',  0, RPROC.options) ;

        if isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs 
          jobinfo(q) = rproc_create('concat_genomewide_predictons_helper', P, mem_req, opts, time_req) ;
        else
          jobinfo(q) = rproc('concat_genomewide_predictons_helper', P, mem_req, opts, time_req);
        end
      else
        concat_genomewide_predictons_helper(P)
      end
    end
  end
end

if ~run_locally
   rproc_wait(jobinfo, 20, 1, -1);
end

return

function jobinfo = convert_predictions2confs(fn_genome_config,fn_pred,fn_svms,num_splits,run_locally, RPROC)
% jobinfo = convert_predictions2confs(fn_genome_config,fn_pred,fn_svms,num_splits,run_locally, RPROC)

jobinfo = rproc_empty(0) ;
% options.ncpus = 1;
% options.identifier = sprintf('%sconf%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;

fid=1;q=0;
for p = 1:num_splits
  fprintf(fid,'partition %i\n',p);
  
  fn_SVM = sprintf('%s%i.mat',fn_svms,p) ;
 
  %--------------------------------------------------------------------------------%
  % load parameters from SVM file
  %--------------------------------------------------------------------------------%
  
  genome_info = init_genome(fn_genome_config);
  clear  prob prob_cum limits Conf Conf_cum

  if ~(all(ismember({'prob', 'prob_cum','limits'}, who_file(fn_SVM))))
    warning('Variables prob, prob_cum and or limits not found in %s',fn_SVM)
    error('calculate PLIFS first: execute this function with PAR.tasks.pred_from_seq=0; to calculate PLIFS')
  else
    load(fn_SVM, 'Train');
    PAR_new = load(fn_SVM, 'prob', 'prob_cum', 'limits');
    if ~isfield(Train.PAR.SETs, 'which_pred')
      Train.PAR.SETs.which_pred = Train.PAR.SETs.which_pred_svm;
    end
  end
  assert(Train.PAR.SETs.which_pred == p)
  %------------------------------------------------------------------------------------%
  % perform transformations
  %------------------------------------------------------------------------------------%
  for c = 1:length(genome_info.contig_names)
    for s = '+-'
      PAR_new.fn_pred = sprintf('%scontig_%i%s_split=%i.mat',fn_pred,c,s,p);
      for j=1:20
        if ~fexist(PAR_new.fn_pred)
	  fprintf(fid,'\n waiting for file: %s\n',PAR_new.fn_pred);
          pause(30) 
	end
      end
      if ~fexist(PAR_new.fn_pred)
       fprintf(fid,'\n file not found: %s', PAR_new.fn_pred);
       fprintf(fid,'\n run genomewide_prediction first\n');
      %elseif all(ismember({'Conf', 'Conf_cum'}, who_file(PAR_new.fn_pred))),
      %  continue
      else
        fprintf(fid,'\n start transforming on contig %i%s\n',c,s);
        %%% Convert SVM Outputs on test set `
        q=q+1;
        if ~run_locally

          [mem_req, time_req, opts] = rproc_memtime_policy('Out2Confhelper',  0, RPROC.options) ;

          if isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs 
            jobinfo(q) = rproc_create('Out2Confhelper', PAR_new, mem_req, opts, time_req) ;
          else
            jobinfo(q) = rproc('Out2Confhelper', PAR_new, mem_req, opts, time_req);
          end
        else
          Out2Confhelper(PAR_new);
        end
      end
    end%loop over strands
  end%loop over contigs
end %% loop over partitions

rproc_wait(jobinfo, 20, 1, -1);

return

function POS=  find_consensus(seq,Signal,strand)
%  POS=  find_consensus(seq,Signal,strand) 

consensus = Signal.consensus ; 
lwin_big = Signal.lwin_big;
rwin_big = Signal.rwin_big;
offset = Signal.offset;



POS = zeros(1,floor(length(seq)/10));
num = 0;
seq=upper(seq);
for m = 1:length(consensus)  
  cons = upper(consensus{m});
  pos_tmp = findstr(seq(lwin_big+1:end-rwin_big),cons)+lwin_big;
  POS(num+[1:length(pos_tmp)]) = pos_tmp;
  num = num + length(pos_tmp);
end
POS(num+1:end)=[];
clear num 
POS = POS + offset;
%if strand=='-' & strcmp(Signal.name,'polya')
%  POS = -POS + length(seq)+1-length(cons)+1;
%elseif strand=='-' 
if strand=='-'
  POS = -POS + length(seq)+1;
end
POS = unique(sort(POS));

return

function  jobinfo = gen_all_positions(fn_genome_config,fn_pos,Signal,fn_training_blocks,fn_test_blocks, num_splits,run_locally,RPROC) 
  
jobinfo = rproc_empty(0);
if ~run_locally,  
  jobinfo = rproc_empty(0) ;
  options = RPROC.options;
  % options.ncpus = 1;
  % options.identifier = sprintf('%spos%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;
  % options.identifier = sprintf('pos%s_%s',PAR.Signal_name,PAR.method.name) ;
end

strands = '+-';
genome_info = init_genome(fn_genome_config);

% keyboard

blocks = [];
if num_splits~=1 
  % load(FN.output.fn_regions, 'regions')
  % load(FN.output.fn_split,'split')
  if ~isempty(fn_training_blocks) && fexist(fn_training_blocks)

    % load training splits
    TR = [] ;
    TR.blocks = load_struct(fn_training_blocks, 'blocks');
    tmp = load(fn_training_blocks, 'split') ;
    TR.split = tmp.split ;
    if ~isempty(TR.split)& ~isempty(TR.blocks)
      has_split = isfield(TR.blocks,'split')&&~isempty(TR.blocks(1).split);
      splits = fieldnames(TR.split);
      for f=1:length(splits)
        regids = getfield(TR.split, splits{f});
        [temp i1 i2] = intersect(regids, [TR.blocks.id]);
        if ~has_split
          for ii2=i2, 
            [TR.blocks(ii2).split] = deal(f);
          end 
          %[TR.blocks(i2).split] = deal(f);
        else
          assert(all([TR.blocks(i2).split] == f))      
        end
      end
      clear temp i1 i2 splits regids 
      blocks = [blocks TR.blocks];
      clear TR
    end
  end
  if ~isempty(fn_test_blocks) && fexist(fn_test_blocks)
    TE=[] ;
    TE.blocks = load_struct(fn_test_blocks, 'blocks') ;
    tmp = load(fn_test_blocks, 'split');
    TE.split = tmp.split ;
    has_split = isfield(TE.blocks,'split');  
    splits = fields(TE.split);
    for f=1:length(splits)
      regids = getfield(TE.split, splits{f});
      [temp i1 i2] = intersect(regids, [TE.blocks.id]);
      if ~has_split
        for ii2=i2, 
          [TE.blocks(ii2).split] = deal(f);
        end 
        %[TE.blocks(i2).split] = deal(f);
      else
        assert(all([TE.blocks(i2).split] == f))      
      end
    end
    if isfield(TE.blocks,'right_dist')
      TE.blocks = rmfield(TE.blocks,'right_dist');
    end
    if isfield(TE.blocks,'right_dist')
      TE.blocks = rmfield(TE.blocks,'right_dist');
    end
    blocks = [blocks TE.blocks];
    clear TE;
  end
end

%[temp, num_files] = unix(sprintf('ls %scontig_*.pos | wc -l',fn_pos));
%d=dir(sprintf('%scontig_*.pos',fn_pos));
%num_files = length(d) ;
%[temp, num_files1] = unix(sprintf('ls %scontig_*.svm | wc -l',fn_pos));
%d=dir(sprintf('%scontig_*.svm',fn_pos));
%num_files1=length(d) ;

%----------
% generate positions
%-----------
trivial_regions = init_regions(fn_genome_config);

% comp_region  = ones(1,length(trivial_regions));
num_jobs=0;


for t=1:length(trivial_regions)
  if fexist(sprintf('%scontig_%i%s.pos',fn_pos,trivial_regions(t).chr_num,trivial_regions(t).strand))&...
        fexist(sprintf('%scontig_%i%s.svm',fn_pos,trivial_regions(t).chr_num,trivial_regions(t).strand))
    % comp_region(t)  = 0;
    continue
  end
  num_jobs = num_jobs+1;
  
  P.region = trivial_regions(t); 
  P.blocks = blocks;
  P.Signal = Signal;
  P.fn_pos = fn_pos;
  P.num_splits = num_splits;

  if ~run_locally
    [mem_req, time_req, opts] = rproc_memtime_policy('gen_all_positions_rproc',  0, options) ;
    jobinfo(num_jobs) = rproc_create('gen_all_positions_rproc', P, mem_req, opts, time_req) ;
  else
    gen_all_positions_rproc(P);
  end
end

return

return

function [dummy1, dummy2]=gen_all_positions_rproc(P)

% gen_all_positions_rproc(P)
% P has fields:  
%   - region;  
%   - blocks 
%   - Signal 
%   - fn_pos 
%   - num_splits 
%   
  

dummy1=[] ;
dummy2=[] ;
  
% use the rproc.addpath way to add paths in client processes!!
%
%paths  

region = P.region;  
blocks = P.blocks;
Signal = P.Signal;
fn_pos = P.fn_pos;
num_splits =  P.num_splits;


fprintf('start loading sequence for region %i%s\r',region.chr_num,region.strand)
tt = load_sequence(region);
if ~isempty(Signal.consensus)
  pos = find_consensus(tt.seq,Signal,region.strand);
elseif region.strand=='+'
  pos = Signal.lwin_big+1:length(tt.seq)-Signal.rwin_big; 
elseif region.strand=='-'
  pos = Signal.rwin_big+1:length(tt.seq)-Signal.lwin_big; 
end    

clear tt;

%-------------
% Determine SVM
%-------------
svm_num = zeros(size(pos));

% faster if there are many blocks and few splits
% needs more memory
if ~isempty(blocks),
  splits=[blocks.split] ; 
else
  splits=[] ;
end ;
assert(length(splits)==length(blocks)) ;
for split = unique(splits),
  fprintf('split %i\r',split)
  
  split_idx = find(split==splits) ;

  split_pos = {} ; p=length(split_idx); 
  for b_idx=split_idx,
    assert(~isfield(blocks, 'num_fragments') || blocks(b_idx).num_fragments<=1) ;
    if region.strand==blocks(b_idx).strand(1) && region.chr_num==blocks(b_idx).chr_num(1),
      if isfield(blocks(b_idx),'reg_no_overlap') && ~isempty(blocks(b_idx).reg_no_overlap)
        split_pos{p} = blocks(b_idx).start+[blocks(b_idx).reg_no_overlap(1):blocks(b_idx).reg_no_overlap(2)];
      else
        split_pos{p} = blocks(b_idx).start:blocks(b_idx).stop ;
      end
    end ;
    p=p-1 ;
  end ;
  split_pos = [split_pos{:}] ;
  [tmp,idx]=intersect(pos, split_pos); 
  assert(all(svm_num(idx)==0 | svm_num(idx)==split))
  svm_num(idx) = split ;
end ;

if 0, % slow version
  for b_idx = 1:length(blocks)
    fprintf('block %i\r',b_idx)
    block = blocks(b_idx);
    if ~isfield(block, 'num_fragments'),
      block.num_fragments = 1 ;
    end ;
    if ~isfield(block, 'reg_no_overlap'),
      block.reg_no_overlap = [block.start block.stop] ;
    end ;
    for f_idx = 1:block.num_fragments
      if f_idx>1
        error('block.reg_no_overlap not implemented for more than one fragment')
      end
      if region.strand==block.strand(f_idx) && region.chr_num==block.chr_num(f_idx);
        % idx =
        % find(pos>=block.start(f_idx)+PAR.regions.offset&pos<=block.stop(f_idx)-PAR.regions.offset);
        
        idx = find(pos>=block.reg_no_overlap(1) & pos<=block.reg_no_overlap(2));
        assert(all(svm_num(idx)==0 | svm_num(idx)==block.split))
        svm_num(idx) = block.split;
        % keyboard
      end
    end    
  end
end ;

filename = sprintf('%scontig_%i%s',fn_pos,region.chr_num,region.strand)
if isequal(filename(1),'~')
  filename = sprintf('%s/%s', getenv('HOME'), filename(2:end));
end

if ~isempty(find(svm_num==0))

  idx = find(svm_num==0);
  % nums = repmat([1:num_splits],1,ceil(length(idx)/num_splits));
  % nums = nums(randperm(length(nums)));
  % make blocks instead (faster computing for contents)

  num_blocks = round(max(length(idx)/10000, num_splits));
  nums = zeros(1, num_blocks*ceil(length(idx)/num_blocks)) ;
  p=1 ;
  for i=1:num_blocks
    nums( p : p+ceil(length(idx)/num_blocks)-1 ) = repmat( mod(num_blocks,num_splits) + 1, 1, ceil(length(idx)/num_blocks)) ;
    p = p + ceil( length(idx)/num_blocks ) ;
  end
  assert(p-1==length(nums));
  svm_num(idx) = nums(1:length(idx));

end

if ~isempty(pos)
  save_score_pos(pos,svm_num, filename ,{'svm'});
else
  unix(sprintf('touch %s.pos',filename));
  unix(sprintf('touch %s.svm',filename));
  warning('no position found')
end

return

function jobinfo = genomewide_predictions(fn_genome_config,fn_pos,fn_pred,fn_svms,num_splits,avg_all_svms,run_locally, RPROC, verb)
% jobinfo = genomewide_predictions(fn_pos,fn_pred,fn_genome_config,verb)


if nargin<9
  verb=0;
end
if nargin<7
  run_locally=1;
end
if nargin<6
  avg_all_svms = 0;
end

jobinfo = rproc_empty(0) ;
if ~run_locally
  jobinfo = rproc_empty(0) ;
  options = RPROC.options;
  options.ncpus = 1;
  % options.identifier = sprintf('%spred%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;
  MEMREQ = RPROC.MEMREQ;
end

    
fid=1;
q=0;

genome_info = init_genome(fn_genome_config);
num_pred = num_splits * length(genome_info.contig_names)*2;
fprintf(fid,'total number of jobs for prediction (num_svms*num_contigs*num_strands): %i\n',num_pred);

trivial_regions = init_regions(fn_genome_config);

for p = 1:num_splits
  fprintf(fid,'partition %i\n',p);
  
  fn_SVM = sprintf('%s%i.mat',fn_svms,p) ;
  %--------------------------------------------------------------------------------%
  % load parameters from SVM file
  %--------------------------------------------------------------------------------%
  
  load(fn_SVM,'Train');
  assert(Train.PAR.SETs.which_pred == p)
  
  PAR_new = Train.PAR;
  
  if isfield(PAR_new.Signal.method,'genomewide_predict_fct') && ~isempty(PAR_new.Signal.method.genomewide_predict_fct)
    PAR_new.Signal.method.predict_fct =  PAR_new.Signal.method.genomewide_predict_fct;
  end
  PAR_new.fn_SVMs = fn_SVM;
  PAR_new.pred_from_seq=1;
  PAR_new.avg_all_svms = avg_all_svms ; 
  if ~isfield(PAR_new.SETs,'which_pred')
    PAR_new.SETs.which_pred=PAR_new.SETs.which_pred_svm;
  end  
  assert(Train.PAR.SETs.which_pred == p)
  clear Train
  
  %------------------------------------------------------------------------------------%
  % perform genome-wide predictions
  %------------------------------------------------------------------------------------%
  for r_idx = 1:length(trivial_regions) 
    c = trivial_regions(r_idx).chr_num;
    s = trivial_regions(r_idx).strand;
    PAR_new.fn_pos = fn_pos;
    PAR_new.fn_pred = sprintf('%scontig_%i%s_split=%i.mat',fn_pred,c,s,p);
    if verb
      fprintf(fid,'\n start predicting on contig %i%s\n',c,s);
    end
    if ~fexist(PAR_new.fn_pred) 
      PAR_new.region = trivial_regions(r_idx);
      PAR_new.plif_on = 0;
      PAR_new.eval_on = 0;

      q=q+1; 
      if ~run_locally


        if ~isfield(RPROC,'divide4pred') || ~RPROC.divide4pred

          [mem_req, time_req, opts] = rproc_memtime_policy('load_and_predict',  0, options) ;

          if isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs 
            jobinfo(q) = rproc_create('load_and_predict', PAR_new, mem_req, opts, time_req) ;
          else
            jobinfo(q) = rproc('load_and_predict', PAR_new, mem_req, opts, time_req) ;
          end
        else
          q=q-1;
          
          output = [];
          pos = [];
          all_done=1;Temp_old =[];
          for chunk_idx=1:RPROC.numchunks
            PAR_new.chunk_idx = chunk_idx;PAR_new.RPROC.numchunks = RPROC.numchunks ;
            PAR_new.RPROC.divide4pred = RPROC.divide4pred;

            [mem_req, time_req, opts] = rproc_memtime_policy('load_and_predict',  0, options) ;

            if  ~fexist(sprintf('%s%i.mat',PAR_new.fn_pred, chunk_idx)) && isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs 
              q=q+1;
              jobinfo(q) = rproc_create('load_and_predict', PAR_new, mem_req, opts, time_req);
              all_done=0;
            elseif  ~fexist(sprintf('%s%i.mat',PAR_new.fn_pred, chunk_idx))
              q=q+1;
              jobinfo(q) = rproc('load_and_predict', PAR_new, mem_req, opts, time_req);
              all_done=0;
            else 
              Temp = load(sprintf('%s%i.mat',PAR_new.fn_pred, chunk_idx) )
              Temp.PAR = rmfield(Temp.PAR,'RPROC');
              Temp.PAR = rmfield(Temp.PAR,'chunk_idx');
              if ~isempty(Temp_old)
                assert(isequal(Temp_old.svmFileName,Temp.svmFileName));
                assert(isequal(Temp_old.PAR,Temp.PAR));
                assert(isequal(Temp_old.sg_ver,Temp.sg_ver));
              end
              Temp_old = Temp;
              output = [output Temp.output];
              pos = [pos Temp.pos];
            end
          end
          if all_done
            Temp = rmfield(Temp,'output');Temp = rmfield(Temp,'pos');
            %save(PAR_new.fn_pred,'struct','Temp');
            save_append(PAR_new.fn_pred, 0, Temp);
            %save(PAR_new.fn_pred,'-append','output','pos');
            inventory = {'output', 'pos'};
	    assert(isequal(size(pos),size(output)))
            save_append(PAR_new.fn_pred, 1, 'inventory', inventory,  'output', output, 'pos', pos);
          end
        end
      else
        load_and_predict(PAR_new)
      end
    else
      if verb
        fprintf(fid,'\n file already exists ');
      end
    end
  end%loop over contigs
end %% loop over partitions

if ~exist('jobinfo')| isempty(jobinfo),
  jobinfo = rproc_empty(0) ;
  return 
end

% [jobinfo, meta_jobinfo] = rproc_submit_batch(jobinfo, PAR.RPROC.exm_per_batch) ;
if ~(isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs),
%   [jobinfo,num_crashed] = rproc_wait(jobinfo,20,1,0);
end ;

[jobinfo,num_crashed] = rproc_wait(jobinfo, 30, 1, -1);


return

function jobinfo = gen_training_examples4signal(PAR)
% gen_labels_and_train(PAR)

%paths
  

Signal_name = PAR.Signal_name;
Signal = PAR.Signals.(PAR.Signal_name);
num_splits = PAR.SETs.num_splits;
label_source  = PAR.label_source.(Signal_name);
info = PAR.info_genes.(PAR.Signal_name);


FN = PAR.FN;
fn_genome_config = FN.input.fn_genome_config ;
fn_training_blocks = FN.output.fn_training_blocks;
fn_genes_signals = FN.output.fn_genes_signals ;

input_sig = FN.input_sig.(Signal_name);
fn_candsites =  input_sig.fn_candsites;
fn_filter_settings = input_sig.fn_filter_settings;
fn_examples = input_sig.fn_examples;
fn_example_statistics = input_sig.fn_example_statistics;

run_locally = PAR.tasks.signals.run_locally.pos ;

if ~run_locally
  RPROC = PAR.RPROC;
end



jobinfo = [] ;
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% load  trivial_regions, blocks, splits
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

trivial_regions = init_regions(fn_genome_config);


if ~fexist(fn_training_blocks)
  error('training block file does not exist; create it first with prepare_regions')
else
  blocks = load_struct(fn_training_blocks, 'blocks') ;
  load(fn_training_blocks, 'split') ;
  assert(length(fieldnames(split))==num_splits);
end


generate_examples=1;
if fexist([fn_candsites 'PAR.mat'])
  fprintf(1,'files with candidate pos already exists\n')
  Old = load([fn_candsites 'PAR.mat'],'P');
  diff_fields = compare_PAR(Signal,Old.P.Signal) ;
  if ismember('Signal',Signal_name)
    warning('label file exists, Signal parameters are differnt!')  
    % fprintf('label file exists, Signal parameters are differnt!')  
  end
  %[temp, num_files] = unix(sprintf('ls %scontig_*.pos | wc -l',fn_candsites));
  d=dir(sprintf('%scontig_*.pos',fn_candsites));
  num_files = length(d) ;

  if (num_files==length(trivial_regions))
    generate_examples=0;
    fprintf(1,'all cand files found; no label generation\n')
  else
    fprintf(1,'%i/%i cand files missing; start label generation\n',length(trivial_regions)-num_files,length(trivial_regions))
  end
  clear Old
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% generating .pos .label files
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if generate_examples
  fprintf(1,'generating labels\n')
  jobinfo = rproc_empty(0) ;
  
  P.fn_genes_signals = fn_genes_signals ;
  P.fn_candsites = fn_candsites;
  P.label_source = label_source;
  P.Signal = Signal; 
  P.info = info;
  
  D.P = P ;
  for j=1:length(trivial_regions)
    D.t_region = trivial_regions(j);
    %keyboard
    filename = sprintf('%scontig_%i%s',fn_candsites,trivial_regions(j).chr_num,trivial_regions(j).strand);
    if fexist(filename)
      continue
    end
    if run_locally
      save_label_files_starter(D);
    else
      [mem_req, time_req, opts] = rproc_mem_policy('save_label_files_starter',  0, RPROC.options) ;%RPROC.MEMREQ ;

      jobinfo(j) = rproc('save_label_files_starter', D, mem_req, opts, time_req) ;
      % save_label_files_starter(D)
    end;
    % save_label_files(t_region,PAR,genes)
  end
  if ~isempty(jobinfo),
    [jobinfo_1, num_crashed]=rproc_wait(jobinfo, 10, 1, -1);
  else
    num_crashed = 0 ;
    jobinfo_1 = rproc_empty(0) ;
  end ;
  %[temp, num_files] = unix(sprintf('ls %scontig_*.pos | wc -l',fn_candsites));
  d=dir(sprintf('%scontig_*.pos',fn_candsites));
  num_files = length(d) ;
  assert(num_files==length(trivial_regions))
  if num_crashed>0, 
    num_crashed
    %return
  end 
  if ~isempty(jobinfo_1)
    rproc_cleanup(jobinfo_1) ;
  end ;
end
clear P

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% generating example files
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

modes = {'train','test'};
for mode=modes
  [filename, exists]=gen_example_filename(Signal,fn_filter_settings,fn_examples,mode{1},num_splits);
  if ~exists
   
    fprintf(1, 'SAVE EXAMPLES ....');
    
    P.Signal = Signal;
    P.num_splits = num_splits;
    P.fn_training_blocks = fn_training_blocks;
    P.fn_candsites = fn_candsites;
    P.fn_filter_settings = fn_filter_settings ;
    P.fn_examples = fn_examples;
    P.fn_example_statistics = fn_example_statistics;
    if 0% ~run_locally
      D.blocks = blocks;
      D.split = split;
      D.P = P;

      [mem_req, time_req, opts] = rproc_mem_policy('save_examples',  0, RPROC.options) ;%RPROC.MEMREQ ;

      jobinfo = rproc('save_examples', D, mem_req, opts, time_req);
      [jobinfo_1, num_crashed]=rproc_wait(jobinfo, 60, 1, -1);
    else
      save_examples(blocks,split,P) ; num_crashed = 0 ;
    end
    if num_crashed>0
      return
    end
  end
end


return

function [POS,LABEL,CONF] = get_cand_others(P)
% [POS,LABEL,CONF] = get_cand_others(P)

debug = 0;  

genes = P.genes ;
seq = P.seq ;
Signal = P.Signal ;
info = P.info ;
strand = P.strand ;
signal_name = P.signal_name ;
%clear P

%[Signal,info,info_names,sig] = initialize_cand_search(PAR,genes,strand,signal_name,debug);
[Signal,info_names,sig] = initialize_cand_search(Signal, info, genes, strand, signal_name, debug);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% GET ALL CONSENSUS SITES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isempty(Signal.consensus)
  fprintf(1,'getting all consensus sites for: %s\n',Signal.consensus{:})
  POS =  find_consensus(seq,Signal,strand);
else
  fprintf(1,'no consensus given, all positions are potential candidates\n')
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GET ALL POSITIVE SITES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

num = 0;
pos = zeros(1,length(seq));
conf = -ones(length(info_names),length(seq))*200;
gene_idx = zeros(1,length(seq));
for g_idx = 1:length(genes)
  gene = genes(g_idx) ;
  if ~isempty(gene.(signal_name))
    unique_pos = unique([gene.(signal_name){:}]);
    unique_pos2 = setdiff(unique_pos, pos(1:num)) ;
    if ~isequal(unique_pos2, unique_pos),
      warning('identified other genes with same signal positions. Positions ignored')
      unique_pos = unique_pos2 ;
    end ;
    pos(num+[1:length(unique_pos)]) = unique_pos;
    gene_idx(num+[1:length(unique_pos)]) = repmat(g_idx,length(unique_pos),1);
    % === assertion fails for A. thaliana TSS !!! ===
    assert(length(unique(pos(1:num+length(unique_pos))))==length(pos(1:num+length(unique_pos))))
    conf_temp = zeros(length(info_names),length(unique_pos));
    for i=1:length(info_names)
      idx = find(gene.([signal_name '_info'])== ...
                 PAR.info_genes.(signal_name).(info_names{i}));
      pos_temp = [gene.(signal_name){idx}];
      [temp,idx1,idx2] = intersect(unique_pos,pos_temp);
      assert(length(idx) == length(pos_temp))
      conf_temp(i,idx1) = gene.([signal_name '_conf'])(idx(idx2));
    end
    conf(:,num+[1:length(unique_pos)]) = conf_temp; 
    num = num+length(unique_pos);
  end
end; %%%loop over genes
fprintf( 'found %d positive %s signals\n', num, signal_name );
pos(num+1:end) = [];
conf(:,num+1:end) = [];

if isempty(Signal.consensus)
  invalid_true = 0;
  neg_pos = find(~ismember([1:length(seq)],pos));
else
  invalid_true = length(setdiff(pos,POS));
  if invalid_true>10
    warning('%i positions did not exhibit the defined consensus', invalid_true) ;
  end
  pos = intersect(pos,POS); 
  neg_pos = setdiff(POS,pos);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SAMPLE DOWN
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
randn('state',24593854);
rand('state',24593854);

%neg_pos = sort(neg_pos) ;
fprintf(1,'Sampling down negatives:%d\n',Signal.sampling.downsample_neg);
if Signal.sampling.downsample_neg~=1
  var_ = 1/Signal.sampling.downsample_neg*1/5;
  %mean_ = Signal.sampling.mindist+1/Signal.sampling.downsample_neg;
  mean_ = 1/Signal.sampling.downsample_neg;
  num_new_pos = ceil((length(neg_pos)/mean_)*1.1);
  dist = abs(ceil(randn(num_new_pos,1).*var_+ mean_));
  %dist(dist<Signal.sampling.mindist)=dist(dist<Signal.sampling.mindist)+Signal.sampling.mindist;
  dist(dist==0) = deal(Signal.sampling.mindist);
  dd = cumsum(dist);
  neg_pos = intersect(neg_pos,dd);
  %keep = ceil(rand(1,length(neg_pos)*Signal.sampling.downsample_neg)*length(neg_pos));
  %keep = unique(keep);
  %neg_pos = neg_pos(keep);
elseif Signal.sampling.downsample_neg==1 & Signal.sampling.mindist>1
  remove = 1;
  while remove,
    dist = neg_pos(2:end)-neg_pos(1:end-1) ;
    idx = find(dist<Signal.sampling.mindist)+1;
    neg_pos(idx) = [];
    if isempty(idx)
      remove = 0;
    end
  end
end

POS = [pos neg_pos];
LABEL = [ones(1,length(pos)),-ones(1,length(neg_pos))]; 
COV1 = [conf zeros(size(conf,1),length(neg_pos))]; 


[temp,idx] = unique(POS);
assert(length(temp)==length(POS));
POS = POS(idx);
LABEL = LABEL(idx);
COV1 = COV1(:,idx);
if ~isequal(determine_engine, 'octave'), 
  assert(issorted(POS));
end ;


ALL = initialize_allconfs(POS,info_names);
ALL.COV1 = COV1;

  
fprintf('processing: ') ;
num_invalid_genes = 0;
for g_idx = 1:length(genes)
  if mod(g_idx, 100)==0,
    fprintf('%i/%i .. ', num, length(genes));
  end ;
  num = num+1;
  gene = genes(g_idx) ;
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % GET BASIC STUFF, 
  % if candidate pos is in gene, get gene id, if gene is valid, if
  % candidate pos is in alternative region 
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  [ALL,is_invalid] = basic_cand_stuff(gene,ALL,sig,debug,signal_name);
  num_invalid_genes  = num_invalid_genes + is_invalid;
end %%%loop over genes
fprintf('Done\n') ;


if nargout>=3,
  CONF = set_CONF(Signal, ALL, sig, info_names, signal_name);
end ;


% assert(all(sum(Conf(1:3,LABEL==1),1)))
% assert(size(Conf,1)==length(Signal.Conf_names));
fprintf(1,'number of invalid genes: \t\t\t%i\n',num_invalid_genes)
fprintf(1,'number of true %s sites: \t\t\t%i\n',signal_name,sum(LABEL==1))
fprintf(1,'number of invalid true sites (no consensus): \t%i\n',invalid_true)
fprintf(1,'number of decoy sites:\t\t\t\t%i\n',length(LABEL))
% fprintf(1,'mean confidence:\t\t\t\t%.2f\n',mean(sum(Conf,1)));

return

function [POS,LABEL,CONF] = get_cand_splicesites(P)
  
% [pos,label,Conf] = get_cand_splicesites(genes,seq,PAR)
% WARNING IMPLENENTATION OF COV_SKIPPED NOT FINISHED!!!

debug = 0;  
  
genes = P.genes ;
seq = P.seq ;
Signal = P.Signal ;
info = P.info ;
strand = P.strand ;
%clear P
  
% info = PAR.info_genes.(signal_name);
[Signal,info_names,sig,alt_names] = initialize_cand_search(Signal,info,genes,strand,'transcript',debug);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% GET ALL CONSENSUS SITES / INITIALIZE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

POS = find_consensus(seq,Signal,strand);

ALL = initialize_allconfs(POS, info_names);
ALL.LABEL =  -ones(1,length(POS)); 
clear POS;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% GET TRANSSPLICE SPECIFIC STUFF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isequal(Signal.name,'transacc')
  ALL = get_cand_transacc(genes,ALL,Signal,info_tran);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% detecting exons in regions of alternative splicing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('detecting exons in regions of alternative splicing') ;
for i=1:length(genes)
  genes(i).alt_regions = detectaltregions(genes(i)) ;
  map = zeros(1,size(genes(i).splicegraph{1},2)) ;
  for j=1:size(genes(i).alt_regions,2),
    map = map | segment_overlap(genes(i).alt_regions(1,j)+1, ...
                                genes(i).alt_regions(2,j)-1, ...
                                genes(i).splicegraph{1}(1,:), ...
                                genes(i).splicegraph{1}(2,:)) ;
  end ;
  if ~isempty(genes(i).alt_regions),
    assert(any(map==1)) ;
  end ;
  
  %extent alt_region to neigbouring exons
  map = [0 map 0];
  idx_temp = find(map);
  map(idx_temp+1)=1;
  map(idx_temp-1)=1;
  genes(i).exon_in_alt_region=map(2:end-1) ;
  %% Version 4225 works without extension
end ;

genes = annotate_splicegraph(genes);
% genes = cover_splicegraph(genes);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% START LOOP OVER GENES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

num_invalid_genes = 0;
invalid_true = 0;

for g_idx = 1:length(genes)
  
  
  %fprintf(1,'%i\r',g_idx);
  gene = genes(g_idx) ;
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % GET BASIC STUFF, 
  % if candidate pos is in gene, get gene id, if gene is valid, if
  % candidate pos is in alternative region 
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  [ALL,is_invalid] = basic_cand_stuff(gene,ALL,sig,debug);
  num_invalid_genes  = num_invalid_genes + is_invalid;
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  % GET TRUE SPLICE SITES
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  [don_exons, acc_exons] = ind2sub(size(gene.splicegraph{2}), find(triu(gene.splicegraph{2}))) ;
  true_splice = [];
  
  if (isequal(sig,'acc') && strand=='+')||...
        (isequal(sig,'don') && strand=='-'),
    true_splice = unique(gene.splicegraph{1}(1,acc_exons)) ;
  elseif (isequal(sig,'don')&& strand=='+') || ...
        (isequal(sig,'acc')&& strand=='-')
    true_splice = unique(gene.splicegraph{1}(2,don_exons)) ;
  end
   
  [tmp,ii1,ii2] = intersect(ALL.POS,true_splice);
  
  if length(tmp)<length(true_splice)
    invalid_true = invalid_true+length(true_splice)-length(tmp) ;
  end
  if ~isequal(Signal.name,'transacc')
    ALL.LABEL(ii1) = 1;
  else
    ALL.ISSPLICE(ii1) = 1;
  end
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  % GET TRUE ALTERNATIVE SPLICE SITES
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  

  if ~isempty(alt_names)
    [ALL,label_2] = get_true_alternative_sites(ALL,gene,true_splice,sig,debug);
  else
    label_2 = -ones(3,length(true_splice)); 
  end 
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  %%% CHECK COVERAGE 
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  
  if ~isempty(info_names) 
    ALL = get_coverage(ALL, gene, true_splice, label_2, info, info_names);
  end
      
  % keyboard
end %%%loop over genes

if isequal(Signal.name,'transacc')
  ALL.COV1 = ALL.CONF_TRANS;
end
if nargout>=3,
  CONF = set_CONF(Signal, ALL, sig, info_names, '');
end ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SAMPLE DOWN
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

pos=ALL.POS(ALL.LABEL==1);
neg_pos=ALL.POS(ALL.LABEL==-1);

% pos contains positive examples, neg_pos contains negative examples

%if Signal.sampling.downsample_neg~=1
%  fprintf(1,'Sampling down negatives:%d',Signal.sampling.downsample_neg);
%  randn('state',24593854);
%  dist = abs(ceil(randn(length(neg_pos),1)./Signal.sampling.downsample_neg/5+ Signal.sampling.mindist+1/Signal.sampling.downsample_neg));
%  dist(dist<Signal.sampling.mindist)=dist(dist<Signal.sampling.mindist)+Signal.sampling.mindist;
%  dd = cumsum(dist);
%  neg_pos = intersect(neg_pos,dd);
%elseif Signal.sampling.downsample_neg==1 & Signal.sampling.mindist>1
%  remove = 1;
%  while remove
%    dist = neg_pos(2:end)-neg_pos(1:end-1)
%    idx = find(dist<Signal.sampling.mindist)+1;
%    neg_pos(idx) = [];
%    if isempty(idx)
%      remove = 0;
%    end
%  end
%end

POS=[pos neg_pos];
LABEL=[ones(1,length(pos)), -ones(1,length(neg_pos))];

% keyboard
% assert(all(sum(Conf(1:3,LABEL==1),1)))
% assert(size(Conf,1)==length(Signal.Conf_names));
fprintf(1,'number of invalid genes: \t\t\t%i\n',num_invalid_genes)
fprintf(1,'number of true %s sites: \t\t\t%i\n',Signal.name,sum(LABEL==1))
fprintf(1,'number of invalid true sites (no consensus): \t%i\n',invalid_true)
fprintf(1,'number of decoy sites:\t\t\t\t%i\n',length(LABEL))
% fprintf(1,'mean confidence:\t\t\t\t%.2f\n',mean(sum(Conf,1)));

if invalid_true>0
  % keyboard
end

return

function ALL = get_coverage(ALL,gene,true_splice,label_2,info,info_names)
% [COV1,COV2] = get_coverage(ALL,gene,true_splice,info_names)
% 
% 
% get sum over all coverage for each true_splicesite and certain transcript()
% 
% INPUT  - gene:  gene structure of length 1;  
%          required fields:transcript_info,transcript_valid,exons,exons_confirmed, is_alt
%        - true_splice_site:    vector of all true splice sites (either
%          donor or acceptor, not both) that occur in the
%          splicegraph of the gene.
%        - info_names: cell which tells, you which transcripts to look
%          at ('anno','cDNA',...)
%   
% OUTPUT - cov1  number of coverage by given transcript type (according to info_names) for each true splice site. 
%        - cov2  number of times skipped by given transcript type (according to info_names) for each true splice site. 

  
COV1 = ALL.COV1 ;
COV2 = ALL.COV2;
POS = ALL.POS;
  
genic_idx = find(POS>gene.start&POS<gene.stop);
genic_pos = POS(genic_idx); 
assert(all(all(COV2(:,genic_idx)==0)))
assert(all(all(COV1(:,genic_idx)==0)))

cov1 = zeros(size(info_names,1),length(true_splice));
cov2 = zeros(size(info_names,1),length(true_splice));

% idx_const = find(sum(label_2==1)==0);
const_exons = gene.splicegraph{1}(:,find(~gene.exon_in_alt_region));
[tmp,idx_const,tmp] = intersect(true_splice,const_exons(:));
assert(all(sum(label_2(:,idx_const)==1)==0))
for type = 1:size(info_names,1) 
  if isequal(info_names{type,1},'anno')
    cov1(type,:) = -inf;
    cov2(type,:) = -inf;
  end
  t_idx = [];
  for tt1 = 1:size(info_names,2)
    if ~isempty(info_names{type,tt1})
      t_idx = [t_idx find(gene.transcript_info==info.(info_names{type,tt1}))];  
    end
  end
  if isempty(t_idx)
    continue
  end
  for t = t_idx
    if ~gene.transcript_valid(t)
      continue
    end
    [cov1_t, cov2_t] = get_coverage4transcript(gene,t,true_splice,info);
    if gene.transcript_info(t)==info.anno
      cov1(type,:) = max(cov1(type,:),cov1_t);
      cov2(type,:) = max(cov2(type,:),cov2_t); 
      if ~isempty(idx_const)&&~all(isinf(cov2_t(idx_const)))
        warning('here seems to be bug that should be fixed ... ') ;
        if keyboard_allowed()
          keyboard
        end
      end
    else
      cov1(type,:) = cov1(type,:) +cov1_t;
      cov2(type,:) = cov2(type,:) +cov2_t;
      if ~isempty(idx_const)&&~all(cov2_t(idx_const)==0)
        if keyboard_allowed()
          keyboard
        end
      end
    end
  end %%% loop over transcripts
end %%% loop over transcript types
idx_anno = find(strcmp(info_names,'anno')) ;
not_anno = setdiff(idx_anno,[1:size(info_names,1)]);

ii_alt2 = find(any(label_2==1));
ii_ir2 = find(label_2(2,:)==1);
if ~isempty(ii_alt2) && ~all(sum(cov2(not_anno,ii_alt2))>0|any(~isinf(cov2(idx_anno,ii_alt2))))
  warning('no cov2')
  % hold off;
  % viewsplicegraph(gene);
  if ~isempty(ii_ir2)
    fprintf('intron retention')
    % for a=1:length(true_alt_ir)
    %    hold on ; plot([true_alt_ir(a) true_alt_ir(a)],[1 10],'b')
    % end
  else
    %keyboard
  end
  gene.id
  warning('keyboard')
  %keyboard
end


[ii,ii1,ii2] = intersect(POS,true_splice);
assert(all(ismember(ii1,genic_idx)))

COV1(:,ii1) = cov1(:,ii2);
COV2(:,ii1) = cov2(:,ii2);

%%% SET COV2 OF NEGATIVE GENIC POSITIONS to min over coverage of
% neigbouring truesplicesites

true_splice = true_splice(ii2);
cov1 = cov1(:,ii2);


genic_neg_pos = setdiff(genic_pos,true_splice);
genic_cov2  =   zeros(size(info_names,1),length(genic_neg_pos));
[temp,ii1,ii2] = intersect(POS,genic_neg_pos);
assert(length(temp)==length(genic_neg_pos));
genic_neg_pos = genic_neg_pos(ii2);
if ~isequal(determine_engine, 'octave'),
  assert(issorted(true_splice))
end ;
for a=2:length(true_splice)
  idx = find(genic_neg_pos<true_splice(a)&genic_neg_pos>true_splice(a-1));
  genic_cov2(:,idx) = repmat(min([cov1(:,a),cov1(:,a-1)]'),length(idx),1)';
end
assert(all(all(COV2(:,ii1)==0)))
assert(all(all(COV1(:,ii1)==0)))
COV2(:,ii1) =genic_cov2;
    






function [cov1, cov2] = get_coverage4transcript(gene,transcript_idx,true_splice,info)

% [cov1, cov2] = get_coverage(gene,transcrit_idx,true_splice)
%
% for a given transcript transcrit_idx of the gene counts number of coverage for each
% of the true splice sites. 
% returned is cov1 the number of times a splice site is used in one
% evidence (e.g. transcript). In the case of ESTs this can be more than
% one, because a single transcript is produced out of a cluster of
% several ESTs.This information is stored in the
% field exons_confirmed{transcript_idx}, which is a Nx2 matrix, with the
% first number giving the confirmation for the exon and the second number
% giving the confirmation for the succeding
% intron. (exons_confirmed{transcript_idx}(end,2) shoeld always be 0, as
% there is no intron after the last exon.)
% The second returned value cov2 gives the number of evidences that
% disagree with the splicesite. e.i. the number of times a splice site is skipped.
   
  

exons = gene.exons{transcript_idx}(:,1:2);
exons_conf = gene.exons_confirmed{transcript_idx};

cov1 = -inf(1,length(true_splice));
cov2 = -inf(1,length(true_splice));

for a = 1:length(true_splice)
  %%% CHECK COVERAGE OF INCLUDED FORM     
  [idx1,idx2] = find(exons==true_splice(a));
  if ~isempty(idx1)
    assert(length(idx1)==1)
    cov1(a) = exons_conf(idx1,1); 
  end
  %%% CHECK EXON COVERAGE OF SKIPPED FORM    
  idx  = find(exons(:,1)<true_splice(a)&exons(:,2)>true_splice(a)) ;
  if ~isempty(idx)   
    assert(length(idx)==1)
    % cov2(a) = exons_conf(idx,2); 
    if gene.transcript_info(transcript_idx)==info.est|gene.transcript_info(transcript_idx)==info.cDNA &&...
          (idx==1|idx==size(exons,1))
      if all(label_2(:,a)==-1)
        continue
      else
        % alternative 3' or 5' or first or last exon; est ends
        % are usually unconfirmed 
      end
    elseif gene.transcript_info(transcript_idx)==info.anno
      if min(abs(true_splice(a)-exons(idx,:)))<3 
        % keyboard
        continue
      end
    end
    cov2(a) = exons_conf(idx,1);
  end  
  %%% CHECK INTRON COVERAGE OF SKIPPED FORM
  idx  = find(exons(1:end-1,2)<true_splice(a)&exons(2:end,1)>true_splice(a)) ;
  if ~isempty(idx)            
    assert(length(idx)==1)           
    if gene.transcript_info(transcript_idx)==info.anno
      cov2(a) = max(cov2(a),exons_conf(idx,2));      
    else
      cov2(a) = cov2(a)+exons_conf(idx,2);
    end
  end 
end %% loop over true_splice sites

assert(~any(~isinf(cov1)&~isinf(cov2)))





return

function  [ALL,label_2] = get_true_alternative_sites(ALL,gene,true_splice,sig,debug)

strand = gene.strand;
POS=ALL.POS;
LABEL_2=ALL.LABEL_2;

if (isequal(sig,'acc') & strand=='+')|...
      (isequal(sig,'don') & strand=='-'),
  which_end = 1;
  other_end = 2;
elseif (isequal(sig,'don')& strand=='+') | ...
      (isequal(sig,'acc')& strand=='-')
  which_end = 2;
  other_end = 1;
end
   
  
label_2 = -ones(3,length(true_splice)); 
% GET TRUE ALTERNATIVE SPLICE SITES

ii_es2 = []; ii_o2 = [];  ii_ir2 = [];  ii_3p2 = [];  ii_5p2 = []; 

if gene.alt_exon %% EXON SKIP
  alt_exons = gene.alt_exon_details.exons;
  true_alt_es = gene.splicegraph{1}(which_end,alt_exons(2,:)) ;
  [temp,ii1,ii_es2] = intersect(true_alt_es,true_splice);
  assert(length(temp)==length(unique(true_alt_es)))
  true_alt_es = true_alt_es(ii1);
  % splice_before_es = gene.splicegraph{1}(don_end,alt_exons(1,ii1)) ;
  label_2(1,ii_es2) = 1;
  if debug
    fprintf(1,'exon skip')
    viewsplicegraph(gene,1)
    hold on
    for jj=1:length(true_alt_es)
      plot([true_alt_es(jj) true_alt_es(jj)],[0 10],'b');
    end
    if keyboard_allowed(),
      keyboard
    end ;
  end
end %if gene.alt_exon

if gene.alt_intron
  alt_introns = gene.alt_intron_details.exons;
  true_alt_ir = gene.splicegraph{1}(which_end,alt_introns(other_end,:));
  [temp,ii1,ii_ir2] = intersect(true_alt_ir,true_splice);
  assert(length(temp)==length(unique(true_alt_ir)))
  true_alt_ir = true_alt_ir(ii1);
  % splice_before_ir = gene.splicegraph{1}(acc_end,alt_introns(1,ii1));
  % intron_ret(1,:) = gene.splicegraph{1}(which_end,alt_introns(other_end,ii1));
  % intron_ret(2,:) = gene.splicegraph{1}(other_end,alt_introns(which_end,ii1));
  label_2(2,ii_ir2) = 1;
  if debug
    fprintf(1,'intron retention')
    viewsplicegraph(gene,1)
    hold on
    for jj=1:length(true_alt_ir)
      plot([true_alt_ir(jj) true_alt_ir(jj)],[0 10],'b');
    end
    if keyboard_allowed(),
      keyboard
    end 
  end
end

if gene.alt_5prime&isequal(sig, 'don') 
  %if ~isempty(gene.alt_5prime_details), keyboard; end ;
  if gene.strand=='+',
    alt_5p = gene.alt_5prime_details.exons(1,:);
  else
    alt_5p = gene.alt_5prime_details.exons(2,:);
  end ;
  alt_5p = unique([alt_5p gene.alt_5prime_details.exons(3,:)]);
  true_alt_5p = gene.splicegraph{1}(which_end,alt_5p);
  [temp,ii1,ii_5p2] = intersect(true_alt_5p,true_splice);
  assert(length(temp)==length(unique(true_alt_5p)))
  true_alt_5p = true_alt_5p(ii1);
  % splice_before_5p = gene.splicegraph{1}(acc_end,);
  % don_after = gene.splicegraph{1}(other_end,alt_3p);
  label_2(3,ii_5p2) = 1;
  if debug
    fprintf(1,'alt 5prime')
    viewsplicegraph(gene,1)
    hold on
    for jj=1:length(true_alt_5p)
      plot([true_alt_5p(jj) true_alt_5p(jj)],[0 10],'b');
    end
    if keyboard_allowed(),
      keyboard
    end 
  end
end
if gene.alt_3prime&isequal(sig,'acc')
  %if ~isempty(gene.alt_3prime_details), keyboard; end ;
  if gene.strand=='+',
    alt_3p = gene.alt_3prime_details.exons(2,:);
  else
    alt_3p = gene.alt_3prime_details.exons(1,:);
  end ;
  alt_3p = unique([alt_3p gene.alt_3prime_details.exons(3,:)]);
  true_alt_3p = gene.splicegraph{1}(which_end,alt_3p);
  [temp,ii1,ii_3p2] = intersect(true_alt_3p,true_splice);
  assert(length(temp)==length(unique(true_alt_3p)))
  true_alt_3p = true_alt_3p(ii1);
  % splice_before_3p = gene.splicegraph{1}(don_end,[exon_alt_3prime(find(idx_alt_3prime==g_idx)).fiveprimesite]);
  % don_before = gene.splicegraph{1}(other_end,[exon_alt_3prime(find(idx_alt_3prime==g_idx)).fiveprimesite]);
  % don_after = gene.splicegraph{1}(other_end,alt_3p);
  label_2(3,ii_3p2) = 1;
  if debug
    fprintf(1,'alt 3prime')
    viewsplicegraph(gene,1)
    hold on
    for jj=1:length(true_alt_3p)
      plot([true_alt_3p(jj) true_alt_3p(jj)],[0 10],'b');
    end
    if keyboard_allowed(),
      keyboard 
    end 
  end
end
[ii,ii1,ii2] = intersect(POS,true_splice);
LABEL_2(:,ii1) = label_2(:,ii2);





return

function ALL = initialize_allconfs(POS,info_names)

ALL.POS = POS;  
  
ALL.GENE_ID = zeros(1,length(POS)); 
ALL.ALTGENIC = zeros(1,length(POS)); 
ALL.COV1 = zeros(length(info_names),length(POS)); 
ALL.COV2 = zeros(length(info_names),length(POS)); 
ALL.LABEL_2 =  -ones(3,length(POS)); 
ALL.ISSPLICE = zeros(1,length(POS)); 

ALL.TRANS = zeros(1,length(POS)); 
ALL.TSS = zeros(1,length(POS)); 
ALL.HAS_SIG = zeros(1,length(POS));
ALL.DIST = zeros(1,length(POS)); 
ALL.CONF_TRANS = zeros(1,length(POS)); 

return

function [Signal,info_names,sig,alt_names] = initialize_cand_search(Signal,info,genes,strand,signal_name,debug)
% [Signal,info,info_names,sig,alt_names] = initialize_cand_search(PAR,genes,strand,signal_name,debug)

rand('seed',1234);

% info = PAR.info_genes.(signal_name);
% Signal = PAR.Signals.(PAR.Signal_name);

if isempty(Signal.consensus) && isfield(Signal, 'consensus_fn') && ~isempty(Signal.consensus_fn)
  load(Signal.consensus_fn,'consensus')
  Signal.consensus = consensus;
end

info_names = fieldnames(info);
use = [];
fprintf(1,'getting conf for ');
for i=1:length(info_names)
  if ismember([info_names{i} '_conf'],Signal.Conf_names)
    use = [use i];
    if isequal(signal_name,'transcript')&~isequal(Signal.name,'transacc')
      assert(ismember([info_names{i} '_conf_skip'],Signal.Conf_names))
    end
    fprintf(1,'%s, ',info_names{i})
  end
end
fprintf('\n')
info_names = info_names(use);

% if ismember('cDNA',info_names)
%   use = strmatch('cDNA',info_names,'exact');
%   info_names{use,2} = 'fulllength';
%   fprintf(1,'\nfulllength cDNAs added to cDNAs\n')
% end

fprintf(1,'\n');

if ~isempty(genes)
  strands = unique([genes.strand]);
  assert(strand==strands(1))
  assert(length(strands)==1);
  assert(length(unique([genes.chr_num]))==1);
end


sig='';
switch Signal.name
 case {'acc','transacc','alt_exon_acc'}
  sig = 'acc';
 case {'don','alt_exon_don'}
  sig = 'don';
 otherwise
  sig = 'other';
end

if isequal(sig,'other') || isequal(Signal.name,'transacc') || ...
      isequal(Signal.name,'acc') || isequal(Signal.name,'don') 
  alt_names ={};
  return
end

alt_names = {'alt_exon', 'alt_intron', 'alt_5prime', 'alt_3prime'};
use = []; 
fprintf(1,'getting label 2 for ');
for i=1:length(alt_names)
  if ismember(alt_names{i},Signal.Conf_names)
    fprintf(1,'%s, ',alt_names{i})
  end
end

return

function init_svm_and_kernels(method,pred_from_seq)
% init_svm_and_kernels(method,pred_from_seq)
  
  
  conservWdsWeighting =0;
  %------------------------------------------------------------------------------%
  % init svm
  %------------------------------------------------------------------------------%
  
  % === cleaning
  sg( 'clean_features', 'TRAIN' );
  sg( 'clean_preproc' );
  sg( 'clean_kernel');
  %clear sg;
  
  %sg( 'loglevel', 'ALL');   % DEBUG
  if ~isfield(method, 'svm_name')
    warning( 'no svm specified, set to default: LIGHT')
    method.svm_name='LIGHT'
  end
  
  % === SVM
  sg( 'new_svm', method.svm_name);
  %sg( 'svm_epsilon', 1e-4 );
  %%%sg( 'svm_epsilon', PAR.svmEps);
  sg('svm_epsilon', 1e-4);
  %%%sg( 'send_command', sprintf( 'use_linadd %d', PAR.useLinadd ) );
  sg('use_linadd', true);
  %%%sg( 'svm_qpsize', PAR.qpSize );
  if ~isfield(method, 'qpsize')
    warning( 'qpsize not specified, set to default: 50')
    method.qpsize=50;
  end
  sg('svm_qpsize', method.qpsize) ; 
  if( conservWdsWeighting > 0 )
    sg('use_batch_computation', false);
  else
    sg('use_batch_computation', true);
  end
  
  % === parallel training?
  if isfield(method,'threads')
    sg('threads', method.threads);
  else
    fprintf('number of threads not specified, set to default: 16')
    sg('threads', 16);
  end;
  
  
  %------------------------------------------------------------------------------%
  % init kernels
  %------------------------------------------------------------------------------%
  
  nofKernels = length(method.kernels);
  if nofKernels>1 && ~isfield(method.kernels{2},'name') && ~isfield(method.kernels{2}, 'kernel_name')
    % for content sensors; several orders are trained simulatuously.
    % but this is done by concatination of the feature vector directly.
    nofKernels = 1;
  end
  
  assert( nofKernels >= 1 );
  if( nofKernels == 1 )
    addKernel = {'set_kernel'};
  else
    addKernel = {'add_kernel', 1};
    sg( 'set_kernel', 'COMBINED', 100);
  end;
  
  for k = 1:nofKernels
    kernel = method.kernels{k};
    % for compatibility with older versions
    if ~isfield(kernel,'kernel_name') && isfield(kernel, 'name')
      kernel.kernel_name = kernel.name;
    end
    fprintf(1, '%s: %s\n', sprintf('%s %1.1f', addKernel{:}), kernel.kernel_name)
    
    switch kernel.kernel_name
     case 'WD'
      if ~isfield(kernel, 'mismatch')
        kernel.mismatch=0;
      end
      assert(length(kernel.order)==1)
      assert(length(kernel.mismatch)==1)
      sg( addKernel{:}, 'WEIGHTEDDEGREE', 'CHAR', 10, kernel.order, kernel.mismatch, true, 1) ;
      
     case 'WDS'
      if ~isfield(kernel, 'mismatch')
        kernel.mismatch=0;
      end
      if ~isfield(kernel, 'shift')
        kernel.shift=0;
      end
      if ~isfield(kernel, 'shift_const')
        kernel.shift_const=0;
      end
      assert(length(kernel.order)==1)
      assert(length(kernel.mismatch)==1)
      len = kernel.rwin-kernel.lwin+1;
      shifts = ceil(kernel.shift * abs( (kernel.lwin:kernel.rwin)+1 ));
      shifts = shifts + kernel.shift_const;
      %shifts = sprintf( '%i ', shifts );
      sg( addKernel{:}, 'WEIGHTEDDEGREEPOS3', 'CHAR', 10, kernel.order, kernel.mismatch, len, 1, int32(shifts)) ;
      clear shifts len
      
     case 'SP'
      use_sign = false;
      % use_sign and prediction from position lists does not work
      %assert(~use_sign|~PAR.tasks.pred_from_seq)
      
      %addKernel_ = {'add_kernel', 0};
      
      sg( addKernel{:}, 'COMMSTRING', 'WORD', 10, use_sign);
      if pred_from_seq
        %sg('loglevel','ALL')
        sg( 'use_diagonal_speedup',true)
      end
     case 'RBF'
      if ~isfield(kernel, 'cache')
        kernel.cache = 40;
      end
      assert(length(kernel.sigma)==1)
      sg( addKernel{:}, 'GAUSSIAN', 'REAL', kernel.cache, kernel.sigma) ;
      
     case 'POLY'
      if ~isfield(kernel, 'cache')
        kernel.cache = 40;
      end
      if ~isfield(kernel, 'type')
        kernel.type = 'REAL';
      end 
      if ~isfield(kernel, 'norm')
        kernel.type = 'R';
      end
      assert(length(kernel.degree)==1)
      sg( addKernel{:}, 'POLY', Kernel.type, kernel.cache, kernel.degree, kernel.norm) ;
      
     case 'LINEAR'
      if ~isfield(kernel, 'cache')
        kernel.cache = 40;
      end
      if ~isfield(kernel, 'type')
        kernel.type = 'REAL';
      end
      sg( addKernel{:}, 'LINEAR', kernel.type, kernel.cache) ;
      
     otherwise
      fprintf(1, 'not yet implemented: kernel_name: %s\n', kernel.kernel_name)
    end;
    
  end;%loop over kernels
  
  if isfield(method, 'subkernel_weights') 
    assert(length(method.subkernel_weights)==nofKernels)
    sg('set_subkernel_weights', method.subkernel_weights);
  end
  
  % for compatibility with older versions
  if ~isfield(method.par_ms,'C') && isfield(method.kernels{1}, 'C')
    method.par_ms.C = method.kernels{1}.C;
  end
  sg('c', method.par_ms.C) ;
  
  disp('done with kernel setup');

return

function  [dummy1, dummy2]=load_and_predict(P, Test_Data)
% load_and_predict(P, Test_Data)

% used for both evaluation on the leftout set of the training region and
% prediction on test regions
% 
% * IF PAR.eval_on is set to 1 prediction is performed on the evaluation
%   set specified by PAR.SETs.which_val. The model saved in PAR.fn_SVMs is used for
%   prediction. Additionally RSV and RFV are determined.
%  
% * IF PAR.eval_on is set to 0  only prediction is performed on the test set
%   specified by PAR.SETs.which_pred.
% 
% 

  dummy1 = [] ;
  dummy2 = [] ;

fn_SVMs = P.fn_SVMs ;
SETs = P.SETs;
pred_from_seq = P.pred_from_seq;
Signal = P.Signal;
fn_examples = P.fn_examples;
fn_filter_settings = P.fn_filter_settings;

if isfield(P,'avg_all_svms') 
  avg_all_svms = P.avg_all_svms;
else
  avg_all_svms = 0;
end

if isfield(P,'check_overlap')
  check_overlap = P.check_overlap;
else
  check_overlap = 1;
end

if isfield(P,'eval_on') && P.eval_on
  which_set = SETs.which_val;
  if isfield(P,'fn_eval')
    fn_pred = P.fn_eval;
  else
    fn_pred = [];
  end
elseif isfield(P,'plif_on') && P.plif_on
  which_set = SETs.which_plif;
  if isfield(P,'fn_plifs')
    fn_pred = P.fn_plifs;
  else
    fn_pred = [];
  end
else
  which_set = SETs.which_pred;
  if isfield(P,'fn_pred')
    fn_pred = P.fn_pred;
  else
    fn_pred = [];
  end
end

if pred_from_seq
  fn_pos = P.fn_pos;
  if isequal(fn_pos(1),'~')
    fn_pos = sprintf('%s/%s', getenv('HOME'), fn_pos(2:end));
  end
  region = P.region;
  fn_pred = P.fn_pred;
  chunk_idx = 1;% P.chunk_idx;
  RPROC = struct;
end

load(fn_SVMs, 'Train') ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% load data 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isfield(SETs,'test_subsample_pos')
  subsample_pos = 1;
else
  subsample_pos = SETs.test_subsample_pos;
end
if ~isfield(SETs,'test_subsample_neg')
  subsample_neg = 1;
else
  subsample_neg= SETs.test_subsample_neg;
end


if ~pred_from_seq
  pred_from_seq=0;
  if ~exist('Test_Data')
    Test_Data = feval(Signal.load_fct,Signal,fn_filter_settings,fn_examples, which_set, 'test',subsample_neg, subsample_pos);
  end
  
  %  if isequal(PAR.method.name,'SVM')
    % [RFV RSV]=tssTest(Train, Test_Data, fn_SVMs, fn_pred);
    % varargout{1} = RFV;
    % varargout{2} = RSV;
 %  elseif isequal(PAR.method.name,'PWM')
    % hmm_test(Train, Test_Data, fn_SVMs,fn_pred);
 %  end
else %genomweite Vorhersage
  
  region = load_sequence(region);
  region = retrieve_signals(region,fn_pos,Signal.name,{'svm'});
  Test_Data.seq = region.seq;
  Test_Data.pos = region.Signals.(Signal.name).pos';
  Test_Data.svm = region.Signals.(Signal.name).svm';
  idx = find((Test_Data.pos<Signal.lwin_big+1)|(Test_Data.pos>length(Test_Data.seq)-Signal.rwin_big-1));
  if length(idx)==length(Test_Data.pos), 
    % error('no examples in valid region found'); 
  end
  Test_Data.pos(idx)=[];
  Test_Data.svm(idx)=[];
  % load(PAR.fn_pos,'pos','seq','region_id','svm_num') ;
  if ~all(ismember(unique(Test_Data.seq),['ACGT']))
    warning('sequence alphabet different from ACGT; setting all Ns to A')
    Test_Data.seq(~ismember(Test_Data.seq,['ACGT']))='A';
  end
  if ~avg_all_svms
    idx = find(Test_Data.svm==which_set);
  else
    idx = 1:length(Test_Data.svm);
  end
  if isequal(Signal.method.predict_fct,'svm_content_predict')
    idx = [idx length(Test_Data.svm)];
  end
  if length(idx)==0, warning('no examples for specified svm found'); end
  Test_Data.pos = Test_Data.pos(idx);
  Test_Data.svm = Test_Data.svm(idx);
  if isfield(RPROC, 'divide4pred') && RPROC.divide4pred
    junk_length = ceil(length(Test_Data.pos)/RPROC.numjunks) ;
    idx_sub = (chunk_idx-1)*junk_length+1:min(chunk_idx*junk_length,length(Test_Data.pos));
    Test_Data.pos = Test_Data.pos(idx_sub);
    Test_Data.svm = Test_Data.svm(idx_sub);
    fn_pred = sprintf('%s%i.mat',fn_pred, chunk_idx);
  end
  %   Test_Data.region_id = Test_Data.region_id(idx);
end

Test_Data.PAR = P;

feval(Signal.method.predict_fct, Train, Test_Data,fn_SVMs,fn_pred,pred_from_seq, check_overlap);

return

function [Data,par] = load_data(Signal,fn_filter_settings,fn_examples, which, filter_mode,subsample_neg,subsample_pos)
% [Data,par] = load_data(Signal,fn_filter_settings,fn_examples, which, filter_mode,subsample_neg,subsample_pos)
% [Data,par] = load_data(PAR, which, filter_mode,subsample_neg,subsample_pos)
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% load data of training set
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% method = PAR.method;
% Signal = PAR.Signals.(PAR.Signal_name);
%  FN = PAR.FN;
% input_sig = FN.input_sig.(Signal_name);
% fn_filter_settings = input_sig.fn_filter_settings;
% fn_examples = input_sig.fn_examples;

method = Signal.method;

XT = []; LT = [] ; POS.region_id = []; POS.pos=[]; POS_tmp = []; 
SC = []; use_sc = [];
if nargin<4
  subsample_neg=1;
end
if nargin<5
  subsample_pos=1;
end

use_cons = Signal.method.use_cons;
% if nargin<6
%   use_cons = 0;
% end
for w = which
  % create filename of input file
  filename = gen_example_filename(Signal,fn_filter_settings,fn_examples,filter_mode,w);
  
  fprintf('loading XT, LT and POS from file %s \n',filename)
  L = load(filename,'LT');
  L.XT = load_matrix(filename, 'XT');
  P = load(filename, 'POS');
  %if isfield(signal,'center')
  %  par.center = signal.center;
  %else
  %  par.center = size(L.XT,1)./2;
  %end
  
  have_pos = isfield(P, 'POS') ;
  have_lt = isfield(L, 'LT') ;
  if ~have_lt, L.LT=[] ; end ;
  have_sc = isfield(L, 'SC_') ;
  if ~have_sc, L.SC_=[] ; end ;  

  [L.XT, L.LT, P.POS, L.SC_] = subsample(L.XT, L.LT, P.POS, L.SC_, subsample_neg, subsample_pos) ;

  if ~have_lt, L=rmfield(L, 'LT') ; end ;
  if ~have_sc, L=rmfield(L, 'SC_') ; end ;
  
  if ~isfield(method,'feature_type') || isempty(method.feature_type) ||isequal(method.feature_type, 'char')
    L.XT_char = char(L.XT) ;
    for i=1:size(L.XT,1)
      L.XT_char(i,:) = char(double(L.XT(i,:))) ;
    end ;
    XT = [XT L.XT_char];
  elseif isequal(method.feature_type, 'uint8')
    L.XT_char = uint8(L.XT) ;
    for i=1:size(L.XT,1)
      L.XT_char(i,:) = uint8(double(L.XT(i,:))) ;
    end ;
    XT = [XT L.XT_char];
  elseif isequal(method.feature_type, 'double')
    XT = [XT L.XT];
  else
    error('unknown feature type; should be "char" or "double"')
  end
  if isfield(L,'LT')
    if size(L.LT,1)>size(L.LT,2)
      L.LT = L.LT';
    end
    LT = [LT L.LT] ;
  end
  
  if (isfield(L,'SC_')&& isfield(L.use_sc)) || use_cons
    fprintf('using consensus information\n');
    SC = [SC L.SC_] ;
    use_sc = [use_sc L.use_sc];
  end

  POS.region_id =[POS.region_id P.POS.region_id];
  POS.pos = [POS.pos P.POS.pos];
end ;
%%% just for compatibility with older versions 
  
% check whether everything is ok (matlab bug successfully avoided?)
for i=1:size(XT,1)
  if all(XT(i,:)==0)
    % i
    % keyboard
  end
  if  ~isfield(method,'feature_type') || isequal(method.feature_type, 'char')
    assert(all(XT(i,:)~=0)) ;
  end
end ;

clear L
if ~isempty(LT)
  fprintf('%i positive, %i negative examples (%.2f %% positives), sequences of length %i \n',sum(LT==1),sum(LT==-1),sum(LT==1)/length(LT)*100, size(XT,1));
end

if ~isempty(SC)
  XT(:,~use_sc) = [] ;
  POS(~use_sc) =[];
  LT(~use_sc) = [];
  SC(:,~use_sc) = [];
  if ~isempty(LT)
    fprintf('%i positive, %i negative examples (%.2f %% positives) \n',sum(LT==1),sum(LT==-1),sum(LT==1)/sum(LT==1|LT==-1)*100);
  end
end

Data.LT = LT;
% Data.XT = char(XT) ;
Data.XT = XT;
Data.Pos = POS;
clear XT LT POS
%this assert is very mem consuming
%assert(~any(XT(:)~='A' & XT(:)~='G' & XT(:)~='C' & XT(:)~='T')) ;
% XT = to_upper_case(char(XT)) ;
if ~isempty(SC)
  Data.SC = char(SC) ;
  assert(~any(SC(:)~='A' & SC(:)~='G' & SC(:)~='C' & SC(:)~='T')) ;
  
  % SC = to_upper_case(char(SC)) ;
end


assert(size(Data.XT,2)==length(Data.LT))
assert(length(Data.LT)==length(Data.Pos.pos))
assert(length(Data.LT)==length(Data.Pos.region_id))

return

function [num_data, num_pos, num_neg] = count_data(Signal, fn_filter_settings, fn_examples, which, filter_mode, subsample_neg, subsample_pos)
% [num_data, num_pos, num_neg] = count_data(Signal, fn_filter_settings, fn_examples, which, filter_mode,subsample_neg,subsample_pos)
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% count data of training set
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

method = Signal.method;

LT = [] ; POS.pos=[]; 
if nargin<4
  subsample_neg=1;
end
if nargin<5
  subsample_pos=1;
end

for w = which
  % create filename of input file
  filename = gen_example_filename(Signal,fn_filter_settings,fn_examples,filter_mode,w);
  
  fprintf('loading LT and POS from file %s \n',filename)
  L = load(filename, 'LT');
  P = load(filename, 'POS');
  
  have_pos = isfield(P, 'POS') ;
  have_lt = isfield(L, 'LT') ;
  if ~have_lt, L.LT=[] ; end ;

  [tmp, L.LT, P.POS] = subsample(L.LT, L.LT, P.POS, [], subsample_neg, subsample_pos) ;

  if ~have_lt, L=rmfield(L, 'LT') ; end ;
  
  if isfield(L,'LT')
    if size(L.LT,1)>size(L.LT,2)
      L.LT = L.LT';
    end
    LT = [LT L.LT] ;
  end
  
  POS.pos = [POS.pos P.POS.pos];
end ;
%%% just for compatibility with older versions 
  
clear L
if ~isempty(LT)
  fprintf('%i positive, %i negative examples (%.2f %% positives)\n',sum(LT==1),sum(LT==-1),sum(LT==1)/length(LT)*100);
  num_data = length(LT) ;
  num_pos = sum(LT==1) ;
  num_neg = sum(LT==-1) ;
else
  num_data = length(POS.pos) ;
  num_pos = nan ;
  num_neg = nan ;
end

return

function regions=load_sequence(regions);

% regions=load_sequence(regions);
  
genome_info = init_genome(regions(1).config);


for i=1:length(regions)
  if isfield(regions(i),'num_fragments')&&~isempty(regions(i).num_fragments)
    num_fragments = regions(i).num_fragments;
  else
    num_fragments = 1;
  end
  for p=1:num_fragments
    if mod(i,100)==0,
      fprintf(1,'region %i of %i\r',i,length(regions))
    end ;
    chr = regions(i).chr_num(p) ;
    str = regions(i).strand(p) ;
    start = regions(i).start(p) ;
    stop = regions(i).stop(p) ;
    seq = upper(load_genomic(genome_info.contig_names{chr},str,start,stop,genome_info));
    assert(ischar(seq)) ;
    idx = (~ismember(seq,'ACGT')) ;
    seq(idx)='N';
    if p==1
      regions(i).seq = seq;
    else
      regions(i).seq = [regions(i).seq ,seq];
    end
  end
end

return

function [PAR, jobinfo ,all_done]=masterscript_sensors(PAR,check_files,keep_all_files,verb)
% [PAR, jobinfo ,all_done] = masterscript_sensors(PAR,check_files,verb)

%cd ~/svn/projects/genefinding/sensors
%paths

tasks = PAR.tasks.signals;  
%run_locally = tasks.run_locally;

FN = PAR.FN;  
fn_genome_config = PAR.FN.input.fn_genome_config;
 
if nargin<2 
  check_files =0;
end
if nargin<3 
  keep_all_files = 0;
end
if nargin<4 
  verb = 0;
end

fid =1;
fprintf(fid,'\n\n-----------\n');
fprintf(fid,'SENSOR :  %s\n',PAR.Signal_name);
fprintf(fid,'-----------\n');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% --- Train SENSORS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PAR.method.submit_batch=1;
all_done=0;
%train_ok = 0;

jobinfo=struct ;

if tasks.prepare_labels
  %fprintf('start training for sensor %s on organism %s with method %s: %s\n\n',sensor,organism,method)
  gen_training_examples4signal(PAR) ;
end
if tasks.modelsel | tasks.train
  jobinfo = ms_gfwd_starter(PAR);
  if ~isempty(jobinfo), return ; end ;
end
if tasks.get_train_error
  jobinfo = predict_testplif_geterror(PAR)
  if ~isempty(jobinfo), return ; end ;
end
if tasks.learn_plifs
  calc_plifs(PAR);
end

%if ~train_ok
%  fprintf('something wrong with training files, check first!!  \n')
%end


if ~keep_all_files
  unix(sprintf('rm -rf %s', FN.input_sig.(PAR.Signal_name).fn_candsites));
  unix(sprintf('rm -rf %s*', FN.input_sig.(PAR.Signal_name).fn_examples));
  unix(sprintf('rm -rf %s*', FN.input_sig.(PAR.Signal_name).fn_example_statistics(1:end-4)));  
  unix(sprintf('rm -rf %s', FN.input_sig.(PAR.Signal_name).fn_pos));
  unix(sprintf('rm -rf %s', FN.input_sig.(PAR.Signal_name).fn_filter_settings));

  unix(sprintf('rm -rf %s', FN.output_sig.(PAR.Signal_name).fn_pred));
  unix(sprintf('rm -rf %s/ms_set*', FN.output_sig.(PAR.Signal_name).fn_SVMs));
  unix(sprintf('rm -rf %s/partition=*', FN.output_sig.(PAR.Signal_name).fn_SVMs));  
end

if ~tasks.pred_on_genome
  all_done = 1 ;%train_ok ;
  return
end

return

function [jobinfo,fn_pred]=masterscript_sensors_test(P, check_files, run_locally, do_gzip, save_as, keep_all_files)
% [jobinfo ,all_done]=masterscript_sensors_test(P, check_files, run_locally, do_gzip, save_as, keep_all_files)

num_splits = P.SETs.num_splits ; 
Signal_name = P.Signal_name;

FN = P.FN;  
fn_genome_config = FN.input.fn_genome_config;
fn_training_blocks = FN.output.fn_training_blocks;  
fn_test_blocks = FN.output.fn_test_blocks;

fn_pos =  FN.input_sig.(Signal_name).fn_pos;
fn_pred = FN.output_sig.(Signal_name).fn_pred;
fn_svms = FN.output_sig.(Signal_name).fn_SVMs ;
% fn_svms = sprintf('%sbest_partition=', FN.output_sig.(Signal.name).fn_SVMs) ;

L = load([fn_svms '1.mat'], 'Train');
Signal =  L.Train.PAR.Signal;
clear L 

if nargin<2 
  check_files = 1;
end

if nargin<3 
  run_locally.pos = 1 ;
  run_locally.predict = 1 ;
  run_locally.conf = 1 ;
  run_locally.concat = 1 ;
  run_locally.save = 1 ;
end

if nargin<4 
  do_gzip = 0;
end

if nargin<5 
  save_as.spf_ascii = 1 ;
  save_as.spf_binary = 1 ;
  save_as.wiggle = 0 ;
end

if nargin<6 
  keep_all_files = 0 ;
end

%if ~run_locally  
RPROC = P.RPROC;
%else
%  RPROC = struct;
%end

fid =1;
fprintf(fid,'\n\n-----------\n');
fprintf(fid,'SENSOR :  %s\n',Signal.name);
fprintf(fid,'-----------\n');

all_done = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% --- Generate Positions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

jobinfo=struct ;

%[temp, num_files] = unix(sprintf('ls %scontig_*.pos | wc -l',fn_pos));
d=dir(sprintf('%scontig_*.pos',fn_pos));
num_files = length(d) ;
%[temp, num_files1] = unix(sprintf('ls %scontig_*.svm | wc -l',fn_pos));
d=dir(sprintf('%scontig_*.svm',fn_pos));
num_files1=length(d) ;

genome_info = init_genome(fn_genome_config);
if isempty(num_files)||(num_files<length(genome_info.contig_names)*2)||...
      isempty(num_files1)||(num_files1<length(genome_info.contig_names)*2)
  jobinfo = gen_all_positions(fn_genome_config,fn_pos,Signal,fn_training_blocks,fn_test_blocks,num_splits,run_locally.pos,RPROC);
  if ~isempty(jobinfo),
    fprintf('generating genome position files\n')
    jobinfo = rproc_resubmit(jobinfo);
    jobinfo = rproc_wait(jobinfo, 5, 1, -1); 
  end ; 
else
  fprintf('genome position files already exist\n')
end

avg_all_svms = 0;
subtract_diff = 0;
jobinfo = genomewide_predictions(fn_genome_config,fn_pos,fn_pred,fn_svms,num_splits,avg_all_svms,run_locally.predict, RPROC);
fprintf('all %s predictions done\n',Signal.name)

jobinfo = convert_predictions2confs(fn_genome_config,fn_pred,fn_svms,num_splits, run_locally.conf, RPROC);
fprintf('conversion done\n')


jobinfo = concat_genomewide_predictons(fn_genome_config,fn_pos,fn_pred, num_splits,avg_all_svms,subtract_diff,run_locally.concat, RPROC);
fprintf('concatination done\n')


resolution = Signal.export_settings.resolution ;


conf_cum_thresh = Signal.export_settings.conf_cum_thresh;

inventory = {'output', 'pos', 'Conf', 'Conf_cum'} ;
[jobinfo] = save_predictions(fn_genome_config, fn_pred, ...
                             Signal_name, resolution, ...
                             conf_cum_thresh, '+-', ...
                             run_locally.save, RPROC, ...
                             do_gzip, save_as, inventory);
fprintf('saving done\n')


if check_files
  % fprintf('checking files \n')
  % check_genomewide_output(P) % this fct compares prediction results with test results from training (cross_validation)
  % all_good = check_predictions(fn_genome_config,fn_pred,Signal_name,resolution)
else
  fprintf('WARNING: files not checked\n')
end


fprintf('%s COMPLETE !\n\n',Signal.name)


if ~keep_all_files
 %  unix(sprintf('rm -rf %s*_split=*', FN.output_sig.(Signal_name).fn_pred));
end

return

function jobinfo = ms_gfwd_starter(PAR)

% ms_gfwd_starter(PAR)
%
% used for modelselection as well as simple training and evaluation 
% INPUT : struct PAR as generated by create_PAR
%         fields used: 
%         - Signal_name: string specifying the signal that is actually computed
%           (e.g. 'acc','don') 
% 
%         - Signals.(Signal_name) : struct containing all necessary information to
%           compute the signal,
% 
%         - SETs : struct with information about split and partitions, 
%   
%         - FN : struct containing all used filenames including: 
%           FN.input_sig.(Signal_name).fn_filter_settings; 
%           FN.input_sig.(Signal_name).fn_examples; 
%           FN.output_sig.(Signal_name).fn_models;
%           FN.output_sig.(Signal_name).fn_SVMs;
%           FN.output_sig.(Signal_name).fn_motifs;
%   
%         - tasks.signals struct specifying what will be done
%           fields include run_locally (default=1), run_all,
%  
%         - RPROC (only necessary when tasks.signals.run_locally=0)
  
  
fid = 1;%fopen(PAR_1.FN.fn_log ,'a+');
FN = PAR.FN;
if ~isempty(PAR.Signal_name)
  Signal_name = PAR.Signal_name;
  Signal = PAR.Signals.(Signal_name);
  input_fn = FN.input_sig.(Signal_name);
  output_fn = FN.output_sig.(Signal_name);
  tasks = PAR.tasks.signals;
else
  Signal_name = PAR.Content_name;
  Signal = PAR.Contents.(Signal_name);
  input_fn = FN.input_cont.(Signal_name);
  output_fn = FN.output_cont.(Signal_name);
  tasks = PAR.tasks.contents;
end
fn_filter_settings = input_fn.fn_filter_settings;
fn_examples = input_fn.fn_examples;


method = Signal.method;
SETs = PAR.SETs;

if ~isfield(tasks,'run_locally')
  tasks.run_locally = 1;
end
if ~isfield(tasks,'run_all')
  tasks.run_all = 0;
end 

if ~tasks.run_locally.train
  RPROC = PAR.RPROC;
  organism = PAR.organism.name;
  RPROC.options.identifier = sprintf('%s%s',Signal_name,upper(organism)) ;
  RPROC.options.ncpus = 1;
  %if isfield(RPROC.options, 'addpaths') && iscell(RPROC.options.addpaths)
  %  RPROC.options.addpaths = {RPROC.options.addpaths{:}, fileparts(which(sg))};
  %else
  %  RPROC.options.addpaths = {fileparts(which(sg))};
  %end
  %path
end  

fprintf(fid,'\nstarting ms_gfwd_starter \n');
fprintf(fid,'date: %s\n',date);
fprintf(fid,'\nSITE: %s\n',Signal_name);


%------------------------------------------------------------------------%
% write numeric kernelpars into field for model selection
%------------------------------------------------------------------------%

if isfield(method, 'kernels') && ~isempty(method.kernels{1})
  method.par_ms = collect_ms_parameters(method.kernels, method.par_ms);
end

%------------------------------------------------------------------------%
% initiate parameters
%------------------------------------------------------------------------%
Model_param = getfield(method,'par_ms');
Model_param = orderfields(Model_param) ;

[MODELS,param_names] = generate_all_models(orderfields(method.par_ms));
if ~tasks.modelsel
  fprintf('no model selection. taking first model;\n')
  MODELS = MODELS(:,1);
end

fprintf(fid,'number of models to be tested %s: %i\n',Signal_name,size(MODELS,2));
num_models_tested = size(MODELS,2);
RSV_eval = zeros(SETs.num_partitions,size(MODELS,2));
RFV_eval = zeros(SETs.num_partitions,size(MODELS,2));

%------------------------------------------------------------------------%
% check for saved model files and save models if non are found
%------------------------------------------------------------------------%
if ~fexist(output_fn.fn_models) && ~fexist([output_fn.fn_models '.mat']) 
  save(output_fn.fn_models,'MODELS','Model_param')
elseif (fexist(output_fn.fn_models) || fexist([output_fn.fn_models '.mat']) ) 
  LL = load(output_fn.fn_models);
  assert(isequal(LL.Model_param,Model_param))
  if ~isequal(LL.MODELS,MODELS)
    MODELS=append_new_models(MODELS, LL.MODELS);
    old=LL;
    save(output_fn.fn_models,'MODELS','old')
    warning('different other MODELS have already been trained, new models are appended')
  end
  clear LL old
end
num_not_eval = SETs.num_partitions*sum(num_models_tested) ;

fprintf(fid,'total number of jobs for training: %i\n',sum(SETs.partition_train_on)*sum(num_models_tested));
fprintf(fid,'total number of jobs for evaluation: %i\n',num_not_eval);


P.SETs = SETs;
P.Signal = Signal;
P.fn_filter_settings = fn_filter_settings;
P.fn_examples = fn_examples;
P.train_error_on = tasks.get_train_error;
P.pred_from_seq = 0;

P.use_preproc = 1;

num_crashed = 1;
num_crashed_1 = 0;
rep = 1;

P.check_overlap = 1;
while num_crashed + num_crashed_1 + num_not_eval>0 & rep<5
  q = 0 ; nq = 0 ;
  jobinfo = rproc_empty(0) ;
  jobinfo_1 = rproc_empty(0) ;
  num_not_eval = SETs.num_partitions*sum(num_models_tested) ;
  for p = 1:size(SETs.partitions,1) %% loop over partitions  
    fprintf(fid,'\n\n-----------\n');
    fprintf(fid,'PARTITION %i\n',p);
    fprintf(fid,'-----------\n');
    
    if tasks.run_all
      Train_data=[];
      Test_data=[];
    end
    
    P.partition = p;
    P.train_on = SETs.partition_train_on(p);
    P.SETs.which_train = find(ismember(SETs.partitions(p,:),SETs.info.train));
    P.SETs.which_val = find(ismember(SETs.partitions(p,:),SETs.info.eval));
    P.SETs.which_pred = find(ismember(SETs.partitions(p,:),SETs.info.pred));
    
    fn_save = [sprintf('%sms_set=[',output_fn.fn_SVMs),...
               sprintf('%i',P.SETs.which_train)];
    fn_save_pred  = [fn_save '_' sprintf('%i',P.SETs.which_val)];
    
    for num_models = 1:size(MODELS,2)
      %--------------------------------------------------------------%
      % assign parameters of model number num_models to the current parameter
      % structure
      %--------------------------------------------------------------%
      
      P.Signal.method = assign_model(method, MODELS, param_names,num_models); 
      
      %--------------------------------------------------------------%
      %% generate filename for SVMs and predictions
      %--------------------------------------------------------------%
      
      P.fn_SVMs = sprintf('%s]_model=%i.mat',fn_save,num_models) ;
      P.fn_eval = sprintf('%s]_model=%i.mat',fn_save_pred,num_models) ;
      
      %--------------------------------------------------------------%
      %%%%% TRAIN
      %--------------------------------------------------------------%
      
      %if ~fexist(P.fn_SVMs) & P.train_on
      %  fprintf(fid,'waiting for file %s...\n', P.fn_SVMs);
      %pause(1)
      %end      

      if ~fexist(P.fn_SVMs) & P.train_on
        q = q+1 ;
        fprintf(fid,'\n start training on set ');
        fprintf(fid,'%i',P.SETs.which_train);
        fprintf(fid,'\n')
        
        train_fct = 'load_and_train';
        
        if tasks.run_all
          Data = load_data(Signal, fn_filter_settings, fn_examples, P.SETs.which_train, 'train', SETs.train_subsample_neg, SETs.train_subsample_pos);
          Train_data.(sprintf('partition_%i',p))= Data ; clear Data 
          feval(train_fct,P,Train_data.(sprintf('partition_%i',p)));
        elseif tasks.run_locally.train==0,
          fprintf('creating job (training) %i/%i  \r', q,sum(SETs.partition_train_on)*sum(num_models_tested)) ;
          RPROC.options.identifier = sprintf('Tr%i_%s%s',num_models, Signal_name, upper(organism)) ;

          num_data = count_data(Signal, fn_filter_settings, fn_examples, P.SETs.which_train, 'train', SETs.train_subsample_neg, SETs.train_subsample_pos);

          [mem_req, time_req, opts] = rproc_memtime_policy(train_fct,  num_data, RPROC.options) ;

          jobinfo(q) = rproc_create(train_fct, P, mem_req, opts, time_req) ;
        else
          feval(train_fct,P);
        end 
      	
      %--------------------------------------------------------------%
      %%%%% EVAL 
      %--------------------------------------------------------------%
      elseif fexist(P.fn_SVMs) 
        fprintf(fid,'\nSVM already trained on set ');
        fprintf(fid,'%i',P.SETs.which_train);
	fprintf(fid,'\n')
       
	if ~fexist(P.fn_eval)
          fprintf(fid,'waiting for file %s...\n', P.fn_eval);
	  pause(1)
        end
 
        if ~fexist(P.fn_eval) 
          P.eval_on = 1;
          fprintf(fid,'\n\nstart evaluation on set ');
          fprintf(fid,'%i',P.SETs.which_val);
          fprintf(fid,'\n')
          nq = nq+1;
          
          if isfield(method,'pred_fct')&& strcmp(method.pred_fct,'load_content_and_predict')
            pred_fct = 'load_content_and_predict';
          else
            pred_fct = 'load_and_predict';
          end
  
          if tasks.run_all
            Data = load_data(Signal,fn_filter_settings,fn_examples, P.SETs.which_val, 'test',SETs.test_subsample_neg,SETs.test_subsample_pos);
            Test_data.(sprintf('partition_%i',p)) = Data; clear Data;
            feval(pred_fct,P,Test_data.(sprintf('partition_%i',p)));
          elseif tasks.run_locally.eval==0,
            fprintf('creating job (evaluation) %i/%i  \r', nq,num_not_eval) ;
            RPROC.options.identifier = sprintf('Ev%i_%s%s',num_models,Signal_name,upper(organism)) ;

            num_data = count_data(Signal,fn_filter_settings,fn_examples, P.SETs.which_val, 'test', SETs.test_subsample_neg, SETs.test_subsample_pos);

            [mem_req, time_req, opts] = rproc_memtime_policy(pred_fct,  num_data, RPROC.options) ;

            jobinfo_1(nq) = rproc_create(pred_fct, P, mem_req, opts, time_req) ;
          else
            feval(pred_fct,P);
          end
        
        %--------------------------------------------------------------%
        %%%%% Collext Results 
        %--------------------------------------------------------------%
        elseif fexist(P.fn_eval) 
          fprintf(fid,'\n model already evaluated on set ');
          fprintf(fid,'%i',P.SETs.which_val);
          
          L = load(P.fn_eval, 'RSV', 'RFV');%,'fn_SVMs') ;
          num_not_eval = num_not_eval-1;
              
          assert(isnumeric(L.RFV) && prod(size(L.RFV))==1) ;
          assert(isnumeric(L.RSV) && prod(size(L.RSV))==1) ;
          RFV_eval(p,num_models)= L.RFV ;
          RSV_eval(p,num_models)= L.RSV ;

        else
          fprintf(fid,'\n only training, no evaluation ');
          num_not_eval = num_not_eval-1;
        end
      end   %% if trained eval
    end   %% loop over num_models      
    
    fprintf(fid,'partition %i: DONE\n', P.partition);
  end  %% loop over partitions
  fprintf(fid,'experiment %s: DONE\n',Signal_name );
      
  
  %--------------------------------------------------------------%
  %%%%% RPROC STUFF
  %--------------------------------------------------------------%

  %if (~isempty(jobinfo) || ~isempty(jobinfo_1)) && isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs,
  %  jobinfo = [jobinfo jobinfo_1] ;
  %  return ;
  %end ;

  if ~isempty(jobinfo),
    for ii=1:length(jobinfo), jobinfo(ii) = rproc_resubmit(jobinfo(ii)); end ;
  else
    num_crashed = 0 ;
  end
  if ~isempty(jobinfo_1),
    %[jobinfo_1, meta_jobinfo_1] = rproc_submit_batch(jobinfo_1, RPROC.exm_per_batch) ;
    for ii=1:length(jobinfo_1), jobinfo_1(ii) = rproc_resubmit(jobinfo_1(ii)); end ;
  else
    num_crashed_1 = 0;
  end ;
  if ~isempty(jobinfo),
    [jobinfo,num_crashed] = rproc_wait(jobinfo, 60, 1, -1);
    if num_crashed==length(jobinfo)
      fprintf('all jobs seem to be crashed. Waiting 2min.\n') ;
      pause(120) ;
      [jobinfo,num_crashed] = rproc_wait(jobinfo, 60, 1, -1);
      if num_crashed==length(jobinfo),
        error('all jobs crashed')
      end ;
    end
    if num_crashed == 0,
      rproc_cleanup(jobinfo) ;
    end ;
  end
  if ~isempty(jobinfo_1),
    [jobinfo_1,num_crashed_1] = rproc_wait(jobinfo_1, 60, 1, -1);
    if num_crashed_1==length(jobinfo_1)
      fprintf('all jobs seem to be crashed. Waiting 2min.\n') ;
      pause(120) ;
      [jobinfo,num_crashed] = rproc_wait(jobinfo, 60, 1,-1);
      if num_crashed==length(jobinfo),
        error('all jobs crashed') ;
      end ;
    end
    if num_crashed_1 == 0,
      rproc_cleanup(jobinfo_1) ;
    end ;
  end ;
  
  %--------------------------------------------------------------%
  %%%%% FINAL STUFF
  %--------------------------------------------------------------%
  
  try
    L=load(output_fn.fn_models, 'RSV_eval', 'RFV_eval') ;
  catch
    L=struct ;
  end ;
  if ~isfield(L, 'RSV_eval') || ~isfield(L, 'RFV_eval') || ~isequal(L.RSV_eval, RSV_eval) || ~isequal(L.RFV_eval, RFV_eval),
    %save(output_fn.fn_models,'-append','RSV_eval','RFV_eval')
    save_append(output_fn.fn_models, 1,'RSV_eval',RSV_eval, 'RFV_eval',RFV_eval)
    % save_append(output_fn.fn_models, 1,'RSV_eval',RSV_eval, 'RFV_eval',RFV_eval)
  end 
  if num_crashed+num_crashed_1>0
    rep = rep+1;
  end
end  %% loop over crashed jobs

%fclose(fid)




return

function [dummy1, dummy2]=Out2Confhelper(PAR)
% Out2Confhelper(PAR)
try
  pause('on')
  warning('off', 'MATLAB:typeaheadBufferOverflow');  
catch
  % works only for matlab
end

dummy1=[] ;
dummy2=[] ;

try
  load(PAR.fn_pred,'output') ;
catch
  pause(60)
  load(PAR.fn_pred,'output') ;
end
disp('Convert SVM Output to Confidence on fn_pred')
[Conf, Conf_cum] = Out2Conf(output, PAR.prob, PAR.prob_cum, PAR.limits);

if ~(all(Conf>=0-1e-8&Conf<=1+1e-8))
  save('~/tmp/ws_Out2Confhelper_error');
  error('Conf out of range [0, 1]');
end
if ~(all(Conf_cum>=0-1e-8&Conf_cum<=1+1e-8))
  save('~/tmp/ws_Out2Confhelper_error');
  error('Conf_cum out of range [0, 1]');
end

fprintf('append Conf and Conf_cum to file %s ', PAR.fn_pred)
%save(PAR.fn_pred,'-append','Conf','Conf_cum');
inventory = {'output', 'pos', 'Conf', 'Conf_cum'};
save_append(PAR.fn_pred, 1, 'inventory', inventory,'Conf', Conf, 'Conf_cum', Conf_cum);

return

function ret = predict_testplif_geterror(PAR)

  
SETs = PAR.SETs;
RPROC = PAR.RPROC;
FN = PAR.FN;

if ~isempty(PAR.Signal_name)
  Signal_name = PAR.Signal_name;
  Signal = PAR.Signals.(Signal_name);
  output_fn = FN.output_sig.(Signal_name);
  tasks = PAR.tasks.signals;
else
  Signal_name = PAR.Content_name;
  Signal = PAR.Contents.(Signal_name);
  output_fn = FN.output_cont.(Signal_name);
  tasks = PAR.tasks.contents;
end


fid = 1;%fopen(PAR.FN.fn_log ,'a+');
fprintf(fid,'\n\n starting test on left out set \n');
fprintf(fid,'date: %s\n',date);


%PAR = setfield(PAR,PAR.method,Model_param);


load(output_fn.fn_models,'RSV_eval','RFV_eval','MODELS','Model_param')
param_names = fieldnames(Model_param); 

fprintf(fid,'\n\n-----------\n');

RSV_test = zeros(SETs.num_partitions,1);
RFV_test = zeros(SETs.num_partitions,1);
best_models = zeros(SETs.num_partitions,size(MODELS,1));

options = RPROC.options;
options.identifier = [ Signal_name '_' Signal.method.name];

jobinfo = rproc_empty(0) ;
jobinfo_1 = rproc_empty(0) ;  


for p = 1:size(SETs.partitions,1) %% loop over partitions  
  fprintf(fid,'\n\n-----------\n');
  fprintf(fid,'PARTITION %i\n',p);
  fprintf(fid,'-----------\n');
  fprintf(fid,'\nSITE: %s\n',Signal_name);
  
  which_train = find(ismember(SETs.partitions(p,:),SETs.info.train));
  which_val = find(ismember(SETs.partitions(p,:),SETs.info.eval));
  which_pred = find(ismember(SETs.partitions(p,:),SETs.info.pred));
  which_plif = find(ismember(SETs.partitions(p,:),SETs.info.plif));

  fn_best_SVM = sprintf('%s_best_partition=%i.mat', output_fn.fn_SVMs,p) ;
  fn_SVM_train = [sprintf('%sms_set=[',output_fn.fn_SVMs),...
                  sprintf('%i',which_train)];
  fn_SVM_eval  = [fn_SVM_train '_' sprintf('%i',which_val)];
  fn_pred = sprintf('%spartition=%i.mat',output_fn.fn_SVMs,p);
  fn_plifs = sprintf('%spartition=%i_plifs.mat',output_fn.fn_SVMs,p);
  
  %--------------------------------------------------------------%
  %%% select for each partition best model:
  %--------------------------------------------------------------%
  if ~all(RFV_eval(p,:)~=0) 
    warning('some models not evaluated')
  end
  [tmp,idx] = max(RFV_eval(p,:));
  fn_SVM_eval = sprintf('%s]_model=%i.mat',fn_SVM_eval,idx);
  fn_SVM_train = sprintf('%s]_model=%i.mat',fn_SVM_train,idx)
  load(fn_SVM_train,'Train');
  assert(isequal(Train.PAR.SETs.which_train,which_train))
  best_models(p,:) = MODELS(:,idx) ;
  for ii = 1:length(param_names)
    assert(isequal(getfield(Train.PAR.Signal.method.par_ms,param_names{ii}), MODELS(ii,idx)))
  end
  
  fprintf(fid,'\nbest model (%s,partition=%i): ',Signal_name,p);
  fprintf('\nbest model (%s,partition=%i): \n',Signal_name,p);
  for num_param = 1:size(MODELS,1)
    fprintf(1,'%s = %d; \t',param_names{num_param},getfield(Train.PAR.Signal.method.par_ms,param_names{num_param}));
    fprintf(fid,'%s = %d; \t',param_names{num_param},getfield(Train.PAR.Signal.method.par_ms,param_names{num_param}));
  end
  L = load(fn_SVM_eval) ;
  if ~nan(L.RFV)
    assert(L.RFV == RFV_eval(p,idx)) ;
    assert(L.RSV == RSV_eval(p,idx)) ;
  end
  fprintf(fid,'\nvalidation error: RFV:%.4f \t RSV:%.4f\n',L.RFV,L.RSV);
  %fprintf('\nvalidation error: RFV:%.4f \t RSV:%.4f\n',L.RFV,L.RSV);
  
  %--------------------------------------------------------------%
  %%%%% Predict on TEST SET
  %--------------------------------------------------------------%  
  
  Train.PAR.partition = p;
  Train.PAR.SETs.which_val = which_val;
  Train.PAR.SETs.which_pred = which_pred ;
  Train.PAR.SETs.which_plif = which_plif ;
  Train.PAR.fn_plifs = fn_plifs;
  Train.PAR.fn_SVM_eval = fn_SVM_eval;
  Train.PAR.fn_pred = fn_pred;
  
  PAR_new = Train.PAR;
  PAR_new.eval_on = 0;
  PAR_new.Signal.filter_label.test = Signal.filter_label.test;

  if ~fexist(PAR_new.fn_pred),
    predict = 1;
  else
    pr=load(PAR_new.fn_pred);
    if ~isequal(PAR_new.Signal.method.par_ms, pr.Train.PAR.Signal.method.par_ms);
      predict=1;
      fprintf(1,'prediction file is out of date; it will be overwritten now\n')
    else
      predict=0;
      fprintf(1,'prediction file exists and was created with par_ms of best model\n')
    end
  end
  if predict
    if ~isfield(tasks, 'run_locally') || tasks.run_locally.predict==0,

      [mem_req, time_req, opts] = rproc_memtime_policy('load_and_predict',  0, options) ;%RPROC.MEMREQ ;

      jobinfo(p) = rproc('load_and_predict',PAR_new, mem_req, opts, time_req);
      % load_and_predict(PAR_new)
    else
      %D=load_data(PAR_new,which_pred,'test',PAR.SETs.test_subsample_neg,PAR.SETs.test_subsample_pos); 
      load_and_predict(PAR_new);%, D);
      %clear D
    end
  end ;
  %--------------------------------------------------------------%
  %%%%% Predict on PLIF SET
  %--------------------------------------------------------------%  
  
  PAR_new.eval_on = 0;
  PAR_new.plif_on = 1;
  
  
  
  if ~fexist(PAR_new.fn_plifs)
    predict = 1;
  else
    pr=load(PAR_new.fn_plifs);
    if ~isequal(PAR_new.Signal.method.par_ms, pr.Train.PAR.Signal.method.par_ms);
      predict=1;
      fprintf(1,'plif file is out of date; it will be overwritten now')
    else
      predict=0;
      fprintf(1,'plif file exists and was created with par_ms of best model')
    end
  end
  if predict
    if ~isfield(tasks, 'run_locally') || tasks.run_locally.predict==0,
      [mem_req, time_req, opts] = rproc_memtime_policy('load_and_predict',  0, options) ;

      jobinfo_1(p) = rproc('load_and_predict', PAR_new, mem_req, opts, time_req);
      % load_and_predict(PAR_new)
    else
      %D=load_data(PAR,which_plif,'test',PAR.SETs.test_subsample_neg,PAR.SETs.test_subsample_pos); 
      load_and_predict(PAR_new);%, D);
      %clear D
    end
  end ;
end  %% loop over partitions
if ~isempty(jobinfo),
  [jobinfo,num_crashed] = rproc_wait(jobinfo, 20, 1, -1) ;
end ;
if ~isempty(jobinfo),
  [jobinfo_1,num_crashed] = rproc_wait(jobinfo, 20, 1, -1) ;
end ;

if ~isempty(jobinfo),
  unix('sync')
  pause(30)
  unix('sync')
end ;

for p = 1:size(SETs.partitions,1) %% loop over partitions 
  fprintf(fid,'\n\n-----------\n');
  fprintf(fid,'PARTITION %i\n',p);
  fprintf(fid,'-----------\n');
  fprintf(fid,'\nSITE: %s\n',Signal_name);
    
  fn_best_SVM = sprintf('%sbest_partition=%i.mat', output_fn.fn_SVMs,p) ;
  fn_pred = sprintf('%spartition=%i.mat',output_fn.fn_SVMs,p);
  L = load(fn_pred,'RSV','RFV','svmFileName','PAR') ;
  load(L.svmFileName,'Train')
  Train.PAR = L.PAR;
  if ~fexist(fn_best_SVM)
    %save(fn_best_SVM,'-struct', 'L')
    %save(fn_best_SVM, '-append' ,  'Train')
    save_append(fn_best_SVM, 0, L)
    save_append(fn_best_SVM, 2, 'Train', Train)
  else
    old = load(fn_best_SVM,'Train')
    if isequal(old.Train,Train)
      fprintf('file %s already exists\n',fn_best_SVM),
      fprintf('Train identical\n')
    else
      fprintf('file %s already exists\n',fn_best_SVM),
      error('Train not identical\n')
    end
  end
  fprintf(fid,'\ntest error: RFV:%.4f \t RSV:%.4f\n',L.RFV,L.RSV);
  fprintf('\ntest error: RFV:%.4f \t RSV:%.4f\n',L.RFV,L.RSV);
  
  RFV_test(p)= L.RFV ;
  RSV_test(p)= L.RSV ;
  
  clear PAR_new
end  %% loop over partitions

L=load(output_fn.fn_models, 'best_models','Model_param','RSV_test','RFV_test') ;
if ~isfield(L, 'best_models') || ~isfield(L, 'Model_param') || ~isfield(L, 'RSV_test') || ~isfield(L, 'RFV_test') || ...
      ~isequal(L.best_models, best_models) || ~isequal(L.Model_param, Model_param) || ~isequal(L.RSV_test, RSV_test) || ~isequal(L.RFV_test, RFV_test)
  %save(output_fn.fn_models,'-append','best_models','Model_param','RSV_test','RFV_test')
  save_append(output_fn.fn_models, 1, 'best_models', best_models, ...
              'Model_param', Model_param, 'RSV_test', RSV_test, ...
              'RFV_test', RFV_test)
end ;

fprintf(fid,'\n%s\n',Signal_name);
fprintf(fid,'RSV (mean and std over %i partitions): %.4f +/- %.4f\n',length(RSV_test),mean(RSV_test),std(RSV_test));
fprintf(fid,'RFV (mean and std over %i partitions): %.4f +/- %.4f\n',length(RSV_test),mean(RFV_test),std(RFV_test));
ret = [];
%fclose(fid)





return

function [chr, strand, pos, score, signal_name, score_name, chr_name_dic, signal_name_dic, score_name_dic, success]=read_sigpred_bin(sigpred_fname)
% [chr, strand, pos, score, signal_name, score_name, chr_name_dic, signal_name_dic, score_name_dic, success]=read_sigpred_bin(sigpred_fname)

chr_name_dic = {} ;
score_name_dic = {} ;
signal_name_dic = {} ;

chr=[] ; 
signal_name = [] ;
score_name = [] ;
strand = '' ;
pos = [] ;
score = [] ;

if ~fexist(sigpred_fname),
  success=false ;
  return ;
end ;

cnt = 0; last=0;
while ~last,
  try
    SPF=load(sigpred_fname, sprintf('SPF_%i', cnt)) ;
    if ~isfield(SPF, sprintf('SPF_%i', cnt)),
      break
    end ;
  catch
    break ;
  end 
  if ~isempty(SPF)
    eval(sprintf('SPF=SPF.SPF_%i;', cnt)) ;
    cnt=cnt+1 ;
  %else
  %  SPF=load(sigpred_fname) ;
  %  last = 1 ;
  end ;

  if any(ismember(chr_name_dic, SPF.contig_name)),
    contig_idx = find(ismember(chr_name_dic, SPF.contig_name)) ;
  else
    chr_name_dic{end+1} = SPF.contig_name ;
    contig_idx = length(chr_name_dic) ;
  end ;

  if any(ismember(score_name_dic, SPF.score_name)),
    score_idx = find(ismember(score_name_dic, SPF.score_name)) ;
  else
    score_name_dic{end+1} = SPF.score_name ;
    score_idx = length(score_name_dic) ;
  end ;

  if any(ismember(signal_name_dic, SPF.signal_name)),
    signal_idx = find(ismember(signal_name_dic, SPF.signal_name)) ;
  else
    signal_name_dic{end+1} = SPF.signal_name ;
    signal_idx = length(signal_name_dic) ;
  end ;

  chr(end+1:end+length(SPF.pos))=contig_idx ;
  score_name(end+1:end+length(SPF.pos))=score_idx ;
  signal_name(end+1:end+length(SPF.pos))=signal_idx ;
  pos(end+1:end+length(SPF.pos))=SPF.pos ;
  score(end+1:end+length(SPF.pos))=SPF.score ;
  strand(end+1:end+length(SPF.pos))=SPF.strand ;
end ;

success = true ;

return

function [chr, strand, pos, score, signal_name, score_name, chr_name_dic, signal_name_dic, score_name_dic]=read_sigpred(sigpred_fname)
% [chr, strand, pos, score, signal_name, score_name]=read_sigpred(sigpred_fname)

if fexist([sigpred_fname '.gz']) && ~fexist(sigpred_fname),
  ret = unix(sprintf('gzip -d %s',sigpred_fname)) ;
  assert(ret==0) ;
end ;


chr_name_dic = {} ;
score_name_dic = {} ;
signal_name_dic = {} ;

chr=[] ; 
signal_name = [] ;
score_name = [] ;
strand = '' ;
pos = [] ;
score = [] ;

fd = fopen(sigpred_fname, 'r') ;

rest_text = '' ; part = 0 ; part_offset = 0 ;
line_idx = 0 ;
while ~feof(fd),
  part = part + 1 ;
  text=fread(fd, 100000000, 'uint8=>char') ;
  text=[rest_text text'] ;
  rest_text = '' ;
  if length(text)==0, 
    break ; 
  end ;
  last_idx = find(text==sprintf('\n'), 1, 'last') ;
  if last_idx~=length(text) && ~feof(fd),
    rest_text = text(last_idx+1:end) ;
    text(last_idx:end) = [] ;
  end ;
  text = [text sprintf('\n')] ;
  line_idx = find(text==sprintf('\n')) ;

  if length(line_idx)>2, 
    chr(length(chr)+1 : length(chr) + length(line_idx) -2 ) = nan ; 
    signal_name(length(signal_name)+1:length(signal_name) + length(line_idx)-2) = nan ;
    score_name(length(score_name)+1 : length(score_name) + length(line_idx)-2) = nan ;
    strand(length(strand)+1 : length(strand) + length(line_idx)-2) = ' ' ;
    pos(length(pos)+1 : length(pos) + length(line_idx)-2) = nan ;
    score(length(score)+1 : length(score) + length(line_idx)-2) = nan ;
  end ;

  chr_idx = 0 ;
  signal_name_idx = 0 ;
  score_name_idx = 0 ;
  progress = 0 ;
  for i=1:length(line_idx)-1,
    line=text(line_idx(i)+1:line_idx(i+1)-1) ;
    if isempty(line), 
      chr(i) = nan ;
      score(i) = nan ;
      score_name(i) = nan ;
      signal_name(i) = nan ;
      pos(i) = nan ;
      strand(i) = ' ' ;
      continue; 
    end ;
    if isequal(line(1),'#')
      continue
    end

    elem_idx = find(line==sprintf('\t')) ;
    
    if (i-1)/length(line_idx)>=progress,
      fprintf('\rPart %i: Read line %i/%i (%2.0f%%)     ', part, (i-1), length(line_idx), progress*100);
      progress = progress+0.05 ;
    end ;
    if chr_idx==0 || ~isequal(line(1:elem_idx(1)-1), chr_name_dic{chr_idx}),
      chr_idx = strmatch(line(1:elem_idx(1)-1), chr_name_dic, 'exact') ;
      if isempty(chr_idx),
        chr_name_dic{end+1} = line(1:elem_idx(1)-1) ;
        chr_idx = length(chr_name_dic) ;
      end ;
    end ;
    
    if signal_name_idx==0 || ~isequal(line(elem_idx(1)+1:elem_idx(2)-1), signal_name_dic{signal_name_idx}),
      signal_name_idx = strmatch(line(elem_idx(1)+1:elem_idx(2)-1), signal_name_dic, 'exact') ;
      if isempty(signal_name_idx),
        signal_name_dic{end+1} = line(elem_idx(1)+1:elem_idx(2)-1) ;
        signal_name_idx = length(signal_name_dic) ;
      end ;
    end ;
    
    if score_name_idx==0 || ~isequal(line(elem_idx(2)+1:elem_idx(3)-1), score_name_dic{score_name_idx}),
      score_name_idx = strmatch(line(elem_idx(2)+1:elem_idx(3)-1), score_name_dic, 'exact') ;
      if isempty(score_name_idx),
        score_name_dic{end+1} = line(elem_idx(2)+1:elem_idx(3)-1) ;
        score_name_idx = length(score_name_dic) ;
      end ;
    end ;
    
    chr(i+part_offset) = chr_idx ;
    signal_name(i+part_offset) = signal_name_idx ;
    score_name(i+part_offset) = score_name_idx ;
    strand(i+part_offset) = line(elem_idx(4)+1:elem_idx(5)-1) ;
    pos(i+part_offset) = str2double(line(elem_idx(3)+1:elem_idx(4)-1)) ;
    score(i+part_offset) = str2double(line(elem_idx(5)+1:end)) ;
  end

  part_offset = part_offset + length(line_idx)-1 ;
end ;
fclose(fd) ;

fprintf('\rPart %i: Read line %i/%i (%2.0f%%)              \n', part, ...
        length(line_idx), length(line_idx), 100);
signal_name(isnan(chr)) = [] ;
score_name(isnan(chr)) = [] ;
strand(isnan(chr)) = [] ;
pos(isnan(chr)) = [] ;
score(isnan(chr)) = [] ;
chr(isnan(chr)) = [] ;

assert(all(chr~=0)) ;


return

function [total_num_pos, num_pos] = retrieve_num_pos(regions,base_dir)
% num_pos = retrieve_num_pos(regions,base_dir)
%

num_pos = zeros(1,length(regions)) ;
for r=1:length(regions)
  f_name = sprintf('%scontig_%i%s',base_dir,regions(r).chr_num,regions(r).strand);
  d=dir([f_name '.pos']) ;
  if d(1).bytes==0,
    warning('empty pos file') ;
    num_pos(r) = 0 ;
  else
    num_pos(r) = get_num_pos(f_name, [regions(r).start; regions(r).stop]) ;
  end ;
end
total_num_pos = sum(num_pos) ;

return

function regions = retrieve_signals(regions,base_dir,signal_name,fields,which_pos,offset)

% regions = retrieve_signals(regions,base_dir,signal_name,fields,which_pos)
%
% loads all candidate positions and addional informations as specified in
% fields,(i.e.'label','anno_cover','est_cover','cdna_cover','Conf_cum' )
% from binary fiels and adds them to the field Signals.signal_name of the
% structure regions 
%
% see also save_label_files
if nargin<6
  offset = 0;
end
  
if nargin<5
  which_pos.local = 1;
  which_pos.contig = 1;
end

genome_info = init_genome(regions(1).config);

f_name = sprintf('%scontig_%i%s.pos', base_dir,regions(1).chr_num,regions(1).strand );
if ~fexist(f_name),
  fprintf('file %s does not exist\n', f_name);
end

if ~isempty(fields)
  f_idx = [];
  for f=1:length(fields),
    f_name = sprintf('%scontig_%i%s.%s', base_dir,regions(1).chr_num,regions(1).strand, fields{f});
    if fexist(f_name),
      f_idx = [f_idx f];
    else
      fprintf('file %s does not exist\n', f_name);
    end
  end
  if ~isempty(f_idx),
    fields = fields(f_idx);
  else
    error('no files available')
  end
end

if ~isfield(regions,'Signals')
  regions(1).Signals = [];
end


for r=1:length(regions)
  if isfield(regions(r),'num_fragments')&&~isempty(regions(r).num_fragments)
    num_fragments = regions(r).num_fragments;
  else
    num_fragments = 1;
  end
  prefix = 0;
  
  SIGNALS.pos = [];
  SIGNALS.contig_pos = [];
  for f=1:length(fields)
    SIGNALS.(fields{f})= [];
  end
  if ismember('pos2',fields)
    SIGNALS.contig_pos2 = [];
  end
  for p=1:num_fragments
    contig_length = dir(genome_info.flat_fnames{regions(r).chr_num(p)}) ;
    contig_length = contig_length.bytes;
    %if 1%r==5145
    %  fprintf(1, 'regions.stop: %i, regions.start: %ir: %i',regions(r).start,regions(r).stop, r)
    %end
    if regions(r).start(p)<1
      warning('regions.start<1; set to 1!')
      regions(r).start(p) = 1;
    end
    if regions(r).stop(p)>contig_length
      warning('regions.stop>contig length; set to contig_length!')
      regions(r).stop(p) = contig_length;
    end
    f_name = sprintf('%scontig_%i%s',base_dir, regions(r).chr_num(p), ...
                     regions(r).strand(p));
    if fexist([f_name '_sha1sum.mat'])
      [tmp,sha1sum] = unix(sprintf('sha1sum %s.pos',f_name));
      sha1sum = separate(sha1sum,' ');
      Old = load([f_name '_sha1sum'],'sha1sum');
      Old.sha1sum = separate(Old.sha1sum,' ');
      if ~isequal(Old.sha1sum{1},sha1sum{1})
        warning('sha1sum different')
      end
      %  else
      %    warning('no sha1sum check performed')
    end
    
    if isempty(fields)
      fields{1} ='pos';
    end
    if regions(r).start(p)+offset>regions(r).stop(p)-offset
      regions(r).Signals.(signal_name) = SIGNALS;
      continue
    end
    d=dir([f_name '.pos']) ;
    if d(1).bytes==0,
      contig_pos = [] ;
      score = zeros(0, length(fields)) ;
    else
      [contig_pos,score] = interval_query(f_name,fields,[regions(r).start(p)+offset;regions(r).stop(p)-offset]);
    end ;
    if ismember('pos2',fields)
      if ~isempty(contig_pos)
        idx_p = find(strcmp('pos2',fields));
        contig_pos2 = score(:,idx_p);
      else
        contig_pos2 =[];
      end
    end
    assert(isequal(contig_pos, sort(contig_pos))) ;
    %assert(issorted(contig_pos))
    if ~isempty(contig_pos)
      if ~(contig_pos(1)>=regions(r).start(p)&contig_pos(end)<=regions(r).stop(p))
        if keyboard_allowed()
          keyboard
        end;
      end
    end
    if which_pos.local
      if regions(r).strand(p)=='+'
        pos = contig_pos - regions(r).start(p) + 1+prefix;
        if exist('contig_pos2')
          pos2 = contig_pos2 - regions(r).start(p) + 1+prefix;
        end
        SIGNALS.pos = [SIGNALS.pos pos];
      else
        pos = regions(r).stop(p) - contig_pos + 1;
        [pos,idx] = sort(pos);
        score = score(idx,:);
        contig_pos = contig_pos(idx);
        if exist('contig_pos2')
          contig_pos2 = contig_pos2(idx);
          pos2 = regions(r).stop(p) - contig_pos2 + 1;
          SIGNALS.pos = [SIGNALS.pos pos2];
        else
          SIGNALS.pos = [SIGNALS.pos pos];
        end
      end
    end
    if which_pos.contig
      SIGNALS.contig_pos = [SIGNALS.contig_pos contig_pos];
      if exist('contig_pos2')
        SIGNALS.contig_pos2 = [SIGNALS.contig_pos2 contig_pos2];
        if regions(r).strand(p)=='+'
          SIGNALS.pos2 = [SIGNALS.pos2 pos2];
        else
          SIGNALS.pos2 = [SIGNALS.pos2 pos];
        end
      end
    end
    for f=1:length(fields)
      if strmatch(fields{f},'label') ix=f; end
      if ~isequal(fields{f},'pos')&~isequal(fields{f},'pos2')
        SIGNALS.(fields{f})= [SIGNALS.(fields{f}), score(:,f)];
      end
    end
    if exist('ix')
      u = unique(score(:,ix));
      if length(u)==1 assert(u==-1|u==1),
      elseif length(u)==2 assert(all(u==[-1;1])),
      elseif isempty(u),
      else error('unique(label) is not [-1;1]');
      end
    end
    regions(r).Signals.(signal_name) = SIGNALS;
    prefix = prefix +regions(r).stop(p)- regions(r).start(p);
  end %%loop over fragments
end %% loop over blocks

return

function save_examples(blocks,split,P)
%save_examples(blocks,split,P)
%
% INPUT 	regions with field Signals with field `Signal.name` with fields:
%			-pos
%			-anno_cover
%			-est_cover
%			-cdna_cover
%	PAR with fields
%			-organism
%			-Signal.filter_label
%			-Signal.name
%			-Signal.consensus
%			-FN.input_sig.fn_examples
%			-FN.input.fn_split
%			-FN.input.fn_regions


%only for RPROC
if nargin<3
  %addpath ..
  % use the rproc.addpath way to add paths in client processes!!
  %paths
  P = blocks.P;
  split = blocks.split;
  blocks = blocks.blocks;
end

Signal = P.Signal ; 
num_splits = P.num_splits;

fn_training_blocks = P.fn_training_blocks;
fn_candsites = P.fn_candsites;
fn_filter_settings = P.fn_filter_settings;
fn_examples = P.fn_examples;
fn_example_statistics = P.fn_example_statistics;

Signal_name = Signal.name;
lwin = Signal.lwin_big;
rwin = Signal.rwin_big;

fields = fieldnames(Signal.filter_label);
fprintf( 1, 'save_examples... (%d fields, %d splits)\n', length(fields), num_splits );
for ff = 1:length(fields)
   for ss=1:num_splits
    %---------------------------------------------
    %CREATE OUTPUT FILENAME
    %----------------------------------------------  
    filename = gen_example_filename(Signal,fn_filter_settings,fn_examples,ff,ss);
    if ~fexist(filename)
      num_examples = 0;
      POS.fn_training_blocks = fn_training_blocks;	 
      split_reg_ids = getfield(split,sprintf('split_%i',ss));
      blocks_idx = find(ismember([blocks.id],split_reg_ids));
      split_regs = blocks(blocks_idx);
      clear blocks_idx split_reg_ids
      
      %if ~isfield(split_regs,'seq') 
      %  fprintf(1,'loading sequence...\n')
      %  split_regs = load_sequence(split_regs);
      %  fprintf(1,'\n')
      %end
      
      %---------------------------------------------
      % retreive label information
      %---------------------------------------------
      
      % if ~isfield(split_regs, 'Signals')
      %   temp_region = retrieve_signals(split_regs(1),fn_candsites,Signal_name,{'label',Sig.Conf_names{:}});
      % end
      
      
      %---------------------------------------------
      %----heuristic--->estimate size of chararray; 
      % -some positions are filtered out by the filerfunction: Signal.filter_fct
      % -if the size is to small then this function is very time-consuming
      % -otherwise it is very space-consuming
      %---------------------------------------------
      %total_num_pos = retrieve_num_pos(split_regs, fn_candsites) ;
      %num_pos = ceil(total_num_pos) ;
     
      %--------------------------------------------------
      % determine the maximal number of training examples
      % for the given sensor
      %--------------------------------------------------
      t = cputime;
      num_pos_ex = 0;
      num_neg_ex = 0;
      for r=1:length(split_regs)
        if ~isfield(split_regs, 'Signals')
          temp_region = retrieve_signals(split_regs(r),fn_candsites,Signal_name,{'label'});
	  num_pos_ex = num_pos_ex+sum(temp_region.Signals.(Signal_name).label == 1);
	  num_neg_ex = num_neg_ex+sum(temp_region.Signals.(Signal_name).label == -1);
        end
      end
      fprintf(1,'it took %fs to determine the true number of positive and negative examples\n', cputime-t);
      vec=subsample_policy([num_pos_ex num_neg_ex], Signal_name);
      max_num_pos = vec(1);
      max_num_neg = vec(2);
      reduction_factor_pos = max_num_pos/num_pos_ex;
      reduction_factor_neg = max_num_neg/num_neg_ex;
      num_pos = max_num_pos+max_num_neg;

      XT=zeros([lwin+rwin,num_pos],'int8');
      LT=zeros(1,num_pos);
      POS.pos=zeros(1,num_pos);
      POS.region_id=zeros(1,num_pos);

      if isfield(Signal.filter_label.(fields{ff}), 'USE_ALL')&&...
            Signal.filter_label.(fields{ff}).USE_ALL
        Signal.Conf_names = {} ;
      end ;
      
      t = cputime;
      fprintf(1,'starting with split %i\n',ss)
      for r=1:length(split_regs)
        if mod(r,100)==0,
          fprintf('region %i/%i\r', r, length(split_regs)) ;
        end ;
        
        if ~isfield(split_regs, 'Signals')
          temp_region = retrieve_signals(split_regs(r),fn_candsites,Signal_name,{'label',Signal.Conf_names{:}});
        end
        
        sig_field = getfield(temp_region.Signals,Signal_name);
        if isfield(Signal.filter_label.(fields{ff}),'use_label')
          sig_field.label = sig_field.(Signal.filter_label.(fields{ff}).use_label);
        end
        
        if ~isfield(split_regs,'seq') 		
          if isfield(Signal.filter_label.(fields{ff}), 'USE_ALL')&&...
                Signal.filter_label.(fields{ff}).USE_ALL
            if temp_region.strand == '+',
              loffset=0 ; % remember the offset for later
              tmp = max(1,temp_region.start-lwin) ;
              loffset = temp_region.start - tmp ;
              temp_region.start = tmp ;
              temp_region.stop = temp_region.stop + rwin ;
            else
              tmp = temp_region.start-rwin ;
              temp_region.start = tmp ;
              temp_region.stop = temp_region.stop + lwin ;
              loffset = lwin ;
            end ;
          end ;
          temp_region = load_sequence(temp_region);
        end
        seq = temp_region.seq;
        
        %---------------------
        %FILTER EXAMPLES
        %---------------------
        if ~isfield(Signal.filter_label.(fields{ff}), 'USE_ALL')||...
              ~Signal.filter_label.(fields{ff}).USE_ALL
          if ~isfield(Signal.filter_label.(fields{ff}),'conf_pos') ...
                && isfield(Signal.filter_label.(fields{ff}),'conf')
            error('fix this: filtering will not work')
          end
          idx = feval(Signal.filter_fct, temp_region, Signal, fields{ff},1);
        else
          sig_field.pos = sig_field.pos + loffset ;
          idx = find(sig_field.pos>lwin&sig_field.pos<length(seq)-rwin+1);
	  pos_idx = find(sig_field.label==1);
          if reduction_factor_pos<1
	    keep = randperm(length(pos_idx));
	    pos_idx = pos_idx(keep(1:floor(length(pos_idx)*reduction_factor_pos)));
	  end
	  neg_idx = find(sig_field.label==-1);
	  if reduction_factor_neg<1
	    keep = randperm(length(neg_idx));
	    neg_idx = neg_idx(keep(1:floor(length(neg_idx)*reduction_factor_neg)));
	  end
	  if reduction_factor_pos<1||reduction_factor_neg<1
            if ~isempty(pos_idx) && ~isempty(neg_idx),
              un=[pos_idx', neg_idx'] ;
            elseif isempty(pos_idx),
              un = neg_idx' ;
            else
              un = pos_idx' ;
            end ;
	    idx = intersect(idx, un);
	  end
        end
        if isempty(idx); continue; end
        
        %--------------------
        %CREATE XTs
        %--------------------
        XT_temp = zeros(lwin+rwin,length(idx),'int8');
        rid_temp= zeros(1,length(idx))+temp_region.id;
        for row=1:lwin+rwin
          XT_temp(row,:)= seq(sig_field.pos(idx)-lwin+row-1);
        end%loop over rows
        LT_temp= sig_field.label(idx)';
        pos_temp = sig_field.pos(idx)';
        assert(size(LT,2)==num_examples||LT(num_examples+1)==0)
        assert(num_examples==0||LT(num_examples)~=0)
        
        %[size(XT,2) num_examples+length(idx) ceil(size(XT,2)*r/length(split_regs))]
        if size(XT,2)<num_examples+length(idx), 
          warning('extending XT') ;
          POS.pos(ceil(size(XT,2)*1.2)) = 0 ;
          POS.region_id(ceil(size(XT,2)*1.2)) = 0 ;
          LT(ceil(size(XT,2)*1.2)) = 0 ;
          XT(1,ceil(size(XT,2)*1.2)) = 0 ;
        end ;
        
        XT(:, num_examples+1:num_examples+length(idx))= XT_temp;	
        LT(num_examples+1:num_examples+length(idx))= LT_temp;
        POS.pos(num_examples+1:num_examples+length(idx))= pos_temp;
        POS.region_id(num_examples+1:num_examples+length(idx))=rid_temp;
        num_examples = num_examples+length(idx);
      end%loop over blocks
      fprintf(1, 'time required for one split: %.2f\n',cputime-t)
      fprintf(1,'save file %s\n',filename)
      %=== remove zeros
      assert(all(LT(num_examples+1:end)==0))
      LT = LT(1:num_examples);
      XT = XT(:,1:num_examples);
      POS.pos = POS.pos(1:num_examples);
      POS.region_id = POS.region_id(1:num_examples);
      
      %----------------------------------------------- 
      % remove examples that appear more than once
      %----------------------------------------------- 
      if 0
        [tmp1 idx tmp2] = unique(XT','rows');
        idx = idx(randperm(length(idx)));
        XT = XT(:,idx);
        LT = LT(idx);
        POS.pos = POS.pos(idx);
        POS.region_id = POS.region_id(idx);
      end
      %-----------------------------------------------
      % exclude examples containing non ACGT-Symbols 
      %-----------------------------------------------
      %xtpos=XT~='A'&XT~='C'&XT~='G'&XT~='T';
      %save space
      xtpos=XT~='A';
      for s='CGT'
        xtpos=xtpos&XT~=s;
      end
      cols = find(sum(xtpos));
      clear xtpos
      if length(cols)>0
        warning(sprintf('%i examples containing non ACGT symbols excluded',length(cols)))
        XT(:,cols)=[];
        LT(cols)=[];
        POS.pos(cols)=[];
        POS.region_id(cols)=[];
      end
      %----------------------------------------------- 
      % save matrix in pieces into one file
      %----------------------------------------------- 
      assert(sum(LT==1)>0);
      fprintf( 'trying to save %s\n', filename );
      save_matrix(filename, XT, 'XT');
      %save(filename,'-append','LT','POS','PAR');
      save_append(filename, 1,'LT', LT, 'POS', POS, 'P', P);
      % save_append(filename, 1,'LT', LT, 'POS', POS, 'PAR', PAR);
    end%if ~fexist(filename)
   end%loop over splits
   
   stat = save_example_statistics(Signal, fn_filter_settings, ...
                                  fn_examples, fn_example_statistics, ...
                                  num_splits);
end%loop over filter_label fields


return

function stat=save_example_statistics(Signal,fn_filter_settings,fn_examples,fn_example_statistics,num_splits)
% stat = save_example_statistics(Signal,fn_filter_settings,fn_examples,fn_example_statistics,num_splits)


pos = 0;
neg = 0;
for j=1:num_splits
  f = gen_example_filename(Signal,fn_filter_settings,fn_examples,1,j) ;
  l=load(f, 'LT');
  pos = pos + sum(l.LT==1);
  neg = neg + sum(l.LT==-1);
  stat.(sprintf('split_%i_num_pos',j))=sum(l.LT==1);
  stat.(sprintf('split_%i_num_neg',j))=sum(l.LT==-1);
  stat.(sprintf('split_%i_frac_pos',j))=sum(l.LT==1)/sum(l.LT==-1)*100;
end

stat.all_num_pos = pos;
stat.all_num_neg = neg;
stat.all_frac_pos=pos/neg*100;

assert(pos>0);
assert(neg>0);


fn = fn_example_statistics(1:end-4);
num = 1;
while fexist(sprintf('%s_%i.mat',fn,num))
  num=num+1;
end

save(sprintf('%s_%i.mat',fn,num), 'stat');

return

function save_predictions_as_sigpred_bin(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand)
% save_predictions_as_sigpred_bin(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand)

%assert(issorted(pos))
assert(isequal(pos, sort(pos)))
assert(length(pos)==length(score))

fn_out=[out_fn_prefix '_' score_name '_spf.mat'];

if fexist(fn_out) 
  fprintf(1, 'file exists, it will NOT be overwritten\n')
  return
end
fprintf(1,'save in binary signal prediction format (spf): %s\n', fn_out);
save(fn_out, '-V7', 'contig_name', 'signal_name', 'score_name', 'pos', 'strand', 'score') ;

return

function save_predictions_as_sigpred(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)
% save_predictions_as_wiggle(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)

%assert(issorted(pos))
assert(isequal(pos, sort(pos)))
assert(length(pos)==length(score))

if ~exist('do_gzip', 'var'),
  do_gzip = 1 ;
end ;

fn_out=[out_fn_prefix '_' score_name '.spf'];

if fexist(fn_out) | fexist([fn_out '.gz'])
  fprintf(1, 'file exists, it will NOT be overwritten\n')
  return
end
fprintf(1,'save in signal prediction format (spf): %s\n', fn_out);
[fd msg] = fopen(fn_out, 'w+');

fprintf(fd, '##Seqid\tType\tSource\tStart\tStrand\tScore\n') ;
for j=1:length(pos)
  fprintf(fd, '%s\t%s\t%s\t%i\t%c\t%.3f\n', contig_name, signal_name, score_name, pos(j), strand, score(j)) ;
end

fclose(fd);

if do_gzip,
  ret = unix(sprintf('gzip %s',fn_out)) ;
  if ret~=0
    warning('gzip failed') ;
  end
end 


return

function fn_out = save_predictions_as_wiggle(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)
% fn_out = save_predictions_as_wiggle(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)

%assert(issorted(pos))
assert(isequal(pos, sort(pos)))
assert(length(pos)==length(score))

if ~exist('do_gzip', 'var'),
  do_gzip = 1 ;
end ;

fn_out=[out_fn_prefix '_' score_name '.wiggle'];

if fexist(fn_out)|fexist([fn_out '.gz'])
  fprintf(1, 'file exists, it will NOT be overwritten\n')
%  keyboard
  return
end

fprintf(1,'save as wiggle: %s\n', fn_out);
[fd msg] = fopen(fn_out, 'w+');
disp(msg)

fprintf(fd,'browser position chr%s:1-50,000\n', contig_name);
fprintf(fd,'browser hide all\n');
fprintf(fd,'browser pack Genes\n');
fprintf(fd,'browser full altGraph\n');
fprintf(fd,'browser viewLimits=%1.2f:%1.2f\n', min(score), max(score));


fprintf(fd,['track type=wiggle_0 name="%s_%s%s" description="%s chr%s %s" ' ...
            '\n'],signal_name, score_name, strand, score_name, ...
        contig_name, strand);
fprintf(fd,'variableStep chrom=chr%s\n',contig_name);

for j=1:length(pos)
  fprintf(fd,'%i\t%.3f\n', pos(j), score(j));
end

fclose(fd);

if do_gzip,
  ret = unix(sprintf('gzip %s',fn_out));
  if ret~=0
    warning('gzip failed')
  end
end ;

return

function [dummy1, dummy2]=save_predictions_helper(P)
% save_predictions_helper(P) 
try
  pause('on')
  warning('off', 'MATLAB:typeaheadBufferOverflow');  
catch
  % works only for matlab
end


dummy1=[] ;
dummy2=[] ;

fn_genome_config = P.fn_genome_config;
chrom = P.chrom;
strand = P.strand;
fn_pred = P.fn_pred;
Signal_name = P.Signal_name;
resolution = P.resolution;
conf_cum_thresh = P.conf_cum_thresh;
do_gzip = P.do_gzip ;
save_as = P.save_as ;

genome_info = init_genome(fn_genome_config); 
S = dir(genome_info.flat_fnames{chrom}) ;
contig_length = S.bytes;
CHR_NAME = genome_info.contig_names{chrom};
fprintf('contig: %i%s\n', chrom,strand);

%----------------------------
%% GET ALL PREDICTIONS from mat file
%----------------------------

filename = sprintf('%scontig_%i%s_all.mat', fn_pred, chrom, strand) ;
for kk=1:30 % it can take easily several minutes until the file is visible
   try 
     clear pos output Conf Conf_cum
     load(filename, 'inventory');
     for j=1:length(inventory)
        if kk>1,
           fprintf('trying to load variable "%s" from %s (trial %i)\n', inventory{j}, filename, kk);
        end ;
        load(filename,inventory{j})
        assert(exist(inventory{j}, 'var')==1);	 
     end
     break ;
   catch
     if kk==30      
       fprintf('giving up waiting for file %s\n', filename)
     else
       fprintf('waiting for file: %s \n', filename);    
       pause(30)
     end	
  end
end


% always save global coordinates on forward strand
if isequal(strand, '-'),
  pos = contig_length - pos + 1;
end
assert(all(pos>=1));

[pos sort_idx] = sort(pos);
output = output(sort_idx);
if exist('Conf') && exist('Conf_cum')
  Conf = Conf(sort_idx);
  Conf_cum = Conf_cum(sort_idx);
  
  %----------------------------
  %% downsample if demanded
  %----------------------------
  
  assert(all(Conf>=0-1e-8&Conf<=1+1e-8));
  Conf(Conf<0)=0 ;
  Conf(Conf>1)=1 ;
end

side = floor(resolution/2) ;
if (resolution~=1), 
  fprintf('Reducing resolution to one prediction per %i nucleotides\n', resolution) ;
  idx = 1:resolution:length(pos) ;
  cc = zeros(1,length(idx)) ;
  co = zeros(1,length(idx)) ;
  oo = zeros(1,length(idx)) ;
  pp = zeros(1,length(idx)) ;
  
  for j=1:length(idx),
    %if mod(j,1000)==0, fprintf(' %i  \r', j), end ;
    win = max(1,idx(j)-side):min(idx(j)+side,length(pos)) ;
    neighbour = find((pos(win)>=pos(idx(j))-side) & (pos(win)<pos(idx(j))+side)) ;
    %pos(win(neighbour))
    if exist('Conf') && exist('Conf_cum')
      [mv,mi] = max(Conf_cum(win(neighbour))) ;
      assert(Conf_cum(win(neighbour(mi)))==mv) ;
      cc(j) = Conf_cum(win(neighbour(mi))) ;
      co(j) = Conf(win(neighbour(mi))) ;
    else
      [mv,mi] = max(output(win(neighbour))) ;
    end
    pp(j) = pos(win(neighbour(mi))) ;
    oo(j) = output(win(neighbour(mi))) ;
  end;
  pos = pp ;
  Conf_cum = cc ;
  Conf = co ;
  output = oo ;
end ;

%----------------------------
%%check if files exist
%----------------------------
filename = sprintf('%scontig_%i%s', fn_pred, chrom, strand);
if fexist([filename '.pos']),
  fprintf('removing position file (%s). \n', filename);
  unix(sprintf('rm %s', [filename '.pos']));
end
if fexist([filename '.output']),
  fprintf('removing output file (%s)\n', filename);
  unix(sprintf('rm %s', [filename '.output']));
end
if fexist([filename '.Conf']),
  fprintf('removing Conf file (%s)\n', filename);
  unix(sprintf('rm %s', [filename '.Conf']));
end
if fexist([filename '.Conf_cum']),
  fprintf('removing Conf_cum file (%s)\n', filename);
  unix(sprintf('rm %s', [filename '.Conf_cum']));
end

%Conf=round(Conf*1000)/1000;
%output = round(output*1000)/1000;
%Conf_cum=round(Conf_cum*1000)/1000; 
if exist('Conf') && exist('Conf_cum') %&& ~all(Conf_cum==0)&&~all(Conf==0)
  save_score_pos(pos, [output; Conf; Conf_cum], filename, {'output', 'Conf', 'Conf_cum'});
else
  save_score_pos(pos, [output], filename, {'output'});
end

if isequal(Signal_name, 'transacc'),
  disp('Using acceptor predictions in combination with transacc predictions (for wiggle files only)') ;
  FN2.fn_pred = strrep(fn_pred,'transacc', 'acc') ;
  
  filename2 = sprintf('%scontig_%i%s_all.mat', FN2.fn_pred,chrom,strand);
  L=load(filename2, 'pos', 'Conf', 'Conf_cum', 'output');
  % always save global coordinates on forward strand
  if isequal(strand, '-'),
    L.pos = contig_length - L.pos + 1;
  end
  assert(all(pos>=1));
  if ~isequal(sort(L.pos),pos)
    warning('position lists not identical') ;
    setdiff(L.pos, pos)
    setdiff(pos, L.pos)
    
    [tmp,idx1,idx2]=intersect(pos,L.pos) ;
    
    L.Conf = L.Conf(idx2) ;
    L.Conf_cum = L.Conf_cum(idx2) ;
    L.output = L.output(idx2) ;
    L.pos = L.pos(idx2) ;
    
    Conf = Conf(idx1) ;
    Conf_cum = Conf_cum(idx1) ;
    output = output(idx1) ;
    pos = pos(idx1) ;
  else
    [L.pos sort_idx] = sort(L.pos);
    L.Conf = L.Conf(sort_idx);
    L.Conf_cum = L.Conf_cum(sort_idx);
    L.output = L.output(sort_idx);
  end ;
  
  Conf_cum = Conf_cum .* L.Conf_cum ;
  Conf = Conf .* L.Conf ;
  output = output + L.output ;
  clear L ;
end ;

%----------------------------
%% SAVE PREDICTIONS as SPF files
%---------------------------- 

% saving in signal prediction format (without filtering)
if save_as.spf_ascii,
  save_predictions_as_sigpred(filename, pos, output, 'output', Signal_name, CHR_NAME, strand, do_gzip);
end ;
if save_as.spf_binary,
  save_predictions_as_sigpred_bin(filename, pos, output, 'output', Signal_name, CHR_NAME, strand);
end ;

if exist('Conf') %&& ~all(Conf==0)
  if save_as.spf_ascii,
    save_predictions_as_sigpred(filename, pos, Conf, 'Conf', Signal_name, CHR_NAME, strand, do_gzip);
  end ;
  if save_as.spf_binary,
    save_predictions_as_sigpred_bin(filename, pos, Conf, 'Conf', Signal_name, CHR_NAME, strand);
  end ;
end ;

if exist('Conf_cum') %&&  ~all(Conf_cum==0)
  if save_as.spf_ascii,
    save_predictions_as_sigpred(filename, pos, Conf_cum, 'Conf_cum', Signal_name, CHR_NAME, strand, do_gzip);
  end ;
  if save_as.spf_binary,
    save_predictions_as_sigpred_bin(filename, pos, Conf_cum, 'Conf_cum', Signal_name, CHR_NAME, strand);
  end
end ;

%----------------------------
%% SAVE PREDICTIONS as wiggle files
%---------------------------- 

% rounding for wiggle format output
output = round(output*1000)/1000;

%----------------------------
% REMOVE PREDICTIOND BELOW THRESHOLD
%----------------------------


if exist('Conf') && exist('Conf_cum') && save_as.wiggle

  fprintf('Removing all predictions below %.2f before saving wiggle file',conf_cum_thresh) 
  idx = find(Conf > conf_cum_thresh);
  output   = output(idx);
  pos = pos(idx);
  Conf = round(Conf*1000)/1000;
  Conf_cum = round(Conf_cum*1000)/1000; 
  Conf     = Conf(idx); 
  Conf_cum = Conf_cum(idx);
  if save_as.wiggle
    save_predictions_as_wiggle(filename, pos, Conf, 'Conf', Signal_name, CHR_NAME, strand, do_gzip);
    save_predictions_as_wiggle(filename, pos, Conf_cum, 'Conf_cum', Signal_name, CHR_NAME, strand, do_gzip);
  end ;

end

if save_as.wiggle
  save_predictions_as_wiggle(filename, pos, output, 'output', Signal_name, CHR_NAME, strand, do_gzip);
end

return

function [jobinfo] = save_predictions(fn_genome_config, fn_pred, ...
                                      Signal_name, resolution, conf_cum_thresh, strands, run_locally, RPROC, do_gzip, save_as, inventory)
%  [jobinfo] =
%  save_predictions(fn_genome_config,fn_pred,strands,run_locally,RPROC,do_gzip,save_as,inventory)
%
%  Loads predictions corresponding to each contig for a signal
%  defined in PAR.Signal_name.
%  Saves them to binary files which can be read by 
%  interval_query.
%

%
%  see retrieve_signals, interval_query 
if nargin<2
  strands='+-';
end

if ~exist('do_gzip', 'var'),
  do_gzip = 1 ;
end ;
if ~exist('save_as', 'var'),
  save_as.spf_ascii = 1 ;
  save_as.spf_binary = 1 ;
  save_as.wiggle = 0 ;
end ;

jobinfo = rproc_empty(0) ;
% options = RPROC.options;
% options.ncpus = 1;
% options.identifier = sprintf('%scheck%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;
% MEMREQ = RPROC.MEMREQ;

fid=1;q=0;

genome_info = init_genome(fn_genome_config);

P.fn_genome_config = fn_genome_config;
P.Signal_name = Signal_name;
P.fn_pred = fn_pred;
P.conf_cum_thresh = conf_cum_thresh;
P.resolution = resolution ;
P.save_as = save_as ;


% options.identifier = sprintf('%ssave%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;
for c = 1:length(genome_info.contig_names)
  for s = strands
    P.chrom = c;
    P.strand = s;
    P.do_gzip = do_gzip ;
    filename = sprintf('%scontig_%i%s', fn_pred, c, s);

    one_missing=0 ;
    for k=1:length(inventory)
      if ~fexist([filename '.' inventory{k}]) || ~fexist([filename '_' inventory{k} '.spf']) || ~fexist([filename '_' inventory{k} '_spf.mat'])
        one_missing = 1 ;
      end ;
    end ;
    if one_missing,
      if ~run_locally
        q=q+1;

        [mem_req, time_req, opts] = rproc_memtime_policy('save_predictions_helper',  0, RPROC.options) ;%RPROC.MEMREQ ;

        if isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs 
          jobinfo(q) = rproc_create('save_predictions_helper', P, mem_req, opts, time_req) ;
        else
          jobinfo(q) = rproc('save_predictions_helper', P, mem_req, opts, time_req);
        end
      else
        save_predictions_helper(P)
      end
    end
  end %%% loop over strand
end %%% loop over contigs 


rproc_wait(jobinfo, 20, 1, -1);



return

function flag = segment_overlap(start,stop,starts,stops)
% flag = segment_overlap(start,stop,starts,stops)  

flag = ((start >= starts) & (start<=stops)) | ...
       ((stop >= starts) & (stop<=stops)) | ...
       ((start <=starts) & (stop>=stops)) | ...
       ((start >=starts) & (stop<=stops)) ;
  
return

function CONF = set_CONF(Signal,ALL,sig,info_names,signal_name)
% CONF = set_CONF(Signal,ALL,sig,info_names)

%%% GENERAL 
idx = find(strcmp(Signal.Conf_names,'altgenic'));
if ~isempty(idx)
  CONF(idx,:) = ALL.ALTGENIC;
end
idx = find(strcmp(Signal.Conf_names,'gene_id'));
if ~isempty(idx)
  CONF(idx,:) = ALL.GENE_ID;
end

for i=1:length(info_names)
  idx = find(strcmp(Signal.Conf_names,[info_names{i} '_conf']));
  CONF(idx,:) = ALL.COV1(i,:);
  if ~isequal(sig,'other')&~isequal(Signal.name,'transacc')
    idx = find(strcmp(Signal.Conf_names,[info_names{i} '_conf_skip']));
    CONF(idx,:) = ALL.COV2(i,:);
  end
end

%%% ALT SPLICE
idx = find(strcmp(Signal.Conf_names,'alt_exon'));
if ~isempty(idx)
  CONF(idx,:) = ALL.LABEL_2(1,:);
end
idx = find(strcmp(Signal.Conf_names,'alt_intron'));
if ~isempty(idx)
  CONF(idx,:) = ALL.LABEL_2(2,:);
end
idx = find(strcmp(Signal.Conf_names,'alt_3prime'));
if ~isempty(idx)
  CONF(idx,:) = ALL.LABEL_2(3,:);
end
idx = find(strcmp(Signal.Conf_names,'alt_5prime'));
if ~isempty(idx)
  CONF(idx,:) = ALL.LABEL_2(3,:);
end

idx = find(strcmp(Signal.Conf_names,['is' sig]));
if ~isempty(idx)
  CONF(idx,:) = ALL.ISSPLICE;
end

%%% SPECIAL THINGS
idx = find(strcmp(Signal.Conf_names,['has_' signal_name]));
if ~isempty(idx)
  CONF(idx,:) = ALL.HAS_SIG;
end
idx = find(strcmp(Signal.Conf_names,'has_trans'));
if ~isempty(idx)
  CONF(idx,:) = ALL.TRANS;
end


idx = find(strcmp(Signal.Conf_names,'dist'));
if ~isempty(idx)
  CONF(idx,:) = ALL.DIST;
end

if exist('CONF', 'var') && ~(size(CONF,1)==length(Signal.Conf_names))
  warning('this was an assert') ;
end ;

return

function setFeatures(method, Data, svmMode, pred_from_seq, use_preproc);
% setFeatures(method,Data, svmMode,pred_from_seq,use_preproc);


% method = PAR.method;
% tasks = PAR.tasks;
% pred_from_seq = tasks.pred_from_seq;

  
%------------------------------------------------------------------------------%
% check
%------------------------------------------------------------------------------%

assert( strcmp(svmMode,'TRAIN') || strcmp(svmMode,'TEST') );

%assert( isscalar(Data.pos0) );
%assert( size(XT,2) > 0 );
%assert( ischar( XT ) );

if nargin<5
  use_preproc=1;
end
if ~isfield(method,'conservWdsWeighting')
  conservWdsWeighting=0;
else
  conservWdsWeighting= method.conservWdsWeighting;
end
%------------------------------------------------------------------------------%
% pass data
%------------------------------------------------------------------------------%

%sg('loglevel', 'ALL');
if pred_from_seq
  seq = Data.seq;
  idx = find(seq~='A'&seq~='C'&seq~='G'&seq~='T');
  if ~isempty(idx)
    warning('non AGCT-Symbols found; changed to As')
    seq(idx)='A';
  end
  pos = Data.pos;
else
  XT = Data.XT;
  %pos0=Data.pos0;
  if isfield(Data, 'LT')
    LT = Data.LT;
  end
  if isfield(Data, 'W')
    W = Data.W;
  end
end

%==============================================================================%
% set labels
%==============================================================================%
if strcmp(svmMode,'TRAIN')
  sg( 'set_labels', 'TRAIN', LT ); 
end
%==============================================================================%
% set features
%==============================================================================%



nofKernels = length(method.kernels);
if nofKernels>1 && ~isfield(method.kernels{2},'name') && ~isfield(method.kernels{2}, 'kernel_name')
  % for content sensors; several orders are trained simulatuously.
  % but this is done by concatination of the feature vector directly.
  nofKernels = 1;
end
assert( nofKernels >= 1 );
if( nofKernels == 1 )
  addFeats = 'set_features';
else
  addFeats = 'add_features';
end;


for k = 1:nofKernels
  kernel = method.kernels{k};
  % for compatibility with older versions
  if ~isfield(kernel, 'kernel_name') && isfield(kernel,'name')
    kernel.kernel_name = kernel.name;
  end
  if ~pred_from_seq   
    if ~isfield(method,'center') || isempty(method.center)
      select= [1:size(XT,1)];
    else
      if ~isfield(kernel,'lwin') 
        warning('no lwin specified for kernel')
        kernel.lwin = -method.center;
      end
      if ~isfield(kernel,'rwin') 
        warning('no rwin specified for kernel')
        kernel.rwin = -method.center+size(XT,1)-1;
      end
      select = (kernel.lwin:kernel.rwin)+method.center+1;
    end
    if ~isequal(select,[1:size(XT,1)])
      XT_temp = XT(select,:);
    else
      XT_temp = XT;
    end
    %keyboard;
  else
    %positiones should be a 0-based index
    pos_temp=pos+kernel.lwin-1;
    winsize_temp=kernel.rwin-kernel.lwin+1;
    assert(length(pos_temp)>0)
    assert(length(seq)>max(pos_temp)+winsize_temp)
    assert(winsize_temp>0)

    % work-around for from_position_list bug
    if length(pos_temp)==1,
      pos_temp = [pos_temp pos_temp] ;
    end ;
  end
  %------------------------------------------------------------------------------%
  % kernel: WD or WDS
  %------------------------------------------------------------------------------%

  if strcmp(kernel.kernel_name,'WD') || strcmp(kernel.kernel_name,'WDS')
    if ~pred_from_seq
      fprintf( '--> XT[%d:%d], num_examples=%i,  order=%d\n', min(select), max(select),size(XT,2), kernel.order );
      sg( addFeats, svmMode, XT_temp, 'DNA' );
    else
      sg( addFeats, svmMode,{seq}, 'DNA' );
      sg('from_position_list','TEST', winsize_temp, int32(pos_temp));
      % NEW
      %sg( addFeats, svmMode, {seq}, 'DNA', 'from_position_list', winsize_temp, int32(pos_temp), 1);
    end
  end;
  

  %------------------------------------------------------------------------------%
  % kernel: Spectrum
  %------------------------------------------------------------------------------%

  if strcmp(kernel.kernel_name,'SP' )
    if~pred_from_seq 
      fprintf( '--> XT[%d:%d], wordlen=%d\n', min(select), max(select), kernel.wordlen );
      sg( addFeats, svmMode, XT_temp, 'DNA');
    
      if( ~isfield(kernel,'gaps') )
        sg( 'convert', svmMode, 'STRING', 'CHAR', 'STRING', 'WORD', kernel.wordlen, kernel.wordlen-1 ) ;
      else
        fprintf('--> convert\n')
        sg( 'convert', svmMode, 'STRING', 'CHAR', 'STRING', 'WORD', kernel.wordlen, kernel.wordlen-1, kernel.gaps );
      end;
      
      if( strcmp(svmMode,'TRAIN') && use_preproc )
        fprintf('--> clean_preproc\n')
        sg( 'clean_preproc' );
        fprintf('--> add_preproc SORTWORDSTRING\n')
        sg( 'add_preproc', 'SORTWORDSTRING' );
        fprintf('--> attach_preproc %s\n', svmMode)
        sg( 'attach_preproc', svmMode );
      end;
    else
      % from_position_list doesnot work for training for spectrum kernel  
      assert(strcmp(svmMode,'TEST')) ;
      sg( addFeats, svmMode, {seq}, 'DNA' );
      % window is cut out from sequence if vonvertion is made after calling 'from_position_list
      % if convertion is done at this place the window size must be reduced by -ordnung-1;
      % (not yet tested) this woud save a lot of time and space. 
      % 
      if( ~isfield(kernel,'gaps') )
        sg( 'convert', svmMode, 'STRING', 'CHAR', 'STRING', 'WORD', kernel.wordlen, 0);
      else
        fprintf('--> convert\n')
        sg( 'convert', svmMode, 'STRING', 'CHAR', 'STRING', 'WORD', kernel.wordlen, 0, kernel.gaps );
      end;
      sg('from_position_list','TEST', winsize_temp- kernel.wordlen+1, int32(pos_temp+kernel.wordlen-1));
    end
  end;
 
  %------------------------------------------------------------------------------%
  % kernel:  RBF 
  %------------------------------------------------------------------------------%
 
  if strcmp(kernel.kernel_name,'RBF')
    fprintf( '--> XT[%d:%d], num_examples=%i,  sigma=%d\n', min(select), max(select),size(XT,2), kernel.sigma );
    sg( addFeats, svmMode, XT_temp);    
  end;
  
  %------------------------------------------------------------------------------%
  % kernel:  POLYNOMIAL
  %------------------------------------------------------------------------------%
 
  if strcmp(kernel.kernel_name,'POLY')
    fprintf( '--> XT[%d:%d], num_examples=%i,  degree=%d\n', min(select), max(select),size(XT,2), kernel.degree );
    sg( addFeats, svmMode, XT_temp);    
  end;
  
  %------------------------------------------------------------------------------%
  % kernel:  LINEAR
  %------------------------------------------------------------------------------%
  
  if strcmp(kernel.kernel_name,'LINEAR')
    fprintf( '--> XT[%d:%d], num_examples=%i,\n', min(select), max(select),size(XT,2) );
    sg( addFeats, svmMode, XT_temp);    
  end;
end%loop over kernels
%------------------------------------------------------------------------------%
% done
%------------------------------------------------------------------------------%

if( use_preproc & strcmp(svmMode,'TEST')  & ~pred_from_seq )
   fprintf('--> attach_preproc %s', svmMode)
  sg( 'attach_preproc', svmMode );
end;

clear XT;
sg( 'init_kernel', svmMode );

% --- set feature weights
if( conservWdsWeighting > 0 )
  W1 = W( select1, : );
  if( conservWdsWeightNorming )
    fprintf( '--> conserv(WDS): using normalized feature weights!\n' );
    meanW = mean( W1, 1 );
    W1 = W1 .* repmat( 1./meanW, size(W1,1), 1 );
  else
    fprintf( '--> conserv(WDS): using raw feature weights!\n' );
    W1 = 3/size(W1,1) * W1;
  end;
  sg( 'set_subkernel_weights_combined', W1, 1 );
  clear W1;
end;



return

function [XT,LT,POS,SC] = subsample(XT, LT, POS, SC, subsample_neg, subsample_pos)
% [XT,LT,POS,SC] = subsample(XT, LT, POS, SC, subsample_neg, subsample_pos)

if subsample_neg<1 & subsample_neg>0 & subsample_pos<=1 & subsample_pos>0 % & ~isempty(LT) 

  fprintf('subsampling %i %% of negative examples \n',subsample_neg*100);
  fprintf('subsampling %i %% of positive examples \n',subsample_pos*100);
  posidx = find(LT==1);
  negidx = find(LT==-1);
  rand('seed', pi) ;
  idx2 = randperm(length(negidx));
  idx3 = randperm(length(posidx));
  num_neg = round(subsample_neg*sum(LT==-1));
  num_pos = round(subsample_pos*sum(LT==1));
  
  XT = [XT(:,posidx(idx3(1:num_pos))) XT(:,negidx(idx2(1:num_neg)))] ;
  
  if ~isempty(SC)
    SC = [SC(:,posidx(idx3(1:num_pos))) SC(:,negidx(idx2(1:num_neg)))] ;
  end
  LT = [LT(posidx(idx3(1:num_pos))) LT(negidx(idx2(1:num_neg)))] ; 
  POS.pos = [POS.pos(posidx(idx3(1:num_pos))) POS.pos(negidx(idx2(1:num_neg)))] ; 
  POS.region_id = [POS.region_id(posidx(idx3(1:num_pos))) POS.region_id(negidx(idx2(1:num_neg)))] ; 
  fprintf('%i examples remaining (%.2f %% positives)\n',length(LT),sum(LT==1)/length(LT)*100);

end

return

function [RFV, RSV, train_test_overlap] = svm_predict(Train, Data, svmFileName, outFileName, pred_from_seq, check_overlap);
% [RFV, RSV] = svm_predict(Train, Data, svmFileName,outFileName);
%
%------------------------------------------------------------------------------%
% deal with parameters 
%------------------------------------------------------------------------------%
% for usage with rproc

  if nargin==1
    Data  = Train.Data;
    svmFileName = Train.svmFileName;
    outFileName = Train.outFileName;
    Train = Train.Train;
  end
  
  fprintf('working for outputs to be written to %s\n', outFileName);
  
  method = Train.PAR.Signal.method ;
  
  if ~exist('check_overlap', 'var'),
    check_overlap=1 ;
  end ;
  
  if( ~isfield(method,'conservWeighting') )
    method.conservWeighting = 0;
  end;
  if( method.conservWeighting == 0 )
    W1 = [];
    meanCons = [];
  else
    W1 = res.W;
    assert( ~isempty(W1) );
    meanCons = mean( W1, 1 );
  end;
  
  %------------------------------------------------------------------------------%
  % init svm and set kernel parameters
  %------------------------------------------------------------------------------%
  if (pred_from_seq && ~isempty(Data.pos)) || (~pred_from_seq && ~isempty(Data.XT))
    init_svm_and_kernels(method,pred_from_seq);
    sg_ver = sg('get_version');
  else
    sg_ver = 'empty_prediction' ;
  end ;
  %------------------------------------------------------------------------------%
  % init training features with labels and examples that only contain those of 
  % support vectors
  %------------------------------------------------------------------------------%

  if (pred_from_seq && ~isempty(Data.pos)) || (~pred_from_seq && ~isempty(Data.XT))
    sg('clean_features', 'TRAIN');
    % D.pos0 = method.center;
    D.XT = Train.XT;
    D.LT = Train.LT;
    
    setFeatures(method, D,'TRAIN',0,0);
  end 
  
  clear D 
  
  %------------------------------------------------------------------------------%
  % init test features with examples if pred_from_seq==0 or with seq and positions
  % in the case of genome wide predictions
  %------------------------------------------------------------------------------%

  if (pred_from_seq && ~isempty(Data.pos)) || (~pred_from_seq && ~isempty(Data.XT))
    sg('clean_features', 'TEST' );
    if ~pred_from_seq
      D.XT = Data.XT;
      if isfield(method, 'feature_type') && isequal(method.feature_type,'double') && method.normalize_features
        D.XT(Train.idx_rm,:)  = [];
        D.XT = D.XT - repmat(Train.mean,size(D.XT,2),1)';
        D.XT = D.XT./repmat(Train.std,size(D.XT,2),1)';
      end
      % D.pos0 = method.center;
      D.pos = Data.Pos;
    else
      D.seq = Data.seq;
      assert(size(Data.pos,1)==1|size(Data.pos,2)==2|isempty(Data.pos))
      if size(Data.pos,1)~=1
        D.pos = Data.pos';
      else
        D.pos = Data.pos;
      end
    end
    
    if ~isempty(D.pos)
      setFeatures(method, D, 'TEST', pred_from_seq, 0);
      D_pos = D.pos ;
      clear D
      %------------------------------------------------------------------------------%
      % setup svm with trainingdata
      %------------------------------------------------------------------------------%
      %Train.XT should contain only support vector examples
      assert(size(Train.alphas, 1)==size(Train.XT,2))
      % alphas are in sparse format col_1=value; col_2=0-based index
      % change index to 0:size(alphas,2)-1 consistent with XTs  
      Train.alphas(:,2)= (0:size(Train.alphas, 1)-1)';
      sg('set_svm', Train.b, Train.alphas);
      
      %------------------------------------------------------------------------------%
      % classify
      %------------------------------------------------------------------------------%
      if( method.use_mkl )
        sg( 'set_subkernel_weights', betas );
      end;
      output = sg( 'svm_classify' );
      
      % work-around for from_position_list bug
      if length(D_pos) == 1 && isequal(class(D_pos), 'double') && length(output)==2,
        output=output(1) ;
      end ;
      clear D_pos
    else
      output = [];
    end
  else
    output = [] ;
  end 
  
  % check for overlaps between training and testing data
  
  if check_overlap && ~isequal(determine_engine, 'octave'),
    
    train_subset = randperm(size(Train.XT, 2)) ;
    train_subset = train_subset(1:min(length(train_subset), 100)) ;
    
    if ~pred_from_seq
      [tmp, idx, idx_] = intersect(Train.XT(:, train_subset)', Data.XT', 'rows') ;
    else
      P=[] ; 
      for i=1:length(train_subset), 
        P = [P strfind(Data.seq, Train.XT(:,train_subset(i))') + method.center] ; 
      end ;
      [tmp,idx,idx_]=intersect(P, Data.pos);
    end ;
    if length(idx)>0,
      warning('Overlap between Train and Test sets discovered (%i/%i sequences)\n', length(idx), length(train_subset)) ;
      if length(idx)/length(train_subset)>0.1,
        warning('Large overlap between Train and Test sets discovered (%i/%i sequences)\n', length(idx), length(train_subset)) ;
      end ;
    end ;
  end 
      
      
      
  %------------------------------------------------------------------------------%
  % save output
  %------------------------------------------------------------------------------%
  
  PAR = Data.PAR;
  PAR.Signal = Train.PAR.Signal;
  
  if isfield(Data, 'LT')
    RSV = calcrocscore(output, Data.LT); 
    if isnan(RSV), 
      RSV=0.5 ;
    end ;
    RFV= calcrfcscore(output, Data.LT);
    if isnan(RFV),
      RFV=0 ;
    end;
    LT = Data.LT;
    %   save( outFileName, 'output','LT','PAR', 'svmFileName','RSV','RFV' );
    POS = Data.Pos;
    fprintf( 'save output to %s\n',outFileName );
    inventory = {'output', 'POS'};
    %save( outFileName, '-V7', 'inventory', 'output','LT','POS', 'Train', 'svmFileName','RSV','RFV','sg_ver','PAR' );
    save( outFileName, '-V7', 'inventory', 'output','LT','POS', 'svmFileName','RSV','RFV','sg_ver','PAR' );
  else
    pos = Data.pos;
    fprintf( 'save output to %s\n',outFileName );
    %save( outFileName, '-V7', 'output', 'pos', 'Train', 'svmFileName','sg_ver','PAR' );
    if isfield(PAR.Signal, 'export_settings') && isfield(PAR.Signal.export_settings, 'resolution') && PAR.Signal.export_settings.resolution>1
      pos_short = zeros(1,floor(length(pos)/2));
      output_short = zeros(1,floor(length(pos)/2));
      % reduce number of predictions (take the better one out of two) 
      for j=2:2:length(pos)
        if output(j-1)>output(j)
          output_short(j/2) = output(j-1);
          pos_short(j/2) = pos(j-1);
        else
          output_short(j/2) = output(j);
          pos_short(j/2) = pos(j);
        end
      end
      output = output_short;
      pos = pos_short;
    end
    
    inventory = {'output', 'pos'};
    assert(isequal(size(pos),size(output)) || (isempty(pos) && isempty(output)));
    save( outFileName, '-V7', 'inventory', 'output', 'pos', 'svmFileName','sg_ver','PAR' );
  end
      
      
return

function [ alphas, b, betas, PAR1 ] = svm_train(Data);

P = Data.P;

method  = P.Signal.method;
fn_SVMs = P.fn_SVMs;
pred_from_seq = P.pred_from_seq;
use_preproc = P.use_preproc;


%------------------------------------------------------------------------------%
% SVM already trained?
%------------------------------------------------------------------------------%

%if( ~isfield(PAR,'conservWeighting') )
method.conservWeighting = 0;
%end;
if( ~exist('testDataPar','var') )
  testDataPar = [];
end;


%------------------------------------------------------------------------------%
% train SVM
%------------------------------------------------------------------------------%


if( method.conservWeighting == 0 )
  Data.W = [];
else
  Data.W = res.W;
  assert( ~isempty(W) );
end;

% keyboard
init_svm_and_kernels(method,pred_from_seq);


if isequal(method.feature_type,'double') && method.normalize_features
  idx_rm = find(std(Data.XT')<1e-8);
  Data.XT(idx_rm,:)  = [];
  mean_tr = mean(Data.XT');
  Data.XT = Data.XT - repmat(mean_tr,size(Data.XT,2),1)';
  std_tr = std(Data.XT');
  Data.XT = Data.XT./repmat(std_tr,size(Data.XT,2),1)';
end

setFeatures(method, Data,'TRAIN',pred_from_seq,use_preproc);

m = length( Data.LT );
% PAR1 = P;
% PAR1.POS = Data.Pos;
% PAR1.lenTrain = size( Data.XT, 1 );

sg_ver=sg('get_version');

% === train
nofKernels = length(method.kernels);
t0 = cputime;
sg('set_kernel_optimization_type', 'FASTBUTMEMHUNGRY' );
%sg( 'set_kernel_optimization_type', 'SLOWBUTMEMEFFICIENT' );
if( method.use_mkl )
  sg('use_mkl', true );
  sg('mkl_parameters', 1e-3, 0 );
  betas = sg( 'get_subkernel_weights' );
  assert( size(betas) == [nofKernels,1] );
end;
sg('svm_train' );
fprintf( '\n' );
t = cputime - t0;

%------------------------------------------------------------------------------%
% get alphas and betas
%------------------------------------------------------------------------------%

% === alphas
[ b, alphas ] = sg( 'get_svm' );


% === betas (kernel weights)
if(method.use_mkl)
  betas = sg( 'get_subkernel_weights' );
  assert( size(betas) == [nofKernels,1] );
  betas
else
  betas = ones( nofKernels, 1 );
end;


%------------------------------------------------------------------------------%
% save SVM
%------------------------------------------------------------------------------%
Train.alphas = alphas;
Train.b = b;
Train.PAR = P;
Train.POS = Data.Pos;
sup_vec_idx = alphas(:,2)+1;
Train.XT = Data.XT(:,sup_vec_idx);
Train.LT = Data.LT(sup_vec_idx);

w = zeros(size(Train.XT,1),1);
for a=1:length(Train.LT)
  w = w+Train.alphas(a,1).*Train.XT(:,a);
end
Train.w = w;

Train.POS.pos = Data.Pos.pos(sup_vec_idx);
Train.POS.region_id = Data.Pos.region_id(sup_vec_idx);

if isequal(method.feature_type,'double') && method.normalize_features
  Train.idx_rm = idx_rm
  Train.mean = mean_tr;
  Train.std = std_tr;
end

clear P Data
sg( 'clean_features', 'TRAIN' );
sg( 'clean_features', 'TEST' );

Train.time=t;
Train.sg_ver = sg_ver ; 
fprintf('save: %s',fn_SVMs)
save(fn_SVMs, '-V7', 'Train')






return

function Content = cds_exon

name = 'cds_exon';

Content = content_template(name);

Content.wordlen = {[3],[4],[5]};
Content.stepping = 1; 
Content.offset = 0;
return

function Content=content_template(name)

Content.name = name;
Content.consensus = [];
Content.lwin_big = 0;
Content.rwin_big = 0;
%Content.order = [3 4 5];
Content.label_fct = 'get_cand_contents';
Content.load_fct = 'load_content_data';

Content.filter_label.train.USE_ALL = 1;
Content.filter_label.test.USE_ALL = 1;
Content.filter_label.eval.USE_ALL = 1;
Content.Conf_names = {} ;
Content.C = [1]; 

Content.wordlen = { };
Content.stepping = 1;
Content.offset = 0;

Content.export_settings.conf_cum_thresh = 0.5 ;
Content.export_settings.resolution = 1 ;


%PAR.Contents.type_descrs = { 'intergenic', 'utr_exon', 'cds_exon','intron', 'intercistronic' };
%PAR.Contents.type_abbrs = { 'ige' 'utr' 'cex' 'ino' 'ics' };
%PAR.Contents.num_types = length(PAR.Contents.type_descrs); 



return

function Content = frame0(info)

name = 'frame0';

Content = content_template(name);
Content.wordlen = {[3],[4],[5],[6]};
Content.stepping = 3;

Content.offset = mod(3-mod(Content.wordlen{1},3),3);
 

return

function Content = intergenic

name = 'intergenic';

Content = content_template(name);

Content.wordlen = {[3],[4],[5]};
Content.stepping = 1; 
Content.offset = 0;
return

function Content=intron

name = 'intron';

Content = content_template(name);

Content.wordlen = {[3],[4],[5]};
Content.stepping = 1; 
Content.offset = 0;
return

function Content = polya_tail

name = 'polya_tail';

Content = content_template(name);

Content.wordlen = { [3:5]};

return

function Content = utr3exon

name = 'utr3exon';

Content = content_template(name);

Content.wordlen = {[3],[4],[5]};
Content.stepping = 1; 
Content.offset = 0;
return

function Content=utr5exon

name = 'utr5exon';

Content = content_template(name);

Content.wordlen = {[3],[4],[5],[6]};
Content.stepping = 1; 
Content.offset = 0;
return

function PAR=exp_galaxy_A_thaliana(PAR)

if nargin<1 
  PAR=init_PAR;
end

PAR.date=date;
PAR.exp_name='';
PAR.dir_name ='';

PAR.organism.name = 'A_thaliana';  

PAR.SETs.num_partitions=5;
PAR.SETs.num_splits=5;
%PAR.SETs.which_pred_svm = 1;

PAR.tasks.modelsel_on = 1;
PAR.tasks.learn_plifs_on =1;
PAR.tasks.pred_from_seq=1;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;
PAR.tasks.prepare_labels=0;
PAR.tasks.pred_on_genome = 0 ;
PAR.tasks.train_on = 1;
PAR.tasks.run_locally=1;

PAR.FN.input_dir='' ;
PAR.FN.output_dir='' ;

PAR.RPROC.collect_jobs = 1;
PAR.RPROC.options.ncpus = 1;
PAR.RPROC.exm_per_batch = 1;
PAR.RPROC.time_req = 3600;
PAR.RPROC.MEMREQ = 3000;

return

function PAR=exp_galaxy_C_elegans(PAR)

if nargin<1 
  PAR=init_PAR;
end

PAR.date=date;
PAR.exp_name='';
PAR.dir_name ='';

PAR.organism.name = 'C_elegans'; 

PAR.SETs.num_partitions=1;
PAR.SETs.num_splits=1;
PAR.SETs.which_pred_svm = 1;

PAR.tasks.modelsel_on = 0;
PAR.tasks.learn_plifs_on =0;
PAR.tasks.pred_from_seq=1;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;
PAR.tasks.prepare_labels=0;
PAR.tasks.pred_on_genome = 1;
PAR.tasks.train_on = 0;
PAR.tasks.run_locally=1;

PAR.FN.input_dir='/tmp';
PAR.FN.output_dir='/tmp';

PAR.RPROC.collect_jobs = 1;
PAR.RPROC.options.ncpus = 1;
PAR.RPROC.exm_per_batch = 1;
PAR.RPROC.time_req = 3600;
PAR.RPROC.MEMREQ = 3000;


return

function PAR=exp_galaxy_D_melanogaster(PAR)

if nargin<1 
  PAR=init_PAR;
end

PAR.date=date;
PAR.exp_name='';
PAR.dir_name ='';

PAR.organism.name = 'D_melanogaster';  

PAR.SETs.num_partitions=1;
PAR.SETs.num_splits=1;
PAR.SETs.which_pred_svm = 1;

PAR.tasks.modelsel_on = 0;
PAR.tasks.learn_plifs_on =0;
PAR.tasks.pred_from_seq=1;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;
PAR.tasks.prepare_labels=0;
PAR.tasks.pred_on_genome = 1;
PAR.tasks.train_on = 0;
PAR.tasks.run_locally=1;

PAR.FN.input_dir='/tmp';
PAR.FN.output_dir='/tmp';

PAR.RPROC.collect_jobs = 1;
PAR.RPROC.options.ncpus = 1;
PAR.RPROC.exm_per_batch = 1;
PAR.RPROC.time_req = 3600;
PAR.RPROC.MEMREQ = 3000;


return

function PAR=exp_galaxy_D_rerio(PAR)

if nargin<1 
  PAR=init_PAR;
end

PAR.date=date;
PAR.exp_name='';
PAR.dir_name ='';

PAR.organism.name = 'D_rerio';  

PAR.SETs.num_partitions=1;
PAR.SETs.num_splits=1;
PAR.SETs.which_pred_svm = 1;

PAR.tasks.modelsel_on = 0;
PAR.tasks.learn_plifs_on =0;
PAR.tasks.pred_from_seq=1;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;
PAR.tasks.prepare_labels=0;
PAR.tasks.pred_on_genome = 1;
PAR.tasks.train_on = 0;
PAR.tasks.run_locally=1;

PAR.FN.input_dir='/tmp';
PAR.FN.output_dir='/tmp';

PAR.RPROC.collect_jobs = 1;
PAR.RPROC.options.ncpus = 1;
PAR.RPROC.exm_per_batch = 1;
PAR.RPROC.time_req = 3600;
PAR.RPROC.MEMREQ = 3000;


return

function PAR=exp_galaxy(PAR_config)
% PAR=exp_galaxy(PAR_config)

PAR.SETs.num_partitions=5 ;
PAR.SETs.num_splits=5 ;

PAR.FN.input_dir=PAR_config.dir ;
PAR.FN.output_dir=PAR_config.dir;
PAR.FN.genome_dir=PAR_config.dir ;
PAR.FN.input.fn_genome_config = PAR_config.genome_config ;

% preparing RPROC settings
PAR.RPROC.MEMREQ = 6000 ;
PAR.RPROC.collect_jobs = 0 ;
PAR.RPROC.exm_per_batch = 1 ;
if 0
PAR.RPROC.options.addpaths = {'~/svn/projects/genefinding/sensors',
                    '~/svn/projects/genefinding/data_processing_sensors',
                    '~/svn/projects/genefinding/contents',
                    '~/svn/projects/genefinding/galaxy/tools/',
                    '~/svn/projects/genefinding/utils',
                    '~/svn/projects/genefinding/shogun',
                    '~/svn/projects/genefinding/data_processing_lsl',
                    '~/svn/tools/genomes',
                    '~/svn/tools/utils'} ;

end

PAR.RPROC.options.addpaths = {} ;

signals = {'acc','don','tis','cdsStop','tss','cleave'};
for i=1:length(signals)
  PAR.label_source.(signals{i}).from_fn_candsites = 1 ;
  PAR.Signals.(signals{i}).filter_label.train.USE_ALL = 1;
  PAR.Signals.(signals{i}).filter_label.test.USE_ALL = 1;
  PAR.Signals.(signals{i}).filter_label.eval.USE_ALL = 1;
  PAR.Signals.(signals{i}).Conf_names = {} ;
  PAR.Signals.(signals{i}).method.threads = 1;
end


if 0,
  PAR.Signals.acc.order = 10 ;
  PAR.Signals.acc.shift = 0 ;

  PAR.Signals.don.order = 10 ;
  PAR.Signals.don.shift = 0 ;
  
  PAR.Signals.tis.order =  {[],[5],[]};
  PAR.Signals.tis.shift =  {[],[0],[]};
  
  PAR.Signals.cdsStop.order = {[],[5],[]};
  PAR.Signals.cdsStop.shift = {[],[0],[]};
  
  PAR.Signals.tss.order = {[],[5],[]};
  PAR.Signals.tss.shift = {[],[0],[]};
  
  PAR.Signals.cleave.order = {[],[5],[]};
  PAR.Signals.cleave.shift = {[],[0],[]};

end


contents = {'intergenic','utr5exon','cds_exon','utr3exon','polya_tail','intron','frame0'};

for i=1:length(contents)
  PAR.label_source.(contents{i}).from_fn_candsites = 1 ;
  PAR.Contents.(contents{i}).filter_label.train.USE_ALL = 1;
  PAR.Contents.(contents{i}).filter_label.test.USE_ALL = 1;
  PAR.Contents.(contents{i}).filter_label.eval.USE_ALL = 1;
  PAR.Contents.(contents{i}).Conf_names = {} ;
  PAR.Contents.(contents{i}).method.threads = 1;
end

PAR.LSL.method.use.signals.polya = 0 ;
PAR.LSL.method.exm_per_solve = 100 ;

PAR.regions.max_length = 30000 ;


if 1,

%----- REGULARIZATION
method=PAR.LSL.method ;

%C = 1 ;
C = 1e-3;
%%% linear regularization
method.par_ms.C_regul.lengths = C ;        % linear smoothness for len_hist
method.par_ms.C_regul.signals = C ;        % for sig_hist
method.par_ms.C_regul.contents = C ;       % for content_hist
method.par_ms.C_regul.transitions = C ;    % for

method.par_ms.C_regul.tiling = C ;  
method.par_ms.C_regul.rna_seq = C ;  


%%% quadratic regularization
C_sq = 1e-3;

method.par_ms.C_regul.transitions_sq = C_sq;
method.par_ms.C_regul.plif_ys_sq = C_sq*0.1 ;  % small_plif_ys_penalty
method.par_ms.C_regul.smoothness_sq = C_sq ;

PAR.LSL.method=method ;

end

return

function FN=init_FN(signal_names,content_names)
 
  
if nargin<1
  signal_names = {'acc','don','tis','cdsStop','tss','cleave','polya','transacc'};
end
if nargin<2
  content_names = {'intergenic','utr5exon','cds_exon','utr3exon','polya_tail','intron','frame0'};
end
 
%--------------------------------------------------------------------------
% Directories
% -------------------------------------------------------------------------------------

FN.genome_dir= '/fml/ag-raetsch/share/databases/genomes/';
FN.input_dir= '/fml/ag-raetsch/share/projects/genefinding/';
FN.output_dir = '/fml/ag-raetsch/share/projects/genefinding/';

FN.dir_name = '';
FN.exp_name = '';
FN.lsl_exp_name = '';
%--------------------------------------------------------------------------
% INPUT FILE NAMES
% -------------------------------------------------------------------------------------

FN.genome.fn_confirmed_sequences = 'confirmed_sequences.mat';
FN.genome.fn_genome_config_orig  = 'genome_uncondensed.config';
FN.genome.fn_genome_config  = 'genome.config';
FN.genome.fn_map = 'condense_map.txt';
FN.genome.fn_genes_anno = 'annotated_genes.mat';
FN.genome.fn_genes_merged = '';
FN.genome.fn_genes_conf = 'confirmed_sequences.mat';
FN.genome.fn_name_to_contig_mapping = 'sequences/contig_orig_new_mapping';


FN.input.fn_genome_config = 'genome.config';
FN.input.fn_trainings_regions = 'training_regions.mat';
FN.input.fn_test_regions = 'test_regions.mat';
FN.input.fn_genes_anno_orig ='annotated_genes.mat';
FN.input.fn_genes_conf_orig = 'confirmed_sequences.mat';
FN.input.fn_ESTs_matches = {'est_matches.mat'};

%--------------------------------------------------------------------------
% OUTPUT FILE NAMES
% -------------------------------------------------------------------------------------

FN.output.fn_genes_anno ='anno_genes.mat';
FN.output.fn_genes_conf = 'conf_genes.mat';
FN.output.fn_genes_merged = 'genes_all.mat';
FN.output.fn_genes_signals = 'genes_signals.mat';
FN.output.fn_genes_contents = 'annotated_genes.mat';
FN.output.fn_genes_lsl = 'annotated_genes.mat';
FN.output.fn_ESTs_matches = [];
FN.output.fn_trivial_regions = 'trivial_regions.mat';
FN.output.fn_blocks = 'blocks.mat';
FN.output.fn_training_blocks ='training_blocks.mat';
FN.output.fn_test_blocks = 'test_blocks.mat';

%--------------------------------------------------------------------------
% SIGNAL FILE NAMES
% -------------------------------------------------------------------------------------

for j=1:length(signal_names)
  sig_name = signal_names{j};
  FN.input_sig.(sig_name).fn_candsites = 'cands/' ;
  FN.input_sig.(sig_name).fn_examples = 'examples';
  FN.input_sig.(sig_name).fn_pos = 'pos/';
  FN.input_sig.(sig_name).fn_filter_settings= 'filter_settings.mat';
  FN.input_sig.(sig_name).fn_example_statistics = 'example_statistics.mat';

  FN.output_sig.(sig_name).fn_models='MS/models';
  FN.output_sig.(sig_name).fn_SVMs='MS/';
  FN.output_sig.(sig_name).fn_pred='pred/';
  FN.output_sig.(sig_name).fn_motifs='MS/motifs';
end


if any(strcmp(signal_names,'tss'))
  FN.input_sig.tss.fn_est_cluster = 'tss_est_clusters.mat';
  FN.input_sig.tss.fn_fulllength  = 'tss_fulllength.mat';
  FN.input_sig.tss.fn_db = 'tss_db.mat' ;
end

if any(strcmp(signal_names,'cleave'))
  FN.input_sig.cleave.fn_est_cluster = 'cleave_est_clusters.mat';
  FN.input_sig.cleave.fn_fulllength = 'cleave_fulllength.mat';
  FN.input_sig.cleave.fn_db  = 'cleave_db.mat';
  FN.input_sig.cleave.fn_anno  = 'cleave_anno.mat';
end
if any(strcmp(signal_names,'polya'))
  FN.input_sig.polya.fn_polya = 'truesites_and_consensus.mat';
  FN.input_sig.polya.fn_log = 'log';
end
if any(strcmp(signal_names,'tis'))
  FN.input_sig.tis.fn_anno = 'tis_anno.mat';
end
if any(strcmp(signal_names,'cdsStop'))
  FN.input_sig.cdsStop.fn_anno = 'cdsStop_anno.mat';
end

if any(strcmp(signal_names,'transacc'))
  FN.input_sig.transacc.fn_anno_SL1 = 'transacc_anno_SL1' ;
  FN.input_sig.transacc.fn_anno_SL2 = 'transacc_anno_SL2' ;
  FN.input_sig.transacc.fn_pred = 'transacc_pred';
  FN.input_sig.transacc.fn_db = []; % only for C_elegans
end

%--------------------------------------------------------------------------
% CONTENT FILE NAMES
% -------------------------------------------------------------------------------------

for j=1:length(content_names)
  content_name = content_names{j};
  FN.input_cont.(content_name) = struct;
  FN.input_cont.(content_name).fn_candsites = 'cands/' ;
  FN.input_cont.(content_name).fn_examples = 'examples';
  FN.input_cont.(content_name).fn_pos = 'pos/';
  FN.input_cont.(content_name).fn_filter_settings= 'filter_settings.mat';
  FN.input_cont.(content_name).fn_example_statistics = 'example_statistics.mat';

  FN.output_cont.(content_name).fn_models = 'MS/models';
  FN.output_cont.(content_name).fn_SVMs = 'MS/';
  FN.output_cont.(content_name).fn_pred = 'pred/';
  FN.output_cont.(content_name).fn_motifs = 'MS/motifs';
end


%--------------------------------------------------------------------------
% LSL FILE NAMES
% -------------------------------------------------------------------------------------

FN.input_lsl.fn_test_blocks = 'data/test_blocks';
FN.input_lsl.fn_training_blocks =  'data/training_blocks.mat';
FN.input_lsl.fn_training_split =  'data/blocks_split.mat';

FN.input_lsl.fn_val_blocks =  'data/validation_blocks.mat';
FN.input_lsl.fn_blocks_all =  'data/blocks_all.mat';
FN.input_lsl.fn_boundary_model = 'data/boundary_model.mat';


FN.output_lsl.fn_init =  'train/init.mat';
FN.output_lsl.fn_train_block_preds = 'train/single_block_predictions/';
FN.output_lsl.fn_train = 'train/train';
FN.output_lsl.fn_pred = 'predictions/';
FN.output_lsl.fn_blocks_pred = 'predictions/blocks_pred';
FN.output_lsl.fn_pred_genome = 'genomewide_predictions/';
FN.output_lsl.fn_log = 'log_files/log_';



return

function PAR=init_PAR(signal_names,content_names);
 
  
% -------------------------------------------------------------------------------------
% Experiments
% -------------------------------------------------------------------------------------
if nargin<1
  signal_names = {'acc','don','tis','cdsStop','tss','cleave','polya','transacc'};
end
if nargin<2
  content_names = {'intergenic','utr5exon','cds_exon','utr3exon','polya_tail','intron','frame0'};
end




PAR.date = date;
PAR.user = whoami;
  
PAR.organism.clade = [];
PAR.organism.name = '';
PAR.organism.full_name = [];
PAR.organism.release = [];
PAR.organism.version = [];
PAR.organism.genebuilt = [];

% -------------------------------------------------------------------------------------
% info regarding original gene structure
% -------------------------------------------------------------------------------------


info_genes.transcript.anno = -1;
info_genes.transcript.est = 0;
info_genes.transcript.cDNA = 1;
info_genes.transcript.fulllength = 2;

info_genes.transcript_status.confirmed = 1;
info_genes.transcript_status.partially_confirmed = 0;
info_genes.transcript_status.predicted = -1;


info_genes.pseudo.dummy = [];

% -------------------------------------------------------------------------------------
% info regarding label source
% -------------------------------------------------------------------------------------

label_source.use_anno = [];
label_source.use_confirmed = [];
for j=1:length(signal_names)
  signal_name = signal_names{j};
  label_source.(signal_name).anno = [];
  label_source.(signal_name).est = [];
  label_source.(signal_name).cDNA = [];
  label_source.(signal_name).fulllength = [];
  label_source.(signal_name).db = [];
  label_source.(signal_name).from_fn_candsites = [];
  
  info_genes.(signal_name).anno = -1;
  info_genes.(signal_name).est = 0;
  info_genes.(signal_name).cDNA = 1;
  info_genes.(signal_name).fulllength = 2;
  info_genes.(signal_name).db = 3;
  info_genes.(signal_name).from_fn_candsites = 4;
end
for j=1:length(content_names)
  content_name = content_names{j};
  label_source.(content_name).from_fn_candsites = [];
end

label_source.tis.maxorf = [];
info_genes.tis.maxorf = 5;
label_source.cdsStop.maxorf = [];
info_genes.cdsStop.maxorf = 5;

label_source.transacc.anno_SL1 = [];
label_source.transacc.anno_SL2 = [];
label_source.transacc.pred = [];
info_genes.transacc.anno_SL1 = 5;
info_genes.transacc.anno_SL2 = 6;
info_genes.transacc.pred = 7;

label_source.polya.from_cleave = [];
info_genes.polya.from_cleave = 5;


PAR.label_source = label_source;
PAR.info_genes = info_genes;


PAR.Signal_name = [];
for j=1:length(signal_names)
  signal_name = signal_names{j};
  PAR.Signals.(signal_name) = struct;
end

cnt=0;
PAR.Content_name = [];
for j=1:length(content_names)
  PAR.Contents.(content_names{j}) = struct;
  PAR.Contents.(content_names{j}).name = content_names{j};
  PAR.Contents.(content_names{j}).type = cnt;
  cnt=cnt+1;
end

PAR.LSL.method = struct;

% -------------------------------------------------------------------------------------
% memory requirements
% -------------------------------------------------------------------------------------

PAR.RPROC.collect_jobs = 0 ; 
PAR.RPROC.options.ncpus = 1 ; 
PAR.RPROC.options.waitonfull = [];
PAR.RPROC.options.maxjobs = [] ;
PAR.RPROC.options.immediately = 0;
PAR.RPROC.options.immediately_bg = 0;
PAR.RPROC.options.addpaths = [];
PAR.RPROC.options.rmpaths = [];

PAR.RPROC.MEMREQ = [];
PAR.RPROC.time_req = [];
PAR.RPROC.exm_per_batch = 1 ;


% -------------------------------------------------------------------------------------
% SPLITS AND PARTITIONS
% -------------------------------------------------------------------------------------

PAR.SETs.num_splits = 5;
PAR.SETs.num_partitions = 5;
PAR.SETs.train_subsample_neg = 1;
PAR.SETs.test_subsample_neg = 1;
PAR.SETs.train_subsample_pos = 1;
PAR.SETs.test_subsample_pos = 1;
%for lsl
PAR.SETs.num_train = [] ; 
PAR.SETs.ignore_incomplete = 1 ; 


% -------------------------------------------------------------------------------------
% original data processing stuff
% -------------------------------------------------------------------------------------

PAR.regions.max_length = 1000000;
PAR.regions.min_genes_per_region = 1;
PAR.regions.offset_test = 20000;
PAR.regions.offset_train = 1000;
PAR.regions.minCutDist = 250;
PAR.regions.cluster_paralogs = 0;
PAR.regions.merge_confirmed = 0;

% -------------------------------------------------------------------------------------
% TASKS
% -------------------------------------------------------------------------------------

PAR.tasks.signals.prepare_labels = 0;
PAR.tasks.signals.train = 0;
PAR.tasks.signals.modelsel = 0;
PAR.tasks.signals.learn_plifs = 0;
PAR.tasks.signals.get_train_error = 0;

PAR.tasks.signals.pred_on_genome = 0; 
PAR.tasks.signals.verbose = 0; 

PAR.tasks.signals.run_locally = 0;

% -------------------------------------------------------------------------------------
% transfer from other organisms
% -------------------------------------------------------------------------------------

PAR.Source_organisms.names = {};
PAR.Source_organisms.dir_names = {};
PAR.Source_organisms.exp_names = {};


% -------------------------------------------------------------------------------------
% FN
% -------------------------------------------------------------------------------------
PAR.FN = struct;


return

function lengths_range = default_lengths

lengths_range.intergenic = [0, 21000] ;
%lengths_range.intergenictrans = [0, 21000] ;
lengths_range.utr5exon   = [0, 8000] ;
% lengths_range.transexon  = [1, 1000] ;
lengths_range.single_cds_exon = [100, 8000] ; %%%100, from don => acc+stop
lengths_range.first_cds_exon  = [3, 8000] ;
lengths_range.middle_cds_exon   = [3, 8000] ;
lengths_range.last_cds_exon   = [1, 8000] ;
lengths_range.utr3exon    = [1, 8000] ;
lengths_range.polya_tail  = [0, 250] ;
% lengths_range.intercistronic = [1, 5000] ;
lengths_range.intergenic_long = [10500-50, 10500+50] ; % due to irregular spacing 
lengths_range.intron      = [30, 21000] ;%%%30, from don => acc+stop

return

function LSL = lsl_template
  

method.name = 'lsl' ;
method.model_fct = 'make_model';
method.train_fct = 'lsl_train';
method.predict_fct = 'lsl_predict';

method.dt = datestr(now,'dd-mm-yyyy');
method.user = 'GS';
method.plif_bins   = 20 ;   % number of plif bins

%%%

%%Signals
use.signals.trans = 0 ;
use.signals.transacc = 0 ;
use.signals.tss = 1;
use.signals.polya = 1;
use.signals.cleave =  1;

%%States
use.states.SL1 = 0 ;
use.states.SL2 = 0;

%%transitions
use.transitions.allow_missing_tss = 0;
use.transitions.one_gene = 0 ; % force exactly one gene
use.transitions.use_multi = 1 ; % allow zero or more than one gene 

%%loss
use.loss.orf_only = 0;
use.model.one_gene = 0;
use.model.allow_missing_tss = 0;
use.model.use_multi = 1;

%%Segments & lengths
use.segments.transexon = 0 ;
use.segments.intercistronic = 0 ;
use.segments.intergenictrans = 0;
use.segments.polya_tail = 1;

%%Content 

use.contents.frame = 1;
use.contents.pre_comp = 1;

%%additional features
use.addfeatures.conservation = 0 ;
use.addfeatures.ests = 0 ;
use.addfeatures.proteins = 0 ;
use.addfeatures.tiling = 0 ;
use.addfeatures.rna_seq = 0 ;

% keyboard
method.use = use;



%----- REGULARIZATION
%C = 1 ;
C = 1e-1;
%%% linear regularization
method.par_ms.C_regul.lengths = C ;        % linear smoothness for len_hist
method.par_ms.C_regul.signals = C ;        % for sig_hist
method.par_ms.C_regul.contents = C ;       % for content_hist
method.par_ms.C_regul.transitions = C ;    % for

method.par_ms.C_regul.tiling = C ;  
method.par_ms.C_regul.rna_seq = C ;  


%%% quadratic regularization
C_sq = 1e-1;

method.par_ms.C_regul.transitions_sq = C_sq;
method.par_ms.C_regul.plif_ys_sq = C_sq*0.1 ;  % small_plif_ys_penalty
method.par_ms.C_regul.smoothness_sq = C_sq ;


%----------- for predictions only
method.par_ms.cleave_offset = 6;
%-----------

method.solver = 'dual';
method.ignore_init_file = 1;

method.opt_step = 1000 ;
method.max_neg = 2 ;
method.margin = 1 ;

method.INF = 1e20;
method.max_num_iterations = 100;

method.exm_per_solve = 400 ;
% method.exm_per_batch = 5 ;

%method.cleave_offset = 6 ;
% -------
% -------
LSL.method = method;


return

function method = method_template

% Signals = method_template(Signal,signal_name)
% Signals is a struct with a field for each Signal. In this function only
% the field Signal.(signal_name).method is added. Default values are set.

method.name = [];
method.train_fct = [];
method.predict_fct = [];

% PAR.method.svm_name = 'GPBTSVM';PAR.method.qpsize=500;
% use conservation information
% method.use_cons = 0;
% load training examples only once and calculate all models in one job
% method.submit_batch=0;

method.plif.bins = 50;
method.plif.min_incr = 1e-6;

method.use_cons = 0;
method.feature_type = [];
%assert(max(abs([PAR.Signals.(PAR.Signal_name).lwin{:}]))<PAR.Signals.(PAR.Signal_name).lwin_big)

%assert(max(abs([PAR.Signals.(PAR.Signal_name).lwin{:}]))<PAR.Signals.(PAR.Signal_name).rwin_big)




return

function method = SVM_COMBINED(Signal)

% Signals = SVM_COMBINED(Signals,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. In this function only
%          the field (signal_name) is modified.
%          Particuarly a field method is added.
%        - signal_name can be something in {'acc','don','tss','cleave','polya','tis','cdsStop'};
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: SVM_COMBINMED -> SVM -> method_template


method = SVM(Signal) ;

nofKernels = length(Signal.kernel_name);

assert(length(Signal.lwin)==nofKernels)
assert(length(Signal.rwin)==nofKernels)


for k=1:nofKernels
  for n = Signal.kernel_par_names 
    method.kernels{k}.(n{1}) = Signal.(n{1}){k};
  end
end
if isfield(Signal,'subkernel_weights')
  method.subkernel_weights = Signal.subkernel_weights;
end


return

function method = SVM_Cont(Content)

  
method = SVM(Content);
method.feature_type = 'char';
method.center = [];
method.kernels{1}.name = 'LINEAR';
method.genomewide_predict_fct = 'svm_content_predict';

for k=1:length(Content.wordlen)
  method.kernels{k}.wordlen = Content.wordlen{k};
  method.kernels{k}.stepping = Content.stepping ;
  method.kernels{k}.offset = Content.offset ;
end



return

function method = SVM(Signal)
  
method = method_template;

% set parameters, model selection 

method.name = 'SVM';
method.train_fct = 'svm_train';
method.predict_fct = 'svm_predict';
method.kernels = {[]};
method.poim_maxOrder = 8;
method.svm_name = 'LIGHT';
method.qpsize = 50;
method.threads = 16;
method.center = Signal.lwin_big;
method.par_ms.C = Signal.C;

%PAR.method.svm_name = 'GPBTSVM';PAR.method.qpsize=500;

% use multi kernel learning
method.use_mkl = 0;





return

function method = SVM_WD(Signal)

method = SVM(Signal);
method.feature_type = 'char';

kernel.name = 'WD';
kernel.order = Signal.order;
kernel.shift = 0 ;
kernel.lwin = Signal.lwin{1};
kernel.rwin = Signal.rwin{1};

method.kernels{1} = kernel; 


return

function method = SVM_WDS(Signal)

% Signals = SVM_WDS(Signals,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. In this function only
%          the field (signal_name) is modified.
%          Particuarly a field method is added.
%        - signal_name can be something in {'acc','don','tss','cleave','polya','tis','cdsStop'};
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: SVM_WDS -> SVM -> method_template

  
method = SVM(Signal) ;
method.feature_type = 'char';


kernel.name = 'WDS';
kernel.order = Signal.order;
kernel.shift = Signal.shift;
kernel.lwin =  Signal.lwin{1};
kernel.rwin = Signal.rwin{1};

method.kernels{1} = kernel; 

return

function PAR=A_thaliana(PAR)

PAR = organism_template(PAR);

PAR.organism.name = 'A_thaliana';

PAR.label_source.use_anno = 0;
PAR.label_source.use_confirmed = 1;

PAR.label_source.tss.est_clusters = 1;
PAR.label_source.tss.fulllength = 0;
PAR.label_source.tss.db = 0;

PAR.label_source.cleave.est_clusters = 1;
PAR.label_source.cleave.fulllength = 0;
PAR.label_source.cleave.db = 0;

PAR.label_source.cds.anno = 0;
PAR.label_source.cds.maxorf = 0;


PAR.regions.cluster_paralogs = 1;

return

function [organism label_source] = C_elegans(label_source_orig,signal_names)

[organism label_source] = Nematodes(label_source_orig,signal_names);

organism.name = 'C_elegans';
organism.full_name = 'Caenorhabditis_elegans';
organism.release = '';
organism.version = '';
organism.genebuilt = '';
return

function PAR = D_melanogaster(PAR)


PAR = organism_template(PAR);

PAR.organism.clade = 'insect';
PAR.organism.name = 'D_melanogaster';

PAR.label_source.use_anno = 0;
PAR.label_source.use_confirmed = 1;

PAR.label_source.cleave.est_clusters = 1;
PAR.label_source.cleave.fulllength = 1;
PAR.label_source.cleave.db = 0;

PAR.label_source.tss.est_clusters  = 1;
PAR.label_source.tss.fulllength   = 0;
PAR.label_source.tss.db = 0;

return

function PAR=D_rerio(PAR)
  
PAR = organism_template(PAR);

PAR.organism.name = 'D_rerio';

PAR.label_source.use_anno = 0;
PAR.label_source.use_confirmed = 1;

PAR.label_source.tss.est_clusters  = 1;
PAR.label_source.tss.fulllength   = 1;
PAR.label_source.tss.db = 0;

PAR.label_source.cleave.est_clusters = 1;
PAR.label_source.cleave.fulllength = 1;
PAR.label_source.cleave.db = 0;

return

function [organism label_source] = dummy_organism(label_source_orig,signal_names)

[label_source] = organism_template(label_source_orig,signal_names);
  
organism.clade = [];
organism.name = '';
organism.full_name = '';
organism.release = '';
organism.version = '';
organism.genebuilt = '';


return

function [label_source] = organism_template(label_source_orig,signal_names)


% PAR = set_label_source(PAR)
% 
% sets defaults for all label_sources (don't use anno, use EST confirmed only.)
% values that are already assigned will NOT be overriden.
  
  
%% default values for general use of annotation and EST confrimed sequences
label_source = label_source_orig;

if isempty(label_source.use_anno)
  label_source.use_anno = 0;
end
if isempty(label_source.use_confirmed)
  label_source.use_confirmed = 1;
end


%% default values for individual signals


for j=1:length(signal_names)
  signal_name = signal_names{j};
  if label_source.use_anno
    label_source.(signal_name).anno = 1;
  else
    label_source.(signal_name).anno = 0;
  end
  if label_source.use_confirmed & ~isequal(signal_name,'transacc')
    label_source.(signal_name).est_clusters = 1;
    label_source.(signal_name).cDNA = 1;
    label_source.(signal_name).fulllength = 1;
  elseif ~label_source.use_confirmed & ~isequal(signal_name,'transacc')
    label_source.(signal_name).est_clusters = 0;
    label_source.(signal_name).cDNA = 0;
    label_source.(signal_name).fulllength = 0;
  end
  label_source.(signal_name).db = 0;
  label_source.(signal_name).from_fn_candsites = 1 ;
end

label_source.tis.maxorf = 0;
label_source.cdsStop.maxorf = 0;

label_source.transacc.anno_SL1 = 0;
label_source.transacc.anno_SL2 = 0;
label_source.transacc.pred = 1;
label_source.transacc.db = 0;



return

function Signal = acceptor(label_source,signal_name)

% Signal = acceptor(label_source,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. In this function only
%          the field acc (transcc) is modified.  
%        - label_source is a struct that should contain a field
%          (signal_name). this is used in the called function splicesite_template.
%        - name can either be 'acc' or 'transacc'  
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: acceptor/donor -> splicesite_template -> signal_template
% (function acceptor is also called from transacc)
  
  
if nargin<3 
  signal_name = 'acc';
end

Signal = splicesites_template(label_source,signal_name);

%%%% SET ACCEPTOR STUFF

Signal.consensus = {'AG'};
Signal.offset = 2;

Signal.lwin = {[-60]} ;
Signal.rwin = {[80]} ;

Signal.order = [ 22];
Signal.shift = 0.5;
Signal.C = [3]; 

Signal.Conf_names = {Signal.Conf_names{:},'alt_3prime'};

Signals.(signal_name) = Signal;



return

function Signal = cdsStop(label_source)
  
% Signals = cdsStop(Signals,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field cdsStop is modified.
%        - label_source is a struct that should contain a field cdsStop. 
%          this is used in the called function signal_template.
%       
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tis/cdsStop -> translatends_template -> signal_template
   
signal_name = 'cdsStop';

Signal = translatends_template(label_source,signal_name);

%%%% SET cdsStop STUFF

Signal.consensus = {'TAA','TAG','TGA'};


Signal.kernel_name= {'SP','WDS','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};
Signal.lwin = {[-200] [-110] [0]} ; 
Signal.rwin = {[0] [10] [180]};
Signal.shift={[],[0],[]};
Signal.shift_const={[],[0],[]};
Signal.order={[],[20],[]};  
Signal.wordlen = {[2] [] [4]}; 
Signal.C = 2.8284; % logC=1.5



return

function Signal = cleave(label_source)

% Signals = cleave(Signals,signal_name,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field cleave is modified.
%        - label_source is a struct that should contain a field cleave. 
%          this is used in the called function signal_template.
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tss/cleave -> transcriptends_template -> signal_template

  
signal_name = 'cleave';

Signal = transcriptends_template(label_source,signal_name);

%%%% SET CLEAVE STUFF

Signal.lwin_big = 410;
Signal.rwin_big = 810;

Signal.kernel_name= {'SP','WDS','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};

Signal.lwin = {[-400], [-60], [0]};
Signal.rwin = {[100], [60], [800]};
Signal.wordlen = {[5], [], [3]}; 

Signal.order = {[],[18],[]};
Signal.shift = {[],[0.1],[]};
Signal.shift_const = {[],[0],[]};
Signal.C = 1; % log2(C)=0



return

function Signal = donor(label_source)

% Signals = donor(Signals,signal_name,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field don is modified.
%        - label_source is a struct that should contain a field don. 
%          this is used in the called function signal_template.
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: acceptor/donor -> splicesite_template -> signal_template

 
signal_name = 'don';

Signal = splicesites_template(label_source,signal_name);

%%%% SET DONOR STUFF

Signal.consensus = {'GC' 'GT'};
Signal.offset = 0;

Signal.lwin = {[-80]};
Signal.rwin = {[60]};

Signal.order = [22];
Signal.shift = 0.5;
Signal.C = [3]; 

Signal.Conf_names = {Signal.Conf_names{:},'alt_5prime'};




return

function Signal = polya(label_source,fn_polya)
  
% Signals = polya(Signals,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field polya is modified.
%        - label_source is a struct that should contain a field polya. 
%          this is used in the called function signal_template.
%       
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: polya -> signal_template

signal_name = 'polya' ; 
Signal = signal_template(label_source,signal_name);

%%%% SET POLYA STUFF

Signal.name = signal_name;
Signal.type = signal_name; 
Signal.label_fct = 'get_cand_others';
Signal.filter_fct = 'filter_cands';

Signal.offset = 0;

Signal.lwin_big = 310;
Signal.rwin_big = 310;

Signal.kernel_name= {'WDS','SP','SP','SP','SP','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};
Signal.subkernel_weights = [1 0.2 0.2 0.2 0.2 0.2];

Signal.lwin = {-300 -300 -149 1 151 -300} ;
Signal.rwin = {306 -150 0 150 306 306};
Signal.shift={[0 0.1 0.4],[],[],[],[],[]};
%Signal.shift={[0.1],[],[],[],[],[]};
Signal.shift_const={[0 10 20],[],[],[],[],[]};
%Signal.shift_const={[0],[],[],[],[],[]};
Signal.wordlen={[],[4],[4],[4],[4],[4]};
Signal.order={[18],[],[],[],[],[]};
Signal.C = [0.01 0.5 10];

Signal.consensus = {''};
if exist('fn_polya')
  Signal.consensus_fn = fn_polya;
  if fexist(Signal.consensus_fn)
    load(Signal.consensus_fn,'consensus')
    Signal.consensus = consensus;
  end
end

Signal.filter_label.train.use_intergenic=0;
Signal.filter_label.train.use_alt=0;
%Signal.filter_label.eval.use_intergenic=0;
%Signal.filter_label.eval.use_alt=0;
Signal.filter_label.test.use_intergenic=0;
Signal.filter_label.test.use_alt=0;

Signal.filter_label.train.conf_pos.anno = 0;
Signal.filter_label.train.conf_pos.from_cleave = [201 301 401];
Signal.filter_label.train.conf_pos.db = 0;
Signal.filter_label.train.conf_pos.fulllength = 0;
Signal.filter_label.train.onlywithpos = 0; 

%Signal.filter_label.eval.conf_pos.anno = 0;
%Signal.filter_label.eval.conf_pos.from_cleave = [201 301 401];
%Signal.filter_label.eval.conf_pos.db = 0;
%Signal.filter_label.eval.conf_pos.fulllength = 0;
%Signal.filter_label.eval.onlywithpos = 0; 

Signal.filter_label.test.conf_pos.anno = 0;
Signal.filter_label.test.conf_pos.from_cleave = [201 301 401];
Signal.filter_label.test.conf_pos.db = 0;
Signal.filter_label.test.conf_pos.fulllength = 0;
Signal.filter_label.test.onlywithpos = 0; 


Signal.Conf_names = {Signal.Conf_names{:},['has_' signal_name]};



Signal.label_source_par.polya.wl = 1;
Signal.label_source_par.polya.POSconstr =1;
Signal.label_source_par.polya.maxpcCov = 95;
Signal.label_source_par.polya.logdir = [];

return

function Signal = signal_template(label_source,signal_name)

% Signals = signal_template(Signal,label_source,signal_name)
% Signals is a struct with a field for each Signal. In this function only
% the field signal_name is added. Default values are set.


if nargin<2
  signal_name = 'dummy';
end

%%%%%

Signal.type = []; 
Signal.name = signal_name;
Signal.consensus = {[]};
Signal.offset = [];

Signal.lwin_big = [];
Signal.rwin_big = [];

Signal.label_fct = [];
Signal.filter_fct = 'filter_cands';
Signal.load_fct = 'load_data';
Signal.score_fct = [];

Signal.Conf_names = { 'altgenic','gene_id' };

Signal.sampling.downsample_neg = 1; % <=1 (for no downsampling: 1)  
Signal.sampling.mindist = 1; % >=1 (for all: 1)  
Signal.sampling.pos_neg_ratio = 1/10;

Signal.resolution   = 1; % for lsl

conf.USE_ALL = 0;
conf.conf_pos = struct;
conf.conf_pos_skip = struct;
conf.any_conf_pos = [];
conf.any_conf_pos_skip = [];
conf.conf_neg = struct;
conf.conf_neg_skip = struct;
conf.any_conf_neg = [];
conf.any_conf_neg_skip = [];
conf.use_intergenic=[];
conf.use_alt = [];
conf.use_label = 'label';

Signal.filter_label.train = conf ; 
Signal.filter_label.test = conf ;
Signal.filter_label.eval = conf ;

Signal.export_settings.conf_cum_thresh = 0 ;
Signal.export_settings.resolution = 1 ;

Signal.domain_adapt.name = 'target_only';
Signal.domain_adapt.Source_organisms = [];

Signal.method = struct;

label_source = label_source.(signal_name);
sources = fieldnames(label_source);
for i=1:length(sources)
  if label_source.(sources{i})
    Signal.Conf_names = {Signal.Conf_names{:},[sources{i} '_conf']};
    if isequal(signal_name,'acc')|isequal(signal_name,'don')
      Signal.Conf_names = {Signal.Conf_names{:},[sources{i} '_conf_skip']};
    end
  end
end




return

function Signal = splicesites_template(label_source,signal_name)

% Signals = splicesites_template(Signals,label_source,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field signal_name is added or
%          modified. 
%          Default values are set.
%        - label_source is a struct that should contain a field (signal_name). 
%          this is used in the called function signal_template.
%        - signal_name: 'don', 'acc','transacc'
%  
% OUTPUT Signals with modified field signal_name; 
%  
% HIERACHY: acceptor/donor -> splicesite_template -> signal_template
%
% see also transcriptends_template/translat_template
  
  
INF=1e6;

Signal = signal_template(label_source,signal_name);

%%%% SET SPLICE SITE STUFF

Signal.type = 'splicesites'; 
Signal.label_fct = 'get_cand_splicesites';

Signal.lwin_big = 199;
Signal.rwin_big = 199;

Signal.filter_label.train.use_intergenic=0;
Signal.filter_label.train.use_alt=0;
Signal.filter_label.train.use_alt_neg=0;
Signal.filter_label.test.use_intergenic=0;
Signal.filter_label.test.use_alt=0;
Signal.filter_label.test.use_alt_neg=0;

Signal.filter_label.train.use_label = 'label';
Signal.filter_label.test.use_label = 'label';


Signal.filter_label.train.use_decoys = 1;
Signal.filter_label.test.use_decoys = 1;


Signal.filter_label.train.conf_pos.est_clusters = [0 INF];
Signal.filter_label.train.conf_pos.cDNA = [0 INF];
Signal.filter_label.train.conf_pos.fulllength = [0 INF];
Signal.filter_label.train.conf_pos.anno = [0:1] ;

Signal.filter_label.train.conf_pos_skip.est_clusters = [0 INF];
Signal.filter_label.train.conf_pos_skip.cDNA = [0 INF];
Signal.filter_label.train.conf_pos_skip.fulllength = [0 INF];
Signal.filter_label.train.conf_pos_skip.anno = [-inf] ;

Signal.filter_label.train.conf_neg.est_clusters = [0 INF];
Signal.filter_label.train.conf_neg.cDNA = [0 INF];
Signal.filter_label.train.conf_neg.fulllength = [0 INF];
Signal.filter_label.train.conf_neg.anno = [-1:1] ;

Signal.filter_label.train.conf_neg_skip.est_clusters = [0 INF];
Signal.filter_label.train.conf_neg_skip.cDNA = [0 INF];
Signal.filter_label.train.conf_neg_skip.fulllength = [0 INF];
Signal.filter_label.train.conf_neg_skip.anno = [-1:1] ;

Signal.filter_label.test = Signal.filter_label.train;
Signal.filter_label.eval = Signal.filter_label.train;

% Signal.save_confs = 1;
% Signal.save_alts = 1;


Signal.export_settings.conf_cum_thresh = 0.5 ;
Signal.export_settings.resolution = 1 ;


Signal.Conf_names = {Signal.Conf_names{:},'alt_exon','alt_intron'};





return

function Signal = tis(label_source)
  
% Signals = tis(Signals,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field tis is modified.
%        - label_source is a struct that should contain a field tis. 
%          this is used in the called function signal_template.
%       
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tis/cdsStop -> translatends_template -> signal_template

    
signal_name = 'tis';

Signal = translatends_template(label_source,signal_name);

%%%% SET TIS STUFF

Signal.consensus = {'ATG'};


Signal.kernel_name= {'SP','WDS','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};
Signal.lwin = {[-200] [-30] [0]} ; 
Signal.rwin = {[0] [110] [180]}; 
Signal.shift={[],[0],[]};
Signal.shift_const={[],[0],[]};
Signal.order={[],[24],[]};  
Signal.wordlen = {[3] [] [5]}; 
Signal.C = 1; % log2(C)=1.0










return

function Signal = transacceptor(label_source)

% Signals = transacceptor(Signals,label_source)
% INPUT  - Signals is a struct with a field for each Signal. In this function only
%          the field transcc is modified.  
%        - label_source is a struct that should contain a field
%          transacc. this is used in the called function splicesite_template.
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tansacceptor -> acceptor -> splicesite_template -> signal_template

INF = 1e6;  
signal_name = 'transacc';

Signal = acceptor(label_source,signal_name);

%%%% SET transacc STUFF

Signal.lwin = {[-190]} ;
Signal.rwin = {[80]};


Signal.filter_label.train.conf.anno_SL1 = 0;
Signal.filter_label.train.conf.anno_SL2 = [0 INF];
Signal.filter_label.train.conf.pred = [0 INF];

Signal.filter_label.eval.conf.anno_SL1 = 0;
Signal.filter_label.eval.conf.anno_SL2 = [0 INF];
Signal.filter_label.eval.conf.pred = [0 INF];


Signal.filter_label.test.conf.anno_SL1 = 0;
Signal.filter_label.test.conf.anno_SL2 = [0 INF];
Signal.filter_label.test.conf.pred = [0 INF];


Signal.filter_label.train.use_intergenic = 1;
Signal.filter_label.train.use_alt = 0;
Signal.filter_label.eval.use_intergenic = 1;
Signal.filter_label.eval.use_alt = 0;
Signal.filter_label.test.use_intergenic = 1;
Signal.filter_label.test.use_alt = 0;
 
Signal.filter_label.train.use_pred = 1;
Signal.filter_label.eval.use_pred = 1;
Signal.filter_label.test.use_pred = 1;

Signal.filter_label.train.only_acc = 1;
Signal.filter_label.eval.only_acc = 1;
Signal.filter_label.test.only_acc = 1;

Signal.filter_label.train.pos_dist = 20;
Signal.filter_label.eval.pos_dist = 20;
Signal.filter_label.test.pos_dist = 20;

Signal.export_settings.conf_cum_thresh = 0.05 ;
Signal.export_settings.resolution = 1 ;

Signal.Conf_names = {Signal.Conf_names{:},'isacc','has_tss','has_trans','dist'};


Signal.label_source_par.transacc.win_anno_SL1 = 50;
Signal.label_source_par.transacc.win_anno_SL2 = 50;
Signal.label_source_par.transacc.win_pred = 5;
Signal.label_source_par.transacc.threshold_pred = 0.5;


return

function Signal = transcriptends_template(label_source,signal_name)

% Signals = transcriptends_template(Signals,label_source,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field signal_name is added or
%          modified. 
%          Default values are set.
%        - label_source is a struct that should contain a field (signal_name). 
%          this is used in the called function signal_template.
%        - signal_name:'tss', 'cleave'
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tss/cleave -> transcriptends_template -> signal_template
%
% see also splicesite_template/translat_template
  
  
  
Signal = signal_template(label_source,signal_name);

%%%% SET TRANSCRIPT END STUFF

Signal.type = 'transcriptends'; 
Signal.label_fct = 'get_cand_others';


Signal.consensus = [];
Signal.offset = 0;

Signal.sampling.downsample_neg = 1/10; %% take randomly every 10th porsitions 
Signal.sampling.mindist = 10 ;


Signal.resolution   = 20; % for lsl

Signal.filter_label.train.conf_pos.anno = [0 1];
Signal.filter_label.train.conf_pos.est_clusters = [2:3];
Signal.filter_label.train.conf_pos.db = 0;
Signal.filter_label.train.conf_pos.fulllength = 0;
Signal.filter_label.train.onlywithpos=0; 

Signal.filter_label.train.use_intergenic=1;
Signal.filter_label.train.use_alt=0;

Signal.filter_label.test = Signal.filter_label.train;
Signal.filter_label.eval = Signal.filter_label.train;

Signal.export_settings.conf_cum_thresh = 0.01 ;
Signal.export_settings.resolution = 10 ;

Signal.Conf_names = {Signal.Conf_names{:},['has_' signal_name]};



return

function Signal = translatends_template(label_source,signal_name)

% Signals = translatends_template(Signals,label_source,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field signal_name is added or
%          modified. 
%          Default values are set.
%        - label_source is a struct that should contain a field (signal_name). 
%          this is used in the called function signal_template.
%        - signal_name:'tis', 'cdsStop'
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tis/cdsStop -> translatends_template -> signal_template
%
% see also splicesite_template/transcriptends_template
  
INF=1e6;
  
Signal = signal_template(label_source,signal_name);

%%%% SET TRANSLAT END STUFF

Signal.type = 'translatends' ;
Signal.label_fct = 'get_cand_others';

Signal.offset = 0;
Signal.lwin_big = 210;
Signal.rwin_big = 200;



Signal.filter_label.train.conf_pos.anno = [0 1] ;
Signal.filter_label.train.conf_pos.maxorf = INF;
Signal.filter_label.test.conf_pos.anno = [0 1] ;
Signal.filter_label.test.conf_pos.maxorf =INF;

Signal.filter_label.train.use_intergenic=0;
Signal.filter_label.train.use_alt=0;
Signal.filter_label.test.use_intergenic=0;
Signal.filter_label.test.use_alt=0;

Signal.filter_label.eval = Signal.filter_label.test;

% PAR.label_source.cds.par.max_nof_paths = 100;
% PAR.label_source.cds.par.nof_best_kept = 2;


return

function Signal = tss(label_source,has_transacc)
  
% Signals = tss(Signals,label_source,has_transacc)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field tss is modified.
%        - label_source is a struct that should contain a field tss. 
%          this is used in the called function signal_template.
%        - has_transacc: is per default set to zero, only 1 for nematodes. 
%       
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tss/cleave -> transcriptends_template -> signal_template

  
if nargin<3
  has_transacc=0; 
end
   
signal_name = 'tss';

Signal = transcriptends_template(label_source,signal_name);

%%%% SET TSS STUFF


Signal.lwin_big = 601;
Signal.rwin_big = 901;

Signal.kernel_name= {'SP','WDS','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};

Signal.lwin = { [-600] [-70] [   0] };
Signal.rwin = { [+100] [+70] [+900] };
Signal.wordlen = { [4] [] [4] };

Signal.order ={[] [16] []};
Signal.shift ={[] [0] []};
Signal.shift_const ={[] [28] []};
Signal.C = 2; % log2(C)=1.0

if has_transacc
  Signal.Conf_names = {Signal.Conf_names{:},'has_trans'};
  Signal.filter_label.train.rm_trans = 1;
  Signal.filter_label.eval.rm_trans = 1;
  Signal.filter_label.test.rm_trans = 1;
else
  Signal.filter_label.train.rm_trans = 0;
  Signal.filter_label.eval.rm_trans = 0;
  Signal.filter_label.test.rm_trans = 0;
end

%%%%%% for human
%Signal.sampling.downsample_neg = 1/160; % <=1 (for no downsampling: 1)



return

function diff_fields = compare_PAR(PAR_1,PAR_2,silent)

if nargin<3
  silent=0;
end

diff_fields ={};
if isequal(PAR_1,PAR_2)
  fprintf(1,'\t parameters identical\n')
else
  fields_1 = fieldnames(PAR_1);
  fields_2 = fieldnames(PAR_2);
  common_fields = intersect(fields_1,fields_2);
  for i=1:length(common_fields)
    par_1 = getfield(PAR_1,common_fields{i});
    par_2 = getfield(PAR_2,common_fields{i});
    if ~isequal(par_1,par_2)
      diff_fields{end+1} = common_fields{i};
      if ~silent
        fprintf(1,'\t field "%s" not identical\n',common_fields{i})
        fprintf(1,'\t %s, 1:\n',common_fields{i});disp(par_1)
        fprintf(1,'\t %s, 2:\n',common_fields{i});disp(par_2)
      end
    end
  end
  fields_x1= setdiff(fields_1,fields_2);
  if ~isempty(fields_x1)
    if ~silent
      fprintf(1,'\t additional fields in PAR_1: \n')
    end
    diff_fields = {diff_fields{:},fields_x1{:}};
  end
  fields_x2 = setdiff(fields_2,fields_1);
  if ~isempty(fields_x2)
    if ~silent
      fprintf(1,'\t additional fields in PAR_2: \n')
    end
    diff_fields = {diff_fields{:},fields_x2{:}};
  end
end
   
return

function [regions,N]=get_connected_regions(bw);
  
 % similar to bw_label just for 1xm matrix 

reg_starts = find(bw(1:end)-[0 bw(1:end-1)]==1);
reg_ends = find(bw(1:end)-[bw(2:end) 0]==1);
assert(length(reg_starts)==length(reg_ends))

N = length(reg_starts);

regions = zeros(size(bw));
for i=1:N
  assert(reg_starts(i)<=reg_ends(i))
  regions(reg_starts(i):reg_ends(i))=i;
end
return

function new_field = set_default(old_struct,field,defaults,silent)  ;

% new_struct = set_default(old_struct,defaults,overwrite)  ;
% sets all values of all fields in old_struct to the default values specified in
% defaults IF field is empty in old_struct. If overwrite is set all entries
% will be overwritten.
% defaults must be a struct.
%
% e.g. 
% default.use_cons = 0;
% default.plif.bins = 50;
% method.use_cons = 0;
% method = set_default(method,default,1) 

if ~isfield(old_struct,field)
  new_field = defaults;
  return
end  

old_struct = old_struct.(field);

if nargin<4
  silent = 1 ;
end

names = fieldnames(defaults);
for j=1:length(names)
  if ~isfield(old_struct,names{j}) || isempty(old_struct.(names{j}))||...
        (iscell(old_struct.(names{j}))&&length(old_struct.(names{j}))==1&&isempty(old_struct.(names{j}){1}))
    if ~isfield(old_struct,names{j}) && ~silent
      warning('new field added')
      names{j}
    end
    
    old_struct.(names{j}) = defaults.(names{j});
  elseif isstruct(defaults.(names{j}))
    old_struct.(names{j}) = set_default(old_struct,names{j},defaults.(names{j}));
    %names2 =  fieldnames(defaults.(names{j}));
    %for k= 1:length(names2)
    %  if ~isfield(old_struct.(names{j}),names2{k})||isempty(old_struct.(names{j}).(names2{k}))||overwrite
    %    old_struct.(names{j}).(names2{k}) = defaults.(names{j}).(names2{k});
    %  end
    %end
  end
end
% if length(fieldnames(PAR))>length(names)
%   keyboard
% end
% PAR = orderfields(PAR,defaults);

new_field = orderfields(old_struct,defaults);

return

function genes = build_splice_graph_caller(genes) ;
% genes = build_splice_graph_caller(genes) ;
  
for gene_idx = 1:length(genes)
  % construct a new empty splice graph, based on
  % http://proline.bic.nus.edu.sg/dedb/
  vertices =  [];
  edges = [];
%  fprintf(1,'gene index:\t%d\n',gene_idx);
  if (mod(gene_idx,100)==0)
    fprintf(1,'.');
  end

  for transcript_idx = 1:length(genes(gene_idx).transcripts)
    exon_start_end = genes(gene_idx).exons{transcript_idx};
    
    if (size(exon_start_end,1) == 1)
      exon1_start = exon_start_end(1,1);
      exon1_end = exon_start_end(1,2);

      
      if isempty(vertices)
	vertices(1,1) = exon1_start;
	vertices(2,1) = exon1_end;
	edges = 0;
	num_exons = 1;
      else
	vertices(1,num_exons+1) = exon1_start;
	vertices(2,num_exons+1) = exon1_end;
	edges(1,num_exons+1) = 0;
	edges(num_exons+1,1) = 0;
	num_exons = num_exons + 1;
      end

      
    else
      

      for exon_idx = 1:(size(exon_start_end,1)-1)
	exon1_start = exon_start_end(exon_idx,1);
	exon1_end = exon_start_end(exon_idx,2);
	exon2_start = exon_start_end(exon_idx+1,1);
	exon2_end = exon_start_end(exon_idx+1,2);


	
	if isempty(vertices)
	  vertices(1,1) = exon1_start;
	  vertices(2,1) = exon1_end;
	  vertices(1,2) = exon2_start;
	  vertices(2,2) = exon2_end;
	  edges = zeros(2);
	  edges(1,2) = 1;
	  edges(2,1) = 1;
	  num_exons = 2;
	else
	  exon1_idx = 0;
	  exon2_idx = 0;
	  idx = 1;
	  while idx <= num_exons
%	    if ((vertices(1,idx)==exon1_start)||...
%		(vertices(1,idx)+1==exon1_start)||...
%		(vertices(1,idx)-1==exon1_start))...
%		  && ((vertices(2,idx)==exon1_end)||...
%		      (vertices(2,idx)+1==exon1_end)||...
%		      (vertices(2,idx)-1==exon1_end))
	    if ((vertices(1,idx)==exon1_start) &&...
		(vertices(2,idx)==exon1_end))
	      exon1_idx = idx;
	    end
	    idx = idx+1;
	  end
	  idx = 1;
	  while idx <= num_exons
%	    if ((vertices(1,idx)==exon2_start)||...
%		(vertices(1,idx)+1==exon2_start)||...
%		(vertices(1,idx)-1==exon2_start))...
%		  && ((vertices(2,idx)==exon2_end)||...
%		      (vertices(2,idx)+1==exon2_end)||...
%		      (vertices(2,idx)-1==exon2_end))
	    if ((vertices(1,idx)==exon2_start) &&... 
		(vertices(2,idx)==exon2_end))
	      exon2_idx = idx;
	    end
	    idx = idx+1;
	  end
	  if (exon1_idx~=0) && (exon2_idx~=0)
	    edges(exon1_idx,exon2_idx) = 1;
	    edges(exon2_idx,exon1_idx) = 1;
	  else
	    if ((exon1_idx==0) && (exon2_idx~=0))
	      vertices(1,num_exons+1) = exon1_start;
	      vertices(2,num_exons+1) = exon1_end;
	      edges(exon2_idx,num_exons+1) = 1;
	      edges(num_exons+1,exon2_idx) = 1;
	      num_exons = num_exons + 1;
	    elseif ((exon2_idx==0) && (exon1_idx~=0))
	      vertices(1,num_exons+1) = exon2_start;
	      vertices(2,num_exons+1) = exon2_end;
	      edges(exon1_idx,num_exons+1) = 1;
	      edges(num_exons+1,exon1_idx) = 1;
	      num_exons = num_exons + 1;
	    else
	      assert((exon1_idx==0)&&(exon2_idx==0));
	      vertices(1,num_exons+1) = exon1_start;
	      vertices(2,num_exons+1) = exon1_end;
	      num_exons = num_exons + 1;
	      vertices(1,num_exons+1) = exon2_start;
	      vertices(2,num_exons+1) = exon2_end;
	      num_exons = num_exons + 1;	  
	      
	      edges(num_exons-1,num_exons) = 1;
	      edges(num_exons,num_exons-1) = 1;
	    end
	  end
	end
      end
    end
  end




  genes(gene_idx).splicegraph = {vertices,edges};
end


fprintf(1,'\n');


return






return

function [regions,level] = detectaltregions(gene)
% regions = detectaltregions(gene)
  
[max_exon_level,max_intron_level,exon_level,intron_level,level] = ...
    detectsplicegraph(gene) ;

if all(level(3,:)<=1), 
  regions=[] ;
  return ;
else
  regions = [] ;
  idx=find(level(3,:)>1) ;
  idx=unique([idx(1) find(level(3,2:end)>1 & level(3,1:end-1)<=1)+1]);

  
  for i=1:length(idx)
    %level(4,idx(i))
    %if level(4,idx(i))==0, keyboard; end ;

  %    if idx(i)==1
      start = level(1,idx(i)) ;
%    else
%      start = floor(mean([level(1,idx(i)-1) level(1,idx(i))])) ;
%    end ;
%    if idx(i)==size(level,2)
      stop = level(2,idx(i)) ;
%    else
%      stop = floor(mean([level(2,idx(i)) level(2,idx(i)+1)]))-1 ;
%    end ;
    depth = level(3,idx(i)) ;
    for j=idx(i):size(level,2),
      if level(3,j)>1 
%        if j==size(level,2)
          stop = level(2,j) ;
%        else
%          stop = floor(mean([level(2,j) level(2,j+1)]))-1 ;
%        end ;
        depth = max(depth, level(3,j)) ;
      else
        %if level(4,j)==0, keyboard; end ;
        break ;
      end ;
    end ;
    regions=[regions, [start; stop; depth]] ;
  end ;
end ;
return

function genes= alt_const(genes) ;
%function genes = alt_const(genes) ;

tot_exons = 0;
for ix=1:length(genes)
  if (mod(ix,1000)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2) ;
  tot_exons = tot_exons + num_exons ;
  vertices = genes(ix).splicegraph{1} ;
  edges = genes(ix).splicegraph{2} ;

  if isempty(edges)
    genes(ix).is_alt_spliced=0 ;
    genes(ix).is_alt=0 ;
    continue ;
  end 
  
  init_alt_idx = [] ;
  term_alt_idx = [] ;
  for i=1:num_exons,
    if ~any(edges(1:i-1,i)) % initial
      idx=setdiff(find(vertices(2,i)==vertices(2,:)),i) ;
      if ~isempty(idx) % other exon with same end
        is_simple=1 ; 
        for j=idx,
          if ~isequal(find(edges(j,j+1:end))+j,find(edges(i,i+1:end))+i)
            is_simple = 0 ;
            break ;
          end ;
          idx_prev = find(edges(1:j-1,j)) ;
          for k=idx_prev,
            if vertices(2,k)>vertices(1,i)
              is_simple = 0 ;
              break ;
            end ;
          end ;
        end ;
        if ~is_simple, continue ; end ;
        init_alt_idx(end+1) = i ;
      end
    end ;
    if ~any(edges(i,i+1:num_exons)) % terminal
      idx=setdiff(find(vertices(1,i)==vertices(1,:)),i) ;
      if ~isempty(idx) % other exon with same start
        is_simple=1 ; 
        for j=idx,
          if ~isequal(find(edges(1:j-1,j)),find(edges(1:i-1,i)))
            is_simple = 0 ;
            break ;
          end ;
          idx_next = find(edges(j,j+1:end))+j ;
          for k=idx_next,
            if vertices(1,k)<vertices(2,i)
              is_simple = 0 ;
              break ;
            end ;
          end ;
        end ;
        if ~is_simple, continue ; end ;
        term_alt_idx(end+1) = i ;
      end ;
    end ;
  end ;
  take_idx = setdiff(1:num_exons, [init_alt_idx,term_alt_idx]) ;
  vertices = genes(ix).splicegraph{1}(:,take_idx) ;
  edges = genes(ix).splicegraph{2}(take_idx,take_idx) ;

  start=min(vertices(1,:)-50) ;
  stop=max(vertices(2,:)+50) ;
  
  exon_loc = zeros(1,stop-start);

  for i=1:size(edges,1),
    for j=i+1:size(edges,1),
      if edges(i,j)==1,
        cur_edge = [vertices(2,i)+1-start, vertices(1,j)-1-start] ;
        exon_loc(cur_edge(1):cur_edge(2)) = ...
            exon_loc(cur_edge(1):cur_edge(2)) + 1;
      end ;
    end ;
  end ;

  for i=1:size(vertices,2)
    cur_vertex = vertices(:,i) - [start;start];
    exon_loc(cur_vertex(1):cur_vertex(2)) = ...
        exon_loc(cur_vertex(1):cur_vertex(2)) + 1;
  end ;

  if max(exon_loc)>1
    genes(ix).is_alt_spliced=1 ;
    genes(ix).is_alt=1 ;
  else
    genes(ix).is_alt_spliced=0 ;
    if isempty([init_alt_idx,term_alt_idx])
      genes(ix).is_alt=0 ;
    else
      genes(ix).is_alt=1 ;
    end ;
  end
end

fprintf(1,'\n\nTotal genes:\t\t\t\t\t\t\t%d\n',...
        length(genes));
fprintf(1,'Total exons:\t\t\t\t\t\t\t%d\n',...
        tot_exons);
fprintf(1,'Total genes with alternative isoforms:\t\t\t\t%d\n',...
        sum([genes(:).is_alt]));
fprintf(1,'Total genes alternatively spliced:\t\t\t\t%d\n',...
        sum([genes(:).is_alt_spliced]));
fprintf(1,'Total constitutively spliced:\t\t\t\t\t%d\n',...
        sum(~[genes(:).is_alt_spliced]));


return

function genes = annotate_splicegraph(genes)
% function annotate_splicegraph(info,origpath)
%
% annotate the genes structure with counts for:
% - exon skipping
% - intron retention
% - alternative donor (5 prime)
% - alternative acceptor (3 prime)
% - alternative transcription start
% - alternative transcription end
%
% Written by: Cheng Soon Ong, 14 May 2007
% Modified on: 31 May 2007 to include transcription starts and stops


%addpath(sprintf('%s/utils', origpath)) ;
%addpath(sprintf('%s/splicegraphs', origpath)) ;
%addpath(sprintf('%s/splicegraphs/detect_altsplice', origpath)) ;
%addpath(sprintf('%s/gff', origpath)) ;
%addpath(sprintf('%s/gff/read_sequences', origpath)) ;
%addpath /fml/ag-raetsch/share/software/matlab_tools/utils

%genome_info = init_genome(info) ;
%fprintf('directory: %s\n', genome_info.basedir) ;
%load(sprintf('%s/confirmed_sequences.mat', genome_info.basedir), 'genes') ;
%fprintf('loaded confirmed_sequences.mat\n\n') ;

if ~isfield(genes, 'strands'),
  for i=1:length(genes),
    genes(i).strands = repmat(genes(i).strand, 1, length(genes(i).transcripts)) ;
  end ;
end ;

if isfield(genes,'alt_3prime_details') || isfield(genes,'alt_5prime_details')
  warning('annotate_splicegraph adds to alt_3prime_details and alt_5prime_details,\nrunning more than one creates inconsistencies');
  return;
end

idx_alt=[] ;
for i=1:length(genes)
  if genes(i).is_alt_spliced
    idx_alt=[idx_alt, i] ;
  end
  if ~isfield(genes(i),'strands') || isempty(genes(i).strands)
    genes(i).strands(1) = genes(i).strand;
  end
end

[idx_exon_skips, exon_exon_skips] = detect_exonskips(genes, idx_alt);
[idx_alt_5prime,exon_alt_5prime, idx_alt_3prime,exon_alt_3prime] = ...
    detect_altprime_pair(genes, idx_alt);
[idx_intron_reten,intron_intron_reten] = detect_intronreten(genes, idx_alt) ;
[idx_xor_exons, exon_xor_exons] = detect_xorexons(genes, idx_alt) ;
[idx_alt_tstart, exon_alt_tstart] = detect_alttstart(genes) ;
idx_alt_tstart = [idx_alt_tstart{:}];
[idx_alt_tend, exon_alt_tend] = detect_alttend(genes) ;
idx_alt_tend = [idx_alt_tend{:}];

%keyboard
for ix = 1:length(genes)
    
  if mod(ix,1000)==0
    fprintf('.');
  end

  exons_is_alt = zeros(1,size(genes(ix).splicegraph{1},2));

  % exon skips
  if ismember(ix,idx_exon_skips)
    tix = find(idx_exon_skips==ix);
    genes(ix).alt_exon = length(tix);
    genes(ix).alt_exon_details.exons = exon_exon_skips(:,tix);
    exons_is_alt(exon_exon_skips(:,tix)) = exons_is_alt(exon_exon_skips(:,tix)) + 1;
  else
    genes(ix).alt_exon = 0;
  end

  % intron retentions
  if ismember(ix,idx_intron_reten)
    tix = find(idx_intron_reten==ix);
    genes(ix).alt_intron = length(tix);
    genes(ix).alt_intron_details.exons = intron_intron_reten(:,tix);
    exons_is_alt(intron_intron_reten(:,tix)) = exons_is_alt(intron_intron_reten(:,tix)) + 1;
  else
    genes(ix).alt_intron = 0;
  end

  % alternative 5prime
  if ismember(ix,idx_alt_5prime)
    genes(ix).alt_5prime_details.exons = zeros(3,0);
    tix = find(idx_alt_5prime==ix);
    genes(ix).alt_5prime = length(tix);
    for ixt = tix
      if genes(ix).strands(1) == '+'
	idx_exon_l = exon_alt_5prime(ixt).fiveprimesites(1);
	idx_exon_r = exon_alt_5prime(ixt).threeprimesite;
	idx_exon_alt = exon_alt_5prime(ixt).fiveprimesites(2);
      else
	idx_exon_r = exon_alt_5prime(ixt).fiveprimesites(1);
	idx_exon_l = exon_alt_5prime(ixt).threeprimesite;
	idx_exon_alt = exon_alt_5prime(ixt).fiveprimesites(2);
      end
      genes(ix).alt_5prime_details.exons(:,end+1) = [idx_exon_l;idx_exon_r;idx_exon_alt];
      exons_is_alt([idx_exon_l;idx_exon_r;idx_exon_alt]) = exons_is_alt([idx_exon_l;idx_exon_r;idx_exon_alt]) + 1;
    end
  else
    genes(ix).alt_5prime = 0;
  end

  % alternative 3prime
  if ismember(ix,idx_alt_3prime)
    genes(ix).alt_3prime_details.exons = zeros(3,0);
    tix = find(idx_alt_3prime==ix);
    genes(ix).alt_3prime = length(tix);
    for ixt = tix
      if genes(ix).strands(1) == '+'
	idx_exon_l = exon_alt_3prime(ixt).fiveprimesite;
	idx_exon_r = exon_alt_3prime(ixt).threeprimesites(1);
	idx_exon_alt = exon_alt_3prime(ixt).threeprimesites(2);
      else
	idx_exon_r = exon_alt_3prime(ixt).fiveprimesite;
	idx_exon_l = exon_alt_3prime(ixt).threeprimesites(1);
	idx_exon_alt = exon_alt_3prime(ixt).threeprimesites(2);
      end
      genes(ix).alt_3prime_details.exons(:,end+1) = [idx_exon_l;idx_exon_r;idx_exon_alt];
      exons_is_alt([idx_exon_l;idx_exon_r;idx_exon_alt]) = exons_is_alt([idx_exon_l;idx_exon_r;idx_exon_alt]) + 1;
    end
  else
    genes(ix).alt_3prime = 0;
  end

  % xor exons
  if ismember(ix,idx_xor_exons)
    tix = find(idx_xor_exons==ix);
    genes(ix).alt_xor_exon = length(tix);
    for ixt = tix
      exons_is_alt(exon_xor_exons(:,tix)) = exons_is_alt(exon_xor_exons(:,tix)) + 1;
    end
  else
    genes(ix).alt_xor_exon = 0;
  end

  % alternative transcriptions
  if ismember(ix,idx_alt_tstart)
    tix = find([idx_alt_tstart == ix]);
    % there is only one entry per gene.
    assert(all(size(tix) == [1,1]))
    genes(ix).alt_tstart = length(exon_alt_tstart{tix}.exons);
    genes(ix).alt_tstart_details.exons = exon_alt_tstart{tix}.exons;
  else
    genes(ix).alt_tstart = 0;
  end

  if ismember(ix,idx_alt_tend)
    tix = find([idx_alt_tend == ix]);
    % there is only one entry per gene.
    assert(all(size(tix) == [1,1]))
    genes(ix).alt_tend = length(exon_alt_tend{tix}.exons);
    genes(ix).alt_tend_details.exons = exon_alt_tend{tix}.exons;
  else
    genes(ix).alt_tend = 0;
  end

  genes(ix).exons_is_alt = exons_is_alt;
end


if 0,
  fid = fopen(sprintf('%s/alt_splice_type.tsv', genome_info.basedir),'w');
  fprintf(fid,'Clust\tExon Skip\tIntron Reten\tAlt 5prime\tAlt 3prime\n');
  for ix = idx_alt  
    fprintf(fid,'%d\t%d\t%d\t%d\t%d\n',ix,...
	    genes(ix).alt_exon,genes(ix).alt_intron,genes(ix).alt_5prime,genes(ix).alt_3prime);
  end
  fclose(fid);
end

return

function [idx_alt_5prime,exon_alt_5prime, idx_alt_3prime,exon_alt_3prime] ...
    = detect_altprime_pair(genes, idx_alt);
% function [idx_alt_5prime,exon_alt_5prime, idx_alt_3prime,exon_alt_3prime] ...
%    = detect_altprime_pair(genes, idx_alt);
%
% Detect the alternative 5 and 3 prime ends of the intron. Note that 5 prime refers to the left
% and 3 prime to the right for a positive strand
% The difference between this function and detect_altprime.m is that the other function collects
% all the alternative sites corresponding to an acceptor or donor.
% note that the 'overlap' relationship is not transitive

MIN_OVERLAP = 11; % two exons that are alternative should overlap by at least the BLAT seed

idx_alt_5prime = [];
idx_alt_3prime = [];
exon_alt_5prime = {};
exon_alt_3prime = {};

if ~isfield(genes, 'strands'),
  for i=1:length(genes),
    genes(i).strands = repmat(genes(i).strand, 1, length(genes(i).transcripts)) ;
  end ;
end ;

for ix=idx_alt
  if (mod(ix,50)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2) ;
  vertices = genes(ix).splicegraph{1} ;
  edges = genes(ix).splicegraph{2} ;
  
  % Find alternative sites on the right of the intron,
  % same site on the left of the intron.
  for exon_idx = 1:num_exons-2
    nr_exons=sum(edges(exon_idx,exon_idx+1:num_exons));
    if nr_exons>=2,
      which_exons = find([zeros(1,exon_idx),edges(exon_idx,exon_idx+1:num_exons)]);
      exons=[vertices(:, which_exons)] ;
      for i=1:nr_exons-1
	for j=i+1:nr_exons
	  % check that the left splice site of the exons are different
	  % make sure exons overlap - either:
	  % - left splice site of exon(i) is in exon(j)
	  % - left splice site of exon(j) is in exon(i)
	  
	  if ((exons(1,i) ~= exons(1,j)) && ...
	       (( (exons(1,i) >= exons(1,j)) && (exons(1,i) <= exons(2,j)) ) ...
		|| ( (exons(1,j) >= exons(1,i)) && (exons(1,j) <= exons(2,i)) )) && ...
	       (min(exons(2,i),exons(2,j))-max(exons(1,i),exons(1,j)) >= MIN_OVERLAP ) )
	    assert(~((exons(1,i) == exons(1,j)) && (exons(2,i) == exons(2,j))));
	    assert(exons(1,i) ~= exons(1,j));
	    % construct the output
	    if (genes(ix).strands(1)=='+'),
	      exon_alt_3prime(end+1).fiveprimesite=exon_idx ;
	      if exons(1,i) > exons(1,j)
		exon_alt_3prime(end).threeprimesites=which_exons([i j]);
	      else
		exon_alt_3prime(end).threeprimesites=which_exons([j i]);
	      end
	      idx_alt_3prime(end+1) = ix;
	    end
	    if (genes(ix).strands(1)=='-'),
	      exon_alt_5prime(end+1).threeprimesite=exon_idx ;
	      if exons(1,i) > exons(1,j)
		exon_alt_5prime(end).fiveprimesites=which_exons([i j]);
	      else
		exon_alt_5prime(end).fiveprimesites=which_exons([j i]);
	      end
	      idx_alt_5prime(end+1) = ix;
	    end
	  end % if altsplice
	  
	end % for j=i+1:nr_exons
      end % i=1:nr_exons-1
    end
  end % for exon_idx

  
  % Find alternative sites on the left of the intron,
  % same site on the right of the intron.
  for exon_idx = 3:num_exons
    nr_exons=sum(edges(1:exon_idx-1,exon_idx)) ;
    if nr_exons>=2,
      which_exons = find([edges(1:exon_idx-1,exon_idx)', zeros(1,num_exons-exon_idx+1)]);
      exons=[vertices(:, which_exons)] ;
      for i=1:nr_exons-1
	for j=i+1:nr_exons
	  % check that the 5prime sites are different
	  % make sure exons overlap - either:
	  % - right splice site of exon(i) is in exon(j)
	  % - right splice site of exon(j) is in exon(i)
	  if ((exons(2,i) ~= exons(2,j)) &&...
	      (( (exons(2,i) <= exons(2,j)) && (exons(2,i) >= exons(1,j)) )...
	      || ( (exons(2,j) <= exons(2,i)) && (exons(2,j) >= exons(1,i)) )) && ...
	      (min(exons(2,i),exons(2,j))-max(exons(1,i),exons(1,j)) >= MIN_OVERLAP ) )
	    assert(~((exons(1,i) == exons(1,j)) && (exons(2,i) == exons(2,j))));
	    assert(exons(2,i) ~= exons(2,j));

	    % construct the output
	    if (genes(ix).strands(1)=='+'),
	      exon_alt_5prime(end+1).threeprimesite=exon_idx ;
	      if exons(2,i) < exons(2,j)
		exon_alt_5prime(end).fiveprimesites=which_exons([i j]);
	      else
		exon_alt_5prime(end).fiveprimesites=which_exons([j i]);
	      end
	      idx_alt_5prime(end+1) = ix;
	    end
	    if (genes(ix).strands(1)=='-'),
	      exon_alt_3prime(end+1).fiveprimesite=exon_idx ;
	      if exons(2,i) < exons(2,j)
		exon_alt_3prime(end).threeprimesites=which_exons([i j]);
	      else
		exon_alt_3prime(end).threeprimesites=which_exons([j i]);
	      end
	      idx_alt_3prime(end+1) = ix;
	    end
	  end % if altsplice
	  
	end % for j=i+1:nr_exons
      end % for i=1:nr_exons-1
    end

  end % construct output
end % for exon_idx
  



fprintf(1,'\n\nNumber of alternative 5 prime sites:\t\t\t\t%d\n',...
	length(idx_alt_5prime));

fprintf(1,'Number of alternative 3 prime sites:\t\t\t\t%d\n',...
	length(idx_alt_3prime));






return

function [idx_alt_tend, exon_alt_tend] = detect_alttend(genes) ;

idx_alt_tend = {};
exon_alt_tend = {};

for ix=1:length(genes)
  if (mod(ix,1000)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2) ;
  vertices = genes(ix).splicegraph{1} ;
  edges = triu(genes(ix).splicegraph{2}) ;
  
  term_alt_idx = [] ;
  for i=1:num_exons,
    if sum(edges(i,:),2)==0,
      term_alt_idx(end+1) = i ;
    end ;
  end ;
  if length(term_alt_idx)>1
    idx_alt_tend{end+1} = ix;
    exon_alt_tend{end+1}.exons = term_alt_idx;
  end
end

fprintf(1,'\nTotal alternative transcription ends:\t\t\t\t%d\n',...
        length(idx_alt_tend));

return

function [idx_alt_tstart, exon_alt_tstart] = detect_alttstart(genes) ;

idx_alt_tstart = {};
exon_alt_tstart = {};

for ix=1:length(genes)
  if (mod(ix,1000)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2) ;
  vertices = genes(ix).splicegraph{1} ;
  edges = triu(genes(ix).splicegraph{2}) ;
  
  init_alt_idx = [] ;
  for i=1:num_exons,
    if sum(edges(:,i),1)==0, % initial
      init_alt_idx(end+1) = i ;
    end ;
  end ;

  if length(init_alt_idx)>1,
    idx_alt_tstart{end+1} = ix;
    exon_alt_tstart{end+1}.exons = init_alt_idx;
  end
end

fprintf(1,'\nTotal alternative transcription starts:\t\t\t\t%d\n',...
        length(idx_alt_tstart));

return

function [idx_exon_skips, exon_exon_skips] = detect_exonskips(genes, idx_alt) ;
% [idx_exon_skips, exon_exon_skips] = detect_exonskips(genes, idx_alt) ;

idx_exon_skips = [];
exon_exon_skips = [];
for ix=idx_alt
  if (mod(ix,50)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2);
  edges = genes(ix).splicegraph{2};
  for exon_idx = 1:num_exons-2
    for exon_idx1 = (exon_idx+1):(num_exons-1)
      for exon_idx2 = (exon_idx1+1):num_exons
        if (edges(exon_idx,exon_idx1))&&...
              (edges(exon_idx,exon_idx2))&&...
              (edges(exon_idx1,exon_idx2))
          idx_exon_skips = [idx_exon_skips,ix];
          exon_exon_skips = [exon_exon_skips,[exon_idx; exon_idx1; exon_idx2]];
        end
      end
    end
  end
end
sum_skips=length(idx_exon_skips) ;
fprintf(1,'\n\nNumber of single exon skips:\t\t\t\t\t%d\n', sum_skips);



return

function [idx_intron_reten,intron_intron_reten] = detect_intronreten(genes,idx_alt) ;

idx_intron_reten = [];
intron_intron_reten = [];
for ix=idx_alt
  %viewsplicegraph(genes(ix)) ;
  if (mod(ix,50)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2);
  vertices = genes(ix).splicegraph{1};
  edges = genes(ix).splicegraph{2};
  introns=[] ;
  for exon_idx = 1:num_exons-1 %start of intron
    %exon_idx
    %edges(exon_idx,exon_idx+1:length(edges))
    idx=find(edges(exon_idx,exon_idx+1:length(edges))==1) ;
    if isempty(idx), continue; end
    idx=idx+exon_idx ;
    for exon_idx2=idx, %end of intron
      is_intron_reten=0 ;
      for exon_idx1=1:num_exons %exon
	% check that the exon covers the intron
        if (vertices(2,exon_idx) >= vertices(1,exon_idx1)) && (vertices(1,exon_idx2) <= vertices(2,exon_idx1))
	
	% check that the exon covers the intron, but doesn't extend beyond
        %if (vertices(2,exon_idx) > vertices(1,exon_idx1)) && (vertices(1,exon_idx2) < vertices(2,exon_idx1))...
	%      && (vertices(1,exon_idx) <= vertices(1,exon_idx1)) && (vertices(2,exon_idx2) >= vertices(2,exon_idx1))
          is_intron_reten=1 ;
          long_exon=exon_idx1 ;
          for len=1:size(introns,2),
            if (vertices(2,exon_idx)==introns(1,len)) && (vertices(1,exon_idx2)==introns(2,len))
              is_intron_reten=0 ;
            end  
          end
        end
      end
      if is_intron_reten==1,
        idx_intron_reten = [idx_intron_reten, ix];
        intron_intron_reten = [intron_intron_reten, [exon_idx ; exon_idx2 ...
                   ; long_exon]] ;
        introns=[introns, [vertices(2,exon_idx) ; vertices(1,exon_idx2)]] ;
      end
    end
    %pause ;
  end
  
  %pause ;
end
fprintf(1,'\n\nNumber of intron retentions:\t\t\t\t\t%d\n', ...
	length(idx_intron_reten));


return

function [idx_xor_exons, exon_xor_exons,id_xor_exons] = detect_xorexons(genes, idx_alt);

idx_xor_exons = zeros(1,0); 
id_xor_exons = zeros(1,0); 
exon_xor_exons = zeros(4,0); %%%% 5primesite of first exon, the 2 skipped
                     %exons, 3primesite of last exon %%%
id = 0 ;
for ix=idx_alt
  if (mod(ix,50)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2);
  edges = genes(ix).splicegraph{2};
  vertices = genes(ix).splicegraph{1};
  for exon_idx1 = 1:num_exons-3
    id=id+1 ;
    for exon_idx2 = (exon_idx1+1):(num_exons-2)
      if edges(exon_idx1, exon_idx2)
        for exon_idx3=(exon_idx2+1):(num_exons-1)
          if edges(exon_idx1, exon_idx3) & edges(exon_idx2, exon_idx3)==0 & ...
             (vertices(1,exon_idx3) > vertices(2,exon_idx2)),
            for exon_idx4=(exon_idx3+1):num_exons
              if edges(exon_idx2, exon_idx4) & edges(exon_idx3, exon_idx4) & ...
                    edges(exon_idx1, exon_idx4)==0
                idx_xor_exons = [idx_xor_exons, ix];
                id_xor_exons = [id_xor_exons, id];
                exon_xor_exons = [exon_xor_exons,[exon_idx1; exon_idx2; exon_idx3; exon_idx4]] ;
              end
            end
          end
        end
      end
    end
  end
end
fprintf(1,'\n\nNumber of XOR exons:\t\t\t\t\t\t%d\n',...
	length(idx_xor_exons));



return

function [max_exon_level,max_intron_level,exon_level,intron_level,level] = detectsplicegraph(gene)

  [max_exon_level,max_intron_level,exon_level,intron_level,level] = countlevel(gene.exons,gene.name,...
                                                    gene.transcripts,[], ...
                                                    gene.splicegraph{1},...
                                                    gene.splicegraph{2});
  return


function [max_level,max_leveli,exon_level,intron_level,level] = countlevel(truepos,name,labels,colors,...
                                                    vertices,edges)
  
  
  start=inf ;
  stop=-inf ;
  for k=1:length(truepos)
    start=min(start,truepos{k}(1,1)-50) ;
    stop=max(stop,truepos{k}(end,2)+50) ;
  end ;
  
  
  max_level = 0;
  max_leveli = 0;

  splice = unique(vertices(:)) ;
  
  exon_loc = zeros(1,stop-start);
  exon_level = zeros(3,size(vertices,2));
  for i=1:size(vertices,2)
    cur_vertex = vertices(:,i) - [start;start];
    exon_loc(cur_vertex(1):cur_vertex(2)-1) = ...
        exon_loc(cur_vertex(1):cur_vertex(2)-1) + 1;
    %exon_loc(cur_vertex(1):cur_vertex(2)-1) = ...
    %    max(exon_loc(cur_vertex(1):cur_vertex(2)-1));
    
    level = max(exon_loc(cur_vertex(1):cur_vertex(2)-1));
    if isempty(level), level=0 ; end ;
    max_level = max(level,max_level);
    exon_level(1,i) = vertices(1,i) ;  
    exon_level(2,i) = vertices(2,i) ;  
    exon_level(3,i) = level;  
  end
  
  intron_loc = zeros(1,stop-start);
  intron_level = zeros(3,0) ;
  if (size(edges,1) > 1)
    for i=1:size(vertices,2)
      for j=i+1:size(vertices,2)
        if edges(i,j)
          if (vertices(1,i) < vertices(1,j))
            istart = vertices(2,i);
            istop =  vertices(1,j);
            %level1 = exon_level(i);
            %level2 = exon_level(j);
          else
            istart = vertices(2,j);
            istop =  vertices(1,i);
            %level1 = exon_level(j);
            %level2 = exon_level(i);
          end
          
          cur_intron = [istart-start,istop-start];
          intron_loc(cur_intron(1):cur_intron(2)-1) = ...
              intron_loc(cur_intron(1):cur_intron(2)-1) + 1;
          intron_level(3,end+1) = max(intron_loc(cur_intron(1):cur_intron(2)-1));
          intron_level(1,end) = istart ;
          intron_level(2,end) = istop ;

          max_leveli = max(intron_level(3,end),max_leveli);
        end
      end
    end
  end
  
  splice_loc = exon_loc+intron_loc ;
  level=zeros(5,length(splice)-1) ;
  for i=1:length(splice)-1,
    level(:,i)=[splice(i);
                splice(i+1);
                max(splice_loc([splice(i)-start:splice(i+1)-start-1]));
                max(exon_loc([splice(i)-start:splice(i+1)-start-1]));
                max(intron_loc([splice(i)-start:splice(i+1)-start-1]));
               ] ;
  end ;

return


return

function genes = infer_splice_graph_caller(genes) ;
% genes = infer_splice_graph_caller(genes) ;
  
%infer_splice_graph

MISALIGN = 5;

fprintf(1,'Number of genes:%d\n',length(genes));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Some simple merging to reduce exons
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf(1,'Merging some genes\n');
genes = reduce_splice_graph_caller(genes) ;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% infer_splice_graph infers the relevant splice graph given a set of ESTs. 
%
% It is based on two assumptions, neither which are true in reality, but
% are necessary for any reasonable processing.
% 1. All splicing events are independent. That is a splice site
%    at location A in ESTA does not depend on splice site B in ESTA, and
%    neither does it depend on splice site C in ESTB.
% 2. There exists a splicing event at a particular location if and only if
%    there exists an EST that is spliced at this point.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


fprintf(1,'performing inference\n');

for gene_idx = 1:length(genes)

  if (mod(gene_idx,1000)==0)
    fprintf(1,'%d\n',gene_idx);
  end
  

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % For each gene, we consider each pair of exons in the splicegraph.
  % After sorting the exons by the location of the left side, we make
  % a local copy of the splicegraph.
  %
  % Note that 'left' means the 5 prime end of a positive strand, but
  % the 3 prime end of a negative strand.
  %
  % The two exons considered are indexed by exon_idx, and test_exon_idx.
  % The general idea of the algorithm is as follows:
  % For each pair:
  %   check which case it is in (more on this later)
  %   if it is possible to merge:
  %     form a new exon
  %   end merge.
  %   if new exon is not in current set
  %     add to splicegraph
  %   end new exon insert
  % end for
  % 
  % In words:
  % Each time two exons are considered, and they can be merged to form
  % a new exon, they are added to the list of exons. Then, each of the
  % original parent exons are considered for deletion (variable to_keep).
  % All the exons are kept till the very end, when all exons labelled
  % to_keep == 0 are removed.
  %
  % Hence the number of exons could potentially grow quadratically.
  % Since we have to also consider merging the pairs of exons which are
  % newly created, the number of pairs considered can be larger than
  % m^2 where m is the number of original exons. However, I believe that
  % the algorithm should terminate because there are only finitely many
  % splice sites and hence at worst, we have all possible left and right
  % exon ends used in our splicegraph.
  %
  % I am not sure what the best termination condition for the loops are.
  %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if isempty(genes(gene_idx).splicegraph{1}),
    continue ; 
  end ;
  [dummy,exon_order] = sort(genes(gene_idx).splicegraph{1}(1,:),2,'ascend');
  vertices = genes(gene_idx).splicegraph{1}(:,exon_order);
  edges = genes(gene_idx).splicegraph{2}(exon_order,exon_order);
  to_keep = ones(1,size(vertices,2));

  changed = 0;
  exon_idx = 1;
  first_merge_exon = 1;
  test_exon_idx = 0;
  while exon_idx <= size(vertices,2)

    if changed
      changed = 0;
      exon_idx = first_merge_exon;
      continue;
    end
    

    num_exons = size(vertices,2);
    [dummy,exon_order] = sort(vertices(1,:),2,'ascend');
    if ~isequal(vertices(1,:),dummy)
      vertices = vertices(:,exon_order);
      edges = edges(exon_order,exon_order);
      to_keep = to_keep(exon_order);
    end ;
    
    
    exon_take_idx = find(((vertices(1,exon_idx)-2*MISALIGN)<vertices(2,:))&...
			 ((vertices(2,exon_idx)+2*MISALIGN)>vertices(1,:)));
    if ~isempty(exon_take_idx)
      first_merge_exon = exon_take_idx(1);
    end
    exon_take_idx = exon_take_idx(exon_take_idx>exon_idx);
    internal_idx = 1;


    while ((internal_idx <= length(exon_take_idx)) && ~changed)
      test_exon_idx = exon_take_idx(internal_idx);
      if (test_exon_idx < exon_idx) && keyboard_allowed(), keyboard;end;
      cur_edge_left = sum(edges(1:exon_idx,exon_idx));
      test_edge_left = sum(edges(1:test_exon_idx,test_exon_idx));
      cur_edge_right = sum(edges(exon_idx:end,exon_idx));
      test_edge_right = sum(edges(test_exon_idx:end,test_exon_idx));
	  

      new_vertex = zeros(2,1);
	
      
      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      % In the following, the cases are labelled with a binary string
      % [cur_edge_left, cur_edge_right, test_edge_left, test_edge_right]
      %
      % cur refers to exon_idx
      % test refers to test_exon_idx
      % edge means introns.
      % e.g. cur_edge_left == 1 means that there exists an intron on
      % the left of exon_idx
      %
      % Hence there are 16 cases, and we treat each one individually.
      % For each case, the two logical tests are:
      % 1. should we merge?
      % 2. is the new exon known?
      %
      % Note that a splice site is defined by an intron-exon boundary.
      % Furthermore, we allow misalignment by MISALIGN nucleotides,
      % that is we shorten an exonAendA if it doesn't have a splice site
      % and there exists a splice site exonBendB within MISALIGN nucleotides.
      % 
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
      
      
      % 0000
      if (~cur_edge_left&&~cur_edge_right&&~test_edge_left&&~test_edge_right)

	if ((~isequal(vertices(1,exon_idx),vertices(1,test_exon_idx)))||...
	    (~isequal(vertices(2,exon_idx),vertices(2,test_exon_idx))))&&...
	      ((to_keep(test_exon_idx)~=0)||(to_keep(exon_idx)~=0))&&...
	      (vertices(2,exon_idx)>=vertices(1,test_exon_idx))
	  
	  
	  new_vertex(1) = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	  new_vertex(2) = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	  new_edges = zeros(1,num_exons);
	
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix))
	      known = ix;
	      break;
	    end
	  end
	
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  else
	    if (known ~= exon_idx) && to_keep(exon_idx)
	      to_keep(exon_idx) = 0;
	      changed = 1;
	    end
	    if (known ~= test_exon_idx) && to_keep(test_exon_idx)
	      to_keep(test_exon_idx) = 0;
	      changed = 1;
	    end
	  end
	end
	      
	      

	% 0001
      elseif (~cur_edge_left&&~cur_edge_right&&~test_edge_left&&test_edge_right)
	if (vertices(2,exon_idx)-MISALIGN<=vertices(2,test_exon_idx))&&...
	      (vertices(2,exon_idx)>=vertices(1,test_exon_idx))
	
	  new_vertex(1) = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	  new_vertex(2) = vertices(2,test_exon_idx);
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	  
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix)) && issubset(new_edges,edges(ix,:)) 
	      known = ix;
	      break;
	    end
	  end
	  assert(known~=exon_idx);
	  
	  if to_keep(exon_idx)
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif (known ~= test_exon_idx) && to_keep(test_exon_idx)
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	% 0100
      elseif (~cur_edge_left&&cur_edge_right&&~test_edge_left&&~test_edge_right)
	if (vertices(2,exon_idx)>=vertices(2,test_exon_idx)-MISALIGN)&&...
	     (vertices(1,exon_idx)<=vertices(2,test_exon_idx))
	
	  new_vertex(1) = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	  new_vertex(2) = vertices(2,exon_idx);
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix)) && issubset(new_edges,edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert(known~=test_exon_idx);
	
	  if to_keep(test_exon_idx)
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif (known ~= exon_idx) && to_keep(exon_idx)
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	
	% 0010
      elseif (~cur_edge_left&&~cur_edge_right&&test_edge_left&&~test_edge_right)
	if (vertices(1,exon_idx)+MISALIGN>=vertices(1,test_exon_idx))&&...
	      (vertices(1,exon_idx)<=vertices(2,test_exon_idx))
	    
	    
	  new_vertex(1) = vertices(1,test_exon_idx);
	  new_vertex(2) = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	  
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix)) && issubset(new_edges,edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert(known~=exon_idx);
	  
	  if to_keep(exon_idx)
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif (known ~= test_exon_idx) && to_keep(test_exon_idx)
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	% 1000
      elseif (cur_edge_left&&~cur_edge_right&&~test_edge_left&&~test_edge_right)
	if (vertices(1,exon_idx)<=vertices(1,test_exon_idx)+MISALIGN)&&...
	      (vertices(2,exon_idx)>=vertices(1,test_exon_idx))
	
	    
	  new_vertex(1) = vertices(1,exon_idx);
	  new_vertex(2) = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	  
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix)) && issubset(new_edges,edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert(known~=test_exon_idx);
	  
	  if to_keep(test_exon_idx)
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif (known ~= exon_idx) && to_keep(exon_idx)
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	end
	    
	% 0011
      elseif (~cur_edge_left&&~cur_edge_right&&test_edge_left&&test_edge_right)
	if (vertices(1,exon_idx)+MISALIGN>=vertices(1,test_exon_idx))&&...
	     (vertices(2,exon_idx)-MISALIGN<=vertices(2,test_exon_idx))
	
	  new_vertex = vertices(:,test_exon_idx);
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	  
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix)) && issubset(new_edges,edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert(known~=exon_idx);
	  
	  if to_keep(exon_idx)
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	  
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif (known ~= test_exon_idx) && to_keep(test_exon_idx)
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	% 1100
      elseif (cur_edge_left&&cur_edge_right&&~test_edge_left&&~test_edge_right)
	if (vertices(1,exon_idx)<=vertices(1,test_exon_idx)+MISALIGN)&&...
	      (vertices(2,exon_idx)>=vertices(2,test_exon_idx)-MISALIGN)
	
	  new_vertex = vertices(:,exon_idx);
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	  
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix)) && issubset(new_edges,edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert(known~=test_exon_idx);
	  
	  if to_keep(test_exon_idx)
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif (known ~= exon_idx) && to_keep(exon_idx)
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	end
	    
	% 0101
      elseif (~cur_edge_left&&cur_edge_right&&~test_edge_left&&test_edge_right)
	if (vertices(2,exon_idx)==vertices(2,test_exon_idx))

	  if (~isequal(edges(exon_idx,:),edges(test_exon_idx,:)))
	    edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	    edges(:,exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	    edges(test_exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	    edges(:,test_exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	      
	    if (vertices(1,exon_idx) > vertices(1,test_exon_idx))
	      to_keep(exon_idx) = 0;
	    else
	      to_keep(test_exon_idx) = 0;
	    end
	  
	  
	    changed = 1;
	  else
	    if (vertices(1,exon_idx) > vertices(1,test_exon_idx)) && (to_keep(exon_idx))
	      to_keep(exon_idx) = 0;
	      changed = 1;
	    end
	    if (vertices(1,exon_idx) <= vertices(1,test_exon_idx)) && (to_keep(test_exon_idx))
	      to_keep(test_exon_idx) = 0;
	      changed = 1;
	    end
	  end
	end
	    

	    
	    
	% 1010
      elseif (cur_edge_left&&~cur_edge_right&&test_edge_left&&~test_edge_right)
	if (vertices(1,exon_idx)==vertices(1,test_exon_idx))
	
	  if (~isequal(edges(exon_idx,:),edges(test_exon_idx,:)))
	    edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	    edges(:,exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	    edges(test_exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	    edges(:,test_exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	    
	    if (vertices(2,exon_idx) < vertices(2,test_exon_idx))
	      to_keep(exon_idx) = 0;
	    else
	      to_keep(test_exon_idx) = 0;
	    end
	  
	  
	    changed = 1;
	  else
	    if (vertices(2,exon_idx) < vertices(2,test_exon_idx)) && (to_keep(exon_idx))
	      to_keep(exon_idx) = 0;
	      changed = 1;
	    end
	    if (vertices(2,exon_idx) >= vertices(2,test_exon_idx)) && (to_keep(test_exon_idx))
	      to_keep(test_exon_idx) = 0;
	      changed = 1;
	    end
	  end
	end
	    
	    
	    
	% 0110  
      elseif (~cur_edge_left&&cur_edge_right&&test_edge_left&&~test_edge_right)
	if (vertices(1,exon_idx)+MISALIGN>=vertices(1,test_exon_idx))&&...
	      (vertices(2,exon_idx)>=vertices(2,test_exon_idx)-MISALIGN)&&...
	      (vertices(1,exon_idx)<=vertices(2,test_exon_idx))

	  new_vertex = [vertices(1,test_exon_idx);vertices(2,exon_idx)];
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	
	  known = 0;
	  for ix = 1:size(vertices,2)
	    % assert(isequal(find(new_edges),intersect(find(new_edges),find(edges(ix,:)))) == ~any(new_edges & ~edges(ix,:)));
	    if isequal(new_vertex,vertices(:,ix)) && ~any(new_edges & ~edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert((known~=exon_idx)&&(known~=test_exon_idx));
	  
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif to_keep(exon_idx) || to_keep(test_exon_idx)
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end

	% 1001
      elseif (cur_edge_left&&~cur_edge_right&&~test_edge_left&&test_edge_right)
	if (vertices(1,exon_idx)<=vertices(1,test_exon_idx)+MISALIGN)&&...
	      (vertices(2,exon_idx)-MISALIGN<=vertices(2,test_exon_idx))&&...
	      (vertices(2,exon_idx)>=vertices(1,test_exon_idx))

	
	  new_vertex = [vertices(1,exon_idx);vertices(2,test_exon_idx)];
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	  known = 0;
	  for ix = 1:size(vertices,2)
	    % assert(isequal(find(new_edges),intersect(find(new_edges),find(edges(ix,:))))==~any(new_edges & ~edges(ix,:)));
	    if isequal(new_vertex,vertices(:,ix)) && ~any(new_edges & ~edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert((known~=exon_idx)&&(known~=test_exon_idx));
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif to_keep(exon_idx) || to_keep(test_exon_idx)
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end
	  

	    
	  
	    
	% 0111 
      elseif (~cur_edge_left&&cur_edge_right&&test_edge_left&&test_edge_right)
	if (vertices(2,exon_idx)==vertices(2,test_exon_idx))
	  
	  %    (vertices(1,exon_idx)+MISALIGN>=vertices(1,test_exon_idx))
	  
	  % index to the right
	  right_edge = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	  idx = find(vertices(1,:)>right_edge, 1, 'first');
	  
	  
	  if (~isequal(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end)))
	    
	    edges(exon_idx,idx:end) = or(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end));
	    edges(test_exon_idx,idx:end) = edges(exon_idx,idx:end);
	    edges(idx:end,exon_idx) = or(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end));
	    edges(idx:end,test_exon_idx) = edges(idx:end,exon_idx);
	    
	    
	    %to_keep(exon_idx) = 0;
	    
	    changed = 1;
	  end
	  if (vertices(1,exon_idx)+MISALIGN >= vertices(1,test_exon_idx)) && (to_keep(exon_idx))
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	end
	    
	% 1101
      elseif (cur_edge_left&&cur_edge_right&&~test_edge_left&&test_edge_right)
	if (vertices(2,exon_idx)==vertices(2,test_exon_idx))
	  
	  %    (vertices(1,exon_idx)-MISALIGN<=vertices(1,test_exon_idx))
	  
	  % index to the right
	  right_edge = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	  idx = find(vertices(1,:)>right_edge,1,'first');
	
	
	  if (~isequal(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end)))
	    
	    edges(exon_idx,idx:end) = or(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end));
	    edges(test_exon_idx,idx:end) = edges(exon_idx,idx:end);
	    edges(idx:end,exon_idx) = or(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end));
	    edges(idx:end,test_exon_idx) = edges(idx:end,exon_idx);
	    
	    
	    %to_keep(test_exon_idx) = 0;
	    
	    changed = 1;
	  end
	  if (vertices(1,exon_idx)-MISALIGN <= vertices(1,test_exon_idx)) && (to_keep(test_exon_idx))
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	% 1011  
      elseif (cur_edge_left&&~cur_edge_right&&test_edge_left&&test_edge_right)
	if (vertices(1,exon_idx)==vertices(1,test_exon_idx))
	  
	  %(vertices(2,exon_idx)-MISALIGN<=vertices(2,test_exon_idx))
	  % index to the left
	  left_edge = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	  idx = find(vertices(2,:)<left_edge,1,'last');
	  
	  
	  if (~isequal(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx)))
	    
	    edges(exon_idx,1:idx) = or(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx));
	    edges(test_exon_idx,1:idx) = edges(exon_idx,1:idx);
	    edges(1:idx,exon_idx) = or(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx));
	    edges(1:idx,test_exon_idx) = edges(1:idx,exon_idx);
	    
	    
	    %to_keep(exon_idx) = 0;
	      
	    changed = 1;
	  end
	  
	  if (vertices(2,exon_idx)-MISALIGN <= vertices(2,test_exon_idx)) && (to_keep(exon_idx))
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	
	% 1110
      elseif (cur_edge_left&&cur_edge_right&&test_edge_left&&~test_edge_right)
	if (vertices(1,exon_idx)==vertices(1,test_exon_idx))
	  
	  %    (vertices(2,exon_idx)+MISALIGN>=vertices(2,test_exon_idx))
	  
	  % index to the left
	  left_edge = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	  idx = find(vertices(2,:)<left_edge,1,'last') ;

	
	  if (~isequal(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx)))
	    edges(exon_idx,1:idx) = or(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx));
	    edges(test_exon_idx,1:idx) = edges(exon_idx,1:idx);
	    edges(1:idx,exon_idx) = or(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx));
	    edges(1:idx,test_exon_idx) = edges(1:idx,exon_idx);
	  
	    
	    %to_keep(test_exon_idx) = 0;
	    
	    changed = 1;
	  end
	  
	  if (vertices(2,exon_idx)+MISALIGN >= vertices(2,test_exon_idx)) && (to_keep(test_exon_idx))
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	% 1111
      elseif (cur_edge_left&&cur_edge_right&&test_edge_left&&test_edge_right)
	
	if (vertices(1,exon_idx)==vertices(1,test_exon_idx))&&...
	      (vertices(2,exon_idx)==vertices(2,test_exon_idx))
	  if (~isequal(edges(exon_idx,:),edges(test_exon_idx,:)))
	    edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	    edges(test_exon_idx,:) = edges(exon_idx,:);
	    edges(:,exon_idx) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	    edges(:,test_exon_idx) = edges(:,exon_idx);
	  
	    
	    to_keep(test_exon_idx) = 0;
	    
	    changed = 1;
	  elseif (to_keep(test_exon_idx))
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	elseif (vertices(1,exon_idx)==vertices(1,test_exon_idx))&&...
	      (vertices(2,exon_idx)~=vertices(2,test_exon_idx))&&...
	      (to_keep(exon_idx)||to_keep(test_exon_idx))
	  % index to the left
	  %left_edge = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	  %idx = find(vertices(2,:)<left_edge, 1, 'last');
	  idx = find(vertices(2,:)<vertices(1,exon_idx), 1, 'last');

	  if (~isequal(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx)))
	    edges(exon_idx,1:idx) = or(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx));
	    edges(test_exon_idx,1:idx) = edges(exon_idx,1:idx);
	    edges(1:idx,exon_idx) = or(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx));
	    edges(1:idx,test_exon_idx) = edges(1:idx,exon_idx);
	    	    
	    changed = 1;
	  end

	elseif (vertices(1,exon_idx)~=vertices(1,test_exon_idx))&&...
	      (vertices(2,exon_idx)==vertices(2,test_exon_idx))&&...
	      (to_keep(exon_idx)||to_keep(test_exon_idx))
	  % index to the right
	  %right_edge = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	  %idx = find(vertices(1,:)>right_edge,1,'first');
	  idx = find(vertices(1,:)>vertices(2,exon_idx),1,'first');
	
          
	  if (~isequal(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end)))
	  
	    edges(exon_idx,idx:end) = or(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end));
	    edges(test_exon_idx,idx:end) = edges(exon_idx,idx:end);
	    edges(idx:end,exon_idx) = or(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end));
	    edges(idx:end,test_exon_idx) = edges(idx:end,exon_idx);
	    
	    changed = 1;
	  end
	  
	end
	
      else
	fprintf(1,'Unknown case!!\n');
        if keyboard_allowed(), keyboard; end ;
	
      end  % cases
      

      %test_exon_idx = test_exon_idx+1;
      internal_idx = internal_idx + 1;

    end
    exon_idx = exon_idx + 1;
  end


  
  
  

    
  genes(gene_idx).splicegraph = {};
  to_keep = find(to_keep);
  genes(gene_idx).splicegraph = {vertices(:,to_keep),edges(to_keep,to_keep)};
end



fprintf(1,'\n');












return

function genes=reduce_splice_graph_caller(genes) ;
% genes=reduce_splice_graph_caller(genes) ;

for gene_idx = 1:length(genes)
%gene_idx
%genes(gene_idx)
  vertices = genes(gene_idx).splicegraph{1};
  edges = genes(gene_idx).splicegraph{2};
  %fprintf(1,'gene index:\t%d\n',gene_idx);
  if (mod(gene_idx,1000)==0)
    fprintf(1,'%d\n',gene_idx);
  end
  if isempty(vertices), continue ; end ;

  % find all the intron locations
  [dummy,exon_order] = sort(vertices(1,:),2,'ascend');
  vertices = vertices(:,exon_order);
  edges = edges(exon_order,exon_order);
  intron_loc = [];
  for ix1 = 1:size(vertices,2)-1
    for ix2 = 2:size(vertices,2)
      if edges(ix1,ix2)
	intron_loc = union(intron_loc,[vertices(2,ix1)+1:vertices(1,ix2)-1]);
      end
    end
  end
  
  
  
  if (size(edges,1) < 2)
    changed = 1;
    while changed
      changed = 0;
      exon_idx = 1;
      while exon_idx <= size(vertices,2)
	
	test_exon_idx = exon_idx+1;
	while test_exon_idx <= size(vertices,2)
	  if (vertices(1,exon_idx)<=vertices(1,test_exon_idx)) &&...
		(vertices(2,exon_idx)>=vertices(2,test_exon_idx))
	    vertices(1,exon_idx) = min(vertices(1,exon_idx),...
				       vertices(1,test_exon_idx));
	    vertices(2,exon_idx) = max(vertices(2,exon_idx),...
				       vertices(2,test_exon_idx));
	    new_index = [1:test_exon_idx-1,test_exon_idx+1:size(vertices,2)];
	    
	    
	    vertices = vertices(:,new_index);
	  
	    %disp(vertices)
	    
	    %test_exon_idx = exon_idx+1;
	    changed = 1;
	    
	  end
	  test_exon_idx = test_exon_idx+1;
	end
	exon_idx = exon_idx + 1;
      end
    end
    
  else
    changed = 1;
    while changed
      changed = 0;
      exon_idx = 1;
      while exon_idx <= size(vertices,2)

	[dummy,exon_order] = sort(vertices(1,:),2,'ascend');
	if ~isequal(vertices(1,:),dummy)
	  vertices = vertices(:,exon_order);
	  edges = edges(exon_order,exon_order);
	end
	
	test_exon_idx = exon_idx + 1;
	while test_exon_idx <= size(vertices,2)
	  reduce_now = 0;
	  
          if (test_exon_idx < exon_idx) && keyboard_allowed(), keyboard; end;
	  
	  cur_edge_left = sum(edges(1:exon_idx,exon_idx));
	  test_edge_left = sum(edges(1:test_exon_idx,test_exon_idx));
	  cur_edge_right = sum(edges(exon_idx:end,exon_idx));
	  test_edge_right = sum(edges(test_exon_idx:end,test_exon_idx));
	  
	  

	  
	  % 0000
	  if (~cur_edge_left&&~cur_edge_right&&~test_edge_left&&~test_edge_right)
	    
	    if ((vertices(2,exon_idx)>=vertices(1,test_exon_idx))&&...
		(vertices(1,exon_idx)<=vertices(1,test_exon_idx)))||...
		  ((vertices(2,test_exon_idx)>=vertices(1,exon_idx))&&...
		   (vertices(1,test_exon_idx)<=vertices(1,exon_idx)))&&...
		  (sum(ismember([min(vertices(1,exon_idx),vertices(1,test_exon_idx)):...
				 max(vertices(2,exon_idx),vertices(2,test_exon_idx))],intron_loc))==0)
	      
	      vertices(1,exon_idx) = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	      vertices(2,exon_idx) = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	      new_index = [1:test_exon_idx-1,test_exon_idx+1:size(vertices,2)];
	    
	    
	      vertices = vertices(:,new_index);
	      edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	      edges(:,exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	      edges = edges(new_index,new_index);
	    
	  
	    
	      reduce_now = 1;
	      changed = 1;
	    end
	    
	    
	  % 0101
	  elseif (~cur_edge_left&&cur_edge_right&&~test_edge_left&&test_edge_right)

	    if (vertices(2,exon_idx)==vertices(2,test_exon_idx))&&...
		  (sum(ismember([min(vertices(1,exon_idx),vertices(1,test_exon_idx)):...
				 vertices(2,exon_idx)],intron_loc))==0)
	      
	      vertices(1,exon_idx) = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	      new_index = [1:test_exon_idx-1,test_exon_idx+1:size(vertices,2)];
	    
	    
	      vertices = vertices(:,new_index);
	      edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	      edges(:,exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	      edges = edges(new_index,new_index);
	    
	  
	    
	      reduce_now = 1;
	      changed = 1;
	    end

	    
	  % 1010
	  elseif (cur_edge_left&&~cur_edge_right&&test_edge_left&&~test_edge_right)
	    if (vertices(1,exon_idx)==vertices(1,test_exon_idx))&&...
		  (sum(ismember([vertices(1,exon_idx):...
				 max(vertices(2,exon_idx),vertices(2,test_exon_idx))],intron_loc))==0)
	      
	      vertices(2,exon_idx) = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	      new_index = [1:test_exon_idx-1,test_exon_idx+1:size(vertices,2)];
	    
	    
	      vertices = vertices(:,new_index);
	      edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	      edges(:,exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	      edges = edges(new_index,new_index);
	    
	  
	    
	      reduce_now = 1;
	      changed = 1;
	    end

	  
	  % 1111
	  elseif (cur_edge_left&&cur_edge_right&&test_edge_left&&test_edge_right)
	    if (vertices(1,exon_idx)==vertices(1,test_exon_idx))&&...
		 (vertices(2,exon_idx)==vertices(2,test_exon_idx))
	    
	      new_index = [1:test_exon_idx-1,test_exon_idx+1:size(vertices,2)];
	    
	    
	      vertices = vertices(:,new_index);
	      edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	      edges(:,exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	      edges = edges(new_index,new_index);
	      
	      reduce_now = 1;
	      changed = 1;
	  
	    end
	    
	  end % cases
	  

          if ~reduce_now
	    test_exon_idx = test_exon_idx+1;
	  end
	  

	end
	exon_idx = exon_idx + 1;
      end %while exon_idx <= size(vertices,2)
    end %while changed

  
  
  
  end 
    
  genes(gene_idx).splicegraph = {};
  genes(gene_idx).splicegraph = {vertices,edges};
end


fprintf(1,'\n');

return





return

function [tmp_fname,STATS] = gff3_statistics(gff_fname,thesources, contig)
% [tmp_fname,STATS] = gff3_statistics(gff_fname,thesources, contig)
  

if nargin<2
  thesources = [];
end

                                                                                                                                                       
if nargin<3                                                                                     
  contig = [];                                                                                    
end 
                                                                                    

%%% SOFA mapping
SOFA_short_names = {'gene','mRNA','exon','CDS','intron','polyA_sequence','polyA_site','five_prime_UTR','three_prime_UTR'};
SOFA_IDs = {'SO:0000704','SO:0000234','SO:0000147','SO:0000316','SO:0000188','SO:0000610','SO:0000553','SO:0000204','SO:0000205'};

%%% GFF file summary

[fd msg] = fopen(gff_fname, 'r');
disp(msg);

tmp_fname = tempname;
if ~isempty(thesources) 
  fprintf('filtering for source :\n');
  unix(sprintf('echo > %s', tmp_fname)) ;
  for i=1:length(thesources),
    fprintf('%s\n',thesources{i});
    unix(sprintf('grep -e "[[:space:]]%s[[:space:]]" %s |grep -v "#" >> %s', thesources{i}, gff_fname, tmp_fname));
  end 
else
  unix(sprintf('grep -v "#" %s > %s', gff_fname, tmp_fname));
end

fprintf('\n------FILE SUMMARY-----\n');
[tmp,seqids] = unix(sprintf('sed -e "1d" %s | cut -f 1 |sort -u', tmp_fname));
if isempty(contig) 
   fprintf('gff file containes following %i seqids in column 1 :\n',length(seqids));
   if length(seqids)<100
      fprintf('%s\n',seqids)
   end
end

[tmp, sources]=unix(sprintf('sed -e "1d" %s | cut -f 2 |sort -u', tmp_fname));
if isempty(sources)
  error('specified sources not contained in file')
end
fprintf('gff file containes following sources in column 2 :\n');
sources = separate(sources,sprintf('\n'));
sources(end)=[];
for t=1:length(sources)
  [tmp,num_sources]=unix(sprintf('cut -f 2 %s |grep -e "^%s$" | wc -l',tmp_fname,sources{t}));
  fprintf('%s\t%i\n',sources{t},str2num(num_sources));
end
  
if ~isempty(contig) 
  fprintf('filtering for contig :\n');
  fprintf('^%s[[:space:]]\n',contig);

  seqids_array = separate(upper(seqids), sprintf('\n')) ;
  if ~isempty(seqids_array) && isempty(seqids_array{end})
    seqids_array(end)=[] ;
  end ;

  cnamepos = strmatch(upper(contig), seqids_array, 'exact');
  % contig = seqids(cnamepos:cnamepos+length(contig)-1); 
  if isempty(cnamepos)%all(isempty(strfind(upper(seqids),upper(contig))))
    fprintf('contig not found in file\n')
    return
  end
  tmp_fname1 = tempname;
  unix(sprintf('grep -e "^%s[[:space:]]" %s > %s', contig, tmp_fname, tmp_fname1));
  [tmp,num_seqs] = unix(sprintf('sed -e "1d" %s | cut -f 1 |sort -u | wc -l',tmp_fname1));
  if (str2num(num_seqs)~=1)
    error('grep for contig doesnt work, repeat with by_contig=0')
  end
else
  tmp_fname1 = tmp_fname;
end
[tmp,types_]=unix(sprintf('sed -e "1d" %s | cut -f 3 |sort -u',tmp_fname1));
fprintf('\nfiltered gff file containes following types in column 3 :\n');
types = separate(types_,sprintf('\n')) ;
idx = find(cellfun('isempty',types));
types(idx)=[];


for t=1:length(types)
  [tmp,num_types]=unix(sprintf('cut -f 3 %s | grep -e "^%s$" |wc -l',tmp_fname1,types{t}));
  fprintf('%s\t%i\n',types{t},str2num(num_types));
  STATS.(types{t})=str2num(num_types);
end
% assert(all(ismember(lower(types),lower(SOFA_short_names)))|all(ismember(types,SOFA_IDs)))

tmp_fname= tmp_fname1;

return

function genes = init_genes(num_genes,PAR,genes)
% genes = init_genes(num_genes,PAR,genes)
  
if nargin<1
  num_genes=0;
end
if nargin<2 || isempty(PAR),
  PAR.organism.name =[];
end
    
genes.id = [];
genes.anno_id = [];
genes.confgenes_id = [];
genes.name = '' ;
genes.alias = [];
genes.name2 = []; 
genes.strand = '';
genes.chr = '';
genes.chr_num = [];
genes.paralogs = [];

genes.start = [];
genes.stop = [];

genes.transcripts = {};
genes.transcript_info = [];
genes.transcript_status = [];
genes.transcript_valid = [];
genes.exons = {}; 
genes.exons_confirmed = {};
genes.cds_exons = {}; 
genes.utr5_exons = {}; 
genes.utr3_exons = {}; 
genes.tis = {};
genes.tis_conf = [];
genes.tis_info = [];

genes.cdsStop = {};
genes.cdsStop_conf = [];
genes.cdsStop_info = [];

if isequal(PAR.organism.name,'C_elegans')||(isfield(PAR.organism,'clade')&&isequal(PAR.organism.clade,'nematode'))
  genes.transacc = {};
  genes.transacc_info = [];
  genes.transacc_conf = [];
end
genes.tss = {};
genes.tss_info = [];
genes.tss_conf = [];
genes.cleave = {};
genes.cleave_info = [];
genes.cleave_conf = [];
genes.polya = {};
genes.polya_info = [];
genes.polya_conf = [];

genes.is_alt = [];
genes.is_alt_spliced = [];
genes.is_valid = [];
genes.transcript_complete = [];
genes.is_complete = [];
genes.is_correctly_gff3_referenced = [];

genes.splicegraph =[];

if isequal(PAR.organism.name,'C_elegans')||(isfield(PAR.organism,'clade')&&isequal(PAR.organism.clade,'nematode'))
  genes.in_operon=[];
end

if num_genes==0
  genes=genes([]);
elseif num_genes>1,
  genes(num_genes) = genes ;
end

return

function genome_info=init_genome(config_fname)
% genome_info=init_genome(config_fname)  
%
% global g_genome_info ;

global g_genome_info genome_strings;

genome_strings = [];
g_est_fnames={} ;
g_prot_fnames={} ;
g_cdna_fnames={} ;
g_flcdna_fnames={} ;
g_contig_names={} ;
g_flat_fnames={} ;
g_fasta_fnames={} ;
g_annotation_fnames={} ;
g_alphabet='acgt' ;
g_basedir='' ;
g_max_intron_len=1000000 ;
g_min_intron_len=10 ;
g_merge_est_transcripts=1 ;

fd=fopen(config_fname, 'r') ;
while ~feof(fd),
  line = fgetl(fd) ;
  if ~ischar(line), break ; end ;
  if isempty(line), continue ; end ;
  if line(1)=='#', continue ; end ;
  if length(line)>8 && isequal(line(1:7),'BASEDIR')
    g_basedir=deblank(line(9:end)) ;
  elseif length(line)>21 && isequal(line(1:21), 'MERGE_EST_TRANSCRIPTS')
    g_merge_est_transcripts = str2num(line(22:end)) ; 
  elseif length(line)>15 && isequal(line(1:15),'MAX_DOUBLE_QGAP')
  elseif length(line)>15 && isequal(line(1:15),'MAX_DOUBLE_TGAP')
  elseif length(line)>17 && isequal(line(1:17),'MAX_CDNA_DELETION')
  elseif length(line)>18 && isequal(line(1:18),'MAX_CDNA_INSERTION')
  elseif length(line)>21 && isequal(line(1:21),'TERMINAL_EXON_END_TOL')
  elseif length(line)>14 && isequal(line(1:14),'MIN_INTRON_LEN')
    g_min_intron_len = str2num(line(15:end)) ; 
    assert(~isempty(g_min_intron_len) & g_min_intron_len<1000) ;
  elseif length(line)>14 && isequal(line(1:14),'MAX_INTRON_LEN')
    g_max_intron_len = str2num(line(15:end)) ; 
    assert(~isempty(g_max_intron_len) & g_max_intron_len>1000) ;
  elseif length(line)>18 && isequal(line(1:18),'MIN_EST_COVER_FRAC')

  elseif length(line)>19 && isequal(line(1:19),'MIN_PROT_COVER_FRAC')

  elseif length(line)>19 && isequal(line(1:19),'MIN_CDNA_COVER_FRAC')
  
  elseif length(line)>18 && isequal(line(1:18),'BLAT_BEST_HIT_ONLY')
  
  elseif length(line)>20 && isequal(line(1:20),'BLAT_BEST_HIT_MARGIN')

  elseif length(line)>8 && isequal(line(1:8),'ALPHABET')
    g_alphabet=deblank(line(10:end)) ;
  elseif length(line)>7 && isequal(line(1:7),'CONTIGS'),
    for i=1:str2num(line(8:end))
      l2=fgetl(fd) ; 
      idx=strfind(l2,'  ') ;
      while ~isempty(idx), 
        l2(idx)=[] ;
        idx=strfind(l2,'  ') ;
      end ;
      l2(l2==' ')=sprintf('\t') ;
      elems=separate(l2) ; 
      assert(length(elems)==3) ;
      g_contig_names{i}=elems{1} ;
      g_flat_fnames{i}=path_with_basedir(elems{2}, g_basedir) ;
      g_fasta_fnames{i}=path_with_basedir(elems{3}, g_basedir) ; 
    end ;
  elseif length(line)>8 && isequal(line(1:8),'ESTFILES'),
    for i=1:str2num(line(9:end))
      g_est_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>9 && isequal(line(1:9),'PROTFILES'),
    for i=1:str2num(line(10:end))
      g_prot_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>9 && isequal(line(1:9),'CDNAFILES'),
    for i=1:str2num(line(10:end))
      g_cdna_fnames{end+1}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>11 && isequal(line(1:11),'FLCDNAFILES'),
    for i=1:str2num(line(12:end))
      g_flcdna_fnames{end+1}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>15 && isequal(line(1:15),'ANNOTATIONFILES'),
    for i=1:str2num(line(16:end))
      g_annotation_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif ~isempty(line),
    error(sprintf('cannot understand "%s" in config file', line)) ;
  end ;
end ;
fclose(fd) ;

genome_info=[] ;
genome_info.est_fnames=g_est_fnames ;
genome_info.prot_fnames=g_prot_fnames ;
genome_info.cdna_fnames=g_cdna_fnames ;
genome_info.flcdna_fnames=g_flcdna_fnames ;
genome_info.contig_names=g_contig_names ;
genome_info.flat_fnames=g_flat_fnames ;
genome_info.fasta_fnames=g_fasta_fnames ;
genome_info.annotation_fnames=g_annotation_fnames ;
genome_info.alphabet=g_alphabet ;
genome_info.basedir=g_basedir ;
genome_info.max_intron_len = g_max_intron_len ;
genome_info.min_intron_len = g_min_intron_len ;
genome_info.merge_est_transcripts = g_merge_est_transcripts ;
g_genome_info=genome_info ;

return ;


function p=path_with_basedir(p1,basedir)
  
  if p1(1)=='/',
    p=p1; 
    return ;
  else
    if basedir(end)=='/',
      p=[basedir p1] ;
    else
      p=[basedir '/' p1] ;
    end ;
  end ;
  
  

return

function [str,valid_splice,ok] = load_genomic(contig,strand,start,stop, genome_info,check_splice)
% function [str,valid_splice,ok]=load_genomic(contig,strand,start,stop, genome_info,check_splice)

if nargin<6
  check_splice=0;
end ;
ok=1 ;

if ~check_splice 
  valid_splice=[];
  % fprintf(1,'no splice site consensus check!')
end ;


if length(start)>1 | length(stop)>1,
  if nargout<3
    assert(length(start)==length(stop)) ;
    assert(all([start(2:end)-stop(1:end-1)]>=0));
  else
    if ~(length(start)==length(stop)) || ~(all([start(2:end)-stop(1:end-1)]>=0)),
      ok=0 ;
      str='' ;
      valid_splice = [] ;
      return ;
    end ;
  end ;
  valid_splice = zeros(length(start),2); 
  if strand == '+', 
    idx = 1:length(start) ; 
  else 
    idx = length(start):-1:1 ; 
  end ;
  str = '' ;
  for i=idx  %1:length(start)
    seq = load_genomic(contig, strand, start(i)-5, stop(i)+5, genome_info) ;
    str = [str seq(6:end-5)] ;

    % check internal splicing sites
    if check_splice
      % if (i~=1 && strand == '+') || (i~=length(start) && strand == '-')
      if (strand == '+') || (i~=length(start) && strand == '-')
        valid_splice(i,1) = isequal(seq(4:5),'ag') ;
        %if ~isequal(seq(4:5),'ag'), fprintf('acc: %s %c\n', seq(3:6), strand); end ;
        % assert(isequal(seq(4:5),'ag'))
      end ;
      %if i~=idx(end)
      % if (i~=1 && strand == '-') || (i~=length(start) && strand == '+')
      if (strand == '-') || (i~=length(start) && strand == '+')        
        valid_splice(i,2) = ismember(seq(end-5:end-4), {'gt','gc'}) ;
        %if ~isequal(seq(end-4:end-3),'gt')||isequal(seq(end-4:end-3),'gc'), fprintf('don: %s %c\n', seq(end-5:end-2), strand); end ;
        % assert(isequal(seq(end-4:end-3),'gt')||isequal(seq(end-4:end-3),'gc'))
      end ; 
    end ;
  end
  return ;
else
  valid_splice=[0 0];
end ;


if ischar(contig),
  %contig_idx = find(ismember(upper(genome_info.contig_names),upper(contig))) ;
  contig_idx = strmatch(upper(contig), upper(genome_info.contig_names), 'exact') ;
  assert(all(size(contig_idx)==1) & ~isempty(contig_idx)) ;
else
  contig_idx = contig ;
end ;
  
fname=genome_info.flat_fnames{contig_idx} ;
if genome_info.alphabet(1)=='a',
  NN='n' ;
else
  NN='N' ;
end ;

d=dir(fname) ; left_n = 0 ; right_n=0 ;
if isinf(stop), stop=d.bytes; end ;

if start<1 | stop>d.bytes,
  warning('load_genomic:contig_boundary', 'boundary of contig reached (start: %i, stop: %i, d.bytes: %i), padding with "n"', start, stop,d.bytes) ;

  if start<1,
    left_n = -(start-1) ;
    start = 1 ;
  end ;
  if stop>d.bytes,
    right_n = stop-d.bytes ;
  end ;
  if start>d.bytes && stop>d.bytes,
    str = char(NN*ones(1,stop-start)) ;
    return ;
  end ;
end ;

fd=fopen(fname,'r') ;
fseek(fd, start-1, -1) ;
str=char(fread(fd,stop-start+1, 'char=>char'))' ;
fclose(fd) ;
if left_n>0,
  str=char([NN*ones(1,left_n) str]) ;
end ;
if right_n>0,
  str=char([str NN*ones(1,right_n)]) ;
end ;

assert(length(str)==stop-start+left_n+1)
%let = bigunique(str) ;
let = unique(str) ;
if genome_info.alphabet(1)=='a' && ~isempty(setdiff(let,'acgtn')),
  % warning(sprintf('invalid letter in genome! ("%s")', setdiff(let,'acgtn')));
  let = setdiff(let, 'acgtn') ;
  for c=let,
    str(str==c) = 'n';
  end ;
elseif genome_info.alphabet(1)=='A' && ~isempty(setdiff(let,'ACGTN')),
  warning(sprintf('invalid letter in genome! ("%s")', setdiff(let,'ACGTN')));
  let = setdiff(let, 'ACGTN') ;
  for c=let,
    str(str==c) = 'N';
  end ;
end

if strand=='-',
  str=reverse_complement(str) ;
end ;

return ;

return

function [genes, FAULTS] = parse_gff3(gff_fname, save_fname, thesources, contig, TYPE,log_fname)
%        genes = parse_gff3(gff_fname, save_fname, thesources, contig, TYPE.log_fname)
% can also be used for gff2 format:
% TYPE.short_names = {'CDS','exon','coding_exon'};
% TYPE.IDs = {'1' '2' '3'};
% TYPE.hierachies = [2 3 3];
  
if nargin<3
  thesources = [];
end
  
  
if nargin<4
  contig = [];
end

if nargin<5||isempty(TYPE)
  %%% SOFA mapping
  SOFA_short_names = {'gene','mRNA','exon','CDS','intron','polyA_sequence','polyA_site','five_prime_UTR','three_prime_UTR'};
  SOFA_IDs = {'SO:0000704','SO:0000234','SO:0000147','SO:0000316','SO:0000188','SO:0000610','SO:0000553','SO:0000204','SO:0000205'};
  hierachies = [1 2 3 3 3 3 3 3 3];
else 
  SOFA_short_names = TYPE.short_names;
  SOFA_IDs = TYPE.IDs;
  hierachies = TYPE.hierachies;
end
 
status={'Predicted','Partially_confirmed','Confirmed'};

%%% GFF file summary

filtered_fname = gff3_statistics(gff_fname, thesources, contig);


if ~isempty(save_fname) && fexist(sprintf('%s.mat', save_fname))
  warning(sprintf('file %s.mat already exists\n', save_fname));
  %fprintf('\nwill be overwritten %s\n', save_fname);
  %load(save_fname,'genes');
  %assert(length(genes)==num_genes) 
end

FAULTS.Gene.no_strand = {};
FAULTS.Gene.no_id = {};
FAULTS.Gene.id_not_unique = {};
FAULTS.Transcript.unknown_parent = {};
FAULTS.Transcript.no_gene = {};
FAULTS.Transcript.diff_strand_contig = {};
FAULTS.Transcript.id_not_unique = {};
FAULTS.Transcript.outside_boundaries = {};
FAULTS.Exon.no_transcript = {};
FAULTS.Exon.diff_strand_contig = {};


% first parse: genes only are extracted 

types = SOFA_short_names(hierachies==1);
types_IDs = SOFA_IDs(hierachies==1);

fprintf('\n------starting first parse: only look for the following types\n');

tmp_fname1 = tempname;
unix(sprintf('touch %s',tmp_fname1));
for i=1:length(types)
  [tmp,num] = unix(sprintf('cut -f 3 %s | grep -e "^%s$" |wc -l',filtered_fname,types{i}));
  unix(sprintf('grep -e "[[:space:]]%s[[:space:]]" %s >> %s',types{i},filtered_fname,tmp_fname1));
  num_genes(i) = str2double(num);
  fprintf('%s\t%i\n',types{i},num_genes(i));
end

num_genes = sum(num_genes);
if num_genes ==0
  warning('no genes found; looking for hierachy 2 (mRNAs instead)\n')
  types = SOFA_short_names(hierachies==2);
  types_IDs = SOFA_IDs(hierachies==2);
  for i=1:length(types)
    [tmp,num] = unix(sprintf('cut -f 3 %s | grep -e "^%s$" |wc -l',filtered_fname,types{i}));
    unix(sprintf('grep -e "[[:space:]]%s[[:space:]]" %s >> %s',types{i},filtered_fname,tmp_fname1));
    num_genes(i) = str2double(num);
    fprintf('\t%s\t%i\n',types{i},num_genes(i));
  end
  num_genes
  num_genes = sum(num_genes);
end
if num_genes ==0
   warning('no genes found')
  return
end

genes = init_genes(num_genes);
[fd msg] = fopen(tmp_fname1, 'r');

genes_cnt = 0;
gene_names = {} ;
while ~feof(fd),

  attributes = {} ;
  type_name = '' ;
  elems = {} ;
  
  
  Line = fgetl(fd);
  elems = separate(Line);
  assert(~isempty(elems{1}));
  
  if Line(1)=='#',
    continue ;
  end
  if ~isempty(contig)
    if ~isequal(upper(elems{1}),upper(contig)),
      continue ;
    end
  end
  if ~ischar(Line),
    break ;
  end
  assert(str2double(elems{4})<=str2double(elems{5}) || str2double(elems{5})<0)
  if~(elems{7}=='+' || elems{7}=='-') ;
    if elems{7}~='.',
      FAULTS.Gene.no_strand{end+1} = elems{9};
      continue
      % warning('strand missing')
    end ;
  end
  
  if ~isempty(thesources)
    if isempty(strmatch(elems{2}, thesources))
      error('something wrong with source')
    end  
  end
  
  %if ~ismember(elems{3},types) && ~ismember(elems{3},types_IDs)
  if isempty(strmatch(elems{3}, {types{:} types_IDs{:}}))
    continue ;
  end
  attributes = separate(elems{9}, ';');
  
  if ismember(elems{3},types);
    type_name = elems{3};
  elseif ismember(elems{3},types_IDs);
    type_name =types(isequal(elems{3},types_IDs));
  else
    error('wrong type')
  end
  

  identifier = '';    alias = '';
  for i=1:length(attributes),
    id=strfind(attributes{i},'ID=');
    if ~isempty(id),
      identifier = attributes{i}(id+3:end);
      break;
      % keyboard
    end
  end
  if isempty(identifier)
    FAULTS.Gene.no_id{end+1} = elems{9};
    % warning('no ID found 1')
    identifier = attributes{1};
  end
      
  for i=1:length(attributes) 
    id=strfind(attributes{i},'Alias=');
    if ~isempty(id),
      alias = attributes{i}(id+6:end);
      break;
    end 
  end
  if length(identifier)>=3 
    assert(~isequal(identifier(1:3),'ID=')) ;
  end
  assert(~isempty(identifier));

  other_idx=strmatch(identifier,gene_names,'exact');
  if ~isempty(other_idx)
    % warning('gene appears more than once\n')
    FAULTS.Gene.id_not_unique{end+1} = elems{9}; 
    assert(isequal(genes(other_idx).alias,alias));
    assert(isequal(genes(other_idx).chr,elems{1}));
    assert(isequal(genes(other_idx).strand,elems{7})) ;
    if ~isequal(genes(other_idx).start,str2double(elems{4}))
      genes(other_idx).start = min(genes(other_idx).start,str2double(elems{4}));
    end
    if ~isequal(genes(other_idx).stop,str2double(elems{5}))
      genes(other_idx).stop = max(genes(other_idx).stop,str2double(elems{5}));
    end 
    % assert(isequal(genes(other_idx).start,str2double(elems{4})));
    % assert(isequal(genes(other_idx).stop,str2double(elems{5})));
    continue
  end
  genes_cnt =  genes_cnt +1;
  genes(genes_cnt).name = identifier;
  genes(genes_cnt).alias = alias;
  genes(genes_cnt).chr = elems{1};
  genes(genes_cnt).strand = elems{7} ;
  genes(genes_cnt).start(end+1) = str2double(elems{4});
  genes(genes_cnt).stop(end+1) = str2double(elems{5});
  genes(genes_cnt).is_correctly_gff3_referenced = 1 ;

  gene_names{end+1} = identifier ;
  if mod(genes_cnt,1000)==0||genes_cnt==num_genes
    fprintf('  parsed %i genes\r', genes_cnt);  
  end
end % end first parse
% assert(genes_cnt==num_genes)
genes=genes(1:genes_cnt);
fclose(fd);

fprintf('\n\n');
if length(FAULTS.Gene.no_strand)>0 || length(FAULTS.Gene.no_id)>0 || length(FAULTS.Gene.id_not_unique)>0,
  fprintf('Following problems were found:\n') ;
else
  fprintf('No problems found.\n') ;
end ;
if length(FAULTS.Gene.no_strand)>0,
  fprintf('number of genes without annotated strand (column 7):\t\t %i\n', length(FAULTS.Gene.no_strand)) ;
  fprintf('\t=> genes are ignored\n') ;
end 
if length(FAULTS.Gene.no_id)>0,
  fprintf('number of genes without annotated ID (column 9, ID=... ):\t %i\n', length(FAULTS.Gene.no_id)) ;
  fprintf('\t=> first part (to the first semicolon) of attribute field (column 9) is used as ID\n') ;
end
if length(FAULTS.Gene.id_not_unique)>0,
  fprintf('number of genes with non-unique ID (column 9, ID=... ):\t\t %i\n', length(FAULTS.Gene.id_not_unique)) ;
  fprintf('\t=> all on the same seqid and strand, min of start (column 4) and max of end (column 5) used\n ') ;
end ;

unix(sprintf('rm %s',tmp_fname1));




% second parse: mRNAs only are extracted 

types = SOFA_short_names(hierachies==2);
types_IDs = SOFA_IDs(hierachies==2);

fprintf('\n------starting second parse: only look for the following types\n');
tmp_fname1 = tempname;
unix(sprintf('touch %s',tmp_fname1));
num_transcripts = zeros(1,length(types)) ;
for i=1:length(types)
  [tmp,num] = unix(sprintf('cut -f 3 %s | grep -e "^%s$" |wc -l',filtered_fname,types{i}));
  unix(sprintf('grep -e "[[:space:]]%s[[:space:]]" %s >> %s',types{i},filtered_fname,tmp_fname1));
  num_transcripts(i) = str2double(num);
  fprintf('%s\t%i\n',types{i},num_transcripts(i));
end

num_transcripts = sum(num_transcripts);
transcript_gene_map = zeros(1,num_transcripts);
transcript_names = cell(1,num_transcripts);
transcript_ids = cell(1,num_transcripts);
gene_names = {genes.name} ;

[fd msg] = fopen(tmp_fname1, 'r');
transcript_cnt = 0;
while ~feof(fd),

  attributes = {} ;
  type_name = '' ;
  elems = {} ;
  
  
  Line = fgetl(fd);
  elems = separate(Line);
  assert(~isempty(elems{1}));
  
  if Line(1)=='#',
    continue ;
  end
  if ~isempty(contig)
    if ~isequal(upper(elems{1}),upper(contig)),
      continue ;
    end
  end
  if ~ischar(Line),
    break ;
  end
  assert(str2double(elems{4})<=str2double(elems{5}) || str2double(elems{5})<0)
  if~(elems{7}=='+' || elems{7}=='-') ;
    if elems{7}~='.',
      % warning('strand missing')
    end ;
  end
  
  if ~isempty(thesources)
    if isempty(strmatch(elems{2}, thesources))
      error('something wrong with source')
    end  
  end
  
  if ~ismember(elems{3},types) && ~ismember(elems{3},types_IDs)
    continue ;
  end
  attributes = separate(elems{9}, ';');
  
  if ismember(elems{3},types);
    type_name = elems{3};
  elseif ismember(elems{3},types_IDs);
    type_name =types(isequal(elems{3},types_IDs));
  else
    error('wrong type')
  end

  identifier = ''; parent = ''; pred_status = '';
  for i=1:length(attributes),
    id=strfind(attributes{i},'ID=');
    if ~isempty(id),
      identifier = attributes{i}(id+3:end);
    end 
    id=strfind(upper(attributes{i}),'PARENT=');
    if ~isempty(id),
      parents=separate(attributes{i}(id+7:end), ',') ;
      parent =parents{1} ;
    end
    id=strfind(attributes{i},'prediction_status=');
    if ~isempty(id),
      pred_status = attributes{i}(id+18:end);
      pred_status = strmatch(pred_status,status)-2;
    end
  end
  if isempty(identifier)&&isempty(parent)% &length(attributes)==1
    identifier = attributes{1};
    parent = attributes{1};
  end    
  parent_idx=strmatch(parent,gene_names,'exact');
  if isempty(parent_idx)
    parent_idx=strmatch(identifier,gene_names,'exact');
  end
  if isempty(parent_idx),
    if isempty(parent),
      FAULTS.Transcript.unknown_parent{end+1} = elems{9};
      % warning(sprintf('Empty parent: using transcript name as parent (%s)', identifier)) ;
      parent = identifier ;
    end 
    %keyboard
    % warning(sprintf('Could not find parent gene ("%s"): Created a new gene descriptor', parent)) ;
    FAULTS.Transcript.no_gene{end+1} = elems{9};
    genes(end+1).name = parent ;
    genes(end).alias = parent ;
    genes(end).chr = elems{1};
    genes(end).strand = elems{7} ;
    genes(end).start(end+1) = 0 ;
    genes(end).stop(end+1) = inf ;
    genes(end).is_correctly_gff3_referenced = 0 ;
    gene_names = {genes.name} ;
    parent_idx=length(genes) ;
  end ;
  if ~isequal(elems{1}, genes(parent_idx).chr) ;
    FAULTS.Transcript.diff_strand_contig{end+1} = elems{9};
  end
  if ~isequal(elems{7}, genes(parent_idx).strand) ;
    FAULTS.Transcript.diff_strand_contig{end+1} = elems{9};
    % warning(sprintf('strand of parent gene (%s) different',genes(parent_idx).name))
    continue
  end
  assert(~isempty(parent_idx))
  if ~isempty(genes(parent_idx).transcripts)
    other_idx = strmatch(identifier,genes(parent_idx).transcripts,'exact');
    if ~isempty(other_idx);
      FAULTS.Transcript.id_not_unique{end+1} = elems{9};
      % warning('transcript appears more than once!')
      if ~isequal(genes(parent_idx).start(other_idx), str2double(elems{4})) 
        genes(parent_idx).start = min(genes(parent_idx).start(other_idx),str2double(elems{4}));             
      end 
      if ~isequal(genes(parent_idx).stop(other_idx), str2double(elems{5}))            
        genes(parent_idx).stop(other_idx) = max(genes(parent_idx).stop(other_idx),str2double(elems{5}));             
      end
      continue
    end
  end
  genes(parent_idx).transcripts{end+1} = identifier;
  if ~isempty(pred_status)
    genes(parent_idx).transcript_status(end+1) = pred_status;
  end
  if isempty(genes(parent_idx).exons) 
    genes(parent_idx).exons = cell(1,1) ;
  else
    genes(parent_idx).exons{end+1} = [] ;
  end
  %transcript must lie within gene boundaries
  if ~(str2double(elems{4})>=genes(parent_idx).start(1)&&str2double(elems{4})<=genes(parent_idx).stop(1))
    FAULTS.Transcript.outside_boundaries{end+1} = elems{9};
    % warning(sprintf('transcript outside gene boundaries (%s; %s)', identifier, parent)) ;
  end
  if ~(str2double(elems{5})>=genes(parent_idx).start(1)&&str2double(elems{5})<=genes(parent_idx).stop(1))
    FAULTS.Transcript.outside_boundaries{end+1} = elems{9};
    % warning(sprintf('transcript outside gene boundaries (%s; %s)', identifier, parent)) ;
  end
  genes(parent_idx).start(end+1) = str2double(elems{4});
  genes(parent_idx).stop(end+1) = str2double(elems{5}); 
  transcript_cnt = transcript_cnt +1;
  transcript_gene_map(transcript_cnt) = parent_idx;
  transcript_names{transcript_cnt} = identifier;
  tmp = separate(identifier,':');
  if length(tmp)>1
    transcript_ids{transcript_cnt} = tmp{2}; 
  else
    transcript_ids{transcript_cnt} = tmp{1}; 
  end
  if mod(transcript_cnt,1000)==0||transcript_cnt == num_transcripts
    fprintf('  parsed %i transcripts\r', transcript_cnt);  
  end
end %end second parse
transcript_names = transcript_names(1:transcript_cnt); 
transcript_ids = transcript_ids(1:transcript_cnt); 

% assert(transcript_cnt==num_transcripts)
fclose(fd);
unix(sprintf('rm %s',tmp_fname1));

fprintf('\n\n');
if length(FAULTS.Transcript.unknown_parent)>0 || length(FAULTS.Transcript.no_gene)>0 || length(FAULTS.Transcript.diff_strand_contig)>0 || length(FAULTS.Transcript.id_not_unique)>0 || length(FAULTS.Transcript.outside_boundaries)
  fprintf('Following problems were found:\n')
else
  fprintf('No problems found.\n')
end ;
if length(FAULTS.Transcript.unknown_parent)>0 
  fprintf('Number of transcripts without annotated parent (keyword ''Parent='' missing in column 9):\t %i\n',length(FAULTS.Transcript.unknown_parent))
  fprintf('\t=> using ID as Parent\n')
end
if length(FAULTS.Transcript.no_gene)>0
  fprintf('Number of transcripts without annotated parent gene:\t\t\t %i\n',length(FAULTS.Transcript.no_gene))
  fprintf('\t=> created a new gene descriptor\n')
end
if length(FAULTS.Transcript.diff_strand_contig)>0
  fprintf('Number of transcripts with different strand or contig than parent gene:\t %i\n',length(FAULTS.Transcript.diff_strand_contig))
  fprintf('\t=> transcript ignored\n')
end
if length(FAULTS.Transcript.id_not_unique)>0
  fprintf('Number of transcripts with non-unique ID (column 9, ID=... ):\t\t %i\n',length(FAULTS.Transcript.id_not_unique))
  fprintf('\t=> only one transcript added, min of start (column 4) and max of end (column 5) used\n')
end
if length(FAULTS.Transcript.outside_boundaries)
  fprintf('Number of transcripts outside boundaries of gene:\t\t\t %i\n',length(FAULTS.Transcript.outside_boundaries))
  fprintf('\t=> gene boundaries extended\n')
end ;

% correcting gene boundaries for orphan transcripts
for i=genes_cnt+1:length(genes),
  genes(i).start(1) = min(genes(i).start(2:end)) ;
  genes(i).stop(1) = max(genes(i).stop(2:end)) ;
end ;

% third parse: genes are filled with exons

% keyboard
types = SOFA_short_names(hierachies==3);
types_IDs = SOFA_IDs(hierachies==3);
fprintf('\n------starting third parse: only look for the following type\n');
tmp_fname1 = tempname;
unix(sprintf('touch %s',tmp_fname1));

for i=1:length(types)
  [tmp,num] = unix(sprintf('cut -f 3 %s | grep "^%s$" |wc -l',filtered_fname,types{i}));
  unix(sprintf('grep -e "[[:space:]]%s[[:space:]]" %s >> %s',types{i},filtered_fname,tmp_fname1));
  num_segments(i) = str2num(num);
  fprintf('%s\t%i\n',types{i},num_segments(i));
end
num_segments = sum(num_segments);

%num_segments = zeros(1,length(types)) ;
%for i=1:length(types)
%  [tmp,num] = unix(sprintf('cut -f 3 %s | grep %s |wc -l',tmp_fname,types{i}));
%  num_segments(i) = str2double(num) ;
%  fprintf('%s\t%i\n',types{i},num_segments(i));
%end
%num_segments = sum(num_segments);

%disp('creating hash table for transcript names') ;
%[transcript_hash_names, transcript_hash_idx]= init_hash_search(transcript_names) ;

[fd msg] = fopen(tmp_fname1, 'r');
disp(msg);
segment_cnt = 0;
segment_skipped=0;
while ~feof(fd),

  attributes = {} ;
  type_name = '' ;
  elems = {} ;
  
  Line = fgetl(fd);
  elems = separate(Line);
  assert(~isempty(elems{1}));
  
  if Line(1)=='#',
    continue ;
  end
  if ~isempty(contig)
    if ~isequal(upper(elems{1}),upper(contig)),
      continue ;
    end
  end
  if ~ischar(Line),
    break ;
  end
  assert(str2double(elems{4})<=str2double(elems{5}) || str2double(elems{5})<0)
  if~(elems{7}=='+' || elems{7}=='-') ;
    if elems{7}~='.',
      % warning('strand missing')
    end ;
  end
  
  if ~isempty(thesources)
    if isempty(strmatch(elems{2}, thesources))
      error('something wrong with source')
    end  
  end
  
  if ~ismember(elems{3},types) && ~ismember(elems{3},types_IDs)
    continue ;
  end
  attributes = separate(elems{9}, ';');
  
  if ismember(elems{3},types);
    type_name = elems{3};
  elseif ismember(elems{3},types_IDs);
    type_name =types(isequal(elems{3},types_IDs));
  else
    error('wrong type')
  end

  switch type_name ,
   case {'CDS','coding_exon'}
    type = 3;
   case 'three_prime_UTR'
    type = 4;
   case 'five_prime_UTR'
    type = 2; 
   case 'exon'
    type = 0;
   otherwise
    segment_skipped = segment_skipped+1;
    % disp(Line);
    continue
  end
  
  parent = '';
  for i=1:length(attributes), 
    id=strfind(upper(attributes{i}),'PARENT=');
    if ~isempty(id),
      parent=attributes{i}(id+7:end) ;
      %parent = parents{1} ;
    end
  end
  if isempty(parent) %&  length(attributes)==1
    parent = attributes{1};
  end
  assert(~isempty(parent));
  parents = separate(parent, ',') ;
  for parent=parents,

    if ~ischar(parent),
      assert(length(parent)==1) ;
      parent=parent{1} ;
    end ;

    parent_idx = strmatch(parent, transcript_names, 'exact');
    %parent_idx = hash_search(transcript_hash_names, transcript_hash_idx, parent) ;
    %assert(isequal(parent_idx, parent_idx_)) ;

    id_seg = separate(parent, ':') ;
    if length(id_seg)>1
      id_seg = id_seg{2} ;
    end
    if isempty(parent_idx)
      id_seg = separate(parent, ':') ;
      if length(id_seg)>1
        id_seg = id_seg{2} ;
      end
      parent_idx=strmatch(id_seg, transcript_ids, 'exact');
    end
    gene_idx = [];
    if isempty(parent_idx),
      gene_idx =strmatch(parent,gene_names,'exact');
    end

    if isempty(parent_idx)&&isempty(gene_idx),
      % make sure that we only create a parent, if none of the parents exists
      assert(~isempty(parent)) ;
      % warning(sprintf('Could not find parent transcript ("%s"): Created a new gene and transcript descriptor', parent)) ;
      FAULTS.Exon.no_transcript{end+1} = elems{9};
      genes(end+1).name = [parent '-gene'] ;
      genes(end).alias = parent ;
      genes(end).chr = elems{1};
      genes(end).strand = elems{7} ;
      genes(end).start(end+1) = 0 ;
      genes(end).stop(end+1) = inf ;
      genes(end).is_correctly_gff3_referenced = 0 ;
      genes(end).transcripts{end+1} = parent ;
      genes(end).exons{end+1} = [] ;
      
      transcript_cnt = transcript_cnt +1;
      transcript_gene_map(transcript_cnt) = length(genes) ;
      transcript_names{transcript_cnt} = parent ;

      % add to hash table
      %[transcript_hash_names, transcript_hash_idx]= init_hash_search(transcript_names, transcript_cnt, transcript_hash_names, transcript_hash_idx) ;
      break ;
    else
      break ;
    end ;
  end ;

  for parent = parents,
    segment_invalid = 0 ;
    if ~ischar(parent),
      parent=parent{1} ;
    end ;

    parent_idx = strmatch(parent, transcript_names, 'exact') ;
    %parent_idx = hash_search(transcript_hash_names, transcript_hash_idx, parent) ;
    %assert(parent_idx==parent_idx_) ;

    if isempty(parent_idx),
      fprintf('parent %s not found\n', parent) ;
      gene_idx = transcript_gene_map(parent_idx);
      if isempty(gene_idx),
        continue ;
      end
      genes(gene_idx).is_correctly_gff3_referenced = 0 ;
      transcript_idx = 1;
    else
      assert(~isempty(parent_idx) && length(parent_idx)==1);
      gene_idx = transcript_gene_map(parent_idx);
      transcript_idx = strmatch(parent, genes(gene_idx).transcripts, 'exact');
    end ;
    
    if isempty(transcript_idx)
      id_seg = separate(parent, ':') ;
      if length(id_seg)>1
        id_seg = id_seg{2} ;
      end
      for j = 1:length(genes(gene_idx).transcripts)
        tmp =  separate(genes(gene_idx).transcripts{j}, ':') ;
        id_trans{j} = tmp{2};
      end
      transcript_idx=strmatch(id_seg,id_trans,'exact');
    end
    assert(~isempty(transcript_idx) & length(transcript_idx)==1);

    if ~isequal(elems{1}, genes(gene_idx).chr)
      segment_invalid = 1 ;
      FAULTS.Exon.diff_strand_contig{end+1} = elems{9};
      % fprintf('Segment''s contig name does not match parent (is %s, should be %s). Ignoring segment.\nLine:\n %s\n', elems{1}, genes(gene_idx).chr, Line) ;
    end ;
    if ~isequal(elems{7}, genes(gene_idx).strand),
      segment_invalid = 1 ;
      FAULTS.Exon.diff_strand_contig{end+1} = elems{9};
      % fprintf('Segment''s strand name does not match parent (is %s, should be %s). Ignoring segment.\nLine:\n %s\n', elems{7}, genes(gene_idx).strand, Line) ;
    end ;

    if ~segment_invalid,
      if elems{7}=='+'
        genes(gene_idx).exons{transcript_idx}(end+1,:) = [str2double(elems{4}), str2double(elems{5})+1, type];
      else
        genes(gene_idx).exons{transcript_idx}(end+1,:) = [str2double(elems{4})-1, str2double(elems{5}), type];
      end
    end ;
%    assert(str2double(elems{4})>=genes(gene_idx).start(1+transcript_idx)&str2double(elems{4})<=genes(gene_idx).stop(1+transcript_idx))
%    assert(str2double(elems{5})>=genes(gene_idx).start(1+transcript_idx)&str2double(elems{5})<=genes(gene_idx).stop(1+transcript_idx))
  end ;
  segment_cnt = segment_cnt+1;
  if mod(segment_cnt,5000)==0
    fprintf('  parsed %i gene segments\r', segment_cnt);
  end
end % end third parse
fclose(fd);


fprintf('\n\n');
if length(FAULTS.Exon.no_transcript)>0 || length(FAULTS.Exon.diff_strand_contig)>0,
  fprintf('Following problems were found:\n');
else
  fprintf('No problems found.\n');
end ;
if length(FAULTS.Exon.no_transcript)>0, 
  fprintf('number of gene segments without parent transcript:\t %i\n', length(FAULTS.Exon.no_transcript)) ;
  fprintf('\t=> created a new gene and transcript descriptor\n') ;
end ;
if length(FAULTS.Exon.diff_strand_contig)>0,
  fprintf('number of gene segments with different strand or contig than parent transcript:\t %i\n', length(FAULTS.Exon.diff_strand_contig)) ;
  fprintf('\t=> Segment ignored\n') ;
end ;

for i=1:length(genes)
  genes(i).start = min(genes(i).start);
  genes(i).stop = max(genes(i).stop);
end 

if ~isequal(gff_fname, tmp_fname1),
  unix(['rm ' tmp_fname1]);
end ;

if ~isempty(save_fname),
   save(save_fname, 'genes', 'FAULTS');
end ;

return

function [attributes, type_name, elems, action] = parse_starter(fd, types, types_IDs, contig, source)
% parse_starter(fd)

attributes = {} ;
type_name = '' ;
elems = {} ;
action = '' ;


Line = fgetl(fd);
elems = separate(Line);
assert(~isempty(elems{1}));

if Line(1)=='#',
  action = 'continue' ;
  return ;
end
if ~isempty(contig)
  if ~isequal(upper(elems{1}),upper(contig)),
    action='continue' ;
    return ;
  end
end
if ~ischar(Line),
  action='break' ;
  return ;
end

if isempty(elems{4}) | isempty(elems{4})
  action = 'continue' ;
  return ;
end


assert(str2double(elems{4})<=str2double(elems{5}) || str2double(elems{5})<0)

if~(elems{7}=='+' || elems{7}=='-') ;
  if elems{7}~='.',
    warning('strand missing')
  end ;
end

if ~isempty(source)
  if ~isequal(elems{2},source)
    error('something wrong with source')
  end  
end

if ~ismember(elems{3},types) && ~ismember(elems{3},types_IDs)
  action = 'continue' ;
  return ;
end
attributes = separate(elems{9}, ';');

if ismember(elems{3},types);
  type_name = elems{3};
elseif ismember(elems{3},types_IDs);
  type_name =types(isequal(elems{3},types_IDs));
else
  error('wrong type')
end

action='' ;

return

function [seqs,names]=read_fasta_file(fname, name_only)
% [seqs,names]=read_fasta_file(fname)
% 
% reads all entries in a fasta file
  
if nargin<2, name_only=0; end ; 

fd=fopen(fname,'r') ;
str=fread(fd) ;

% fix new lines
num13=sum(str==13) ;
num10=sum(str==10) ;
if num10==0 && num13>0,
  str(str==13)=10 ;
end ;
if num10>0 && num13>0,
  assert(num10==num13) ;
  str(str==13)=[] ;
end ;

newseq=find(str=='>') ;
newseq(end+1) = length(str)+1 ;

seqs={} ;
names={} ;

for i=1:length(newseq)-1,
  seq=char(str(newseq(i):newseq(i+1)-1)) ;
  idx=find(seq==10, 1, 'first') ;
  assert(length(idx)==1) ;
  name = seq(2:idx-1)' ;
  names{end+1}=name ;
  if ~name_only,
    seq  = seq(idx+1:end)' ;
    seq(seq==10) = [] ;
    seqs{end+1} = seq ;
  end ;
end ;

fclose(fd) ;

return

function x=reverse_complement(y)

if length(y)>200*1024*1024
  x=uint8(y) ;
  for i=1:length(y),
    x(i)=y(end-i+1) ;
  end ;
  y=x ;
else
  y=y(end:-1:1) ;
  x=y ;
end ;

x(y=='a')='t' ;
x(y=='c')='g' ;
x(y=='g')='c' ;
x(y=='t')='a' ;
x(y=='n')='n' ;
x(y=='A')='T' ;
x(y=='C')='G' ;
x(y=='G')='C' ;
x(y=='T')='A' ;
x(y=='N')='N' ;

x=char(x) ;

return

function  [prob, prob_cum,limits,prob_unsort,prob_cum_unsort] = learn_plifs(Out,L,bins,do_balancing,min_inc);
% [prob, prob_cum,limits,prob_unsort,prob_cum_unsort] = learn_plifs_qp(Output,Labels,numberofbins,do_balancing,min_inc);
% 
% learn piecewise linear functions (PLIF), with which outputs of an SVM
% can be transformed to probability and cumulative
% probability values
% 
% INPUT    Out                vector of SVM Outputs on an evaluation set
%
%          L                  vector of corresponding labels (-1,1)  
%
%          bins               number of bins used for interpolation (default 50)
%                             (limits are choosen such that the number of
%                             examples in every bin is similar)  
%
%          do_balancing       should be set to 1 if negative class is
%                             much bigger than positive class,
%                             (negatives are downsampled for
%                             determining the bins) 
%
%          min_inc            probabilities have to be monotonically
%                             increasing (higher SVM output value
%                             corresponds to higher probability of true positive) 
%                             (default 1e-5)
%
%
% OUTPUT   prob               vector of length 'bins'. prob_cum(i) is the
%                             estimated probability of examples with 
%                             Output = limits(i) to be true positive
% 
%          prob_cum           vector of length 'bins'. prob_cum(i) is the
%                             estimated probability of examples with 
%                             Output> limits(i) to be true positive
%
%          limits             vector of length 'bins' with supporting
%                             points (SVM Output values)
%
% see also: Out2Conf  
  
if nargin<5, min_inc = 1e-5 ; end ;
if nargin<4, do_balancing = 1; end ;
if nargin<3, bins = 50; end ;

if do_balancing
  s1 = [min(Out) max(Out)];
  s1 = [s1 Out(L==1)];
  idx = find(L==-1);
  num = min(sum(L==-1),sum(L==1));
  % ii = randperm(length(idx));
  idx = idx(randperm(length(idx)));
  s1 = sort([s1 Out(idx(1:num))]);
  limits = [s1(round(linspace(1,length(s1),bins-1))) inf] ;
else
  s1 = sort(Out) ;
  limits = [s1(round(linspace(1,length(s1),bins-1))) inf] ;
end
  
prob = zeros(bins,1);
prob_cum = zeros(bins,1);
num = zeros(bins,1) ;
num_cum = zeros(bins,1) ;
for i=1:bins-1, 
  num_cum(i) = sum(Out>=limits(i)) ;
  prob_cum(i) = (sum(Out>=limits(i) & L==1 )+1e-10)/(sum(Out>=limits(i))+1e-10) ;
  num(i) = sum(Out>=limits(i) & Out<limits(i+1)) ;
  prob(i) = (sum(Out>=limits(i) & Out<limits(i+1) & L==1 )+1e-10)/(sum(Out>=limits(i) & Out<limits(i+1))+1e-10) ;
end 
prob_cum(bins) = 1;
prob(bins) = 1;

prob_unsort=prob ;
prob_cum_unsort=prob_cum ;

prob_cum = sort(prob_cum) ;
prob = sort(prob) ;

return ;

return

function  [Conf, Conf_cum] = Out2Conf(Out,prob, prob_cum,limits);

% [Conf, Conf_cum] = Out2Conf(Output ,prob, prob_cum,limits);
%  
% function transforms outputs of an SVM to probability and cumulative
% probability values by means of piecewise linear functions (PLIF). 
% Use learn_plifs to determine the function (it will return supporting
% points 'limits' and corresponding y-values, 'prob' and 'prob_cum')
%
% INPUT   Out       vector of SVM Output to be transformed
%         prob
%         prob_cum
%         limits    
%
% OUTPUT  Conf      vector (same length as Out) with estimated
%                   probability for true positive
%         Conf_cum  vector (same length as Out) with estimated 
%                   cumulative probability true positive
% 
% Functions called: -> pwl_add
%
% see also: learn_plifs

bins = length(prob); 
Conf = zeros(1, length(Out)) ;
Conf_cum = zeros(1, length(Out)) ;
for i=1:length(Out)
  x(:,1) = pwl_add(zeros(bins, 1), Out(i), limits) ;
  Conf_cum(i) = x'*prob_cum;
  Conf(i) = x'*prob;
end ;

return

function vec = pwl_add(vec, value, limits)
% vec = pwl_add(vec, value, limits)
  
idx = sum(limits<=value) ;

if idx==0,
  vec(1)=vec(1)+1 ;
elseif idx==length(limits)-1,
  vec(end)=vec(end)+1 ;
else
  vec(idx+1) = vec(idx+1) + (value-limits(idx))/(limits(idx+1)-limits(idx)) ;  
  vec(idx)   = vec(idx)   + (limits(idx+1)-value)/(limits(idx+1)-limits(idx)) ;
end ;
return

function user_name = whoami
% user_name = whoami
  
global whoami_user_name  
  
if isempty(whoami_user_name)
  
  [ret, whoami_user_name]=unix('whoami') ; 
  if ret~=0,
    if keyboard_allowed(),
      keyboard ;
    end ;
    whoami_user_name='unknown' ;
  end ;
end ;

user_name= whoami_user_name ;
idx=find(user_name<30, 1, 'first') ;
user_name = user_name(1:idx-1) ;

return

function [score,tp,tdr]=calcrfcscore(output,LT)
% score=calcrfcscore(output,LT)
map=(LT==1) ;
[s,idx]=sort(-output) ; s=-s ;
map=map(idx) ;

tp=cumsum(map)/sum(LT==1) ;
tdr=cumsum(map)./[1:length(map)] ;

t=tp(2:end)-tp(1:end-1) ;
score = sum(0.5*[tdr(1:end-1)+tdr(2:end)].*t) ;



return

function [false_alarms,hits] = calcroc(output,LTE)

assert(all(size(output)==size(LTE))) ;

[ld,idx] = sort(output); 

i=0 ;

hits=1-cumsum(LTE(idx)>0)/sum(LTE > 0) ;
false_alarms=1-cumsum(LTE(idx)<0)/sum(LTE < 0) ;

% Da die Kurve nie alle Negativen als positiven zaehlt, hat
% Falsealarms nie 
% den Wert 1. Cumsum zaehlt das erste Element als negatives, diese
% fehlt bei
%  den False_alarms.
% Daher wird vorne an den Vektor noch eine 1 gehaengt

hits = [1 hits ];
false_alarms = [1 false_alarms];

if nargout<2,
  false_alarms = [false_alarms;hits];
end ;

return

function s = calcrocscore(oute,LTE)
% rocscore = calcrocscore(output,LT)
%
% computes the area under the ROC curve

[a,b]=calcroc(oute,LTE) ;

s=0 ;
for i=1:length(a)-1,
  s = s + (a(i)-a(i+1))*(b(i)+b(i+1))/2 ;
end ;


return

function [engine, environment, basedir, mccdir, license_type] = determine_engine() ;
% [engine, environment, basedir, mccdir, license_type] = determine_engine() ;
% 
% returns either 'matlab' or 'octave' in variable engine
% and 'internal' or 'galaxy' in variable environment
% basedir is the directory of the matlab or octave instance
% mccdir is the directory of the matlab compiler (does not yet exist for octave)

global g_license_type ;

if isempty(g_license_type),
  g_license_type = 'academic' ;
end ;
license_type = g_license_type ;

lic=license ;
if ~isequal(lic, 'GNU General Public License'),
  engine='matlab' ;
else
  engine='octave' ;
end ;

environment='internal' ;
if isequal(whoami, 'galaxy'),
  environment='galaxy' ;
end ;

if isequal(engine, 'matlab') && isequal(environment,  'internal'),
  basedir = '/fml/ag-raetsch/share/software/matlab/' ;
elseif isequal(engine, 'matlab') && isequal(environment,  'galaxy'),
  basedir = '/home/galaxy/matlab-7.6/' ;
elseif isequal(engine, 'octave') && isequal(environment,  'internal'),
  basedir = '/fml/ag-raetsch/share/software/octave-3.0.3/' ;
elseif isequal(engine, 'octave') && isequal(environment,  'galaxy'),
  basedir = '/home/galaxy/octave-3.0.4' ;
end ;

if isequal(environment,  'internal'),
  mccdir = '/fml/ag-raetsch/share/software/matlab/' ;
elseif isequal(environment,  'galaxy'),
  mccdir = '/home/galaxy/matlab-7.6/' ;
end ;

return

return

function x = equal(M1, M2)
% EQUAL(M1, M2) tests M1 and M2 for equality
% Function to test whether two matrices are identical
%
% returns 1 if true and 0 if false

x=isequal(M1, M2) 

return

function x = fexist(filename,gz)
% FEXIST Check if file exists
%        FEXIST('filename') returns:
%        0 if filename does not exist
%        1 if filename exists in the given path

if nargin<2,
  gz=0;
end ;

fd = fopen(filename);
if fd ~= -1
  x = 1;
  fclose(fd);
elseif gz,
  fd = fopen([filename '.gz']);
  if fd~=-1,
    x=1 ;
    fclose(fd) ;
  else
    x=0 ;
  end ;
else
  x = 0;
end

return

function [models,param_names] = generate_all_models(Model_param)
% models = generate_all_models(Model_param) 
  
param_names = fieldnames(Model_param); 
if isempty(param_names)
  models =[];param_names=[];
  return
end

for j = 1:length(param_names)
  param_nums(j) = length(getfield(Model_param,param_names{j}));
end
models = zeros(length(param_names),prod(param_nums));
for j = 1:length(param_names)
  par_tmp  = getfield(Model_param,param_names{j});
  par1 = [];
  for k = 1:length(par_tmp)
    par1 = [par1 repmat(k,1,prod(param_nums(j+1:end)))];
  end
  models(j,:) = par_tmp(repmat(par1,1,prod(param_nums(1:j-1))));
end

if prod(param_nums)<1
  assert(length(unique(models','rows'))==prod(param_nums))
else
  assert(~isempty(models))
end

return

function var=load_matrix(filename, varname)
% var=load_matrix(filename, varname)

load(filename, 'meta')
if ~exist('meta')
  load(filename, varname);
  eval(sprintf('var = %s;',varname))
  return
end

var = [] ;
col = 1;
for j=1:meta.num_splits
  %if col>1
  %  assert(~all(var(:,col-1)==-1* ones(meta.size(1),1)))
  %end
  %assert(all(var(:,col)==-1*ones(meta.size(1),1)))
   
  load(filename, sprintf('%s_split_%i',varname,j));
  eval(sprintf('temp = %s_split_%i;',varname,j));

  if j==1,
    if isnumeric(temp)
      var = zeros(meta.size(1), meta.size(2), class(temp));
    else
      var = zeros(meta.size(1), meta.size(2),'uint8');
      var = char(var);
    end
  end ;

  var(:,col:col + size(temp, 2)-1)=temp;
  col = col + size(temp, 2);
end
%assert(~all(var(:,end)==-1* zeros(meta.size(1),1)))

return

function array = load_struct(infile_name, arrayname)
% array = load_struct(infile_name, arrayname)
  
if nargin<2,
  arrayname='array' ;
end ;

L_=load(infile_name) ;
if isfield(L_,'numvars')
  fprintf('arrayload2 loads %i chunks:', L_.numvars) ;
  ix=1;
  L=load(sprintf('%s_%i.mat', infile_name, ix), sprintf('%s%d', arrayname, ix)) ;
  eval(sprintf('array = L.%s1;', arrayname)) ;
  fprintf('.') ;
  for ix = 2:L_.numvars
    L=load(sprintf('%s_%i.mat', infile_name, ix), sprintf('%s%d', arrayname, ix)) ;
    if size(array,1) == 1
      array = [array, eval(sprintf('L.%s%d', arrayname, ix))];
    elseif size(array,2) == 1
      array = [array; eval(sprintf('L.%s%d', arrayname, ix))];
    end
    fprintf('.') ;
  end
  fprintf('\n') ;
  if isfield(L_, 'dimensions'),
    array = reshape(array, L_.dimensions) ;
  end ;
else
  eval(sprintf('array=L_.%s;', arrayname)) ;
end

return

function save_append(fname__, append_flag, varargin)
% save_append(fname__, append_flag, varargin)
%
% octave compatible version of save -append

try,
  if append_flag
    L__=load(fname__) ;
  else
    L__=struct ;
  end ;
catch 
  L__=struct ;
end 

if length(varargin)>1,
  
  % assume varargin to have the following structure: varname1, var1, ...
  
  assert(mod(length(varargin),2)==0) ;
  for i=1:2:length(varargin),
    L__.(varargin{i}) = varargin{i+1} ;
  end ;
  
  f=fieldnames(L__) ;
  for i=1:length(f),
    eval(sprintf('%s=L__.%s;', f{i}, f{i})) ;
  end ;
  
  eval(sprintf('save -V7 %s %s', fname__, sprintf('%s ', f{:}))) ;
else
  
  % assume the second argument is a struct
  assert(length(varargin)==1) ;
  
  f = fieldnames(varargin{1}) ;
  for i=1:2:length(f),
    L__.(f{i}) = varargin{1}.(f{i}) ;
  end ;
  
  f=fieldnames(L__) ;
  for i=1:length(f),
    eval(sprintf('%s=L__.%s;', f{i}, f{i})) ;
  end ;
  
  eval(sprintf('save -V7 %s %s', fname__, sprintf('%s ', f{:}))) ;
  
end ;




return

function save_matrix(filen, M, varname, split_size)
%save_matrix(filen, M, varname, split_size)
%split matrix into parts with size of about split_size 
%bytes (1e+8 default) and save them in one file

if nargin<4
  split_size=1e+8;
end

info = whos('M');
num_splits = ceil(info.bytes/split_size);
num_cols = ceil(size(M, 2)/num_splits);

  
  meta.size=size(M);
  meta.num_splits = num_splits;
  mata.name = varname;
  if fexist(filen)
    save_append(filen, 1, 'meta', meta);
  else
    save(filen, 'meta');
  end
  for j=1:num_splits-1
    %eval(sprintf('%s_split_%i = M(:, 1:num_cols);',varname, j));
    var_split__ = M(:, 1:num_cols);
    M(:, 1:num_cols)=[];
    %save(filen, '-append', sprintf('%s_split_%i',varname,j));
    save_append(filen, 1, sprintf('%s_split_%i',varname,j), var_split__);
    clear var_split__
  end
  if isempty(j), j=0; end;
  var_split__ = M; 
  save_append(filen, 1, sprintf('%s_split_%i',varname,j+1), var_split__);

return

function save_struct(outfile_name, array, arrayname, MAXSIZE)
% save_struct(outfile_name, array[, arrayname])

if nargin<3
  arrayname='array' ;
end ;

if nargin<4, 
  MAXSIZE = 1e9; % about 1Gb
end ;

whosinfo = whos('array');
if whosinfo.bytes > MAXSIZE
  % linearize first, store original size
  dimensions = size(array) ;
  array = reshape(array, 1, prod(dimensions)) ;

  elementsize = whosinfo.bytes/length(array);
  chunklength = round(MAXSIZE/elementsize);
  numvars = ceil(length(array)/chunklength);
  %fprintf('arraysave2 processes %i chunks:', numvars) ;
  for ix = 1:numvars-1
    eval(sprintf('%s%d = array(%d:%d);',arrayname, ix,(ix-1)*chunklength+1, ...
                 ix*chunklength));
    %fprintf('.') ;
  end
  %fprintf('\n') ;
  eval(sprintf('%s%d = array(%d:end);',arrayname, numvars,(numvars-1)*chunklength+1));
  clear array ix ;
  fprintf('save_struct saves %i chunks:', numvars) ;
  save(outfile_name, '-V7', 'chunklength', 'elementsize', 'numvars', 'dimensions') ;
  for ix = 1:numvars
    save(sprintf('%s_%i.mat', outfile_name, ix), '-V7', sprintf('%s%d', arrayname, ix)) ;
    fprintf('.') ;
  end ;
  fprintf('\n') ;
else
  %eval(sprintf('%s=array;', arrayname)) ;
  save_append(outfile_name, 0, arrayname, array) ;
end

return

function f=separate(str, delim, merge_multiple) ;
% f=separate(str, delimiter, merge_multiple_delimiters) ;
  
if nargin<2, delim=sprintf('\t') ; end ;
if nargin<3, merge_multiple=0 ; end ;
f={} ;
idx=[0 find(str==delim) length(str)+1] ;
if merge_multiple,
  idx2=find(idx(1:end-1)~=idx(2:end)-1) ;
  idx=[idx(idx2) idx(end)] ;
end 
for i=1:length(idx)-1
  f{i}=deblank(str(idx(i)+1:idx(i+1)-1)) ;
end ;

return

function A=spdiag(diagonal) ;
% A=spdiag(diagonal) ;

if size(diagonal,1)>1
  A=spdiags(diagonal, 0, length(diagonal),length(diagonal)) ;
else
  A=spdiags(diagonal', 0, length(diagonal),length(diagonal)) ;
end ;

return

function A=spzeros(m,n,szmax) ;
% A=spzeros(m,n) ;

if nargin<=1
  A=sparse([],[],[],m,m) ;
  return ;
end 
if nargin<=2
  A=sparse([],[],[],m,n) ;
  return ;
end 
A=sparse([],[],[],m,n,szmax) ;

return

function list=who_file(fname) ;
% list=who_file(fname) ;
  
engine = determine_engine() ;
if isequal(engine, 'matlab')
  list=who('-file', fname) ;
else
  L=load(fname) ;
  list=fieldnames(L) ;
end ;

return

function genes = gen_gene_segmentation(P)
% genes = gen_gene_segmentation(P)
% INPUT
% P.genes
% P.segment_ids  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% generate segmentation for all valid transcripts of a gene %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
genes = P.genes;
segment_ids = P.segment_ids;  
  
for g=1:length(genes)
  if ~genes(g).is_complete
    continue;
  end
  exons = genes(g).exons;
  for j=1:length(exons),
    if ~genes(g).transcript_valid(j)
      genes(g).segments{j}=[];
      continue;
    end
    if ~genes(g).transcript_complete(j)
      continue;
    end
    if genes(g).strand=='+'
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    % INSERT SEGMENTS FOR POSITIVE STRAND %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
      isoform = [];
      % insert a row for each 5'utr exon and each intron between
      %----------------------------------------------------
      for k=1:size(genes(g).utr5_exons{j},1)
        isoform = [isoform; [genes(g).utr5_exons{j}(k,1:2) segment_ids.utr5exon]];
        if k<size(genes(g).utr5_exons{j},1)
          isoform = [isoform; [genes(g).utr5_exons{j}(k,2) genes(g).utr5_exons{j}(k+1,1) segment_ids.intron]];
        end
      end
      % if there is an intron directly before the tis site (AGATG):
      %----------------------------------------------------
      if  ~isempty(genes(g).utr5_exons{j})
        if genes(g).utr5_exons{j}(end,2)<genes(g).cds_exons{j}(1,1)
           isoform = [isoform; [genes(g).utr5_exons{j}(end,2) genes(g).cds_exons{j}(1,1) segment_ids.intron]];
        end
      end
      % insert a row for each cds_exon and each intron between
      %----------------------------------------------------
      for k=1:size(genes(g).cds_exons{j},1)
        isoform = [isoform; [genes(g).cds_exons{j}(k,1:2) segment_ids.cds_exon]];
        if k<size(genes(g).cds_exons{j},1)
          isoform = [isoform; [genes(g).cds_exons{j}(k,2) genes(g).cds_exons{j}(k+1,1) segment_ids.intron]];
        else
          % set end of last cds_exon to the beginning of the cdsStop-consensus
          isoform(end,2)=isoform(end,2)-3;
        end
      end   
      % insert an intron between last cds_exon and 3'utr if necessary
      %----------------------------------------------------   
      if  ~isempty(genes(g).utr3_exons{j}) 
        if  genes(g).cds_exons{j}(end,2)<genes(g).utr3_exons{j}(1,1)
          %isoform = [isoform; [genes(g).cds_exons{j}(end,2) genes(g).utr3_exons{j}(1,1) segment_ids.intron]];
          isoform = [isoform; [isoform(end,2) genes(g).utr3_exons{j}(1,1) segment_ids.intron]];
        end
      end
      % insert a row for each 3'utr exon and each intron between
      %----------------------------------------------------
      for k=1:size(genes(g).utr3_exons{j},1)
        if k==1 && isoform(end,3)==segment_ids.cds_exon
          % set end of first 3'utrexon to the beginning of the cdsStop-consensus
          isoform = [isoform; [genes(g).utr3_exons{j}(k,1)-3 genes(g).utr3_exons{j}(k,2) segment_ids.utr3exon]];
        % insert polya_tail if applicable
        elseif k==size(genes(g).utr3_exons{j},1)&&length(genes(g).polya)>=j&&...
               genes(g).polya(j)>genes(g).utr3_exons{j}(k,1)&&genes(g).polya(j)<genes(g).utr3_exons{j}(k,2)
          isoform = [isoform; [genes(g).utr3_exons{j}(k,1) genes(g).polya(j) segment_ids.utr3exon]];
          isoform = [isoform; [genes(g).polya(j) genes(g).utr3_exons{j}(k,2) segment_ids.polya_tail]];
        else
          isoform = [isoform; [genes(g).utr3_exons{j}(k,1:2) segment_ids.utr3exon]];
        end
        if k<size(genes(g).utr3_exons{j},1)
          isoform = [isoform; [genes(g).utr3_exons{j}(k,2) genes(g).utr3_exons{j}(k+1,1) segment_ids.intron]];
        end
      end   
    elseif genes(g).strand=='-'
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    % INSERT SEGMENTS FOR NEGATIVE STRAND %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
      isoform=[];
      % insert a row for each 3'utr exon and each intron between
      %----------------------------------------------------
      for k=1:size(genes(g).utr3_exons{j},1)
        % insert polya_tail if applicable
        if k==1&&length(genes(g).polya)>=j&&genes(g).polya(j)>genes(g).utr3_exons{j}(k,1)&&...
           genes(g).polya(j)<genes(g).utr3_exons{j}(k,2)
          isoform = [isoform; [genes(g).utr3_exons{j}(k,1) genes(g).polya(j) segment_ids.polya_tail]];
          isoform = [isoform; [genes(g).polya(j) genes(g).utr3_exons{j}(k,2) segment_ids.utr3exon]];
        else
          isoform = [isoform; [genes(g).utr3_exons{j}(k,1:2) segment_ids.utr3exon]];
        end
        if k<size(genes(g).utr3_exons{j},1)
          isoform = [isoform; [genes(g).utr3_exons{j}(k,2) genes(g).utr3_exons{j}(k+1,1) segment_ids.intron]];
        else
          isoform(end,2)=isoform(end,2)+3;
        end
      end
      % if there is an intron between cdsStop and first utr3_exon:
      %----------------------------------------------------
      if  ~isempty(genes(g).utr3_exons{j})
        if genes(g).utr3_exons{j}(end,2)<genes(g).cds_exons{j}(1,1)
          isoform(end,2) = isoform(end,2)-3;%undo moving the last utr3-exon position
          isoform = [isoform; [isoform(end,2) genes(g).cds_exons{j}(1,1)+3 segment_ids.intron]];
        end
      end
      % insert a row for each cds_exon and each intron between
      %----------------------------------------------------
      for k=1:size(genes(g).cds_exons{j},1)
        if k==1 %&& ~isempty(isoform)&&isoform(end,3)==segment_ids.utr3exon
          isoform = [isoform; [genes(g).cds_exons{j}(k,1)+3 genes(g).cds_exons{j}(k,2) segment_ids.cds_exon]];
        else
          isoform = [isoform; [genes(g).cds_exons{j}(k,1:2) segment_ids.cds_exon]];
        end  
        if k<size(genes(g).cds_exons{j},1)
          isoform = [isoform; [genes(g).cds_exons{j}(k,2) genes(g).cds_exons{j}(k+1,1) segment_ids.intron]];
        end
      end  
      % insert an intron between last cds_exon and 5'utr if necessary
      %----------------------------------------------------   
      if  ~isempty(genes(g).utr5_exons{j}) 
        if genes(g).cds_exons{j}(end,2)<genes(g).utr5_exons{j}(1,1)
          isoform = [isoform; [genes(g).cds_exons{j}(end,2) genes(g).utr5_exons{j}(1,1) segment_ids.intron]];
        end
      end
      % insert a row for each 5'utr exon and each intron between
      %----------------------------------------------------
      for k=1:size(genes(g).utr5_exons{j},1)
        isoform = [isoform; [genes(g).utr5_exons{j}(k,1:2) segment_ids.utr5exon]];
        if k<size(genes(g).utr5_exons{j},1)
          isoform = [isoform; [genes(g).utr5_exons{j}(k,2) genes(g).utr5_exons{j}(k+1,1) segment_ids.intron]];
        end
      end
    end %strand case
    % assert a tight segmentation
    assert(all(isoform(2:end, 1) == isoform(1:end-1, 2)));
    if ~(all(isoform(2:end, 1) == isoform(1:end-1, 2)))
       if genes(g).utr5_exons{j}(end,2)<genes(g).tis(j)
         fprintf('last utr5_exon stop: %i \n tis: \t%i \n first cds_exon_start: %i\n\n',...
                genes(g).utr5_exons{j}(end,2),genes(g).tis(j),genes(g).cds_exons{j}(1,1))
       else 
       end
    end

    % assert that there is an ORF
    assert(sum(isoform(:,3)==segment_ids.cds_exon ...
               | isoform(:,3)==segment_ids.utr5exon ...
               | isoform(:,3)==segment_ids.utr3exon)==size(genes(g).cds_exons{j},1)+size(genes(g).utr5_exons{j},1)+size(genes(g).utr3_exons{j},1));
    cds_idx = find(isoform(:,3)==segment_ids.cds_exon);
    assert(mod(sum(isoform(cds_idx,2)-isoform(cds_idx)),3)==0)
    %mod(sum(isoform(cds_idx,2)-isoform(cds_idx)),3)
    %keyboard

    %str = [];
    %for k=1:length(cds_idx),
    %  str = [str block.genestr(isoform(cds_idx(k),1):isoform(cds_idx(k),2)-1)];
    %end
    %f = find_open_frames(lower(str),1); 
    %assert(f(1)~=0);
    genes(g).segments{j} = isoform;
  end
end

return



return

function [SEG, LABEL] = get_cand_contents(P)
% [SEG, LABEL] = get_cand_contents(P)


genes = P.genes ;
dist_to_genes = P.intergenic_dist_to_genes;
use_frame = P.use_frame;
which_segments = P.which_segments;
segment_ids = P.segment_ids;
genome_info = P.genome_info;

clear P

%i=min(find([genes.transcript_complete]));
pos_segments_all = ones(1e6, 1) ;%size(genes(i).segments{1},2));
neg_segments_all = ones(1e6, 1) ;%size(genes(i).segments{1},2));
%i=min(find([genes.transcript_complete]));
%pos_segments_all = ones(1e6,size(genes(i).segments{1},2));
%neg_segments_all = ones(1e6,size(genes(i).segments{1},2));
num_pos = 0;
num_neg = 0;

last_gene_stop=1;
[tmp,idx] = sort([genes.start]);
genes = genes(idx);
for i=1:length(genes)
  gene = genes(i);
  if ~gene.is_valid || ~any(gene.transcript_complete)
    last_gene_stop = gene.stop;
    continue
  end
  % find common/consens segments from different truths
  %----------------------------------------------------------------
  if isempty(gene.segments)
    last_gene_stop = gene.stop;
    continue
  end
  if ~use_frame
    cons_segments = gene.segments{1};
    for t = 2:length(gene.segments)
      if isempty(cons_segments) 
        cons_segments=gene.segments{t} ;
        %break
      end
      if ~gene.transcript_complete(t),
        continue ;
      end ;
      if isempty(gene.segments{t})
        continue
      end
      [tmp,idx1,idx2] = intersect(cons_segments(:,1),gene.segments{t}(:,1) ) ;
      ii = find(cons_segments(idx1,2)==gene.segments{t}(idx2,2));
      ii2 = find(cons_segments(idx1(ii),3)==gene.segments{t}(idx2(ii),3));
      cons_segments = cons_segments(idx1(ii(ii2)),:);
      % doesn't work in octave
      % cons_segments = intersect(cons_segments,gene.segments{t},'rows');
    end
    if isempty(cons_segments)
      cons_segments=zeros(0,3);
    end
    
    % extract lines from common/consens segments 
    %----------------------------------------------------------------
    % if any(cons_segments(2:end,2)-cons_segments(1:end-1,2)<0)
    %   keyboard
    % end
    
    pos_segments = cons_segments(find(ismember(cons_segments(:,3), which_segments{1})), :);
    neg_segments = cons_segments(find(ismember(cons_segments(:,3), which_segments{2})), :);
    
    if ismember(segment_ids.intergenic,which_segments{1})
      assert(isempty(pos_segments))
      if gene.start - dist_to_genes(1)>last_gene_stop
        pos_segments = [pos_segments;  max(last_gene_stop,gene.start-dist_to_genes(2)), gene.start segment_ids.intergenic];
      end
    end

    if ismember(segment_ids.intergenic,which_segments{2})
      if gene.start - dist_to_genes(1)>last_gene_stop
        neg_segments = [neg_segments; max(last_gene_stop,gene.start-dist_to_genes(2)), gene.start segment_ids.intergenic];
      end
    end
  else % use_frame
    cds_exons = gene.cds_exons{1};
    % just checks
    if isempty(cds_exons), continue; end
    if genes(i).strand=='+'
      cds_exons(:,2) = cds_exons(:,2)-1;
    else
      cds_exons(:,1) = cds_exons(:,1)+1;
    end
    [cds,valid_splice] =  load_genomic(gene.chr, char(gene.strand),cds_exons(:,1),cds_exons(:,2),genome_info,1);
    f = find_open_frames(cds,1) ;
    too_short = cds_exons(1,2)-cds_exons(1,1)+1<3||cds_exons(end,2)-cds_exons(end,1)+1<3;
    if mod(length(cds),3)~=0 || f(1)~=1 || isempty(cds) || any(cds(1:3)~='atg') || ~any(ismember({'taa', 'tag', 'tga'}, cds(end-2:end)))|| too_short
      warning('invalid transcript') ;
      continue
    end ;
    % create positive examples
    cds_exons = gene.cds_exons{1};
    %% remove ATG and TAG
    cds_exons(1,1) = cds_exons(1,1)+3;
    cds_exons(end,2) = cds_exons(end,2)-3;
    if genes(i).strand=='+'
      cds_exons(cds_exons(:,1)==1,1) = cds_exons(cds_exons(:,1)==1,1)+2;
      cds_exons(cds_exons(:,1)==1,2) = cds_exons(cds_exons(:,2)==2,1)+1;
    else
      cds_exons(cds_exons(:,1)==1,2) = cds_exons(cds_exons(:,1)==1,2)-2;
      cds_exons(cds_exons(:,2)==2,2) = cds_exons(cds_exons(:,2)==2,2)-1;
    end
    pos_segments = cds_exons(:,1:2);
    %% create negative examples
    cds_exons = gene.cds_exons{1};
    %% remove ATG and TAG
    cds_exons(1,1) = cds_exons(1,1)+3;
    cds_exons(end,2) = cds_exons(end,2)-3; 
    neg_segments = cds_exons(cds_exons(:,3)~=0,1:2);
    if genes(i).strand=='+'
      neg_segments = [neg_segments;  [cds_exons(cds_exons(:,3)==0,1)+1 cds_exons(cds_exons(:,3)==0,2)]];
      neg_segments = [neg_segments;  [cds_exons(cds_exons(:,3)==0,1)+2 cds_exons(cds_exons(:,3)==0,2)]];
      neg_segments = [neg_segments;  [cds_exons(cds_exons(:,3)==1,1)+1 cds_exons(cds_exons(:,3)==1,2)]];
      neg_segments = [neg_segments;  [cds_exons(cds_exons(:,3)==2,1)+2 cds_exons(cds_exons(:,3)==2,2)]];
    else
      neg_segments = [neg_segments;  [cds_exons(cds_exons(:,3)==0,1) cds_exons(cds_exons(:,3)==0,2)-1]];
      neg_segments = [neg_segments;  [cds_exons(cds_exons(:,3)==0,1) cds_exons(cds_exons(:,3)==0,2)-2]];
      neg_segments = [neg_segments;  [cds_exons(cds_exons(:,3)==1,1) cds_exons(cds_exons(:,3)==1,2)-1]];
      neg_segments = [neg_segments;  [cds_exons(cds_exons(:,3)==2,1) cds_exons(cds_exons(:,3)==2,2)-2]];
    end
    idx = find(neg_segments(:,2)<=neg_segments(:,1));
    neg_segments(idx,:)=[];
    idx = find(pos_segments(:,2)<=pos_segments(:,1));
    pos_segments(idx,:)=[];
  end
  pos_segments_all(num_pos+1:num_pos+size(pos_segments,1),1:size(pos_segments,2))  = pos_segments;
  neg_segments_all(num_neg+1:num_neg+size(neg_segments,1),1:size(neg_segments,2))  = neg_segments;
  
  
  num_pos = num_pos+size(pos_segments,1);
  num_neg = num_neg+size(neg_segments,1);

  last_gene_stop = gene.stop;
end

% assert(num_pos>0)
% assert(num_neg>0)

pos_segments_all(num_pos+1:end,:)=[];
neg_segments_all(num_neg+1:end,:)=[];

SEG = [pos_segments_all; neg_segments_all];
LABEL = [ones(num_pos,1); -ones(num_neg,1)];
[tmp,idx] = sort(SEG(:,1));
SEG = SEG(idx,:);
LABEL = LABEL(idx);

assert(all(SEG(:,2)-SEG(:,1)>=0))
idx = find(SEG(:,2)-SEG(:,1)==0);
SEG(idx,:) = [];
LABEL(idx) = [];

if ~isempty(find(SEG(2:end,1)-SEG(1:end-1,2)<0)) && ~use_frame
  idx = find(SEG(2:end,1)-SEG(1:end-1,2)<0);
  fprintf(1, 'removing %i overlapping segments\n',length(idx)*2)
  SEG(idx,:)=[];
  LABEL(idx)=[]; 
  assert(all(SEG(2:end,1)-SEG(1:end-1,2)>=0))
end
SEG = SEG(:,1:2);
return

function save_predictions_as_contpred(out_fn_prefix, pos, score, score_name, content_name, contig_name, strand, do_gzip)
% save_predictions_as_wiggle(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)

%assert(issorted(pos))
  
assert(isempty(pos) || isequal(pos(:,1), sort(pos(:,1))))
assert(size(pos,1)==length(score))

if ~exist('do_gzip', 'var'),
  do_gzip = 1 ;
end ;

fn_out=[out_fn_prefix '_' score_name '.cpf'];

if fexist(fn_out) | fexist([fn_out '.gz'])
  fprintf(1, 'file exists, it will NOT be overwritten\n')
  return
end
fprintf(1,'save in content prediction format (cpf): %s\n', fn_out);
[fd msg] = fopen(fn_out, 'w+');

fprintf(fd, '##Seqid\tType\tSource\tStart\tEnd\tStrand\tScore\n') ;

for j=1:min(10000,length(pos))
  fprintf(fd, '%s\t%s\t%s\t%i\t%i\t%c\t%.3f\n', contig_name, content_name, score_name, pos(j,1),pos(j,2), strand, score(j)) ;
end
if length(pos)>10000,
  fprintf(fd, '##Attention: this file has been truncated.\n') ;
end ;
fclose(fd);

if do_gzip,
  ret = unix(sprintf('gzip %s',fn_out)) ;
  if ret~=0
    warning('gzip failed') ;
  end
end 


return

function save_predictions_as_contpred_bin(out_fn_prefix, pos, score, score_name, content_name, contig_name, strand)
% save_predictions_as_sigpred_bin(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand)

%assert(issorted(pos))

assert(isempty(pos) || isequal(pos(:,1), sort(pos(:,1))))
assert(size(pos,1)==length(score))

fn_out=[out_fn_prefix '_' score_name '_cpf.mat'];

if fexist(fn_out) 
  fprintf(1, 'file exists, it will NOT be overwritten\n')
  return
end
fprintf(1,'save in binary content prediction format (cpf): %s\n', fn_out);
save(fn_out, 'contig_name', 'content_name', 'score_name', 'pos', 'strand', 'score') ;

return

function [train,test] = assign_subsets(trivial_regions,fn_trainings_regions,fn_test_regions,genes,SETs,regions_par,tasks)
% [train,test] = assign_subsets(trivial_regions,fn_trainings_regions,fn_test_regions,genes,SETs,regions_par,tasks)


% label_source
% if fexist(fn_genes)&(label_source.use_confirmed|label_source.use_anno)
  
if ~isempty(fn_trainings_regions)&fexist(fn_trainings_regions)
  assert(ismember('training_regions',who('-file',fn_trainings_regions)))
  load(fn_trainings_regions,'training_regions');
else
  training_regions = trivial_regions;
  fprintf('no training region specified, training on whole genome except test region\n')
end

if ~isempty(fn_test_regions)&fexist(fn_test_regions)
  assert(ismember('test_regions',who('-file',fn_test_regions)))
  load(fn_test_regions,'test_regions');
else
  test_regions = trivial_regions(1);
  test_regions.stop = 0;
  test_regions.start = 0;
  fprintf('no test region specified, training on whole genome\n')
end
fprintf('make sure there is no overlap between training and test regions\n')  
if ~(tasks.train)
  training_regions = assert_noRegionOverlap(test_regions,training_regions);
else
  warning('region overlap not checked!! Training and Test region could overlap!\n')
end
rand('seed',1234);
idx = [];

fprintf('Splitting testing regions\n') ;
regions_par.offset = regions_par.offset_test;
test.blocks = split_regions(test_regions, [], regions_par, 0);
fprintf('Done\n') ;

fprintf('Splitting training regions\n') ;
regions_par.offset = regions_par.offset_train;
train.blocks = split_regions(training_regions, genes, regions_par,0);
fprintf('Done\n') ;

if ~isempty(test.blocks)
  test.blocks = rmfield(test.blocks, 'left_dist');
  test.blocks = rmfield(test.blocks, 'right_dist');
  % test.blocks = sort_regions(test.blocks) ;
  % for j=1:length(test.blocks)
  %   test.blocks(j).id = j;
  % end
  [test.blocks, test.split] = make_split(test.blocks,SETS)
else
  test.split = [];
end

% merge training blocks and generate training split
if ~isempty(train.blocks)
  % train.blocks = sort_regions(train.blocks) ; 
  % for j=1:length(train.blocks)
  %   train.blocks(j).id = j;
  % end
  if regions_par.merge_confirmed
    train.blocks = construct_merged_blocks(train.blocks,PAR);
  else
    train.blocks = rmfield(train.blocks,'left_dist');
    train.blocks = rmfield(train.blocks,'right_dist');
  end
  if regions_par.cluster_paralogs
    split_idx = splits_by_paralogs(train.blocks,SETs.num_splits);
    for s = 1:SETs.num_splits
      idx = setfield(idx,sprintf('split_%i',s),split_idx{s});
      error('something wron here; check implementation')
      train.blocks(idx.(sprintf('split_%i',s))) = s;
    end
  else
    [train.blocks, train.split] = make_split(train.blocks,SETs);
  end
else
  train.split = [];
end

return

function [blocks, split] = make_split(blocks,SETs)

blocks_per_split = ceil(length(blocks)/SETs.num_splits);
block_idx  = randperm(length(blocks));
for s = 1:SETs.num_splits
  IDX = (s-1)*blocks_per_split+[1:blocks_per_split];
  IDX(IDX>length(blocks)) = [];
  IDX = block_idx(IDX);
  split.(sprintf('split_%i',s)) = IDX;
% [blocks(IDX).split] = deal(s); % problematic piece of code
  for i=IDX
    blocks(i).split = s;
  end ;
end

y = [];
for s = 1:SETs.num_splits
  y = [y getfield(split,sprintf('split_%i',s))];
end
assert(length(y)==length(unique(y)))

return

function blocks = split_regions(REGS,genes,regions_par, do_assertion)

% blocks=split_regions(REGS,genes,regions_par, do_assertion)
% 
% REGS are splitted into short blocks containing only region_par.min_genes_per_region genes if 
% split criterion holds
% split criterion: splits must have minimal distance of region_par.minCutDist (default 250)
% to both genes
% If region_par.merge_confirmed is set, only 1 gene is put into region,
% regions will be merged later with only complete and confirmed genes

  
if nargin<4
  do_assertion=0;
end
fprintf(1,'Start split_regions...\n')

if ~isfield(regions_par,'minCutDist')
  regions_par.minCutDist = 250;
end

if ~isempty(genes)
  %   load(fn_genes,'genes'); 
  map = zeros(1,length(genes));

  if regions_par.cluster_paralogs
    
    % filter out genes from regions other than the given trivial regions
    % this is necessary for correct detection of paralog blocks
    assert(isequal([genes.id],1:length(genes)))
    
    all_genes_in_REGS_idx = [];
    fprintf(1,'sorting paralogs\n')
    for r=1:length(REGS)
      fprintf(1,'check region %i\r',r)
      contig_genes_idx = find(REGS(r).chr_num==[genes.chr_num]&[genes.strand]==REGS(r).strand);
      all_genes_in_REGS_idx = [all_genes_in_REGS_idx contig_genes_idx];
    end
    for j=1:length(genes)
      fprintf(1,'check genes %i\r',j)
      other_genes_idx = find(~ismember(genes(j).paralogs,all_genes_in_REGS_idx));
      genes(j).paralogs(other_genes_idx) = [];
    end
    all_genes_in_REGS_idx = unique(all_genes_in_REGS_idx);
    genes = genes(all_genes_in_REGS_idx);
  end
else
  warning('no genes: blocks are produced without looking at gene boundaries!!!')
  % keyboard
  genes=[];
  map  =[];
end

blocks=[];
for r=1:size(REGS,2)
  REGS(r).TR_idx = r;
  [blocks,map] = split_regions_helper(blocks,REGS(r),genes,regions_par,map);
end % loop over REGS

if ~isempty(blocks)&&~isempty([blocks.gene_ids])
  assert(isequal(find(map),sort([blocks.gene_ids])))
else
  assert(~any(map))
end
cluster = {};
covered = [];
if isfield(genes,'paralogs')
  idx1 = find(~cellfun('isempty',{genes.paralogs}));
  for i = idx1
    paralogs = unique([map(genes(i).paralogs)]);
    if isempty(intersect(paralogs,covered))
      covered = [covered paralogs];
      cluster{end+1} = paralogs;
      for j= paralogs
        assert(isempty(blocks(j).paralogs))
        blocks(j).paralogs = paralogs;
      end
    else
      idx_c = [];
      covered = unique([covered paralogs]);
      for j=1:length(cluster)
        if any(ismember(paralogs,cluster{j}))
          idx_c = [idx_c,j];
        end
      end
      cluster{idx_c(1)} = [cluster{idx_c(1)},paralogs];
      for j=2:length(idx_c)
        cluster{idx_c(1)} = ([cluster{idx_c(1)},cluster{idx_c(j)}]);
        cluster{idx_c(j)} = [];
      end
      cluster{idx_c(1)} = unique(cluster{idx_c(1)});
      for j= cluster{idx_c(1)}
        blocks(j).paralogs = cluster{idx_c(1)};
      end
      
    end
  end
end
 

if do_assertion
  %check split
  fprintf(1,'check pairwise overlap of blocks\n')
  sum1 = 0;
  %ids = zeros(1,length(blocks));
  for r1 = 1:length(blocks)
    fprintf(1,'%i of %i blocks checked \r',r1,length(blocks)) 
    sum1 = sum1 + blocks(r1).stop-blocks(r1).start+1;
    ids(r1) =  blocks(r1).id;
    num_genes(r1) = length(blocks(r1).gene_ids);
   % rgenes = [];
   % for g=1:num_genes(r1)
   %   gidx = find([genes.id]==blocks(r1).gene_ids(g));
   %   assert(length(gidx)==1)
   %   rgenes = [rgenes genes(gidx)];
   % end
   % assert(max([rgenes.stop])<=blocks(r1).stop-PAR.blocks.minCutDist)
   % assert(min([rgenes.start])>=blocks(r1).start+PAR.blocks.minCutDist)
   % for r2 = (r1+1):size(blocks,2)
   %   check_overlap(blocks(r1),blocks(r2));
   % end
  end
  sum2 = 0;
  for r = 1:size(REGS,2)
    sum2 = sum2 + REGS(r).stop-REGS(r).start+1;
  end
  assert(sum1==sum2)
  assert(all(sort(ids)==1:length(blocks)))
  assert(min(num_genes)>= regions_par.min_genes_per_region)
  assert(isequal(sort([blocks.ids]),1:length(genes)))%all genes are assigned to exactly one region
  fprintf(1,'created %i blocks with min %i and max %i genes\n',length(blocks),min(num_genes), max(num_genes))
  fprintf(1,'num of genes: %ii\n',sum(num_genes))
end



if 0% ~isempty(genes)
  assert(isempty(setdiff([genes.id],[blocks.gene_ids])))
end

return

function [regions,map] = split_regions_helper(regions,REG,GENES,regions_par,map)
% [regions,map] = split_regions_helper(regions,REG,GENES,regions_par,map)
  
new_tr=1;
g=0;
contigstart = REG.start;
contigstop = REG.stop;

if regions_par.merge_confirmed
  min_genes_per_region = 1;
  fpritnf('putting only 1 gene in region; regions will be merged later with only complete genes')
else 
  min_genes_per_region = regions_par.min_genes_per_region;
end


if ~isempty(GENES)
  GENES_chr_num=[GENES.chr_num] ;
  GENES_strand=[GENES.strand] ;
  GENES_start=[GENES.start]+5 ; % hack for some minor problems
  GENES_stop=[GENES.stop]-5 ;   % hack for some minor problems

  idx =  find(REG.chr_num==GENES_chr_num& GENES_strand==REG.strand & ...
              REG.start <= GENES_start & REG.stop >= GENES_stop);
  idx_overlap = find(REG.chr_num==GENES_chr_num & GENES_strand==REG.strand & ...
                     REG.start > GENES_start & REG.start < GENES_stop);
  assert(isempty(idx_overlap))
  idx_overlap = find(REG.chr_num==GENES_chr_num & GENES_strand==REG.strand & ...
                     REG.stop > GENES_start & REG.stop < GENES_stop);
  assert(isempty(idx_overlap))
else 
  idx=[];
end

fprintf(1,'split region %s%s, num of genes found: %i  \r',REG.chr,REG.strand, length(idx)) 
if ~isempty(idx)
  genes = GENES(idx);
  [tmp pos] = sort([genes.start]);
  genes = genes(pos);
  clear pos idx start_pos
else
  block_length = REG.stop-REG.start;
  if ~isfield(regions_par,'max_length')|isempty(regions_par.max_length)
    num_blocks = 1;
  else
    num_blocks = ceil(block_length/regions_par.max_length);
    if num_blocks>0,
      block_length = ceil(block_length/num_blocks);
    else
      block_length = 0 ;
    end ;
  end
  start = REG.start+2*regions_par.minCutDist;
  genes = [];
  %if keyboard_allowed(), keyboard ; end ;
  for j=1:num_blocks
    stop = min(start+block_length, REG.stop);
    genes(j).id = [];
    genes(j).transcript_status =[];
    genes(j).is_complete = [];
    genes(j).start = start+2*regions_par.minCutDist+10;
    genes(j).stop = stop-2*regions_par.minCutDist-10;
    start = stop;
  end
end

%genes
cnt=0 ;
while g<size(genes,2)
  cnt = cnt + 1 ;
  if cnt>g+20, fprintf('break') ; break ; end ;

  region = [];
  region.organism = REG.organism;
  region.chr = REG.chr;
  region.chr_num  = REG.chr_num;
  region.strand  = REG.strand;
  region.id = length(regions)+1;
  region.TR_idx = REG.TR_idx ;
  region.num_fragments = 1;
  region.config = REG.config;
  region.paralogs = [];
  r_genes = [];
  region.gene_ids=[];
  region.gene_status=[];
  region.gene_complete=[];
  genedist_too_short=0;
  if new_tr
    region.start = REG.start;
    region.stop = REG.start;
    new_tr=0;
  else
    region.start = region_end_stop;
  end
  %add genes to region as long as min gene num is not achieved, genedist is too short or more if you are at 
  %the end of the contig and last region would be to short
  while  g<size(genes,2)&&...
        (length(r_genes)<min_genes_per_region||...
         g+min_genes_per_region>size(genes,2)||...
         genedist_too_short ||...
         (length(r_genes)>0 && isfield(region, 'stop') && ...
          (region.stop+regions_par.offset>contigstop||region.stop-regions_par.offset<1))) 
    % is this the last gene of the contig?
    % enough genes for this region collected?
    % still enough genes to fill another region?
    % next gene closer than 2*minCutDist?
    g=g+1;
    r_genes = [r_genes genes(g)];
    region.gene_ids = [region.gene_ids genes(g).id];
    region.gene_status = [region.gene_status max(genes(g).transcript_status)];
    region.gene_complete = [region.gene_complete max(genes(g).is_complete)];
    map(genes(g).id) = region.id;
    gene_stop = max([r_genes.stop]);
    if g==size(genes,2)
      region.stop = REG.stop;
    else
      genedist = genes(g+1).start-gene_stop-1;
      rand_num = ceil(rand*(genedist-2*regions_par.minCutDist));
      region.stop = gene_stop+regions_par.minCutDist+rand_num;
      genedist_too_short = (genedist<=2*regions_par.minCutDist+1);
    end
  end%while add genes
  region.left_dist = min([r_genes.start])- region.start;
  region.right_dist = region.stop - gene_stop;
  %   assert(length(region.gene_ids)>=min_genes_per_region)
  region_end_stop = region.stop+1;
  
  %if ~isempty(GENES)
   % check_block_gene_ids(region, GENES);
  %end

  region.reg_no_overlap(1) = max([contigstart+regions_par.offset region.start]);
  region.reg_no_overlap(2) = min([contigstop-regions_par.offset region.stop]);
  region.start = max([contigstart region.start-regions_par.offset]);
  region.stop = min([contigstop region.stop+regions_par.offset]);
  region.reg_no_overlap = region.reg_no_overlap-region.start+1;
 
  switch region.strand
   case '+',
    region.offset = region.start - 1;
   case '-',
    region.offset = region.stop + 1;
  end
  % keyboard
  regions = [regions region];
end % loop over genes in contig


return

function check_block_gene_ids(blocks, genes)
% find all genes that overlap with a block and check if the gene_ids list of that block is correct and complete
% 
% this is also a check for the blocks being produced correctly without cutting any gene.


chrs = unique([blocks.chr_num]);
strands = unique([blocks.strand]);
wrong_idx = [];
cut_idx = [];

block_chr_num = [blocks.chr_num] ;
genes_chr_num = [genes.chr_num] ;
block_strand  = [blocks.strand] ;
genes_strand  = [genes.strand] ;

for c = chrs
  %cblocks = blocks(block_chr_num==c);
  %cgenes  = genes(genes_chr_num==c); 
  for s=strands 
    %sblocks = cblocks([cblocks.strand]==s);
    %sgenes  = cgenes([cgenes.strand]==s); 
    sblocks = blocks(block_chr_num==c & block_strand==s) ;
    sgenes = genes(genes_chr_num==c & genes_strand==s) ;
    for j=1:length(sblocks),
      if mod(j,100)==0, fprintf('block: %i (%i)\r',j,length(sblocks)), end 
      idx = find([sgenes.stop]>sblocks(j).start & [sgenes.start]<sblocks(j).stop);
      %assert(isequal([sgenes(idx).id],sblocks(j).gene_ids))
      if ~isequal(sort([sgenes(idx).id]),sort(sblocks(j).gene_ids)),
        if keyboard_allowed(),
          keyboard
        end ;
        wrong_idx = [wrong_idx sblocks(j).id]; 
        g = sgenes(idx);
        for k = 1:length(g)
          if g(k).stop>sblocks(j).stop|g(k).start<sblocks(j).start
            cut_idx = [cut_idx g(k).id];
          end
        end
        %keyboard
      end, 
    end%sblocks
  end%strand
end%chr
if length(wrong_idx)>0
  fprintf('%i blocks have incorrect lists of associated genes\n',length(wrong_idx))
end
if length(cut_idx)>0
  fprintf('%i genes have been cut by a block boundary\n',length(cut_idx))
end
return

function [chr, strand, pos, score, content_name, score_name, chr_name_dic, content_name_dic, score_name_dic, success]=read_contpred_bin(contpred_fname)
% [chr, strand, pos, score, content_name, score_name, chr_name_dic, content_name_dic, score_name_dic, success]=read_contpred_bin(contpred_fname)

chr_name_dic = {} ;
score_name_dic = {} ;
content_name_dic = {} ;

chr=[] ; 
content_name = [] ;
score_name = [] ;
strand = '' ;
pos = [] ;
score = [] ;

if ~fexist(contpred_fname),
  success=false ;
  return ;
end ;

cnt = 0; last=0;
while ~last,
  try
    CPF=load(contpred_fname, sprintf('CPF_%i', cnt)) ;
    if ~isfield(CPF, sprintf('CPF_%i', cnt)),
      break
    end ;
  catch
    break ;
  end 
  if ~isempty(CPF)
    eval(sprintf('CPF=CPF.CPF_%i;', cnt)) ;
    cnt=cnt+1 ;
  %else
  %  CPF=load(contpred_fname) ;
  %  last = 1 ;
  end ;

  if isempty(CPF.pos)
    assert(isempty(CPF.score))
    continue;
  end

  if any(ismember(chr_name_dic, CPF.contig_name)),
    contig_idx = find(ismember(chr_name_dic, CPF.contig_name)) ;
  else
    chr_name_dic{end+1} = CPF.contig_name ;
    contig_idx = length(chr_name_dic) ;
  end ;

  if any(ismember(score_name_dic, CPF.score_name)),
    score_idx = find(ismember(score_name_dic, CPF.score_name)) ;
  else
    score_name_dic{end+1} = CPF.score_name ;
    score_idx = length(score_name_dic) ;
  end ;

  if any(ismember(content_name_dic, CPF.content_name)),
    content_idx = find(ismember(content_name_dic, CPF.content_name)) ;
  else
    content_name_dic{end+1} = CPF.content_name ;
    content_idx = length(content_name_dic) ;
  end ;

  chr(end+1:end+size(CPF.pos,1))=contig_idx ;
  score_name(end+1:end+size(CPF.pos,1))=score_idx ;
  content_name(end+1:end+size(CPF.pos,1))=content_idx ;
  pos(end+1:end+size(CPF.pos,1),:)=CPF.pos ;
  score(end+1:end+size(CPF.pos,1))=CPF.score ;
  strand(end+1:end+size(CPF.pos,1))=CPF.strand ;
end ;

success = true ;

return

function [PAR, jobinfo ,all_done]=masterscript_contents(PAR,check_files,verb)
% [PAR, jobinfo ,all_done] = masterscript_contents(PAR,check_files,verb)

%cd ~/svn/projects/genefinding/sensors
%paths

tasks = PAR.tasks.contents;  
%run_locally = tasks.run_locally;
RPROC = PAR.RPROC;
SETs = PAR.SETs;

FN = PAR.FN;  
fn_genome_config = PAR.FN.input.fn_genome_config;
 
if nargin<2 
  check_files =0;
end
if nargin<3 
  verb = 0;
end

fid =1;
fprintf(fid,'\n\n-----------\n');
fprintf(fid,'CONTENT :  %s\n',PAR.Content_name);
fprintf(fid,'-----------\n');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% --- Train CONTENTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PAR.method.submit_batch=1;
all_done=0;
%train_ok = 0;

jobinfo=struct ;

if tasks.prepare_labels
  %fprintf('start training for sensor %s on organism %s with method %s: %s\n\n',sensor,organism,method)
  gen_training_examples4content(PAR) ;
end
if tasks.modelsel | tasks.train
  jobinfo = ms_gfwd_starter(PAR);
  if ~isempty(jobinfo), return ; end ;
end
if tasks.get_train_error
  jobinfo = predict_testplif_geterror(PAR)
  if ~isempty(jobinfo), return ; end ;
end
if tasks.learn_plifs
  calc_plifs(PAR);
end

%if ~train_ok
%  fprintf('something wrong with training files, check first!!  \n')
%end


if ~tasks.pred_on_genome
  all_done = 1 ;%train_ok ;
  return
end


return

function jobinfo = gen_training_examples4content(PAR)
% gen_labels_and_train(PAR)

%paths
  

Content_name = PAR.Content_name;
Content = PAR.Contents.(PAR.Content_name);
num_splits = PAR.SETs.num_splits;
label_source  = PAR.label_source.(Content_name);
info = PAR.info_genes.transcript;


FN = PAR.FN;
fn_genome_config = FN.input.fn_genome_config ;
fn_training_blocks = FN.output.fn_training_blocks;
fn_genes_contents = FN.output.fn_genes_contents ;

input_cont = FN.input_cont.(Content_name);
fn_candsites =  input_cont.fn_candsites;
fn_filter_settings = input_cont.fn_filter_settings;
fn_examples = input_cont.fn_examples;
fn_example_statistics = input_cont.fn_example_statistics;

run_locally = PAR.tasks.contents.run_locally.pos ;

if ~run_locally
  RPROC = PAR.RPROC;
end



jobinfo = [] ;
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% load  trivial_regions, blocks, splits
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

trivial_regions = init_regions(fn_genome_config);


if ~fexist(fn_training_blocks)
  error('training block file does not exist; create it first with prepare_regions')
else
  blocks = load_struct(fn_training_blocks, 'blocks') ;
  load(fn_training_blocks, 'split') ;
  assert(length(fieldnames(split))==num_splits);
end


generate_examples=1;
if fexist([fn_candsites 'PAR.mat'])
  fprintf(1,'files with candidate pos already exists\n')
  Old = load([fn_candsites 'PAR.mat'],'P');
  diff_fields = compare_PAR(Content,Old.P.Content) ;
  if ismember('Content',Content_name)
    warning('label file exists, Content parameters are differnt!')  
    % fprintf('label file exists, Content parameters are differnt!')  
  end
  %[temp, num_files] = unix(sprintf('ls %scontig_*.pos | wc -l',fn_candsites));
  d=dir(sprintf('%scontig_*.pos',fn_candsites));
  num_files = length(d) ;

  if (num_files==length(trivial_regions))
    generate_examples=0;
    fprintf(1,'all cand files found; no label generation\n')
  else
    fprintf(1,'%i/%i cand files missing; start label generation\n',length(trivial_regions)-num_files,length(trivial_regions))
  end
  clear Old
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% generating .pos .label files
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if generate_examples
  fprintf(1,'generating training examples\n')
  jobinfo = rproc_empty(0) ;
  
  P.fn_genes_contents = fn_genes_contents ;
  P.fn_candsites = fn_candsites;
  P.label_source = label_source;
  P.Content = Content; 
  P.info = info;
  
  D.P = P ;
  for j=1:length(trivial_regions)
    D.t_region = trivial_regions(j);
    %keyboard
    filename = sprintf('%scontig_%i%s',fn_candsites,trivial_regions(j).chr_num,trivial_regions(j).strand);
    if fexist(filename)
      continue
    end
    if run_locally
      save_label_files_starter(D);
    else
      [mem_req, time_req, opts] = rproc_memtime_policy('save_label_files_starter',  0, RPROC.options) ;%RPROC.MEMREQ ;

      jobinfo(j) = rproc('save_label_files_starter', D, mem_req, opts, time_req);
      % save_label_files_starter(D)
    end;
    % save_label_files(t_region,PAR,genes)
  end
  if ~isempty(jobinfo),
    [jobinfo_1, num_crashed]=rproc_wait(jobinfo, 10, 1, -1);
  else
    num_crashed = 0 ;
    jobinfo_1 = rproc_empty(0) ;
  end ;
  %[temp, num_files] = unix(sprintf('ls %scontig_*.pos | wc -l',fn_candsites));
  d=dir(sprintf('%scontig_*.pos',fn_candsites));
  num_files = length(d) ;
  if num_files~=length(trivial_regions)
    error('number of files (%i) not equal to number of trivial_regions (%i)',num_files, length(trivial_regions));
  end
  if num_crashed>0, 
    num_crashed
    %return
  end 
  if ~isempty(jobinfo_1)
    rproc_cleanup(jobinfo_1) ;
  end ;
end
clear P

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% generating example files
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

modes = {'train','test'};
for mode=modes
  [filename, exists]=gen_example_filename(Content,fn_filter_settings,fn_examples,mode{1},num_splits);
  if ~exists
        
    P.Content = Content;
    P.num_splits = num_splits;
    P.fn_training_blocks = fn_training_blocks;
    P.fn_candsites = fn_candsites;
    P.fn_filter_settings = fn_filter_settings ;
    P.fn_examples = fn_examples;
    P.fn_example_statistics = fn_example_statistics;

    if ~run_locally
      D.blocks = blocks;
      D.split = split;
      D.P = P;

      [mem_req, time_req, opts] = rproc_memtime_policy('save_content_examples',  0, RPROC.options) ;%RPROC.MEMREQ ;
      
      jobinfo = rproc('save_content_examples', D, mem_req, opts, time_req);

      [jobinfo_1, num_crashed]=rproc_wait(jobinfo, 60, 1, -1);
    else
      save_content_examples(blocks,split,P) ; num_crashed = 0 ;
    end
    if num_crashed>0
      return
    end
  end
end


return

function x = save_content_examples(blocks,split,P)
%save_examples(blocks,split,P)
%
% INPUT 	regions with field Signals with field `Signal.name` with fields:
%			-pos
%			-anno_cover
%			-est_cover
%			-cdna_cover
%	PAR with fields
%			-organism
%			-Content.filter_label
%			-Content.name
%			-Content.consensus
%			-FN.input_sig.fn_examples
%			-FN.input.fn_split
%			-FN.input.fn_regions

%dummy return argument
x=0;

%only for RPROC
if nargin<3
  %addpath ..
  % use the rproc.addpath way to add paths in client processes!!
  %paths
  P = blocks.P;
  split = blocks.split;
  blocks = blocks.blocks;
end

Content = P.Content ; 
num_splits = P.num_splits;

fn_training_blocks = P.fn_training_blocks;
fn_candsites = P.fn_candsites;
fn_filter_settings = P.fn_filter_settings;
fn_examples = P.fn_examples;
fn_example_statistics = P.fn_example_statistics;

Content_name = Content.name;

fields = fieldnames(Content.filter_label);
fprintf( 1, 'save_examples... (%d fields, %d splits)\n', length(fields), num_splits );

min_seg_length = 7;

for ff = 1:length(fields)
   for ss=1:num_splits
    %---------------------------------------------
    %CREATE OUTPUT FILENAME
    %----------------------------------------------  
    filename = gen_example_filename(Content,fn_filter_settings,fn_examples,ff,ss);
    if ~fexist(filename)
      num_examples = 0;
      POS.fn_training_blocks = fn_training_blocks;	 
      split_reg_ids = getfield(split,sprintf('split_%i',ss));
      blocks_idx = find(ismember([blocks.id],split_reg_ids));
      split_regs = blocks(blocks_idx);
      clear blocks_idx split_reg_ids
     

      %-------------------------------------------------- 
      % determine the number of positive and negative 
      % examples in this split
      %-------------------------------------------------- 
      t = cputime;
      num_pos_ex = 0;
      num_neg_ex = 0;
      for r=1:length(split_regs)
        temp_region = retrieve_signals(split_regs(r),fn_candsites,Content_name,{'label'});
	num_pos_ex = num_pos_ex+sum(temp_region.Signals.(Content_name).label == 1);
	num_neg_ex = num_neg_ex+sum(temp_region.Signals.(Content_name).label == -1);
      end
      fprintf(1,'it took %.2fs to determine the number of examples\n', cputime-t);
      vec=subsample_policy([num_pos_ex num_neg_ex], Content_name);
      max_num_pos = vec(1);
      max_num_neg = vec(2);
      reduction_factor_pos = max_num_pos/num_pos_ex;
      reduction_factor_neg = max_num_neg/num_neg_ex;
      num_pos = max_num_pos+max_num_neg;

 
      %-------------------------------------------------- 
      % initialize arrays 
      %-------------------------------------------------- 
      XT = cell(1,num_pos);
      LT = zeros(1,num_pos);
      POS.pos=zeros(1,num_pos);
      POS.region_id=zeros(1,num_pos);
      
      if isfield(Content.filter_label.(fields{ff}), 'USE_ALL')&&...
            Content.filter_label.(fields{ff}).USE_ALL
        Content.Conf_names = {} ;
      end ;
      
      t = cputime;
      fprintf(1,'starting with split %i\n',ss)
      for r=1:length(split_regs)
        if mod(r,100)==0,
          fprintf('region %i/%i\r', r, length(split_regs)) ;
        end ;

        
        if fn_candsites(1)=='~'
          fn_candsites = sprintf('%s/%s', getenv('HOME'), fn_candsites(2:end));
        end
        if ~isfield(split_regs, 'Contents')
          temp_region = retrieve_signals(split_regs(r),fn_candsites,Content_name,{'pos2','label',Content.Conf_names{:}});
        end
        
        
        sig_field = getfield(temp_region.Signals,Content_name);
        if isfield(Content.filter_label.(fields{ff}),'use_label')
          sig_field.label = sig_field.(Content.filter_label.(fields{ff}).use_label);
        end
        
        if ~isfield(split_regs,'seq') 		
          temp_region = load_sequence(temp_region);
        end
        seq = temp_region.seq;

        %---------------------
        %FILTER EXAMPLES
        %---------------------
        if ~isfield(Content.filter_label.(fields{ff}), 'USE_ALL')||...
              ~Content.filter_label.(fields{ff}).USE_ALL
          if ~isfield(Content.filter_label.(fields{ff}),'conf_pos') ...
                && isfield(Content.filter_label.(fields{ff}),'conf')
            error('fix this: filtering will not work')
          end
          idx = feval(Content.filter_fct, temp_region, Content, fields{ff},1);
        else
          idx = 1:length(sig_field.pos);
        end
        idx1 = find(sig_field.pos2<=length(seq));

        idx = intersect(idx,idx1);

	pos_idx = find(sig_field.label==1);
        if reduction_factor_pos<1
	  keep = randperm(length(pos_idx));
	  pos_idx = pos_idx(keep(1:floor(length(pos_idx)*reduction_factor_pos)));
	end
	neg_idx = find(sig_field.label==-1);
	if reduction_factor_neg<1
	  keep = randperm(length(neg_idx));
	  neg_idx = neg_idx(keep(1:floor(length(neg_idx)*reduction_factor_neg)));
	end
	if reduction_factor_pos<1||reduction_factor_neg<1
          if ~isempty(pos_idx) && ~isempty(neg_idx),
            un=[pos_idx', neg_idx'] ;
          elseif isempty(pos_idx),
            un = neg_idx' ;
          else
            un = pos_idx' ;
          end ;
	  idx = intersect(idx, un);
	end

        if isempty(idx); continue; end
        
        %--------------------
        %CREATE XTs
        %--------------------
        XT_temp = cell(1,length(idx));
        rid_temp = zeros(1,length(idx))+temp_region.id;
        for num=1:length(idx)
          if any(~ismember(seq(max(1,sig_field.pos(idx(num))):sig_field.pos2(idx(num))),'ACGT'))
            warning(sprintf(' examples containing non ACGT symbols excluded'))
            continue
          end
          if length(seq(max(1,sig_field.pos(idx(num))):sig_field.pos2(idx(num))))<min_seg_length
            warning(sprintf('very short segments are excluded (<5bp)'))
            continue
          end
          if isempty(seq(max(1,sig_field.pos(idx(num))):sig_field.pos2(idx(num))))
            warning(sprintf('empty range'))
          end
          XT_temp{num} = seq(max(1,sig_field.pos(idx(num))):sig_field.pos2(idx(num)));
        end%loop over rows
        LT_temp= sig_field.label(idx)';
        pos_temp = sig_field.pos(idx)';
        pos2_temp = sig_field.pos2(idx)';
        assert(size(LT,2)==num_examples||LT(num_examples+1)==0)
        assert(num_examples==0||LT(num_examples)~=0)
        
        %[size(XT,2) num_examples+length(idx) ceil(size(XT,2)*r/length(split_regs))]
        if size(XT)<num_examples+length(idx), 
          warning('extending XT') ;
          LT(ceil(size(XT,2)*1.2)) = 0 ;
          
          XT(1,ceil(size(XT,2)*1.2)) = 0 ;
        end ;
        XT(num_examples+1:num_examples+length(idx)) =  XT_temp(:);	
        LT(num_examples+1:num_examples+length(idx)) =  LT_temp;
        POS.pos(num_examples+1:num_examples+length(idx))= pos_temp;
        POS.region_id(num_examples+1:num_examples+length(idx))=rid_temp;
        num_examples = num_examples+length(idx);
      end%loop over blocks
      
      
     
      fprintf(1, 'time required for one split: %.2f\n',cputime-t)
      fprintf(1,'save file %s\n',filename)
      %=== remove zeros
      assert(all(LT(num_examples+1:end)==0))
      LT = LT(1:num_examples);
      XT = XT(1:num_examples);
      
      
      % balance length distribution of negative examples
      %---------------------------------------------------------------
      fprintf(1,'start fit_length_hist\n');
      bins = 100;
      reduce_pos=0;% throw away positive examples if necessary to achieve similar histograms
      
      neg_examples = XT(LT==-1);
      pos_examples = XT(LT==1);
      % if ~use_frame
      neg_examples = fit_length_hist(pos_examples,neg_examples,bins, reduce_pos);
      % end
      % keyboard
      LT = [-ones(size(neg_examples)) ones(size(pos_examples))];
      XT = {neg_examples{:} pos_examples{:}};
      idx = randperm(length(LT));
      LT = LT(idx);
      XT = XT(idx);
      %----------------------------------------------- 
      % remove examples that appear more than once
      %----------------------------------------------- 
      if 0
        [tmp1 idx tmp2] = unique(XT','rows');
        idx = idx(randperm(length(idx)));
        XT = XT(:,idx);
        LT = LT(idx);
        POS.pos = POS.pos(idx);
        POS.region_id = POS.region_id(idx);
      end
     
      %----------------------------------------------- 
      % save matrix in pieces into one file
      %----------------------------------------------- 
      % assert(sum(LT==1)>0);
      % fprintf( 'trying to save %s\n', filename );
      save(filename,'XT');
      % save_matrix(filename, XT, 'XT');
      % save(filename,'-append','LT','POS','PAR');
      save_append(filename, 1,'LT', LT, 'POS', POS, 'P', P);
      % save_append(filename, 1,'LT', LT, 'POS', POS, 'PAR', PAR);
    
    end%if ~fexist(filename)
    
   end%loop over splits
   
   stat = save_example_statistics(Content, fn_filter_settings, ...
                                  fn_examples, fn_example_statistics, ...
                                  num_splits);
end%loop over filter_label fields


return

function [neg pos] = fit_length_hist(pos, neg, steps, reduce_pos, make_plot)

% pos and neg need to be cell arrays of strings

if nargin<5
  make_plots=0;
end

if nargin<4
  reduce_pos=0;
end

% count lengths 
neg_lengths = zeros(1,length(neg));
for j=1:length(neg)
  neg_lengths(j)=length(neg{j});
end
pos_lengths = zeros(1,length(pos));
for j=1:length(pos)
  pos_lengths(j)=length(pos{j});
end

max_len = max([pos_lengths neg_lengths]);
step=ceil(max_len/steps);
bins = logspace(0,log10(max_len),steps);
bins(1)=0;
if max(bins)<max_len,
  bins = [bins max_len];
end

pos_hist = histc(pos_lengths,bins);
neg_hist = histc(neg_lengths,bins);
if make_plots
  figure;
  plot(1:length(bins),neg_hist,1:length(bins),pos_hist)
end

% scale histograms such that they are comparable
neg_hist_norm = neg_hist/length(neg);
pos_hist_norm = pos_hist/length(pos);

if make_plots
  figure;
  plot(1:length(bins),neg_hist_norm,1:length(bins),pos_hist_norm)
end

% calculate the number of sequences that has to be deleted for each length bin
overh = ceil(length(neg)*(neg_hist_norm-pos_hist_norm));

rm_idx=[];
for j = 1:length(neg)  
  len = length(neg{j});
  bin = sum(bins<=len);
  if overh(bin)>0
    overh(bin) = overh(bin)-1;
    rm_idx = [rm_idx j];
  end
end
waste = {neg{rm_idx}};
w_lens = neg_lengths(rm_idx);
neg(rm_idx)=[];
neg_lengths_2 = neg_lengths;
neg_lengths_2(rm_idx)=[];

% sort waste
[w_lens idx] = sort(w_lens,'descend');
waste = {waste{idx}};
clear idx

% recycle
new_neg = {}; 
j = 0;
while j<length(waste)
  j = j+1;
  %idcs = find(overh<0); 
  %idx = idxs(ceil(rand*length(idcs)));
  [tmp idx] = min(overh);
  if tmp>=0
    break;
  elseif w_lens(j)<bins(idx)
    continue;
  elseif idx == length(bins)
    continue;
  else
    len = floor(bins(idx)+rand*(bins(idx+1)-bins(idx)));
    p = min(length(waste{j}),len);
    new_neg = {new_neg{:} waste{j}(1:p)};
    if p<w_lens(j)-30 
      waste = {waste{:} waste{j}(p+1:end)};
      w_lens(end+1) = length(waste{end});
    end
    overh(idx) = overh(idx)+1;
  end
end

neg_hist_2 = histc(neg_lengths_2,bins);

if make_plots
  figure;
  plot(1:length(bins),neg_hist_2,1:length(bins),pos_hist)
end

neg = {neg{:} new_neg{:}};

if make_plots
  for j=1:length(neg)
    neg_lengths_3(j)=length(neg{j});
  end
  neg_hist_3 = histc(neg_lengths_3,bins);

  hist_opt = neg_hist_3-overh;
  figure;
  plot(1:length(bins),neg_hist_3/sum(neg_hist_3),1:length(bins),pos_hist/sum(pos_hist))
end

if reduce_pos
  rm_idx=[];
  for j = 1:length(pos)  
    len = length(pos{j});
    bin = sum(bins<=len);
    if overh(bin)<0
      overh(bin) = overh(bin)+1;
      rm_idx = [rm_idx j];
    end
  end
  pos(rm_idx) = [];
end
%handle = gca;
%prop = get(handle);
%prop.XTick = bins;
%set(handle, 'XTick',bins);

%keyboard 




return


function [ n, bin ] = histc( x, edges );

assert( all( edges == edges ) );

last = repmat( edges(end), size(x) );
bin = zeros( size(x) );
for( i = 1:length(edges) )
  edge = repmat( edges(i), size(x) );
  bin = bin + 1 * ( edge <= x & x <= last );
end;

% --- the following code fails due to
%  /fml/ag-raetsch/share/software/matlab-7.6/toolbox/matlab/datafun/hist.m
% ---
%n = hist( bin, 0:length(edges) );
%n(1) = [];
% --- end of failing code ---

n = zeros( size(edges) );
for( i = 1:length(edges) )
  n(i) = sum( bin == i );
end;


return

function [Data,par] = load_content_data(Content,fn_filter_settings,fn_examples, which, filter_mode,subsample_neg,subsample_pos)
% [Data,par] = load_content_data(Content,fn_filter_settings,fn_examples, which, filter_mode,subsample_neg,subsample_pos)


method = Content.method;

XT = []; LT = [] ;  POS.region_id = []; POS.pos=[]; POS_tmp = []; 

if nargin<4
  subsample_neg=1;
end
if nargin<5
  subsample_pos=1;
end

use_cons = method.use_cons;
% if nargin<6
%   use_cons = 0;
% end
for w = which
  % create filename of input file
  filename = gen_example_filename(Content,fn_filter_settings,fn_examples,filter_mode,w);
  
  fprintf('loading XT, LT and POS from file %s \n',filename)
  L = load(filename,'LT','XT','POS');
  %  L.XT = load_matrix(filename, 'XT');
  % [L.XT, L.LT, P.POS, L.SC_] = subsample(L.XT, L.LT, P.POS, L.SC_, subsample_neg, subsample_pos) ;

  if isfield(L,'LT')
    if size(L.LT,1)>1,
      L.LT = L.LT';
    end
    LT = [LT L.LT] ;
  end
  POS.region_id =[POS.region_id L.POS.region_id];
  POS.pos = [POS.pos L.POS.pos];
  
  % generate feature vectors
  %----------------------------------------------------------------
  dim = 0;
  for o=1:length(method.kernels)
    dim = dim + 4^method.kernels{o}.wordlen;
  end
  xt = zeros(dim,length(L.XT));
  for j=1:length(L.XT) 
    dim = 0;
    for o=1:length(method.kernels)
      [mask hist] = compute_mers(L.XT{j}, method.kernels{o}.wordlen, method.kernels{o}.stepping, method.kernels{o}.offset);
      xt(dim+[1:4^method.kernels{o}.wordlen],j) = hist';
      dim = dim+ 4^method.kernels{o}.wordlen;
    end
  end
  clear L
  XT=[XT xt];
  
end ;

if ~isempty(LT)
  fprintf('%i positive, %i negative examples (%.2f %% positives), sequences of length %i \n',sum(LT==1),sum(LT==-1),sum(LT==1)/length(LT)*100, size(XT,1));
end


Data.LT = LT;
% Data.XT = char(XT) ;
Data.XT = XT;
Data.Pos = POS;
clear XT LT POS

assert(size(Data.XT,2)==length(Data.LT))
% assert(length(Data.LT)==length(Data.Pos.pos))
% assert(length(Data.LT)==length(Data.Pos.region_id))







return

function [mask hist] = compute_mers(string, order, stepping, offset)
%function [mask hist] = compute_mers(string, order, stepping)

mask    = zeros(1,4^order) ;
hist    = zeros(1,4^order) ;

%if nargin<3,
%  stepping = 1;
%end;
%
%if nargin<4,
%  offset = 0;
%end;
%
%if isempty(string), 
%  return; 
%end;
string = lower(char(string)) ;

if ~isempty(find(~ismember(string,'acgt')))
  string(find(~ismember(string,'acgt'))) = 'a';
end;

assert(order<=8);

if length(string)<order,
  return ;
end
t = sg('translate_string', double(string), order, order-1) ;
if length(t)<order
  return ;
end ;

% transform t to matlab coordinates
t = t(1:end-order+1)+1 ;

% throw away kmers if stepping > 1
assert(stepping>0);
%t = t(1:stepping:end);
t = t(end-offset:-stepping:1);

% reduce t to the information whether or not a certain kmer occurs 
u = unique(t) ;

% get histogram counts
t = sort(t);
[dummy loc] = ismember(u, t);
occ = loc - [0 loc(1:end-1)];

% return histogram
hist(u) = occ/sum(occ);

% return "binary histogram" 
% telling whether or not a certain kmer is present
mask(u) = 1/sqrt(length(u)) ;

return

function [jobinfo, fn_pred]=masterscript_contents_test(P, check_files, run_locally, do_gzip, save_as, keep_all_files)
  % [jobinfo ,all_done]=masterscript_contents_test(P, check_files, run_locally, do_gzip, save_as, keep_all_files)

num_splits = P.SETs.num_splits ; 
Content_name = P.Content_name;

FN = P.FN;  
fn_genome_config = FN.input.fn_genome_config;
fn_training_blocks = FN.output.fn_training_blocks;  
fn_test_blocks = FN.output.fn_test_blocks;

fn_pos =  FN.input_cont.(Content_name).fn_pos;
fn_pred = FN.output_cont.(Content_name).fn_pred;
fn_svms = FN.output_cont.(Content_name).fn_SVMs ;
% fn_svms = sprintf('%sbest_partition=', FN.output_cont.(Content.name).fn_SVMs) ;

L = load([fn_svms '1.mat'], 'Train');
Content =  L.Train.PAR.Signal;
clear L 

if nargin<2 
  check_files = 1;
end

if nargin<3 
  run_locally.pos = 1;
  run_locally.predict = 1;
  run_locally.concat = 1;
  run_locally.save = 1;
end

if nargin<4 
  do_gzip = 0;
end

if nargin<5 
  save_as.spf_ascii = 1 ;
  save_as.spf_binary = 1 ;
  save_as.wiggle = 0 ;
end

if nargin<6 
  keep_all_files = 0 ;
end

%if ~run_locally  
  RPROC = P.RPROC;
%else
%  RPROC = struct;
%end

fid =1;
fprintf(fid,'\n\n-----------\n');
fprintf(fid,'CONTENT :  %s\n',Content.name);
fprintf(fid,'-----------\n');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% --- Generate Positions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

jobinfo=struct ;

%[temp, num_files] = unix(sprintf('ls %scontig_*.pos | wc -l',fn_pos));
d=dir(sprintf('%scontig_*.pos',fn_pos));
num_files = length(d) ;
%[temp, num_files1] = unix(sprintf('ls %scontig_*.svm | wc -l',fn_pos));
d=dir(sprintf('%scontig_*.svm',fn_pos));
num_files1=length(d) ;

genome_info = init_genome(fn_genome_config);
if isempty(num_files)||(num_files<length(genome_info.contig_names)*2)||...
      isempty(num_files1)||(num_files1<length(genome_info.contig_names)*2)
  jobinfo = gen_all_positions(fn_genome_config,fn_pos,Content,fn_training_blocks,fn_test_blocks,num_splits, run_locally.pos, RPROC);
  if ~isempty(jobinfo),
    fprintf('generating genome position files\n')
    jobinfo = rproc_resubmit(jobinfo);
    jobinfo = rproc_wait(jobinfo, 5, 1, -1); 
  end ; 
else
  fprintf('genome position files already exist\n')
end

avg_all_svms=0;
subtract_diff=1;
% keyboard
jobinfo = genomewide_predictions(fn_genome_config,fn_pos,fn_pred,fn_svms,num_splits,avg_all_svms,run_locally.predict, RPROC);
fprintf('all %s predictions done\n',Content.name)


% jobinfo = convert_predictions2confs(fn_genome_config,fn_pred,fn_svms,num_splits,run_locally, RPROC);
% fprintf('conversion done\n')

%run_locally_concat = run_locally ;
jobinfo = concat_genomewide_predictons(fn_genome_config,fn_pos,fn_pred, num_splits,avg_all_svms,subtract_diff, run_locally.concat, RPROC);
fprintf('concatination done\n')

resolution = Content.export_settings.resolution ;
conf_cum_thresh = Content.export_settings.conf_cum_thresh;

%run_locally_save = run_locally ;
inventory = {'output', 'pos'} ;
[jobinfo] = save_predictions(fn_genome_config, fn_pred, ...
                             Content_name, resolution, ...
                             conf_cum_thresh, '+-', ...
                             run_locally.save, RPROC, ...
                             do_gzip, save_as, inventory);
fprintf('saving done\n')


if check_files
  % fprintf('checking files \n')
  % check_genomewide_output(P) % this fct compares prediction results with test results from training (cross_validation)
  % all_good = check_predictions(fn_genome_config,fn_pred,Content_name,resolution)
else
  fprintf('WARNING: files not checked\n')
end


fprintf('%s COMPLETE !\n\n',Content.name)


if ~keep_all_files
 %  unix(sprintf('rm -rf %s*_split=*', FN.output_sig.(Content_name).fn_pred));
end

return

function [RFV, RSV, train_test_overlap] = svm_content_predict(Train, Data, svmFileName,outFileName,pred_from_seq, check_overlap);
% [RFV, RSV] = svm_predict(Train, Data, svmFileName,outFileName);
%
%------------------------------------------------------------------------------%
% deal with parameters 
%------------------------------------------------------------------------------%
% for usage with rproc

if nargin==1
  Data  = Train.Data;
  svmFileName = Train.svmFileName;
  outFileName = Train.outFileName;
  Train = Train.Train;
end

method = Train.PAR.Signal.method ;

if ~exist('check_overlap', 'var'),
  check_overlap=1 ;
end ;

sg_ver = sg('get_version');

assert(size(Data.pos,1)==1|size(Data.pos,2)==2|isempty(Data.pos))
if size(Data.pos,1)~=1
  Data.pos = Data.pos';
end

if length(Train.w)~=sum(4.^[3:6]),
  fprintf('Assumed degrees 3:6 are not used in trained content predictor (dim %i)\n', length(Train.w)) ;
  Train.w(sum(4.^[3:6]))=0 ;
end ;

assert(isequal(sort(Data.pos), Data.pos));
sg('init_dyn_prog', 1);
sg('precompute_content_svms', Data.seq, int32(Data.pos-1), Train.w);
output = sg('get_lin_feat');
% output = output - output(end)/Data.pos(end).*Data.pos;

%------------------------------------------------------------------------------%
% save output
%------------------------------------------------------------------------------%

PAR = Data.PAR;
PAR.Signal = Train.PAR.Signal;

if isfield(Data, 'LT')
  RSV = calcrocscore(output, Data.LT); 
  RFV= calcrfcscore(output, Data.LT);
  LT = Data.LT;
  %   save( outFileName, 'output','LT','PAR', 'svmFileName','RSV','RFV' );
  POS = Data.Pos;
  fprintf( 'save output to %s\n',outFileName );
  inventory = {'output', 'POS'};
  %save( outFileName, 'inventory', 'output','LT','POS', 'Train', 'svmFileName','RSV','RFV','sg_ver','PAR' );
  save( outFileName, 'inventory', 'output','LT','POS', 'svmFileName','RSV','RFV','sg_ver','PAR' );
else
  pos = Data.pos;
  fprintf( 'save output to %s\n',outFileName );
  inventory = {'output', 'pos'};
  %save( outFileName, 'inventory', 'output', 'pos', 'Train', 'svmFileName','sg_ver','PAR' );
  save( outFileName, 'inventory', 'output', 'pos', 'svmFileName','sg_ver','PAR' );
end

return
%%%%TEST



L = load(['galaxy/testit/content_cds_exon_classifier_files/examples_settings=2_split=1.mat']);
LL = load('galaxy/testit/content_cds_exon_classifier_files/output_SVMCOMBINED/MS/partition=1.mat')

for i=1:length(L.LT)
  sg('init_dyn_prog', 1);
  sg('precompute_content_svms', L.XT{i},int32(0:length(L.XT{i})-1),LL.Train.w);
  output{i} = sg('get_lin_feat');
  output1(i) = output{i}(end);
end

%%%


XT = zeros(5440,length(L.XT));
method = LL.Train.PAR.Signal.method;
for j=1:length(L.XT) 
  dim = 0;
  for o=1:length(method.kernels)
    [mask hist] = compute_mers(L.XT{j}, method.kernels{o}.wordlen, method.kernels{o}.stepping, method.kernels{o}.offset);
    XT(dim+[1:4^method.kernels{o}.wordlen],j) = hist';
    dim = dim+ 4^method.kernels{o}.wordlen;
  end
end
init_svm_and_kernels(method,0);
sg('clean_features', 'TRAIN');
D.XT = LL.Train.XT;
D.LT = LL.Train.LT;
setFeatures(method, D,'TRAIN',0,0);
clear D

sg('clean_features', 'TEST' );
D.XT = XT;
setFeatures(method, D,'TEST',0,0);

sg('set_svm',LL.Train.b,LL.Train.alphas);
output2 = sg( 'svm_classify' );

return

function found = issubset(A,B)
% function found = issubset(A,B)
%
% returns true if A is a subset of B, where 
% both A and B are vectors with 1 where elements exists and 0 otherwise.
found = isequal(find(A),intersect(find(A),find(B)));
return

function ret=keyboard_allowed()
% ret=keyboard_allowed()
%
% returns 1, if a keyboard command would be allowed

global g_ignore_keyboard
global THIS_IS_A_RPROC_PROCESS  

if isequal(g_ignore_keyboard, 1),
  ret=0 ;
  return ;
end ;

if isequal(THIS_IS_A_RPROC_PROCESS, 1),
  ret=0 ;
  return ;
end ;

ret=1 ;


return

function f=bigunique(s) ;
% f=bigunique(s) ;

f=[] ; step=ceil(length(s)/10) ;
for i=1:10,
  idx=1+(i-1)*step:min(length(s), i*step) ;
  f=unique([f unique(s(idx))]) ;
end ;


return

